// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosEventChannelAdmin.*;
import java.io.*;
import java.util.*;

public class PushServer
{
    public static void
    main(String args[])
    {
	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	try
	{
            //
            // Create ORB.
            //
            ORB orb = ORB.init(args, props);

	    //
	    // Filter options.
	    //
	    args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
	    
	    //
	    // Parse command line arguments.
	    //
	    String channelName = null;
	    for(int i = 0 ; i < args.length ; i++)
	    {
		if(args[i].equals("--help") || args[i].equals("-h"))
		{
		    usage();
                    System.exit(0);
		}
		else if(!args[i].startsWith("-"))
		{
		    channelName = args[i];
		}
		else
		{
		    System.err.println("Unknown option `" + args[i] + "'");
		    usage();
		    System.exit(1);
		}
	    }

	    //
	    // Get event channel
	    //
	    EventChannel e = GetEventChannel(orb, channelName);
	    
	    //
	    // Get ProxyPushConsumer.
	    //
	    SupplierAdmin supplierAdmin = e.for_suppliers();
	    ProxyPushConsumer consumer = supplierAdmin.obtain_push_consumer();

	    //
	    // Since callbacks are not needed a PushSupplier_impl is not
	    // created
	    //
	    try
	    {
		consumer.connect_push_supplier(null);
	    }
	    catch(AlreadyConnected ex)
	    {
		ex.printStackTrace();
		System.exit(1);
	    }

	    //
	    // Main loop
	    //
	    while(true)
	    {
		java.util.Date date = new java.util.Date();
		String s = "PushSupplier says: " + date.toString();
		
		Any any = orb.create_any();
		any.insert_string(s);
		
		try
		{
		    consumer.push(any);
		}
		catch(org.omg.CosEventComm.Disconnected ex)
		{
		    break;
		}
		catch(org.omg.CORBA.SystemException ex)
		{
		    ex.printStackTrace();
		    break;
		}

		try
		{
		    Thread.sleep(1000);
		}
		catch(InterruptedException ex)
		{
		}
	    }
	 
	    try
	    {
		consumer.disconnect_push_consumer();
	    }
	    //
	    // Expected if event service has done away
	    //
	    catch(org.omg.CORBA.TRANSIENT ex)
	    {
	    }
	    catch(org.omg.CORBA.COMM_FAILURE ex)
	    {
	    }
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	System.exit(0);
    }

    private static void
    usage()
    {
        System.out.println(
            "Usage: java event.PushServer [options] [channel-name]\n\n" +
	    "Options:\n" +
	    "-h, --help             Show this message.\n");
    }

    static private EventChannel
    GetEventChannel(org.omg.CORBA.ORB orb, String name)
    {
	EventChannel e = null;
	if(name == null)
	{
	    //
	    // Get event channel from the initial reference.
	    //
	    try
	    {
		org.omg.CORBA.Object obj =
		    orb.resolve_initial_references("EventService");
		e = EventChannelHelper.narrow(obj);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("can't resolve `EventService'");
		System.exit(1);
	    }
	    
	    if(e == null)
	    {
		System.err.println("`EventService' is not an EventChannel " +
				   " object reference");
		System.exit(1);
	    }
	}
	else
	{
	    //
	    // Get event channel factory. Note that this uses ORBacus
	    // proprietary methods.
	    //
	    com.ooc.OBEventChannelFactory.EventChannelFactory f = null;
	    try
	    {
		org.omg.CORBA.Object obj =
		    orb.resolve_initial_references("EventChannelFactory");
		f = com.ooc.OBEventChannelFactory.EventChannelFactoryHelper.
		    narrow(obj);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("Can't resolve `EventChannelFactory'");
		System.exit(1);
	    }
	
	    if(f == null)
	    {
		System.err.println("`EventChannelFactory' is not an " +
				   "EventChannelFactory object reference");
		System.exit(1);
	    }

	    //
	    // Get event channel.
	    //
	    try
	    {
		e = f.get_channel_by_id(name);
	    }
	    catch(com.ooc.OBEventChannelFactory.ChannelNotAvailable ex)
	    {
		try
		{
		    e = f.create_channel(name);
		    System.err.println("`Channel " + name + "' created");
		}
		catch(com.ooc.OBEventChannelFactory.ChannelAlreadyExists ex1)
		{
		    System.err.println("Channel exists: try again");
		    System.exit(1);
		}
	    }
	}
	return e;
    }
}
