// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosTypedEventChannelAdmin.*;
import org.omg.CosEventChannelAdmin.*;
import java.io.*;
import java.util.*;

public class TypedPushServer
{
    public static void
    main(String args[])
    {
	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	try
	{
            //
            // Create ORB.
            //
            ORB orb = ORB.init(args, props);

	    //
	    // Filter options.
	    //
	    args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
	    
	    //
	    // Parse command line arguments.
	    //
	    String channelName = null;
	    for(int i = 0 ; i < args.length ; i++)
	    {
		if(args[i].equals("--help") || args[i].equals("-h"))
		{
		    usage();
                    System.exit(0);
		}
		else if(!args[i].startsWith("-"))
		{
		    channelName = args[i];
		}
		else
		{
		    System.err.println("Unknown option `" + args[i] + "'");
		    usage();
		    System.exit(1);
		}
	    }

	    //
	    // Get event channel.
	    //
	    TypedEventChannel e = GetTypedEventChannel(orb, channelName);
	    
	    //
	    // Get TypedProxyPushConsumer.
	    //
	    TypedSupplierAdmin supplierAdmin = e.for_suppliers();
	    TypedProxyPushConsumer consumer = null;

	    try
	    {
		consumer = supplierAdmin.obtain_typed_push_consumer(
		    "IDL:event/TimerUpdate:1.0");
	    }
	    catch(InterfaceNotSupported ex)
	    {
		ex.printStackTrace();
		System.exit(1);
	    }

	    //
	    // Since callbacks are not needed a PushSupplier_impl is
	    // not created.
	    //
	    try
	    {
		consumer.connect_push_supplier(null);
	    }
	    catch(AlreadyConnected ex)
	    {
		ex.printStackTrace();
		System.exit(1);
	    }


	    org.omg.CORBA.Object typedConsumer = consumer.get_typed_consumer();

	    TimerUpdate timer = TimerUpdateHelper.narrow(typedConsumer);
	    if(timer == null)
	    {
		System.out.println("narrow failed!");
		System.exit(1);
	    }

	    while(true)
	    {
		java.util.Date date = new java.util.Date();
		String s = "PushSupplier says: " + date.toString();
		try
		{
		    timer.time_update(s);
		}
		catch(org.omg.CORBA.SystemException ex)
		{
		    ex.printStackTrace();
		    break;
		}

		try
		{
		    Thread.sleep(1000);
		}
		catch(InterruptedException ex)
		{
		}
	    }
	    try
	    {
		consumer.disconnect_push_consumer();
	    }
	    //
	    // Expected if event service has gone away
	    //
	    catch(org.omg.CORBA.TRANSIENT ex)
	    {
	    }
	    catch(org.omg.CORBA.COMM_FAILURE ex)
	    {
	    }
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}

	System.exit(0);
    }

    private static void
    usage()
    {
        System.out.println(
            "Usage: java event.TypedPushServer [options] [channel-name]\n\n" +
	    "Options:\n" +
	    "-h, --help             Show this message.\n");
    }

    static private TypedEventChannel
    GetTypedEventChannel(org.omg.CORBA.ORB orb, String name)
    {
	TypedEventChannel e = null;
	if(name == null)
	{
	    //
	    // Get event channel from the initial reference.
	    //
	    try
	    {
		org.omg.CORBA.Object obj =
		    orb.resolve_initial_references("TypedEventService");
		e = TypedEventChannelHelper.narrow(obj);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("can't resolve `TypedEventService'");
		System.exit(1);
	    }
	    
	    if(e == null)
	    {
		System.err.println("`TypedEventService' is not an " +
				   " TypedEventChannel object reference");
		System.exit(1);
	    }
	}
	else
	{
	    //
	    // Get event channel factory. Note that this uses ORBacus
	    // proprietary methods.
	    //
	    com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactory f
		= null;
	    try
	    {
		org.omg.CORBA.Object obj =
		    orb.resolve_initial_references("TypedEventChannelFactory");
		f = com.ooc.OBTypedEventChannelFactory.
		    TypedEventChannelFactoryHelper. narrow(obj);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("Can't resolve `TypedEventChannelFactory'");
		System.exit(1);
	    }
	
	    if(f == null)
	    {
		System.err.println("`TypedEventChannelFactory' is not an " +
				   "TypedEventChannelFactory object " +
				   "reference");
		System.exit(1);
	    }

	    //
	    // Get event channel.
	    //
	    try
	    {
		e = f.get_channel_by_id(name);
	    }
	    catch(com.ooc.OBEventChannelFactory.ChannelNotAvailable ex)
	    {
		try
		{
		    e = f.create_channel(name);
		    System.err.println("`Channel " + name + "' created");
		}
		catch(com.ooc.OBEventChannelFactory.ChannelAlreadyExists ex1)
		{
		    System.err.println("Channel exists: try again");
		    System.exit(1);
		}
	    }
	}
	return e;
    }
}
