// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventAdmin;

import org.omg.CORBA.*;
import org.omg.CosTypedEventChannelAdmin.*;
import org.omg.CosEventChannelAdmin.*;
import java.util.*;

public class EventAdmin
{
 
    private static void
    usage(String progName)
    {
	System.err.println(" Usage:\n" +
	    progName + " [options] command\n" +
	    "\n" +
	    "Options:\n"  +
	    "-h, --help             Show this message.\n"  +
	    "-v, --version          Show version number.\n" +
	    "-t, --typed-service    Admin for typed event service.\n" +
	    "-u, --untyped-service  Admin for untyped event service " +
				    "[default].\n" +
	    "\n" +
	    "Commands:\n" +
	    "create channel-id      Create a new event channel.\n" +
	    "destroy channel-id     Destroy an event channel.\n" +
	    "list                   Show the active event channels.\n" +
	    "shutdown               Terminate the event service.\n");
    }

    public static void
    main(String args[])
    {
	Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");
	
	try
	{
	    int i;
	    boolean typedService = false;

	    //
	    // Create ORB
	    //
	    ORB orb =ORB.init(args, props);

            //
            // Filter options.
            //
            args = ((com.ooc.CORBA.ORB)orb).filter_options(args);

	    //
	    // Get options
	    //
	    for(i = 0 ; i < args.length && 
			args[i].substring(0,1).equals("-"); i++)
	    {
		if(args[i].equals("--help") ||
		   args[i].equals("-h"))
		{
		    usage(args[0]);
		    System.exit(0);
		}
		else if(args[i].equals("--version") ||
			args[i].equals("-v"))
		{
		    System.err.println("ORBacus Event Service" + 
				       com.ooc.CORBA.Version.version);
		    System.exit(0);
		}
		else if(args[i].equals("--untyped-service") ||
			args[i].equals("-u"))
		{
		}
		else if(args[i].equals("--typed-service") ||
			args[i].equals("-t"))
		{
		    typedService = true;
		}
		else
		{
		    System.err.println(args[0] + ": unknown option `" +
					args[i]);
		    usage(args[0]);
		    System.exit(0);
		}
	    }

	    if(args.length == i)
	    {
		usage(args[0]);
		System.exit(0);
	    }

	    String serviceName = 
		    typedService ? "TypedEventChannelFactory" : "EventChannelFactory";
	    
	    org.omg.CORBA.Object obj;
	    try
	    {
		obj = orb.resolve_initial_references(serviceName);

		if(obj == null)
		{
		    System.err.println("No " + serviceName + 
				       " initial reference.");
		    System.exit(1);
		}

		if(typedService)
		{
		    com.ooc.OBTypedEventChannelFactory.
			TypedEventChannelFactory f =
			    com.ooc.OBTypedEventChannelFactory.
				TypedEventChannelFactoryHelper.narrow(obj);

		    if(f == null)
		    {
			System.err.println("`" + serviceName + "' initial " +
					   "reference is invalid.");
			System.exit(1);
		    }

		    if(i + 1 < args.length && args[i].equals("create"))
		    {
			try
			{
			    TypedEventChannel channel =
				f.create_channel(args[i + 1]);
			}
			catch(com.ooc.OBEventChannelFactory.
				  ChannelAlreadyExists ex)
			{
			    System.err.println("Channel `" + args[i + 1] + 
					       "' already exists");
			    System.exit(1);
			}
		    }
		    else if(i + 1 < args.length && args[i].equals("destroy"))
		    {
			try
			{
			    TypedEventChannel channel =
				f.get_channel_by_id(args[i + 1]);
			    channel.destroy();
			}
			catch(com.ooc.OBEventChannelFactory.
				  ChannelNotAvailable ex)
			{
			    System.err.println("Channel `" + args[i + 1] + 
					       "' not found");
			    System.exit(1);
			}
		    }
		    else if(i < args.length && args[i].equals("list"))
		    {
			String[] ids = f.get_channels();
			if(ids.length == 0)
			    System.out.println("No active channels");
			else
			{
			    for(int j = 0 ; j < ids.length ; j++)
				System.out.println("ids[j]");
			}
		    }
		    else if(i < args.length && args[i].equals("shutdown"))
		    {
			try
			{
			    f.shutdown();
			}
			catch(TRANSIENT ex)
			{
			    //
			    // This call will deactivate the server and thus
			    // throw an exception.
			    //
			}
		    }
		    else
			usage(args[0]);
		}
		else
		{
		    com.ooc.OBEventChannelFactory.EventChannelFactory f =
			com.ooc.OBEventChannelFactory.
			    EventChannelFactoryHelper.narrow(obj);
		    if(f == null)
		    {
			System.err.println("`" + serviceName + "' initial " +
					   "reference is invalid.");
			System.exit(1);
		    }
		    if(i + 1 < args.length && args[i].equals("create"))
		    {
			try
			{
			    EventChannel channel =
				f.create_channel(args[i + 1]);
			}
			catch(com.ooc.OBEventChannelFactory.
				  ChannelAlreadyExists ex)
			{
			    System.err.println("Channel `" + args[i + 1] + 
					       "' already exists");
			    System.exit(1);
			}
		    }
		    else if(i + 1 < args.length && args[i].equals("destroy"))
		    {
			try
			{
			    EventChannel channel =
				f.get_channel_by_id(args[i + 1]);
			    channel.destroy();
			}
			catch(com.ooc.OBEventChannelFactory.
				  ChannelNotAvailable ex)
			{
			    System.err.println("Channel `" + args[i + 1] + 
					       "' not found");
			    System.exit(1);
			}
		    }
		    else if(i < args.length && args[i].equals("list"))
		    {
			String[] ids = f.get_channels();
			if(ids.length == 0)
			    System.out.println("No active channels");
			else
			{
			    for(int j = 0 ; j < ids.length ; j++)
				System.out.println(ids[j]);
			}
		    }
		    else if(i < args.length && args[i].equals("shutdown"))
		    {
			try
			{
			    f.shutdown();
			}
			catch(TRANSIENT ex)
			{
			    //
			    // This call will deactivate the server and thus
			    // throw an exception.
			    //
			}
		    }
		    else
			usage(args[0]);
		}
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
	    }
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	System.exit(0);
    }
}
