// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventChannelAdmin.impl;

import com.ooc.CosEventServer.*;
import org.omg.CORBA.*;

class AnyPullerThread extends PropagatorBaseThread
{
    //
    // The proxy.
    //
    private org.omg.CosEventChannelAdmin.ProxyPullConsumer consumer_;

    //
    // The supplier.
    //
    private org.omg.CosEventComm.PullSupplier supplier_;
    
    //
    // My pipe segement.
    //
    private PipeSegment myPipe_;
    
    //
    // Interval between successive pulls.
    //
    private long pullInterval_;
    
    //
    // Is there a delay between each pull call?
    //
    private boolean hasDelay_;

    AnyPullerThread(
	org.omg.CosEventComm.PullSupplier supplier,
	org.omg.CosEventChannelAdmin.ProxyPullConsumer consumer,
	long id,
	PipeSegment pipe,
	long pullInterval,
	long maxRetries,
	long retryTimeout,
	double retryMultiplier)
    {
	super("AnyPullerThread", id, maxRetries, retryTimeout,
	      retryMultiplier);
	
	consumer_ = consumer;
	supplier_ = supplier;
	myPipe_ = pipe;
	pullInterval_ = pullInterval;
	hasDelay_ = pullInterval > 0;
    }

    public void run()
    {
        while(!done_)
        {
            try
            {
                Any any = supplier_.pull();
                resetRetry();
                Event_impl event = new Event_impl(any, id_);
                myPipe_.receive(event);
                if(hasDelay_)
                    delay(pullInterval_);
            }
	    //
	    // Some error conditions are transient and should cause a retry.
	    //
            catch(COMM_FAILURE ex)
            {
                done_ = !retry();
            }
	    catch(TRANSIENT ex)
	    {
		done_ = !retry();
	    }
	    catch(NO_RESPONSE ex)
	    {
		done_ = !retry();
	    }
            catch(Exception ex)
            {
                done_ = true;
            }

	    if(done_)
	    {
                try
                {
                    consumer_.disconnect_pull_consumer();
                }
                catch(SystemException e)
                {
                }
	    }
        }
    }
}
