// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventChannelAdmin.impl;

import org.omg.CORBA.*;
import com.ooc.CosEventServer.*;

public class EventChannel
    extends org.omg.CosEventChannelAdmin._EventChannelImplBase
{
    //
    // The ORB.
    //
    private ORB orb_;

    //
    // My pipe segment.
    //
    PipeSegment myPipe_;

    //
    // The consumer admin.
    //
    private ConsumerAdmin consumerAdmin_;

    //
    // The supplier admin.
    //
    private SupplierAdmin supplierAdmin_;

    //
    // The channel's Id.
    //
    private String myId_;

    //
    // Has the channel been destroyed?
    //
    private boolean destroy_ = false;

    // -----------------------------------------------------------------
    // EventChannel constructor
    // -----------------------------------------------------------------

    public EventChannel(ORB orb, String myId, Reaper reaper)
    {
	orb_ = orb;
	myId_ = myId;
	myPipe_ = new PipeSegment();
        consumerAdmin_ = new ConsumerAdmin(orb_, reaper, myPipe_);
        supplierAdmin_ = new SupplierAdmin(orb_, reaper, myPipe_);

	SetDefaultProperties.setDefaults();
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------
    
    synchronized public org.omg.CosEventChannelAdmin.ConsumerAdmin
    for_consumers()
    {
	if(destroy_)
	    throw new OBJECT_NOT_EXIST();

	return consumerAdmin_;
    }

    synchronized public org.omg.CosEventChannelAdmin.SupplierAdmin
    for_suppliers()
    {
	if(destroy_)
	    throw new OBJECT_NOT_EXIST();

	return supplierAdmin_;
    }

    synchronized public void
    destroy()
    {
	if(destroy_)
	    throw new OBJECT_NOT_EXIST();

	destroy_ = true;

	orb_.disconnect(this);
        
	myPipe_.destroy();

        consumerAdmin_.disconnect();
        supplierAdmin_.disconnect();
    }

    // ------------------------------------------------------------------
    // Internal methods
    // ------------------------------------------------------------------
    
    synchronized public void
    discardRequests()
    {
	if(destroy_)
	    return;

	myPipe_.destroy();

	consumerAdmin_.discardRequests();
	supplierAdmin_.discardRequests();
    }

    synchronized public boolean
    destroyed()
    {
	return destroy_;
    }
}
