// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventChannelAdmin.impl;

import com.ooc.CosEventServer.*;
import org.omg.CORBA.*;

class ProxyPullConsumer
    extends org.omg.CosEventChannelAdmin._ProxyPullConsumerImplBase
    implements ProxyBase
{
    //
    // The ProxyBaseImpl Object
    //
    private ProxyBaseImpl proxyImpl_;

    //
    // My pull supplier.
    //
    private org.omg.CosEventComm.PullSupplier supplier_;

    //
    // The event puller.
    //
    private PropagatorBaseThread puller_;

    // ------------------------------------------------------------------
    // ProxyPullConsumer constructor
    // ------------------------------------------------------------------

    ProxyPullConsumer(ProxyConstructorInfo info)
    {
	info.base = "ProxyPullConsumer";
	proxyImpl_ = new ProxyBaseImpl(info, this);
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    synchronized public void
    connect_pull_supplier(org.omg.CosEventComm.PullSupplier supplier)
	throws org.omg.CosEventChannelAdmin.AlreadyConnected
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	if(proxyImpl_.getState() == ProxyState.Connected)
	    throw new org.omg.CosEventChannelAdmin.AlreadyConnected();

	if(supplier == null)
	    throw new BAD_PARAM();
	
	supplier_ = supplier;
	
	proxyImpl_.setState(ProxyState.Connected);
	
	if(puller_ != null)
	    throw new InternalError();
	
	QoSParameters params = new QoSParameters();
	proxyImpl_.resolveQoSParams(params);
	    
	puller_ = new AnyPullerThread(supplier, this, proxyImpl_.getId(),
				      proxyImpl_.getParentPipe(),
				      params.pullInterval, params.maxRetries,
				      params.retryTimeout,
				      params.retryMultiplier);
	puller_.start();
    }

    synchronized public void
    disconnect_pull_consumer()
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	proxyImpl_.disconnect();
    }
    
    // ------------------------------------------------------------------
    // ProxyBase implementation
    // ------------------------------------------------------------------

    //
    // Disconnect the servant from the ORB
    //
    public void deactivateObject()
    {
	proxyImpl_.getOrb().disconnect(this);
    }
    
    //
    // Called during disconnect
    //
    public void disconnectPeer()
    {
	//
	// Stop the puller, if running.
	//
	if(puller_ != null)
	    puller_.halt();

	//
	// If we're connected then tell the supplier.
	//
	if(supplier_ != null)
	{
	    try
	    {
		supplier_.disconnect_pull_supplier();
	    }
	    catch(SystemException e)
	    {
		// A non-existent supplier doesn't matter in this context.
	    }
	    supplier_ = null;
	}
    }

    //
    // Disconnect the proxy
    //
    public synchronized void disconnect()
    {
	proxyImpl_.disconnect();
    }

    //
    // Don't accept further requests
    //
    public synchronized void discardRequests()
    {
	proxyImpl_.discardRequests();

	//
	// Stop the puller, wait for it to complete, if running.
	//
	if(puller_ != null)
	    puller_.halt();
    }

    //
    // Has this proxy been destroyed?
    //
    public synchronized boolean destroyed()
    {
	return proxyImpl_.destroyed();
    }

    //
    // Can this proxy be reaperd?
    //
    public boolean canReap()
    {
	return false;
    }

    //
    // What was the time of the last request?
    //
    public long lastRequestTime()
    {
	return proxyImpl_.lastRequestTime();
    }

    //
    // Set the time of the last request
    //
    public void stampRequest()
    {
	proxyImpl_.stampRequest();
    }

    //
    // Get the name of the proxy
    //
    public String getName()
    {
	return proxyImpl_.getName();
    }
}
