// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventChannelAdmin.impl;

import com.ooc.CosEventServer.*;
import java.util.*;
import org.omg.CORBA.*;

//
// The event channel ProxyPullSupplier implementation
//
class ProxyPullSupplier
    extends org.omg.CosEventChannelAdmin._ProxyPullSupplierImplBase
    implements ProxyBase
{
    //
    // The ProxyBaseImpl Object
    //
    private ProxyBaseImpl proxyImpl_;

    //
    // My pull pipe segment
    //
    private PullPipeSegment pullPipe_;

    //
    // My pull consumer
    //
    private org.omg.CosEventComm.PullConsumer consumer_;

    //
    // Maximum number of pending events.
    //
    private long maxEvents_;

    //
    // Can this proxy be reaped?
    //
    private int canReap_;

    // ------------------------------------------------------------------
    // ProxyPullSupplier constructor
    // ------------------------------------------------------------------

    ProxyPullSupplier(ProxyConstructorInfo info)
    {
	info.base = "ProxyPullSupplier";
	proxyImpl_ = new ProxyBaseImpl(info, this);
	canReap_ = 0;

	//
	// Resolve the QoS params
	//
	com.ooc.CORBA.Properties properties =
	    com.ooc.CORBA.Properties.instance();
	try
	{
	    maxEvents_ = Long.parseLong(
		properties.getProperty(EventQoS.MAX_EVENTS));
	}
	catch(NumberFormatException e)
	{
	    throw new InternalError();
	}
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    synchronized public void
    connect_pull_consumer(org.omg.CosEventComm.PullConsumer consumer)
	throws org.omg.CosEventChannelAdmin.AlreadyConnected
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	if(proxyImpl_.getState() == ProxyState.Connected)
	    throw new org.omg.CosEventChannelAdmin.AlreadyConnected();

	//
	// The consumer is permitted to be nil
	//
	if(consumer != null)
	    consumer_ = consumer;

	proxyImpl_.stampRequest();
	proxyImpl_.setState(ProxyState.Connected);

	pullPipe_ = new PullPipeSegment(maxEvents_, proxyImpl_.getId());
	proxyImpl_.setMyPipe(pullPipe_);

	//
	// Connect my pipe segment to my parent segment
	//
	proxyImpl_.getParentPipe().connect(pullPipe_);
    }

    public Any
    pull()
	throws org.omg.CosEventComm.Disconnected
    {
	synchronized(this)
	{
	    canReap_++;
	}

	//
	// Get the first event, and return the data. If myPipe_ is not
	// initialized or the PipeSegment has been destroyed
	// getFirstEventWait() returns 0.
	//
	Event_impl event = null;
	if(pullPipe_ != null)
	    event = pullPipe_.getFirstEventWait();
	
	synchronized(this)
	{
	    canReap_--;

	    if(proxyImpl_.getState() == ProxyState.Destroyed)
		throw new OBJECT_NOT_EXIST();

	    if(proxyImpl_.getState() == ProxyState.Discarding)
		throw new TRANSIENT();

	    if(proxyImpl_.getState() != ProxyState.Connected)
		throw new org.omg.CosEventComm.Disconnected();

	    proxyImpl_.stampRequest();

	    if(event == null)
		throw new InternalError();
	    return event.any();
	}
    }

    synchronized public Any
    try_pull(BooleanHolder hasEvent)
	throws org.omg.CosEventComm.Disconnected
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	if(proxyImpl_.getState() != ProxyState.Connected)
	    throw new org.omg.CosEventComm.Disconnected();

	proxyImpl_.stampRequest();

	Event_impl event = pullPipe_.getFirstEvent();

	//
	// Determine if we have an event.
	//
	hasEvent.value = (event != null);

	Any any;
	if(hasEvent.value)
	{
	    //
	    // Yes, then get the first event. Return the data.
	    //
	    any = event.any();
	}
	else
	{
	    any = proxyImpl_.getOrb().create_any();
	}
	return any;
    }

    synchronized public void
    disconnect_pull_supplier()
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	proxyImpl_.disconnect();
    }

    // ------------------------------------------------------------------
    // ProxyBase implementation
    // ------------------------------------------------------------------

    //
    // Disconnect the servant from the ORB
    //
    public void deactivateObject()
    {
	proxyImpl_.getOrb().disconnect(this);
    }
    
    //
    // Called during disconnect
    //
    public void disconnectPeer()
    {
	//
	// If we're connected then tell the consumer.
	//
	if(consumer_ != null)
	{
	    try
	    {
		consumer_.disconnect_pull_consumer();
	    }
	    catch(SystemException e)
	    {
		// A non-existent supplier doesn't matter in this context.
	    }
	    consumer_ = null;
	}
    }

    //
    // Disconnect the proxy
    //
    public synchronized void disconnect()
    {
	proxyImpl_.disconnect();
    }

    //
    // Don't accept further requests
    //
    public synchronized void discardRequests()
    {
	proxyImpl_.discardRequests();
    }

    //
    // Has this proxy been destroyed?
    //
    public synchronized boolean destroyed()
    {
	return proxyImpl_.destroyed();
    }

    //
    // Can this proxy be reaperd?
    //
    public synchronized boolean canReap()
    {
	return canReap_ == 0;
    }

    //
    // What was the time of the last request?
    //
    public long lastRequestTime()
    {
	return proxyImpl_.lastRequestTime();
    }

    //
    // Set the time of the last request
    //
    public void stampRequest()
    {
	proxyImpl_.stampRequest();
    }

    //
    // Get the name of the proxy
    //
    public String getName()
    {
	return proxyImpl_.getName();
    }
}
