// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventChannelAdmin.impl;

import com.ooc.CosEventServer.*;
import org.omg.CORBA.*;

class ProxyPushConsumer
    extends org.omg.CosEventChannelAdmin._ProxyPushConsumerImplBase
    implements ProxyBase
{
    //
    // The ProxyBaseImpl Object
    //
    private ProxyBaseImpl proxyImpl_;

    //
    // My push supplier.
    //
    private org.omg.CosEventComm.PushSupplier supplier_;

    // ------------------------------------------------------------------
    // ProxyPushConsumer constructor
    // ------------------------------------------------------------------

    ProxyPushConsumer(ProxyConstructorInfo info)
    {
	info.base = "ProxyPushConsumer";
	proxyImpl_ = new ProxyBaseImpl(info, this);
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------
    
    synchronized public void
    connect_push_supplier(org.omg.CosEventComm.PushSupplier supplier)
	throws org.omg.CosEventChannelAdmin.AlreadyConnected
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	if(proxyImpl_.getState() == ProxyState.Connected)
	    throw new org.omg.CosEventChannelAdmin.AlreadyConnected();

	//
	// The supplier is permitted to be nil
	//
	if(supplier != null)
	    supplier_ = supplier;

	proxyImpl_.stampRequest();
	proxyImpl_.setState(ProxyState.Connected);
    }

    synchronized public void
    disconnect_push_consumer()
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	proxyImpl_.disconnect();
    }
    
    synchronized public void
    push(Any any)
	throws org.omg.CosEventComm.Disconnected
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	if(proxyImpl_.getState() != ProxyState.Connected)
	    throw new org.omg.CosEventComm.Disconnected();

	proxyImpl_.stampRequest();

	Event_impl event = new Event_impl(any, proxyImpl_.getId());
	
	proxyImpl_.getParentPipe().receive(event);
    }

    // ------------------------------------------------------------------
    // ProxyBase implementation
    // ------------------------------------------------------------------

    //
    // Disconnect the servant from the ORB
    //
    public void deactivateObject()
    {
	proxyImpl_.getOrb().disconnect(this);
    }
    
    //
    // Called during disconnect
    //
    public void disconnectPeer()
    {
	//
	// Since we don't necessarily have a supplier, we need to verify
	// before invoking a method.
	//
	if(supplier_ != null)
	{
	    try
	    {
		supplier_.disconnect_push_supplier();
	    }
	    catch(SystemException e)
	    {
	    }
	    supplier_ = null;
	}
    }

    //
    // Disconnect the proxy
    //
    public synchronized void disconnect()
    {
	proxyImpl_.disconnect();
    }

    //
    // Don't accept further requests
    //
    public synchronized void discardRequests()
    {
	proxyImpl_.discardRequests();
    }

    //
    // Has this proxy been destroyed?
    //
    public synchronized boolean destroyed()
    {
	return proxyImpl_.destroyed();
    }

    //
    // Can this proxy be reaperd?
    //
    public boolean canReap()
    {
	return true;
    }

    //
    // What was the time of the last request?
    //
    public long lastRequestTime()
    {
	return proxyImpl_.lastRequestTime();
    }

    //
    // Set the time of the last request
    //
    public void stampRequest()
    {
	proxyImpl_.stampRequest();
    }

    //
    // Get the name of the proxy
    //
    public String getName()
    {
	return proxyImpl_.getName();
    }
}
