// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
package com.ooc.CosEventServer;

import org.omg.CORBA.*;
import java.util.Vector;

public class AdminBaseImpl
{
    //
    // The ORB.
    //
    private ORB orb_;
    
    //
    // My pipe segment
    //
    private PipeSegment myPipe_;

    //
    // Set of ProxyConsumers.
    //
    private Vector proxies_ = new Vector();

    //
    // ID of the next assigned proxy.
    //
    private long nextId_;

    //
    // Has this admin been destroyed?
    //
    private boolean destroyed_;

    //
    // The reaper
    //
    private Reaper reaper_;

    // ----------------------------------------------------------------------
    // AdminBaseImpl private member implementation
    // ----------------------------------------------------------------------

    //
    // Reap proxies
    //
    private void reap()
    {
	//
	// Find the element, remove it from the sequence.
	//
	int i = 0;
	while(i < proxies_.size())
	{
	    ProxyBase proxy = (ProxyBase)proxies_.elementAt(i);
	    if(proxy.destroyed())
	    {
		proxies_.removeElementAt(i);
		continue;
	    }
	    ++i;
	}
    }

    // ----------------------------------------------------------------------
    // AdminBaseImpl constructor
    // ----------------------------------------------------------------------

    public AdminBaseImpl(ORB orb, Reaper reaper, PipeSegment pipe)
    {
	orb_ = orb;
	reaper_ = reaper;
	myPipe_ = pipe;
	nextId_ = 0;
	destroyed_ = false;
    }

    // ----------------------------------------------------------------------
    // AdminBaseImpl public member implementation
    // ----------------------------------------------------------------------

    //
    // Add a proxy
    //
    public void addProxy(ProxyBase proxy)
    {
	reap();
	proxies_.addElement(proxy);
	if(reaper_ != null)
	    reaper_.add(proxy);
    }

    //
    // Fill constructor info
    //
    public void fillProxyConstructorInfo(ProxyConstructorInfo info)
    {
	info.orb = orb_;
	info.pipe = myPipe_;
	info.id = nextId_++;
    }

    //
    // Called from the event channel upon destruction.
    //
    public void disconnect()
    {
	if(destroyed_)
	    throw new InternalError();
	destroyed_ = true;

	//
	// disconnect each proxy.
	//
	for(int i = 0 ; i < proxies_.size() ; ++i)
	    ((ProxyBase)proxies_.elementAt(i)).disconnect();
	proxies_.removeAllElements();
    }

    //
    // Stop accepting new requests.
    //
    public void discardRequests()
    {
	if(!destroyed_)
	{
	    for(int i = 0 ; i < proxies_.size() ; ++i)
		((ProxyBase)proxies_.elementAt(i)).discardRequests();
	}
    }

    //
    // Get the ORB
    //
    public ORB getOrb()
    {
	return orb_;
    }

    //
    // Has this admin been destroyed?
    //
    public boolean destroyed()
    {
	return destroyed_;
    }
}
