// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventServer;

import org.omg.CORBA.*;
import com.ooc.OBEventChannelFactory.impl.EventChannelFactory;

//
// This class encapsulates the event service
// 
class EventService
{
    //
    // My ORB
    //
    private ORB orb_;

    //
    // My BOA
    //
    private BOA boa_;

    //
    // The event channel factory
    //
    private EventChannelFactory channelFactory_;

    //
    // My proxy reaper
    //
    private Reaper reaper_;

    //
    // Are we initialized?
    //
    private boolean initialized_;

    // ------------------------------------------------------------------
    // EventService constructor/shutdown
    // ------------------------------------------------------------------

    public EventService(ORB orb, BOA boa)
    {
	orb_ = orb;
	boa_ = boa;
	initialized_ = false;
    }

    // ------------------------------------------------------------------
    // EventService public member implementation
    // ------------------------------------------------------------------

    //
    // Shutdown the event service
    //
    public void shutdown()
    {
	//
	// Stop the proxy reaper
	//
	if(reaper_ != null)
	{
	    reaper_.destroy();
	    while(reaper_.isAlive())
	    {
		try
		{
		    reaper_.join();
		}
		catch(InterruptedException ex)
		{
		}
	    }
	    reaper_ = null;
	}

	//
	// Shutdown the event service
	//
	if(channelFactory_ != null)
	{
	    channelFactory_.shutdown();
	    channelFactory_.destroy();
	    channelFactory_ = null;
	}
    }

    //
    // Initialize the event service
    //
    void
    initialize()
    {
	if(initialized_)
	    return;

	initialized_ = true;

	//
	// Create the proxy reaper
	//
	reaper_ = new Reaper();
	reaper_.start();
    
	//
	// Create event channel.
	//
	channelFactory_ = new EventChannelFactory(orb_, boa_, reaper_);
	((com.ooc.CORBA.ORB)orb_).connect(channelFactory_,
					  "DefaultEventChannelFactory");
    }

    //
    // Get event channel factory
    //
    com.ooc.OBEventChannelFactory.EventChannelFactory
    eventChannelFactory()
    {
	return channelFactory_;
    }
}
