// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventServer;

import java.util.Vector;

//
// A PipeSegment. An event channel is composed of a set of connected
// PipeSegments. Each segment can have 0 or 1 parents and 0, 1 or many
// children.
//
// Note: The Java PipeSegment class differs from the C++ PipeSegment
//       class.  Since reference counting is not a concern, the list
//       of reaped children and its reference count are not necessary.
//
public class PipeSegment
{
    //
    // The set of connected children
    //
    private Vector children_ = new Vector();

    // ------------------------------------------------------------------
    // PipeSegment private member implementation
    // ------------------------------------------------------------------

    // Perform a reap of the supplied children
    //
    private void reap(Vector reap)
    {
	for(int i = 0 ; i < reap.size() ; i++)
	{
	    for(int j = 0 ; j < children_.size() ; j++)
	    {
		if(children_.elementAt(j) == reap.elementAt(i))
		{
		    children_.removeElementAt(j);
		    break;
		}
	    }
	}
    }
    
    //
    // Has the segment been destroyed?
    //
    protected boolean destroy_ = false;

    // ------------------------------------------------------------------
    // PipeSegment constructor
    // ------------------------------------------------------------------

    public PipeSegment()
    {
    }

    // ------------------------------------------------------------------
    // PipeSegment public member implementation
    // ------------------------------------------------------------------

    //
    // Receive an event
    //
    // This code is carefully crafted to maximize concurrency. First a
    // copy of the child list is made. Next receive is called on all child
    // pipe segments -- this is fully concurrent. Any children that should
    // be reaped are added to a reap list. Once the receive is complete
    // the elements in the reap list are removed from the list of children.
    //
    public boolean receive(Event_impl event)
    {
	if(destroy_)
	    return false;

	Vector copy = null;

	synchronized(children_)
	{
	    copy = (Vector)children_.clone();
	}

	Vector reapList = new Vector();
	for(int i = 0 ; i < copy.size() ; i++)
	{
	    PipeSegment pipeSegment = (PipeSegment)copy.elementAt(i);
	    if(!pipeSegment.receive(event))
		reapList.addElement(pipeSegment);
	}

	synchronized(children_)
	{
	    if(reapList.size() > 0)
		reap(reapList);

	    return true;
	}
    }

    //
    // Connect a PipeSegment as a child of this PipeSegment
    //
    public void connect(PipeSegment child)
    {
	synchronized(children_)
	{
	    children_.addElement(child);
	}
    }

    //
    // Destroy this PipeSegment
    //
    public void destroy()
    {
        destroy_ = true;
    }
}
