// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// *********************************************************************

package com.ooc.CosEventServer;

import org.omg.CORBA.*;

//
// Implementation for ProxyBase
//
public class ProxyBaseImpl
{
    //
    // My ORB
    //
    private ORB orb_;

    //
    // My parent pipe segment
    //
    private PipeSegment parentPipe_;

    //
    // My pipe segment (nil for proxy consumers)
    //
    private PipeSegment myPipe_;

    //
    // My proxy
    //
    private ProxyBase proxy_;

    //
    // My id
    //
    private long id_;

    //
    // My name
    //
    private String name_;

    //
    // The last time stampRequest() was called
    //
    private long lastRequestTime_;

    //
    // The proxy state
    //
    private ProxyState state_;

    // ----------------------------------------------------------------------
    // ProxyBaseImpl constructor/finalize
    // ----------------------------------------------------------------------

    public ProxyBaseImpl(ProxyConstructorInfo info, ProxyBase proxy)
    {
	proxy_ = proxy;
	lastRequestTime_ = System.currentTimeMillis();
	state_ = ProxyState.Disconnected;
	orb_ = info.orb;
	id_ = info.id;
	parentPipe_ = info.pipe;

	name_ = info.base + "-" + id_;

	if(com.ooc.CORBA.MessageViewer.instance().getTraceLevel() >= 2)
	{
	    com.ooc.CORBA.MessageViewer.instance().trace(2,
		"Create " + name_ + " from " +
		EventUtil.getConnectionAddrDescription(orb_));
	}
    }

    protected void finalize()
        throws Throwable
    {
	if(com.ooc.CORBA.MessageViewer.instance().getTraceLevel() >= 2)
	    com.ooc.CORBA.MessageViewer.instance().trace(2, "Destroy " + name_);
    }

    // ----------------------------------------------------------------------
    // ProxyBaseImpl public member implementation
    // ----------------------------------------------------------------------

    //
    // Resolve the QoS parameters
    //
    public void resolveQoSParams(QoSParameters params)
    {
	try
	{
	    com.ooc.CORBA.Properties properties =
		com.ooc.CORBA.Properties.instance();
	    params.retryTimeout =
		Long.parseLong(properties.getProperty(EventQoS.RETRY_TIMEOUT));
	    params.retryMultiplier =
		Double.valueOf(properties.getProperty(
		    EventQoS.RETRY_MULTIPLIER)).doubleValue();
	    params.maxRetries =
		Long.parseLong(properties.getProperty(EventQoS.MAX_RETRIES));
	    params.maxEvents =
		Long.parseLong(properties.getProperty(EventQoS.MAX_EVENTS));
	    params.pullInterval =
		Long.parseLong(properties.getProperty(EventQoS.PULL_INTERVAL));
	}
	catch(NumberFormatException e)
	{
	    throw new InternalError();
	}
    }

    //
    // Disconnect the proxy
    //
    public void disconnect()
    {
	if(state_ == ProxyState.Destroyed)
	    return;

	state_ = ProxyState.Destroyed;

	//
	// Destroy the supplier event pipe
	//
	if(myPipe_ != null)
	    myPipe_.destroy();

	proxy_.disconnectPeer();
	proxy_.deactivateObject();
    }

    //
    // Don't accept further requests
    //
    public void discardRequests()
    {
	if(state_ == ProxyState.Destroyed || state_ == ProxyState.Discarding)
	    return;

	state_ = ProxyState.Discarding;

	//
	// Destroy the supplier event pipe
	//
	if(myPipe_ != null)
	    myPipe_.destroy();
    }

    //
    // Has this proxy been destroyed?
    //
    public boolean destroyed()
    {
	return state_ == ProxyState.Destroyed;
    }

    //
    // What was the time of the last request?
    //
    public long lastRequestTime()
    {
	return lastRequestTime_;
    }

    //
    // Set the time of the last request
    //
    public void stampRequest()
    {
	lastRequestTime_ = System.currentTimeMillis();
    }

    //
    // Get the name of the proxy
    //
    public String getName()
    {
	return name_;
    }

    //
    // Get the ORB
    //
    public ORB getOrb()
    {
	return orb_;
    }

    //
    // Get the parent pipe segment
    //
    public PipeSegment getParentPipe()
    {
	return parentPipe_;
    }

    //
    // Get my pipe segment (nil for proxy consumers)
    //
    public PipeSegment getMyPipe()
    {
	return myPipe_;
    }

    //
    // Set my pipe segment
    //
    public void setMyPipe(PipeSegment pipe)
    {
	myPipe_ = pipe;
    }

    //
    // Get the id
    //
    public long getId()
    {
	return id_;
    }

    //
    // Get the proxy state
    //
    public ProxyState getState()
    {
	return state_;
    }

    //
    // Set the proxy state
    //
    public void setState(ProxyState state)
    {
	state_ = state;
    }
}
