// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// *********************************************************************

package com.ooc.CosEventServer;

import java.util.Vector;

public class Reaper extends Thread
{
    //
    // Is the reaper done?
    //
    private boolean done_;

    //
    // How long does the reaper sleep between reaps (seconds)?
    //
    private long sleepTime_;

    //
    // How long can a proxy be inactive before its reaped (seconds)?
    //
    private long reapTime_;

    //
    // All proxies
    //
    private Vector proxies_ = new Vector();

    // ------------------------------------------------------------------
    // Reaper private member implementation
    // ------------------------------------------------------------------

    private void reap()
    {
	if(com.ooc.CORBA.MessageViewer.instance().getTraceLevel() >= 2)
	    com.ooc.CORBA.MessageViewer.instance().trace(2, "Reaping");

	long now = System.currentTimeMillis();

	//
	// Find the element, remove it from the sequence.
	//
	int i = 0;
	while(i < proxies_.size())
	{
	    ProxyBase proxy = (ProxyBase)proxies_.elementAt(i);
	    if(proxy.destroyed())
	    {
		proxies_.removeElementAt(i);
		continue;
	    }
	    if(proxy.canReap())
	    {
		long last = proxy.lastRequestTime();
		if(now - last >= reapTime_ * 1000)
		{
		    if(com.ooc.CORBA.MessageViewer.instance().
			getTraceLevel() >= 2)
		    {
			com.ooc.CORBA.MessageViewer.instance().trace(2,
			    "Reaping: " + proxy.getName());
		    }
		    proxy.disconnect();
		    proxies_.removeElementAt(i);
		    continue;
		}
	    }
	    ++i;
	}
    }

    // ------------------------------------------------------------------
    // Reaper constructor
    // ------------------------------------------------------------------

    public Reaper()
    {
	done_ = false;
	sleepTime_ = 30*60; // 30 minutes
	reapTime_ = 60*60 * 4; // 4 hours

	com.ooc.CORBA.Properties props = com.ooc.CORBA.Properties.instance();

	String val = props.getProperty("ooc.event.reap_frequency");
	if(val != null)
	{
	    try
	    {
		sleepTime_ = Long.parseLong(val);
	    }
	    catch(NumberFormatException e)
	    {
	    }
	}

	val = props.getProperty("ooc.event.inactivity_timeout");
	if(val != null)
	{
	    try
	    {
		reapTime_ = Long.parseLong(val);
	    }
	    catch(NumberFormatException e)
	    {
	    }
	}
    }

    // ------------------------------------------------------------------
    // Reaper public member implementation
    // ------------------------------------------------------------------

    public synchronized void add(ProxyBase proxy)
    {
	//
	// When this method is called, canReap() will only return
	// false for proxies that can never be reaped.
	//
	if(proxy.canReap())
	    proxies_.addElement(proxy);
    }

    public synchronized void destroy()
    {
	done_ = true;
	notify();
    }

    public synchronized void run()
    {
	if(sleepTime_ == 0)
	{
	    if(com.ooc.CORBA.MessageViewer.instance().getTraceLevel() >= 2)
		com.ooc.CORBA.MessageViewer.instance().warning(
		    "Reaping disabled");
	    return;
	}

	while(!done_)
	{
	    try
	    {
		wait(sleepTime_ * 1000);
	    }
	    catch(InterruptedException e)
	    {
	    }
	    if(!done_)
		reap();
	}
    }
}
