// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTypedEventChannelAdmin.impl;

import java.util.*;
import com.ooc.CosEventServer.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ContainedPackage.*;

class ChannelType
{
    //
    // The ORB.
    //
    private ORB orb_;

    //
    // Repository IDs for the channel.
    //
    private Vector typeIds_ = new Vector();

    //
    // The operations supported by the channel's interface. 
    //
    private Vector operations_ = new Vector();

    // ------------------------------------------------------------------
    // ChannelType private member implementation
    // ------------------------------------------------------------------

    //
    // Get assigned type-ids for this channel.
    //
    private void typeIds(InterfaceDef iface)
    {
	typeIds_.addElement(iface.id());

	InterfaceDef[] base = iface.base_interfaces();
	for(int i = 0 ; i < base.length ; ++i)
	{
	    //
	    // Get the ID.
	    //
	    String id = base[i].id();

	    //
	    // Find out whether this ID is already in the set of IDs.
	    //
	    int j;
	    for(j = 0 ; j < typeIds_.size() ; ++j)
	    {
		String typeId = (String)typeIds_.elementAt(j);
		if(typeId.equals(id))
		    break;
	    }

	    //
	    // Nope, then add all the typeIds for this interface.
	    //
	    if(j == typeIds_.size())
		typeIds(base[i]);
	}
    }

    // ------------------------------------------------------------------
    // ChannelType constructor
    // ------------------------------------------------------------------

    public ChannelType(ORB orb)
    {
	orb_ = orb;
    }

    // ------------------------------------------------------------------
    // ChannelType public member implementation
    // ------------------------------------------------------------------

    //
    // Verify if the channel has a compatible typeId assigned.
    //
    public boolean isCompatible(String id)
    {
	for(int i = 0; i < typeIds_.size(); ++i)
	{
	    String typeId = (String)typeIds_.elementAt(i);
	    if(typeId.equals(id))
		return true;
	}
	return false;
    }

    //
    // Assign the type-id to the channel, if not already assigned.
    //
    public boolean assignTypeId(String id)
    {
	//
	// Do we already have a type-id assigned?
	//
	if(typeIds_.size() != 0)
	    return isCompatible(id);

	Repository ir = null;
	try
	{
	    org.omg.CORBA.Object irObj =
		orb_.resolve_initial_references("InterfaceRepository");
	    ir = RepositoryHelper.narrow(irObj);
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    //
	    // This is really an internal error since we ensure that there
	    // is a InterfaceRepository at process startup.
	    //
	}
	//
	// We want COMM_FAILURE and TRANSIENT exceptions to cause
	// INTF_REPOS exceptions to be raised. Another valid choice would
	// be InterfaceNotSupported. However, I have chosen to raise
	// INTF_REPOS in the case that the IR isn't available so that the
	// user can differentiate this case from the ID not present in the
	// IR.
	//
	catch(COMM_FAILURE ex)
	{
	}
	catch(TRANSIENT ex)
	{
	}
	if(ir == null)
	    throw new INTF_REPOS();
	
	Contained contained = ir.lookup_id(id);
	
	//
	// Not in the interface repository? Not supported.
	//
	if(contained == null)
	    return false;
	
	InterfaceDef iface = InterfaceDefHelper.narrow(contained);
	if(iface == null)
	    return false;

	//
	// Retrieve the type-ids.
	//
	typeIds(iface);

	//
	// Query off operation descriptions for all supported operations.
	//
	Contained[] ops = iface.contents(DefinitionKind.dk_Operation, false);

	operations_.removeAllElements();

	for(int i = 0 ; i < ops.length ; i++)
	{
	    Description description = ops[i].describe();

	    OperationDescription opDesc = null;
	    try
	    {
		opDesc = OperationDescriptionHelper.extract(description.value);
	    }
	    catch(BAD_OPERATION ex)
	    {
	    }
	    if (opDesc == null)
		throw new InternalError();

	    operations_.addElement(opDesc);
	}

	return true;
    }


    //
    // Get the operations available on the channel type. 
    //
    public OperationDescription[]
    getOperations()
    {
	OperationDescription[] rc =
		new OperationDescription[operations_.size()];
	for(int i = 0; i < operations_.size(); ++i)
	    rc[i] = (OperationDescription)operations_.elementAt(i);
	return rc;
    }

    public String[]
    getTypeIds()
    {
	String[] rc = new String[typeIds_.size()];
	for(int i = 0; i < typeIds_.size(); ++i)
	    rc[i] = (String)typeIds_.elementAt(i);
	return rc;
    }
}
