// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTypedEventChannelAdmin.impl;

import com.ooc.CosEventServer.*;
import org.omg.CORBA.*;

class ProxyPushSupplier
    extends org.omg.CosEventChannelAdmin._ProxyPushSupplierImplBase
    implements ProxyBase
{
    //
    // The ProxyBaseImpl Object
    //
    private ProxyBaseImpl proxyImpl_;

    //
    // My consumer.
    //
    private org.omg.CosEventComm.PushConsumer consumer_;

    // ------------------------------------------------------------------
    // ProxyPushSupplier constructor
    // ------------------------------------------------------------------

    ProxyPushSupplier(ProxyConstructorInfo info)
    {
	info.base = "TypedProxyPushSupplier";
	proxyImpl_ = new ProxyBaseImpl(info, this);
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------
    
    synchronized public void
    connect_push_consumer(org.omg.CosEventComm.PushConsumer consumer)
	throws org.omg.CosEventChannelAdmin.AlreadyConnected
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	if(proxyImpl_.getState() == ProxyState.Connected)
	    throw new org.omg.CosEventChannelAdmin.AlreadyConnected();

	if(consumer == null)
	    throw new BAD_PARAM();

	consumer_ = consumer;
	
	proxyImpl_.setState(ProxyState.Connected);

	QoSParameters params = new QoSParameters();
	proxyImpl_.resolveQoSParams(params);
	    
	PusherBaseThread pusher =
	    new TypedPusherThread(consumer, this, proxyImpl_.getId(),
				  params.maxEvents, params.maxRetries,
				  params.retryTimeout, params.retryMultiplier);
	pusher.start();

	if(proxyImpl_.getMyPipe() != null)
	    throw new InternalError();
	proxyImpl_.setMyPipe(new PushPipeSegment(pusher));
	proxyImpl_.getParentPipe().connect(proxyImpl_.getMyPipe());
    }

    synchronized public void
    disconnect_push_supplier()
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	proxyImpl_.disconnect();
    }

    // ------------------------------------------------------------------
    // ProxyBase implementation
    // ------------------------------------------------------------------

    //
    // Disconnect the servant from the ORB
    //
    public void deactivateObject()
    {
	proxyImpl_.getOrb().disconnect(this);
    }
    
    //
    // Called during disconnect
    //
    public void disconnectPeer()
    {
	//
	// If we're connected then tell the consumer.
	//
	if(consumer_ != null)
	{
	    try
	    {
		consumer_.disconnect_push_consumer();
	    }                                              
	    catch(SystemException e)
	    {
	    }
	    consumer_ = null;
	}
    }

    //
    // Disconnect the proxy
    //
    public synchronized void disconnect()
    {
	proxyImpl_.disconnect();
    }

    //
    // Don't accept further requests
    //
    public synchronized void discardRequests()
    {
	proxyImpl_.discardRequests();
    }

    //
    // Has this proxy been destroyed?
    //
    public synchronized boolean destroyed()
    {
	return proxyImpl_.destroyed();
    }

    //
    // Can this proxy be reaperd?
    //
    public boolean canReap()
    {
	return false;
    }

    //
    // What was the time of the last request?
    //
    public long lastRequestTime()
    {
	return proxyImpl_.lastRequestTime();
    }

    //
    // Set the time of the last request
    //
    public void stampRequest()
    {
	proxyImpl_.stampRequest();
    }

    //
    // Get the name of the proxy
    //
    public String getName()
    {
	return proxyImpl_.getName();
    }
}
