// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTypedEventChannelAdmin.impl;

import java.util.*;
import org.omg.CORBA.*;
import com.ooc.CosEventServer.*;

public class TypedEventChannel
    extends org.omg.CosTypedEventChannelAdmin._TypedEventChannelImplBase
{
    //
    // The ORB.
    //
    private ORB orb_;

    //
    // My pipe segment.
    //
    private PipeSegment myPipe_;

    //
    // Channel type information.
    //
    private ChannelType type_;

    //
    // The consumer admin.
    //
    private TypedConsumerAdmin consumerAdmin_;

    //
    // The supplier admin.
    //
    private TypedSupplierAdmin supplierAdmin_;

    //
    // The channel's Id.
    //
    private String myId_;

    //
    // Has the channel been destroyed?
    //
    private boolean destroy_ = false;

    // -----------------------------------------------------------------
    // TypedEventChannel constructor
    // -----------------------------------------------------------------

    public TypedEventChannel(ORB orb, String myId, Reaper reaper)
    {
	orb_ = orb;
	myId_ = myId;
	myPipe_ = new PipeSegment();
	type_ = new ChannelType(orb);
        consumerAdmin_ = new TypedConsumerAdmin(orb_, reaper, myPipe_, type_);
        supplierAdmin_ = new TypedSupplierAdmin(orb_, reaper, myPipe_, type_);

	SetDefaultProperties.setDefaults();
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------
    
    synchronized public org.omg.CosTypedEventChannelAdmin.TypedConsumerAdmin
    for_consumers()
    {
	if(destroy_)
	    throw new OBJECT_NOT_EXIST();

	return consumerAdmin_;
    }

    synchronized public org.omg.CosTypedEventChannelAdmin.TypedSupplierAdmin
    for_suppliers()
    {
	if(destroy_)
	    throw new OBJECT_NOT_EXIST();

	return supplierAdmin_;
    }

    synchronized public void
    destroy()
    {
	if(destroy_)
	    throw new OBJECT_NOT_EXIST();

	destroy_ = true;

	orb_.disconnect(this);
        
	myPipe_.destroy();

        consumerAdmin_.disconnect();
        supplierAdmin_.disconnect();
    }

    // ------------------------------------------------------------------
    // Internal methods
    // ------------------------------------------------------------------
    
    synchronized public void
    discardRequests()
    {
	if(destroy_)
	    return;

	myPipe_.destroy();

	consumerAdmin_.discardRequests();
	supplierAdmin_.discardRequests();
    }

    synchronized public boolean
    destroyed()
    {
	return destroy_;
    }
}
