// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTypedEventChannelAdmin.impl;

import com.ooc.CosEventServer.*;
import org.omg.CORBA.*;

public class TypedProxyPushConsumer
    extends org.omg.CosTypedEventChannelAdmin._TypedProxyPushConsumerImplBase
    implements ProxyBase
{
    //
    // The ProxyBaseImpl object
    //
    private ProxyBaseImpl proxyImpl_;

    //
    // My push supplier.
    //
    private org.omg.CosEventComm.PushSupplier supplier_;

    //
    // The DSI implementation
    //
    private TypedProxyPushConsumerDSI dsiImpl_;

    // ------------------------------------------------------------------
    // TypedProxyPushConsumerDSI constructor
    // ------------------------------------------------------------------

    TypedProxyPushConsumer(ProxyConstructorInfo info, ChannelType type)
    {
	info.base = "TypedProxyPushConsumer";
	proxyImpl_ = new ProxyBaseImpl(info, this);

	dsiImpl_ = new TypedProxyPushConsumerDSI(info, type, this);
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------
    
    synchronized public org.omg.CORBA.Object
    get_typed_consumer()
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	return dsiImpl_;
    }

    synchronized public void
    connect_push_supplier(org.omg.CosEventComm.PushSupplier supplier)
	throws org.omg.CosEventChannelAdmin.AlreadyConnected
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	if(proxyImpl_.getState() == ProxyState.Connected)
	    throw new org.omg.CosEventChannelAdmin.AlreadyConnected();

	proxyImpl_.stampRequest();

	//
	// The supplier is permitted to be nil
	//
	if(supplier != null)
	    supplier_ = supplier;

	//
	// Technically it's not permitted to "push" events until the
	// channel is connected. However, the spec doesn't say what do
	// to in the typed channel case.
	//
	proxyImpl_.setState(ProxyState.Connected);
    }

    synchronized public void
    disconnect_push_consumer()
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	proxyImpl_.disconnect();
    }
    
    public void
    push(Any any)
	throws org.omg.CosEventComm.Disconnected
    {
	throw new NO_IMPLEMENT();
    }

    // ------------------------------------------------------------------
    // ProxyBase implementation
    // ------------------------------------------------------------------

    //
    // Disconnect the servant from the ORB
    //
    public void deactivateObject()
    {
	//
	// Disconnect the DSI implementation.
	//
	dsiImpl_.disconnect();
	    
	proxyImpl_.getOrb().disconnect(this);
    }
    
    //
    // Called during disconnect
    //
    public void disconnectPeer()
    {
	//
	// Since we don't necessarily have a supplier, we need to verify
	// before invoking a method.
	//
	if(supplier_ != null)
	{
	    try
	    {
		supplier_.disconnect_push_supplier();
	    }
	    catch(SystemException e)
	    {
	    }
	    supplier_ = null;
	}
    }

    //
    // Disconnect the proxy
    //
    public synchronized void disconnect()
    {
	proxyImpl_.disconnect();
    }

    //
    // Don't accept further requests
    //
    public synchronized void discardRequests()
    {
	proxyImpl_.discardRequests();

	//
	// Ask the DSI implementation to go away.
	//
	dsiImpl_.discardRequests();
    }

    //
    // Has this proxy been destroyed?
    //
    public synchronized boolean destroyed()
    {
	return proxyImpl_.destroyed();
    }

    //
    // Can this proxy be reaperd?
    //
    public boolean canReap()
    {
	return true;
    }

    //
    // What was the time of the last request?
    //
    public long lastRequestTime()
    {
	return proxyImpl_.lastRequestTime();
    }

    //
    // Set the time of the last request
    //
    public void stampRequest()
    {
	proxyImpl_.stampRequest();
    }

    //
    // Get the name of the proxy
    //
    public String getName()
    {
	return proxyImpl_.getName();
    }
}
