// **********************************************************************
//
// Copyright (c) 2000 
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosTypedEventChannelAdmin.*;
import org.omg.CosEventChannelAdmin.*;
import java.io.*;
import java.util.*;

public class Admin
{
    private static int id = 0;

    private static int 
    GetNextID()
    {
	return id++;
    }

    private static void
    Usage() 
    {
	System.err.println("Usage: admin\n\n" +
		       "\tShutdown\n" +
	               "\tCreateChannel\n" +
	               "\tCreateConsumerAdmin <channel-ior>\n" +
	               "\tCreateSupplierAdmin <channel-ior>\n" +
	               "\tCreatePushConsumer <admin-ior>\n" +
	               "\tCreatePushSupplier <admin-ior>\n" +
	               "\tCreatePullConsumer <admin-ior>\n" +
	               "\tCreatePullSupplier <admin-ior>\n" +
	               "\tDestroyChannel <channel-ior>\n" +
	               "\tDestroyProxyConsumer <proxy-ior>\n" + 
	               "\tDestroyProxySupplier <proxy-ior>\n" +
	               "\tTypedShutdown\n" +
	               "\tCreateTypedChannel\n" +
	               "\tCreateTypedConsumerAdmin <channel-ior>\n" +
	               "\tCreateTypedSupplierAdmin <channel-ior>\n" +
	               "\tCreateTypedPushConsumer <admin-ior>\n" +
	               "\tCreateTypedPushSupplier <admin-ior>\n" +
	               "\tCreateTypedPullConsumer <admin-ior>\n" +
	               "\tCreateTypedPullSupplier <admin-ior>\n" +
	               "\tDestroyTypedChannel <channel-ior>\n" +
	               "\tDestroyTypedProxyConsumer <proxy-ior>\n" + 
	               "\tDestroyTypedProxySupplier <proxy-ior>\n");
    }

    private static com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactory
    GetTypedFactory(ORB orb)
    {
	com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactory factory;
	org.omg.CORBA.Object obj;
	try
	{
	    obj = orb.resolve_initial_references("TypedEventChannelFactory");
	    if(obj == null)
	    {
		TSU.PrintStatusMsg("Admin", 1, 
		                   "TypedEventChannelFactory not found");
		System.err.println("ERROR: No `TypedEventChannelFactory' " +
				   "initial reference."); 
		System.exit(1);
	    }
	    
	    factory = com.ooc.OBTypedEventChannelFactory.
			  TypedEventChannelFactoryHelper.narrow(obj);
	    if(factory == null)
	    {
		TSU.PrintStatusMsg("Admin", 1, "TypedEventChannelFactory is " +
			           "not a TypedEventChannelFactory");
		System.out.println("ERROR: `TypedEventChannelFactory' " +
				   "initial reference is not a " +
				   "TypedEventChannelFactory");
		System.exit(1);
	    }
	    return factory;
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    System.err.println("Can't resolve `TypedEventChannelFactory`");
	    System.exit(1);
	}
	return null;
    }

    private static com.ooc.OBEventChannelFactory.EventChannelFactory
    GetFactory(ORB orb)
    {
	com.ooc.OBEventChannelFactory.EventChannelFactory factory;
	org.omg.CORBA.Object obj;
	try
	{
	    obj = orb.resolve_initial_references("EventChannelFactory");
	
	    if(obj == null)
	    {
		TSU.PrintStatusMsg("Admin", 1, "EventChannelFactory not found");
		System.err.println("ERROR: No `EventChannelFactory' " +
				   "initial reference."); 
		System.exit(1);
	    }

	    factory = com.ooc.OBEventChannelFactory.
			  EventChannelFactoryHelper.narrow(obj);
	    if(factory == null)
	    {
		TSU.PrintStatusMsg("Admin", 1, "EventChannelFactory is not" +
				   " an EventChannelFactory");
		System.out.println("ERROR: `EventChannelFactory' initial " +
				   "reference is not an EventChannelFactory");
		System.exit(1);
	    }
	    return factory;
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    System.err.println("Can't resolve `EventChannelFactory`");
	    System.exit(1);
	}
	return null;
    }

    private static EventChannel
    GetChannel(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    EventChannel channel = EventChannelHelper.narrow(obj);
	    if (channel == null)
	    {
	    	System.err.println("Couldn't narrow channel");
		System.exit(1);
	    }
	    return channel;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static TypedEventChannel
    GetTypedChannel(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    TypedEventChannel channel = TypedEventChannelHelper.narrow(obj);
	    if (channel == null)
	    {
	    	System.err.println("Couldn't narrow channel");
		System.exit(1);
	    }
	    return channel;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static ConsumerAdmin
    GetConsumerAdmin(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    ConsumerAdmin admin = ConsumerAdminHelper.narrow(obj);
	    if (admin == null)
	    {
	    	System.err.println("Couldn't narrow Consumer Admin");
		System.exit(1);
	    }
	    return admin;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static TypedConsumerAdmin
    GetTypedConsumerAdmin(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    TypedConsumerAdmin admin = TypedConsumerAdminHelper.narrow(obj);
	    if (admin == null)
	    {
	    	System.err.println("Couldn't narrow Consumer Admin");
		System.exit(1);
	    }
	    return admin;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static SupplierAdmin
    GetSupplierAdmin(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    SupplierAdmin admin = SupplierAdminHelper.narrow(obj);
	    if (admin == null)
	    {
	    	System.err.println("Couldn't narrow Supplier Admin");
		System.exit(1);
	    }
	    return admin;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static TypedSupplierAdmin
    GetTypedSupplierAdmin(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    TypedSupplierAdmin admin = TypedSupplierAdminHelper.narrow(obj);
	    if (admin == null)
	    {
	    	System.err.println("Couldn't narrow Supplier Admin");
		System.exit(1);
	    }
	    return admin;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static ProxyPullSupplier
    GetProxyPullSupplier(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    ProxyPullSupplier proxy = ProxyPullSupplierHelper.narrow(obj);
	    return proxy;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static ProxyPushSupplier
    GetProxyPushSupplier(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    ProxyPushSupplier proxy = ProxyPushSupplierHelper.narrow(obj);
	    return proxy;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static ProxyPullConsumer
    GetProxyPullConsumer(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    ProxyPullConsumer proxy = ProxyPullConsumerHelper.narrow(obj);
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static ProxyPushConsumer
    GetProxyPushConsumer(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    ProxyPushConsumer proxy = ProxyPushConsumerHelper.narrow(obj);
	    return proxy;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private static TypedProxyPushConsumer
    GetTypedProxyPushConsumer(String ior, ORB orb)
    {
	try
	{
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);
	    TypedProxyPushConsumer proxy = 
		TypedProxyPushConsumerHelper.narrow(obj);
	    return proxy;
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    public static void 
    Shutdown(ORB orb)
    {
	com.ooc.OBEventChannelFactory.EventChannelFactory factory = 
	    GetFactory(orb);
	try
	{
	    factory.shutdown();
	}
	catch(org.omg.CORBA.TRANSIENT ex)
	{
	    //
	    // This call will deactivate the server and thus
	    // throw an exception.
	    //
	}
    }
	
    public static void 
    CreateChannel(ORB orb)
    {
	com.ooc.OBEventChannelFactory.EventChannelFactory factory = 
	    GetFactory(orb);
        String id = "";
	try
	{
	    id += GetNextID();
	    EventChannel channel = factory.create_channel(id);

	    TSU.PrintStatusMsg("Admin", 1, "Created Channel " + id);

	    String ior = orb.object_to_string(channel);
	    System.out.println(ior);
	}
	catch(com.ooc.OBEventChannelFactory.ChannelAlreadyExists ex)
	{
	    TSU.PrintStatusMsg("Admin", 1, "Channel already exists");
	    System.out.println("-1");
	}
	catch(SystemException ex)
	{
	    System.err.println("Exception when creating channel " + id);
	    ex.printStackTrace(); 
	    System.exit(1);
	}
    }	

    private static void 
    DestroyChannel(String ch_ior, ORB orb)
    {
	EventChannel channel = GetChannel(ch_ior, orb);
	channel.destroy();
	TSU.PrintStatusMsg("Admin", 1, "Destroyed Channel");
    }   

    public static void
    CreateConsumerAdmin(String ch_ior, ORB orb)
    {
	EventChannel channel = GetChannel(ch_ior, orb);
	ConsumerAdmin consumer = channel.for_consumers();
	String ior = orb.object_to_string(consumer);
	TSU.PrintStatusMsg("Admin", 1, "Created Consumer Admin");
	System.out.println(ior);
    }

    public static void
    CreateSupplierAdmin(String ior, ORB orb)
    {
	EventChannel channel = GetChannel(ior, orb);
	SupplierAdmin supplier = channel.for_suppliers();
	String supplier_ior = orb.object_to_string(supplier);
	TSU.PrintStatusMsg("Admin", 1, "Created Supplier Admin");
	System.out.println(supplier_ior);
    }

    public static void
    CreateConsumer(String ior, ORB orb, String model) 
    {
	SupplierAdmin supplier = GetSupplierAdmin(ior, orb);
	String consumer_ior;
	
	if(model.equalsIgnoreCase("CreatePushConsumer"))
	{
	    ProxyPushConsumer consumer = supplier.obtain_push_consumer();
	    consumer_ior = orb.object_to_string(consumer);
	}
	else
	{
	    ProxyPullConsumer consumer = supplier.obtain_pull_consumer();
	    consumer_ior = orb.object_to_string(consumer);
	}
	TSU.PrintStatusMsg("Admin", 1, "Created Proxy Consumer");
	System.out.println(consumer_ior);
    }

    public static void
    CreateSupplier(String ch_ior, ORB orb, String model) 
    {
	ConsumerAdmin consumer = GetConsumerAdmin(ch_ior, orb);
	String ior;
	
	if(model.equalsIgnoreCase("CreatePushSupplier"))
	{
	    ProxyPushSupplier supplier = consumer.obtain_push_supplier();
	    ior = orb.object_to_string(supplier);
	}
	else
	{
	    ProxyPullSupplier supplier = consumer.obtain_pull_supplier();
	    ior = orb.object_to_string(supplier);
	}
	TSU.PrintStatusMsg("Admin", 1, "Created Proxy Supplier");
	System.out.println(ior);
    }

    private static void
    DestroyProxySupplier(String ior, ORB orb) 
    {
	//
	// Push Supplier
	//
	ProxyPushSupplier pushSupplier = GetProxyPushSupplier(ior, orb);
	if(pushSupplier != null)
	{
	    pushSupplier.disconnect_push_supplier();
	    TSU.PrintStatusMsg("Admin", 1, "Destroyed Proxy Push Supplier");
	    return;
	}
	
	//
	// Pull Supplier
	//
	ProxyPullSupplier pullSupplier = GetProxyPullSupplier(ior, orb);
	if(pullSupplier != null)
	{
	    pullSupplier.disconnect_pull_supplier();
	    TSU.PrintStatusMsg("Admin", 1, "Destroyed Proxy Pull Supplier");
	    return;
	}
    }

    private static void
    DestroyProxyConsumer(String ior, ORB orb) 
    {
	//
	// Push Consumer 
	//
	ProxyPushConsumer pushConsumer = GetProxyPushConsumer(ior, orb);
	if(pushConsumer != null)
	{
	    pushConsumer.disconnect_push_consumer();
	    TSU.PrintStatusMsg("Admin", 1, "Destroyed Proxy Push Consumer");
	    return;
	}
	
	//
	// PullConsumer 
	//
	ProxyPullConsumer pullConsumer = GetProxyPullConsumer(ior, orb);
	if(pullConsumer != null)
	{
	    pullConsumer.disconnect_pull_consumer();
	    TSU.PrintStatusMsg("Admin", 1, "Destroyed Proxy Push Supplier");
	    return;
	}
    }

    private static void
    TypedShutdown(ORB orb)
    {
	com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactory factory =
	    GetTypedFactory(orb); 
	try
	{
	    factory.shutdown();
	}
	catch(org.omg.CORBA.TRANSIENT ex)
	{
	    //
	    // This call will deactivate the server and thus
	    // throw an exception.
	    //
	}
	catch(SystemException ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
    }
	
    private static void 
    CreateTypedChannel(ORB orb)
    {
	try
	{
	    TypedEventChannel e;
	    try
	    {
		org.omg.CORBA.Object obj =
		    orb.resolve_initial_references("TypedEventService");
		e = TypedEventChannelHelper.narrow(obj);
		if(e == null)
		{
		    System.err.println("`TypedEventService' is not a Typed " +
				       "Event Channel");
		    System.exit(1);
		}
		
		TSU.PrintStatusMsg("Admin", 1, "Created default event channel");
		String ior = orb.object_to_string(e);
		System.out.println(ior);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("`TypedEventService' is an invalid name");
		System.exit(1);
	    }
	}
	catch(SystemException ex)
	{
	    System.err.println("Exception caught when creating channel");
	    ex.printStackTrace();
	    System.exit(1);
	}
    }	

    private static void
    DestroyTypedChannel(String ch_ior, ORB orb)
    {
	TypedEventChannel channel = GetTypedChannel(ch_ior, orb);
	channel.destroy();
	TSU.PrintStatusMsg("Admin", 1, "Destroyed Channel");
    }   

    private static void
    CreateTypedConsumerAdmin(String ch_ior, ORB orb)
    {
	TypedEventChannel channel = GetTypedChannel(ch_ior, orb);
	TypedConsumerAdmin consumer = channel.for_consumers();
	String ior = orb.object_to_string(consumer);
	TSU.PrintStatusMsg("Admin", 1, "Create Typed Consumer Admin");
        System.out.println(ior);
    }

    private static void
    CreateTypedSupplierAdmin(String ch_ior, ORB orb)
    {
	TypedEventChannel channel = GetTypedChannel(ch_ior, orb);
	TypedSupplierAdmin supplier = channel.for_suppliers();
	String ior = orb.object_to_string(supplier);
	TSU.PrintStatusMsg("Admin", 1, "Create Typed Supplier Admin");
        System.out.println(ior);
    }

    private static void
    CreateTypedConsumer(String ch_ior, String ref, ORB orb, String type)
    {
	TypedSupplierAdmin supplier = GetTypedSupplierAdmin(ch_ior, orb);
	String ior;
	
	try
	{
	    if(type.equalsIgnoreCase("CreateTypedPushConsumer"))
	    {
		TypedProxyPushConsumer consumer = 
		    supplier.obtain_typed_push_consumer(ref);
		ior = orb.object_to_string(consumer);
		TSU.PrintStatusMsg("Admin", 1, "Created Typed Proxy Consumer");
		System.out.println(ior);
	    }
	    else
	    {
		ProxyPullConsumer consumer = 
		    supplier.obtain_typed_pull_consumer(ref);
		ior = orb.object_to_string(consumer);
		TSU.PrintStatusMsg("Admin", 1, "Created Typed Proxy Consumer");
		System.out.println(ior);
	    }
	}
	catch(InterfaceNotSupported ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	catch(NoSuchImplementation ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
    }

    private static void
    CreateTypedSupplier(String ch_ior, String ref, ORB orb)
    {
	TypedConsumerAdmin consumer = GetTypedConsumerAdmin(ch_ior, orb);
	String ior;
	
	try
	{
	    ProxyPushSupplier supplier = 
		consumer.obtain_typed_push_supplier(ref);
	    ior = orb.object_to_string(supplier);
	    TSU.PrintStatusMsg("Admin", 1, "Created Typed Proxy Supplier");
	    System.out.println(ior);
	}
	catch(NoSuchImplementation ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
    }

    private static void
    DestroyTypedProxyConsumer(String ior, ORB orb) 
    {
	//
	// Push Consumer 
	//
	TypedProxyPushConsumer pushConsumer =
	    GetTypedProxyPushConsumer(ior, orb);
	if(pushConsumer != null)
	{
	    pushConsumer.disconnect_push_consumer();
	    TSU.PrintStatusMsg("Admin", 1, 
				"Destroyed Typed Proxy Push Consumer");
	}
    }


    static int 
    run(ORB orb, String args[])    
    {
	if(args.length < 1)
	{
	    Usage();
	    return 1;
	}
	
	//
	// See what the command is and evaluate the proper function.
	//
	TSU.PrintStatusMsg("Admin", 3, "Received instruction: " + 
			    args[0]);
 
	if(args[0].equalsIgnoreCase("Shutdown"))
	{
	    if(args.length != 1)
	    {
		Usage();
		return 1;
	    }
	    Shutdown(orb);
	}
	else if(args[0].equalsIgnoreCase("CreateChannel"))
	{
	    if(args.length != 1)
	    {
		Usage();
		return 1;
	    }
	    CreateChannel(orb);
	}
	else if(args[0].equalsIgnoreCase("DestroyChannel"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    DestroyChannel(args[1], orb);
	}
	else if(args[0].equalsIgnoreCase("CreateConsumerAdmin"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    CreateConsumerAdmin(args[1], orb);
	}
	else if(args[0].equalsIgnoreCase("CreateSupplierAdmin"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    CreateSupplierAdmin(args[1], orb);
	}
	else if(args[0].equalsIgnoreCase("CreatePullConsumer") ||
	   args[0].equalsIgnoreCase("CreatePushConsumer"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    CreateConsumer(args[1], orb, args[0]);
	}
	else if(args[0].equalsIgnoreCase("CreatePushSupplier") ||
	   args[0].equalsIgnoreCase("CreatePullSupplier"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    CreateSupplier(args[1], orb, args[0]);
	}
	else if(args[0].equalsIgnoreCase("DestroyProxySupplier") ||
	   args[0].equalsIgnoreCase("DestroyTypedProxySupplier"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    DestroyProxySupplier(args[1], orb);
	}
	else if(args[0].equalsIgnoreCase("DestroyProxyConsumer"))
	{
	    if(args.length != 2)
	    {
		Usage();
		System.exit(1);
		return 1;
	    }
	    DestroyProxyConsumer(args[1], orb);
	}
	else if(args[0].equalsIgnoreCase("TypedShutdown"))
	{
	    if(args.length != 1)
	    {
		Usage();
		return 1;
	    }
	    TypedShutdown(orb);
	}
	else if(args[0].equalsIgnoreCase("CreateTypedChannel"))
	{
	    if(args.length != 1)
	    {
		Usage();
		return 1;
	    }
	    CreateTypedChannel(orb);
	}
	else if(args[0].equalsIgnoreCase("DestroyTypedChannel"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    DestroyTypedChannel(args[1], orb);
	}
	else if(args[0].equalsIgnoreCase("CreateTypedConsumerAdmin"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    CreateTypedConsumerAdmin(args[1], orb);
	}
	else if(args[0].equalsIgnoreCase("CreateTypedSupplierAdmin"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    CreateTypedSupplierAdmin(args[1], orb);
	}
	else if(args[0].equalsIgnoreCase("CreateTypedPushConsumer") ||
	   args[0].equalsIgnoreCase("CreateTypedPullConsumer"))
	{
	    if(args.length != 3)
	    {
		Usage();
		return 1;
	    }
	    CreateTypedConsumer(args[1], args[2], orb, args[0]);
	}
	else if(args[0].equalsIgnoreCase("CreateTypedPullSupplier") ||
	   args[0].equalsIgnoreCase("CreateTypedPushSupplier"))
	{
	    if(args.length != 3)
	    {
		Usage();
		return 1;
	    }
	    CreateTypedSupplier(args[1], args[2], orb);
	}
	else if(args[0].equalsIgnoreCase("DestroyTypedProxyConsumer"))
	{
	    if(args.length != 2)
	    {
		Usage();
		return 1;
	    }
	    DestroyTypedProxyConsumer(args[1], orb);
	}
	else
	{
	    Usage();
	    return 1;
	}
	return 0;
    }

    public static void
    main(String args[])
    {
	int status = 0;
	ORB orb = null;

	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");
	
	try
	{
            //
            // Create ORB.
            //
            orb = ORB.init(args, props);

            //
            // Filter options.
            //
            args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
	    
	    status = run (orb, args);	
	}
	catch(SystemException ex)
	{
	    TSU.PrintStatusMsg("PushEventServer", 3, 
	    		       "Caught System Exception");
	    ex.printStackTrace();
            status = 1;
	}
	System.exit(status);
    }
}
