#!/bin/perl
# **********************************************************************
#
# Copyright (c) 2000 
# Object Oriented Concepts, Inc.
# Billerica, MA, USA
#
# All Rights Reserved
#
# **********************************************************************

package Extraction;

use TestSuiteUtilities;
use strict;
use Exporter;

use vars qw($count 
	    %commands 
	    @ISA 
	    @EXPORT);

@ISA = qw(Exporter);
@EXPORT = qw(&extractionSetup 
	     &getCommandCount 
	     &getCommand
	     &readCommands 
	     &parseInput);

#
# Set up any necessary environment variables.
#
sub extractionSetup()
{
    $count = 0;
}


sub printHelpMessage()
{
    print "Usage:  runtest.pl [--use-existing-service] [INPUTFILE]\n";
    exit();
}

sub getCommandCount()
{
    return $count;
}

sub getCommand($)
{
    my $index = shift;
    return $commands{$index};
}

#
# Read from the given file the list of tests to run.
#
sub readCommands()
{
    my $flag1 = shift or "regression";
    my $ins = shift or "regression";
    my $input_string; 
    my $indent = 0;

    #
    # Check to see if we do not want to run the service
    #
    if($flag1 eq "-h" || $flag1 eq "--help" || $flag1 eq "-?")
    {
        &printHelpMessage(); 
    }
    elsif($flag1 eq "--use-existing-service")
    {
	&useExistingService();
    }
    elsif($flag1 =~ /^-/)
    {
        print "Error: $flag1 is not a recognized argument\n";
	exit();
    }
    else
    {
	&useOwnService();
	$ins = $flag1;
    }

    #
    # Open and read the input data file
    #
    if($ins eq "")
    {
	$ins = "regression";
    }

    open(IN, $ins) or
	die ("Error:  Can't open required datafile: $ins\n");

    while(<IN>)
    {
	if(/^REGRESSION/) { last; }
    }
     
    while(<IN>)
    {

	#
	# Skip any commented lines
	#
	if(!(/^#/))
	{
	    $_ =~ s/\n$//;
	    my @chars = split(//, $_);
	    $_ =~ s/^\s+//;

	    #
	    # Count the number of spaces
	    #
	    my $i = 0;
	    my $spaces = 0;
	    while (1)
	    {
		if($chars[$i] eq " ")
		{
		    $spaces += 1;
		}
		elsif($chars[$i] eq "\t")
		{
		    $spaces += 8;
		}
		else
		{
		    last;
		}
		$i += 1;
	    }

	    #
	    # Append the current string to the complete input string.
	    #	   

	    #	
	    # This is the first line
	    #
	    if($indent == 0)
	    {
		$indent = $spaces;
		$input_string = $input_string . "{" . $_; 
	    }
	    
	    #
	    # Have a new level
	    #
	    elsif($indent < $spaces)
	    {
		$indent = $spaces;
		$input_string = $input_string . "{" . $_;
	    }
	    
	    #
	    # Close the previous 3 levels 
	    #
	    elsif($indent > $spaces + 8) 
	    {
		$indent = $spaces;
		$input_string = $input_string . "}}}," . $_; 
	    }
	    
	    #
	    # Close the previous 2 levels 
	    #
	    elsif($indent > $spaces + 4) 
	    {
		$indent = $spaces;
		$input_string = $input_string . "}}," . $_; 
	    }
	    #
	    # Close the previous level 
	    #
	    elsif($indent > $spaces) 
	    {
		$indent = $spaces;
		$input_string = $input_string . "}," . $_; 
	    }
	    
	    #
	    # On the same level 
	    #
	    else 
	    {
		$input_string = $input_string . "," . $_;
	    }
	}
    }
    $input_string = $input_string . "}";

    return $input_string;
}

#
# Take the unformatted string and append the correct substrings to
# the given header.  Append complete commands to the global hash
# %commands.   
#
sub parseInput($)
{
    my $unformatted = shift;
    my $header = shift or "";
    my $sub_command;
   
    my $len = length($header);
    if($len > 0) 
    {
	$sub_command = $header . " ";
    }
    
    #
    # If the first character is a '{', then we are at the first iteration of
    # the recursive program.  We need to skip that character.
    #
    my $pos = index($unformatted, "{");
    if($pos == 0)
    {
	$unformatted =~ s/.(.*)/$1/;
    }
    
    while(1)
    {
    
	#
	# Remove the first string from the unformatted string.  Stop when 
	# a , '{', or '}' is found.
	#

	$pos = index($unformatted, "{");
	my $pos2 = index($unformatted, "}");
	if(($pos2 >= 0 && $pos > $pos2) ||
	    ($pos == -1))	
	{
	    $pos = $pos2;
	}
	my $pos2 = index($unformatted, ",");
	if($pos2 >= 0 && $pos > $pos2)
	{
	    $pos = $pos2;
	}
     
	#
	# Do we have a string to be added to the command line.
	#
	if($pos > 0)
	{
	    $sub_command = $sub_command . substr($unformatted, 0, $pos);
	    $unformatted = substr($unformatted, $pos);
	    
	    #
	    # The next character will tell us what to do next.  Remove it.
	    #
	    my $char = substr($unformatted, 0, 1);
	    $unformatted = substr($unformatted, 1);
	    
	    #
	    # If a comma, then append the sub_command to the command hash.  
	    # Set the sub_command to the header, and start over.
	    #
	    if($char eq ",")
	    {
		$commands{$count} = $sub_command;
		$count++;
		if($len > 0)
		{
		    $sub_command = $header . " ";
		}
		else
		{
		    $sub_command = "";
		}
		redo;
	    }
	    
	    #
	    # If it is a }, then append the sub_command to the command hash.
	    # Return the unformatted string.
	    #
	    elsif($char eq "}")
	    {
		$commands{$count} = $sub_command;
		$count++;
		return $unformatted;
	    }
	    
	    #
	    # If it is a {, then we have another layer.  Call parseInput
	    # passing it the sub_command and the unformatted string.  Upon
	    # returning, reset the sub_command to the header, and start over.
	    #
	    else
	    {
		$unformatted = parseInput($unformatted, $sub_command);
	
		#
		# Have we finished parsing the string?
		#
		my $unformatted_len = length($unformatted);
		if($unformatted_len == 0) 
		{
		    return;
		}

		if($len > 0) 
		{
		    $sub_command = $header . " ";
		}
		else
		{
		    $sub_command = "";
		}
		redo;
	    }
	}
	
	#
	# We did not have a string.  Thus there must be either a 
	# '}' or a ','
	#
	else
	{
	    my $char = substr($unformatted, 0, 1);
	    $unformatted = substr($unformatted, 1);
	    
	    #
	    # If a comma, then continue from the start.  
	    #
	    if($char eq ",")
	    {
		redo;
	    } 
	    else
	    {
		return $unformatted;
	    }
	}
    }
}
   
