// **********************************************************************
//
// Copyright (c) 2000 
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosEventChannelAdmin.*;
import org.omg.CosEventComm.*;
import java.io.*;
import java.util.*;


class EventPullConsumer_impl extends _PullConsumerImplBase
{
    private boolean disconnected_;
    private StringBuffer buf_;

    public 
    EventPullConsumer_impl(StringBuffer buf)
    {
	disconnected_ = false;
	buf_ = buf;
    }

    public boolean
    disconnected()
    {
	return disconnected_;
    }

    public void
    disconnect_pull_consumer()
    {
	buf_.append("Disconnect\n");
	disconnected_ = true;
        TSU.PrintStatusMsg("PullEventClient", 2, "Disconnect Pull Consumer");
    }
}

//
// Base class for proxy wrapper for pulling and displaying events.
//
abstract class EventPullerBase
{
    StringBuffer buf_;

    public void 
    receiveAny(Any event)
    {
        try
        {
            String s = event.extract_string();
	    buf_.append(s);
	    buf_.append(" ");
        }
        catch(MARSHAL ex) 
        {
            buf_.append("unknown\n");
        }       
    }
    
    public
    EventPullerBase(StringBuffer buf) 
    {
	buf_ = buf;
    }

    public abstract boolean pull() throws Disconnected;
}

class UntypedEventPuller extends EventPullerBase
{
    private ProxyPullSupplier proxy_;

    public
    UntypedEventPuller(ProxyPullSupplier proxy, StringBuffer buf)
    {
	super(buf);
	proxy_ = proxy;
    }

    public boolean
    pull()
    {
	BooleanHolder hasEvent = new BooleanHolder();
	try
	{
	    Any event = proxy_.try_pull(hasEvent);
	    if(hasEvent.value)
		receiveAny(event);
	}
	catch(org.omg.CosEventComm.Disconnected ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return hasEvent.value;
    }
}

class EventPuller extends EventPullerBase
{
    EventPullerBase pullerImpl_;

    public
    EventPuller(StringBuffer buf)
    {
	super(buf);
	pullerImpl_ = null;
    }

    public void 
    connect(ProxyPullSupplier proxy)
    {
	pullerImpl_ = new UntypedEventPuller(proxy, buf_);
    }

    public boolean
    pull()
	throws Disconnected
    {
	if(pullerImpl_ == null)
	    return false;

	return pullerImpl_.pull();
    }
};

public class PullEventClient
{ 
    private static int evt_num = 0;
    
    //
    // Checking conditions about the time taken and the number of events
    // received.  'taken' refers to the number of pulls taken thus far.
    // 'allowed' refers to the number of pulls allowed before exiting.
    // 'expect' is the numberof events expected to be received.  Finally,
    // 'received' is the number of events that have been received thus far.
    //
    private static boolean 
    CheckConditions(int taken, int allowed, int expect, int received)
    {
	//
	// See if we have waited too long for an event
	//
	if (taken == allowed)
	{
	    if (expect == -1)
	    {
		return true;
	    }
	    else if (received < expect)
	    {
		TSU.PrintStatusMsg("PullEventClient", 2, "Not enough events");
		return true;
	    }
	    else 
	    {
		TSU.PrintStatusMsg("PullEventClient", 2, 
			       "Received correct number of events");
		return true;
	    }
	}
	return false;
    }

    private static void 
    Usage()
    {
	System.out.println("pulleventclient <proxy-ior> [--expect n] " +
			   "[--disconnect]\n" +
			   "   --expect:  Specify the number of events to " +
			   "expect\n" +
			   "   --disconnect:  Call connect on the proxy and " + 
			   "disconnect\n");
    }

    static int 
    run(ORB orb, BOA boa, String args[])    
    {
	//
	// The numer of pull attempts allowed to be taken without receiving
	// an event.
	//
	int allowed = 10;

	if(args.length > 4)
	{
	    Usage();
	    System.exit(1);
	}

	//
	// How many events should we expect?
	//
	int expect = -1;

	//
	// Connect and exit.
	//
	boolean disconnect = false;

	//
	// Check for disconnect or expect arguments.
	//

	int i;
	for(i = 0 ; i < args.length ; i++)
	{
	    if(args[i].equalsIgnoreCase("--disconnect"))
	    {
		disconnect=true;
	    }
	    if(args[i].equalsIgnoreCase("--expect"))
	    {
		expect = Integer.parseInt(args[i+1]);
	    }
	}

	TSU.PrintStatusMsg("PullEventClient", 2, "Starting");

	StringBuffer outputBuffer = new StringBuffer();
	//
	// Instantiate a servant to receive disconnect callbacks.
	//
	EventPullConsumer_impl consumerImpl = 
	    new EventPullConsumer_impl(outputBuffer);
	
	//
	// We need to create persistent object references for the
	// connection reliability tests.
	//
	((com.ooc.CORBA.ORB)orb).connect(consumerImpl, 
					 "DefaultConsumer");
	((com.ooc.CORBA.BOA)boa).init_servers();

	EventPuller puller = new EventPuller(outputBuffer);
	
	//
	// Initialize the proxy.
	//
	org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	ProxyPullSupplier proxy = 
	    ProxyPullSupplierHelper.narrow(obj);
	if (proxy == null)
	{
	    System.err.println("Couldn't narrow Proxy Pull Supplier");
	    return 1;
	}
	
	puller.connect(proxy);

	PullConsumer consumer = consumerImpl;

	if (consumer == null)
	{
	    System.err.println("PullConsumer not initialized");
	    return 1;
	}

	try
	{
	    proxy.connect_pull_consumer(consumer);
	}
	catch(org.omg.CosEventChannelAdmin.AlreadyConnected ex)
	{
	    TSU.PrintStatusMsg("PullEventClient", 2, 
			   "Caught Already Connected Exception");
	}
	if(disconnect) 
	{
	    TSU.PrintStatusMsg("PullEventClient", 2, 
			    "Connected Pull Consumer");
	    return 0;
	}

	//
	// Maintain some statistics about the event transfer.
	//
	int received = 0;
	int taken = 0; // In seconds.

	int result = 0;
	while(!consumerImpl.disconnected())
	{
	    try
	    {
		// 
		// We've exceeded the parameters for the run.  Exit.
		//
		if(CheckConditions(taken, allowed, expect, received))
		{
		    TSU.PrintStatusMsg("PullEventClient", 1, 
				   "Runtime limits reached.  Exiting");
		    result = 1;
		    break;
		}
		    
		boolean hasEvent = puller.pull();
		    
		//
		// A brief pause to allow reception of the event.
		//
		if(!hasEvent)
		{
		    taken++;
		    try
		    {
			Thread.sleep(1000); 
		    }
		    catch(java.lang.InterruptedException ex)
		    {
		    }
		    continue;
		}

		TSU.PrintStatusMsg("PullEventClient", 4, 
				   "Received Event"); 
		    
		//
		// We have an event, so now we can reset the time taken
		// and increment the number of received events.
		//
		taken = 0;
		received++;

		if((expect != -1) && (received > expect))
		{
		    TSU.PrintStatusMsg("PullEventClient", 1, 
				       "Too many events returned, " +
					   "Exiting");
		    result = 1;
		    break;
		}
		    
	    }
	    catch(Disconnected ex)
	    {
		TSU.PrintStatusMsg("PullEventClient", 2, 
				   "ERROR: Caught unexpected " +
				   "disconnected exception");
		result = 1;
		break;
	    }
	    catch(org.omg.CORBA.OBJECT_NOT_EXIST ex)
	    {
		TSU.PrintStatusMsg("PullEventClient", 2, 
				   "CAUGHT OBJECT NOT EXIST");
		try
		{
		    Thread.sleep(4000); 
		}
		catch(java.lang.InterruptedException excpt)
		{
		}
		result = 1;
		break;
	    }
	}

	System.out.println(outputBuffer);
	return result;
    }

    public static void
    main(String args[])
    {
	int status = 0;
	ORB orb = null;

	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");
	
	try
	{
            //
            // Create ORB.
            //
            orb = ORB.init(args, props);

            //
            // Filter options.
            //
            args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
	    
            //
            // Create BOA.
            //
	    BOA boa = ((com.ooc.CORBA.ORB)orb).BOA_init(args, props);

            //
            // Filter options.
            //
	    args = ((com.ooc.CORBA.BOA)boa).filter_options(args);
	    
	    status = run(orb, boa, args);	
	}
	catch(SystemException ex)
	{
	    TSU.PrintStatusMsg("PushEventServer", 3, 
	    		       "Caught System Exception");
	    ex.printStackTrace();
            status = 1;
	}
	System.exit(status);
    }
}
