// **********************************************************************
//
// Copyright (c) 2000 
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosEventChannelAdmin.*;
import org.omg.CosEventComm.*;
import java.io.*;
import java.util.*;


class PullSupplier_impl extends _PullSupplierImplBase
{
    private BOA boa_;
    private ORB orb_;
    private Vector eventList_;
    private int current_;
    
    //
    // numEventPulls_ is used as a way of determining if there should
    // be a set number of pulls before disconnecting.  If it is -2, then
    // we are not concerned with the number of pulls.  Otherwise we use
    // this value to tell how many "auto" events are left to be pulled.
    //
    private int numEventPulls_;
    
    //
    // streaming_ is used as a flag indicating that a continuous
    // stream of "auto" events are to be generated and pulled.
    //
    private int streaming_;

    public
    PullSupplier_impl(Vector eventList, BOA boa, ORB orb)
    {
        boa_ = boa;
        orb_ = orb;
	eventList_ = eventList;
	current_ = 0;
	numEventPulls_ = -2;

	if (((String)eventList.elementAt(0)).equals("streaming"))
	    streaming_ = 1;
	else
	{
	    try 
	    {
		if (Integer.parseInt(((String)eventList.elementAt(0))) > 0)
		    numEventPulls_ = 
			Integer.parseInt((String)eventList.elementAt(0));
	    }
	    catch(NumberFormatException ex)
	    {
		numEventPulls_ = -2;
	    }
	}
    }

    public void
    disconnect_pull_supplier()
    {
	boa_.deactivate_impl(null);
    }

    public Any
    pull()
	throws Disconnected
    {
	Any any = orb_.create_any();

	if (numEventPulls_ == -1)
	{
	    throw new InternalError();
	}

	if (numEventPulls_ > -1)
	{
	    any.insert_string("AUTO");
	    TSU.PrintStatusMsg("PullEventServer", 1, "Pulling Any Auto Event");
	    numEventPulls_--;
	    return any;
	}
	    
	if (streaming_ == 1)
	{
	    any.insert_string((String)eventList_.elementAt(current_));
	    TSU.PrintStatusMsg("PullEventServer", 1, "Pulling Any Auto Event");
	    return any;
	}

	if(current_ >= eventList_.size())
	{
	    throw new Disconnected();
	}
	else 
	{
	    any.insert_string((String)eventList_.elementAt(current_));
	    current_++;
	}
	TSU.PrintStatusMsg("PullEventServer", 1, "Pulling Any Event");
	return any;
    }
    
    public Any
    try_pull(BooleanHolder hasEvent)
    {
	Any any = orb_.create_any();

	if (numEventPulls_ == -1)
	{
	    hasEvent.value = false;
	    throw new InternalError();
	}

	if (numEventPulls_ > -1)
	{
	    any.insert_string("AUTO");
	    hasEvent.value = true;
	    TSU.PrintStatusMsg("PullEventServer", 1, "Pulling Any Auto Event");
	    numEventPulls_--;
	    return any;
	}
	    
	if (streaming_ == 1)
	{
	    any.insert_string((String)eventList_.elementAt(current_));
	    hasEvent.value = true;
	    TSU.PrintStatusMsg("PullEventServer", 1, "Pulling Any Auto Event");
	    return any;
	}

	if(current_ >= eventList_.size())
	{
	    hasEvent.value = false;
	    throw new InternalError();
	}
	else 
	{
	    any.insert_string((String)eventList_.elementAt(current_));
	    current_++;
	}
	hasEvent.value = true;
	
	TSU.PrintStatusMsg("PullEventServer", 1, "Pulling Any Event");
	return any;
    }
};

public class PullEventServer
{
    private static void
    Usage() 
    {   
	System.out.println("pulleventserver <proxy-ior> [eventList]\n" +
			   "  eventList: 'streaming', a number, or a list of " +
			   "events\n");
    }
   
    static int 
    run(ORB orb, BOA boa, String args[])    
    {
	if(args.length < 1)
	{
	    Usage();
	    return 1;
	}
	
	//
	// Get the event descriptions from argv[2]+
	//
	Vector eventList = new Vector();
	for(int i = 1 ; i < args.length ; i++)
	{
	    eventList.addElement(args[i]);
	}
	
	//
	// Initialize the proxy.
	//
	org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	ProxyPullConsumer proxyConsumer = 
	    ProxyPullConsumerHelper.narrow(obj);
	if (proxyConsumer == null)
	{
	    System.err.println("Couldn't narrow Proxy Pull Consumer");
	    return 1;
	}

	//
	// Create the implementation object.
	//
	PullSupplier_impl ps = new PullSupplier_impl(eventList, boa,
						     orb);
	
	((com.ooc.CORBA.ORB)orb).connect(proxyConsumer, 
					"DefaultConsumer");
	((com.ooc.CORBA.BOA)boa).init_servers();

	//
	// Connect the proxy. If we're already connected we'll get an
	// exception. This should only happen if we're reusing the
	// same proxy-object -- this this case the connected
	// PullSupplier must be the same servant.
	//
	try
	{
	    proxyConsumer.connect_pull_supplier(ps);
	    TSU.PrintStatusMsg("PullEventServer", 1, 
			   "Connected Any Pull Supplier");
	}
	catch(AlreadyConnected ex)
	{
	    ex.printStackTrace();
	    return 1;
	}
	catch(TypeError ex)
	{
	    ex.printStackTrace();
	    return 1;
	}
	    
	boa.impl_is_ready(null);
	return 0;
    }

    public static void
    main(String args[])
    {
	int status = 0;
	ORB orb = null;

	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");
	
	try
	{
            //
            // Create ORB.
            //
            orb = ORB.init(args, props);

            //
            // Filter options.
            //
            args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
	    
            //
            // Create BOA.
            //
	    BOA boa = ((com.ooc.CORBA.ORB)orb).BOA_init(args, props);

            //
            // Filter options.
            //
	    args = ((com.ooc.CORBA.BOA)boa).filter_options(args);
	    
	    status = run (orb, boa, args);	
	}
	catch(SystemException ex)
	{
	    TSU.PrintStatusMsg("PushEventServer", 3, 
	    		       "Caught System Exception");
	    ex.printStackTrace();
            status = 1;
	}
	System.exit(status);
    }

}
