// **********************************************************************
//
// Copyright (c) 2000 
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosEventChannelAdmin.*;
import org.omg.CosEventComm.*;
import java.io.*;
import java.util.*;

public class PushEventClient
{
    private static void
    Usage()
    {
	System.out.println("pusheventclient <proxy-ior> [--disconnect]\n" +
			   "   --disconnect:  Call connect on the proxy " +
			   "and disconnect");
    }

    static int 
    run(ORB orb, BOA boa, String args[])    
    {
	if(args.length > 3)
	{
	    Usage();
	    return 1;
	}

	//
	// Just connect and exit.
	//
	boolean disconnect = false;

	//
	// Find whether to disconnect or not
	//
	for(int i = 0 ; i < args.length ; i++)
	{
	    if(args[i].equals("--disconnect"))
	    {
		disconnect=true;
	    }
	}

	//
	// Initialize the proxy.
	//
	org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	ProxyPushSupplier proxy = ProxyPushSupplierHelper.narrow(obj);

	if (proxy == null)
	{
	    System.err.println("Proxy Push Supplier is null!");
	    return 1;
	}

	StringBuffer outputBuffer = new StringBuffer();
	PushConsumer_impl pc = new PushConsumer_impl(boa, outputBuffer);
	((com.ooc.CORBA.ORB)orb).connect(pc, "DefaultPushConsumer");

	//
	// Connect the proxy. If we're already connected we'll get an
	// exception. This should only happen if we're reusing the
	// same proxy-object -- this this case the connected
	// SequencePushConsumer must be the same servant.
	//
	try
	{
	    proxy.connect_push_consumer(pc);

	    TSU.PrintStatusMsg("PushEventClient", 2, 
			   "Connected Any Push Consumer");
	}
	catch(org.omg.CosEventChannelAdmin.AlreadyConnected ex)
	{
	}
	catch(org.omg.CosEventChannelAdmin.TypeError ex)
	{
	    ex.printStackTrace();
	    return 1;
	}
	
	if(disconnect)
	{
	    return 0;
	}
	
	try
	{
	    boa.impl_is_ready(null);
	}
	catch(org.omg.CORBA.TRANSIENT ex)
	{
	}
	catch(org.omg.CORBA.COMM_FAILURE ex)
	{
	}
	System.out.println(outputBuffer);
	return 0;
    }

    public static void
    main(String args[])
    {
	int status = 0;
	ORB orb = null;

	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");
	
	try
	{
            //
            // Create ORB.
            //
            orb = ORB.init(args, props);

            //
            // Filter options.
            //
            args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
	    
            //
            // Create BOA.
            //
	    BOA boa = ((com.ooc.CORBA.ORB)orb).BOA_init(args, props);

            //
            // Filter options.
            //
	    args = ((com.ooc.CORBA.BOA)boa).filter_options(args);
	    
	    status = run (orb, boa, args);	
	}
	catch(SystemException ex)
	{
	    TSU.PrintStatusMsg("PushEventServer", 3, 
	    		       "Caught System Exception");
	    ex.printStackTrace();
            status = 1;
	}
	System.exit(status);
    }
}

class PushConsumer_impl extends _PushConsumerImplBase
{
    private BOA boa_;
    private StringBuffer buf_;

    PushConsumer_impl(BOA boa, StringBuffer buf)
    {
        boa_ = boa;
	buf_ = buf;
    }
    
    public void
    disconnect_push_consumer()
    {
	buf_.append("Disconnect\n");
	boa_.deactivate_impl(null);
    }
    
    public void
    push(Any any)
    {
        try
        {
            String s = any.extract_string();
            buf_.append(s);
	    buf_.append(" ");
        }
        catch(MARSHAL ex) 
        {
	    buf_.append("unknown\n");
        }       
    }
}
