// **********************************************************************
//
// Copyright (c) 2000 
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosTypedEventChannelAdmin.*;
import org.omg.CosEventChannelAdmin.*;
import org.omg.CosEventComm.*;
import java.io.*;
import java.util.*;

public class TypedPushEventClient
{
    private static void 
    Usage()
    {
	System.out.println("typedpusheventclient <proxy-ior> [--disconnect]\n" +
			   "   --disconnect:  Call connect on the proxy and " +
			   "disconnect\n");
    }
    
    static int  
    run(ORB orb, BOA boa, String args[])    
    {
	if(args.length > 3)
	{
	    Usage();
	    return 1;
	}

	//
	// Just connect and exit.
	//
	boolean disconnect = false;

	//
	// Find whether to append timestamps or not and make sure that no 
	// "new" arguments are given to ClientCommon.
	//
	for(int i = 0 ; i < args.length ; i++)
	{
	    if(args[i].equalsIgnoreCase("--disconnect"))
	    {
		disconnect=true;
	    }
	}
	TSU.PrintStatusMsg("TypedPushEventClient", 2, "Starting");
	
	//
	// Get the proxy object.
	//
	org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	ProxyPushSupplier proxy = 
	    ProxyPushSupplierHelper.narrow(obj);
	if (proxy == null)
	{
	    System.err.println("Couldn't narrow Proxy Push Supplier");
	    return 1;
	}

	StringBuffer outputBuffer = new StringBuffer();
	//
	// Create the implementation object.
	//
	Echo_impl pc = new Echo_impl(orb, boa, outputBuffer);

	//
	// Connect the consumer to the ORB with the same name each
	// time.
	//
	((com.ooc.CORBA.ORB)orb).connect(pc, "DefaultConsumer");
	((com.ooc.CORBA.BOA)boa).init_servers();

	TSU.PrintStatusMsg("TypedPushEventClient", 2, 
		       "Created and connected to Echo");
	//
	// Connect the proxy. If we're already connected we'll get an
	// exception. This should only happen if we're reusing the
	// same proxy-object -- this this case the connected
	// SequencePushConsumer must be the same servant.
	//
	try
	{
	    proxy.connect_push_consumer(pc);
	}
	catch(TypeError ex)
	{
	    ex.printStackTrace();
	    return 1;
	}
	catch(AlreadyConnected ex)
	{
	}
	catch(SystemException ex)
	{
	    TSU.PrintStatusMsg("TypedPushEventClient", 2, "FAILURE");
	    ex.printStackTrace();
	    return 1;
	}
	
	if(disconnect)
	{
	    return 0;
	}

	boa.impl_is_ready(null);

	System.out.println(outputBuffer);
	return 0;
    }

    public static void
    main(String args[])
    {
	int status = 0;
	ORB orb = null;

	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");
	
	try
	{
            //
            // Create ORB.
            //
            orb = ORB.init(args, props);

            //
            // Filter options.
            //
            args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
	    
            //
            // Create BOA.
            //
	    BOA boa = ((com.ooc.CORBA.ORB)orb).BOA_init(args, props);

            //
            // Filter options.
            //
	    args = ((com.ooc.CORBA.BOA)boa).filter_options(args);
	    
	    status = run(orb, boa, args);	
	}
	catch(SystemException ex)
	{
	    TSU.PrintStatusMsg("PushEventServer", 3, 
	    		       "Caught System Exception");
	    ex.printStackTrace();
            status = 1;
	}
	System.exit(status);
    }
}
