// **********************************************************************
//
// Copyright (c) 2000 
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosTypedEventChannelAdmin.*;
import org.omg.CosEventChannelAdmin.*;
import java.io.*;
import java.util.*;

public class TypedPushEventServer
{

    private static void 
    Usage()
    {
	System.out.println("pusheventserver <proxy-ior) <EventList>\n" +
		           "  EventList:  'auto', or a list of events " +
			   "seperated by spaces\n");
    }

    static int 
    run(ORB orb, String args[])    
    {
	if(args.length < 3)
	{
	    Usage();
	    return 1;
	}
	
	//
	// Get the event descriptions from argv[4]+
	//
	Vector eventDescriptions = new Vector();
	for(int i = 1 ; i < args.length ; i++)
	{
	    eventDescriptions.addElement(args[i]);
	}

	TSU.PrintStatusMsg("TypedPushEventServer", 1, "Starting");

	//
	// Initialize the server and get the proxy object.
	//
	org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	TypedProxyPushConsumer proxyConsumer = 
	    TypedProxyPushConsumerHelper.narrow(obj);
	if (proxyConsumer == null)
	{
	    System.err.println("Couldn't narrow Proxy Push Consumer");
	    return 1;
	}

	try 
	{
	    proxyConsumer.connect_push_supplier(null);
	}
	catch(AlreadyConnected ex)
	{
	}

	org.omg.CORBA.Object typedConsumer = 
	    proxyConsumer.get_typed_consumer();

	Echo timer = EchoHelper.narrow(typedConsumer);

	//
	// Put streaming event generation and pushing here.
	//
	if (((String)eventDescriptions.elementAt(0)).equals("auto"))
	{
	    TSU.PrintStatusMsg("PushEventServer", 1, 
			       "Pushing Typed AUTO Events");
	    while(true)
	    {
		timer.echo("AUTO");
	    }
	}
	
	//
	// Put 'manual' event generation and pushing here.
	//
	for(int j = 0 ; j < eventDescriptions.size() ; j++)
	{
	    timer.echo((String)eventDescriptions.elementAt(j));
	    TSU.PrintStatusMsg("PushEventServer", 1, "Pushed Any Event");
	}
	
	try
	{
	    proxyConsumer.disconnect_push_consumer();
	    TSU.PrintStatusMsg("PushEventServer", 1, "Disconnected Any " +
			       "Push Consumer");
	}
	catch(org.omg.CORBA.TRANSIENT ex)
	{
	}
	return 0;
    }

    public static void
    main(String args[])
    {
	int status = 0;
	ORB orb = null;

	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");
	
	try
	{
            //
            // Create ORB.
            //
            orb = ORB.init(args, props);

            //
            // Filter options.
            //
            args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
	    
	    status = run (orb, args);	
	}
	catch(SystemException ex)
	{
	    TSU.PrintStatusMsg("PushEventServer", 3, 
	    		       "Caught System Exception");
	    ex.printStackTrace();
            status = 1;
	}
	System.exit(status);
    }
}

