// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNaming;

import org.omg.CORBA.*;
import java.util.Hashtable;

public class NamingService
{
    //
    // My ORB
    //
    private ORB orb_;

    //
    // My BOA
    //
    private BOA boa_;

    //
    // My set of naming contexts
    //
    Hashtable ncs_;

    //
    // My naming service database
    //
    private NamingDatabase store_;

    //
    // My naming service database logger
    //
    private Logger logger_;

    //
    // My root naming context
    //
    private org.omg.CosNaming.NamingContext root_;

    //
    // The naming service options
    //
    private NamingServiceOptions options_;

    //
    // Are we initialized?
    //
    private boolean initialized_;

    // ------------------------------------------------------------------
    // NamingService constructor
    // ------------------------------------------------------------------

    public NamingService(ORB orb, BOA boa, NamingServiceOptions options)
    {
	orb_ = orb;
	boa_ = boa;
	options_ = options;
	initialized_ = false;
    }

    // ------------------------------------------------------------------
    // NamingService public member implementation
    // ------------------------------------------------------------------

    //
    // Shutdown the naming service
    //
    public void shutdown()
    {
	//
	// Stop the naming service database logger
	//
	if(logger_ != null)
	{
	    logger_.halt();
	    while(logger_.isAlive())
	    {
		try
		{
		    logger_.join();
		}
		catch(InterruptedException e)
		{
		}
	    }
	    logger_ = null;
	}

	//
	// Release the root naming context
	//
	root_ = null;

	//
	// Clean up all contexts
	//
	ncs_.clear();
	ncs_ = null;

	//
	// Release the naming service database
	//
	store_ = null;
    }

    //
    // Initialize the naming service
    //
    public void initialize()
    {
	if(initialized_)
	    return;

	initialized_ = true;

	//
	// Create the set of naming contexts
	//
	ncs_ = new Hashtable(63);

	//
	// Create the timeout policy
	//
	Any timeoutAny = orb_.create_any();
	timeoutAny.insert_long(1000 * options_.callbackTimeout);
	Policy[] timeoutPolicy = new Policy[1];
	try
	{
	    timeoutPolicy[0] = ((com.ooc.CORBA.ORB)orb_).
		create_policy(com.ooc.OB.TIMEOUT_POLICY.value, timeoutAny);
	}
	catch(org.omg.CORBA.PolicyError ex)
	{
	    throw new RuntimeException();
	}

	if(options_.databaseName != null)
	{
	    //
	    // Create a new naming service database
	    //
	    store_ = new NamingDatabase(boa_, orb_, options_.databaseName, ncs_,
					timeoutPolicy, options_.noUpdates,
					!options_.databaseExists);

	    //
	    // Start the naming service database logger
	    //
	    logger_ = new Logger(store_, 60 * options_.databaseTimeout);
	    logger_.start();

	    //
	    // Add version, host and port to the journal
	    //
	    store_.info();
	}

	//
	// Get the root naming context
	//
	if(!options_.databaseExists || options_.databaseName == null)
	{
	    root_ = new com.ooc.CosNaming.NamingContext(
		orb_, store_, ncs_, "NameService", timeoutPolicy,
		options_.noUpdates);
	}
	else
	{
	    root_ = (org.omg.CosNaming.NamingContext) ncs_.get("NameService");
	}
    }

    //
    // Get the root naming context
    //
    public org.omg.CosNaming.NamingContext
    rootNamingContext()
    {
	return root_;
    }
}
