// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole;

import javax.swing.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import com.ooc.CosNaming.OBNamingContextPackage.*;
import com.ooc.CosNamingConsole.Util.*;

final public class BindingInfo
{
    //
    // Bound object, nc_ is null if this object is no naming context.
    // This information is stored for fast access.
    //
    private org.omg.CORBA.Object obj_;
    private NamingContext nc_;

    //
    // Context this binding is defined in
    //
    private NamingContext parent_;

    //
    // Corresponding extended binding
    //
    private ExtendedBinding extBinding_;

    public BindingInfo(NamingContext parent, ExtendedBinding b)
    {
	parent_ = parent;
	extBinding_ = b;

	if(extBinding_.ncOrObj.discriminator() == BindingType.ncontext)
	{
	    nc_ = extBinding_.ncOrObj.nc();
	    obj_ = nc_;
	}
	else
	    obj_ = extBinding_.ncOrObj.obj();
    }

    public String
    getID()
    {
	return extBinding_.binding_name[0].id;
    }

    public String
    getKind()
    {
	return extBinding_.binding_name[0].kind;
    }

    public int
    getTime()
    {
	return extBinding_.timestamp;
    }

    public String
    toString()
    {
	return BindingUtil.getFullName(extBinding_);
    }

    public org.omg.CORBA.Object
    getObject()
    {
	return obj_;
    }

    public NamingContext
    getContext()
    {
	return nc_;
    }

    public NamingContext
    getParentContext()
    {
	return parent_;
    }

    public ExtendedBinding
    getBinding()
    {
	return extBinding_;
    }

    public void
    setObject(org.omg.CORBA.Object obj)
    {
	obj_ = obj;
	extBinding_.ncOrObj.obj(obj);
    }

    public void
    setContext(NamingContext nc)
    {
	obj_ = nc;
	nc_ = nc;
	extBinding_.ncOrObj.nc(nc);
    }
}
