// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole;

import java.util.*;
import javax.swing.tree.*;
import org.omg.CosNaming.*;
import com.ooc.CosNaming.OBNamingContextPackage.*;
import com.ooc.CosNamingConsole.Util.*;

final public class BindingNode extends DefaultMutableTreeNode
{
    //
    // The binding model
    //
    private BindingModel bindingModel_;

    //
    // true if children have already been loaded
    //
    private boolean hasLoaded_;

    public BindingNode(BindingModel bindingModel, BindingInfo info)
    {
	super(info);

	bindingModel_ = bindingModel;
    }

    public BindingModel
    getModel()
    {
	return bindingModel_;
    }

    public void
    destroy()
    {
	//
	// Remove all children
        //
        while(!isLeaf())
        {
            BindingNode child = (BindingNode)getFirstChild();
	    remove(child);
            child.destroy();
        }

        //
        // Destroy listener
        //
        bindingModel_.destroy();
    }

    //
    // Always called by the model before a node is going to be displayed.
    //
    public int
    getChildCount()
    {
	if(!hasLoaded_)
	    loadChildren();

	return super.getChildCount();
    }

    private void
    loadChildren()
    {
	hasLoaded_ = true;

	BindingInfo info = (BindingInfo)getUserObject();
	NamingContext nc  = info.getContext();

	try
	{
	    //
	    // Get all contexts
	    //
	    Vector contexts = BindingUtil.getContextsFromContext(nc);

	    Enumeration e = contexts.elements();
	    while(e.hasMoreElements())
	    {
		ExtendedBinding b = (ExtendedBinding)e.nextElement();
		
		bindingModel_.context_added(nc, b.ncOrObj.nc(),
					    b.binding_name[0]);
	    }
	}
	catch(org.omg.CORBA.OBJECT_NOT_EXIST ex)
	{
	    // Ignore
	}
    }

    public boolean
    hasLoaded()
    {
	return hasLoaded_;
    }

    public Vector
    getChildren()
    {
	Vector children = super.children;
	if(children != null)
	    return (Vector)children.clone();
	else
	    return new Vector(0);
    }
}
