// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole;

import javax.swing.*;
import javax.swing.table.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.ooc.CosNaming.OBNamingContextPackage.*;
import com.ooc.CosNamingConsole.Util.*;
import com.ooc.CosNamingConsole.GUI.*;
import com.ooc.Util.*;
import com.ooc.Util.GUI.*;

final public class BindingTableModel extends AbstractTableModel
{
    //
    // Column identifiers
    //
    private final static int COL_ID = 0;
    private final static int COL_KIND = 1;
    private final static int COL_MODIFIED = 2;

    //
    // Column names
    //
    private final static String[] names_ = new String[3];

    //
    // Bindings in this table
    //
    private java.util.Vector bindings_ = new java.util.Vector();

    //
    // Object that sorts table entries
    //
    private Sorter sorter_;

    //
    // Error log object
    //
    private MessageLogger logger_;

    //
    // The current context
    //
    private NamingContext nc_;

    //
    // If true, show time
    //
    private boolean showDetails_ = false;

    //
    // Model to delegate changes to
    //
    private BindingModel model_;

    BindingTableModel()
    {
	logger_ = WindowFactory.getErrorWindowInstance().getLogger();

	sorter_ = new Sorter();

	if(names_[COL_ID] == null)
	{
	    names_[COL_ID] = AppStandards.getString("TableIdKey");
	    names_[COL_KIND] = AppStandards.getString("TableKindKey");
	    names_[COL_MODIFIED] = AppStandards.getString("TableTimeKey");
	}
    }

    public void
    setModel(BindingModel model)
    {
	model_ = model;
    }

    public int
    getRowCount()
    {
	return bindings_.size();
    }

    public int
    getColumnCount()
    {
	return showDetails_ ? names_.length : names_.length - 1;
    }

    public java.lang.Object
    getValueAt(int row, int col)
    {
	BindingInfo info = (BindingInfo)bindings_.elementAt(row);
	if(info != null)
	{
	    switch(col)
	    {
	    case COL_ID:
		return info.getID();
	    case COL_KIND:
		return info.getKind();
	    case COL_MODIFIED:
		return BindingUtil.getTime((long)info.getTime() * 1000);
	    default:
		//
		// Special case, returns BindingInfo object itself
		//
		if(bindings_.size() != 0)
		    return info;
		else
		    return null;
	    }
	}
	else
	    return null;
    }

    public void
    setValueAt(java.lang.Object value, int row, int col)
    {
	BindingInfo info = (BindingInfo)bindings_.elementAt(row);
	ExtendedBinding b = info.getBinding();
	NamingContext nc = info.getParentContext();

	String id, kind;
	switch(col)
	{
	case COL_ID:
	    id = value.toString();
	    kind = b.binding_name[0].kind;
	    break;
	case COL_KIND:
	    id = b.binding_name[0].id;
	    kind = value.toString();
	    break;
	default:
	    super.setValueAt(value, row, col);
	    return;
	}

	if(!id.equals(b.binding_name[0].id) ||
	   !kind.equals(b.binding_name[0].kind))
	{
	    //
	    // Check for leading and trailing spaces
	    //
	    if(id.length() != id.trim().length() ||
	       kind.length() != kind.trim().length())
	    {
		java.lang.Object[] message = new java.lang.Object[2];
		message[0] = new JLabel(AppStandards.getString("Trim1Key"));
		message[1] = new JLabel(AppStandards.getString("Trim2Key"));

		int response = JOptionPane.showConfirmDialog(
		    null, message, AppStandards.getString("TrimTitleKey"),
		    JOptionPane.YES_NO_CANCEL_OPTION,
		    JOptionPane.QUESTION_MESSAGE);

		switch(response)
		{
		case JOptionPane.CANCEL_OPTION:
		    return;
		case JOptionPane.YES_OPTION:
		    id = id.trim();
		    kind = kind.trim();
		    if(id.equals(b.binding_name[0].id) &&
		       kind.equals(b.binding_name[0].kind))
			return;
		    break;
		case JOptionPane.NO_OPTION:
		    break;
		}
	    }

	    //
	    // Compose new name
	    //
	    NameComponent[] name = new NameComponent[1];
	    name[0] = new NameComponent(id, kind);

	    //
	    // Remember old name
	    //
	    NameComponent oldName = b.binding_name[0];

	    com.ooc.CosNaming.OBNamingContext obnc =
		com.ooc.CosNaming.OBNamingContextHelper.narrow(nc);
	    if(obnc != null)
	    {
		try
		{
		    obnc.rename(b.binding_name, name);
		    b.binding_name = name;

		    fireTableCellUpdated(row, col);
		}
		catch(UserException ex)
		{
		    logger_.printException(ex);
		}
		catch(SystemException ex)
		{
		    logger_.printException(ex);
		}
	    }
	    else
	    {
		//
		// First bind new name, then delete old name
		//
		if(doBind(nc, info, name))
		{
		    try
		    {
			nc.unbind(b.binding_name);
		    }
		    catch(UserException ex)
		    {
			logger_.printException(ex);
		    }
		    catch(SystemException ex)
		    {
			logger_.printException(ex);
		    }

		    b.binding_name = name;

		    //
		    // Not required as long as we explicitly update
		    // using the model (see below).
		    //
//		    fireTableCellUpdated(row, col);
		}
	    }

	    //
	    // Update the model (and thus the tree)
	    //
	    if(obnc == null || model_.getNoUpdates())
	    {
		if(info.getContext() != null)
		    model_.context_changed(nc, info.getContext(), oldName,
					   name[0]);
	    }
	}
    }

    public String
    getColumnName(int col)
    {
	return names_[col];
    }

    //
    // Only id and kind are editable
    //
    public boolean
    isCellEditable(int row, int col)
    {
	return col < COL_MODIFIED;
    }

    //
    // Display mode selection
    //
    void
    showDetails(boolean b)
    {
	if(showDetails_ != b)
	{
	    showDetails_ = b;

	    fireTableStructureChanged();
	}
    }

    //
    // Set new sorting mode
    //
    void
    setSortingMode(int mode)
    {
	sorter_.setSortingMode(mode);
	sorter_.sort(bindings_, 0, bindings_.size() - 1);

	fireTableDataChanged();
    }

    //
    // Create sorted list of naming context members
    //
    public boolean
    setContext(NamingContext nc)
    {
	boolean valid = true;

        ExtendedBinding[] bindings = new ExtendedBinding[0];

        try
        {
            bindings = BindingUtil.getBindingsFromContext(nc);
        }
        catch(SystemException ex)
        {
	    logger_.printException(ex);
	    valid = false;
        }

	bindings_ = new java.util.Vector(bindings.length);
	bindings_.setSize(bindings.length);

	for(int i = 0 ; i < bindings.length ; i++)
	    bindings_.setElementAt(new BindingInfo(nc, bindings[i]), i);

	sorter_.sort(bindings_, 0, bindings_.size() - 1);

	nc_ = nc;

	fireTableDataChanged();

	return valid;
    }

    //
    // Bind context/object
    //
    private boolean
    doBind(NamingContext nc, BindingInfo info, NameComponent[] name)
    {
	try
	{
	    NamingContext n = info.getContext();
	    if(n != null)
		nc.bind_context(name, n);
	    else
		nc.bind(name, info.getObject());
	}
	catch(UserException ex)
	{
	    logger_.printException(ex);

	    return false;
	}
	catch(SystemException ex)
	{
	    logger_.printException(ex);

	    return false;
	}

	return true;
    }

    private void
    bindingAdded(BindingModel model, BindingInfo info)
    {
	if(model_ == model)
	{
	    bindings_.addElement(info);

	    fireTableRowsInserted(bindings_.size() - 1, bindings_.size() - 1);
	}
    }

    private void
    bindingRemoved(BindingModel model, NameComponent n)
    {
	if(model_ == model)
	{
	    int len = bindings_.size();
	    for(int i = 0 ; i < len ; i++)
	    {
		BindingInfo info = (BindingInfo)bindings_.elementAt(i);
		if(info.getID().equals(n.id) && info.getKind().equals(n.kind))
		{
		    bindings_.removeElementAt(i);
		    fireTableRowsDeleted(i, i);
		    break;
		}
	    }
	}
    }

    private void
    bindingChanged(BindingModel model, NameComponent old_name,
		   NameComponent new_name)
    {
	if(model_ == model)
	{
	    int len = bindings_.size();
	    for(int i = 0 ; i < len ; i++)
	    {
		BindingInfo info = (BindingInfo)bindings_.elementAt(i);
		ExtendedBinding b = info.getBinding();
		if(b.binding_name[0].id.equals(old_name.id) &&
		   b.binding_name[0].kind.equals(old_name.kind))
		{
		    b.binding_name[0].id = new_name.id;
		    b.binding_name[0].kind = new_name.kind;

		    //
		    // Approximate the time of the change
		    //
		    b.timestamp = (int)(System.currentTimeMillis() / 1000);

		    fireTableRowsUpdated(i, i);

		    return;
		}
	    }
	}
    }

    public void
    contextAdded(BindingModel model, NamingContext parent,
		 NamingContext child, NameComponent n)
    {
	//
	// Avoid duplicate table entries for contexts that are already
	// present when contextAdded is messaged by a tree node that is
	// currently loaded. Don't know a better solution than this hack.
	//
	int len = bindings_.size();
	for(int i = 0 ; i < len ; i++)
	{
	    BindingInfo info = (BindingInfo)bindings_.elementAt(i);
	    if(n.id.equals(info.getID()) && n.kind.equals(info.getKind()))
		return;
	}

	NameComponent[] name = new NameComponent[1];
	name[0] = n;
	NcOrObj ncOrObj = new NcOrObj();
	ncOrObj.nc(child);
	ExtendedBinding b = new ExtendedBinding(
	    name, ncOrObj, (int)(System.currentTimeMillis() / 1000));
	BindingInfo info = new BindingInfo(parent, b);
	bindingAdded(model, info);
    }

    public void
    contextRemoved(BindingModel model, NameComponent n)
    {
	bindingRemoved(model, n);
    }

    public void
    contextChanged(BindingModel model, NameComponent old_name,
		   NameComponent new_name)
    {
	bindingChanged(model, old_name, new_name);
    }

    public void
    objectAdded(BindingModel model, NamingContext parent,
		org.omg.CORBA.Object child, NameComponent n)
    {
	NameComponent[] name = new NameComponent[1];
	name[0] = n;
	NcOrObj ncOrObj = new NcOrObj();
	ncOrObj.obj(child);
	ExtendedBinding b = new ExtendedBinding(
	    name, ncOrObj, (int)(System.currentTimeMillis() / 1000));
	BindingInfo info = new BindingInfo(parent, b);
	bindingAdded(model, info);
    }

    public void
    objectRemoved(BindingModel model, NameComponent n)
    {
	bindingRemoved(model, n);
    }

    public void
    objectChanged(BindingModel model, NameComponent old_name,
		  NameComponent new_name)
    {
	bindingChanged(model, old_name, new_name);
    }
}
