// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole;

import java.util.*;
import javax.swing.tree.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import com.ooc.CosNamingConsole.Util.*;
import com.ooc.CosNaming.OBNamingContextPackage.*;

final public class BindingTreeModel extends DefaultTreeModel
{
    BindingTreeModel(BindingNode root)
    {
	super(root);
    }

    //
    // Refresh node and subnodes
    //
    public void
    refresh(BindingNode parentNode)
    {
	//
	// Don't do anything if node contents are not visible yet
	//
	if(parentNode.hasLoaded())
	{
	    int i, j;

	    //
	    // Create list with all contexts to be displayed in this node
	    //
	    NamingContext nc =
		((BindingInfo)parentNode.getUserObject()).getContext();
	    Vector contexts = new Vector();
	    try
	    {
		contexts = BindingUtil.getContextsFromContext(nc);
	    }
	    catch(SystemException ex)
	    {
		// Ignore
	    }

	    //
	    // Get list with child nodes
	    //
	    Vector nodesToRemove = parentNode.getChildren();

	    //
	    // Create lists with nodes that have no corresponding context
	    // and with contexts that have no corresponding node
	    //
	    for(i = 0 ; i < nodesToRemove.size() ; i++)
	    {
		BindingNode node = ((BindingNode)nodesToRemove.elementAt(i));
		BindingInfo info = (BindingInfo)node.getUserObject();

		for(j = 0 ; j < contexts.size() ; j++)
		{
		    ExtendedBinding b =
			(ExtendedBinding)contexts.elementAt(j);

		    //
		    // Check if there is still a valid context for this node
		    //
		    if(info.getID().equals(b.binding_name[0].id) &&
		       info.getKind().equals(b.binding_name[0].kind))
		    {
			//
			// The node for this context is still valid, so
			// remove it from the list of contexts to be removed.
			//
			nodesToRemove.removeElementAt(i--);

			//
			// No new node is required for this context
			//
			contexts.removeElementAt(j--);

			break;
		    }
		}
	    }

	    Enumeration e;

	    //
	    // Remove nodes that have no corresponding context
	    //
	    e = nodesToRemove.elements();
	    while(e.hasMoreElements())
		removeNodeFromParent((BindingNode)e.nextElement());

	    //
	    // Create nodes for nodeless contexts
	    //
	    int nodesToAdd = contexts.size();
	    for(i = 0 ; i < nodesToAdd ; i++)
	    {
		ExtendedBinding b = (ExtendedBinding)contexts.elementAt(i);
		NamingContext child =
		    BindingUtil.getContextByName(nc, b.binding_name);
		if(child != null)
		{
		    BindingModel model = parentNode.getModel();
		    model.context_added(nc, child, b.binding_name[0]);
		}
	    }

	    //
	    // Refresh subnodes
	    //
	    e = parentNode.children();
	    while(e.hasMoreElements())
		refresh((BindingNode)e.nextElement());
	}
    }
}
