// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import javax.swing.event.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.ooc.CosNaming.OBNamingContextPackage.*;
import com.ooc.CosNamingConsole.Util.*;
import com.ooc.CosNamingConsole.GUI.*;
import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import com.ooc.Util.CORBA.*;
import com.ooc.Util.CORBA.GUI.*;

final class ClientWindow extends JFrame
    implements TreeSelectionListener,
	       ListSelectionListener,
	       ClipboardListener,
	       MouseListener
{
    final static private double version = 0.1;
    final static private String pattern = "NamingConsole";

    static private Icon oocIcon_;
    static private Icon upwardsIcon_;
    static private Icon newContextIcon_;
    static private Icon newBindingIcon_;
    static private Icon cutIcon_;
    static private Icon copyIcon_;
    static private Icon pasteIcon_;
    static private Icon deleteIcon_;
    static private Icon listIcon_;
    static private Icon detailsIcon_;
    static private Icon refreshIcon_;

    //
    // List of console windows
    //
    private static Vector windows_ = new Vector();

    private ORB orb_;
    private BOA boa_;

    private org.omg.CORBA.Object root_;

    private NamingContext nc_;

    //
    // True, if client is connected to an extended NamingService
    //
    private boolean extended_;

    //
    // True, if extended NamingService provides callbacks
    //
    private boolean callbacks_;

    //
    // True, if the user doesn't want automatic updates
    //
    private static boolean noUpdates_;

    //
    // Flags whether next paste operation follows a cut or a copy
    //
    private boolean wasCut_;

    //
    // The default and busy cursor
    //
    static private Cursor defaultCursor_ = Cursor.getDefaultCursor();
    static private Cursor busyCursor_ =
        Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);

    //
    // Object to log errors to
    //
    private MessageLogger logger_;

    //
    // File choosers
    //
    private JFileChooser contextFileChooser_;
    private JFileChooser iorFileChooser_;

    private JPopupMenu popupMenu_;
    private JComponent toolBar_;
    private IORStatusBar statusBar_;
    private JTree tree_;
    private BindingTable table_;
    private BindingTableModel bindingTableModel_;
    private BindingTreeModel bindingTreeModel_;
    private JLabel tableLabel_;
    private BindingNode currentNode_;
    private BindingInfo currentContext_;

    //
    // Dialog windows
    //
    private EnterIORDialog enterRootIORDialog_;
    private EnterIORDialog enterGeneralIORDialog_;

    //
    // Menu items that can be toggled/checked
    //
    private JCheckBoxMenuItem menuItemToolBar_;
    private JCheckBoxMenuItem menuItemStatusBar_;
    private JCheckBoxMenuItem menuItemErrorWindow_;
    private JRadioButtonMenuItem menuItemList_;
    private JRadioButtonMenuItem menuItemDetails_;

    //
    // Actions
    //
    private AbstractAction upwardsAction_;
    private AbstractAction newContextAction_;
    private AbstractAction newBindingAction_;
    private AbstractAction loadContextAction_;
    private AbstractAction saveContextAsAction_;
    private AbstractAction saveIORAction_;
    private AbstractAction changeIDAction_;
    private AbstractAction changeKindAction_;
    private AbstractAction changeIORAction_;
    private AbstractAction cutAction_;
    private AbstractAction copyAction_;
    private AbstractAction pasteAction_;
    private AbstractAction deleteAction_;
    private AbstractAction linkAction_;
    private AbstractAction unlinkAction_;
    private AbstractAction listAction_;
    private AbstractAction detailsAction_;
    private AbstractAction refreshAction_;
    private AbstractAction pingAction_;
    private AbstractAction cleanupAction_;
    private AbstractAction openAction_;
    private AbstractAction sortByTimeAction_;

    //
    // Constructor
    //
    ClientWindow(Point location, BOA boa, ORB orb, org.omg.CORBA.Object obj)
    {
	super(AppStandards.getString("TopWindowTitleKey"));

        synchronized(this.getClass())
        {
	    windows_.addElement(this);
        }

	orb_ = orb;
	boa_ = boa;
	root_ = obj;

	new Policies(orb);

	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	addWindowListener(new WindowAdapter()
	{
	    public void
	    windowClosing(WindowEvent e)
	    {
		closeWindow();
	    }
	});

	//
	// Initialize clipboard
	//
	Clipboard.instance().addClipboardListener(this);

	//
	// Initialize menus and toolbar
	//
	loadImages();
	createActions();
	setJMenuBar(createMenuBar());
	popupMenu_ = createPopupMenu();
	toolBar_ = createToolBar();

	enterRootIORDialog_ = new EnterIORDialog(this);
	enterGeneralIORDialog_ = new EnterIORDialog(this);

	JSplitPane hSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
	getContentPane().add("North", toolBar_);
	getContentPane().add("Center", hSplitPane);

	bindingTableModel_ = new BindingTableModel();
	table_ = new BindingTable(bindingTableModel_);
	table_.getSelectionModel().addListSelectionListener(this);
	table_.getTableHeader().addMouseListener(this);
	table_.addMouseListener(this);

	//
	// Create tree with leafs as closed nodes
	//
	tree_ = new JTree();
        DefaultTreeCellRenderer renderer =
            (DefaultTreeCellRenderer)tree_.getCellRenderer();
	Icon icon = renderer.getClosedIcon();
	renderer.setLeafIcon(icon);
	TableRenderer.setContextIcon(icon);

	tree_.setShowsRootHandles(true);
	tree_.getSelectionModel().setSelectionMode
	    (TreeSelectionModel.SINGLE_TREE_SELECTION);
	tree_.getSelectionModel().addTreeSelectionListener(this);
	tree_.addMouseListener(this);

	JScrollPane scrolledTree = new JScrollPane(tree_);
	scrolledTree.setMinimumSize(new Dimension(0, 0));
	scrolledTree.setPreferredSize(new Dimension(300, 400));
	JScrollPane scrolledTable = new JScrollPane(table_);
	scrolledTable.setPreferredSize(new Dimension(400, 400));
	scrolledTable.setMinimumSize(new Dimension(0, 0));

	logger_ = initErrorWindow();

	JPanel p;
	p = new JPanel(new BorderLayout(), false);
	p.add("North", new JLabel(" " +
				  AppStandards.getString("AllContextsKey")));
	p.add("Center", scrolledTree);
	hSplitPane.setLeftComponent(p);
	p = new JPanel(new BorderLayout(), false);
	tableLabel_ = new JLabel("");
	p.add("North", tableLabel_);
	p.add("Center", scrolledTable);
	hSplitPane.setRightComponent(p);

	statusBar_ = new IORStatusBar();
	getContentPane().add("South", statusBar_);

	setRootContext(obj);

	pack();
        setLocation(location);
	show();
    }

    static void
    setNoUpdates(boolean noUpdates)
    {
	noUpdates_ = noUpdates;
	BindingModel.setNoUpdates(noUpdates);
    }

    //
    // Create actions
    //
    private void
    createActions()
    {
	loadContextAction_ = new LoadContextAction();
	saveContextAsAction_ = new SaveContextAsAction();
	saveIORAction_ = new SaveIORAction();
	changeIDAction_ = new ChangeIDAction();
	changeKindAction_ = new ChangeKindAction();
	changeIORAction_ = new ChangeIORAction();
	cutAction_ = new CutAction();
	copyAction_ = new CopyAction();
	pasteAction_ = new PasteAction();
	deleteAction_ = new DeleteAction();
	linkAction_ = new LinkAction();
	unlinkAction_ = new UnlinkAction();
	listAction_ = new ShowListAction();
	detailsAction_ = new ShowDetailsAction();
	pingAction_ = new PingAction();
	cleanupAction_ = new CleanupAction();
	openAction_ = new OpenAction();
	sortByTimeAction_ = new SortByTimeAction();

	newContextAction_ = new AbstractAction()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    newEntry(BindingType.ncontext, null, null);
		}
	    };

	newBindingAction_ = new AbstractAction()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    enterGeneralIORDialog_.setVisible(true);
		    if(enterGeneralIORDialog_.getResult())
			newEntry(BindingType.nobject, null,
				 enterGeneralIORDialog_.getObject());
		}
	    };

	refreshAction_ = new AbstractAction()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    refresh();
		}
	    };

	upwardsAction_ = new AbstractAction()
	    {
		public void actionPerformed(ActionEvent e)
		{
		    selectNode((BindingNode)(currentNode_.getParent()));

		    updateItemSensitivity();
		}
	    };
    }

    //
    // Create menu item with accelerator and action
    //
    private JMenuItem
    createMenuItem(JMenu menu, String key, AbstractAction action)
    {
	JMenuItem item = menu.add(action);

	item.setText(AppStandards.getString(key + "Key"));
	item.setMnemonic(AppStandards.getChar(key + "MnemKey"));

	char accel = AppStandards.getChar(key + "AccelKey");
	if(accel != 0)
	{
	    //
	    // Simple hack for Delete shortcut (no Ctrl)
	    //
	    int keyMask = 0;
	    if(!key.equals("Delete"))
		keyMask = getToolkit().getMenuShortcutKeyMask();
	    item.setAccelerator(KeyStroke.getKeyStroke(accel, keyMask));
	}

	return item;
    }

    //
    // Create popup menu item with and action
    //
    private JMenuItem
    createMenuItem(JPopupMenu menu, String key, AbstractAction action)
    {
	JMenuItem item = menu.add(action);

	item.setText(AppStandards.getString(key + "Key"));
	item.setMnemonic(AppStandards.getChar(key + "MnemKey"));

	return item;
    }

    //
    // Create menubar
    //
    private JMenuBar
    createMenuBar()
    {
	JMenu menu, subMenu;
	JMenuBar menuBar = new JMenuBar();
	JMenuItem menuItem;

	//
	// File menu
	//
	menu = new JMenu(AppStandards.getString("FileMenuKey"), true);
	menu.setMnemonic(AppStandards.getChar("FileMenuAccelKey"));
	menuBar.add(menu);

	createMenuItem(menu, "NewWindow", new NewWindowAction());
	menu.addSeparator();

	createMenuItem(menu, "SwitchRoot", new SwitchRootAction());
	menu.addSeparator();

	createMenuItem(menu, "LoadContext", loadContextAction_);
	createMenuItem(menu, "SaveContextAs", saveContextAsAction_);
	createMenuItem(menu, "SaveIOR", saveIORAction_);
	menu.addSeparator();

	createMenuItem(menu, "CloseWindow", new CloseWindowAction());
	createMenuItem(menu, "Exit", new ExitAction());

	//
	// Edit menu
	//
	menu = new JMenu(AppStandards.getString("EditMenuKey"), true);
	menu.setMnemonic('E');
	menuBar.add(menu);

	createMenuItem(menu, "NewContext", newContextAction_);
	createMenuItem(menu, "NewBinding", newBindingAction_);
	createMenuItem(menu, "Delete", deleteAction_);
	createMenuItem(menu, "Link", linkAction_);
	createMenuItem(menu, "Unlink", unlinkAction_);
	menu.addSeparator();

	createMenuItem(menu, "Cut", cutAction_);
	createMenuItem(menu, "Copy", copyAction_);
	createMenuItem(menu, "Paste", pasteAction_);
	menu.addSeparator();

	createMenuItem(menu, "ChangeId", changeIDAction_);
	createMenuItem(menu, "ChangeKind", changeKindAction_);
	createMenuItem(menu, "ChangeIOR", changeIORAction_);
	menu.addSeparator();

	createMenuItem(menu, "SelectAll", new SelectAllAction());
	createMenuItem(menu, "InvertSelection", new InvertAction());

	//
	// View menu
	//
	menu = new JMenu(AppStandards.getString("ViewMenuKey"), true);
	menu.setMnemonic('V');
	menuBar.add(menu);

	menuItemToolBar_ = new JCheckBoxMenuItem(
	    AppStandards.getString("ToggleToolBarKey"), true);
	menuItemToolBar_.setMnemonic(
	    AppStandards.getChar("ToggleToolBarAccelKey"));
	menuItemToolBar_.addActionListener(new AbstractAction()
            {
		public void actionPerformed(ActionEvent e)
		{
		    toolBar_.setVisible(menuItemToolBar_.getState());
		    validate();
		}
	    });
	menu.add(menuItemToolBar_);

	menuItemStatusBar_ = new JCheckBoxMenuItem(
	    AppStandards.getString("ToggleStatusBarKey"), true);
	menuItemStatusBar_.setMnemonic(
	    AppStandards.getChar("ToggleStatusBarAccelKey"));
	menuItemStatusBar_.addActionListener(new AbstractAction()
	    {
		public void actionPerformed(ActionEvent e)
	        {
		    statusBar_.setVisible(menuItemStatusBar_.getState());
		    validate();
		}
	    });
	menu.add(menuItemStatusBar_);

	menuItemErrorWindow_ = new JCheckBoxMenuItem(
	    AppStandards.getString("ToggleErrorWindowKey"), false);
	menuItemErrorWindow_.setMnemonic(
	    AppStandards.getChar("ToggleErrorWindowAccelKey"));
	menuItemErrorWindow_.addActionListener(new AbstractAction()
	    {
		public void actionPerformed(ActionEvent e)
	        {
		    WindowFactory.getErrorWindowInstance().setVisible(
			menuItemErrorWindow_.getState());
//		    WindowFactory.getErrorWindowInstance().setState(
//			menuItemErrorWindow_.getState()
//			? Frame.NORMAL : Frame.ICONIFIED);
		}
	    });
	menu.add(menuItemErrorWindow_);

	menu.addSeparator();
	menuItemList_ = new JRadioButtonMenuItem(
	    AppStandards.getString("ShowListKey"));
	menuItemList_.setSelected(true);
	menuItemList_.setMnemonic(
	    AppStandards.getChar("ShowListAccelKey"));
	menuItemList_.addActionListener(listAction_);
	menu.add(menuItemList_);

	menuItemDetails_ = new JRadioButtonMenuItem(
	    AppStandards.getString("ShowDetailsKey"));
	menuItemDetails_.setMnemonic(
	    AppStandards.getChar("ShowDetailsAccelKey"));
	menuItemDetails_.addActionListener(detailsAction_);
	menu.add(menuItemDetails_);

	menu.addSeparator();

	subMenu =new JMenu(AppStandards.getString("SortMenuKey"));
	subMenu.setMnemonic(AppStandards.getChar("SortMenuAccelKey"));
	menu.add(subMenu);

	createMenuItem(subMenu, "SortById", new AbstractAction()
		       {
			   public void actionPerformed(ActionEvent e)
			   {
			       bindingTableModel_.setSortingMode(
				   Sorter._SortByID);
			       table_.clearSelection();
			   }
		       });
	createMenuItem(subMenu, "SortByKind", new AbstractAction()
		       {
			   public void actionPerformed(ActionEvent e)
			       {
				   bindingTableModel_.setSortingMode(
				       Sorter._SortByKind);
				   table_.clearSelection();
			       }
		       });
	createMenuItem(subMenu, "SortByTime", sortByTimeAction_);

	//
	// Tools menu
	//
	menu = new JMenu(AppStandards.getString("ToolsMenuKey"), true);
	menu.setMnemonic(AppStandards.getChar("ToolsMenuAccelKey"));
	menuBar.add(menu);

	createMenuItem(menu, "Ping", pingAction_);
	createMenuItem(menu, "Cleanup", cleanupAction_);

	return menuBar;
    }

    //
    // Create popup menu
    //
    private JPopupMenu
    createPopupMenu()
    {
	JPopupMenu menu = new JPopupMenu();

	createMenuItem(menu, "Open", openAction_);
	menu.addSeparator();

	createMenuItem(menu, "Cut", cutAction_);
	createMenuItem(menu, "Copy", copyAction_);
	createMenuItem(menu, "Paste", pasteAction_);
	menu.addSeparator();

	createMenuItem(menu, "ChangeId", changeIDAction_);
	createMenuItem(menu, "ChangeKind", changeKindAction_);
	createMenuItem(menu, "ChangeIOR", changeIORAction_);
	menu.addSeparator();

	createMenuItem(menu, "Delete", deleteAction_);
	createMenuItem(menu, "Unlink", unlinkAction_);
	menu.addSeparator();

	createMenuItem(menu, "SaveIOR", saveIORAction_);

	return menu;
    }

    //
    // Create toolBar
    //
    private JComponent
    createToolBar()
    {
	JButton b;
        Insets margin = new Insets(2, 2, 2, 2);

	JToolBar toolbar = new JToolBar();
	toolbar.setFloatable(false);
//        toolbar.setBorder(BorderFactory.createEmptyBorder(3, 0, 3, 0));

	b = toolbar.add(upwardsAction_);
	b.setToolTipText(AppStandards.getString("ToolbarUpKey"));
	b.setIcon(upwardsIcon_);
	b.setMargin(margin);
	toolbar.addSeparator();

	b = toolbar.add(newContextAction_);
	b.setToolTipText(AppStandards.getString("NewContextKey"));
	b.setIcon(newContextIcon_);
	b.setMargin(margin);
	b = toolbar.add(newBindingAction_);
	b.setToolTipText(AppStandards.getString("NewBindingKey"));
	b.setIcon(newBindingIcon_);
	b.setMargin(margin);
	toolbar.addSeparator();

	b = toolbar.add(cutAction_);
	b.setToolTipText(AppStandards.getString("ToolbarCutKey"));
	b.setIcon(cutIcon_);
	b.setMargin(margin);
	b = toolbar.add(copyAction_);
	b.setToolTipText(AppStandards.getString("ToolbarCopyKey"));
	b.setIcon(copyIcon_);
	b.setMargin(margin);
	b = toolbar.add(pasteAction_);
	b.setToolTipText(AppStandards.getString("ToolbarPasteKey"));
	b.setIcon(pasteIcon_);
	b.setMargin(margin);
	toolbar.addSeparator();

	b = toolbar.add(deleteAction_);
	b.setToolTipText(AppStandards.getString("ToolbarDeleteKey"));
	b.setIcon(deleteIcon_);
	b.setMargin(margin);
	toolbar.addSeparator();

	b = toolbar.add(listAction_);
	b.setToolTipText(AppStandards.getString("ToolbarShowListKey"));
	b.setIcon(listIcon_);
	b.setMargin(margin);
	b = toolbar.add(detailsAction_);
	b.setToolTipText(AppStandards.getString("ToolbarShowDetailsKey"));
	b.setIcon(detailsIcon_);
	b.setMargin(margin);

	toolbar.addSeparator();
	b = toolbar.add(refreshAction_);
	b.setToolTipText(AppStandards.getString("ToolbarRefreshKey"));
	b.setIcon(refreshIcon_);
	b.setMargin(margin);

	return toolbar;
    }

    //
    // Load the toolbar images
    //
    private void
    loadImages()
    {
	if(oocIcon_ == null)
	{
	    oocIcon_ = getImageResource("ooc.gif");
	    upwardsIcon_ = getImageResource("levelup.gif");
	    newContextIcon_ = getImageResource("newContext.gif");
	    newBindingIcon_ = getImageResource("newBinding.gif");
	    cutIcon_ = getImageResource("cut.gif");
	    copyIcon_ = getImageResource("copy.gif");
	    pasteIcon_ = getImageResource("paste.gif");
	    deleteIcon_ = getImageResource("delete.gif");
	    listIcon_ = getImageResource("simple.gif");
	    detailsIcon_ = getImageResource("details.gif");
	    refreshIcon_ = getImageResource("refresh.gif");
	}
    }

    //
    // Load images from 'images' folder
    //
    private Icon
    getImageResource(String imageFile)
    {
        Icon result = null;
 
        java.net.URL url = getClass().getResource("images/" + imageFile);
        if(url != null)
            result = new ImageIcon(url);
 
        return result;
    }

    //
    // Initialize error window instance
    //
    private MessageLogger
    initErrorWindow()
    {
	LogWindow errorWindow = WindowFactory.getErrorWindowInstance();
	errorWindow.addWindowListener(new WindowAdapter()
	{
	    public void windowClosing(WindowEvent e)
	    {
		menuItemErrorWindow_.setState(false);
	    }

	    public void windowOpened(WindowEvent e)
	    {
		menuItemErrorWindow_.setState(true);
	    }
	});

	return errorWindow.getLogger();
    }

    //
    // Enable/Disable menu and toolbar items
    //
    private void
    updateItemSensitivity()
    {
	boolean isContext = false;
	boolean isRoot = currentNode_.isRoot();
	boolean isNodeSelected = !tree_.isSelectionEmpty();
	boolean isRootSelected = !tree_.isSelectionEmpty() && isRoot;
	BindingInfo[] info = getSelected();
	int itemsSelected = info.length;
	boolean isSelected = itemsSelected != 0;
	if(itemsSelected == 1)
	    isContext = info[0].getContext() != null;

	openAction_.setEnabled(isSelected && isContext);
	loadContextAction_.setEnabled(isSelected && isContext);
	saveContextAsAction_.setEnabled(isSelected && isContext);
	saveIORAction_.setEnabled(itemsSelected == 1);
	changeIDAction_.setEnabled(itemsSelected == 1 && !isNodeSelected);
	changeKindAction_.setEnabled(itemsSelected == 1 && !isNodeSelected);
	changeIORAction_.setEnabled(itemsSelected == 1 && !isRootSelected &&
	    !isContext);
	deleteAction_.setEnabled(isSelected && !isRootSelected);
	linkAction_.setEnabled(itemsSelected == 1 && isSelected && isContext);
	unlinkAction_.setEnabled(isSelected && !isRootSelected);
	cutAction_.setEnabled(isSelected && !isRootSelected);
	copyAction_.setEnabled(isSelected && !isRootSelected);
	pingAction_.setEnabled(isSelected);
	cleanupAction_.setEnabled(isSelected && !isRootSelected);
	upwardsAction_.setEnabled(!isRoot);

	//
	// Check clipboard
	//
	pasteAction_.setEnabled(Clipboard.instance().getData() != null &&
				isContext);
    }

    //
    // Get selected node
    //
    private BindingNode
    getSelectedNode()
    {
	TreePath path = tree_.getSelectionPath();
	if(path != null)
	    return (BindingNode)path.getLastPathComponent();
	else
	    return null;
    }

    //
    // Get selected bindings
    //
    private BindingInfo[]
    getSelected()
    {
	BindingInfo[] nb;

	//
	// Check if tree element is selected
	//
	BindingNode node = getSelectedNode();
	if(node != null)
	{
	    nb = new BindingInfo[1];
	    nb[0] = (BindingInfo)node.getUserObject();
	}
	//
	// Otherwise table elements are selected
	//
	else
	{
	    int[] selectedRows = table_.getSelectedRows();

	    nb = new BindingInfo[selectedRows.length];
	    for(int i = 0 ; i < selectedRows.length ; i++)
		nb[i] = (BindingInfo)bindingTableModel_.getValueAt(
		    selectedRows[i], -1);
	}

	return nb;
    }

    //
    // New window
    //
    class NewWindowAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    Point location = getLocation();
	    location.x += 20;
	    location.y += 20;
	    new ClientWindow(location, boa_, orb_, root_);
	}
    }

    //
    // Switch to other naming service
    //
    class SwitchRootAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    enterRootIORDialog_.setVisible(true);
	    org.omg.CORBA.Object obj = enterRootIORDialog_.getObject();
	    if(obj != null)
		if(!setRootContext(obj))
		    MessageDialog.showError(
			ClientWindow.this,
			AppStandards.getString("SwitchErrorKey"));
	}
    }

    //
    // Create a new naming service entry with a unique name
    //
    private void
    newEntry(BindingType type, NamingContext ctx, org.omg.CORBA.Object obj)
    {
	BindingModel model = currentNode_.getModel();

	NamingContext nc = currentContext_.getContext();

	for(int i = 0 ; i < Integer.MAX_VALUE ; i++)
	{
	    String newName = type == BindingType.ncontext ?
		"NewContext" : "NewObject";
	    if(i != 0)
		newName += i;
	    NameComponent[] name = new NameComponent[1];
	    name[0] = new NameComponent(newName, "");

	    NcOrObj ncOrObj = new NcOrObj();

	    try
	    {
		if(type == BindingType.ncontext)
		{
		    NamingContext newContext = null;

		    if(ctx != null)
		    {
			nc.bind_context(name, ctx);
			newContext = ctx;
		    }
		    else
			newContext = nc.bind_new_context(name);

		    ncOrObj.nc(newContext);

		    if(!callbacks_)
			model.context_added(nc, newContext, name[0]);
		}
		else
		{
		    nc.bind(name, obj);
		    ncOrObj.obj(obj);

		    if(!callbacks_)
			model.object_added(nc, obj, name[0]);
		}

                ExtendedBinding b = new ExtendedBinding(
                    name, ncOrObj, (int)(System.currentTimeMillis() / 1000));

		break;
	    }
	    catch(AlreadyBound ex)
	    {
		// Next try
		continue;
	    }
	    catch(UserException ex)
	    {
		logger_.printException(ex);

		break;
	    }
	    catch(SystemException ex)
	    {
		logger_.printException(ex);

		break;
	    }
	}

	BindingInfo[] nb = getSelected();
	if(nb.length == 1)
	    statusBar_.displayIOR(nb[0].getObject());
	else
	    statusBar_.clear();
    }

    //
    // Delete binding
    //
    class DeleteAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    deleteBindings(getSelected(), true);
	}
    }

    //
    // Unlink binding
    //
    class UnlinkAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    deleteBindings(getSelected(), false);
	}
    }

    //
    // Link
    //
    class LinkAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
            enterGeneralIORDialog_.setVisible(true);
            org.omg.CORBA.Object obj = enterGeneralIORDialog_.getObject();
            if(obj != null)
            {
                BindingInfo info = getSelected()[0];
                ExtendedBinding b = info.getBinding();
                NamingContext nc = info.getContext();

		try
		{
                    if(info.getContext() != null)
                    {
			NamingContext newContext =
			    NamingContextHelper.narrow(obj);
			if(newContext != null)
			{
			    newEntry(BindingType.ncontext, newContext, obj);
			}
			else
			    throw new NotFound(NotFoundReason.not_object,
					       b.binding_name);
		    }
                    else
                    {
			enterGeneralIORDialog_.setVisible(true);
			if(enterGeneralIORDialog_.getResult())
			    newEntry(BindingType.nobject, null,
				     enterGeneralIORDialog_.getObject());
                    }
		}
		catch(SystemException ex)
		{
		    logger_.printException(ex);
		}
		catch(UserException ex)
		{
		    logger_.printException(ex);
		}
	    }
	}
    }

    //
    // Ping objects
    //
    class PingAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    WindowFactory.getPingWindowInstance().ping(getSelected());
	}
    }

    //
    // Clean up objects
    //
    class CleanupAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    setCursor(busyCursor_);

	    BindingInfo[] bindings = getSelected();

	    for(int i = 0 ; i < bindings.length ; i++)
	    {
		BindingInfo info = bindings[i];
		org.omg.CORBA.Object obj = info.getObject();
		if(obj != null)
		{
		    boolean exists = true;
		    try
		    {
			exists = !obj._non_existent();
		    }
                    catch(NO_RESPONSE ex)
                    {
                        // Do nothing if there is a timeout
                    }
		    catch(COMM_FAILURE ex)
		    {
			exists = false;
		    }

		    if(!exists)
		    {
			NamingContext nc = currentContext_.getContext();
			try
			{
			    nc.unbind(info.getBinding().binding_name);

			    logger_.println("`" + info.toString() +
					    "' was unbound");
			}
			catch(InvalidName exc)
			{
			}
			catch(NotFound exc)
			{
			}
			catch(CannotProceed exc)
			{
			}
                        catch(SystemException ex)
                        {
                            logger_.printException(ex);
                        }
		    }
		}
	    }

	    setCursor(defaultCursor_);
	}
    }

    //
    // Load context
    //
    class LoadContextAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    if(contextFileChooser_ == null)
	    {
		FileChooserFilter filter =
		    new FileChooserFilter("Naming Contexts (*.ctx)", "ctx");
		contextFileChooser_ = new JFileChooser();
		contextFileChooser_.addChoosableFileFilter(filter);
		contextFileChooser_.setFileFilter(filter);
	    }

	    contextFileChooser_.setDialogTitle(
		AppStandards.getString("LoadContextTitleKey"));

	    int result = contextFileChooser_.showDialog(ClientWindow.this,
							"OK");

	    if(result == JFileChooser.APPROVE_OPTION)
	    {
		File file = contextFileChooser_.getSelectedFile();

		NamingContext nc = currentContext_.getContext();
		
		setCursor(busyCursor_);

		try
		{
		    if(IO.load(orb_, file, nc))
		    {
			displayContext(nc);
		    }
		}
		catch(FileNotFoundException ex)
		{
		    setCursor(defaultCursor_);
		    MessageDialog.showError(ClientWindow.this,
					    AppStandards.getFormattedString(
						"FileNotFoundKey",
						file.getName()));
		}
		catch(IOException ex)
		{
		    setCursor(defaultCursor_);
		    MessageDialog.showError(ClientWindow.this,
					    AppStandards.getFormattedString(
						"FileErrorKey",
						file.getName()));
		}

		setCursor(defaultCursor_);
	    }
	}
    }

    //
    // Save context
    //
    class SaveContextAsAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    if(contextFileChooser_ == null)
	    {
		contextFileChooser_ = new JFileChooser();
		contextFileChooser_.addChoosableFileFilter(
		    new FileChooserFilter("Naming Contexts (*.ctx)", "ctx"));
	    }

	    contextFileChooser_.setDialogTitle(
		AppStandards.getString("SaveContextTitleKey"));

	    int result = contextFileChooser_.showDialog(ClientWindow.this,
							"OK");

	    if(result == JFileChooser.APPROVE_OPTION)
	    {
		File file = contextFileChooser_.getSelectedFile();

		// Overwrite file?
		if(file.exists())
		{
		    if(MessageDialog.showConfirm(
			ClientWindow.this, AppStandards.getFormattedString(
			    "FileExistsKey", file.getName())) != 0) return;
		}

		setCursor(busyCursor_);

		try
		{
		    ExtendedBinding b = currentContext_.getBinding();
		    IO.save(orb_, file, currentContext_.getContext(),
			    new Binding(b.binding_name,
					b.ncOrObj.discriminator()), logger_);
		}
		catch(IOException ex)
		{
		    setCursor(defaultCursor_);
		    MessageDialog.showError(ClientWindow.this,
					    AppStandards.getFormattedString(
						"FileErrorKey",
						file.getName()));
		}
		catch(COMM_FAILURE ex)
		{
		    logger_.printException(ex);
		}

		setCursor(defaultCursor_);
	    }
	}
    }

    //
    // Save IOR to file
    //
    class SaveIORAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    BindingNode parent = (BindingNode)(currentNode_.getParent());
	    boolean up = tree_.getSelectionPath() != null;

	    BindingInfo[] info = getSelected();

	    String ior = null;
	    try
	    {
		ior = orb_.object_to_string(info[0].getObject());
	    }
	    catch(SystemException ex)
	    {
		logger_.printException(ex);
	    }

	    if(ior != null)
	    {
		if(iorFileChooser_ == null)
		{
		    FileChooserFilter filter =
			new FileChooserFilter("IORs (*.ior)", "ior");
		    iorFileChooser_ = new JFileChooser();
		    iorFileChooser_.setDialogTitle(
			AppStandards.getString("SaveIORTitleKey"));
		    iorFileChooser_.addChoosableFileFilter(filter);
		    iorFileChooser_.setFileFilter(filter);
		}

		int result = iorFileChooser_.showDialog(
		    ClientWindow.this, "OK");

		if(result == JFileChooser.APPROVE_OPTION)
		{
		    File file = iorFileChooser_.getSelectedFile();

		    // Overwrite file?
		    if(file.exists())
		    {
			if(MessageDialog.showConfirm(
			    ClientWindow.this, AppStandards.getFormattedString(
				"FileExistsKey", file.getName())) != 0) return;
		    }

		    try
		    {
			PrintWriter out =
			    new PrintWriter(new FileOutputStream(file));
			out.println(ior);
			out.close();
		    }
		    catch(IOException ex)
		    {
			MessageDialog.showError(
			    ClientWindow.this,
			    AppStandards.getFormattedString(
				"FileErrorKey", file.getName()));
		    }
		}
	    }
	}
    }

    //
    // Close window
    //
    class CloseWindowAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    closeWindow();
	}
    }

    //
    // Exit
    //
    class ExitAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    checkForExit();
	}
    }

    //
    // Show simple list
    //
    class ShowListAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    menuItemList_.setSelected(true);
	    menuItemDetails_.setSelected(false);

	    bindingTableModel_.showDetails(false);
	    table_.sizeColumnsToFit(-1);
	}
    }

    //
    // Show details
    //
    class ShowDetailsAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    menuItemList_.setSelected(false);
	    menuItemDetails_.setSelected(true);

	    bindingTableModel_.showDetails(true);
	    table_.sizeColumnsToFit(-1);
	}
    }

    //
    // Sort by time
    //
    class SortByTimeAction extends AbstractAction
    {
	public void actionPerformed(ActionEvent e)
	{
	    bindingTableModel_.setSortingMode(Sorter._SortByTime);
	    table_.clearSelection();
	}
    }

    //
    // Open context
    //
    class OpenAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    BindingInfo[] info = getSelected();

	    Enumeration en = currentNode_.children();
	    while(en.hasMoreElements())
	    {
		//
		// Identify node
		//
		BindingNode node = (BindingNode)en.nextElement();
		BindingInfo inf = (BindingInfo)node.getUserObject();
		if(inf.getID().equals(info[0].getID())
		   && inf.getKind().equals(info[0].getKind()))
		{
		    TreePath path = new TreePath(node.getPath());
		    tree_.expandPath(path);
		    tree_.setSelectionPath(path);

		    //
		    // Focus must be set explicitly here
		    //
		    tree_.requestFocus();

		    return;
		}
	    }
	}
    }

    //
    // Cut
    //
    class CutAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    BindingInfo[] info = getSelected();
	    Clipboard.instance().setData(new ClipboardData(null, info));

	    wasCut_ = true;
	}
    }

    //
    // Copy
    //
    class CopyAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    BindingInfo[] info = getSelected();
	    Clipboard.instance().setData(new ClipboardData(null, info));

	    wasCut_ = false;
	}
    }

    //
    // Paste
    //
    class PasteAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    setCursor(busyCursor_);

	    BindingModel model = currentNode_.getModel();

	    NamingContext target = getSelected()[0].getContext();
	    ClipboardData clipData = Clipboard.instance().getData();
	    BindingInfo[] info = (BindingInfo[])clipData.data;

	    for(int i = 0 ; i < info.length ; i++)
	    {
		ExtendedBinding b = info[i].getBinding();

		NameComponent[] name = new NameComponent[1];
		name[0] = b.binding_name[b.binding_name.length - 1];

		try
		{
		    NamingContext parent = info[i].getParentContext();

		    if(b.ncOrObj.discriminator() == BindingType.ncontext)
		    {
			NamingContext src = info[i].getContext();

			NamingContext dst = null;
			if(!wasCut_)
			{
			    dst = target.new_context();
			    BindingUtil.deepCopy(orb_, dst, src);
			    target.bind_context(name, dst);
			}
			else
			{
			    dst = src;
			    parent.unbind(name);
			    target.bind_context(name, src);
			}

			if(!callbacks_)
			    model.context_added(target, dst, name[0]);
		    }
		    else
		    {
			org.omg.CORBA.Object obj = info[i].getObject();

			if(wasCut_)
			    parent.unbind(name);
			target.bind(name, obj);

			if(!callbacks_)
			    model.object_added(target, obj, name[0]);
		    }
		}
		catch(UserException ex)
		{
		    logger_.printException(ex, info[i]);
		}
		catch(SystemException ex)
		{
		    logger_.printException(ex, info[i]);
		}
	    }

	    if(wasCut_)
		Clipboard.instance().setData(null);

	    setCursor(defaultCursor_);
	}
    }

    //
    // Change IOR of an object binding
    //
    class ChangeIORAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    enterGeneralIORDialog_.setVisible(true);
	    org.omg.CORBA.Object obj = enterGeneralIORDialog_.getObject();
	    if(obj != null)
	    {
		BindingInfo info = getSelected()[0];
		ExtendedBinding b = info.getBinding();
		NamingContext nc = info.getParentContext();

		try
		{
		    nc.rebind(b.binding_name, obj);
		    info.setObject(obj);
		}
		catch(UserException ex)
		{
		    logger_.printException(ex);
		}
		catch(SystemException ex)
		{
		    logger_.printException(ex);
		}
	    }
	}
    }

    //
    // Change ID
    //
    class ChangeIDAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    table_.editCellAt(table_.getSelectedRow(), 0,
			      new EventObject(ClientWindow.this));
	}
    }

    //
    // Change kind
    //
    class ChangeKindAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    table_.editCellAt(table_.getSelectedRow(), 1,
			      new EventObject(ClientWindow.this));
	}
    }

    //
    // Select all
    //
    class SelectAllAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    if(table_.getRowCount() != 0)
		table_.setRowSelectionInterval(0, table_.getRowCount() - 1);
	}
    }

    //
    // Invert
    //
    class InvertAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    for(int i = 0 ; i < table_.getRowCount() ; i++)
	    {
		if(table_.isRowSelected(i))
		    table_.removeRowSelectionInterval(i, i);
		else
		    table_.addRowSelectionInterval(i, i);
	    }
	}
    }

    //
    // Select tree node
    //
    private void
    selectNode(BindingNode node)
    {
	tree_.setSelectionPath(new TreePath(node.getPath()));
	tree_.requestFocus();
    }

    //
    // Delete bindings
    //
    private void
    deleteBindings(BindingInfo[] info, boolean del)
    {
	//
	// Are there any selected bindings?
	//
	if(info.length == 0)
	    return;

	setCursor(busyCursor_);

	BindingModel model = currentNode_.getModel();

	//
	// All bindings belong to the same context
	//
	NamingContext nc = info[0].getParentContext();

	for(int i = 0 ; i < info.length ; i++)
	{
	    ExtendedBinding b = info[i].getBinding();
	    try
	    {
		//
		// Recursively delete context contents
		//
		if(b.ncOrObj.discriminator() == BindingType.ncontext)
		{
		    NamingContext child = info[i].getContext();

		    if(del)
		    {
			try
			{
			    child.destroy();
			}
			catch(NotEmpty ex)
			{
			    //
			    // Ask for confirmation if a context to be
			    // deleted is not empty
			    //
			    if(MessageDialog.showConfirm(
				ClientWindow.this,
				AppStandards.getFormattedString(
				    "NotEmptyKey",
				    BindingUtil.getFullName(b))) != 0)
				break;

			    BindingUtil.deleteContents(orb_, child);
			    child.destroy();
			}
		    }
		}

		nc.unbind(b.binding_name);

		if(!callbacks_)
		{
		    if(b.ncOrObj.discriminator() == BindingType.nobject)
			model.object_removed(nc, b.binding_name[0]);
		    else
			model.context_removed(nc, b.binding_name[0]);
		}
	    }
	    catch(UserException ex)
	    {
		logger_.printException(ex, info[i]);
	    }
	    catch(SystemException ex)
	    {
		logger_.printException(ex, info[i]);
	    }
	}

	setCursor(defaultCursor_);
    }

    com.ooc.CosNaming.OBNamingContext
    setContext(NamingContext nc)
    {
        if(bindingTableModel_.setContext(nc))
        {
            nc_ = nc;

            return com.ooc.CosNaming.OBNamingContextHelper.narrow(nc);
        }
        else
	{
	    BindingInfo info = (BindingInfo)currentNode_.getUserObject();

            return null;
	}
    }

    //
    // Set root context
    //
    private boolean
    setRootContext(org.omg.CORBA.Object obj)
    {
	NamingContext nc = null;
	try
	{
	    nc = NamingContextHelper.narrow(obj);
	}
	catch(SystemException ex)
	{
	    logger_.printException(ex);
	}

	if(nc == null)
	    return false;

	com.ooc.CosNaming.OBNamingContext obnc = setContext(nc);
	if(obnc != null)
	{
	    extended_ = true;
	    callbacks_ = obnc.callbacks_active() && !noUpdates_;
	}
	else
	{
	    extended_ = false;
	    callbacks_ = false;
	}

        //
        // Delete all previous nodes (and thus all models)
        //
        if(currentNode_ != null)
        {
            BindingNode root = (BindingNode)currentNode_.getRoot();
            root.removeFromParent();
        }

	//
	// Availability of detailed list depends on NamingService type
	//
	menuItemList_.setEnabled(extended_);
	menuItemDetails_.setEnabled(extended_);
	listAction_.setEnabled(extended_);
	detailsAction_.setEnabled(extended_);
	sortByTimeAction_.setEnabled(extended_);
	if(!extended_)
	    bindingTableModel_.showDetails(false);

	NameComponent[] name = new NameComponent[1];
	name[0] = new NameComponent("Root", "");
	NcOrObj ncOrObj = new NcOrObj();
	ncOrObj.nc(nc);
	currentContext_ = new BindingInfo(null, new ExtendedBinding(
	    name, ncOrObj, (int)System.currentTimeMillis() / 1000));

        BindingModel bindingModel = new BindingModel(boa_, orb_,
						     currentContext_);
	currentNode_ = bindingModel.getNode();
	bindingTreeModel_ = new BindingTreeModel(currentNode_);

	bindingModel.init(tree_, bindingTreeModel_, bindingTableModel_);
	new Thread(bindingModel).start();

	tree_.setModel(bindingTreeModel_);
	bindingTableModel_.setModel(bindingModel);

	//
	// Preselect root node
	//
	tree_.setSelectionRow(0);

	refresh();

	return true;
    }

    //
    // Display context
    //
    private void
    displayContext(NamingContext nc)
    {
        bindingTableModel_.setModel(currentNode_.getModel());

	TreeNode[] path = currentNode_.getPath();
	String s = " " + AppStandards.getString("ContentsOfKey") + " '";
	for(int i = 0 ; i < path.length ; i++)
	{
	    if(i != 0)
		s += "/";
	    s += path[i];
	}
	s += "'";
	tableLabel_.setText(s);

	setContext(nc);

	updateItemSensitivity();
    }

    //
    // Refresh complete tree
    //
    private void
    refresh()
    {
	setCursor(busyCursor_);

	try
	{
	    bindingTreeModel_.refresh(
		(BindingNode)bindingTreeModel_.getRoot());
	}
	catch(SystemException ex)
	{
	    logger_.printException(ex);
	}

	table_.clearSelection();

	displayContext(currentContext_.getContext());

	setCursor(defaultCursor_);
    }

    //
    // Exit, close all windows
    //
    private void
    checkForExit()
    {
	if(MessageDialog.showConfirm(
	    this, AppStandards.getString("QuitKey")) == MessageDialog.OK)
	{
	    synchronized(this.getClass())
	    {
		while(true)
		{
		    try
		    {
			ClientWindow c = (ClientWindow)windows_.firstElement();
			c.cleanup();
		    }
		    catch(NoSuchElementException ex)
		    {
			// No more windows
			break;
		    }
		}
	    }

	    System.exit(0);
	}
    }

    //
    // Close the console window
    //
    private void
    closeWindow()
    {
	if(windows_.size() == 1)
	    checkForExit();
	else
	    cleanup();
    }

    //
    // Clean up and unregister
    //
    private void
    cleanup()
    {
        //
        // Delete all nodes (and thus all models)
        //
        BindingNode root = (BindingNode)currentNode_.getRoot();
        root.destroy();

        windows_.removeElement(this);
        setVisible(false);
        dispose();
    }

    // From ListSelectionListener
    public void
    valueChanged(ListSelectionEvent e)
    {
	if(e.getValueIsAdjusting())
	    return;

	//
	// Check for selection, don't react on deselection
	//
	if(table_.getSelectedRowCount() != 0)
	{
	    tree_.clearSelection();
    
	    BindingInfo[] nb = getSelected();
	    if(nb.length == 1)
		statusBar_.displayIOR(nb[0].getObject());
	    else
		statusBar_.clear();
	}
	else
	    statusBar_.clear();

	updateItemSensitivity();
    }

    // From TreeSelectionListener
    public void
    valueChanged(TreeSelectionEvent e)
    {
	//
	// Stop editing
	//
	if(table_.isEditing())
	{
	    TableCellEditor editor = table_.getCellEditor(
		table_.getEditingRow(), table_.getEditingColumn());
	    editor.stopCellEditing();
	}

	if(e.isAddedPath())
	{
	    currentNode_ = (BindingNode)e.getPath().getLastPathComponent();
	    currentContext_ = (BindingInfo)currentNode_.getUserObject();

	    displayContext(currentContext_.getContext());

	    table_.clearSelection();

	    statusBar_.displayIOR(currentContext_.getObject());

	    updateItemSensitivity();
	}
    }

    // From ClipboardListener
    public void
    clipboardChanged(ClipboardData data)
    {
        updateItemSensitivity();
    }

    // From MouseListener
    public void
    mouseClicked(MouseEvent e)
    {
	Component invoker = (Component)(e.getSource());
	if(invoker == table_.getTableHeader())
	{
	    int column = table_.getTableHeader().columnAtPoint(e.getPoint());
	    int modelIndex =
		table_.getColumnModel().getColumn(column).getModelIndex();
	    bindingTableModel_.setSortingMode(modelIndex);
	    table_.clearSelection();
	}
    }

    // From MouseListener
    public void
    mouseEntered(MouseEvent e)
    {
    }

    // From MouseListener
    public void
    mouseExited(MouseEvent e)
    {
    }

    // From MouseListener
    public void
    mousePressed(MouseEvent e)
    {
	Component invoker = (Component)(e.getSource());
	if(invoker != table_.getTableHeader())
	{
	    //
	    // isPopupTrigger() always returns false under NT, so
	    // additionally check for right mouse button explicitly
	    //
	    if(e.isPopupTrigger() || SwingUtilities.isRightMouseButton(e))
		popupMenu_.show(invoker, e.getPoint().x, e.getPoint().y);
	}
    }

    // From MouseListener
    public void
    mouseReleased(MouseEvent e)
    {
    }
}
