// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole.GUI;

import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import org.omg.CORBA.*;
import com.ooc.CosNamingConsole.*;
import com.ooc.Util.CORBA.*;
import com.ooc.Util.GUI.*;

final public class PingWindow extends LogWindow
{
    private javax.swing.Timer timer_;
    private Vector bindings_;

    PingWindow(String title)
    {
	super(title);

	timer_ = new javax.swing.Timer(1000, this);
	timer_.setInitialDelay(0);
	timer_.setCoalesce(true);
    }

    protected void
    addButtons(JPanel buttonPanel)
    {
	super.addButtons(buttonPanel);

	JButton button;
        button = AppStandards.createButton("StopButtonKey");
        button.setActionCommand("stop");
        button.addActionListener(this);
        buttonPanel.add(button);
        button = AppStandards.createButton("ContButtonKey");
        button.setActionCommand("cont");
        button.addActionListener(this);
        buttonPanel.add(button);
    }

    //
    // Ping objects in BindingInfo list
    //
    public void
    ping(BindingInfo[] bindings)
    {
	if(bindings_ != null)
	    stop();

	if(bindings != null && bindings.length != 0)
	{
	    bindings_ = new Vector(bindings.length);

	    for(int i = 0 ; i < bindings.length ; i++)
		bindings_.addElement(bindings[i]);

	    timer_.start();
	}
    }

    //
    // Stop timer
    //
    private void
    stop()
    {
	timer_.stop();
    }

    //
    // Continue
    //
    private void
    cont()
    {
	timer_.start();
    }

    //
    // WIndow is being closed
    //
    protected void
    close()
    {
	stop();
    }

    //
    // ActionListener interface
    //
    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();

	//
	// Check for timer event
	//
	if(command == null)
	{
	    int timeout = 5000;

	    Enumeration en = bindings_.elements();
	    while(en.hasMoreElements())
	    {
		BindingInfo info = (BindingInfo)en.nextElement();
		org.omg.CORBA.Object obj = info.getObject();
		if(obj != null)
		{
		    //
		    // Get object with timeout
		    //
		    obj = Policies.setTimeout(obj, timeout);

		    getLogger().print("Pinging `" + info.toString() +
				      "' ... ");

		    try
		    {
			long timeBefore = System.currentTimeMillis();
			obj._non_existent();
			long timeAfter = System.currentTimeMillis();

			getLogger().print((timeAfter - timeBefore) + " ms\n");
		    }
		    catch(NO_RESPONSE ex)
		    {
			getLogger().println("No response within " +
					    timeout + " ms");
		    }
		    catch(TRANSIENT ex)
		    {
			getLogger().println("Status couldn't be determined");
		    }
		    catch(SystemException ex)
		    {
			getLogger().printException(ex, null);

			//
			// Don't ping this object again
			//
			bindings_.removeElement(info);
		    }
		}
		else
		    getLogger().println("`" + info.toString() +
					"' is a nil object reference");
	    }
	}
	else if(command.equals("stop"))
	    stop();
	else if(command.equals("cont"))
	    cont();
	else
	    super.actionPerformed(e);
    }
}
