// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import com.ooc.Util.CORBA.*;
import com.ooc.Util.CORBA.GUI.*;

final public class Main
{
    //
    // NOTE: If you want to use a locale-specific resource bundle,
    //       you do NOT need to change the value for bundle_
    //
    static final String bundle_ =
        "com.ooc.CosNamingConsole.Util.AppResources";
 
    //
    // The official name for the initial reference for the Naming Service
    //
    static final String serviceName_ = "NameService";

    public static
    void main(String[] allArgs)
    {
	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

        GUIOptionFilter opt;
        BooleanOption helpOption;
	BooleanOption versionOption;
	BooleanOption iorOption;
	BooleanOption noUpdatesOption;
        StringOption fileOption;

        opt = new GUIOptionFilter();
        helpOption = new BooleanOption('h', "help");
	versionOption = new BooleanOption('v', "version");
        iorOption = new BooleanOption('i', "ior");
	noUpdatesOption = new BooleanOption('n', "no-updates");
        fileOption = new StringOption('f', "file");
        opt.addOption(helpOption);
	opt.addOption(versionOption);
	opt.addOption(iorOption);
	opt.addOption(noUpdatesOption);
        opt.addOption(fileOption);

        //
        // Initialize the AppStandards class, which loads our locale-
        // specific resource bundle
        //
        AppStandards.init(bundle_);

        //
        // Process the command-line arguments
        //
	String[] args = null;
        try
        {
            args = opt.parse(allArgs);
        }
        catch(OptionException ex)
        {
            System.err.println("Error: " + ex.getMessage());
            System.err.println();
            usage();
        }
 
	if(versionOption.isDefined())
	{
	    System.out.println("ORBacus Names Console " +
			       com.ooc.CORBA.Version.version);
	    System.exit(0);
	}
        else if(helpOption.isDefined())
            usage();

	try
        {
            //
            // Initialize ORB and BOA
            //
	    ORB orb = ORBManager.init(args, props);
	    BOA boa = ((com.ooc.CORBA.ORB)orb).BOA_init(args, props);

	    //
	    // Filter options
	    //
	    args = ((com.ooc.CORBA.ORB)orb).filter_options(args);
	    args = ((com.ooc.CORBA.BOA)boa).filter_options(args);

	    //
	    // Any remaining options? If there are, they are unknown
	    //
	    if(args.length != 0)
		usage();

	    //
	    // Set default reconnection policy to true
	    //
	    Any reconnectAny = orb.create_any();
	    reconnectAny.insert_boolean(true);
	    Any timeoutAny = orb.create_any();
	    timeoutAny.insert_long(5000);
	    Policy[] policies = new Policy[2];

	    try
	    {
		policies[0] = ((com.ooc.CORBA.ORB)orb).
		    create_policy(com.ooc.OB.RECONNECT_POLICY.value,
				  reconnectAny);
		policies[1] = ((com.ooc.CORBA.ORB)orb).
		    create_policy(com.ooc.OB.TIMEOUT_POLICY.value,
				  timeoutAny);

		PolicyManager pm = PolicyManagerHelper.narrow(
		    orb.resolve_initial_references("ORBPolicyManager"));

		pm.add_policy_overrides(policies);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		// Do nothing if anything fails
	    }
	    catch(org.omg.CORBA.PolicyManagerPackage.InvalidPolicies ex)
	    {
		// Do nothing if anything fails
	    }
	    catch(PolicyError ex)
	    {
		// Do nothing if anything fails
	    }

	    //
	    // Get naming service with the following priorities:
	    // 1. If a file was specified, read the IOR from this file.
	    // 2. Try to resolve the NameService initial reference.
	    //
	    org.omg.CORBA.Object namingObj = null;

	    if(fileOption.isDefined())
	    {
		try
		{
		    FileReader r = new FileReader(fileOption.getValue());
		    BufferedReader in = new BufferedReader(r);
		    String ref = in.readLine();
		    r.close();
 
		    namingObj = orb.string_to_object(ref);
		}
		catch(IOException ex)
		{
		    ex.printStackTrace();
		    System.exit(1);
		}
	    }
	    else
	    {
		//
		// Check for initial service
		//
		try
		{
		    namingObj = orb.resolve_initial_references("NameService");
		}
		catch(org.omg.CORBA.ORBPackage.InvalidName ex)
		{
		}
	    }

	    boolean exists = true;

	    //
	    // Check if service is accessible
	    //
	    if(namingObj != null)
	    {
		try
		{
		    exists = !namingObj._non_existent();
		}
		catch(COMM_FAILURE ex)
		{
		    exists = false;
		}
	    }
	    else
		exists = false;

	    if(!exists)
	    {
		MessageDialog.showError(null,
					"Couldn't access Naming Service.");
		    System.exit(0);
	    }

	    if(iorOption.isDefined() && namingObj != null)
		System.out.println(orb.object_to_string(namingObj));

	    ClientWindow.setNoUpdates(noUpdatesOption.isDefined());
	    ClientWindow clientWindow = new ClientWindow(
		new Point(100, 100), boa, orb, namingObj);

	    boa.impl_is_ready(null);

	}
        catch(SystemException ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }
    }

    private static void
    showError(String errorMessage)
    {
        JFrame frame = new JFrame("");
        java.lang.Object[] options = { "OK" };
        JOptionPane.showOptionDialog(null, errorMessage, "Warning",
            JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null,
            options, options[0]);
        System.exit(1);
    }

    private static void
    usage()
    {
	System.out.println(AppStandards.getString("UsageKey"));
        System.exit(1);
    }
}
