// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole.Util;

import java.util.*;
import javax.swing.tree.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import com.ooc.CosNaming.OBNamingContextPackage.*;
import com.ooc.CosNamingConsole.*;

final public class BindingModel
    extends com.ooc.CosNaming._BindingListenerImplBase
    implements Runnable
{
    private BOA boa_;
    private ORB orb_;

    //
    // True, if user doesn't want automatic updates
    //
    private static boolean noUpdates_;

    //
    // The extended naming context this model listens to
    //
    com.ooc.CosNaming.OBNamingContext obnc_;

    //
    // The tree
    //
    private javax.swing.JTree tree_;

    //
    // The tree model to delegate to
    //
    private BindingTreeModel treeModel_;

    //
    // The tree node corresponding to this model
    //
    private BindingNode node_;

    //
    // The table model to delegate to
    //
    private BindingTableModel tableModel_;

    public BindingModel(BOA boa, ORB orb, BindingInfo info)
    {
	boa_ = boa;
	orb_ = orb;

	orb.connect(this);

        node_ = new BindingNode(this, info);

        //
        // Register as listener
        //
        if(!noUpdates_)
        {
	    obnc_ = com.ooc.CosNaming.OBNamingContextHelper.narrow(
		info.getContext());
	    try
	    {
		if(obnc_ != null)
		    obnc_.add_listener(this);
	    }
	    catch(SystemException ex)
	    {
		// Ignore listener that is not reachable
	    }
	}
    }

    public static void
    setNoUpdates(boolean noUpdates)
    {
	noUpdates_ = noUpdates;
    }

    public static boolean
    getNoUpdates()
    {
	return noUpdates_;
    }

    public BindingNode
    getNode()
    {
        return node_;
    }

    public void
    init(javax.swing.JTree tree, BindingTreeModel treeModel,
	 BindingTableModel tableModel)
    {
	tree_ = tree;
        treeModel_ = treeModel;
        tableModel_ = tableModel;
    }

    public void
    destroy()
    {
        //
        // Disconnect as listener
        //
        if(obnc_ != null)
        {
            try
            {
                obnc_.remove_listener(this);
            }
            catch(SystemException ex)
            {
                // Exceptions don't matter here
            }
        }
    }

    // From java.lang.Thread
    public void
    run()
    {
	boa_.impl_is_ready(null);
    }

    // From BindingListener
    synchronized public void
    context_added(NamingContext parent, NamingContext child, NameComponent n)
    {
        if(node_.hasLoaded())
        {
            NameComponent[] name = new NameComponent[1];
            name[0] = n;
            NcOrObj bndUnion = new NcOrObj();
            bndUnion.nc(child);
            ExtendedBinding b = new ExtendedBinding(
                name, bndUnion, (int)(System.currentTimeMillis() / 1000));
            BindingInfo info = new BindingInfo(parent, b);
            BindingModel newModel = new BindingModel(boa_, orb_, info);
            newModel.init(tree_, treeModel_, tableModel_);
            BindingNode newNode = newModel.getNode();
            String newNodeName = newNode.toString();

            //
            // Insert node in alphabetical order
            //
            Enumeration e = node_.children();
            int i = 0;
            while(e.hasMoreElements())
            {
                BindingNode node = (BindingNode)e.nextElement();
                if(newNodeName.compareTo(node.toString()) < 0)
                    break;

                i++;
            }

	    treeModel_.insertNodeInto(newNode, node_, i);
        }

        tableModel_.contextAdded(this, parent, child, n);
    }

    // From BindingListener
    synchronized public void
    context_removed(NamingContext parent, NameComponent name)
    {
        String fullName = BindingUtil.getFullName(name);
        int count = node_.getChildCount();
        for(int i = 0 ; i < count ; i++)
        {
            //
            // Identify tree node using the context's name
            //
            BindingNode node = (BindingNode)node_.getChildAt(i);
            if(fullName.equals(node.toString()))
            {
		//
		// Select parent node if selected node was removed
		//
		TreePath path = new TreePath(node.getPath());
		TreePath spath = tree_.getSelectionPath();
		if(spath != null && spath.equals(path))
		{
		    BindingNode parentNode = (BindingNode)node.getParent();
		    tree_.setSelectionPath(
			new TreePath(parentNode.getPath()));
		    tree_.requestFocus();
		}

		treeModel_.removeNodeFromParent(node);

                break;
            }
        }

        tableModel_.contextRemoved(this, name);
    }

    // From BindingListener
    synchronized public void
    context_changed(NamingContext parent, NamingContext child,
		    NameComponent old_name, NameComponent new_name)
    {
	String fullName = BindingUtil.getFullName(old_name);
	int count = node_.getChildCount();
	for(int i = 0 ; i < count ; i++)
	{
	    //
	    // Identify tree node using the context's name
	    //
	    BindingNode node = (BindingNode)node_.getChildAt(i);
	    if(fullName.equals(node.toString()))
	    {
		BindingInfo info = (BindingInfo)node.getUserObject();
		ExtendedBinding b = info.getBinding();

		if(new_name.id.equals(old_name.id) &&
		   new_name.kind.equals(old_name.kind))
		    //
		    // Child's IOR has changed
		    //
		{
		    TreePath oldPath = new TreePath(node.getPath());
		    TreePath path = tree_.getSelectionPath();
		    boolean isSelected = path != null && path.equals(oldPath);

		    context_removed(parent, old_name);
		    context_added(parent, child, new_name);

		    if(isSelected)
		    {
			node = (BindingNode)node_.getChildAt(i);
			TreePath newPath = new TreePath(node.getPath());
			tree_.setSelectionPath(newPath);
		    }
		}
		else
		//
		// Child's name has changed
		//
		{
		    b.binding_name[0].id = new_name.id;
		    b.binding_name[0].kind = new_name.kind;
		    treeModel_.nodeChanged(node);	    
		}

		break;
	    }
	}

	tableModel_.contextChanged(this, old_name, new_name);
    }

    // From BindingListener
    synchronized public void
    object_added(NamingContext parent, org.omg.CORBA.Object child,
		 NameComponent name)
    {
	tableModel_.objectAdded(this, parent, child, name);
    }

    // From BindingListener
    synchronized public void
    object_removed(NamingContext parent, NameComponent name)
    {
	tableModel_.objectRemoved(this, name);
    }

    // From BindingListener
    synchronized public void
    object_changed(NamingContext parent, org.omg.CORBA.Object child,
		   NameComponent old_name, NameComponent new_name)
    {
	tableModel_.objectChanged(this, old_name, new_name);
    }
}
