// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole.Util;

import java.io.*;
import java.util.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.ooc.CosNamingConsole.*;

final public class IO
{
    final static private double version_ = 1.0;
    final static private String pattern_ = "NamingContext";

    private IO()
    {
    }

    //
    // Load naming context
    //
    public static boolean
    load(ORB orb, File file, NamingContext nc)
	throws IOException
    {
	BufferedReader in = null;
	String ident;

	in = new BufferedReader(new FileReader(file));

	ident = in.readLine();

	if(ident == null)
	    return false;

	StringTokenizer t = new StringTokenizer(ident);

	String pattern = null;
	String version = null;
	try
	{
	    pattern = t.nextToken();
	    version = t.nextToken();
	}
	catch(NoSuchElementException ex)
	{
	    return false;
	}

	if(!pattern.equals("#!" + pattern_))
	    return false;

	load(orb, in, nc);
	in.close();

	return true;
    }

    //
    // Load recursively
    //
    private static void
    load(ORB orb, java.io.BufferedReader in, NamingContext nc)
	throws IOException
    {
	do
	{
	    String tag, id, kind, ior;

	    tag = in.readLine();
	    if(tag == null || tag.equals("EndContext"))
		return;

	    id = in.readLine();
	    if(id == null)
		return;

	    kind = in.readLine();
	    if(kind == null)
		return;

	    ior = in.readLine();
	    if(ior == null)
		return;

	    NameComponent[] name = new NameComponent[1];
	    name[0] = new NameComponent(id, kind);
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);

	    try
	    {
		if(tag.equals("StartContext"))
		{
		    NamingContext newContext = nc.bind_new_context(name);

		    load(orb, in, newContext);
		}
		else if(tag.equals("StartObject"))
		    nc.bind(name, obj);
	    }
	    catch(CannotProceed ex)
	    {
	    }
	    catch(InvalidName ex)
	    {
	    }
	    catch(NotFound ex)
	    {
	    }
	    catch(AlreadyBound ex)
	    {
	    }
	}
	while(true);
    }

    //
    // Save naming context
    //
    public static void
    save(ORB orb, File file, NamingContext nc, Binding binding,
	 MessageLogger logger)
	throws IOException
    {
	PrintWriter out = new PrintWriter(new FileOutputStream(file));
	out.println("#!" + pattern_ + " " + version_);

	save(orb, out, nc, binding, new Vector(), logger);

	out.close();
    }

    //
    // Save recursively
    //
    private static void
    save(ORB orb, java.io.PrintWriter out, org.omg.CORBA.Object ctx,
	 Binding binding, Vector previousContexts, MessageLogger logger)
    {
	BindingListHolder blh = new BindingListHolder();
	BindingIteratorHolder bih = new BindingIteratorHolder();

	try
	{
	    NamingContext nc = NamingContextHelper.narrow(ctx);
	    nc.list(Integer.MAX_VALUE, blh, bih);

	    for(int i = 0 ; i < blh.value.length ; i++)
	    {
		Binding b = blh.value[i];

		org.omg.CORBA.Object obj = null;
		try
		{
		    obj = nc.resolve(b.binding_name);
		}
		catch(InvalidName ex)
		{
		}
		catch(CannotProceed ex)
		{
		}
		catch(NotFound ex)
		{
		}

		if(b.binding_type == BindingType.ncontext)
		    out.println("StartContext");
		else
		    out.println("StartObject");

		out.println(b.binding_name[0].id);
		out.println(b.binding_name[0].kind);
		out.println(orb.object_to_string(obj));

		//
		// Next level
		//
		if(b.binding_type == BindingType.ncontext)
		{
		    String s = orb.object_to_string(obj);
		    if(!previousContexts.contains(s))
		    {
			Vector contextVec = (Vector)previousContexts.clone();
			contextVec.addElement(s);
			save(orb, out, obj, b, contextVec, logger);
		    }
		    else
		    {
			logger.println("Recursion detected at `" +
				       BindingUtil.getFullName(b) + "'");
		    }

		    out.println("EndContext");
		}
	    }
	}
	catch(COMM_FAILURE ex)
	{
	    logger.println("Couldn't access NamingContext `" +
			   BindingUtil.getFullName(binding) + "'");
	}
    }
}
