// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole.Util;

import org.omg.CosNaming.*;
import com.ooc.CosNamingConsole.*;

final public class Sorter
{	
    //
    // Sorting modes
    //
    public static final int _SortByID = 0;
    public static final int _SortByKind = 1;
    public static final int _SortByTime = 2;

    //
    // Current sorting mode
    //
    private int sortingMode_ = _SortByID;

    //
    // Sorting order (ascending = 1, descending = -1)
    //
    private int ascending_ = 1;

    //
    // Set sorting mode, toggle sorting order
    //
    public void
    setSortingMode(int mode)
    {
	if(sortingMode_ != mode)
	{
	    sortingMode_ = mode;
	    ascending_ = 1;
	}
	//
	// Toggle sorting order on reselection
	//
	else
	    ascending_ = -ascending_;
    }

    //
    // Sort BindingInfo list
    //
    public void
    sort(java.util.Vector data, int l, int h)
    {
	int low = l;
	int hi = h;
	BindingInfo mid;

        if(h > l)
        {
            mid = (BindingInfo)data.elementAt((l + h) / 2);
 
            while(low <= hi)
            {
                while(low < h && compare((BindingInfo)data.elementAt(low),
					 mid) < 0)
                    low++;
 
                while(hi > l && compare((BindingInfo)data.elementAt(hi),
					mid) > 0)
                    hi--;
 
                if(low <= hi)
                {
		    Object tmp = data.elementAt(low);
		    data.setElementAt(data.elementAt(hi), low);
		    data.setElementAt(tmp, hi);
 
                    low++;
                    hi--;
                }
            }

           if(l < hi)
                sort(data, l, hi);
 
            if(low < h)
                sort(data, low, h);
        }
    }

    //
    // Compare binding infos according to sorting mode
    //
    private int
    compare(BindingInfo nb1, BindingInfo nb2)
    {
	return compare(nb1, nb2, sortingMode_, ascending_);
    }

    private static int
    compare(BindingInfo nb1, BindingInfo nb2, int mode, int order)
    {
	int result = 0;
	NameComponent n1 = nb1.getBinding().binding_name[0];
	NameComponent n2 = nb2.getBinding().binding_name[0];

	switch(mode)
	{
	case _SortByID:
	    //
	    // Contexts always first
	    //
	    if(nb1.getContext() == null && nb2.getContext() != null)
		return order;
	    else if(nb1.getContext() != null && nb2.getContext() == null)
		return -order;

	    result = n1.id.compareTo(n2.id);
	    if(result == 0)
		result = n1.kind.compareTo(n2.kind);

	    break;

	case _SortByKind:
	    result = n1.kind.compareTo(n2.kind);
	    if(result == 0)
		result = n1.id.compareTo(n2.id);
	    break;

	case _SortByTime:
	    result = nb2.getTime() - nb1.getTime();
	    if(result == 0)
	    {
		result = n1.id.compareTo(n2.id);
		if(result == 0)
		    result = n1.kind.compareTo(n2.kind);
	    }

	    break;

	default:
	    break;
	}

	return result * order;
    }
}
