// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package random;

import org.omg.CORBA.*;
import java.io.*;
import java.util.*;

final public class Client
{
    public static void main(String args[])
    {
        Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

	try
	{
	    //
	    // Create ORB
	    //
	    ORB orb = ORB.init(args, props);

	    //
	    // We are connecting to an Orbix server. Note that we can
	    // either use the generated IOR, or the iioploc:// form. The
	    // object key for iioploc:// was extracted using iordump.
	    //
	    String randomIOR = "IOR:000000000000000f49444c3a52616e646f6d3a312e3000000000000100000000000000500001000000000016706c616e7874792e6473672e63732e7463642e69650006220000002c3a5c706c616e7874792e6473672e63732e7463642e69653a52616e646f6d3a303a3a49523a52616e646f6d00";

	    //
	    // Note that this doesn't work since this type of IOR doesn't
	    // have a RepositoryID. This means that ORBacus calls _is_a on
	    // the call to _narrow below -- but unfortunately ORBIX has
	    // some bugs related to _is_a which causes the _narrow to
	    // fail.
	    //
	    // String randomIOR = "iioploc://planxty.dsg.cs.tcd.ie:1570/"
	    // ":\\planxty.dsg.cs.tcd.ie:Random:0::IR:Random";
	    //
	    org.omg.CORBA.Object obj = orb.string_to_object(randomIOR);
	    if(obj == null)
		throw new RuntimeException();

	    Random random = RandomHelper.narrow(obj);
	    if(random == null)
		throw new RuntimeException();

            //
            // Main loop
            //
            System.out.println("Enter 'r' for a random number, " +
			       "or 'x' for exit:");

            int c;

	    try
            {
                String input;
                do
                {
                    System.out.print("> ");
                    java.io.DataInputStream dataIn =
                        new java.io.DataInputStream(System.in);
                    BufferedReader in =
                        new BufferedReader(new InputStreamReader(dataIn));
                    input = in.readLine();

                    if(input.equals("r"))
		    {
			int l = random.lrand48();
			System.out.println(l);
		    }
                }
                while(!input.equals("x"));
            }
            catch(IOException ex)
            {
                System.err.println("Can't read from `" +
                                   ex.getMessage() + "'");
                System.exit(1);
            }
        }
        catch(COMM_FAILURE ex)
        {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
            System.exit(1);
        }

        System.exit(0);
    }
}
