// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CORBA;

//
// This class encapsulates the some of the various INS
// object-reference encodings.
//
class CorbaURL
{
    private String urlStr_;
    private String scheme_;
    private java.util.Vector addresses_;
    private byte[] key_;
    private int fragmentStartIdx_;
    private boolean rirURL_;

    // 
    // construct a corbaURL from a string. 
    // 
    public
    CorbaURL(final String urlStr)
    {
        urlStr_ = urlStr;
        parseScheme();
    }

    //   
    // Convert this URL to an object reference, based on URL scheme.
    // Schemes supported: corbaloc/corbaname/ ORBacus iiop.
    //                    iioploc/iiopname 
    // 
    public org.omg.CORBA.Object
    toObject(ORB orb)
    {
        if(scheme_.equals(iorPrefix_))
            return IORToObject(orb);

        if(scheme_.equals(corbalocPrefix_))
            return CORBALOCToObject(orb);

        if(scheme_.equals(corbanamePrefix_))
            return CORBANAMEToObject(orb);

        if(scheme_.equals(iiopPrefix_))
            return OBIIOPToObject(orb);

        if(scheme_.equals(iioplocPrefix_) ||
           scheme_.equals(iiopnamePrefix_))
        {
            String msg = new String("URL: use of deprecated scheme `");
            msg += urlStr_;
            msg += "'";
            MessageViewer.instance().warning(msg);
            return new CorbaURL(toNewURLString()).toObject(orb);
        }
        
        throw new org.omg.CORBA.BAD_PARAM(
            "URL: unknown scheme name", 
            MinorBadParam.MinorBadSchemeSpecificPart,
            org.omg.CORBA.CompletionStatus.COMPLETED_NO); 
    }

    // 
    // Convert a iiopname/iioploc to a corbaname/corbaloc
    // 
    private String
    toNewURLString()
    {
        // 
        // Check if URL is too short
        // 
        int fromIdx = scheme_.length()+3; // +3 = "://"
        if(fromIdx >= urlStr_.length())
        {
            throw new org.omg.CORBA.BAD_PARAM(
                "URL: incomplete, only scheme specified", 
                MinorBadParam.MinorBadSchemeSpecificPart,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        // 
        // Must be a iiopname or iioploc
        // 
        boolean isName = scheme_.equals(iiopnamePrefix_);

        if(isName == false && !scheme_.equals(iioplocPrefix_))
            throw new org.omg.CORBA.BAD_PARAM(
                "URL: unknown scheme name", 
                MinorBadParam.MinorBadSchemeSpecificPart,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        // 
        // Construct beginning of new URL
        // 
        StringBuffer newURL = new StringBuffer(2*urlStr_.length());
        newURL.append( isName ? corbanamePrefix_ : corbalocPrefix_);
        newURL.append(':');
        
        int toIdx = newURL.length();
        int addrEnd = urlStr_.indexOf('/', fromIdx);

        // 
        // URL missing key or stringified name
        // 
        if(addrEnd == -1)
            throw new org.omg.CORBA.BAD_PARAM(
                "URL: missing key or stringified name", 
                MinorBadParam.MinorBadSchemeSpecificPart,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        newURL.append(':');
        
        if(addrEnd == fromIdx)
        {
            MessageViewer.instance().warning(
                "URL: assigning deprecated default address " +
                "of `localhost:9999'");
            newURL.append("localhost:9999");
        }
        else
        {
            for(; fromIdx < addrEnd ; ++fromIdx)
            {
                char cp = urlStr_.charAt(fromIdx);
                newURL.append(cp);
                if(cp == ',')
                    newURL.append(':');
            }
        }
        
        if(isName)
        {
            newURL.append('#');
            ++fromIdx;
            
        }
    
        newURL.append(urlStr_.substring(fromIdx));
        return newURL.toString();
    }

    //
    // Convert an iiop:// object-URL to an Object.
    // The resulting object key will have a terminating null byte.
    //
    private org.omg.CORBA.Object
    OBIIOPToObject(ORB orb)
    {
	//
	// This avoids the need for a new protocol handler.
	//
	String u = "http" + urlStr_.substring(iiopPrefix_.length());
	org.omg.IOP.IOR ior;
	try
	{
	    java.net.URL url = new java.net.URL(u);
	    String host = url.getHost();
            if(host.length() == 0)
		throw new org.omg.CORBA.BAD_PARAM(
		    "URL: no host specified", 
                    MinorBadParam.MinorBadAddress,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                
	    int port = url.getPort();

	    if(port == -1 || port > 65535)
		throw new org.omg.CORBA.BAD_PARAM(
		    "URL: invalid port number", 
                    MinorBadParam.MinorBadAddress,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            
	    String name = null;
            String file = url.getFile();
            if(file.length() > 1)
	        name = file.substring(1);
            
            if(name == null || name.length() == 0)
		throw new org.omg.CORBA.BAD_PARAM(
                    "URL: missing key", MinorBadParam.MinorBadAddress,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            
	    return INS.getInetObject(orb, host, port, name, "");
	}
	catch(java.net.MalformedURLException ex)
	{
	    throw new org.omg.CORBA.BAD_PARAM(
		"URL: malformed", MinorBadParam.MinorOther,
		org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	}
    }

    // 
    // Convert an OMG corbaname URL to an object reference
    // 
    private org.omg.CORBA.Object
    CORBANAMEToObject(ORB orb)
    {
        org.omg.CORBA.Object service = CORBALOCToObject(orb);
        
        if(fragmentStartIdx_ == -1)
        {
            // 
            // If there is no URL fragment "#....." then the URL
            // refers to the naming context itself
            // 
            return service;
        }
	try
        {
            if(fragmentStartIdx_ + 1 >= urlStr_.length())
                throw new org.omg.CORBA.BAD_PARAM(
                    "corbaname: stringified name is missing", 
                    MinorBadParam.MinorBadSchemeSpecificPart,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);

            //
            // Create typecodes for Name and NameComponent
            //
            org.omg.CORBA.StructMember[] contents =
                new org.omg.CORBA.StructMember[2];
            
            contents[0] = new org.omg.CORBA.StructMember();
            contents[0].name = "id";
            contents[0].type = orb.create_string_tc(0);
            
            contents[1] = new org.omg.CORBA.StructMember();
            contents[1].name = "kind";
            contents[1].type = orb.create_string_tc(0);
            
            org.omg.CORBA.TypeCode tcNameComponent =
                orb.create_struct_tc("IDL:omg.org/CosNaming/NameComponent:1.0",
                                     "NameComponent", contents);
            
            org.omg.CORBA.TypeCode tcName = 
                orb.create_sequence_tc(0, tcNameComponent);
            
            //
            // Parse path, create NameComponent sequence
            //
            String strName = unescapeURL(urlStr_, 
                                         fragmentStartIdx_+1, -1);
            byte[] nameBytes = strToKey(strName);
            
            IIOPNameParser parser = new IIOPNameParser(nameBytes);
            if(!parser.isValid())
                throw new org.omg.CORBA.BAD_PARAM(
                    "URL: stringified name is invalid", 
                    MinorBadParam.MinorBadSchemeSpecificPart,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            
            String[] content = parser.getContents();

            org.omg.CORBA.Any[] as =
                new org.omg.CORBA.Any[content.length / 2];
            for(int i = 0 ; i < content.length ; i += 2)
            {
                //
                // Create the DynStruct containing the id and kind
                // fields
                //
                org.omg.CORBA.DynStruct name =
                    orb.create_dyn_struct(tcNameComponent);
                name.insert_string(content[i]);
                name.insert_string(content[i + 1]);
                as[i / 2] = name.to_any();
            }
            
            //
            // Create the DynSequence representing the name
            //
            org.omg.CORBA.DynSequence seq = orb.create_dyn_sequence(tcName);
            seq.length(as.length);
            seq.set_elements(as);
            
            //
            // Create the DII Request to the naming service to resolve
            // the name
            //
            org.omg.CORBA.Request request = service._request("resolve");
            
            //
            // Copy in the arguments
            //
            org.omg.CORBA.Any any = request.add_in_arg();
            org.omg.CORBA.Any seqany = seq.to_any();
            any.read_value(seqany.create_input_stream(), seqany.type());
            seq.destroy();
            
            request.set_return_type(orb.get_primitive_tc(
                org.omg.CORBA.TCKind.tk_objref));
            
            try
            {
                //
                // Invoke the request
                //
                request.invoke();
                
                //
                // Return the result if there was no exception
                //
                if(request.env().exception() == null)
                    return request.return_value().extract_Object();
            }
            catch(org.omg.CORBA.SystemException ex)
            {
                // Fall through
            }
            
            throw new org.omg.CORBA.BAD_PARAM(
                "URL: corbaname evaluation error", 
                MinorBadParam.MinorOther,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

	//
	// All of the following exceptions denote some sort of
	// internal error
	//
	catch(org.omg.CORBA.ORBPackage.InconsistentTypeCode e)
	{
	}
	catch(org.omg.CORBA.DynAnyPackage.InvalidValue e)
	{
	}
	catch(org.omg.CORBA.DynAnyPackage.Invalid e)
	{
	}
	catch(org.omg.CORBA.DynAnyPackage.InvalidSeq e)
	{
	}
	catch(org.omg.CORBA.BAD_OPERATION e)
	{
	}

        throw new org.omg.CORBA.INTERNAL(
            "URL: corbaname evaluation error",
            MinorBadParam.MinorOther,
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }

    // 
    // Convert a corbaloc URL to an object reference
    // 
    private org.omg.CORBA.Object
    CORBALOCToObject(ORB orb)
    {
        int addrEndIdx = parseCORBALOCAddresses();
        
        if(addresses_.size() == 0)
	    throw new org.omg.CORBA.BAD_PARAM(
		"URL: no protocol address", 
                MinorBadParam.MinorBadSchemeSpecificPart,
		org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        // 
        // Normal key form scheme:protocol:addr[/key]#fragment
        // 
        String keyStr = null;
        
        if(addrEndIdx != -1 && urlStr_.charAt(addrEndIdx) == '/')
        {
            int keyStartIdx = addrEndIdx + 1;
            fragmentStartIdx_ = parseKey(keyStartIdx);
            if(fragmentStartIdx_ == -1)
                keyStr = urlStr_.substring(keyStartIdx);
            else
                keyStr = urlStr_.substring(keyStartIdx,fragmentStartIdx_);
        }
        else if(addrEndIdx == -1 || urlStr_.charAt(addrEndIdx) == '#')
        {
            fragmentStartIdx_ = addrEndIdx;

            if(rirURL_ == true || scheme_.equals(corbanamePrefix_))
                keyStr = "NameService";
            else
                keyStr = "";

            key_ = strToKey(keyStr);
        }

        //
        // Although no object-key is allowed by the INS specification
        // we're going to make this an invalid IOR
        //
        if(keyStr.length() == 0)
            throw new org.omg.CORBA.BAD_PARAM(
                "URL: no key specified",
                MinorBadParam.MinorBadSchemeSpecificPart,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        
        // 
        //  An rir URL simply calls resolve_initial_references
        // 
        if(rirURL_ == true)     
        {
            try
            {
                return orb.resolve_initial_references(keyStr);
            }
            catch (org.omg.CORBA.ORBPackage.InvalidName invName)
            {
                throw new org.omg.CORBA.BAD_PARAM(
                    "URL: invalid initial reference token", 
                    MinorBadParam.MinorBadSchemeSpecificPart,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        }
        else
        {
            org.omg.IOP.IOR ior = new org.omg.IOP.IOR();
            ior.type_id = "";
            ior.profiles = new org.omg.IOP.TaggedProfile[addresses_.size()];

            // 
            // Set the IOR profiles
            // 
            for(int pr = 0 ; pr < addresses_.size() ; pr++)
            {
                URLAddr addr = (URLAddr)addresses_.elementAt(pr);
                addr.setProfile(ior, pr, key_);
            }
            return orb._OB_createObject(ior);
        }
    }

    // 
    // Parse the URL key starting at the given index.
    // Key is terminated by either end of string or the 
    // corbaname fragment character '#'.
    // 
    private int
    parseKey(int keyStartIdx)
    {
        int keyEndIdx = urlStr_.indexOf('#', keyStartIdx);
        int endCopy = (keyEndIdx != -1) ? keyEndIdx : urlStr_.length();

        String keyStr = unescapeURL(urlStr_, keyStartIdx, endCopy);
        key_ = strToKey(keyStr);

        return keyEndIdx;
    }
    
    // 
    // Convert a string to an object key. The returned key
    // does *not* have a terminating null byte.
    // 
    private byte[]
    strToKey(String keyStr)
    {
        byte[] retKey = new byte[keyStr.length()];

        for(int i = 0 ; i < retKey.length ; i++)
        {
            if(keyStr.charAt(i) > 255)
                throw new org.omg.CORBA.BAD_PARAM(
                    "URL: key string characters out of range", 
                    MinorBadParam.MinorBadSchemeSpecificPart,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            
            retKey[i] = (byte)keyStr.charAt(i);
        }

        return retKey;
    }
    
    // 
    // Parse the protocol/address information from a corbaloc/corbaname
    // URL
    // 
    private int
    parseCORBALOCAddresses()
    {
        // 
        // addr starts scheme's ':'
        // 
        int addrStartIdx = scheme_.length() + 1; // +1 = ':'

        // 
        // Make sure there is something after scheme
        // 
        if(addrStartIdx >= urlStr_.length())
	    throw new org.omg.CORBA.BAD_PARAM(
		"URL: incomplete, only scheme specified", 
                MinorBadParam.MinorBadSchemeSpecificPart,
		org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        rirURL_ = false;
        String addrStr = urlStr_.substring(addrStartIdx);
        int addrEndIdx = -1;

        // 
        // Parse the comma separated protocol/address info
        // This section of the URL is terminated by either
        // a '/' or a '#'
        // 
        java.util.StringTokenizer addrToken = 
            new java.util.StringTokenizer(addrStr, ",/#", true);
        addresses_= new java.util.Vector();
        
        while(addrToken.hasMoreTokens())
        {
            String token = addrToken.nextToken();
            if(token.equals(","))
                continue;
            else if(token.equals("#") || token.equals("/"))
            {
                addrEndIdx = urlStr_.indexOf(token.charAt(0));
                break;
            }
            else
                addresses_.addElement(parseSingleProtocolAddr(token));
        }
        return addrEndIdx;
    }
    
    // 
    // Convert an OMG IOR URL to an object reference
    // 
    private org.omg.CORBA.Object
    IORToObject(ORB orb)
    {
        // 
        // Skip the leading "IOR:" during conversion
        // 
        byte[] buf = HexConverter.asciiToOctets(urlStr_, 4);
	if(buf == null)
	    throw new org.omg.CORBA.BAD_PARAM(
		"URL: invalid", MinorBadParam.MinorBadSchemeSpecificPart,
		org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	try
	{
	    InputStream in = new InputStream(buf);
	    in._OB_readEndian();
            return orb._OB_createObject(org.omg.IOP.IORHelper.read(in));
	}
	catch(RuntimeException ex)
	{
		ex.printStackTrace();
	    throw new org.omg.CORBA.BAD_PARAM(
		"URL: invalid", 
                MinorBadParam.MinorBadSchemeSpecificPart,
		org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	}
    }
    
    // 
    // Parse the ':' terminated URL scheme
    // 
    private void
    parseScheme()
    {
        int schemeEnd = urlStr_.indexOf(':');

        if(schemeEnd == -1)
            throw new org.omg.CORBA.BAD_PARAM(
                "URL: no scheme specified",
                MinorBadParam.MinorOther,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        scheme_ = urlStr_.substring(0, schemeEnd).toLowerCase();
    }
    
    // 
    // Parse protocol and contact information from a string
    // returning a URLAddr for the protocol.
    // 
    URLAddr
    parseSingleProtocolAddr(String addrStr)
    {
        addrStr = unescapeURL(addrStr, 0, -1);

        // 
        // iiop
        // 
        if(addrStr.startsWith(iiopProt_) ||
           addrStr.startsWith(iiopProtColon_))
            return new IIOPAddr(addrStr);
        
        // 
        // rir
        // 
        if(addrStr.startsWith(rirProt_))
        {
            rirURL_ = true;
            return new RIRAddr(addrStr);
        }

        throw new org.omg.CORBA.BAD_PARAM(
            "URL: unknown protocol",
            MinorBadParam.MinorBadSchemeSpecificPart, 
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }
    
    // 
    // Remove URL escapes from the string
    // 
    public String
    unescapeURL(final String from, int readIdx, int endIdx)
    {
        // 
        // Possible TODO - 
        // 1. only create new string if there are escape chars.
        // 
        if(endIdx == -1)
            endIdx = from.length();
        
        char[] to = new char[endIdx - readIdx];
        int writeIdx = 0;

        while(readIdx < endIdx)
        {
            char ch = from.charAt(readIdx);
            if(ch == '%')
            {
                if(readIdx + 2 >= endIdx)
                    throw new org.omg.CORBA.BAD_PARAM(
                        "URL: bad escape sequence length",
                        MinorBadParam.MinorBadAddress, 
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);

                int c1 = Character.digit(from.charAt(++readIdx), 16);
                int c2 = Character.digit(from.charAt(++readIdx), 16);
                
                if(c1 == -1 || c2 == -1)
                    throw new org.omg.CORBA.BAD_PARAM(
                        "URL: escape sequence contains " + 
                        "invalid characters", 
                        MinorBadParam.MinorOther,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);

                ch = (char)((c1 << 4) | c2);
            }
            to[writeIdx++] = ch;
            ++readIdx;
        }
        return new String(to, 0, writeIdx);
    }

    // 
    // Corba URL Scheme Names
    // 
    // 
    // OMG Standard
    // 
    static private final String iorPrefix_= "ior";
    static private final String corbalocPrefix_ = "corbaloc";
    static private final String corbanamePrefix_ = "corbaname";

    // 
    // ORBacus Extensions
    // 
    static private final String iiopPrefix_ = "iiop";

    // 
    // OMG Deprecated
    // 
    static private final String iioplocPrefix_ = "iioploc";
    static private final String iiopnamePrefix_ = "iiopname";

    // 
    // corbaloc protocol table
    // 
    static private final String iiopProt_ = "iiop:";
    static private final String iiopProtColon_ = ":";
    static private final String rirProt_ = "rir:";
    
    // 
    // Abstract base class for URL protocol and address info
    // Each protocol specific class derived from URLAddr must
    // be capable of inserting its contact information in a
    // specific IOR profile.
    // 
    static public abstract class URLAddr
    {
        // 
        // Return protocol name
        // 
        public abstract String getProtocolName();

        // 
        //  Set an IOR profile using the supplied key
        // 
        public abstract void
        setProfile(org.omg.IOP.IOR ior, int ProfileNumber, 
                   final byte[] key);    
    }

    // 
    // rir: protocol class. rir cannot set an IOR profile
    // 
    static public class RIRAddr extends URLAddr
    {
        // 
        // Create an RIRAddr. raises an exception if
        // there is anything after the "rir:".
        // 
        public
        RIRAddr(String rirStr)
        {
            if(rirStr.length() > rirProt_.length())
                throw new org.omg.CORBA.BAD_PARAM(
                    "rir: protocol does not allow an address",
                    MinorBadParam.MinorOther,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        
        // 
        // Return rir protocol id string
        // 
        public String
        getProtocolName() 
        {
            return rirProt_;
        }

        // 
        // There is no such thing as an rir: IOR profile. This always
        // raises an exception.
        // 
        public void
        setProfile(org.omg.IOP.IOR ior, int ProfileNumber, final byte[] key)
        {
            throw new org.omg.CORBA.INTERNAL(
                "rir: internal evaluation failure",
                MinorBadParam.MinorOther,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    }

    // 
    // Encapsulate the information required to contact an
    // object for IIOP.
    // 
    static public class IIOPAddr extends URLAddr
    {
        private int major_;
        private int minor_;
        private String host_;
        private int port_;
        
        // 
        // OMG Specified default port for iiop: protocol
        // 
        public static final int DEFAULT_PORT = 2809; 
        
        // 
        // Construct a corbaloc iiop: protocol address 
        // 
        public
        IIOPAddr(String iiopStr)
        {
            int addrEndIdx = iiopStr.length();
            int addrIdx = iiopStr.charAt(0) == iiopProtColon_.charAt(0) ?
                iiopProtColon_.length() : iiopProt_.length();

            //
            // Do we have an iiop version 'X.Y@' portion? (default is 1.0)
            //
            int at = iiopStr.indexOf('@', addrIdx);
            
            if(at == -1)
            {
                major_ = 1;
                minor_ = 0;
            }
            else
            {
                int dot = iiopStr.indexOf('.', addrIdx);

                if(dot == -1)
                    throw new org.omg.CORBA.BAD_PARAM(
                            "URL: iiop version must be of the form `X.Y@'",
                            MinorBadParam.MinorOther,
                            org.omg.CORBA.CompletionStatus.COMPLETED_NO);

                try
                {
                    String m = iiopStr.substring(addrIdx, dot);
                    major_ = Integer.parseInt(m);
                    m = iiopStr.substring(dot + 1, at);
                    minor_ = Integer.parseInt(m);

                    // 
                    // Major Version Must start at 1
                    // Minor Version can start at 0
                    // 
                    if(major_ < 1 || major_ > 255 ||
                       minor_ < 0 || minor_ > 255)
                        throw new org.omg.CORBA.BAD_PARAM(
                                "URL: iiop version invalid",
                                MinorBadParam.MinorOther,
                                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                }
                catch(NumberFormatException e)
                {
                    throw new org.omg.CORBA.BAD_PARAM(
                        "URL: iiop version format error",
                        MinorBadParam.MinorOther,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                }

                //
                // The end of the address portion is now the character
                // after the '@'
                //
                addrIdx = at + 1;
            }

            // 
            // Empty hostname is illegal (as is port ':YYYY' by itself)
            // 
            if(addrIdx >= addrEndIdx || iiopStr.charAt(addrIdx) == ':')
                throw new org.omg.CORBA.BAD_PARAM(
                    "URL: iiop host must be specified",
                    MinorBadParam.MinorOther,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            
            //
            // Find optional ':' port number separator or end of string 
            //
            int colon = iiopStr.indexOf(':', addrIdx);
            host_ = iiopStr.substring(addrIdx,
                                      (colon == -1) ? addrEndIdx : colon);
            
            // 
            // Port specified ?
            // 

            if(colon == -1)
                port_ = DEFAULT_PORT;
            else
            {
                String portStr = iiopStr.substring(colon+1, addrEndIdx);
                try
                {
                    port_ = Integer.parseInt(portStr);
                    if(port_ < 0 || port_ > 65535)
                        throw new NumberFormatException();
                }
                catch(NumberFormatException e)
                {
                    throw new org.omg.CORBA.BAD_PARAM(
                            "URL: iiop bad port number " + portStr,
                            MinorBadParam.MinorOther,
                            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                }
            }
        }
        
        // 
        // Return the iiop protocol id
        // 
        public String
        getProtocolName()
        {
            return iiopProtColon_;
        }

        // 
        // Set an IOR profile using this IIOPAddr info and the specified key
        // 
        public void
        setProfile(org.omg.IOP.IOR ior, int prNo, final byte[] key)
        {
            org.omg.IIOP.ProfileBody body = new org.omg.IIOP.ProfileBody();
            body.iiop_version = new org.omg.IIOP.Version();
            body.iiop_version.major = 1;
            body.iiop_version.minor = 0;
            body.host = host_;
            if(port_ >= 0x8000)
                body.port = (short)(port_ - 0xffff - 1);
            else
                body.port = (short)port_;
            body.object_key = key;
	
            ior.profiles[prNo] = new org.omg.IOP.TaggedProfile();
            ior.profiles[prNo].tag = org.omg.IOP.TAG_INTERNET_IOP.value;
            com.ooc.CORBA.OutputStream out = new com.ooc.CORBA.OutputStream();
            out._OB_writeEndian();
            org.omg.IIOP.ProfileBodyHelper.write(out, body);
            byte[] pData = new byte[out._OB_count()];
            System.arraycopy(out._OB_buffer(), 0, pData, 0, pData.length);
            ior.profiles[prNo].profile_data = pData;
        }
    }
}
