// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CORBA;

final class GIOPServerStarterThreaded extends GIOPServerStarter
{
    private boolean destroy_ = false; // True if destroy() was called
    private ThreadPool threadPool_ = null; // The Thread Pool
    private ThreadGroup serverThreadGroup_; // The server thread group

    //
    // The starter thread
    //
    private final class StarterThread extends Thread
    {
        StarterThread(ThreadGroup group)
        {
            super(group, "ORBACUS:Server:StarterThread");
        }
        
	public void run()
	{
	    starterRun();
	}
    };
    Thread starterThread_;

    //
    // Constructor
    //
    GIOPServerStarterThreaded(BOA boa, com.ooc.OCI.Acceptor acceptor,
			      boolean enabled)
    {
	super(boa, acceptor, enabled);

	//
	// Cache the thread pool, if necessary
	//
	if(boa.conc_model().value() == BOA.ConcModel._ConcModelThreadPool)
	    threadPool_ = boa._OB_getThreadPool();
	serverThreadGroup_ = boa._OB_getServerThreadGroup();

	//
	// Create starter thread
	//
	starterThread_ = new StarterThread(serverThreadGroup_);

	//
	// Start the starter thread if not disabled
	//
        if(enabled)
            starterThread_.start();
    }

    //
    // Destructor
    //
    protected void
    finalize()
	throws Throwable
    {
	if(!destroy_)
	    throw new InternalError();

	super.finalize();
    }

    //
    // Destroy the server starter
    //
    void
    _destroy()
    {
	//
	// Don't destroy twice
	//
	if(destroy_)
	    return;

	//
	// Set the destroy flag
	//
	destroy_ = true;

	//
	// Unblock threads blocking in accept()
	//
	acceptor_.unblock_threads();

	//
	// Join with the starter thread
	//
	if(starterThread_.equals(Thread.currentThread()))
	    throw new InternalError();
	while(starterThread_.isAlive())
	{
	    try
	    {
		starterThread_.join();
	    }
	    catch(InterruptedException ex)
	    {
	    }
	}

	//
	// Close the acceptor
	//
	acceptor_.close();

	//
	// Destroy the thread pool, if present
	//
	if(threadPool_ != null)
	    threadPool_.destroy();
	
	//
	// Destroy all workers
	//
	destroyAllWorkers();
    }

    //
    // Run
    //
    public void
    starterRun()
    {
	while(!destroy_)
	{
	    try
	    {
		//
		// Get new transport, blocking
		//
		com.ooc.OCI.Transport transport = acceptor_.accept(true);
		if(transport == null)
		    throw new InternalError();
		
		//
		// Check if thread should be stopped and destroyed
		//
		if(destroy_)
		{
		    transport.close(true);
		    break;
		}

		//
		// Create new worker
		//
		if(transport != null)
		{
		    synchronized(this)
		    {
			workers_.addElement(
			    new GIOPServerWorkerThreaded(boa_, this,
							 transport,
							 serverThreadGroup_,
							 threadPool_));
		    }
		}
	    }
	    catch(org.omg.CORBA.NO_PERMISSION ex)
	    {
		//
		// Ignore NO_PERMISSION exceptions
		//
	    }
	    catch(org.omg.CORBA.SystemException ex)
	    {
		//
		// Ignore exception. This probably means that the server
		// exceeded the number available file descriptors.
		//
	    }
	}
    }
}
