// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CORBA;

//
// Helper class for dispatching a request in case of a "thread
// pool" concurrency model
//
final class ThreadPool
{
    private boolean destroy_; // True if destroy() was called
    
    private final static class Request
    {
	GIOPServerWorkerThreaded worker;
	InputStream in;
    };
    private java.util.Vector reqVec_ = new java.util.Vector();
    
    private final class Dispatcher extends Thread
    {
	Dispatcher(ThreadGroup group)
	{
	    super(group, "ORBACUS:ThreadPool:Dispatcher");
	}
            
	public void
	run()
	{
	    Request req = null;

	    while(true)
	    {
		req = get();
		if(req == null)
		    break;

		if(!req.worker.incNumberOfRequests())
		    continue;
		
		try
		{
		    req.worker.dispatchRequestSuper(req.in);
		}
		finally
		{
		    req.worker.decNumberOfRequests();
		}
	    }
	}
    }

    ThreadPool(ThreadGroup group, int n)
    {
	for(int i = 0 ; i < n ; i++)
	{
	    Thread t = new Dispatcher(group);
	    t.start();
	}
    }

    protected void
    finalize()
	throws Throwable
    {
	if(!destroy_)
	    throw new InternalError();
	
	super.finalize();
    }
    
    synchronized void
    destroy()
    {
	destroy_ = true;
	notifyAll();
    }

    synchronized void
    add(GIOPServerWorkerThreaded w, InputStream i)
    {
	Request req = new Request();
	req.worker = w;
	req.in = i;
	reqVec_.addElement(req);

	notify();
    }

    synchronized Request
    get()
    {
	while(!destroy_ && reqVec_.size() == 0)
	{
	    try
	    {
		wait();
	    }
	    catch(InterruptedException ex)
	    {
	    }
	}
	
	if(destroy_)
	    return null;
	
	Request req = (Request)reqVec_.firstElement();
	reqVec_.removeElementAt(0);
	return req;
    }
}
