// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

import org.omg.CORBA.*;
import java.io.*;
import java.util.*;

public class Client extends TestBase
{
    static public int
    run(ORB orb)
	throws TestException
    {
	//
	// Get TestIntf
	//
        String refFile = "TestIntf.ref";
	String impl;
	String implDSI;
	String implTie;

	try
	{
	    FileInputStream file = new FileInputStream(refFile);
	    BufferedReader in =
		new BufferedReader(new InputStreamReader(file));
	    impl = in.readLine();
	    implDSI = in.readLine();
	    implTie = in.readLine();
	    file.close();
 	}
	catch(IOException ex)
	{
	    System.err.println("Can't read from `" +
			       ex.getMessage() + "'");
	    return 1;
	}
 
	//
	// Run tests
	//
	System.out.print("Testing constants... ");
	System.out.flush();
	new TestConst();
	System.out.println("OK!");

	System.out.print("Testing TypeCode type... ");
	System.out.flush();
	new TestTypeCode();
	System.out.println("OK!");

	System.out.print("Testing any type... ");
	System.out.flush();
	new TestAny();
	System.out.println("OK!");

 	System.out.print("Testing DynAny... ");
 	System.out.flush();
 	new TestDynAny();
 	System.out.println("OK!");

	System.out.print("Testing some union types... ");
	System.out.flush();
	new TestUnion();
	System.out.println("OK!");

	System.out.print("Testing string_to_object()... ");
	System.out.flush();
 	org.omg.CORBA.Object obj = orb.string_to_object(impl);
 	TEST(obj != null);
	org.omg.CORBA.Object objDSI = orb.string_to_object(implDSI);
	TEST(objDSI != null);
	org.omg.CORBA.Object objTie = orb.string_to_object(implTie);
	TEST(objTie != null);
	System.out.println("OK!");

	System.out.print("Testing _narrow()... ");
	System.out.flush();
 	TestIntf ti = TestIntfHelper.narrow(obj);
 	TEST(ti != null);
	TestIntf tiDSI = TestIntfHelper.narrow(objDSI);
	TEST(tiDSI != null);
	TestIntf tiTie = TestIntfHelper.narrow(objTie);
        TEST(tiTie != null);
 	System.out.println("OK!");
	TestIntf_LongLong ti_ll = TestIntf_LongLongHelper.narrow(obj);
	TestIntf_LongLong tiDSI_ll = TestIntf_LongLongHelper.narrow(objDSI);
	TestIntf_LongLong tiTie_ll = TestIntf_LongLongHelper.narrow(objTie);
	if(ti_ll != null)
	{
	    TEST(tiDSI_ll != null);
	    TEST(tiTie_ll != null);
	}
	
 	System.out.print("Testing interface stub... ");
 	System.out.flush();
 	new TestStub(ti);
 	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestStub(tiDSI);
	System.out.println("OK!");

        System.out.print("Ditto, but with the tie implementation... ");
        System.out.flush();
        new TestStub(tiTie);
        System.out.println("OK!");

	if(ti_ll != null)
	{
	    System.out.print("Testing interface stub with long long type... ");
	    System.out.flush();
	    new TestStub_LongLong(ti_ll);
	    System.out.println("OK!");

	    System.out.print("Ditto, but with the DSI implementation... ");
	    System.out.flush();
	    new TestStub_LongLong(tiDSI_ll);
	    System.out.println("OK!");

	    System.out.print("Ditto, but with the tie implementation... ");
	    System.out.flush();
	    new TestStub_LongLong(tiTie_ll);
	    System.out.println("OK!");
	}
	
	System.out.print("Testing interface stub with contexts... ");
	System.out.flush();
	new TestStubContext(orb, ti);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestStubContext(orb, tiDSI);
	System.out.println("OK!");

        System.out.print("Ditto, but with the tie implementation... ");
        System.out.flush();
        new TestStubContext(orb, tiTie);
        System.out.println("OK!");

	System.out.print("Testing interface stub with exceptions... ");
	System.out.flush();
	new TestStubException(ti, false);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestStubException(tiDSI, true);
	System.out.println("OK!");

        System.out.print("Ditto, but with the tie implementation... ");
        System.out.flush();
        new TestStubException(tiTie, true);
        System.out.println("OK!");

	System.out.print("Testing interface stub with timeouts... ");
	System.out.flush();
	new TestStubTimeout(ti, orb);
	System.out.println("OK!");

	System.out.print("Testing DII... ");
	System.out.flush();
	new TestDII(ti);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestDII(tiDSI);
	System.out.println("OK!");

        System.out.print("Ditto, but with the tie implementation... ");
        System.out.flush();
        new TestDII(tiTie);
        System.out.println("OK!");

	System.out.print("Testing DII with contexts... ");
	System.out.flush();
	new TestDIIContext(ti, orb);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestDIIContext(tiDSI, orb);
	System.out.println("OK!");

        System.out.print("Ditto, but with the tie implementation... ");
        System.out.flush();
        new TestDIIContext(tiTie, orb);
        System.out.println("OK!");

	System.out.print("Testing DII with exceptions... ");
	System.out.flush();
	new TestDIIException(ti);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestDIIException(tiDSI);
	System.out.println("OK!");

        System.out.print("Ditto, but with the tie implementation... ");
        System.out.flush();
        new TestDIIException(tiTie);
        System.out.println("OK!");

	try
	{
	    ti.deactivate();
	}
	catch(org.omg.CORBA.TRANSIENT ex)
	{
	    // This call will deactivate the server and thus throw an
	    // exception
	}
	    
	return 0;
    }

    static public void
    main(String args[])
    {
	int status = 0;

	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

        try
        {
            //
            // Create ORB
            //
            ORB orb = ORB.init(args, props);

            //
	    // Run tests
	    //
	    status = run(orb);
	}
	catch(SystemException ex)
        {
	    System.err.println(ex.getMessage());
	    ex.printStackTrace();
	    status = 1;
	}

	System.exit(status);
    }
}
