// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

import org.omg.CORBA.*;
import com.ooc.CosTime.TimeHelper;

class TestTIO
{
    public static void
    test(org.omg.CosTime.TimeService ts)
    {
	org.omg.CosTime.OverlapType otContainer =
	    org.omg.CosTime.OverlapType.OTContainer;
	org.omg.CosTime.OverlapType otContained =
	    org.omg.CosTime.OverlapType.OTContained;
	org.omg.CosTime.OverlapType otOverlap =
	    org.omg.CosTime.OverlapType.OTOverlap;
	org.omg.CosTime.OverlapType otNoOverlap =
	    org.omg.CosTime.OverlapType.OTNoOverlap;

	org.omg.CosTime.TIO tio = null;
	org.omg.CosTime.UTO uto = null;
	org.omg.TimeBase.IntervalT interval = null;

	//
	// TEST: CosTime::TIO creation
	//
	try
	{
	    tio = ts.new_interval(200, 100);
	    throw new RuntimeException();
	}
	catch(BAD_PARAM ex)
	{
	}
	try
	{
	    tio = ts.new_interval(0x8000000000000064L, 0x8000000000000000L);
	    throw new RuntimeException();
	}
	catch(BAD_PARAM ex)
	{
	}
	try
	{
	    tio = ts.new_interval(0x8000000000000000L, 200);
	    throw new RuntimeException();
	}
	catch(BAD_PARAM ex)
	{
	}
	try
	{
	    tio = ts.new_interval(TimeHelper.MaxTimeT, 0);
	    throw new RuntimeException();
	}
	catch(BAD_PARAM ex)
	{
	}
	tio = ts.new_interval(100, 200);
	tio = ts.new_interval(0x8000000000000000L, 0x8000000000000064L);
	tio = ts.new_interval(200, 0x8000000000000064L);
	tio = ts.new_interval(0, TimeHelper.MaxTimeT);

	//
	// TEST: attribute TimeBase::IntervalT time_interval()
	//

	tio = ts.new_interval(100, 200);
	interval = tio.time_interval();
	if(interval.lower_bound != 100 || interval.upper_bound != 200)
	    throw new RuntimeException();

	tio = ts.new_interval(0x8000000000000000L, 0x8000000000000064L);
	interval = tio.time_interval();
	if(interval.lower_bound != 0x8000000000000000L ||
	   interval.upper_bound != 0x8000000000000064L)
	    throw new RuntimeException();

	tio = ts.new_interval(0, TimeHelper.MaxTimeT);
	interval = tio.time_interval();
	if(interval.lower_bound != 0 ||
	   interval.upper_bound != TimeHelper.MaxTimeT)
	    throw new RuntimeException();

	//
	// TEST: UTO time()
	//

	tio = ts.new_interval(100, 200);
	uto = tio.time();
	if(uto.time() != 150 || uto.inaccuracy() != 50)
	    throw new RuntimeException();

	tio = ts.new_interval(0x8000000000000000L, 0x8000000000000064L);
	uto = tio.time();
	if(uto.time() != 0x8000000000000032L || uto.inaccuracy() != 50)
	    throw new RuntimeException();

	tio = ts.new_interval(0, TimeHelper.MaxTimeT);
	try
	{
	    uto = tio.time();
	    throw new RuntimeException();
	}
	catch(DATA_CONVERSION ex)
	{
	}

	//
	// TEST: OverlapType spans(in UTO time, out TIO overlap)
	//

	org.omg.CosTime.TIOHolder overlap = new org.omg.CosTime.TIOHolder();
	uto = ts.new_universal_time(0x8000000000000000L, 50, uto.tdf());

	//
	// Interval the same as UTO
	//
	tio = ts.new_interval(0x7fffffffffffffceL, 0x8000000000000032L);
	if(tio.spans(uto, overlap) != otContained)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x7fffffffffffffceL ||
	   interval.upper_bound != 0x8000000000000032L)
	    throw new RuntimeException();

	//
	// Interval greater than UTO with no overlap
	//
	tio = ts.new_interval(0x8000000000000064L, 0x8000000000000096L);
	if(tio.spans(uto, overlap) != otNoOverlap)
	    throw new RuntimeException();

	//
	// Interval less than UTO with no overlap
	//
	tio = ts.new_interval(800L, 900L);
	if(tio.spans(uto, overlap) != otNoOverlap)
	    throw new RuntimeException();

	//
	// Interval overlaps at upper end of UTO
	//
	tio = ts.new_interval(0x8000000000000000L, 0x8000000000000096L);
	if(tio.spans(uto, overlap) != otOverlap)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x8000000000000000L ||
	   interval.upper_bound != 0x8000000000000032L)
	    throw new RuntimeException();

	//
	// Interval overlaps at lower end of UTO
	//
	tio = ts.new_interval(800L, 0x8000000000000000L);
	if(tio.spans(uto, overlap) != otOverlap)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x7fffffffffffffceL ||
	   interval.upper_bound != 0x8000000000000000L)
	    throw new RuntimeException();

	//
	// Interval completely contains UTO
	//
	tio = ts.new_interval(0x7fffffffffffffb5L, 0x800000000000004bL);
	if(tio.spans(uto, overlap) != otContainer)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x7fffffffffffffceL ||
	   interval.upper_bound != 0x8000000000000032L)
	    throw new RuntimeException();

	//
	// Interval completely contained by UTO
	//
	tio = ts.new_interval(0x7fffffffffffffe7L, 0x8000000000000019L);
	if(tio.spans(uto, overlap) != otContained)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x7fffffffffffffe7L ||
	   interval.upper_bound != 0x8000000000000019L)
	    throw new RuntimeException();

	//
	// TEST: OverlapType overlaps(in TIO interval, out TIO overlap);
	//

	org.omg.CosTime.TIO tio2 = ts.new_interval(0x7fffffffffffffceL,
						   0x8000000000000032L);

	//
	// Intervals the same
	//
	if(tio2.overlaps(tio2, overlap) != otContained)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x7fffffffffffffceL ||
	   interval.upper_bound != 0x8000000000000032L)
	    throw new RuntimeException();

	//
	// Interval greater than compared Interval with no overlap
	//
	tio = ts.new_interval(0x8000000000000064L, 0x8000000000000096L);
	if(tio.overlaps(tio2, overlap) != otNoOverlap)
	    throw new RuntimeException();

	//
	// Interval less than compared Interval with no overlap
	//
	tio = ts.new_interval(800, 900);
	if(tio.overlaps(tio2, overlap) != otNoOverlap)
	    throw new RuntimeException();

	//
	// Interval overlaps at upper end of compared Interval
	//
	tio = ts.new_interval(0x8000000000000000L, 0x8000000000000096L);
	if(tio.overlaps(tio2, overlap) != otOverlap)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x8000000000000000L ||
	   interval.upper_bound != 0x8000000000000032L)
	    throw new RuntimeException();

	//
	// Interval overlaps at lower end of compared Interval
	//
	tio = ts.new_interval(925, 0x8000000000000000L);
	if(tio.overlaps(tio2, overlap) != otOverlap)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x7fffffffffffffceL ||
	   interval.upper_bound != 0x8000000000000000L)
	    throw new RuntimeException();

	//
	// Interval completely contains compared Interval
	//
	tio = ts.new_interval(0x7fffffffffffffb5L, 0x800000000000004bL);
	if(tio.overlaps(tio2, overlap) != otContainer)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x7fffffffffffffceL ||
	   interval.upper_bound != 0x8000000000000032L)
	    throw new RuntimeException();

	//
	// Interval completely contained by compared Interval
	//
	tio = ts.new_interval(0x7fffffffffffffe7L, 0x8000000000000019L);
	if(tio.overlaps(tio2, overlap) != otContained)
	    throw new RuntimeException();
	interval = overlap.value.time_interval();
	if(interval.lower_bound != 0x7fffffffffffffe7L ||
	   interval.upper_bound != 0x8000000000000019L)
	    throw new RuntimeException();
    }
}
