// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

import org.omg.CORBA.*;
import com.ooc.CosTime.TimeHelper;

class TestTimeHelper
{
    public static void
    test(org.omg.CosTime.TimeService ts)
	throws org.omg.CosTime.TimeUnavailable
    {
	org.omg.CosTime.UTO uto = ts.universal_time();

	//
	// TEST: toString(const TimeBase_UtcT&);
	//       toTimeString(const TimeBase_UtcT&);
	//       toString(TimeBase_TimeT);
	//
     
	//
	// testTime = 2 January 1970 00:00:00 (local time)
	//
	long testTime = 141428L * 86400L; 
	testTime -= uto.tdf() * 60;
	testTime *= 10000000L;

	uto = ts.new_universal_time(testTime, 0, uto.tdf());

	String str = TimeHelper.toString(uto.utc_time());
	if(!str.equals("01/02/70 00:00:00:000"))
	    throw new RuntimeException();

	str = TimeHelper.toTimeString(uto.utc_time());
	if(!str.equals("00:00:00:000"))
	    throw new RuntimeException();

	testTime = 12345670000L;
	uto = ts.new_universal_time(testTime, 0, uto.tdf());
	str = TimeHelper.toString(uto.time());
	if(!str.equals("1234:567"))
	    throw new RuntimeException();

	testTime = 141427L * 86400L * 10000000L - 1L;
	uto = ts.new_universal_time(testTime, 0, uto.tdf());
	str = TimeHelper.toString(uto.utc_time());
	if(!str.equals("Time less than 1 January 1970 00:00:00"))
	    throw new RuntimeException();

	uto = ts.universal_time();

	//
	// TEST: TimeBase_UtcT utcNow(TimeBase_InaccuracyT = 0);
	//
	// This is already taken care of by the TestUTO test
	// for TimeBase::TimeT time()
	//

	//
	// Test: TimeBase_UtcT utcMin();
	//       TimeBase_UtcT utcMax();
	//
	// No sensible test for this.
	//

	//
	// TEST: timeval toJavaMillis(const TimeBase_UtcT&);
	//       timeval toJavaMillis(TimeBase_TimeT);
	//
	long tv1 = TimeHelper.toJavaMillis(uto.utc_time());
	long tv2 = TimeHelper.toJavaMillis(uto.time());
	if(tv1 != tv2)
	    throw new RuntimeException();

	//
	// TEST: comparison operators
	//

	//
	// UTOs the same
	//
	if(TimeHelper.lessThan(uto.utc_time(), uto.utc_time()))
	    throw new RuntimeException();
	if(!TimeHelper.lessThanEqual(uto.utc_time(), uto.utc_time()))
	    throw new RuntimeException();
	if(TimeHelper.greaterThan(uto.utc_time(), uto.utc_time()))
	    throw new RuntimeException();
	if(!TimeHelper.greaterThanEqual(uto.utc_time(), uto.utc_time()))
	    throw new RuntimeException();
	if(!TimeHelper.equal(uto.utc_time(), uto.utc_time()))
	    throw new RuntimeException();
	if(TimeHelper.notEqual(uto.utc_time(), uto.utc_time()))
	    throw new RuntimeException();

	//
	// UTO 2 greater than UTO 1
	//
	org.omg.CosTime.UTO uto2 =
	    ts.new_universal_time(uto.time() + 10, 0, uto.tdf());

	if(!TimeHelper.lessThan(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(!TimeHelper.lessThanEqual(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(TimeHelper.greaterThan(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(TimeHelper.greaterThanEqual(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(TimeHelper.equal(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(!TimeHelper.notEqual(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();

	//
	// UTO 2 less than UTO 1
	//
	uto2 = ts.new_universal_time(uto.time() - 10, 0, uto.tdf());

	if(TimeHelper.lessThan(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(TimeHelper.lessThanEqual(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(!TimeHelper.greaterThan(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(!TimeHelper.greaterThanEqual(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(TimeHelper.equal(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
	if(!TimeHelper.notEqual(uto.utc_time(), uto2.utc_time()))
	    throw new RuntimeException();
    }
}
