// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package loan.client;

public class Query
{
    public static void
    main(String[] args)
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, props);

        //
        // Filter options
        //
        args = ((com.ooc.CORBA.ORB)orb).filter_options(args);

        //
        // Use argc to position us past the last option
        //
        int argc;

        //
        // Check options
        //
        for(argc = 0 ; argc < args.length ; argc++)
            if(args[argc].equals("-h"))
                usage();

        if(args.length != 2 && args.length != 3)
            usage();

        String constraint = args[0], preference = args[1], policyFile = null;
        if(args.length == 3)
            policyFile = args[2];

        //
        // Connect to Trading Service
        //
        org.omg.CosTrading.Lookup lookup = loan.util.DemoUtil.connect(orb);

        if(lookup == null)
            usage();

        try
        {
            String type = "Mortgage";

            org.omg.CosTrading.Policy[] policies;
            if(policyFile != null)
                policies = getPolicies(orb, policyFile);
            else
                policies = new org.omg.CosTrading.Policy[0];

            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);

            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            //
            // Perform the query
            //
            System.out.println("Querying...");
            lookup.query(type, constraint, preference, policies, desiredProps,
                20, offers, iter, limits);

            if(offers.value.length == 0 && iter.value == null)
            {
                System.out.println("No offers found.");
                System.exit(0);
            }

            //
            // Display the offers
            //
            showOffers(offers.value);

            //
            // If we received an iterator, then add all of its
            // offers to the vector
            //
            if(iter.value != null)
            {
                org.omg.CosTrading.OfferSeqHolder seq =
                    new org.omg.CosTrading.OfferSeqHolder();
                boolean more;

                do
                {
                    more = iter.value.next_n(20, seq);
                    showOffers(seq.value);
                }
                while (more);

                iter.value.destroy();
            }
        }
        catch(org.omg.CosTrading.IllegalServiceType e)
        {
            System.err.println("Illegal service type '" + e.type + "'");
        }
        catch(org.omg.CosTrading.UnknownServiceType e)
        {
            System.err.println("Unknown service type '" + e.type + "'");
        }
        catch(org.omg.CosTrading.IllegalConstraint e)
        {
            System.err.println("Illegal constraint");
        }
        catch(org.omg.CosTrading.LookupPackage.IllegalPreference e)
        {
            System.err.println("Illegal preference");
        }
        catch(org.omg.CosTrading.LookupPackage.IllegalPolicyName e)
        {
            System.err.println("Illegal policy '" + e.name + "'");
        }
        catch(org.omg.CosTrading.LookupPackage.PolicyTypeMismatch e)
        {
            System.err.println("Policy type mismatch for '" +
                               e.the_policy.name + "'");
        }
        catch(org.omg.CosTrading.LookupPackage.InvalidPolicyValue e)
        {
            System.err.println("Invalid policy value for '" +
                               e.the_policy.name + "'");
        }
        catch(org.omg.CosTrading.IllegalPropertyName e)
        {
            System.err.println("Illegal property name '" + e.name + "'");
        }
        catch(org.omg.CosTrading.DuplicatePropertyName e)
        {
            System.err.println("Duplicate property name '" + e.name + "'");
        }
        catch(org.omg.CosTrading.DuplicatePolicyName e)
        {
            System.err.println("Duplicate policy name '" + e.name + "'");
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("System error occurred");
            e.printStackTrace();
        }

        System.exit(0);
    }

    static void
    showOffers(org.omg.CosTrading.Offer[] offers)
    {
        for(int i = 0 ; i < offers.length ; i++)
        {
            try
            {
                System.out.println();
                System.out.println("Offer:");

                loan.LenderDemo.Lender lender =
                    loan.LenderDemo.LenderHelper.narrow(offers[i].reference);

                if(lender == null)
                {
                    System.out.println("  Object is not a Lender - skipping");
                    continue;
                }

                //
                // Extract the properties of this offer
                //

                String productId = null;
                float rate = 0, pctDown = 0;
                short years = 0;

                for(int n = 0 ; n < offers[i].properties.length ; n++)
                {
                    org.omg.CosTrading.Property prop =
                        offers[i].properties[n];

                    if(prop.name.equals("rate"))
                        rate = prop.value.extract_float();
                    else if(prop.name.equals("pct_down"))
                        pctDown = prop.value.extract_float();
                    else if(prop.name.equals("years"))
                        years = prop.value.extract_short();
                    else if(prop.name.equals("product_id"))
                        productId = prop.value.extract_string();
                }

                StringBuffer buff = new StringBuffer();

                //
                // Determine the name of the lender, and whether
                // we are approved for the loan
                //
                String company = lender.company_name();
                boolean approved = lender.request_loan(productId);

                buff.append("  " + company + " has ");
                if(! approved)
                    buff.append("NOT ");

                buff.append("approved you for " + productId + ", ");
                buff.append("a " + years + " year loan\n");
                buff.append("  at " + rate + "% with " + pctDown + "% down.");

                System.out.println(buff.toString());
            }
            catch(loan.LenderDemo.UnknownProduct ex)
            {
                System.out.println("  Unknown product '" + ex.product_id +
                                   "' - skipping");
            }
            catch(org.omg.CORBA.SystemException ex)
            {
                System.out.println("  Unable to contact lender - skipping");
            }
        }
    }

    static org.omg.CosTrading.Policy[]
    getPolicies(org.omg.CORBA.ORB orb, String file)
    {
        org.omg.CosTrading.Policy[] result = null;

        java.io.File f = new java.io.File(file);

        if(f.exists())
        {
            System.out.println("Reading policies from " + file + "...");

            try
            {
                //
                // Add all policies to vector
                //
                java.util.Vector vec = new java.util.Vector();

                java.io.FileReader fr = new java.io.FileReader(f);
                java.io.BufferedReader reader = new java.io.BufferedReader(fr);

                String line;
                while((line = reader.readLine()) != null)
                {
                    String trimline = line.trim();
                    if(trimline.length() == 0 || trimline.startsWith("#"))
                        continue;

                    int pos = trimline.indexOf('=');
                    if(pos < 0 || trimline.length() == pos + 1)
                    {
                        System.err.println("Error:  Invalid line '" + line +
                                           "' in file " + f);
                        System.exit(1);
                    }

                    String name = trimline.substring(0, pos);
                    String value = trimline.substring(pos + 1);

                    org.omg.CosTrading.Policy policy = null;

                    if(name.equals("exact_type_match"))
                        policy = getBooleanPolicy(orb, name, value);
                    else if(name.equals("use_modifiable_properties"))
                        policy = getBooleanPolicy(orb, name, value);
                    else if(name.equals("use_dynamic_properties"))
                        policy = getBooleanPolicy(orb, name, value);
                    else if(name.equals("use_proxy_offers"))
                        policy = getBooleanPolicy(orb, name, value);
                    else if(name.equals("search_card"))
                        policy = getULongPolicy(orb, name, value);
                    else if(name.equals("match_card"))
                        policy = getULongPolicy(orb, name, value);
                    else if(name.equals("return_card"))
                        policy = getULongPolicy(orb, name, value);
                    else if(name.equals("link_follow_rule"))
                        policy = getFollowOptionPolicy(orb, name, value);
                    else if(name.equals("hop_count"))
                        policy = getULongPolicy(orb, name, value);
                    else
                    {
                        System.err.println("Error:  Unknown policy '" + name +
                                           "' in file " + f);
                        System.exit(1);
                    }

                    vec.addElement(policy);
                }

                fr.close();

                result = new org.omg.CosTrading.Policy[vec.size()];
                vec.copyInto(result);
            }
            catch(java.io.FileNotFoundException ex)
            {
                System.err.println("Error:  Unable to open file " + f);
                System.exit(1);
            }
            catch(java.io.IOException ex)
            {
                System.err.println("Error:  Unable to read file " + f);
                System.exit(1);
            }
        }
        else
            result = new org.omg.CosTrading.Policy[0];

        return result;
    }

    static org.omg.CosTrading.Policy
    getBooleanPolicy(org.omg.CORBA.ORB orb, String name, String value)
    {
        org.omg.CosTrading.Policy result = null;

        if(value.equalsIgnoreCase("true"))
        {
            org.omg.CORBA.Any any = orb.create_any();
            any.insert_boolean(true);
            result = new org.omg.CosTrading.Policy(name, any);
        }
        else if(value.equalsIgnoreCase("false"))
        {
            org.omg.CORBA.Any any = orb.create_any();
            any.insert_boolean(false);
            result = new org.omg.CosTrading.Policy(name, any);
        }
        else
        {
            System.err.println("Error:  Invalid boolean value for policy '" +
                               name + "'");
            System.exit(1);
        }

        return result;
    }

    static org.omg.CosTrading.Policy
    getULongPolicy(org.omg.CORBA.ORB orb, String name, String value)
    {
        org.omg.CosTrading.Policy result = null;

        try
        {
            int n = Integer.valueOf(value).intValue();
            org.omg.CORBA.Any any = orb.create_any();
            any.insert_ulong(n);
            result = new org.omg.CosTrading.Policy(name, any);
        }
        catch(NumberFormatException ex)
        {
            System.err.println("Error:  Invalid numeric value for policy '" +
                               name + "'");
            System.exit(1);
        }

        return result;
    }

    static org.omg.CosTrading.Policy
    getFollowOptionPolicy(org.omg.CORBA.ORB orb, String name, String value)
    {
        org.omg.CosTrading.Policy result = null;

        if(value.equals("local_only"))
        {
            org.omg.CORBA.Any any = orb.create_any();
            org.omg.CosTrading.FollowOptionHelper.insert(any,
                org.omg.CosTrading.FollowOption.local_only);
            result = new org.omg.CosTrading.Policy(name, any);
        }
        else if(value.equals("if_no_local"))
        {
            org.omg.CORBA.Any any = orb.create_any();
            org.omg.CosTrading.FollowOptionHelper.insert(any,
                org.omg.CosTrading.FollowOption.if_no_local);
            result = new org.omg.CosTrading.Policy(name, any);
        }
        else if(value.equals("always"))
        {
            org.omg.CORBA.Any any = orb.create_any();
            org.omg.CosTrading.FollowOptionHelper.insert(any,
                org.omg.CosTrading.FollowOption.always);
            result = new org.omg.CosTrading.Policy(name, any);
        }
        else
        {
            System.err.println("Error:  Invalid FollowOption value for " +
                               "policy '" + name + "'");
            System.exit(1);
        }

        return result;
    }

    static void
    usage()
    {
        System.err.println();
        System.err.println("Usage:  java loan.client.Query [-h] " +
                           "constraint preference [policy-file]");
        System.err.println();
        System.err.println("Options:");
        System.err.println();
        System.err.println("  -h   Show this help message");
        System.exit(1);
    }
}
