// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package loan.util;

// Things are just too ugly without this
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public abstract class DemoUtil
{
    private static final String serviceType_ = "Mortgage";

    public static String
    getServiceType()
    {
        return serviceType_;
    }

    //
    // Connect to Trading Service by resolving the 'TradingService'
    // initial reference.
    //
    public static org.omg.CosTrading.Lookup
    connect(org.omg.CORBA.ORB orb)
    {
        org.omg.CosTrading.Lookup result = null;

        //
        // Resolve the 'TradingService' initial reference
        //

        try
        {
            org.omg.CORBA.Object obj =
                orb.resolve_initial_references("TradingService");
            result = org.omg.CosTrading.LookupHelper.narrow(obj);
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            System.err.println("Error:  Unable to resolve initial " +
                               "reference 'TradingService'");
        }

        return result;
    }

    public static void
    installServiceType(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        boolean typeExists = false;
        org.omg.CosTradingRepos.ServiceTypeRepository repos = null;

        //
        // Verify that the type "Mortgage" already exists in the
        // service type repository; if not, we need to install it
        //
        try
        {
            org.omg.CORBA.Object obj = trader.type_repos();
            repos = org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.
                    narrow(obj);
            repos.describe_type(serviceType_);
            typeExists = true;
        }
        catch(org.omg.CosTrading.IllegalServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CosTrading.UnknownServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("System error occurred");
            e.printStackTrace();
            System.exit(1);
        }

        if(! typeExists)
        {
            System.out.println("Installing service type '" +
                               serviceType_ + "'...");

            try
            {
                String[] superTypes = new String[0];

                //
                // Define the properties for this service type
                //

                PropStruct[] props = new PropStruct[4];

                props[0] = new PropStruct();
                props[0].name = "product_id";
                props[0].value_type = orb.create_string_tc(0);
                props[0].mode = PropertyMode.PROP_MANDATORY;

                props[1] = new PropStruct();
                props[1].name = "rate";
                props[1].value_type =
                    orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
                props[1].mode = PropertyMode.PROP_MANDATORY;

                props[2] = new PropStruct();
                props[2].name = "pct_down";
                props[2].value_type =
                    orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
                props[2].mode = PropertyMode.PROP_MANDATORY;

                props[3] = new PropStruct();
                props[3].name = "years";
                props[3].value_type =
                    orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_short);
                props[3].mode = PropertyMode.PROP_MANDATORY;

                repos.add_type(serviceType_,
                               loan.LenderDemo.LenderHelper.id(),
                               props, superTypes);
            }
            catch(org.omg.CosTrading.IllegalServiceType ex)
            {
                System.err.println("Illegal service type: " + ex.type);
                System.exit(1);
            }
            catch(org.omg.CosTrading.UnknownServiceType ex)
            {
                System.err.println("Unknown service type: " + ex.type);
                System.exit(1);
            }
            catch(org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.
                ServiceTypeExists ex)
            {
                System.err.println("Service type '" + ex.name +
                                   "' already exists");
                System.exit(1);
            }
            catch(org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.
                InterfaceTypeMismatch ex)
            {
                System.err.println("Interface of service type '" +
                                   ex.derived_service +
                                   "' does not match super type '" +
                                   ex.base_service + "'");
                System.exit(1);
            }
            catch(org.omg.CosTrading.IllegalPropertyName ex)
            {
                System.err.println("Illegal property name '" + ex.name + "'");
                System.exit(1);
            }
            catch(org.omg.CosTrading.DuplicatePropertyName ex)
            {
                System.err.println("Duplicate property name '" +
                                   ex.name + "'");
                System.exit(1);
            }
            catch(org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.
                ValueTypeRedefinition ex)
            {
                System.err.println("Property type for '" +
                                   ex.definition_2.name +
                                   "' is not compatible with the definition " +
                                   "in service type '" + ex.type_1 + "'");
            }
            catch(org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.
                DuplicateServiceTypeName ex)
            {
                System.err.println("Duplicate super type: " + ex.name);
                System.exit(1);
            }
            catch(org.omg.CORBA.SystemException e)
            {
                System.err.println("System error occurred");
                e.printStackTrace();
                System.exit(1);
            }
        }
    }

    public static void
    cleanOffers(org.omg.CosTrading.Lookup trader, String fileName)
    {
        //
        // If the offer ID file exists, then read the offer IDs from the
        // file and withdraw them
        //

        java.io.File f = new java.io.File(fileName);

        if(f.exists())
        {
            String[] offers = readOffers(f);

            System.out.println("Removing old offers...");

            try
            {
                org.omg.CosTrading.Register reg = trader.register_if();

                if(reg == null)
                {
                    System.err.println("Error:  Trader does not support " +
                                       "the Register interface");
                    System.exit(1);
                }

                for(int i = 0 ; i < offers.length ; i++)
                {
                    try
                    {
                        reg.withdraw(offers[i]);
                    }
                    catch(org.omg.CORBA.UserException ex)
                    {
                        // ignore
                    }
                }
            }
            catch(org.omg.CORBA.SystemException e)
            {
                System.err.println("System error occurred");
                e.printStackTrace();
                System.exit(1);
            }
        }
    }

    private static String[]
    readOffers(java.io.File f)
    {
        String[] result = null;

        java.util.Vector vec = new java.util.Vector();

        try
        {
            java.io.FileReader fr = new java.io.FileReader(f);
            java.io.BufferedReader reader = new java.io.BufferedReader(fr);

            //
            // Read all the offer IDs in the file, one ID per line
            //

            String id;
            while((id = reader.readLine()) != null)
                vec.addElement(id);

            fr.close();

            result = new String[vec.size()];
            vec.copyInto(result);

            //
            // Get rid of the file
            //
            f.delete();
        }
        catch(java.io.FileNotFoundException ex)
        {
            System.err.println("Error:  Unable to open file " + f);
            System.exit(1);
        }
        catch(java.io.IOException ex)
        {
            System.err.println("Error:  Unable to read file " + f);
            System.exit(1);
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("System error occurred");
            e.printStackTrace();
            System.exit(1);
        }

        return result;
    }
}
