package printer;

import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class Client extends JFrame
                    implements ActionListener, ListSelectionListener
{
    private org.omg.CORBA.ORB orb_;
    private org.omg.CosTrading.Lookup trader_;
    private org.omg.CosTrading.Offer[] offers_ = null;
    private JList printers_;
    private DefaultListModel listModel_;
    private JTextField queueInfo_;
    private JTextField costInfo_;
    private JTextField speedInfo_;
    private JTextField colorInfo_;
    private JTextField locationInfo_;
    private JTextField constraint_;
    private JTextField preference_;
    private JButton query_;
    private JButton print_;
    private JButton close_;
    private StatusBar status_;

    public Client(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        super("Printer Client");

        orb_ = orb;
        trader_ = trader;

        JScrollPane scroll;

        Container c = getContentPane();

        //
        // Printer list
        //
        JPanel listPanel = new JPanel();
        listPanel.setLayout(new BorderLayout());
        listPanel.add(new JLabel("Printers", JLabel.LEFT), "North");
        listModel_ = new DefaultListModel();
        printers_ = new JList(listModel_);
        scroll = new JScrollPane(printers_);
        scroll.setPreferredSize(new Dimension(200, 150));
        listPanel.add(scroll, "Center");
        listPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 2));

        //
        // Info tab
        //
        JPanel infoPanel = new JPanel();
        infoPanel.setLayout(new GridBagLayout());
        infoPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        queueInfo_ = new JTextField(20);
        queueInfo_.setEditable(false);
        queueInfo_.setBorder(null);
        Constrain.constrain(infoPanel, new JLabel("Queue:", JLabel.RIGHT),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 0, 0, 0, 10);
        Constrain.constrain(infoPanel, queueInfo_, 1, 0, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 0, 0, 0);

        costInfo_ = new JTextField(20);
        costInfo_.setEditable(false);
        costInfo_.setBorder(null);
        Constrain.constrain(infoPanel, new JLabel("Cost:", JLabel.RIGHT),
            0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 10, 0, 0, 10);
        Constrain.constrain(infoPanel, costInfo_, 1, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 0, 0, 0);

        speedInfo_ = new JTextField(20);
        speedInfo_.setEditable(false);
        speedInfo_.setBorder(null);
        Constrain.constrain(infoPanel, new JLabel("Speed:", JLabel.RIGHT),
            0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 10, 0, 0, 10);
        Constrain.constrain(infoPanel, speedInfo_, 1, 2, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 0, 0, 0);

        colorInfo_ = new JTextField(20);
        colorInfo_.setEditable(false);
        colorInfo_.setBorder(null);
        Constrain.constrain(infoPanel, new JLabel("Color:", JLabel.RIGHT),
            0, 3, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 10, 0, 0, 10);
        Constrain.constrain(infoPanel, colorInfo_, 1, 3, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 0, 0, 0);

        locationInfo_ = new JTextField(20);
        locationInfo_.setEditable(false);
        locationInfo_.setBorder(null);
        Constrain.constrain(infoPanel, new JLabel("Location:", JLabel.RIGHT),
            0, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 10, 0, 10, 10);
        Constrain.constrain(infoPanel, locationInfo_, 1, 4, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 0, 10, 0);

        //
        // Query tab
        //
        JPanel queryPanel = new JPanel();
        queryPanel.setLayout(new GridBagLayout());
        queryPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        Constrain.constrain(queryPanel,
            new JLabel("Constraint:", JLabel.LEFT), 0, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 0, 0, 0, 0);

        constraint_ = new JTextField("TRUE", 20);
        Constrain.constrain(queryPanel, constraint_, 0, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 0, 0, 0);

        Constrain.constrain(queryPanel,
            new JLabel("Preference:", JLabel.LEFT), 0, 2, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 0, 0, 0);

        preference_ = new JTextField("first", 20);
        Constrain.constrain(queryPanel, preference_, 0, 3, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 0, 0, 0);

        query_ = new JButton("Query");
        query_.setActionCommand("query");
        query_.addActionListener(this);
        Constrain.constrain(queryPanel, query_, 0, 4, 2, 1,
            GridBagConstraints.NONE, GridBagConstraints.CENTER,
            0.0, 0.0, 20, 0, 0, 0);

        //
        // Tabbed pane
        //
        JTabbedPane tab = new JTabbedPane();
        tab.add("Info", infoPanel);
        tab.add("Query", queryPanel);
        tab.setBorder(BorderFactory.createEmptyBorder(5, 2, 5, 5));

        //
        // Create top split pane
        //
        listPanel.setMinimumSize(new Dimension(100, 100));
        tab.setMinimumSize(new Dimension(200, 100));
        JSplitPane topSplit =
            new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, listPanel, tab);
        topSplit.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));

        //
        // Action buttons
        //
        JPanel actionPanel = new JPanel();
        actionPanel.setLayout(new GridBagLayout());
        print_ = new JButton("Print");
        print_.setActionCommand("print");
        print_.addActionListener(this);
        Constrain.constrain(actionPanel, print_, 0, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 5, 5, 0, 5);

        close_ = new JButton("Close");
        close_.setActionCommand("close");
        close_.setActionCommand("close");
        close_.addActionListener(this);
        Constrain.constrain(actionPanel, close_, 1, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHEAST,
            0.0, 0.0, 5, 5, 0, 5);

        status_ = new StatusBar(false);
        status_.setDefaultText("Ready");
        //status_.setFont(new Font("Courier", Font.PLAIN, 12));
        Constrain.constrain(actionPanel, status_, 0, 1, 2, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 5, 5, 5, 5);

        c.add(topSplit, "Center");
        c.add(actionPanel, "South");

        this.addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent e)
                {
                    System.exit(0);
                }
            }
        );

        pack();

        query();

        printers_.addListSelectionListener(this);
    }

    protected void
    query()
    {
        try
        {
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);

            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            String constraint = constraint_.getText().trim();
            String preference = preference_.getText().trim();

            printLog("Executing query...");

            trader_.query("Printer", constraint, preference, policies,
                          desiredProps, 100, offers, iter, limits);

            offers_ = offers.value;
            printLog("Found " + offers_.length + " " +
                     (offers_.length == 1 ? "printer." : "printers."));

            listModel_.clear();
            for(int i = 0 ; i < offers_.length ; i++)
            {
                org.omg.CosTrading.Property p =
                    findProperty(offers_[i].properties, "queue");
                String name = p.value.extract_string();
                listModel_.addElement(name);
            }

            if(iter.value != null)
                iter.value.destroy();
        }
        catch(org.omg.CosTrading.IllegalConstraint ex)
        {
            printLog("Illegal constraint: " + ex.constr);
        }
        catch(org.omg.CosTrading.LookupPackage.IllegalPreference ex)
        {
            printLog("Illegal preference: " + ex.pref);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            printLog("Unexpected user exception occurred");
            ex.printStackTrace();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            printLog("Unexpected system exception occurred");
            ex.printStackTrace();
        }
    }

    protected void
    printLog(String message)
    {
        status_.setText(message);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        if(e.getActionCommand().equals("close"))
            System.exit(0);
        else if(e.getActionCommand().equals("print"))
        {
            int pos = printers_.getLeadSelectionIndex();
            if(pos >= 0 && pos < offers_.length)
                print(offers_[pos]);
        }
        else if(e.getActionCommand().equals("query"))
            query();
    }

    public void
    valueChanged(ListSelectionEvent e)
    {
        if(e.getValueIsAdjusting())
            return;

        int pos = printers_.getSelectedIndex();
        if(pos >= 0 && pos < offers_.length)
        {
            org.omg.CosTrading.Offer o = offers_[pos];
            org.omg.CosTrading.Property p;

            p = findProperty(o.properties, "queue");
            queueInfo_.setText(p.value.extract_string());

            p = findProperty(o.properties, "cost_per_page");
            costInfo_.setText("$" + p.value.extract_float());

            p = findProperty(o.properties, "ppm");
            speedInfo_.setText("" + p.value.extract_ushort() + " ppm");

            p = findProperty(o.properties, "color");
            colorInfo_.setText("" + (p.value.extract_boolean() ? "Yes" : "No"));

            p = findProperty(o.properties, "bldg");
            locationInfo_.setText(p.value.extract_string());
        }
    }

    protected static org.omg.CosTrading.Property
    findProperty(org.omg.CosTrading.Property[] props, String name)
    {
        for(int i = 0 ; i < props.length ; i++)
        {
            if(props[i].name.equals(name))
                return props[i];
        }

        return null;
    }

    protected void
    print(org.omg.CosTrading.Offer o)
    {
        try
        {
            String name;
            org.omg.CosTrading.Property p = findProperty(o.properties, "queue");
            name = p.value.extract_string();

            printer.PrintServer server =
                printer.PrintServerHelper.narrow(o.reference);
            if(server == null)
            {
                printLog("Object reference for '" + name + "' is invalid");
                return;
            }

            int job = server.print(name, "testfile");
            printLog("Started print job #" + job + " on queue " + name);
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            printLog("Unexpected system exception occurred");
            ex.printStackTrace();
        }
    }

    public static void
    main(String[] args)
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, props);

        org.omg.CosTrading.Lookup trader = getTrader(orb);

        Client c = new Client(orb, trader);
        c.setVisible(true);
    }

    static org.omg.CosTrading.Lookup
    getTrader(org.omg.CORBA.ORB orb)
    {
        org.omg.CosTrading.Lookup result = null;

        try
        {
            org.omg.CORBA.Object obj =
                orb.resolve_initial_references("TradingService");
            if(obj != null)
            {
                result = org.omg.CosTrading.LookupHelper.narrow(obj);
                if(result == null)
                {
                    System.err.println("Error: Invalid CosTrading::Lookup " +
                                       "object - aborting");
                    System.exit(1);
                }
            }
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            // ignore
        }

        if(result == null)
        {
            System.err.println("Error: Unable to resolve `TradingService' " +
                               "initial reference - aborting");
            System.exit(1);
        }

        return result;
    }
}
