package printer;

import java.io.*;
import java.util.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class Server
{
    static final String SERVICE_TYPE = "Printer";

    public static void
    main(String[] args)
    {
        String file = "printers.dat";
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        try
        {
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, props);
            org.omg.CORBA.BOA boa =
                ((com.ooc.CORBA.ORB)orb).BOA_init(args, props);

            //
            // Initialize the server event loop
            //
            ((com.ooc.CORBA.BOA)boa).init_servers();

            org.omg.CosTrading.Lookup trader = getTrader(orb);

            PrintServer_impl impl = new PrintServer_impl();
            orb.connect(impl);

            withdrawOldOffers(trader);
            addServiceType(orb, trader);
            exportOffers(file, orb, trader, impl);

            System.out.println("Server is ready...");
            boa.impl_is_ready(null);
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            System.err.println("Caught system exception:");
            ex.printStackTrace();
            System.exit(1);
        }

        System.exit(0);
    }

    static org.omg.CosTrading.Lookup
    getTrader(org.omg.CORBA.ORB orb)
    {
        org.omg.CosTrading.Lookup result = null;

        try
        {
            org.omg.CORBA.Object obj =
                orb.resolve_initial_references("TradingService");
            if(obj != null)
            {
                result = org.omg.CosTrading.LookupHelper.narrow(obj);
                if(result == null)
                {
                    System.err.println("Error: Invalid CosTrading::Lookup " +
                                       "object - aborting");
                    System.exit(1);
                }
            }
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            // ignore
        }

        if(result == null)
        {
            System.err.println("Error: Unable to resolve `TradingService' " +
                               "initial reference - aborting");
            System.exit(1);
        }

        return result;
    }

    static void
    withdrawOldOffers(org.omg.CosTrading.Lookup trader)
    {
        try
        {
            org.omg.CosTrading.Register reg = trader.register_if();
            reg.withdraw_using_constraint(SERVICE_TYPE, "TRUE");
        }
        catch(org.omg.CosTrading.IllegalServiceType ex)
        {
            // shouldn't happen
            ex.printStackTrace();
        }
        catch(org.omg.CosTrading.UnknownServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CosTrading.IllegalConstraint ex)
        {
            // shouldn't happen
            ex.printStackTrace();
        }
        catch(org.omg.CosTrading.RegisterPackage.NoMatchingOffers ex)
        {
            // ignore
        }
    }

    static void
    addServiceType(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        try
        {
            org.omg.CORBA.Object obj = trader.type_repos();

            org.omg.CosTradingRepos.ServiceTypeRepository repos = 
                org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.narrow(obj);

            try
            {
                repos.describe_type(SERVICE_TYPE);
                return; // type exists - nothing left to do
            }
            catch(org.omg.CosTrading.UnknownServiceType ex)
            {
                // ignore
            }

            PropStruct[] props = new PropStruct[5];

            for(int i = 0 ; i < props.length ; i++)
                props[i] = new PropStruct();

            int n = 0;

            org.omg.CORBA.TypeCode stringTC =
                orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string);

            props[n].name = "bldg";
            props[n].value_type = stringTC;
            props[n].mode = PropertyMode.PROP_NORMAL;
            n++;
            props[n].name = "color";
            props[n].value_type =
                orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_boolean);
            props[n].mode = PropertyMode.PROP_NORMAL;
            n++;
            props[n].name = "cost_per_page";
            props[n].value_type =
                orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
            props[n].mode = PropertyMode.PROP_NORMAL;
            n++;
            props[n].name = "ppm";
            props[n].value_type =
                orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ushort);
            props[n].mode = PropertyMode.PROP_NORMAL;
            n++;
            props[n].name = "queue";
            props[n].value_type = stringTC;
            props[n].mode = PropertyMode.PROP_NORMAL;
            n++;

            String[] superTypes = new String[0];

            repos.add_type(SERVICE_TYPE, printer.PrintServerHelper.id(),
                           props, superTypes);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            System.err.println("Error occurred while adding service type:");
            ex.printStackTrace();
            System.exit(1);
        }
    }

    static void
    exportOffers(String file,
                 org.omg.CORBA.ORB orb,
                 org.omg.CosTrading.Lookup trader,
                 printer.PrintServer printServer)
    {
        FileReader fr = null;
        try
        {
            org.omg.CosTrading.Register reg = trader.register_if();

            fr = new FileReader(file);
            BufferedReader in = new BufferedReader(fr);

            String line;
            while((line = in.readLine()) != null)
            {
                line = line.trim();
                if(line.startsWith("#"))
                    continue;

                org.omg.CosTrading.Property[] props =
                    parseProperties(orb, line);

                reg.export(printServer, SERVICE_TYPE, props);
            }
        }
        catch(org.omg.CORBA.UserException ex)
        {
            System.err.println("Error occurred while exporting offers:");
            ex.printStackTrace();
            System.exit(1);
        }
        catch(FileNotFoundException ex)
        {
            System.err.println("Error - file '" + file + "' not found" +
                               " - aborting");
            System.exit(1);
        }
        catch(IOException ex)
        {
            System.err.println("Error occurred while reading file:");
            ex.printStackTrace();
            System.exit(1);
        }
        finally
        {
            if(fr != null)
            {
                try
                {
                    fr.close();
                }
                catch(IOException ex)
                {
                }
            }
        }
    }

    static org.omg.CosTrading.Property[]
    parseProperties(org.omg.CORBA.ORB orb, String line)
    {
        StringTokenizer tokenizer = new StringTokenizer(line, ",");
        if(tokenizer.countTokens() != 5)
        {
            System.err.println("Invalid line: " + line);
            System.exit(1);
        }

        org.omg.CosTrading.Property[] props = new
            org.omg.CosTrading.Property[5];

        for(int i = 0 ; i < props.length ; i++)
        {
            props[i] = new org.omg.CosTrading.Property();
            props[i].value = orb.create_any();
        }

        String token;
        int n = 0;

        token = tokenizer.nextToken();
        props[n].name = "bldg";
        props[n].value.insert_string(token);
        n++;

        token = tokenizer.nextToken();
        props[n].name = "color";
        props[n].value.insert_boolean(token.equalsIgnoreCase("true"));
        n++;

        token = tokenizer.nextToken();
        float cost = 0;
        try
        {
            Float f = Float.valueOf(token);
            cost = f.floatValue();
        }
        catch(NumberFormatException ex)
        {
            System.err.println("Invalid number: " + ex);
            System.exit(1);
        }
        props[n].name = "cost_per_page";
        props[n].value.insert_float(cost);
        n++;

        token = tokenizer.nextToken();
        short ppm = 0;
        try
        {
            Short s = Short.valueOf(token);
            ppm = s.shortValue();
        }
        catch(NumberFormatException ex)
        {
            System.err.println("Invalid number: " + ex);
            System.exit(1);
        }
        props[n].name = "ppm";
        props[n].value.insert_ushort(ppm);
        n++;

        token = tokenizer.nextToken();
        props[n].name = "queue";
        props[n].value.insert_string(token);
        n++;

        return props;
    }
}
