// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

abstract class AbstractDB_impl implements AbstractDB
{
    protected org.omg.CORBA.ORB orb_;
    private com.ooc.Util.RWLock lock_ = new com.ooc.Util.RWLock();

    protected String logFile_; // The database file
    protected String logFileNew_; // The new database file
    protected String logFileBackup_; // The backup database file
    protected java.io.FileOutputStream os_; // The current output stream
    protected boolean update_; // Do we need a journal update?
    protected boolean noflush_; // Do we flush after each write?

    AbstractDB_impl(org.omg.CORBA.ORB orb)
    {
        orb_ = orb;

        update_ = false;
        noflush_ = false;
    }

    //
    // Acquires a lock of the requested type - may block
    //
    public void
    begin(int mode)
    {
        if(mode == READ)
            lock_.acquireRead();
        else
            lock_.acquireWrite();
    }

    //
    // Releases a lock
    //
    public void
    end()
    {
        lock_.release();
    }

    //
    // Initialize the database
    //
    protected void
    initDatabase(String dbdir, String name)
    {
        com.ooc.CORBA.MessageViewer viewer =
            com.ooc.CORBA.MessageViewer.instance();

        logFile_ = dbdir + java.io.File.separator + name + ".db";
        logFileNew_ = dbdir + java.io.File.separator + name + ".new";
        logFileBackup_ = dbdir + java.io.File.separator + name + ".bak";

        //
        // If the logfile doesn't exist, and the backup file does then
        // this is probably indicative of a crash
        //
        if(!fileExists(logFile_) && fileExists(logFileBackup_))
        {
            viewer.warning("Database is not present: " +
                           logFile_ + ". Using backup file: " +
                           logFileBackup_);

            if(!rename(logFileBackup_, logFile_))
            {
                viewer.warning("Rename of backup file failed.");
                throw new org.omg.CORBA.PERSIST_STORE();
            }
        }

        //
        // If the database exists, then read the contents
        //
        if(fileExists(logFile_))
        {
            java.io.FileInputStream is = null;
            java.io.BufferedReader r = null;
            try
            {
                is = new java.io.FileInputStream(logFile_);
                r = new java.io.BufferedReader(
                    new java.io.InputStreamReader(is));
            }
            catch(java.io.IOException ex)
            {
                viewer.error("Cannot open database file: " + logFile_);
                throw new org.omg.CORBA.PERSIST_STORE();
            }

            try
            {
                readVersion(r);

                while(true)
                {
                    org.omg.CORBA.Any any = readAny(r);
                    if(any == null)
                        break;

                    execute(any);
                }
            }
            catch(java.io.IOException ex)
            {
                try
                {
                    is.close();
                }
                catch(java.io.IOException e)
                {
                }

                viewer.error("Error reading database file: " + logFile_);
                throw new org.omg.CORBA.PERSIST_STORE();
            }

            try
            {
                is.close();
            }
            catch(java.io.IOException e)
            {
            }

            openLog();
            update_ = true;
        }
        else
        {
            //
            // Initialize the database
            //
            try
            {
                openLog();
                writeVersion();
                update();
                update_ = true;
            }
            catch(java.io.IOException ex)
            {
                viewer.error("Error initializing database: " + ex.toString());
                throw new org.omg.CORBA.PERSIST_STORE();
            }
        }
    }

    //
    // Hook for subclass to execute commands during initialization
    //
    protected abstract void
    execute(org.omg.CORBA.Any cmd);

    //
    // Called by Logger to update the journal
    //
    void
    updateJournal()
    {
        if(!update_)
            return;

        com.ooc.CORBA.MessageViewer viewer =
            com.ooc.CORBA.MessageViewer.instance();
        try
        {
            //
            // Close old database
            //
            os_.close();

            os_ = new java.io.FileOutputStream(logFileNew_);
        }
        catch(java.io.IOException ex)
        {
            viewer.error("Cannot open `" + logFileNew_ + "': " +
                         ex.toString());
            throw new org.omg.CORBA.PERSIST_STORE();
        }

        //
        // Write
        //
        noflush_ = true;

        try
        {
            try
            {
                writeVersion();
                update();
            }
            catch(java.io.IOException ex)
            {
                viewer.error("Compact failed: " + ex.toString());
                throw new org.omg.CORBA.PERSIST_STORE();
            }
        }
        catch(org.omg.CORBA.PERSIST_STORE ex)
        {
            noflush_ = false;
            try
            {
                os_.close();
            }
            catch(java.io.IOException e)
            {
            }

            openLog();

            throw ex;
        }

        try
        {
            os_.close();
        }
        catch(java.io.IOException e)
        {
        }

        noflush_ = false;

        //
        // Move the current to backup. If this fails, it's not a big deal.
        //
        rename(logFile_, logFileBackup_);

        //
        // Move the new logfile to current. This failing IS a big deal!
        //
        if(!rename(logFileNew_, logFile_))
            throw new org.omg.CORBA.PERSIST_STORE();

        (new java.io.File(logFileBackup_)).delete();

        openLog();
        update_ = false;
    }

    //
    // Hook for subclass to update the journal
    //
    protected abstract void
    update()
        throws java.io.IOException;

    protected void
    openLog()
    {
        //
        // Open the database
        //
        try
        {
            os_ = new java.io.FileOutputStream(logFile_, true);
        }
        catch(java.io.IOException ex)
        {
            com.ooc.CORBA.MessageViewer.instance().
                error("Cannot open `" + logFile_ + "': " + ex.toString());
            throw new org.omg.CORBA.PERSIST_STORE();
        }
    }

    protected void
    writeLog(org.omg.CORBA.Any any)
    {
        try
        {
            writeAny(any);
            if(!noflush_)
                os_.flush();
        }
        catch(java.io.IOException ex)
        {
            com.ooc.CORBA.MessageViewer.instance().
                error("Error writing to database: " + ex.toString());
            throw new org.omg.CORBA.PERSIST_STORE();
        }
    }

    protected boolean
    fileExists(String file)
    {
        return (new java.io.File(file).exists());
    }

    private void
    writeVersion()
        throws java.io.IOException
    {
        org.omg.CORBA.Any any = orb_.create_any();
        any.insert_string(com.ooc.CORBA.Version.version);
        writeLog(any);
    }

    private boolean
    readVersion(java.io.BufferedReader r)
        throws java.io.IOException
    {
        org.omg.CORBA.Any any = readAny(r);
        String version = any.extract_string();
        return version.equals(com.ooc.CORBA.Version.version);
    }

    protected void
    writeAny(org.omg.CORBA.Any any)
        throws java.io.IOException
    {
        com.ooc.CORBA.OutputStream out = new com.ooc.CORBA.OutputStream();

        //
        // We marshal our endianess
        //
        out._OB_writeEndian();

        //
        // Now marshal the actual data
        //
        out.write_any(any);

        byte[] data =
            com.ooc.CORBA.HexConverter.octetsToAsciiBytes(out._OB_buffer(),
                                                          out._OB_count());
        os_.write(data);
        os_.write('\n');
    }

    protected org.omg.CORBA.Any
    readAny(java.io.BufferedReader r)
        throws java.io.IOException
    {
        String line = r.readLine();
        if(line == null)
            return null;

        //
        // Convert the octet character string to an
        // octet sequence and create input stream
        //
        byte[] data = com.ooc.CORBA.HexConverter.asciiToOctets(line);
        com.ooc.CORBA.InputStream in = new com.ooc.CORBA.InputStream(data);

        //
        // Unmarshal the endianness, and the any
        //
        in._OB_readEndian();
        return in.read_any();
    }

    protected boolean
    rename(String from, String to)
    {
        boolean result =
            (new java.io.File(from)).renameTo(new java.io.File(to));

        if(!result)
        {
            com.ooc.CORBA.MessageViewer.instance().
                error("Rename " + from + " to " + to + " failed.");

            return false;
        }

        return true;
    }
}
