// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

final class BinaryNode extends Node
{
    private org.omg.CORBA.ORB orb_;
    private int token_;
    private Node left_;
    private Node right_;

    BinaryNode(org.omg.CORBA.ORB orb, int token, Node left, Node right)
    {
        orb_ = orb;
        token_ = token;
        left_ = left;
        right_ = right;
    }

    void
    print(java.io.PrintStream out)
    {
        out.print("BinaryNode: ");

        switch(token_)
        {
        case Tokens.TOK_MINUS:
            out.print("-");
            break;
        case Tokens.TOK_MULT:
            out.print("*");
            break;
        case Tokens.TOK_DIV:
            out.print("/");
            break;
        case Tokens.TOK_PLUS:
            out.print("+");
            break;
        case Tokens.TOK_TWIDDLE:
            out.print("~");
            break;
        case Tokens.TOK_IN:
            out.print("in");
            break;
        case Tokens.TOK_EQ:
            out.print("==");
            break;
        case Tokens.TOK_NE:
            out.print("!=");
            break;
        case Tokens.TOK_LT:
            out.print("<");
            break;
        case Tokens.TOK_LE:
            out.print("<=");
            break;
        case Tokens.TOK_GT:
            out.print(">");
            break;
        case Tokens.TOK_GE:
            out.print(">=");
            break;
        case Tokens.TOK_AND:
            out.print("and");
            break;
        case Tokens.TOK_OR:
            out.print("or");
            break;
        default:
            throw new RuntimeException("invalid binary operator");
        }

        System.out.println();

        left_.print(out);
        right_.print(out);
    }

    Value
    evaluate(PropertyList props)
    {
        Value result = null;

        switch(token_)
        {
        case Tokens.TOK_MINUS:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null)
            {
                if(left.kind() == Value.KIND_STRING &&
                   right.kind() == Value.KIND_STRING)
                    result = left.stringSub(right);
                else if(left.numeric() && right.numeric())
                    result = left.subtract(right);
            }
            break;
        }

        case Tokens.TOK_MULT:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null)
                if(left.numeric() && right.numeric())
                    result = left.multiply(right);
            break;
        }

        case Tokens.TOK_DIV:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null)
                if(left.numeric() && right.numeric() && !right.zero())
                    result = left.divide(right);
            break;
        }

        case Tokens.TOK_PLUS:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null)
            {
                if(left.kind() == Value.KIND_STRING &&
                   right.kind() == Value.KIND_STRING)
                    result = left.stringAdd(right);
                else if(left.numeric() && right.numeric())
                    result = left.add(right);
            }
            break;
        }

        case Tokens.TOK_TWIDDLE:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null)
            {
                if(left.kind() == Value.KIND_STRING &&
                   right.kind() == Value.KIND_STRING)
                    result = new Value(right.twiddle(left));
            }
            break;
        }

        case Tokens.TOK_IN:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null)
                result = right.contains(orb_, left);
        }

        case Tokens.TOK_EQ:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null && Value.comparable(left, right))
                result = new Value(left.equals(right));
            break;
        }

        case Tokens.TOK_NE:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null && Value.comparable(left, right))
                result = new Value(!left.equals(right));
            break;
        }

        case Tokens.TOK_LT:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null && Value.comparable(left, right))
                result = new Value(left.lessThan(right));
            break;
        }

        case Tokens.TOK_LE:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null && Value.comparable(left, right))
                result = new Value(left.lessThanEqual(right));
            break;
        }

        case Tokens.TOK_GT:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null && Value.comparable(left, right))
                result = new Value(left.greaterThan(right));
            break;
        }

        case Tokens.TOK_GE:
        {
            Value left = left_.evaluate(props), right = right_.evaluate(props);
            if(left != null && right != null && Value.comparable(left, right))
                result = new Value(left.greaterThanEqual(right));
            break;
        }

        case Tokens.TOK_AND:
        {
            Value left = left_.evaluate(props);
            if(left == null || left.kind() != Value.KIND_BOOLEAN)
                break;

            //
            // Short-circuit
            //
            if(left.get_boolean())
            {
                Value right = right_.evaluate(props);
                if(right != null && right.kind() == Value.KIND_BOOLEAN)
                    result = right;
            }
            else
                result = new Value(false);
            break;
        }

        case Tokens.TOK_OR:
        {
            Value left = left_.evaluate(props);
            if(left == null || left.kind() != Value.KIND_BOOLEAN)
                break;

            //
            // Short-circuit
            //
            if(left.get_boolean())
                result = left;
            else
            {
                Value right = right_.evaluate(props);
                if(right != null && right.kind() == Value.KIND_BOOLEAN)
                    result = right;
            }
            break;
        }

        default:
            break;
        }

        return result;
    }

    boolean
    checkTypes()
    {
        if(!left_.checkTypes() || !right_.checkTypes())
            return false;

        int left = left_.type();
        int right = right_.type();

        switch(token_)
        {
        case Tokens.TOK_MINUS:
        case Tokens.TOK_PLUS:
            //
            // +,- allowed on strings & numeric types
            //
            return ((left_.numeric() || left == TYPE_UNKNOWN) &&
                    (right_.numeric() || right == TYPE_UNKNOWN)) ||
                   ((left == TYPE_STRING || left == TYPE_UNKNOWN) &&
                    (right == TYPE_STRING || right == TYPE_UNKNOWN));

        case Tokens.TOK_MULT:
        case Tokens.TOK_DIV:
            return ((left_.numeric() || left == TYPE_UNKNOWN) &&
                    (right_.numeric() || right == TYPE_UNKNOWN));

        case Tokens.TOK_TWIDDLE:
            return ((left == TYPE_STRING || left == TYPE_UNKNOWN) &&
                    (right == TYPE_STRING || right == TYPE_UNKNOWN));

        case Tokens.TOK_IN:
        case Tokens.TOK_EQ:
        case Tokens.TOK_NE:
        case Tokens.TOK_LT:
        case Tokens.TOK_LE:
        case Tokens.TOK_GT:
        case Tokens.TOK_GE:
            if(left != TYPE_UNKNOWN && right != TYPE_UNKNOWN)
            {
                if(left_.numeric() && right_.numeric())
                    return true;
                else if(left == TYPE_STRING && right == TYPE_STRING)
                    return true;
                else
                    return false;
            }
            else
                return true;

        case Tokens.TOK_AND:
        case Tokens.TOK_OR:
            return ((left == TYPE_BOOLEAN || left == TYPE_UNKNOWN) &&
                    (right == TYPE_BOOLEAN || right == TYPE_UNKNOWN));
        }

        return false;
    }

    int
    type()
    {
        int result = TYPE_UNKNOWN;

        switch(token_)
        {
        case Tokens.TOK_MINUS:
        case Tokens.TOK_PLUS:
            if(left_.type() == TYPE_NUMBER || right_.type() == TYPE_NUMBER)
                result = TYPE_NUMBER;
            else if(left_.type() == TYPE_STRING ||
                    right_.type() == TYPE_STRING)
                result = TYPE_STRING;
            break;

        case Tokens.TOK_MULT:
        case Tokens.TOK_DIV:
            result = TYPE_NUMBER;
            break;

        case Tokens.TOK_TWIDDLE:
        case Tokens.TOK_IN:
        case Tokens.TOK_EQ:
        case Tokens.TOK_NE:
        case Tokens.TOK_LT:
        case Tokens.TOK_LE:
        case Tokens.TOK_GT:
        case Tokens.TOK_GE:
        case Tokens.TOK_AND:
        case Tokens.TOK_OR:
            result = TYPE_BOOLEAN;
            break;

        default:
            break;
        }

        return result;
    }
}
