// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

final class DBManager_impl extends DBManager
{
    private OfferDB_impl offerDB_;
    private TypeDB_impl typeDB_;
    private LinkDB_impl linkDB_;
    private AttributeDB_impl attributeDB_;
    private Logger logger_;

    DBManager_impl(org.omg.CORBA.ORB orb, String dbdir, int interval)
    {
        offerDB_ = new OfferDB_impl(orb, dbdir);
        typeDB_ = new TypeDB_impl(orb, dbdir);
        linkDB_ = new LinkDB_impl(orb, dbdir);
        attributeDB_ = new AttributeDB_impl(orb, dbdir);

        if(dbdir != null)
        {
            AbstractDB_impl[] dbs =
                { offerDB_, typeDB_, linkDB_, attributeDB_ };
            logger_ = new Logger(dbs, interval * 60);
            logger_.start();
        }
    }

    boolean
    isPropertyTypeSupported(org.omg.CORBA.TypeCode type)
    {
        return true;
    }

    boolean
    isPolicyTypeSupported(org.omg.CORBA.TypeCode type)
    {
        return true;
    }

    boolean
    isPropertyNameValid(String name)
    {
        return isNameValid(name);
    }

    boolean
    isPolicyNameValid(String name)
    {
        return isNameValid(name);
    }

    boolean
    allowUnknownProperties()
    {
        return true;
    }

    OfferDB
    getOfferDB()
    {
        return offerDB_;
    }

    TypeDB
    getTypeDB()
    {
        return typeDB_;
    }

    LinkDB
    getLinkDB()
    {
        return linkDB_;
    }

    AttributeDB
    getAttributeDB()
    {
        return attributeDB_;
    }

    void
    shutdown()
    {
        if(logger_ != null)
            logger_.halt();
    }

    protected boolean
    isNameValid(String name)
    {
        //
        // A name cannot be empty, must start with a letter, and must
        // be followed by zero or more letters, digits and underscores
        //

        String s = name.trim();
        if(s.length() == 0)
            return false;

        char ch;

        ch = s.charAt(0);
        if(! Character.isLetter(ch))
            return false;

        for(int i = 0 ; i < s.length() ; i++)
        {
            ch = s.charAt(i);
            if(! Character.isLetterOrDigit(ch) && ch != '_')
                return false;
        }

        return true;
    }
}
