// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Utility methods for interacting with the Interface Repository
//
final class IRUtil
{
    private static org.omg.CORBA.Repository repos_ = null;

    static void
    initialize(org.omg.CORBA.Repository repos)
    {
        repos_ = repos;
    }

    static org.omg.CORBA.Repository
    getRepository()
    {
        return repos_;
    }

    static boolean
    validate(org.omg.CORBA.InterfaceDef def, String interfaceName)
    {
        boolean result = true;

        //
        // Verify that the given InterfaceDef is compatible with the
        // given interface
        //

        try
        {
            org.omg.CORBA.Repository repos = def.containing_repository();

            org.omg.CORBA.Contained c = null;

            if(repos != null)
            {
                if(isScopedName(interfaceName))
                    c = repos.lookup(interfaceName);
                else if(isRepositoryId(interfaceName))
                    c = repos.lookup_id(interfaceName);
            }

            //
            // We validate only if we have both the object's definition
            // and the type's interface definition
            //
            if(c != null)
            {
                String id = c.id();
                result = def.is_a(id);
            }
        }
        catch(org.omg.CORBA.SystemException e)
        {
            // ignore
        }

        return result;
    }

    static boolean
    validateName(String name)
    {
        boolean result;

        if(name.indexOf("::") >= 0)
            result = isScopedName(name);
        else if(name.indexOf(':') >= 0)
            result = isRepositoryId(name);
        else
            result = isScopedName(name);

        return result;
    }

    static org.omg.CORBA.Contained
    getContained(String interfaceName)
    {
        org.omg.CORBA.Contained result = null;

        if(repos_ != null)
        {
            if(isScopedName(interfaceName))
                result = repos_.lookup(interfaceName);
            else if(isRepositoryId(interfaceName))
                result = repos_.lookup_id(interfaceName);
        }

        return result;
    }

    private static boolean
    isScopedName(String name)
    {
        java.util.StringTokenizer tokenizer;
        boolean seenIdent = false;
        String lastToken = null;
        int colonCount = 0;

        tokenizer = new java.util.StringTokenizer(name, ":", true);

        while(tokenizer.hasMoreTokens())
        {
            String tok = tokenizer.nextToken();
            if(tok.equals(":"))
            {
                colonCount++;
                if(colonCount > 2)
                    return false;
            }
            else
            {
                if(colonCount > 0 && colonCount != 2)
                    return false;

                colonCount = 0;
                seenIdent = true;

                // must start with letter
                if(! Character.isLetter(tok.charAt(0)))
                    return false;

                for(int i = 1 ; i < tok.length() ; i++)
                {
                    char ch = tok.charAt(i);
                    if(! Character.isLetterOrDigit(ch) && ch != '_')
                        return false;
                }
            }

            lastToken = tok;
        }

        if(! seenIdent)
            return false;

        if(lastToken.equals(":"))
            return false;

        return true;
    }

    private static boolean
    isRepositoryId(String name)
    {
        java.util.StringTokenizer tokenizer =
            new java.util.StringTokenizer(name, ":");

        if(tokenizer.countTokens() == 0)
            return false;

        String format = tokenizer.nextToken();

        if(format.equals("IDL"))
        {
            //
            // IDL format is "IDL:[prefix/]ID/ID[/ID...]:X.Y"
            //

            if(tokenizer.countTokens() != 2)
                return false;

            //
            // extract the middle portion containing the identifiers
            //
            String ids = tokenizer.nextToken();

            java.util.StringTokenizer idTokenizer =
                new java.util.StringTokenizer(ids, "/");

            if(idTokenizer.countTokens() == 0)
                return false;

            while(idTokenizer.hasMoreTokens())
            {
                String tok = idTokenizer.nextToken();
                if(tok.length() == 0)
                    return false;

                //
                // must start with letter
                //
                if(! Character.isLetter(tok.charAt(0)))
                    return false;

                String legalChars = "_.-";
                for(int i = 1 ; i < tok.length() ; i++)
                {
                    char ch = tok.charAt(i);
                    if(! Character.isLetterOrDigit(ch) &&
                       legalChars.indexOf(ch) == -1)
                        return false;
                }
            }

            //
            // extract the last portion, containing the version
            //
            String version = tokenizer.nextToken();
            int pos = version.indexOf('.');
            if(pos <= 0 || pos == version.length() - 1)    // ".X" or "X."
                return false;
            if(pos != version.lastIndexOf('.'))    // "X.Y."
                return false;

            for(int i = 0 ; i < version.length() ; i++)
            {
                char ch = version.charAt(i);
                if(! Character.isDigit(ch) && ch != '.')
                    return false;
            }
        }
        else if(format.equals("DCE"))
        {
            //
            // DCE format is "DCE:UUID:X"
            //

            if(tokenizer.countTokens() != 2)
                return false;

            //
            // extract the middle portion containing the UUID
            //
            String uuid = tokenizer.nextToken();
            String legalUUIDChars = "0123456789-ABCDEFabcdef";
            for(int i = 0 ; i < uuid.length() ; i++)
            {
                char ch = uuid.charAt(i);
                if(legalUUIDChars.indexOf(ch) == -1)
                    return false;
            }

            //
            // extract the last portion, containing the version
            //
            String version = tokenizer.nextToken();
            for(int i = 0 ; i < version.length() ; i++)
            {
                char ch = version.charAt(i);
                if(! Character.isDigit(ch))
                    return false;
            }
        }
        else if(format.equals("LOCAL"))
        {
            //
            // LOCAL format is "LOCAL:X"
            //

            if(tokenizer.countTokens() == 0)
                return false;
        }
        else
            return false;

        return true;
    }
}
