// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

import com.ooc.OBCosTrading.*;

final class LinkDB_impl extends AbstractDB_impl implements LinkDB
{
    private java.util.Hashtable links_; // The links

    LinkDB_impl(org.omg.CORBA.ORB orb, String dbdir)
    {
        super(orb);

        links_ = new java.util.Hashtable(67);

        if(dbdir != null)
            super.initDatabase(dbdir, "links");
    }

    //
    // Called by super to execute commands read from database
    // during initialization
    //
    protected void
    execute(org.omg.CORBA.Any any)
    {
        org.omg.CORBA.TypeCode tc = any.type();
        if(tc.equal(CreateLinkHelper.type()))
        {
            CreateLink cmd = CreateLinkHelper.extract(any);
            org.omg.CosTrading.LinkPackage.LinkInfo info =
                new org.omg.CosTrading.LinkPackage.LinkInfo();
            info.target = cmd.target;
            // don't initialize info.target_reg here - see describe()
            info.def_pass_on_follow_rule = cmd.pass_on_follow;
            info.limiting_follow_rule = cmd.limiting_follow;
            links_.put(cmd.name, info);
        }
        else if(tc.equal(ModifyLinkHelper.type()))
        {
            ModifyLink cmd = ModifyLinkHelper.extract(any);
            org.omg.CosTrading.LinkPackage.LinkInfo info =
                (org.omg.CosTrading.LinkPackage.LinkInfo)links_.get(cmd.name);
            if(info == null)
            {
                com.ooc.CORBA.MessageViewer.instance().error(
                    "Missing link '" + cmd.name + "' in database.");
                throw new org.omg.CORBA.PERSIST_STORE();
            }
            info.def_pass_on_follow_rule = cmd.pass_on_follow;
            info.limiting_follow_rule = cmd.limiting_follow;
        }
        else if(tc.equal(RemoveLinkHelper.type()))
        {
            RemoveLink cmd = RemoveLinkHelper.extract(any);
            org.omg.CosTrading.LinkPackage.LinkInfo info =
                (org.omg.CosTrading.LinkPackage.LinkInfo)
                links_.remove(cmd.name);
            if(info == null)
            {
                com.ooc.CORBA.MessageViewer.instance().error(
                    "Missing link '" + cmd.name + "' in database.");
                throw new org.omg.CORBA.PERSIST_STORE();
            }
        }
        else
        {
            com.ooc.CORBA.MessageViewer.instance().error(
                "Unknown record in database.");
            throw new org.omg.CORBA.PERSIST_STORE();
        }
    }

    //
    // Called by super to update the journal
    //
    protected void
    update()
        throws java.io.IOException
    {
        java.util.Enumeration e = links_.keys();
        while(e.hasMoreElements())
        {
            String name = (String)e.nextElement();
            org.omg.CosTrading.LinkPackage.LinkInfo info =
                (org.omg.CosTrading.LinkPackage.LinkInfo)links_.get(name);
            CreateLink cmd = new CreateLink();
            cmd.name = name;
            cmd.target = info.target;
            cmd.pass_on_follow = info.def_pass_on_follow_rule;
            cmd.limiting_follow = info.limiting_follow_rule;
            org.omg.CORBA.Any any = orb_.create_any();
            CreateLinkHelper.insert(any, cmd);
            writeLog(any);
        }
    }

    //
    // Returns true if the link with the given name exists
    //
    public boolean
    exists(String name)
    {
        return links_.containsKey(name);
    }

    //
    // Creates a new link
    //
    public void
    create(String name,
           org.omg.CosTrading.Lookup target,
           org.omg.CosTrading.FollowOption passOnFollow,
           org.omg.CosTrading.FollowOption limitingFollow)
    {
        if(os_ != null)
        {
            CreateLink cmd = new CreateLink();
            cmd.name = name;
            cmd.target = target;
            cmd.pass_on_follow = passOnFollow;
            cmd.limiting_follow = limitingFollow;
            org.omg.CORBA.Any any = orb_.create_any();
            CreateLinkHelper.insert(any, cmd);
            writeLog(any);
            update_ = true;
        }

        org.omg.CosTrading.Register target_reg = null;
        try
        {
            target_reg = target.register_if();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
        }

        org.omg.CosTrading.LinkPackage.LinkInfo info =
            new org.omg.CosTrading.LinkPackage.LinkInfo(target, target_reg,
                                                        passOnFollow,
                                                        limitingFollow);
        links_.put(name, info);
    }

    //
    // Removes the link with the given name
    //
    public void
    remove(String name)
    {
        if(os_ != null)
        {
            RemoveLink cmd = new RemoveLink();
            cmd.name = name;
            org.omg.CORBA.Any any = orb_.create_any();
            RemoveLinkHelper.insert(any, cmd);
            writeLog(any);
            update_ = true;
        }

        links_.remove(name);
    }

    //
    // Returns a description of the link with the given name, or null if
    // the link wasn't found
    //
    public org.omg.CosTrading.LinkPackage.LinkInfo
    describe(String name)
    {
        org.omg.CosTrading.LinkPackage.LinkInfo info =
            (org.omg.CosTrading.LinkPackage.LinkInfo)links_.get(name);

        //
        // Obtain the Register reference if necessary
        //
        if(info.target_reg == null)
        {
            try
            {
                info.target_reg = info.target.register_if();
            }
            catch(org.omg.CORBA.SystemException ex)
            {
            }
        }

        return info;
    }

    //
    // Updates the link information
    //
    public void
    modify(String name,
           org.omg.CosTrading.FollowOption passOnFollow,
           org.omg.CosTrading.FollowOption limitingFollow)
    {
        if(os_ != null)
        {
            ModifyLink cmd = new ModifyLink();
            cmd.name = name;
            cmd.pass_on_follow = passOnFollow;
            cmd.limiting_follow = limitingFollow;
            org.omg.CORBA.Any any = orb_.create_any();
            ModifyLinkHelper.insert(any, cmd);
            writeLog(any);
            update_ = true;
        }

        org.omg.CosTrading.LinkPackage.LinkInfo info =
            (org.omg.CosTrading.LinkPackage.LinkInfo)links_.get(name);
        info.def_pass_on_follow_rule = passOnFollow;
        info.limiting_follow_rule = limitingFollow;
    }

    //
    // Returns all link names
    //
    public String[]
    getLinkNames()
    {
        String[] result = new String[links_.size()];
        java.util.Enumeration e = links_.keys();
        int i = 0;
        while(e.hasMoreElements())
            result[i++] = (String)e.nextElement();
        return result;
    }
}
