// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Implementation of CosTrading::Link
//
final class LinkImpl extends org.omg.CosTrading._LinkImplBase
{
    private LinkDB db_;
    private org.omg.CosTrading.Admin admin_;

    LinkImpl(DBManager dbMgr)
    {
        db_ = dbMgr.getLinkDB();
    }

    void
    setAdmin(org.omg.CosTrading.Admin admin)
    {
        admin_ = admin;
    }

    //
    // operations inherited from CosTrading::TraderComponents
    //

    public org.omg.CosTrading.Lookup
    lookup_if()
    {
        return admin_.lookup_if();
    }

    public org.omg.CosTrading.Register
    register_if()
    {
        return admin_.register_if();
    }

    public org.omg.CosTrading.Link
    link_if()
    {
        return this;
    }

    public org.omg.CosTrading.Proxy
    proxy_if()
    {
        return admin_.proxy_if();
    }

    public org.omg.CosTrading.Admin
    admin_if()
    {
        return admin_;
    }


    //
    // operations inherited from CosTrading::SupportAttributes
    //

    public boolean
    supports_modifiable_properties()
    {
        return admin_.supports_modifiable_properties();
    }

    public boolean
    supports_dynamic_properties()
    {
        return admin_.supports_dynamic_properties();
    }

    public boolean
    supports_proxy_offers()
    {
        return admin_.supports_proxy_offers();
    }

    public org.omg.CORBA.Object
    type_repos()
    {
        return admin_.type_repos();
    }


    //
    // operations inherited from CosTrading::LinkAttributes
    //

    public org.omg.CosTrading.FollowOption
    max_link_follow_policy()
    {
        return admin_.max_link_follow_policy();
    }


    //
    // operations inherited from CosTrading::Link
    //

    public void
    add_link(String name,
             org.omg.CosTrading.Lookup target,
             org.omg.CosTrading.FollowOption def_pass_on_follow_rule,
             org.omg.CosTrading.FollowOption limiting_follow_rule)
        throws org.omg.CosTrading.LinkPackage.IllegalLinkName,
               org.omg.CosTrading.LinkPackage.DuplicateLinkName,
               org.omg.CosTrading.InvalidLookupRef,
               org.omg.CosTrading.LinkPackage.DefaultFollowTooPermissive,
               org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive
    {
        checkName(name);

        if(target == null)
            throw new org.omg.CosTrading.InvalidLookupRef(target);

        checkFollow(def_pass_on_follow_rule, limiting_follow_rule);

        db_.begin(AbstractDB.WRITE);

        try
        {
            if(db_.exists(name))
                throw new org.omg.CosTrading.LinkPackage.
                          DuplicateLinkName(name);

            db_.create(name, target, def_pass_on_follow_rule,
                       limiting_follow_rule);
        }
        finally
        {
            db_.end();
        }
    }

    public void
    remove_link(String name)
        throws org.omg.CosTrading.LinkPackage.IllegalLinkName,
               org.omg.CosTrading.LinkPackage.UnknownLinkName
    {
        checkName(name);

        db_.begin(AbstractDB.WRITE);

        try
        {
            if(!db_.exists(name))
                throw new org.omg.CosTrading.LinkPackage.UnknownLinkName(name);

            db_.remove(name);
        }
        finally
        {
            db_.end();
        }
    }

    public org.omg.CosTrading.LinkPackage.LinkInfo
    describe_link(String name)
        throws org.omg.CosTrading.LinkPackage.IllegalLinkName,
               org.omg.CosTrading.LinkPackage.UnknownLinkName
    {
        org.omg.CosTrading.LinkPackage.LinkInfo result = null;

        checkName(name);

        db_.begin(AbstractDB.READ);

        try
        {
            if(!db_.exists(name))
                throw new org.omg.CosTrading.LinkPackage.UnknownLinkName(name);

            result = db_.describe(name);
        }
        finally
        {
            db_.end();
        }

        return result;
    }

    public String[]
    list_links()
    {
        String[] result = null;

        db_.begin(AbstractDB.READ);

        try
        {
            result = db_.getLinkNames();
        }
        finally
        {
            db_.end();
        }

        return result;
    }

    public void
    modify_link(String name,
                org.omg.CosTrading.FollowOption def_pass_on_follow_rule,
                org.omg.CosTrading.FollowOption limiting_follow_rule)
        throws org.omg.CosTrading.LinkPackage.IllegalLinkName,
               org.omg.CosTrading.LinkPackage.UnknownLinkName,
               org.omg.CosTrading.LinkPackage.DefaultFollowTooPermissive,
               org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive
    {
        checkName(name);

        checkFollow(def_pass_on_follow_rule, limiting_follow_rule);

        db_.begin(AbstractDB.WRITE);

        try
        {
            if(!db_.exists(name))
                throw new org.omg.CosTrading.LinkPackage.UnknownLinkName(name);

            db_.modify(name, def_pass_on_follow_rule, limiting_follow_rule);
        }
        finally
        {
            db_.end();
        }
    }

    private void
    checkName(String name)
        throws org.omg.CosTrading.LinkPackage.IllegalLinkName
    {
        if(name == null || name.trim().length() == 0)
            throw new org.omg.CosTrading.LinkPackage.IllegalLinkName(name);
    }

    private void
    checkFollow(org.omg.CosTrading.FollowOption def_pass_on_follow_rule,
                org.omg.CosTrading.FollowOption limiting_follow_rule)
        throws org.omg.CosTrading.LinkPackage.DefaultFollowTooPermissive,
               org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive
    {
        if(def_pass_on_follow_rule.value() > limiting_follow_rule.value())
            throw new org.omg.CosTrading.LinkPackage.
                      DefaultFollowTooPermissive(def_pass_on_follow_rule,
                                                 limiting_follow_rule);

        org.omg.CosTrading.FollowOption maxLinkFollow =
            max_link_follow_policy();
        if(limiting_follow_rule.value() > maxLinkFollow.value())
            throw new org.omg.CosTrading.LinkPackage.
                      LimitingFollowTooPermissive(limiting_follow_rule,
                                                  maxLinkFollow);
    }
}
