// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

abstract class Node
{
    public static final int TYPE_UNKNOWN = 0;
    public static final int TYPE_BOOLEAN = 1;
    public static final int TYPE_STRING  = 2;
    public static final int TYPE_NUMBER  = 3;

    abstract void
    print(java.io.PrintStream out);

    abstract Value
    evaluate(PropertyList props);

    abstract boolean
    checkTypes();

    abstract int
    type();

    boolean
    numeric()
    {
        return numericType(type());
    }

    boolean
    exists(PropertyList props)
    {
        //
        // default implementation - should be overridden by subclasses
        // where necessary
        //
        throw new RuntimeException();
    }

    protected
    Node()
    {
    }

    protected static int
    convertType(org.omg.CORBA.TypeCode tc)
    {
        int result;

        org.omg.CORBA.TypeCode origTC = Value.getOrigType(tc);

        switch(origTC.kind().value())
        {
        case org.omg.CORBA.TCKind._tk_short:
        case org.omg.CORBA.TCKind._tk_ushort:
        case org.omg.CORBA.TCKind._tk_long:
        case org.omg.CORBA.TCKind._tk_ulong:
        case org.omg.CORBA.TCKind._tk_float:
        case org.omg.CORBA.TCKind._tk_double:
        case org.omg.CORBA.TCKind._tk_octet:
            result = TYPE_NUMBER;
            break;

        case org.omg.CORBA.TCKind._tk_boolean:
            result = TYPE_BOOLEAN;
            break;

        case org.omg.CORBA.TCKind._tk_char:
        case org.omg.CORBA.TCKind._tk_string:
            result = TYPE_STRING;
            break;

        default:
            result = TYPE_UNKNOWN;
        }

        return result;
    }

    protected static boolean
    numericType(int type)
    {
        //
        // Boolean is considered a numeric type
        //
        return (type == TYPE_BOOLEAN || type == TYPE_NUMBER);
    }
}
