// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

interface OfferDB extends AbstractDB
{
    //
    // Returns true if the offerId is legal, false otherwise
    //
    boolean
    validateOfferId(String offerId);

    //
    // Returns true if the offer with the given ID exists
    //
    boolean
    exists(String offerId);

    //
    // Returns true if the offer with the given ID is a proxy offer
    //
    boolean
    isProxy(String offerId);

    //
    // Creates a new offer, returning the assigned offer ID
    //
    String
    create(String serviceType,
           org.omg.CORBA.Object obj,
           org.omg.CosTrading.Property[] props);

    //
    // Creates a new proxy offer, returning the assigned offer ID
    //
    String
    createProxy(org.omg.CosTrading.Lookup target,
                String serviceType,
                org.omg.CosTrading.Property[] props,
                boolean ifMatchAll,
                String recipe,
                org.omg.CosTrading.Policy[] policies);

    //
    // Removes the offer with the given ID
    //
    void
    remove(String offerId);

    //
    // Removes the proxy offer with the given ID
    //
    void
    removeProxy(String offerId);

    //
    // Returns a description of the offer with the given ID, or null if
    // the offer wasn't found
    //
    org.omg.CosTrading.RegisterPackage.OfferInfo
    describe(String offerId);

    //
    // Returns a description of the proxy offer with the given ID, or null
    // if the offer wasn't found
    //
    org.omg.CosTrading.ProxyPackage.ProxyInfo
    describeProxy(String offerId);

    //
    // Updates the properties of an offer
    //
    void
    modify(String offerId, org.omg.CosTrading.Property[] props);

    //
    // Returns all offers of the given service type in a hashtable,
    // where the offer ID is the key and OfferInfo is the value;
    // returns null if the service type does not exist
    //
    java.util.Hashtable
    getOffers(String serviceType);

    //
    // Returns all offer IDs of the given service type;
    // returns null if the service type does not exist
    //
    String[]
    getOfferIds(String serviceType);

    //
    // Returns all proxy offers of the given service type in a hashtable,
    // where the offer ID is the key and ProxyInfo is the value;
    // returns null if the service type does not exist
    //
    java.util.Hashtable
    getProxyOffers(String serviceType);

    //
    // Returns all proxy offer IDs of the given service type;
    // returns null if the service type does not exist
    //
    String[]
    getProxyOfferIds(String serviceType);

    //
    // Returns the service type of the given offer;
    // returns null if the offer ID is malformed
    //
    String
    whichService(String offerId);
}
