// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Things are just too ugly without this
//
import org.omg.CosTrading.RegisterPackage.OfferInfo;
import org.omg.CosTrading.ProxyPackage.ProxyInfo;
import com.ooc.OBCosTrading.*;

final class OfferDB_impl extends AbstractDB_impl implements OfferDB
{
    private int nextId_; // The next ID
    private java.util.Hashtable allOffers_; // The offers
    private java.util.Hashtable allProxies_; // The proxy offers
    private java.util.Hashtable offerTables_; // Offer indices
    private java.util.Hashtable proxyTables_; // Proxy indices
    private static final String ID_SEP = "~";

    OfferDB_impl(org.omg.CORBA.ORB orb, String dbdir)
    {
        super(orb);

        nextId_ = 1;
        allOffers_ = new java.util.Hashtable(1031);
        allProxies_ = new java.util.Hashtable(67);
        offerTables_ = new java.util.Hashtable(67);
        proxyTables_ = new java.util.Hashtable(67);

        if(dbdir != null)
            super.initDatabase(dbdir, "offers");
    }

    //
    // Called by super to execute commands read from database
    // during initialization
    //
    protected void
    execute(org.omg.CORBA.Any any)
    {
        org.omg.CORBA.TypeCode tc = any.type();
        if(tc.equal(NextIdHelper.type()))
        {
            NextId cmd = NextIdHelper.extract(any);
            nextId_ = cmd.id;
        }
        else if(tc.equal(CreateOfferHelper.type()))
        {
            CreateOffer cmd = CreateOfferHelper.extract(any);
            allOffers_.put(cmd.id, cmd.info);
            addOfferIndex(cmd.id, cmd.info);
        }
        else if(tc.equal(ModifyOfferHelper.type()))
        {
            ModifyOffer cmd = ModifyOfferHelper.extract(any);
            OfferInfo info = (OfferInfo)allOffers_.get(cmd.id);
            if(info == null)
            {
                com.ooc.CORBA.MessageViewer.instance().error(
                    "Missing offer '" + cmd.id + "' in database.");
                throw new org.omg.CORBA.PERSIST_STORE();
            }
            info.properties = cmd.props;
        }
        else if(tc.equal(WithdrawOfferHelper.type()))
        {
            WithdrawOffer cmd = WithdrawOfferHelper.extract(any);
            OfferInfo info = (OfferInfo)allOffers_.remove(cmd.id);
            if(info == null)
            {
                com.ooc.CORBA.MessageViewer.instance().error(
                    "Missing offer '" + cmd.id + "' in database.");
                throw new org.omg.CORBA.PERSIST_STORE();
            }
            removeOfferIndex(cmd.id, info);
        }
        else if(tc.equal(CreateProxyHelper.type()))
        {
            CreateProxy cmd = CreateProxyHelper.extract(any);
            allProxies_.put(cmd.id, cmd.info);
            addProxyIndex(cmd.id, cmd.info);
        }
        else if(tc.equal(WithdrawProxyHelper.type()))
        {
            WithdrawProxy cmd = WithdrawProxyHelper.extract(any);
            ProxyInfo info = (ProxyInfo)allProxies_.remove(cmd.id);
            if(info == null)
            {
                com.ooc.CORBA.MessageViewer.instance().error(
                    "Missing proxy offer '" + cmd.id + "' in database.");
                throw new org.omg.CORBA.PERSIST_STORE();
            }
            removeProxyIndex(cmd.id, info);
        }
        else
        {
            com.ooc.CORBA.MessageViewer.instance().error(
                "Unknown record in database.");
            throw new org.omg.CORBA.PERSIST_STORE();
        }
    }

    //
    // Called by super to update the journal
    //
    protected void
    update()
        throws java.io.IOException
    {
        org.omg.CORBA.Any any;
        java.util.Enumeration e;

        //
        // Write the next id
        //
        any = orb_.create_any();
        NextId idCmd = new NextId(nextId_);
        NextIdHelper.insert(any, idCmd);
        writeLog(any);

        //
        // Write the offers
        //
        e = allOffers_.keys();
        while(e.hasMoreElements())
        {
            String id = (String)e.nextElement();
            OfferInfo info = (OfferInfo)allOffers_.get(id);
            CreateOffer cmd = new CreateOffer(id, info);
            any = orb_.create_any();
            CreateOfferHelper.insert(any, cmd);
            writeLog(any);
        }

        //
        // Write the proxies
        //
        e = allProxies_.keys();
        while(e.hasMoreElements())
        {
            String id = (String)e.nextElement();
            ProxyInfo info = (ProxyInfo)allProxies_.get(id);
            CreateProxy cmd = new CreateProxy(id, info);
            any = orb_.create_any();
            CreateProxyHelper.insert(any, cmd);
            writeLog(any);
        }
    }

    //
    // Returns true if the offerId is legal, false otherwise
    //
    public boolean
    validateOfferId(String offerId)
    {
        boolean result = false;

        if(offerId != null)
        {
            //
            // not a very complete check
            //
            int index = offerId.indexOf(ID_SEP);
            result = (index > 0);
        }

        return result;
    }

    //
    // Returns true if the offer with the given ID exists
    //
    public boolean
    exists(String offerId)
    {
        return (allOffers_.containsKey(offerId) ||
                allProxies_.containsKey(offerId));
    }

    //
    // Returns true if the offer with the given ID is a proxy offer
    //
    public boolean
    isProxy(String offerId)
    {
        return allProxies_.containsKey(offerId);
    }

    //
    // Creates a new offer, returning the assigned offer ID
    //
    public String
    create(String serviceType,
           org.omg.CORBA.Object obj,
           org.omg.CosTrading.Property[] props)
    {
        String id = serviceType + ID_SEP + nextId_;
        OfferInfo info = new OfferInfo(obj, serviceType, props);

        if(os_ != null)
        {
            org.omg.CORBA.Any any = orb_.create_any();

            NextId idCmd = new NextId(nextId_ + 1);
            NextIdHelper.insert(any, idCmd);
            writeLog(any);

            CreateOffer cmd = new CreateOffer(id, info);
            CreateOfferHelper.insert(any, cmd);
            writeLog(any);

            update_ = true;
        }

        allOffers_.put(id, info);
        addOfferIndex(id, info);
        nextId_++;

        return id;
    }

    //
    // Creates a new proxy offer, returning the assigned offer ID
    //
    public String
    createProxy(org.omg.CosTrading.Lookup target,
                String serviceType,
                org.omg.CosTrading.Property[] props,
                boolean ifMatchAll,
                String recipe,
                org.omg.CosTrading.Policy[] policies)
    {
        String id = serviceType + ID_SEP + nextId_;
        ProxyInfo info = new ProxyInfo(serviceType, target, props, ifMatchAll,
                                       recipe, policies);

        if(os_ != null)
        {
            org.omg.CORBA.Any any = orb_.create_any();

            NextId idCmd = new NextId(nextId_ + 1);
            NextIdHelper.insert(any, idCmd);
            writeLog(any);

            CreateProxy cmd = new CreateProxy(id, info);
            CreateProxyHelper.insert(any, cmd);
            writeLog(any);

            update_ = true;
        }

        allProxies_.put(id, info);
        addProxyIndex(id, info);
        nextId_++;

        return id;
    }

    //
    // Removes the offer with the given ID
    //
    public void
    remove(String offerId)
    {
        OfferInfo info = (OfferInfo)allOffers_.get(offerId);
        if(info != null)
        {
            if(os_ != null)
            {
                org.omg.CORBA.Any any = orb_.create_any();

                WithdrawOffer cmd = new WithdrawOffer(offerId);
                WithdrawOfferHelper.insert(any, cmd);
                writeLog(any);

                update_ = true;
            }

            allOffers_.remove(offerId);
            removeOfferIndex(offerId, info);
        }
    }

    //
    // Removes the proxy offer with the given ID
    //
    public void
    removeProxy(String offerId)
    {
        ProxyInfo info = (ProxyInfo)allProxies_.get(offerId);
        if(info != null)
        {
            if(os_ != null)
            {
                org.omg.CORBA.Any any = orb_.create_any();

                WithdrawProxy cmd = new WithdrawProxy(offerId);
                WithdrawProxyHelper.insert(any, cmd);
                writeLog(any);

                update_ = true;
            }

            allProxies_.remove(offerId);
            removeProxyIndex(offerId, info);
        }
    }

    //
    // Returns a description of the offer with the given ID, or null if
    // the offer wasn't found
    //
    public org.omg.CosTrading.RegisterPackage.OfferInfo
    describe(String offerId)
    {
        return (OfferInfo)allOffers_.get(offerId);
    }

    //
    // Returns a description of the proxy offer with the given ID, or null
    // if the offer wasn't found
    //
    public org.omg.CosTrading.ProxyPackage.ProxyInfo
    describeProxy(String offerId)
    {
        return (ProxyInfo)allProxies_.get(offerId);
    }

    //
    // Updates the properties of an offer
    //
    public void
    modify(String offerId, org.omg.CosTrading.Property[] props)
    {
        OfferInfo info = (OfferInfo)allOffers_.get(offerId);

        if(info != null)
        {
            if(os_ != null)
            {
                org.omg.CORBA.Any any = orb_.create_any();
                ModifyOffer cmd = new ModifyOffer(offerId, props);
                ModifyOfferHelper.insert(any, cmd);
                writeLog(any);
                update_ = true;
            }

            info.properties = props;
        }
    }

    //
    // Returns all offers of the given service type in a hashtable,
    // where the offer ID is the key and OfferInfo is the value;
    // returns null if the service type does not exist
    //
    public java.util.Hashtable
    getOffers(String serviceType)
    {
        return (java.util.Hashtable)offerTables_.get(serviceType);
    }

    //
    // Returns all offer IDs of the given service type;
    // returns null if the service type does not exist
    //
    public String[]
    getOfferIds(String serviceType)
    {
        String[] result = null;

        java.util.Hashtable h =
            (java.util.Hashtable)offerTables_.get(serviceType);

        if(h != null)
        {
            java.util.Enumeration e = h.keys();
            result = new String[h.size()];
            int i = 0;
            while(e.hasMoreElements())
                result[i++] = (String)e.nextElement();
        }

        return result;
    }

    //
    // Returns all proxy offers of the given service type in a hashtable,
    // where the offer ID is the key and ProxyInfo is the value;
    // returns null if the service type does not exist
    //
    public java.util.Hashtable
    getProxyOffers(String serviceType)
    {
        return (java.util.Hashtable)proxyTables_.get(serviceType);
    }

    //
    // Returns all proxy offer IDs of the given service type;
    // returns null if the service type does not exist
    //
    public String[]
    getProxyOfferIds(String serviceType)
    {
        String[] result = null;

        java.util.Hashtable h =
            (java.util.Hashtable)proxyTables_.get(serviceType);

        if(h != null)
        {
            java.util.Enumeration e = h.keys();
            result = new String[h.size()];
            int i = 0;
            while(e.hasMoreElements())
                result[i++] = (String)e.nextElement();
        }

        return result;
    }

    //
    // Returns the service type of the given offer;
    // returns null if the offer ID is malformed
    //
    public String
    whichService(String offerId)
    {
        String result = null;

        int index = offerId.indexOf(ID_SEP);
        if(index > 0)
            result = offerId.substring(0, index);

        return result;
    }

    private void
    addOfferIndex(String id, OfferInfo info)
    {
        java.util.Hashtable h =
            (java.util.Hashtable)offerTables_.get(info.type);
        if(h == null)
        {
            h = new java.util.Hashtable(131);
            offerTables_.put(info.type, h);
        }
        h.put(id, info);
    }

    private void
    removeOfferIndex(String id, OfferInfo info)
    {
        java.util.Hashtable h =
            (java.util.Hashtable)offerTables_.get(info.type);
        h.remove(id);
    }

    private void
    addProxyIndex(String id, ProxyInfo info)
    {
        java.util.Hashtable h =
            (java.util.Hashtable)proxyTables_.get(info.type);
        if(h == null)
        {
            h = new java.util.Hashtable(131);
            proxyTables_.put(info.type, h);
        }
        h.put(id, info);
    }

    private void
    removeProxyIndex(String id, ProxyInfo info)
    {
        java.util.Hashtable h =
            (java.util.Hashtable)proxyTables_.get(info.type);
        h.remove(id);
    }
}
