// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Things are just too ugly without this
//
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

//
// Convenience methods
//
final class OfferUtil
{
    //
    // Validate the properties of a service offer against the service type
    //
    static void
    validateProperties(DBManager dbMgr,
                       org.omg.CosTrading.Property[] exportProps,
                       String typeName,
                       TypeStruct type,
                       boolean allowDynamic)
        throws org.omg.CosTrading.IllegalPropertyName,
               org.omg.CosTrading.PropertyTypeMismatch,
               org.omg.CosTrading.ReadonlyDynamicProperty,
               org.omg.CosTrading.MissingMandatoryProperty,
               org.omg.CosTrading.DuplicatePropertyName
    {
        //
        // Create a hashtable of the service type's properties
        //
        java.util.Hashtable typeProps = new java.util.Hashtable();
        for(int i = 0 ; i < type.props.length ; i++)
            typeProps.put(type.props[i].name, type.props[i]);

        //
        // Also create a hashtable of the exported properties we've processed
        //
        java.util.Hashtable checkedProps = new java.util.Hashtable();

        //
        // Iterate through each of the exported properties
        //
        for(int i = 0 ; i < exportProps.length ; i++)
        {
            //
            // Try to find the property with the same name in the service type
            //
            PropStruct ps = (PropStruct)typeProps.get(exportProps[i].name);

            //
            // Make sure we haven't already processed this property
            //
            if(checkedProps.containsKey(exportProps[i].name))
                throw new org.omg.CosTrading.
                          DuplicatePropertyName(exportProps[i].name);

            //
            // If no PropStruct was found, then we need to make sure
            // that we support "unknown" properties
            //
            if(ps == null && !dbMgr.allowUnknownProperties())
                throw new org.omg.CosTrading.
                          PropertyTypeMismatch(typeName, exportProps[i]);

            checkProperty(dbMgr, typeName, exportProps[i], ps, allowDynamic);
            checkedProps.put(exportProps[i].name, exportProps[i]);
        }

        //
        // Now we need to check for mandatory properties that were not
        // included in the export list
        //
        java.util.Enumeration e = typeProps.elements();
        while(e.hasMoreElements())
        {
            PropStruct ps = (PropStruct)e.nextElement();

            //
            // If property is not being exported...
            //
            if(checkedProps.get(ps.name) == null)
            {
                if(isMandatory(ps.mode))
                    throw new org.omg.CosTrading.
                              MissingMandatoryProperty(typeName, ps.name);
            }
        }
    }

    public static boolean
    isMandatory(PropertyMode mode)
    {
        boolean result;

        result = (mode == PropertyMode.PROP_MANDATORY ||
                  mode == PropertyMode.PROP_MANDATORY_READONLY);

        return result;
    }

    public static boolean
    isReadonly(PropertyMode mode)
    {
        boolean result;

        result = (mode == PropertyMode.PROP_READONLY ||
                  mode == PropertyMode.PROP_MANDATORY_READONLY);

        return result;
    }

    //
    // Check the property's type against the expected type
    //
    public static void
    checkProperty(DBManager dbMgr,
                  String typeName,
                  org.omg.CosTrading.Property prop,
                  PropStruct ps,
                  boolean allowDynamic)
        throws org.omg.CosTrading.IllegalPropertyName,
               org.omg.CosTrading.PropertyTypeMismatch,
               org.omg.CosTrading.ReadonlyDynamicProperty
    {
        org.omg.CosTradingDynamic.DynamicProp dynProp = null;
        org.omg.CORBA.TypeCode propType = prop.value.type();
        org.omg.CORBA.TypeCode origType = Value.getOrigType(propType);

        if(!dbMgr.isPropertyNameValid(prop.name))
            throw new org.omg.CosTrading.IllegalPropertyName(prop.name);

        //
        // Whether or not the property is defined in the service type,
        // we need to make sure the database can store this property;
        // the PropertyTypeMismatch exception is about the best we can do.
        //
        // NOTE: For dynamic properties, we may not be able to store
        //   the extra_info member of the DynamicProp struct;
        //   we assume the database manager also validates extra_info.
        //

        if(!dbMgr.isPropertyTypeSupported(propType))
            throw new org.omg.CosTrading.PropertyTypeMismatch(typeName, prop);

        //
        // If it's a dynamic property then extract the DynamicProp struct
        //
        if(PropUtil.isDynamicProperty(propType))
        {
            dynProp =
                org.omg.CosTradingDynamic.DynamicPropHelper.extract(prop.value);

            //
            // Make sure we allow dynamic properties and that the eval_if
            // object is not nil
            //
            if(!allowDynamic || dynProp.eval_if == null)
                throw new org.omg.CosTrading.
                          PropertyTypeMismatch(typeName, prop);
        }

        //
        // Check the property against the service type definition
        //
        if(ps != null)
        {
            org.omg.CORBA.TypeCode valueType =
                Value.getOrigType(ps.value_type);

            if(dynProp != null)
            {
                //
                // Cannot allow dynamic readonly properties
                //
                if(isReadonly(ps.mode))
                    throw new org.omg.CosTrading.
                              ReadonlyDynamicProperty(typeName, prop.name);

                //
                // Compare the original type of the dynamic property's
                // returned_type against the type specified by the
                // service type
                //
                org.omg.CORBA.TypeCode tc =
                    Value.getOrigType(dynProp.returned_type);

                if(!tc.equal(valueType))
                    throw new org.omg.CosTrading.
                              PropertyTypeMismatch(typeName, prop);
            }
            else
            {
                //
                // Verify the property's type code against that
                // specified by the service type
                //
                if(!origType.equal(valueType))
                    throw new org.omg.CosTrading.
                              PropertyTypeMismatch(typeName, prop);
            }
        }
    }
}
