// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

final class Parser
{
    private org.omg.CORBA.ORB orb_;
    private Scanner scan_;

    private Node
    bool_or()
    {
        Node result = bool_and();

        while(result != null && match("or"))
        {
            scan_.next();
            Node right = bool_and();

            if(right != null)
                result = new BinaryNode(orb_, Tokens.TOK_OR, result, right);
            else
                result = null;
        }

        return result;
    }

    private Node
    bool_and()
    {
        Node result = bool_compare();

        while(result != null && match("and"))
        {
            scan_.next();
            Node right = bool_compare();

            if(right != null)
                result = new BinaryNode(orb_, Tokens.TOK_AND, result, right);
            else
                result = null;
        }

        return result;
    }

    private Node
    bool_compare()
    {
        Node result = expr_in();

        if(result != null)
        {
            int token = scan_.token();

            switch(token)
            {
            case Tokens.TOK_EQ:
            case Tokens.TOK_NE:
            case Tokens.TOK_GE:
            case Tokens.TOK_LE:
            case Tokens.TOK_LT:
            case Tokens.TOK_GT:
            {
                scan_.next();
                Node right = expr_in();
                if(right != null)
                    result = new BinaryNode(orb_, token, result, right);
                else
                    result = null;
                break;
            }

            default:
                break;
            }
        }

        return result;
    }

    private Node
    expr_in()
    {
        Node result = expr_twiddle();

        if(result != null)
        {
            if(match("in"))
            {
                Node right;
                int token = scan_.token();
                token = scan_.next();

                right = ident();

                if(right != null)
                    result = new BinaryNode(orb_, Tokens.TOK_IN, result,
                                            right);
                else
                    result = null;
            }
        }

        return result;
    }

    private Node
    expr_twiddle()
    {
        Node result = expr();

        if(result != null)
        {
            if(scan_.token() == Tokens.TOK_TWIDDLE)
            {
                scan_.next();
                Node right = expr();

                if(right != null)
                    result = new BinaryNode(orb_, Tokens.TOK_TWIDDLE, result,
                                            right);
            }
        }

        return result;
    }

    private Node
    expr()
    {
        Node result = term();

        if(result != null)
        {
            while(result != null &&
                  (scan_.token() == Tokens.TOK_PLUS ||
                   scan_.token() == Tokens.TOK_MINUS))
            {
                int token = scan_.token();

                scan_.next();
                Node right = term();

                if(right != null)
                    result = new BinaryNode(orb_, token, result, right);
                else
                    result = null;
            }
        }

        return result;
    }

    private Node
    term()
    {
        Node result = factor_not();

        if(result != null)
        {
            while(result != null &&
                  (scan_.token() == Tokens.TOK_MULT ||
                   scan_.token() == Tokens.TOK_DIV))
            {
                int token = scan_.token();

                scan_.next();
                Node right = factor_not();

                if(right != null)
                    result = new BinaryNode(orb_, token, result, right);
                else
                    result = null;
            }
        }

        return result;
    }

    private Node
    factor_not()
    {
        Node result = null;

        if(match("not"))
        {
            scan_.next();
            Node n = factor();

            if(n != null)
                result = new UnaryNode(orb_, Tokens.TOK_NOT, n);
        }
        else
            result = factor();

        return result;
    }

    private Node
    factor()
    {
        Node result = null;

        int token = scan_.token();

        if(token == Tokens.TOK_LPAREN)
        {
            scan_.next();
            result = bool_or();
            if(result != null && scan_.token() == Tokens.TOK_RPAREN)
                scan_.next();
            else
                result = null;
        }
        else if(match("exist"))
        {
            Node n;

            scan_.next();
            n = ident();

            if(n != null)
                result = new UnaryNode(orb_, Tokens.TOK_EXIST, n);
        }
        else if(match("TRUE"))
        {
            scan_.next();
            result = new LiteralNode(orb_, true);
        }
        else if(match("FALSE"))
        {
            scan_.next();
            result = new LiteralNode(orb_, false);
        }
        else if(token == Tokens.TOK_DOT || token == Tokens.TOK_DIGITS)
        {
            result = number();
        }
        else if(token == Tokens.TOK_MINUS || token == Tokens.TOK_PLUS)
        {
            scan_.next();
            Node n = number();

            if(n != null)
                result = new UnaryNode(orb_, token, n);
        }
        else if(token == Tokens.TOK_STRING)
        {
            result = string();
        }
        else if(token == Tokens.TOK_IDENT)
        {
            result = ident();
        }

        return result;
    }

    private Node
    ident()
    {
        Node result = null;

        if(scan_.token() == Tokens.TOK_IDENT)
        {
            result = new IdentNode(orb_, scan_.lexeme());
            scan_.next();
        }

        return result;
    }

    private Node
    number()
    {
        StringBuffer buf = new StringBuffer();

        boolean b = mantissa(buf);

        if(b)
            b = exponent(buf);

        if(b)
        {
            String num = buf.toString();

            if(num.indexOf('.') >= 0 ||
               num.indexOf('e') >= 0 ||
               num.indexOf('E') >= 0)
            {
                try
                {
                    double d = Double.valueOf(num).doubleValue();
                    return new LiteralNode(orb_, d);
                }
                catch(NumberFormatException ex)
                {
                    // ignore
                }
            }
            else
            {
                try
                {
                    int l = Integer.parseInt(num);
                    return new LiteralNode(orb_, l);
                }
                catch(NumberFormatException ex)
                {
                }
            }
        }

        return null;
    }

    private Node
    string()
    {
        Node result = null;

        if(scan_.token() == Tokens.TOK_STRING)
        {
            result = new LiteralNode(orb_, scan_.lexeme());
            scan_.next();
        }

        return result;
    }

    private boolean
    mantissa(StringBuffer num)
    {
        if(scan_.token() != Tokens.TOK_DIGITS &&
           scan_.token() != Tokens.TOK_DOT)
            return false;

        if(scan_.token() == Tokens.TOK_DIGITS)
        {
            num.append(scan_.lexeme());
            scan_.next();
        }

        if(scan_.token() == Tokens.TOK_DOT)
        {
            num.append(scan_.lexeme());
            scan_.next();

            if(scan_.token() == Tokens.TOK_DIGITS)
            {
                num.append(scan_.lexeme());
                scan_.next();
            }
        }

        return true;
    }

    private boolean
    exponent(StringBuffer num)
    {
        boolean b = true;

        if(scan_.token() == Tokens.TOK_IDENT)
        {
            if(scan_.lexeme().equalsIgnoreCase("e"))
            {
                num.append(scan_.lexeme());
                scan_.next();
                if(scan_.token() == Tokens.TOK_PLUS ||
                   scan_.token() == Tokens.TOK_MINUS)
                {
                    num.append(scan_.lexeme());
                    scan_.next();

                    if(scan_.token() == Tokens.TOK_DIGITS)
                    {
                        num.append(scan_.lexeme());
                        scan_.next();
                    }
                    else
                        b = false;
                }
                else
                    b = false;
            }
        }

        return b;
    }

    private boolean
    match(String str)
    {
        boolean result = false;

        if(scan_.token() == Tokens.TOK_KEYWORD)
            result = str.equals(scan_.lexeme());

        return result;
    }

    Parser(org.omg.CORBA.ORB orb)
    {
        orb_ = orb;
    }

    Node
    parse(Scanner scan)
    {
        scan_ = scan;

        Node result = bool_or();

        if(scan_.token() != Tokens.TOK_END)
            result = null;

        return result;
    }
}
