// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

final class Preference
{
    private org.omg.CORBA.ORB orb_;
    private Node node_;
    private int token_;

    class PrefValueInfo
    {
        PrefValueInfo(Value v, int p)
        {
            val = v;
            pos = p;
        }

        Value val;
        int pos;
    }

    private void
    quicksort(PrefValueInfo[] arr, int l, int h)
    {
        int low = l;
        int hi = h;
        PrefValueInfo mid;

        if(h > l)
        {
            mid = arr[(l + h) / 2];

            while(low <= hi)
            {
                if(token_ == Tokens.TOK_MIN)
                {
                    while(low < h && arr[low].val.lessThan(mid.val))
                        low += 1;

                    while(hi > l && arr[hi].val.greaterThan(mid.val))
                        hi -= 1;
                }
                else
                {
                    while(low < h && arr[low].val.greaterThan(mid.val))
                        low += 1;

                    while(hi > l && arr[hi].val.lessThan(mid.val))
                        hi -= 1;
                }

                if(low <= hi)
                {
                    PrefValueInfo tmp = arr[low];
                    arr[low] = arr[hi];
                    arr[hi] = tmp;

                    low += 1;
                    hi -= 1;
                }
            }

            if(l < hi)
                quicksort(arr, l, hi);

            if(low < h)
                quicksort(arr, low, h);
        }
    }

    public
    Preference(org.omg.CORBA.ORB orb)
    {
        orb_ = orb;
        node_ = null;
        token_ = 0;
    }

    public boolean
    parse(String pref)
    {
        Scanner scan = new Scanner(pref);
        int token = scan.token();

        if(token == Tokens.TOK_KEYWORD)
        {
            if(scan.lexeme().equals("first"))
            {
                token_ = Tokens.TOK_FIRST;
                scan.next();
                if(scan.token() == Tokens.TOK_END)
                    return true;
            }
            else if(scan.lexeme().equals("random"))
            {
                token_ = Tokens.TOK_RANDOM;
                scan.next();
                if(scan.token() == Tokens.TOK_END)
                    return true;
            }
            else if(scan.lexeme().equals("with"))
            {
                token_ = Tokens.TOK_WITH;
                scan.next();
                Parser parser = new Parser(orb_);
                Node node = parser.parse(scan);

                if(node != null && node.checkTypes() &&
                   (node.type() == Node.TYPE_UNKNOWN ||
                    node.type() == Node.TYPE_BOOLEAN))
                {
                    node_ = node;
                    return true;
                }
            }
            else if(scan.lexeme().equals("min"))
            {
                token_ = Tokens.TOK_MIN;
                scan.next();
                Parser parser = new Parser(orb_);
                Node node = parser.parse(scan);

                if(node != null && node.checkTypes() &&
                   (node.type() == Node.TYPE_UNKNOWN ||
                    node.type() == Node.TYPE_NUMBER))
                {
                    node_ = node;
                    return true;
                }
            }
            else if(scan.lexeme().equals("max"))
            {
                token_ = Tokens.TOK_MAX;
                scan.next();
                Parser parser = new Parser(orb_);
                Node node = parser.parse(scan);

                if(node != null && node.checkTypes() &&
                   (node.type() == Node.TYPE_UNKNOWN ||
                    node.type() == Node.TYPE_NUMBER))
                {
                    node_ = node;
                    return true;
                }
            }
        }

        return false;
    }

    public boolean
    first()
    {
        return (token_ == Tokens.TOK_FIRST);
    }

    public PropertyList[]
    sort(PropertyList[] seq)
    {
        PropertyList[] result = new PropertyList[seq.length];

        int i, j;

        switch(token_)
        {
        case Tokens.TOK_RANDOM:
        {
            java.util.Random rand = new java.util.Random();
            java.util.BitSet flags = new java.util.BitSet(seq.length);

            //
            // For each element of the source array, select a random position
            // in the target array. If the position is already occupied in
            // the target array, search until an empty position is found.
            //
            for(i = 0 ; i < seq.length ; i++)
            {
                int pos = Math.abs(rand.nextInt()) % seq.length;
                while(flags.get(pos))
                {
                    pos = (pos + 1) % seq.length;
                }
                flags.set(pos);
                result[pos] = seq[i];
            }

            break;
        }

        case Tokens.TOK_WITH:
        {
            if(node_ == null)
                throw new RuntimeException();

            java.util.BitSet matches = new java.util.BitSet(seq.length);

            //
            // First evaluate every offer against the preference constraint
            //
            for(i = 0 ; i < seq.length ; i++)
            {
                Value v = node_.evaluate(seq[i]);

                if(v != null && v.kind() == Value.KIND_BOOLEAN &&
                   v.get_boolean())
                    matches.set(i);
            }

            //
            // Now copy all matches
            //
            j = 0;
            for(i = 0 ; i < seq.length ; i++)
            {
                if(matches.get(i))
                {
                    result[j] = seq[i];
                    j++;
                }
            }

            //
            // Finally, copy all remaining offers
            //
            for(i = 0 ; i < seq.length ; i++)
            {
                if(!matches.get(i))
                {
                    result[j] = seq[i];
                    j++;
                }
            }

            if(j != seq.length)
                throw new RuntimeException();

            break;
        }

        case Tokens.TOK_MIN:
        case Tokens.TOK_MAX:
        {
            if(node_ == null)
                throw new RuntimeException();

            PrefValueInfo[] values = new PrefValueInfo[seq.length];
            java.util.BitSet matches = new java.util.BitSet(seq.length);

            //
            // First evaluate every offer against the preference constraint
            //
            int count = 0;
            for(i = 0 ; i < seq.length ; i++)
            {
                Value v = node_.evaluate(seq[i]);

                if(v != null && v.numeric())
                {
                    matches.set(i);
                    values[count++] = new PrefValueInfo(v, i);
                }
            }

            //
            // Sort the values in the appropriate order
            //
            quicksort(values, 0, count - 1);

            //
            // Copy the matching offers
            //
            for(i = 0 ; i < count ; i++)
                result[i] = seq[values[i].pos];

            //
            // Finally, copy all remaining offers
            //
            j = count;
            for(i = 0 ; i < seq.length ; i++)
            {
                if(!matches.get(i))
                {
                    result[j] = seq[i];
                    j++;
                }
            }

            break;
        }

        default:
            throw new RuntimeException();
        }

        return result;
    }
}
