// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Things are just too ugly without this
//
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

//
// Implementation of CosTrading::Proxy
//
final class ProxyImpl extends org.omg.CosTrading._ProxyImplBase
{
    private DBManager dbMgr_;
    private OfferDB db_;
    private org.omg.CosTrading.Admin admin_;

    ProxyImpl(DBManager dbMgr)
    {
        dbMgr_ = dbMgr;
        db_ = dbMgr.getOfferDB();
    }

    void
    setAdmin(org.omg.CosTrading.Admin admin)
    {
        admin_ = admin;
    }

    //
    // Operations inherited from CosTrading::TraderComponents
    //

    public org.omg.CosTrading.Lookup
    lookup_if()
    {
        return admin_.lookup_if();
    }

    public org.omg.CosTrading.Register
    register_if()
    {
        return admin_.register_if();
    }

    public org.omg.CosTrading.Link
    link_if()
    {
        return admin_.link_if();
    }

    public org.omg.CosTrading.Proxy
    proxy_if()
    {
        return this;
    }

    public org.omg.CosTrading.Admin
    admin_if()
    {
        return admin_;
    }

    //
    // Operations inherited from CosTrading::SupportAttributes
    //

    public boolean
    supports_modifiable_properties()
    {
        return admin_.supports_modifiable_properties();
    }

    public boolean
    supports_dynamic_properties()
    {
        return admin_.supports_dynamic_properties();
    }

    public boolean
    supports_proxy_offers()
    {
        return admin_.supports_proxy_offers();
    }

    public org.omg.CORBA.Object
    type_repos()
    {
        return admin_.type_repos();
    }

    //
    // Operations inherited from CosTrading::Proxy
    //

    public String
    export_proxy(org.omg.CosTrading.Lookup target,
                 String type,
                 org.omg.CosTrading.Property[] properties,
                 boolean if_match_all,
                 String recipe,
                 org.omg.CosTrading.Policy[] policies_to_pass_on)
        throws org.omg.CosTrading.IllegalServiceType,
               org.omg.CosTrading.UnknownServiceType,
               org.omg.CosTrading.InvalidLookupRef,
               org.omg.CosTrading.IllegalPropertyName,
               org.omg.CosTrading.PropertyTypeMismatch,
               org.omg.CosTrading.ReadonlyDynamicProperty,
               org.omg.CosTrading.MissingMandatoryProperty,
               org.omg.CosTrading.ProxyPackage.IllegalRecipe,
               org.omg.CosTrading.DuplicatePropertyName,
               org.omg.CosTrading.DuplicatePolicyName
    {
        String result = null;

        if(target == null)
            throw new org.omg.CosTrading.InvalidLookupRef(target);

        org.omg.CORBA.Object obj = type_repos();
        ServiceTypeRepository repos = ServiceTypeRepositoryHelper.narrow(obj);

        //
        // Retrieve complete information about the service type from the
        // repository - may throw IllegalServiceType, UnknownServiceType
        //
        TypeStruct ts = repos.fully_describe_type(type);

        //
        // Do not allow exporting for a masked service type
        //
        if(ts.masked)
            throw new org.omg.CosTrading.UnknownServiceType(type);

        //
        // Validate the exported properties - may throw
        // IllegalPropertyName, PropertyTypeMismatch,
        // MissingMandatoryProperty, DuplicatePropertyName
        //
        OfferUtil.validateProperties(dbMgr_, properties, type, ts,
                                     supports_dynamic_properties());

        //
        // Validate the recipe
        //
        if(!Recipe.validate(recipe, properties))
            throw new org.omg.CosTrading.ProxyPackage.IllegalRecipe(recipe);

        //
        // Check for duplicate policies and policies with invalid names
        // or unsupported types; there are no exceptions declared that
        // we can throw for invalid policy names or unsupported policy
        // value types, so we just ignore them
        //
        java.util.Hashtable policyTable = new java.util.Hashtable();
        java.util.Vector vec = new java.util.Vector();
        for(int i = 0 ; i < policies_to_pass_on.length ; i++)
        {
            org.omg.CosTrading.Policy p = policies_to_pass_on[i];
            if(policyTable.containsKey(p.name))
                throw new org.omg.CosTrading.DuplicatePolicyName(p.name);
            else
            {
                policyTable.put(p.name, p);
                if(dbMgr_.isPolicyNameValid(p.name) &&
                   dbMgr_.isPolicyTypeSupported(p.value.type()))
                    vec.addElement(p);
            }
        }

        //
        // These are the valid policies
        //
        org.omg.CosTrading.Policy[] validPolicies =
            new org.omg.CosTrading.Policy[vec.size()];
        vec.copyInto(validPolicies);

        //
        // Save the offer in the database
        //

        db_.begin(AbstractDB.WRITE);

        try
        {
            result = db_.createProxy(target, type, properties, if_match_all,
                                     recipe, validPolicies);
        }
        finally
        {
            db_.end();
        }

        return result;
    }

    public void
    withdraw_proxy(String id)
        throws org.omg.CosTrading.IllegalOfferId,
               org.omg.CosTrading.UnknownOfferId,
               org.omg.CosTrading.ProxyPackage.NotProxyOfferId
    {
        if(!db_.validateOfferId(id))
            throw new org.omg.CosTrading.IllegalOfferId(id);

        db_.begin(AbstractDB.WRITE);

        try
        {
            if(!db_.exists(id))
                throw new org.omg.CosTrading.UnknownOfferId(id);

            if(!db_.isProxy(id))
                throw new org.omg.CosTrading.ProxyPackage.NotProxyOfferId(id);

            db_.removeProxy(id);
        }
        finally
        {
            db_.end();
        }
    }

    public org.omg.CosTrading.ProxyPackage.ProxyInfo
    describe_proxy(String id)
        throws org.omg.CosTrading.IllegalOfferId,
               org.omg.CosTrading.UnknownOfferId,
               org.omg.CosTrading.ProxyPackage.NotProxyOfferId
    {
        org.omg.CosTrading.ProxyPackage.ProxyInfo result;

        if(!db_.validateOfferId(id))
            throw new org.omg.CosTrading.IllegalOfferId(id);

        db_.begin(AbstractDB.READ);

        try
        {
            if(!db_.exists(id))
                throw new org.omg.CosTrading.UnknownOfferId(id);

            if(!db_.isProxy(id))
                throw new org.omg.CosTrading.ProxyPackage.NotProxyOfferId(id);

            result = db_.describeProxy(id);
        }
        finally
        {
            db_.end();
        }

        return result;
    }
}
