// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Things are just too ugly without this
//
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

//
// Handles a single query operation
//
final class Query implements ThreadPool.Worker
{
    //
    // arguments
    //
    private org.omg.CORBA.ORB orb_;
    private org.omg.CosTrading.Lookup lookup_;
    private ServiceTypeRepository repos_;
    private OfferDB db_;
    private RequestHistory requestHistory_;
    private String type_;
    private String constraint_;
    private String preference_;
    private org.omg.CosTrading.Policy[] policies_;
    private java.util.Hashtable policyTable_;
    private org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps_;

    private org.omg.CosTrading.Policy[] linkPolicies_;
    private java.util.Vector limitsApplied_;
    private TypeStruct typeStruct_ = null;
    private Constraint constraintObj_ = null;
    private Preference preferenceObj_ = null;

    //
    // limiting policies
    //
    private int searchCard_;
    private int matchCard_;
    private int returnCard_;
    private int hopCount_;
    private boolean exactType_;
    private boolean useDynamic_;
    private boolean useModifiable_;
    private boolean useProxyOffers_;
    private org.omg.CosTrading.FollowOption linkFollowRule_;

    private java.util.Vector matchingOffers_ = new java.util.Vector();
    private int localMatchCount_;
    private PropertyList[] offers_;

    //
    // for processing links
    //
    private ThreadPool threadPool_;
    private int numLinks_ = 0;
    private int linksProcessed_ = 0;


    public
    Query(org.omg.CORBA.ORB orb,
          org.omg.CosTrading.Lookup lookup,
          ServiceTypeRepository repos,
          OfferDB db,
          RequestHistory requestHistory,
          String type,
          String constr,
          String pref,
          org.omg.CosTrading.Policy[] policies,
          java.util.Hashtable policyTable,
          org.omg.CosTrading.LookupPackage.SpecifiedProps desired_props)
    {
        orb_ = orb;
        lookup_ = lookup;
        repos_ = repos;
        db_ = db;
        requestHistory_ = requestHistory;
        type_ = type;
        constraint_ = constr;
        preference_ = pref;
        policies_ = policies;
        policyTable_ = policyTable;
        desiredProps_ = desired_props;

        limitsApplied_ = new java.util.Vector();
    }

    //
    // Performs a Lookup::query operation
    //
    public void
    query()
        throws org.omg.CosTrading.IllegalServiceType,
               org.omg.CosTrading.UnknownServiceType,
               org.omg.CosTrading.IllegalConstraint,
               org.omg.CosTrading.LookupPackage.IllegalPreference,
               org.omg.CosTrading.LookupPackage.PolicyTypeMismatch,
               org.omg.CosTrading.LookupPackage.InvalidPolicyValue,
               org.omg.CosTrading.IllegalPropertyName,
               org.omg.CosTrading.DuplicatePropertyName
    {
        boolean unknownType = false;

        //
        // Determine our limiting policies
        //
        collectLimitingPolicies();

        //
        // Search locally - if UnknownServiceType is thrown, we may
        // still want to forward the query to linked traders
        //
        try
        {
            initialize();
            searchLocal();
        }
        catch(org.omg.CosTrading.UnknownServiceType ex)
        {
            unknownType = true;
        }

        //
        // Consider forwarding the query to our links if:
        //
        // a) we support the Link interface
        // b) we have not already met our match limits
        // c) the search is not limited to the local offer space
        // d) the hop count is greater than 0
        //

        if(lookup_.link_if() != null &&
           localMatchCount_ < matchCard_ &&
           linkFollowRule_ != org.omg.CosTrading.FollowOption.local_only &&
           hopCount_ > 0)
            processLinks();

        if(matchingOffers_.size() == 0 && unknownType)
            throw new org.omg.CosTrading.UnknownServiceType(type_);

        //
        // Order the matching offers using the preference - the preference
        // may not exist if the service type was unknown, in which case
        // we just return the offers unordered
        //
        PropertyList[] matching = new PropertyList[matchingOffers_.size()];
        matchingOffers_.copyInto(matching);
        if(preferenceObj_ != null && !preferenceObj_.first())
            offers_ = preferenceObj_.sort(matching);
        else
            offers_ = matching;
    }

    //
    // Retrieve the results of the query operation
    //
    public void
    getResults(int how_many,
               org.omg.CosTrading.OfferSeqHolder offers,
               org.omg.CosTrading.OfferIteratorHolder offer_itr,
               org.omg.CosTrading.PolicyNameSeqHolder limits_applied)
    {
        //
        // Determine how many offers to return
        //
        int returnCount = Math.min(offers_.length, returnCard_);
        int seqCount = Math.min(returnCount, how_many);
        if(seqCount > lookup_.max_list())
            seqCount = lookup_.max_list();

        offers.value = new org.omg.CosTrading.Offer[seqCount];
        int i, count = 0;
        for(i = 0 ; i < offers_.length && count < seqCount ; i++)
        {
            offers.value[count] = new org.omg.CosTrading.Offer();
            offers.value[count].reference = offers_[i].getObject();
            offers.value[count].properties =
                offers_[i].getProperties(desiredProps_);
            count++;
        }

        //
        // Construct an iterator if necessary
        //
        if(seqCount < returnCount)
        {
            //
            // Create a sequence holding the remaining offers
            //
            int restCount = returnCount - seqCount;
            org.omg.CosTrading.Offer[] rest =
                new org.omg.CosTrading.Offer[restCount];

            int pos = 0;
            for(; i < offers_.length && pos < restCount ; i++)
            {
                rest[pos] = new org.omg.CosTrading.Offer();
                rest[pos].reference = offers_[i].getObject();
                rest[pos].properties = offers_[i].getProperties(desiredProps_);
                pos++;
            }

            OfferIteratorImpl iter =
                new OfferIteratorImpl(orb_, rest, lookup_.max_list());
            orb_.connect(iter);
            offer_itr.value = iter;
        }

        limits_applied.value = new String[limitsApplied_.size()];
        limitsApplied_.copyInto((java.lang.Object[])limits_applied.value);
    }

    private void
    initialize()
        throws org.omg.CosTrading.IllegalServiceType,
               org.omg.CosTrading.UnknownServiceType,
               org.omg.CosTrading.IllegalConstraint,
               org.omg.CosTrading.LookupPackage.IllegalPreference
    {
        //
        // Retrieve complete information about the service type from the
        // repository - may throw IllegalServiceType, UnknownServiceType
        //
        typeStruct_ = repos_.fully_describe_type(type_);

        //
        // Check constraint - an empty constraint is equivalent to "TRUE"
        //
        if(constraint_ == null)
            throw new org.omg.CosTrading.IllegalConstraint("");
        else if(constraint_.trim().length() == 0)
            constraint_ = "TRUE";

        //
        // If no preference is supplied, "first" is the default
        //
        if(preference_ == null || preference_.trim().length() == 0)
            preference_ = "first";

        constraintObj_ = new Constraint(orb_);
        preferenceObj_ = new Preference(orb_);

        //
        // Attempt to parse the constraint
        //
        if(!constraintObj_.parse(constraint_))
            throw new org.omg.CosTrading.IllegalConstraint(constraint_);

        //
        // Attempt to parse the preference
        //
        if(!preferenceObj_.parse(preference_))
            throw new org.omg.CosTrading.LookupPackage.
                      IllegalPreference(preference_);
    }

    private void
    searchLocal()
        throws org.omg.CosTrading.LookupPackage.PolicyTypeMismatch,
               org.omg.CosTrading.LookupPackage.InvalidPolicyValue,
               org.omg.CosTrading.IllegalPropertyName,
               org.omg.CosTrading.DuplicatePropertyName
    {
        //
        // Compose a list of all the service types we will consider;
        // we start with just the type specified
        //
        java.util.Vector types = new java.util.Vector();
        types.addElement(type_);

        //
        // If the client wishes us to consider subtypes, then we need to
        // find all service types "compatible" with the requested type
        // (i.e. the type itself, and any subtypes)
        //
        if(!exactType_)
            findCompatibleTypes(type_, types);

        //
        // collect the set of potential offers
        //
        java.util.Vector potentialOffers = null;

        try
        {
            db_.begin(AbstractDB.READ);
            potentialOffers = collectPotentialOffers(types, typeStruct_);
        }
        finally
        {
            db_.end();
        }

        //
        // Instantiate an object to evaluate the offers
        //
        OfferEvaluator eval =
            new OfferEvaluator(orb_, type_, constraint_, constraintObj_,
                               preference_, policies_, desiredProps_,
                               limitsApplied_, potentialOffers, matchCard_);

        //
        // Retrieve the results of the offer evaluation;
        // this will block until all offers have been evaluated or
        // until the match cardinality limits are reached; the
        // matchingOffers vector contains only PropertyList objects
        // (i.e. no ProxyPropertyList objects)
        //
        matchingOffers_ = eval.getResults();
        localMatchCount_ = matchingOffers_.size();
    }

    //
    // Examines any supplied policies and compares them against the
    // trader's limiting policies, updating limitsApplied_ if any
    // policy values were overridden
    //
    private void
    collectLimitingPolicies()
        throws org.omg.CosTrading.LookupPackage.PolicyTypeMismatch,
               org.omg.CosTrading.LookupPackage.InvalidPolicyValue
    {
        if(policyTable_.containsKey("search_card"))
        {
            searchCard_ = getULongPolicyValue(policyTable_, "search_card");
            if(searchCard_ > lookup_.max_search_card())
            {
                searchCard_ = lookup_.max_search_card();
                limitsApplied_.addElement("max_search_card");
            }
        }
        else
            searchCard_ = lookup_.def_search_card();

        if(policyTable_.containsKey("match_card"))
        {
            matchCard_ = getULongPolicyValue(policyTable_, "match_card");
            if(matchCard_ > lookup_.max_match_card())
            {
                matchCard_ = lookup_.max_match_card();
                limitsApplied_.addElement("max_match_card");
            }
        }
        else
            matchCard_ = lookup_.def_match_card();

        if(policyTable_.containsKey("return_card"))
        {
            returnCard_ = getULongPolicyValue(policyTable_, "return_card");
            if(returnCard_ > lookup_.max_return_card())
            {
                returnCard_ = lookup_.max_return_card();
                limitsApplied_.addElement("max_return_card");
            }
        }
        else
            returnCard_ = lookup_.def_return_card();

        if(policyTable_.containsKey("hop_count"))
        {
            hopCount_ = getULongPolicyValue(policyTable_, "hop_count");
            if(hopCount_ > lookup_.max_hop_count())
            {
                hopCount_ = lookup_.max_hop_count();
                limitsApplied_.addElement("max_hop_count");
            }
        }
        else
            hopCount_ = lookup_.def_hop_count();

        //
        // Determine whether we restrict search to exact type; specification
        // says that if the policy is not provided, we are to use "false"
        //
        if(policyTable_.containsKey("exact_type_match"))
            exactType_ =
                getBooleanPolicyValue(policyTable_, "exact_type_match");
        else
            exactType_ = false;

        //
        // Importer cannot override use_dynamic_properties if the
        // trader doesn't support it
        //
        if(policyTable_.containsKey("use_dynamic_properties"))
        {
            useDynamic_ =
                getBooleanPolicyValue(policyTable_, "use_dynamic_properties");
            if(useDynamic_ && !lookup_.supports_dynamic_properties())
            {
                useDynamic_ = false;
                limitsApplied_.addElement("supports_dynamic_properties");
            }
        }
        else
            useDynamic_ = lookup_.supports_dynamic_properties();

        //
        // Importer cannot override use_modifiable_properties if the
        // trader doesn't support it
        //
        if(policyTable_.containsKey("use_modifiable_properties"))
        {
            useModifiable_ =
                getBooleanPolicyValue(policyTable_,
                                      "use_modifiable_properties");
            if(useModifiable_ && !lookup_.supports_modifiable_properties())
            {
                useModifiable_ = false;
                limitsApplied_.addElement("supports_modifiable_properties");
            }
        }
        else
            useModifiable_ = lookup_.supports_modifiable_properties();

        //
        // Importer cannot override use_proxy_offers if the
        // trader doesn't support it
        //
        if(policyTable_.containsKey("use_proxy_offers"))
        {
            useProxyOffers_ =
                getBooleanPolicyValue(policyTable_, "use_proxy_offers");
            if(useProxyOffers_ && !lookup_.supports_proxy_offers())
            {
                useProxyOffers_ = false;
                limitsApplied_.addElement("supports_proxy_offers");
            }
        }
        else
            useProxyOffers_ = lookup_.supports_proxy_offers();

        //
        // Determine the link_follow_rule policy
        //
        if(policyTable_.containsKey("link_follow_rule"))
        {
            linkFollowRule_ =
                getFollowPolicyValue(policyTable_, "link_follow_rule");
            org.omg.CosTrading.FollowOption maxFollow =
                lookup_.max_follow_policy();
            if(linkFollowRule_.value() > maxFollow.value())
            {
                linkFollowRule_ = maxFollow;
                limitsApplied_.addElement("max_follow_policy");
            }
        }
        else
            linkFollowRule_ = lookup_.def_follow_policy();
    }

    //
    // Retrieves a ulong policy value
    //
    private int
    getULongPolicyValue(java.util.Hashtable policies, String name)
        throws org.omg.CosTrading.LookupPackage.PolicyTypeMismatch,
               org.omg.CosTrading.LookupPackage.InvalidPolicyValue
    {
        int result = 0;

        org.omg.CORBA.Any value = (org.omg.CORBA.Any)policies.get(name);
        org.omg.CosTrading.Policy p =
            new org.omg.CosTrading.Policy(name, value);

        if(value != null)
        {
            try
            {
                if(value.type().kind() != org.omg.CORBA.TCKind.tk_ulong)
                    throw new org.omg.CosTrading.LookupPackage.
                        PolicyTypeMismatch(p);

                result = value.extract_ulong();
            }
            catch(org.omg.CORBA.BAD_OPERATION e)
            {
                throw new org.omg.CosTrading.LookupPackage.
                    InvalidPolicyValue(p);
            }
        }

        return result;
    }

    //
    // Retrieves a boolean policy value
    //
    private boolean
    getBooleanPolicyValue(java.util.Hashtable policies, String name)
        throws org.omg.CosTrading.LookupPackage.PolicyTypeMismatch,
               org.omg.CosTrading.LookupPackage.InvalidPolicyValue
    {
        boolean result = false;

        org.omg.CORBA.Any value = (org.omg.CORBA.Any)policies.get(name);
        org.omg.CosTrading.Policy p =
            new org.omg.CosTrading.Policy(name, value);

        if(value != null)
        {
            try
            {
                if(value.type().kind() != org.omg.CORBA.TCKind.tk_boolean)
                    throw new org.omg.CosTrading.LookupPackage.
                        PolicyTypeMismatch(p);

                result = value.extract_boolean();
            }
            catch(org.omg.CORBA.BAD_OPERATION e)
            {
                throw new org.omg.CosTrading.LookupPackage.
                        InvalidPolicyValue(p);
            }
        }

        return result;
    }

    //
    // Retrieves a org.omg.CosTrading.FollowOption policy value
    //
    private org.omg.CosTrading.FollowOption
    getFollowPolicyValue(java.util.Hashtable policies, String name)
        throws org.omg.CosTrading.LookupPackage.PolicyTypeMismatch,
               org.omg.CosTrading.LookupPackage.InvalidPolicyValue
    {
        org.omg.CosTrading.FollowOption result = null;

        org.omg.CORBA.Any value = (org.omg.CORBA.Any)policies.get(name);
        org.omg.CosTrading.Policy p =
            new org.omg.CosTrading.Policy(name, value);

        if(value != null)
        {
            try
            {
                org.omg.CORBA.TypeCode tc =
                    org.omg.CosTrading.FollowOptionHelper.type();

                if(!value.type().equal(tc))
                    throw new org.omg.CosTrading.LookupPackage.
                        PolicyTypeMismatch(p);

                result = org.omg.CosTrading.FollowOptionHelper.extract(value);
            }
            catch(org.omg.CORBA.BAD_OPERATION e)
            {
                throw new org.omg.CosTrading.LookupPackage.
                        InvalidPolicyValue(p);
            }
        }

        return result;
    }

    //
    // Finds all service types compatible with the given type
    //
    private void
    findCompatibleTypes(String type, java.util.Vector types)
    {
        //
        // Obtain the list of all service type names
        //
        SpecifiedServiceTypes whichTypes = new SpecifiedServiceTypes();
        whichTypes._default(ListOption.all);

        String[] allNames = repos_.list_types(whichTypes);

        //
        // Iterate through all service types, obtaining the full description
        // of each to see if the service type in question is a supertype
        //
        for(int i = 0 ; i < allNames.length ; i++)
        {
            try
            {
                TypeStruct ts = repos_.fully_describe_type(allNames[i]);

                for(int n = 0 ; n < ts.super_types.length ; n++)
                {
                    if(type.equals(ts.super_types[n]))
                    {
                        if(!types.contains(allNames[i]))
                            types.addElement(allNames[i]);
                        break;
                    }
                }
            }
            catch(org.omg.CosTrading.IllegalServiceType e)
            {
                // ignore
            }
            catch(org.omg.CosTrading.UnknownServiceType e)
            {
                // ignore
            }
        }
    }

    //
    // Collect all potential offers (including proxy offers), respecting
    // search limits
    //
    private java.util.Vector
    collectPotentialOffers(java.util.Vector types, TypeStruct ts)
    {
        java.util.Vector result = new java.util.Vector();

        //
        // Iterate through all of the selected service types, examining
        // the offers of each, and adding any potential ones to
        // potentialOffers while respecting the search cardinality limits
        //
        int searchCount = 0;
        java.util.Enumeration e = types.elements();
        while(e.hasMoreElements() && searchCount < searchCard_)
        {
            String typeName = (String)e.nextElement();
            java.util.Hashtable table = db_.getOffers(typeName);
            if(table == null)
                continue;

            //
            // Examine the offers of this service type
            //
            java.util.Enumeration o = table.elements();
            while(o.hasMoreElements() && searchCount < searchCard_)
            {
                org.omg.CosTrading.RegisterPackage.OfferInfo info =
                  (org.omg.CosTrading.RegisterPackage.OfferInfo)o.nextElement();

                PropertyList pl =
                    new PropertyList(orb_, info.reference, info.properties);

                if(considerOffer(pl, ts))
                {
                    result.addElement(pl);
                    searchCount++;
                }
            }
        }

        //
        // Iterate through all of the selected service types, examining
        // the proxy offers of each, and adding any potential ones to
        // potentialOffers while respecting the search cardinality limits
        //
        if(useProxyOffers_)
        {
            e = types.elements();
            while(e.hasMoreElements() && searchCount < searchCard_)
            {
                String typeName = (String)e.nextElement();
                java.util.Hashtable table = db_.getProxyOffers(typeName);
                if(table == null)
                    continue;

                //
                // examine the offers of this service type
                //
                java.util.Enumeration o = table.elements();
                while(o.hasMoreElements() && searchCount < searchCard_)
                {
                    org.omg.CosTrading.ProxyPackage.ProxyInfo info =
                     (org.omg.CosTrading.ProxyPackage.ProxyInfo)o.nextElement();

                    ProxyPropertyList pl =
                        new ProxyPropertyList(orb_, info);

                    if(considerOffer(pl, ts))
                    {
                        result.addElement(pl);
                        searchCount++;
                    }
                }
            }
        }

        return result;
    }

    //
    // Initial test to determine whether an offer should be considered
    //
    private boolean
    considerOffer(PropertyList pl, TypeStruct ts)
    {
        if(!useDynamic_)
        {
            org.omg.CosTrading.Property[] props = pl.getProperties();

            //
            // Check if any property in the offer is dynamic
            //
            if(PropUtil.hasDynamicProperties(props))
                return false;
        }

        if(!useModifiable_)
        {
            //
            // Check if any property in the offer is modifiable
            //
            for(int i = 0 ; i < ts.props.length ; i++)
            {
                //
                // If the mode of the property is modifiable, then check
                // if this offer has a definition for the property
                //
                if(ts.props[i].mode == PropertyMode.PROP_NORMAL ||
                   ts.props[i].mode == PropertyMode.PROP_MANDATORY)
                    if(pl.exists(ts.props[i].name))
                        return false;
            }
        }

        return true;
    }

    //
    // Forwards the query operation to any links
    //
    private void
    processLinks()
    {
        //
        // We have to add and/or modify some policies, so we will
        // build up a new set and merge in the existing ones
        //
        java.util.Hashtable policies = new java.util.Hashtable();

        //
        // If no request ID is present, then we might generate one of our own
        //
        if(!policyTable_.containsKey("request_id"))
        {
            byte[] stem = lookup_.admin_if().request_id_stem();
            if(stem.length > 0)
            {
                byte[] requestId = requestHistory_.generateRequestId(stem);
                requestHistory_.add(requestId);
                org.omg.CORBA.Any value = orb_.create_any();
                org.omg.CosTrading.AdminPackage.OctetSeqHelper.
                    insert(value, requestId);
                policies.put("request_id", value);
            }
        }

        //
        // Hop count
        //
        hopCount_--;
        org.omg.CORBA.Any hopCountValue = orb_.create_any();
        hopCountValue.insert_ulong(hopCount_);
        policies.put("hop_count", hopCountValue);

        //
        // Merge in the existing policies, excluding those we've already
        // defined as well as the "link_follow_rule" policy, which is
        // defined for each link
        //
        java.util.Enumeration e = policyTable_.keys();
        while(e.hasMoreElements())
        {
            String name = (String)e.nextElement();
            if(!policies.containsKey(name) &&
               !name.equals("link_follow_rule"))
                policies.put(name, policyTable_.get(name));
        }

        //
        // Create the new policies array
        //
        int numPolicies = policies.size();
        linkPolicies_ = new org.omg.CosTrading.Policy[numPolicies];
        int count = 0;
        e = policies.keys();
        while(e.hasMoreElements())
        {
            String name = (String)e.nextElement();
            org.omg.CORBA.Any value = (org.omg.CORBA.Any)policies.get(name);
            linkPolicies_[count++] =
                new org.omg.CosTrading.Policy(name, value);
        }


        //
        // Create a thread pool to simultaneously process multiple links;
        // we allow up to MAX_LINK_THREADS active threads.
        //
        // For each link, the LinkInfo struct is obtained and "scheduled"
        // with the thread pool.  Eventually, our doWork() method below
        // will be invoked, passing in the LinkInfo struct as the argument.
        //
        // The doWork() method will process the link.  Any matching offers
        // will be added to matchingOffers_.  After processing is complete,
        // doWork() will update the linksProcessed_ counter.  If the counter
        // is equal to numLinks_, the thread pool will be halted since all
        // links will have been evaluated.
        //

        int maxThreads = 10; // TODO: use a property for this?
        threadPool_ = new ThreadPool(this, maxThreads);

        //
        // Schedule a query request for each of our links; we stop early
        // if we reach our match limits
        //
        try
        {
            org.omg.CosTrading.Link link = lookup_.link_if();
            String[] names = link.list_links();
            numLinks_ = names.length;
            linksProcessed_ = 0;

            for(int i = 0 ; i < names.length && !workDone() ; i++)
            {
                try
                {
                    org.omg.CosTrading.LinkPackage.LinkInfo info =
                        link.describe_link(names[i]);
                    threadPool_.schedule(info);
                }
                catch(org.omg.CORBA.UserException ex)
                {
                    // ignore - skip this link
                }
                catch(org.omg.CORBA.SystemException ex)
                {
                    // ignore - skip this link
                }
            }
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            // ignore
        }

        threadPool_.finish();
    }

    //
    // Inherited from ThreadPool.Worker - this method is called by
    // independent threads to process links in parallel
    //
    public boolean
    doWork(java.lang.Object o)
    {
        org.omg.CosTrading.LinkPackage.LinkInfo info =
            (org.omg.CosTrading.LinkPackage.LinkInfo)o;

        //
        // Avoid unnecessary work by checking first to see if we've
        // already met our match limits
        //
        if(workDone())
            return false;

        try
        {
            //
            // Determine the follow policy for this link
            // (see p. 17 of the specification)
            //

            org.omg.CosTrading.FollowOption follow =
                org.omg.CosTrading.FollowOption.from_int(
                    Math.min(linkFollowRule_.value(),
                             info.limiting_follow_rule.value()));

            //
            // If the policy is local_only, we're done with this link
            //
            if(follow == org.omg.CosTrading.FollowOption.local_only)
                return !workDone();

            //
            // Skip this link if the follow option is if_no_local and
            // we have local offers
            //
            if(follow == org.omg.CosTrading.FollowOption.if_no_local &&
               localMatchCount_ > 0)
                return !workDone();

            //
            // Determine the follow policy to be passed on for this link
            // (see p. 17 of the specification)
            //

            org.omg.CosTrading.FollowOption passOnFollow;
            if(policyTable_.containsKey("link_follow_rule"))
                passOnFollow = follow;
            else
                passOnFollow = org.omg.CosTrading.FollowOption.
                    from_int(Math.min(info.def_pass_on_follow_rule.value(),
                                      lookup_.max_follow_policy().value()));

            //
            // Compose the policies to be passed on
            //

            org.omg.CosTrading.Policy[] passOnPolicies =
                new org.omg.CosTrading.Policy[linkPolicies_.length + 1];
            System.arraycopy(linkPolicies_, 0, passOnPolicies, 0,
                             linkPolicies_.length);
            org.omg.CORBA.Any passOnValue = orb_.create_any();
            org.omg.CosTrading.FollowOptionHelper.insert(passOnValue,
                                                         passOnFollow);
            passOnPolicies[linkPolicies_.length] =
                new org.omg.CosTrading.Policy("link_follow_rule",
                                              passOnValue);

            //
            // Call the target trader's query operation
            //

            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder offerIter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limitsApplied =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            info.target.query(type_, constraint_, preference_, passOnPolicies,
                              desiredProps_, 0, offers, offerIter,
                              limitsApplied);

            //
            // Retrieve the matching offers from the iterator
            //
            if(offerIter.value != null)
            {
                org.omg.CosTrading.OfferSeqHolder seq =
                    new org.omg.CosTrading.OfferSeqHolder();
                boolean more;
                do
                {
                    more = offerIter.value.next_n(20, seq);
                    if(!addMatchingOffers(seq.value))
                        more = false;
                }
                while(more);

                offerIter.value.destroy();
            }

            //
            // Concatenate limits (if any)
            //
            if(limitsApplied.value != null && limitsApplied.value.length > 0)
            {
                synchronized(limitsApplied_)
                {
                    for(int l = 0 ; l < limitsApplied.value.length ; l++)
                        if(!limitsApplied_.contains(limitsApplied.value[l]))
                            limitsApplied_.addElement(limitsApplied.value[l]);
                }
            }
        }
        catch(org.omg.CORBA.UserException e)
        {
            // ignore
        }
        catch(org.omg.CORBA.SystemException e)
        {
            // ignore
        }
        finally
        {
            //
            // When we're done processing a link, we bump our counter
            // and compare it with numLinks_; if they're equal, we've
            // processed all links and can shutdown the thread pool
            //
            synchronized(this)
            {
                linksProcessed_++;
                if(linksProcessed_ == numLinks_)
                    threadPool_.halt();
            }
        }

        return !workDone();
    }

    //
    // Adds an array of matching offers, respecting match limits;
    // returns true if we haven't reached our limit, false otherwise
    //
    private synchronized boolean
    addMatchingOffers(org.omg.CosTrading.Offer[] offers)
    {
        int count = 0;
        int size = matchingOffers_.size();
        while(size < matchCard_ && count < offers.length)
        {
            //
            // Create a new PropertyList object for each offer
            //
            PropertyList pl = new PropertyList(orb_, offers[count].reference,
                                               offers[count].properties);
            matchingOffers_.addElement(pl);
            count++;
            size++;
        }

        return (size < matchCard_);
    }

    //
    // Return true if the worker threads are done
    //
    private synchronized boolean
    workDone()
    {
        //
        // We're done if we've met our match cardinality limit
        //
        return (matchingOffers_.size() >= matchCard_);
    }
}
