// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Things are just too ugly without this
//
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

//
// Implementation of CosTradingRepos::ServiceTypeRepository
//
final class RepositoryImpl
  extends org.omg.CosTradingRepos._ServiceTypeRepositoryImplBase
{
    private DBManager dbMgr_;
    private TypeDB database_;
    private boolean useIR_;

    public
    RepositoryImpl(DBManager dbMgr, boolean useIR)
    {
        dbMgr_ = dbMgr;
        database_ = dbMgr.getTypeDB();
        useIR_ = useIR;
    }

    public IncarnationNumber
    incarnation()
    {
        IncarnationNumber result;

        database_.begin(AbstractDB.READ);
        result = database_.getIncarnation();
        database_.end();

        return result;
    }

    public IncarnationNumber
    add_type(String name,
             String if_name,
             PropStruct[] props,
             String[] super_types)
        throws org.omg.CosTrading.IllegalServiceType,
               ServiceTypeExists,
               InterfaceTypeMismatch,
               org.omg.CosTrading.IllegalPropertyName,
               org.omg.CosTrading.DuplicatePropertyName,
               ValueTypeRedefinition,
               org.omg.CosTrading.UnknownServiceType,
               DuplicateServiceTypeName
    {
        IncarnationNumber result = null;

        try
        {
            database_.begin(AbstractDB.WRITE);

            //
            // The type name can be either a scoped name or an
            // interface repository ID
            //
            if(!IRUtil.validateName(name))
                throw new org.omg.CosTrading.IllegalServiceType(name);

            //
            // Make sure another type doesn't already exist with the same name
            //
            if(database_.describeType(name) != null)
                throw new ServiceTypeExists(name);

            //
            // We'll build up a hashtable of type descriptions for later use
            //
            java.util.Hashtable typeInfo = new java.util.Hashtable();

            //
            // Check for duplicate super types
            //
            for(int i = 0 ; i < super_types.length ; i++)
            {
                TypeStruct ts = findType(super_types[i]);
                if(typeInfo.containsKey(super_types[i]))
                    throw new DuplicateServiceTypeName(super_types[i]);
                typeInfo.put(super_types[i], ts);
            }

            //
            // Collect descriptions of all super types
            //

            for(int i = 0 ; i < super_types.length ; i++)
            {
                String[] names = database_.getAllSuperTypes(super_types[i]);
                for(int n = 0 ; n < names.length ; n++)
                {
                    if(!typeInfo.containsKey(names[n]))
                    {
                        TypeStruct ts = findType(names[n]);
                        if(ts == null)
                            throw new org.omg.CosTrading.
                                      UnknownServiceType(names[n]);
                        typeInfo.put(names[n], ts);
                    }
                }
            }

            //
            // Check for interface compatibility
            //
            validateInterface(name, if_name, super_types, typeInfo);

            //
            // Check for duplicate properties and for value type redefinition
            //
            validateProperties(name, props, super_types, typeInfo);

            result = database_.createType(name, if_name, props, super_types);
        }
        finally
        {
            database_.end();
        }

        return result;
    }

    public void
    remove_type(String name)
        throws org.omg.CosTrading.IllegalServiceType,
               org.omg.CosTrading.UnknownServiceType,
               HasSubTypes
    {
        try
        {
            database_.begin(AbstractDB.WRITE);

            checkTypeName(name);

            String subTypeName = database_.findSubType(name);

            if(subTypeName != null)
                throw new HasSubTypes(name, subTypeName);

            if(!database_.removeType(name))
                throw new org.omg.CosTrading.UnknownServiceType(name);
        }
        finally
        {
            database_.end();
        }
    }

    public String[]
    list_types(SpecifiedServiceTypes which_types)
    {
        String[] result = null;

        try
        {
            database_.begin(AbstractDB.READ);

            if(which_types.discriminator() == ListOption.all)
                result = database_.getTypes();
            else
            {
                IncarnationNumber inc = which_types.incarnation();
                result = database_.getTypesSince(inc);
            }
        }
        finally
        {
            database_.end();
        }

        return result;
    }

    public TypeStruct
    describe_type(String name)
        throws org.omg.CosTrading.IllegalServiceType,
               org.omg.CosTrading.UnknownServiceType
    {
        TypeStruct result;

        try
        {
            database_.begin(AbstractDB.READ);

            checkTypeName(name);

            result = findType(name);
        }
        finally
        {
            database_.end();
        }

        return result;
    }

    public TypeStruct
    fully_describe_type(String name)
        throws org.omg.CosTrading.IllegalServiceType,
               org.omg.CosTrading.UnknownServiceType
    {
        TypeStruct result = null;
        java.util.Enumeration e;

        try
        {
            database_.begin(AbstractDB.READ);

            checkTypeName(name);

            result = findType(name);

            String[] superTypeNames = database_.getAllSuperTypes(name);

            //
            // Replace the super_types member
            //
            result.super_types = superTypeNames;

            //
            // Get descriptions of all the supertypes
            //
            java.util.Vector desc = new java.util.Vector();
            for(int i = 0 ; i < superTypeNames.length ; i++)
            {
                TypeStruct ts = findType(superTypeNames[i]);
                desc.addElement(ts);
            }

            //
            // Build up the complete list of properties, starting with the
            // properties in the requested type
            //
            java.util.Vector props = new java.util.Vector();
            for(int i = 0 ; i < result.props.length ; i++)
                props.addElement(result.props[i]);

            e = desc.elements();
            while(e.hasMoreElements())
            {
                TypeStruct ts = (TypeStruct)e.nextElement();
                for(int i = 0 ; i < ts.props.length ; i++)
                {
                    //
                    // Iterate through the current list, making sure we don't
                    // add a property that already exists
                    //
                    boolean found = false;
                    java.util.Enumeration p = props.elements();
                    while(p.hasMoreElements() && !found)
                    {
                        PropStruct ps = (PropStruct)p.nextElement();
                        if(ts.props[i].name.equals(ps.name))
                            found = true;
                    }

                    if(!found)
                        props.addElement(ts.props[i]);
                }
            }

            result.props = new PropStruct[props.size()];
            props.copyInto((java.lang.Object[])result.props);
        }
        finally
        {
            database_.end();
        }

        return result;
    }

    public void
    mask_type(String name)
        throws org.omg.CosTrading.IllegalServiceType,
               org.omg.CosTrading.UnknownServiceType,
               AlreadyMasked
    {
        try
        {
            database_.begin(AbstractDB.WRITE);

            checkTypeName(name);

            TypeStruct ts = findType(name);

            if(ts.masked)
                throw new AlreadyMasked(name);
            else if(!database_.maskType(name))
                throw new org.omg.CosTrading.UnknownServiceType(name);
        }
        finally
        {
            database_.end();
        }
    }

    public void
    unmask_type(String name)
        throws org.omg.CosTrading.IllegalServiceType,
               org.omg.CosTrading.UnknownServiceType,
               NotMasked
    {
        try
        {
            database_.begin(AbstractDB.WRITE);

            checkTypeName(name);

            TypeStruct ts = findType(name);

            if(!ts.masked)
                throw new NotMasked(name);
            else if(!database_.unmaskType(name))
                throw new org.omg.CosTrading.UnknownServiceType(name);
        }
        finally
        {
            database_.end();
        }
    }

    private void
    checkTypeName(String name)
        throws org.omg.CosTrading.IllegalServiceType
    {
        if(name == null || name.trim().length() == 0)
            throw new org.omg.CosTrading.IllegalServiceType("");

        if(!IRUtil.validateName(name))
            throw new org.omg.CosTrading.IllegalServiceType(name);
    }

    private TypeStruct
    findType(String name)
        throws org.omg.CosTrading.UnknownServiceType
    {
        TypeStruct result = database_.describeType(name);

        if(result == null)
            throw new org.omg.CosTrading.UnknownServiceType(name);

        return result;
    }

    private void
    validateInterface(String name,
                      String interfaceName,
                      String[] superTypes,
                      java.util.Hashtable typeInfo)
        throws InterfaceTypeMismatch
    {
        //
        // If we have an interface repository, then validate the type's
        // interface against the interface of each of the supertypes
        //
        if(useIR_)
        {
            org.omg.CORBA.InterfaceDef def = null;

            //
            // retrieve the InterfaceDef object for the interface
            //
            try
            {
                org.omg.CORBA.Contained c = IRUtil.getContained(interfaceName);
                if(c != null)
                    def = org.omg.CORBA.InterfaceDefHelper.narrow(c);

                if(def != null)
                {
                    //
                    // validate the InterfaceDef against each supertype
                    //
                    for(int i = 0 ; i < superTypes.length ; i++)
                    {
                        TypeStruct ts =
                            (TypeStruct)typeInfo.get(superTypes[i]);
                        if(!IRUtil.validate(def, ts.if_name))
                            throw new InterfaceTypeMismatch(superTypes[i],
                                                            ts.if_name,
                                                            name,
                                                            interfaceName);
                    }
                }
            }
            catch(org.omg.CORBA.SystemException e)
            {
                // ignore
            }
        }
    }

    private void
    validateProperties(String name,
                       PropStruct[] props,
                       String[] superTypes,
                       java.util.Hashtable typeInfo)
        throws org.omg.CosTrading.IllegalPropertyName,
               org.omg.CosTrading.DuplicatePropertyName,
               ValueTypeRedefinition
    {
        java.util.Vector goodProps = new java.util.Vector();

        //
        // For each property, we need to validate it against the
        // property with the same name in all supertypes
        //
        for(int i = 0 ; i < props.length ; i++)
        {
            if(!dbMgr_.isPropertyNameValid(props[i].name))
                throw new org.omg.CosTrading.
                          IllegalPropertyName(props[i].name);

            //
            // Make sure we haven't seen this property already
            //
            if(goodProps.contains(props[i].name))
                throw new org.omg.CosTrading.
                          DuplicatePropertyName(props[i].name);

            //
            // If the database doesn't support the property type,
            // then we throw PERSIST_STORE; it's not ideal,
            // but it's about the best we can do, since the specification
            // does not define an exception for use in this situation
            //
            if(!dbMgr_.isPropertyTypeSupported(props[i].value_type))
            {
                com.ooc.CORBA.MessageViewer.instance().error(
                    "unsupported type for property " + props[i].name);
                throw new org.omg.CORBA.PERSIST_STORE();
            }

            //
            // Now ask each supertype if they have a property of the same name;
            // we don't stop after the first match - we have to ask ALL of the
            // super types
            //
            for(int s = 0 ; s < superTypes.length ; s++)
            {
                TypeStruct ts = (TypeStruct)typeInfo.get(superTypes[s]);

                java.util.Vector superProps = new java.util.Vector();
                findProperties(ts, props[i].name, superProps, typeInfo);

                java.util.Enumeration e = superProps.elements();
                while(e.hasMoreElements())
                {
                    PropStruct ps = (PropStruct)e.nextElement();

                    //
                    // make sure the type codes are equivalent and the modes
                    // are compatible
                    //
                    if(!props[i].value_type.equal(ps.value_type) ||
                       !validateMode(props[i].mode, ps.mode))
                        throw new ValueTypeRedefinition(name, props[i],
                                                        superTypes[s], ps);
                }
            }

            //
            // If we've made it here without throwing an exception, add this
            // property to our running list of validated properties
            //
            goodProps.addElement(props[i].name);
        }
    }

    private boolean
    validateMode(PropertyMode subMode, PropertyMode superMode)
    {
        boolean result = false;

        //
        // If both are equal, we're done
        //
        if(superMode == subMode)
            result = true;
        //
        // If mode of superMode is normal, subMode mode can be anything
        //
        else if(superMode == PropertyMode.PROP_NORMAL)
            result = true;
        else if(superMode == PropertyMode.PROP_READONLY &&
                (subMode == PropertyMode.PROP_READONLY ||
                 subMode == PropertyMode.PROP_MANDATORY_READONLY))
            result = true;
        else if(superMode == PropertyMode.PROP_MANDATORY &&
                (subMode == PropertyMode.PROP_MANDATORY ||
                 subMode == PropertyMode.PROP_MANDATORY_READONLY))
            result = true;
        else if( // redundant, but included for completeness
                superMode == PropertyMode.PROP_MANDATORY_READONLY &&
                subMode == PropertyMode.PROP_MANDATORY_READONLY)
            result = true;

        return result;
    }

    //
    // This function is called recursively
    //
    private void
    findProperties(TypeStruct ts,
                   String propName,
                   java.util.Vector v,
                   java.util.Hashtable typeInfo)
    {
        PropStruct prop = findProperty(ts, propName);
        if(prop != null)
            v.addElement(prop);
        else
        {
            //
            // Ask all of our supertypes to add to the list - typeInfo should
            // contain the descriptions for the complete set of supertypes
            //
            for(int i = 0 ; i < ts.super_types.length ; i++)
            {
                TypeStruct superTS =
                    (TypeStruct)typeInfo.get(ts.super_types[i]);
                findProperties(superTS, propName, v, typeInfo);
            }
        }
    }

    private PropStruct
    findProperty(TypeStruct ts, String name)
    {
        PropStruct result = null;

        for(int i = 0 ; i < ts.props.length && result == null ; i++)
            if(name.equals(ts.props[i].name))
                result = ts.props[i];

        return result;
    }
}
