// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Maintains a history of previous requests
//
final class RequestHistory
{
    private java.util.Hashtable requests_;
    private long nextRequest_;

    public
    RequestHistory()
    {
        requests_ = new java.util.Hashtable();
        nextRequest_ = 0L;
    }

    //
    // Adds the new request ID to the history, returning false if
    // the request already exists in the history, or true otherwise.
    // This activity needs to be an atomic operation to avoid race
    // conditions in a multi-threaded environment.
    //
    public synchronized boolean
    add(byte[] requestId)
    {
        boolean result = false;

        String key = new String(requestId);
        if(!requests_.containsKey(key))
        {
            requests_.put(key, requestId);
            result = true;
        }

        return result;
    }

    //
    // Generates a new request ID using the given stem.  The bytes
    // of the stem are prepended to the bytes from a unique long value
    // to form the complete request ID.
    //
    public synchronized byte[]
    generateRequestId(byte[] stem)
    {
        byte[] result;

        //
        // The new request ID is comprised of the stem plus the bytes
        // of the nextRequest_ long value
        //
        result = new byte[stem.length + 8];
        System.arraycopy(stem, 0, result, 0, stem.length);
        int pos = stem.length;
        for(int i = 0 ; i < 8 ; i++, pos++)
            result[pos] = (byte)((nextRequest_ >> (8 * (7 - i))) & 0xFF);

        nextRequest_++;

        return result;
    }
}
