// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Things are just too ugly without this
//
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class Server
{
    public static void
    main(String[] args)
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        try
        {
            //
            // Set the program name
            //
            String progName = "com.ooc.CosTrading.Server";

            //
            // Create ORB
            //
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, props);

            //
            // Set ReconnectPolicy on ORB
            //
            try
            {
                org.omg.CORBA.Any boolAny = orb.create_any();
                boolAny.insert_boolean(true);
                org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[1];
                policies[0] =
                    orb.create_policy(com.ooc.OB.RECONNECT_POLICY.value,
                                      boolAny);
                org.omg.CORBA.PolicyManager pm =
                    org.omg.CORBA.PolicyManagerHelper.narrow(
                        orb.resolve_initial_references("ORBPolicyManager"));
                pm.add_policy_overrides(policies);
            }
            catch(org.omg.CORBA.ORBPackage.InvalidName ex)
            {
                System.err.println(progName + ": No ORBPolicyManager");
                System.exit(1);
            }
            catch(org.omg.CORBA.PolicyError ex)
            {
                System.err.println(progName + ": Error creating policy");
                System.exit(1);
            }
            catch(org.omg.CORBA.PolicyManagerPackage.InvalidPolicies ex)
            {
                System.err.println(progName + ": Error invalid policies");
                System.exit(1);
            }

            //
            // Filter ORB options
            //
            args = ((com.ooc.CORBA.ORB)orb).filter_options(args);

            //
            // Set the IIOP port based on the port defined in the IIOP
            // profile of the service object (if it exists)
            //
            try
            {
                org.omg.CORBA.Object obj =
                    orb.resolve_initial_references("TradingService");
                com.ooc.CORBA.BOA.set_iiop_port_from_object(obj);
            }
            catch(org.omg.CORBA.ORBPackage.InvalidName e)
            {
            }

            //
            // Create BOA
            //
            com.ooc.CORBA.BOA boa = (com.ooc.CORBA.BOA)
                ((com.ooc.CORBA.ORB)orb).BOA_init(args, props);

            //
            // Ensure the BOA thread model is suitable. If the BOA
            // threading model is not thread pool or thread-per-
            // request, then we force thread-per-request.
            //
            com.ooc.CORBA.BOA.ConcModel model = boa.conc_model();
            if(model != com.ooc.CORBA.BOA.ConcModel.ConcModelThreadPool &&
               model != com.ooc.CORBA.BOA.ConcModel.ConcModelThreadPerRequest)
            {
                boa.conc_model(
                    com.ooc.CORBA.BOA.ConcModel.ConcModelThreadPerRequest);
            }

            //
            // Filter BOA options
            //
            args = ((com.ooc.CORBA.BOA)boa).filter_options(args);

            //
            // Get options
            //
            boolean ior = false;
            int timeout = 5; // Every 5 minutes
            String dbdir = null;
            boolean useIR = false;
            boolean allowNil = true;

            //
            // Check properties first
            //
            com.ooc.CORBA.Properties properties =
                com.ooc.CORBA.Properties.instance();
            String[] keys = properties.getKeys("ooc.trading.");
            for(int i = 0 ; i < keys.length ; ++i)
            {
                String key = keys[i];
                String value = properties.getProperty(key);

                if(key.equals("ooc.trading.dbdir"))
                {
                    dbdir = value;
                }
                else if(key.equals("ooc.trading.timeout"))
                {
                    try
                    {
                        timeout = Integer.parseInt(value);
                    }
                    catch(NumberFormatException ex)
                    {
                        System.err.println(
                            "Invalid value for ooc.trading.timeout");
                        System.exit(1);
                    }
                }
                else if(key.equals("ooc.trading.use_ir"))
                {
                    useIR = value.equalsIgnoreCase("true");
                }
                else if(key.equals("ooc.trading.allow_nil_objects"))
                {
                    allowNil = value.equalsIgnoreCase("true");
                }
                else
                {
                    System.err.println(progName + ": `" + key + "': " +
                                       "unknown property");
                }
            }

            for(int i = 0 ; i < args.length && args[i].startsWith("-") ; i++)
            {
                if(args[i].equals("--help") ||
                   args[i].equals("-h"))
                {
                    usage(progName);
                    System.exit(0);
                }
                else if(args[i].equals("--version") ||
                        args[i].equals("-v"))
                {
                    System.err.println("ORBacus Trading Service " +
                                       com.ooc.CORBA.Version.version);
                    System.exit(0);
                }
                else if(args[i].equals("--ior") ||
                        args[i].equals("-i"))
                {
                    ior = true;
                }
                else if(args[i].equals("--timeout") ||
                        args[i].equals("-t"))
                {
                    ++i;
                    if(i >= args.length)
                    {
                        System.err.println(progName + ": " + args[i - 1] +
                                           "expects argument");
                        usage(progName);
                        System.exit(1);
                    }

                    timeout = Integer.parseInt(args[i]);
                }
                else if(args[i].equals("--dbdir") ||
                        args[i].equals("-d"))
                {
                    ++i;
                    if(i >= args.length)
                    {
                        System.err.println(progName + ": " + args[i - 1] +
                                           "expects argument");
                        usage(progName);
                        System.exit(1);
                    }

                    dbdir = args[i];
                }
                else
                {
                    System.err.println(progName + ": unknown option `"
                                       + args[i] + "'\n");
                    usage(progName);
                    System.exit(1);
                }
            }

            if(timeout <= 0)
            {
                System.err.println(progName + ": timeout value must be > 0");
                System.exit(1);
            }

            //
            // Validate the database pathname
            //
            if(dbdir != null)
            {
                java.io.File f = new java.io.File(dbdir);
                if(!f.exists())
                {
                    System.err.println(progName + ": The directory " + dbdir +
                                       " does not exist");
                    System.exit(1);
                }
                else if(!f.isDirectory())
                {
                    System.err.println(progName + ": The path " + dbdir +
                                       " is not a directory");
                    System.exit(1);
                }
            }

            if(useIR)
            {
                try
                {
                    org.omg.CORBA.Object obj =
                        orb.resolve_initial_references("InterfaceRepository");
                    org.omg.CORBA.Repository repos =
                        org.omg.CORBA.RepositoryHelper.narrow(obj);
                    IRUtil.initialize(repos);
                }
                catch(org.omg.CORBA.ORBPackage.InvalidName ex)
                {
                    System.err.println(progName + ": No interface repository" +
                                       " available");
                    System.exit(1);
                }
                catch(org.omg.CORBA.BAD_PARAM ex)
                {
                    System.err.println(progName + ": Invalid reference for" +
                                       " interface repository");
                    System.exit(1);
                }
            }

            //
            // Create the database manager
            //
            DBManager dbMgr = new DBManager_impl(orb, dbdir, timeout);

            //
            // Create the service type repository implementation and
            // connect it to the ORB
            //
            org.omg.CosTradingRepos.ServiceTypeRepository typeRepos =
                new RepositoryImpl(dbMgr, useIR);
            ((com.ooc.CORBA.ORB)orb).connect(typeRepos, "TypeRepository");

            //
            // Create the trader components
            //
            RegisterImpl reg = new RegisterImpl(orb, dbMgr, useIR, allowNil);
            LookupImpl lookup = new LookupImpl(orb, dbMgr);
            ProxyImpl proxy = new ProxyImpl(dbMgr);
            LinkImpl link = new LinkImpl(dbMgr);
            AdminImpl admin =
                new AdminImpl(orb, typeRepos, reg, lookup, proxy, link, dbMgr);
            reg.setAdmin(admin);
            lookup.setAdmin(admin);
            proxy.setAdmin(admin);
            link.setAdmin(admin);

            //
            // Connect the components
            //
            ((com.ooc.CORBA.ORB)orb).connect(lookup, "TradingService");
            orb.connect(reg);
            orb.connect(admin);
            orb.connect(proxy);
            orb.connect(link);

            //
            // Output the IOR of the Lookup object
            //
            if(ior)
            {
                String ref = orb.object_to_string(lookup);
                System.out.println(ref);
            }

            boa.impl_is_ready(null);

            dbMgr.shutdown();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }

        System.exit(0);
    }

    private static void
    usage(String progName)
    {
        System.err.println("Usage:");
        System.err.println(progName + " [options]\n" +
                           "\n" +
                           "Options:\n" +
                           "-h, --help             " +
                           "Show this message.\n" +
                           "-v, --version          " +
                           "Show Trading Service version.\n" +
                           "-i, --ior              " +
                           "Print IOR on standard output.\n" +
                           "-d, --dbdir DIR        " +
                           "Use DIR as the database directory.\n" +
                           "-t, --timeout MINS     " +
                           "Purge the database every MINS minutes.\n" +
                           "                       " +
                           "(The default is 5 minutes.)\n");
    }
}
