// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

//
// Things are just too ugly without this
//
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;
import com.ooc.OBCosTrading.*;

final class TypeDB_impl extends AbstractDB_impl implements TypeDB
{
    private IncarnationNumber inc_; // The next incarnation
    private java.util.Hashtable types_; // The types

    TypeDB_impl(org.omg.CORBA.ORB orb, String dbdir)
    {
        super(orb);

        inc_ = new IncarnationNumber(0, 1);
        types_ = new java.util.Hashtable(67);

        if(dbdir != null)
            super.initDatabase(dbdir, "types");
    }

    //
    // Called by super to execute commands read from database
    // during initialization
    //
    protected void
    execute(org.omg.CORBA.Any any)
    {
        org.omg.CORBA.TypeCode tc = any.type();
        if(tc.equal(CreateTypeHelper.type()))
        {
            CreateType cmd = CreateTypeHelper.extract(any);
            types_.put(cmd.name, cmd.ts);
        }
        else if(tc.equal(MaskTypeHelper.type()))
        {
            MaskType cmd = MaskTypeHelper.extract(any);
            TypeStruct ts = (TypeStruct)types_.get(cmd.name);
            if(ts == null)
            {
                com.ooc.CORBA.MessageViewer.instance().error(
                    "Missing type '" + cmd.name + "' in database.");
                throw new org.omg.CORBA.PERSIST_STORE();
            }
            ts.masked = true;
        }
        else if(tc.equal(UnmaskTypeHelper.type()))
        {
            UnmaskType cmd = UnmaskTypeHelper.extract(any);
            TypeStruct ts = (TypeStruct)types_.get(cmd.name);
            if(ts == null)
            {
                com.ooc.CORBA.MessageViewer.instance().error(
                    "Missing type '" + cmd.name + "' in database.");
                throw new org.omg.CORBA.PERSIST_STORE();
            }
            ts.masked = false;
        }
        else if(tc.equal(RemoveTypeHelper.type()))
        {
            RemoveType cmd = RemoveTypeHelper.extract(any);
            TypeStruct ts = (TypeStruct)types_.remove(cmd.name);
            if(ts == null)
            {
                com.ooc.CORBA.MessageViewer.instance().error(
                    "Missing type '" + cmd.name + "' in database.");
                throw new org.omg.CORBA.PERSIST_STORE();
            }
        }
        else if(tc.equal(IncarnationHelper.type()))
        {
            Incarnation cmd = IncarnationHelper.extract(any);
            inc_ = cmd.inc;
        }
        else
        {
            com.ooc.CORBA.MessageViewer.instance().error(
                "Unknown record in database.");
            throw new org.omg.CORBA.PERSIST_STORE();
        }
    }

    //
    // Called by super to update the journal
    //
    protected void
    update()
        throws java.io.IOException
    {
        org.omg.CORBA.Any any;

        //
        // Write the current incarnation number
        //
        any = orb_.create_any();
        Incarnation incCmd = new Incarnation(inc_);
        IncarnationHelper.insert(any, incCmd);
        writeLog(any);

        //
        // Write the types
        //
        java.util.Enumeration e = types_.keys();
        while(e.hasMoreElements())
        {
            String name = (String)e.nextElement();
            TypeStruct ts = (TypeStruct)types_.get(name);
            CreateType cmd = new CreateType(name, ts);
            any = orb_.create_any();
            CreateTypeHelper.insert(any, cmd);
            writeLog(any);
        }
    }

    //
    // Returns the TypeStruct for the type, or null if the
    // the type doesn't exist
    //
    public TypeStruct
    describeType(String name)
    {
        return (TypeStruct)types_.get(name);
    }

    //
    // Returns true if the type was found, false otherwise
    //
    public boolean
    maskType(String name)
    {
        TypeStruct ts = (TypeStruct)types_.get(name);
        if(ts != null)
        {
            if(os_ != null)
            {
                MaskType cmd = new MaskType(name);
                org.omg.CORBA.Any any = orb_.create_any();
                MaskTypeHelper.insert(any, cmd);
                writeLog(any);
                update_ = true;
            }

            ts.masked = true;
            return true;
        }
        return false;
    }

    //
    // Returns true if the type was found, false otherwise
    //
    public boolean
    unmaskType(String name)
    {
        TypeStruct ts = (TypeStruct)types_.get(name);
        if(ts != null)
        {
            if(os_ != null)
            {
                UnmaskType cmd = new UnmaskType(name);
                org.omg.CORBA.Any any = orb_.create_any();
                UnmaskTypeHelper.insert(any, cmd);
                writeLog(any);
                update_ = true;
            }

            ts.masked = false;
            return true;
        }
        return false;
    }

    //
    // Returns the names of all of the service types
    //
    public String[]
    getTypes()
    {
        String[] result = new String[types_.size()];
        java.util.Enumeration e = types_.keys();
        int i = 0;
        while(e.hasMoreElements())
            result[i++] = (String)e.nextElement();
        return result;
    }

    //
    // Returns the names of all of the service types whose incarnation
    // is the same or greater than the given one
    //
    public String[]
    getTypesSince(IncarnationNumber inc)
    {
        java.util.Vector vec = new java.util.Vector();
        java.util.Enumeration e = types_.keys();
        while(e.hasMoreElements())
        {
            String name = (String)e.nextElement();
            TypeStruct ts = (TypeStruct)types_.get(name);
            if(compare(ts.incarnation, inc) >= 0)
                vec.addElement(name);
        }

        String[] result = new String[vec.size()];
        vec.copyInto(result);
        return result;
    }

    //
    // Returns the current incarnation number (the one that will be
    // assigned to the next service type)
    //
    public IncarnationNumber
    getIncarnation()
    {
        return inc_;
    }

    //
    // Creates a new service type
    //
    public IncarnationNumber
    createType(String name,
               String interfaceName,
               PropStruct[] props,
               String[] superTypes)
    {
        TypeStruct ts = new TypeStruct();
        ts.if_name = interfaceName;
        ts.props = props;
        ts.super_types = superTypes;
        ts.masked = false;
        ts.incarnation = new IncarnationNumber(inc_.high, inc_.low);

        types_.put(name, ts);

        inc_.low++;
        if(inc_.low == 0) // handle wraparound
            inc_.high++;

        if(os_ != null)
        {
            org.omg.CORBA.Any any;

            CreateType cmd = new CreateType(name, ts);
            any = orb_.create_any();
            CreateTypeHelper.insert(any, cmd);
            writeLog(any);

            Incarnation incCmd = new Incarnation(inc_);
            any = orb_.create_any();
            IncarnationHelper.insert(any, incCmd);
            writeLog(any);

            update_ = true;
        }

        return ts.incarnation;
    }

    //
    // Returns true if the type was found, false otherwise
    //
    public boolean
    removeType(String name)
    {
        if(types_.containsKey(name))
        {
            if(os_ != null)
            {
                RemoveType cmd = new RemoveType(name);
                org.omg.CORBA.Any any = orb_.create_any();
                RemoveTypeHelper.insert(any, cmd);
                writeLog(any);
                update_ = true;
            }

            types_.remove(name);
            return true;
        }
        return false;
    }

    //
    // Returns the name of a subtype of the given type, or null if
    // no subtypes were found
    //
    public String
    findSubType(String name)
    {
        String result = null;

        java.util.Enumeration e = types_.keys();
        while(e.hasMoreElements() && result == null)
        {
            String str = (String)e.nextElement();
            TypeStruct ts = (TypeStruct)types_.get(str);
            for(int i = 0 ; i < ts.super_types.length ; i++)
            {
                if(name.equals(ts.super_types[i]))
                {
                    result = str;
                    break;
                }
            }
        }

        return result;
    }

    //
    // Returns the names of all supertypes of the given type,
    // or null if the type wasn't found
    //
    public String[]
    getAllSuperTypes(String name)
    {
        String[] result = null;

        TypeStruct ts = (TypeStruct)types_.get(name);
        if(ts != null)
        {
            java.util.Vector vec = findAllSuperTypes(ts);

            result = new String[vec.size()];
            vec.copyInto((java.lang.Object[])result);
        }

        return result;
    }

    private int
    compare(IncarnationNumber i1, IncarnationNumber i2)
    {
        long i1h = i1.high, i1l = i1.low, i2h = i2.high, i2l = i2.low;
        if(i1h < 0)
            i1h += (1L << 32);
        if(i1l < 0)
            i1l += (1L << 32);
        if(i2h < 0)
            i2h += (1L << 32);
        if(i2l < 0)
            i2l += (1L << 32);

        if(i1h > i2h)
            return 1;
        else if(i1h == i2h)
        {
            if(i1l > i2l)
                return 1;
            else if(i1l == i2l)
                return 0;
            else
                return -1;
        }
        else
            return -1;
    }

    //
    // Returns a vector of strings representing all the super types of
    // this type; this method is called recursively
    //
    private java.util.Vector
    findAllSuperTypes(TypeStruct ts)
    {
        java.util.Vector result = new java.util.Vector();

        //
        // Merge in all of the super types of our super types
        //
        for(int i = 0 ; i < ts.super_types.length ; i++)
        {
            result.addElement(ts.super_types[i]);
            TypeStruct ts2 = (TypeStruct)types_.get(ts.super_types[i]);
            java.util.Vector v = findAllSuperTypes(ts2);
            java.util.Enumeration e = v.elements();
            while(e.hasMoreElements())
            {
                String s = (String)e.nextElement();
                if(!result.contains(s))
                    result.addElement(s);
            }
        }

        return result;
    }
}
