// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

final class UnaryNode extends Node
{
    private int token_;
    private Node node_;

    UnaryNode(org.omg.CORBA.ORB orb, int token, Node node)
    {
        token_ = token;
        node_ = node;
    }

    void
    print(java.io.PrintStream out)
    {
        out.print("UnaryNode: ");

        switch(token_)
        {
        case Tokens.TOK_EXIST:
            out.print("exist");
            break;
        case Tokens.TOK_NOT:
            out.print("not");
            break;
        case Tokens.TOK_MINUS:
            out.print("-");
            break;
        case Tokens.TOK_PLUS:
            out.print("+");
            break;
        default:
            throw new RuntimeException("invalid unary operator");
        }

        out.println();

        node_.print(out);
    }

    Value
    evaluate(PropertyList props)
    {
        Value result = null;

        switch(token_)
        {
        case Tokens.TOK_EXIST:
        {
            boolean exists = node_.exists(props);
            result = new Value(exists);
            break;
        }

        case Tokens.TOK_NOT:
        {
            Value nv = node_.evaluate(props);
            if(nv != null && nv.kind() == Value.KIND_BOOLEAN)
                result = nv.not();
            break;
        }

        case Tokens.TOK_MINUS:
        {
            Value nv = node_.evaluate(props);
            if(nv != null && nv.numeric())
                result = nv.negate();
            break;
        }

        case Tokens.TOK_PLUS:
        {
            Value nv = node_.evaluate(props);
            if(nv != null && nv.numeric())
                result = nv;
            break;
        }

        default:
            throw new RuntimeException();
        }

        return result;
    }

    boolean
    checkTypes()
    {
        if(! node_.checkTypes())
            return false;

        int type = node_.type();

        switch(token_)
        {
        case Tokens.TOK_EXIST:
            return true;

        case Tokens.TOK_NOT:
            return (type == TYPE_BOOLEAN || type == TYPE_UNKNOWN);

        case Tokens.TOK_MINUS:
            return (node_.numeric() || type == TYPE_UNKNOWN);
        }

        return false;
    }

    int
    type()
    {
        int result;

        switch(token_)
        {
        case Tokens.TOK_EXIST:
        case Tokens.TOK_NOT:
            result = TYPE_BOOLEAN;
            break;

        case Tokens.TOK_MINUS:
        case Tokens.TOK_PLUS:
            result = TYPE_NUMBER;
            break;

        default:
            throw new RuntimeException();
        }

        return result;
    }
}
