// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTrading;

final class Value
{
    private int kind_;
    private java.lang.Object value_;

    static final int KIND_UNKNOWN     = 0;
    static final int KIND_ANY         = 1;
    static final int KIND_STRING      = 2;
    static final int KIND_BOOLEAN     = 3;
    static final int KIND_USHORT      = 4;
    static final int KIND_SHORT       = 5;
    static final int KIND_LONG        = 6;
    static final int KIND_ULONG       = 7;
    static final int KIND_LONG_LONG   = 8;
    static final int KIND_ULONG_LONG  = 9;
    static final int KIND_FLOAT       = 10;
    static final int KIND_DOUBLE      = 11;

    Value()
    {
        kind_ = KIND_UNKNOWN;
    }

    Value(org.omg.CORBA.ORB orb, org.omg.CORBA.Any value)
    {
        kind_ = KIND_UNKNOWN;
        setValue(orb, value);
    }

    Value(String value)
    {
        kind_ = KIND_STRING;
        value_ = value;
    }

    Value(boolean value)
    {
        kind_ = KIND_BOOLEAN;
        value_ = new Boolean(value);
    }

    Value(short value)
    {
        kind_ = KIND_SHORT;
        value_ = new Short(value);
    }

    Value(int value)
    {
        kind_ = KIND_LONG;
        value_ = new Integer(value);
    }

    Value(long value)
    {
        kind_ = KIND_LONG_LONG;
        value_ = new Long(value);
    }

    Value(float value)
    {
        kind_ = KIND_FLOAT;
        value_ = new Float(value);
    }

    Value(double value)
    {
        kind_ = KIND_DOUBLE;
        value_ = new Double(value);
    }

    Value(Value v)
    {
        kind_ = v.kind_;
        value_ = v.value_;
    }

    void
    setValue(org.omg.CORBA.ORB orb, org.omg.CORBA.Any value)
    {
        org.omg.CORBA.TypeCode tc = value.type();
        org.omg.CORBA.TypeCode origTC = getOrigType(tc);

        switch(origTC.kind().value())
        {
        case org.omg.CORBA.TCKind._tk_boolean:
            kind_ = KIND_BOOLEAN;
            value_ = new Boolean(value.extract_boolean());
            break;

        case org.omg.CORBA.TCKind._tk_octet:
        {
            int o = value.extract_octet();
            if(o < 0)
                o += 256;
            kind_ = KIND_LONG;
            value_ = new Integer(o);
            break;
        }

        case org.omg.CORBA.TCKind._tk_char:
            kind_ = KIND_STRING;
            value_ = "" + value.extract_char();
            break;

        case org.omg.CORBA.TCKind._tk_string:
            kind_ = KIND_STRING;
            value_ = value.extract_string();
            break;

        case org.omg.CORBA.TCKind._tk_short:
            kind_ = KIND_SHORT;
            value_ = new Short(value.extract_short());
            break;

        case org.omg.CORBA.TCKind._tk_ushort:
            kind_ = KIND_USHORT;
            value_ = new Short(value.extract_ushort());
            break;

        case org.omg.CORBA.TCKind._tk_long:
            kind_ = KIND_LONG;
            value_ = new Integer(value.extract_long());
            break;

        case org.omg.CORBA.TCKind._tk_ulong:
            kind_ = KIND_ULONG;
            value_ = new Integer(value.extract_ulong());
            break;

        case org.omg.CORBA.TCKind._tk_longlong:
            kind_ = KIND_LONG_LONG;
            value_ = new Long(value.extract_longlong());
            break;

        case org.omg.CORBA.TCKind._tk_ulonglong:
            kind_ = KIND_ULONG_LONG;
            value_ = new Long(value.extract_ulonglong());
            break;

        case org.omg.CORBA.TCKind._tk_float:
            kind_ = KIND_FLOAT;
            value_ = new Float(value.extract_float());
            break;

        case org.omg.CORBA.TCKind._tk_double:
            kind_ = KIND_DOUBLE;
            value_ = new Double(value.extract_double());
            break;

        default:
            kind_ = KIND_ANY;
            value_ = value;
        }
    }

    int
    kind()
    {
        return kind_;
    }

    org.omg.CORBA.Any
    get_any()
    {
        if(kind_ != KIND_ANY)
            throw new RuntimeException();
        return (org.omg.CORBA.Any)value_;
    }

    String
    get_string()
    {
        if(kind_ != KIND_STRING)
            throw new RuntimeException();
        return (String)value_;
    }

    boolean
    get_boolean()
    {
        if(kind_ != KIND_BOOLEAN)
            throw new RuntimeException();
        return ((Boolean)value_).booleanValue();
    }

    short
    get_short()
    {
        short result = (short)0;

        switch(kind_)
        {
        case KIND_BOOLEAN:
            result = (get_boolean() ? (short)1 : (short)0);
            break;
        case KIND_USHORT:
            result = ((Short)value_).shortValue();
            break;
        case KIND_SHORT:
            result = ((Short)value_).shortValue();
            break;
        case KIND_LONG:
            result = (short)get_long();
            break;
        case KIND_ULONG:
            result = (short)get_ulong();
            break;
        case KIND_LONG_LONG:
            result = (short)get_longlong();
            break;
        case KIND_ULONG_LONG:
            result = (short)get_ulonglong();
            break;
        case KIND_FLOAT:
            result = (short)get_float();
            break;
        case KIND_DOUBLE:
            result = (short)get_double();
            break;
        case KIND_STRING:
        {
            String str = (String)value_;
            if(str.length() != 1)
                throw new RuntimeException();
            result = (short)str.charAt(0);
            break;
        }
        default:
            throw new RuntimeException("unsupported conversion!");
        }

        return result;
    }

    short
    get_ushort()
    {
        return get_short();
    }

    int
    get_long()
    {
        int result = 0;

        switch(kind_)
        {
        case KIND_BOOLEAN:
            result = (get_boolean() ? 1 : 0);
            break;
        case KIND_USHORT:
            result = get_ushort();
            break;
        case KIND_SHORT:
            result = get_short();
            break;
        case KIND_LONG:
            result = ((Integer)value_).intValue();
            break;
        case KIND_ULONG:
            result = ((Integer)value_).intValue();
            break;
        case KIND_LONG_LONG:
            result = (int)get_longlong();
            break;
        case KIND_ULONG_LONG:
            result = (int)get_ulonglong();
            break;
        case KIND_FLOAT:
            result = (int)get_float();
            break;
        case KIND_DOUBLE:
            result = (int)get_double();
            break;
        case KIND_STRING:
        {
            String str = (String)value_;
            if(str.length() != 1)
                throw new RuntimeException();
            result = (int)str.charAt(0);
            break;
        }
        default:
            throw new RuntimeException("unsupported conversion!");
        }

        return result;
    }

    int
    get_ulong()
    {
        return get_long();
    }

    long
    get_longlong()
    {
        long result = 0;

        switch(kind_)
        {
        case KIND_BOOLEAN:
            result = (get_boolean() ? 1 : 0);
            break;
        case KIND_USHORT:
            result = get_ushort();
            break;
        case KIND_SHORT:
            result = get_short();
            break;
        case KIND_LONG:
            result = get_long();
            break;
        case KIND_ULONG:
            result = get_ulong();
            break;
        case KIND_LONG_LONG:
            result = ((Long)value_).longValue();
            break;
        case KIND_ULONG_LONG:
            result = ((Long)value_).longValue();
            break;
        case KIND_FLOAT:
            result = (long)get_float();
            break;
        case KIND_DOUBLE:
            result = (long)get_double();
            break;
        case KIND_STRING:
        {
            String str = (String)value_;
            if(str.length() != 1)
                throw new RuntimeException();
            result = (long)str.charAt(0);
            break;
        }
        default:
            throw new RuntimeException("unsupported conversion!");
        }

        return result;
    }

    long
    get_ulonglong()
    {
        return get_longlong();
    }

    float
    get_float()
    {
        float result = 0;

        switch(kind_)
        {
        case KIND_BOOLEAN:
            result = (get_boolean() ? 1 : 0);
            break;
        case KIND_USHORT:
            result = get_ushort();
            break;
        case KIND_SHORT:
            result = get_short();
            break;
        case KIND_LONG:
            result = get_long();
            break;
        case KIND_ULONG:
            result = get_ulong();
            break;
        case KIND_LONG_LONG:
            result = get_longlong();
            break;
        case KIND_ULONG_LONG:
            result = get_ulonglong();
            break;
        case KIND_FLOAT:
            result = ((Float)value_).floatValue();
            break;
        case KIND_DOUBLE:
            result = (float)get_double();
            break;
        case KIND_STRING:
        {
            String str = (String)value_;
            if(str.length() != 1)
                throw new RuntimeException();
            result = (float)str.charAt(0);
            break;
        }
        default:
            throw new RuntimeException("unsupported conversion!");
        }

        return result;
    }

    double
    get_double()
    {
        double result = 0;

        switch(kind_)
        {
        case KIND_BOOLEAN:
            result = (get_boolean() ? 1 : 0);
            break;
        case KIND_USHORT:
            result = get_ushort();
            break;
        case KIND_SHORT:
            result = get_short();
            break;
        case KIND_LONG:
            result = get_long();
            break;
        case KIND_ULONG:
            result = get_ulong();
            break;
        case KIND_LONG_LONG:
            result = get_longlong();
            break;
        case KIND_ULONG_LONG:
            result = get_ulonglong();
            break;
        case KIND_FLOAT:
            result = get_float();
            break;
        case KIND_DOUBLE:
            result = ((Double)value_).doubleValue();
            break;
        case KIND_STRING:
        {
            String str = (String)value_;
            if(str.length() != 1)
                throw new RuntimeException();
            result = (double)str.charAt(0);
            break;
        }
        default:
            throw new RuntimeException("unsupported conversion!");
        }

        return result;
    }

    boolean
    numeric()
    {
        boolean result = false;

        if(numericKind(kind_, false))
            result = true;
        else if(kind_ == KIND_STRING)
            result = (((String)value_).length() == 1);  // i.e., a char

        return result;
    }

    boolean
    zero()
    {
        boolean result = false;

        switch(kind_)
        {
        case KIND_BOOLEAN:
            result = (get_boolean() == false);
            break;
        case KIND_USHORT:
            result = (get_ushort() == 0);
            break;
        case KIND_SHORT:
            result = (get_short() == 0);
            break;
        case KIND_LONG:
            result = (get_long() == 0);
            break;
        case KIND_ULONG:
            result = (get_ulong() == 0);
            break;
        case KIND_LONG_LONG:
            result = (get_longlong() == 0);
            break;
        case KIND_ULONG_LONG:
            result = (get_ulonglong() == 0);
            break;
        case KIND_FLOAT:
            result = (get_float() == 0);
            break;
        case KIND_DOUBLE:
            result = (get_double() == 0);
            break;
        case KIND_STRING:
        {
            String str = (String)value_;
            if(str.length() != 1)
                throw new RuntimeException();
            result = (str.charAt(0) == (char)0);
            break;
        }
        }

        return result;
    }

    static boolean
    comparable(Value v1, Value v2)
    {
        boolean b = false;

        int k1 = v1.kind();
        int k2 = v2.kind();

        if((k1 == KIND_BOOLEAN && k2 == KIND_BOOLEAN) ||
           (v1.numeric() && v2.numeric()) ||
           (k1 == KIND_STRING && k2 == KIND_STRING))
            b = true;

        return b;
    }

    static boolean
    comparableKinds(int k1, int k2, boolean strict)
    {
        boolean b = false;

        if((k1 == KIND_BOOLEAN && k2 == KIND_BOOLEAN) ||
           (numericKind(k1, strict) && numericKind(k2, strict)) ||
           (k1 == KIND_STRING && k2 == KIND_STRING))
            b = true;

        return b;
    }

    static int
    toKind(org.omg.CORBA.TypeCode tc)
    {
        int result;
        org.omg.CORBA.TypeCode origTC = getOrigType(tc);

        switch(origTC.kind().value())
        {
        case org.omg.CORBA.TCKind._tk_boolean:
            result = KIND_BOOLEAN;
            break;

        case org.omg.CORBA.TCKind._tk_octet:
            result = KIND_LONG;
            break;

        case org.omg.CORBA.TCKind._tk_char:
        case org.omg.CORBA.TCKind._tk_string:
            result = KIND_STRING;
            break;

        case org.omg.CORBA.TCKind._tk_short:
            result = KIND_SHORT;
            break;

        case org.omg.CORBA.TCKind._tk_ushort:
            result = KIND_USHORT;
            break;

        case org.omg.CORBA.TCKind._tk_long:
            result = KIND_LONG;
            break;

        case org.omg.CORBA.TCKind._tk_ulong:
            result = KIND_ULONG;
            break;

        case org.omg.CORBA.TCKind._tk_longlong:
            result = KIND_LONG_LONG;
            break;

        case org.omg.CORBA.TCKind._tk_ulonglong:
            result = KIND_ULONG_LONG;
            break;

        case org.omg.CORBA.TCKind._tk_float:
            result = KIND_FLOAT;
            break;

        case org.omg.CORBA.TCKind._tk_double:
            result = KIND_DOUBLE;
            break;

        default:
            result = KIND_ANY;
        }

        return result;
    }

    Value
    contains(org.omg.CORBA.ORB orb, Value v)
    {
        //
        // The left-hand value cannot be a complex type (which
        // would result in its kind being KIND_ANY).
        //
        // The kind for the right-hand value must be KIND_ANY -
        // the sequence or array is contained in the any.
        //
        if(v.kind() == KIND_ANY || kind_ != KIND_ANY)
            return null;

        //
        // As an optimization, ensure that the type of v is compatible
        // with the type of the sequence/array elements.
        //
        org.omg.CORBA.Any any = (org.omg.CORBA.Any)value_;
        org.omg.CORBA.TypeCode origTC = getOrigType(any.type());
        if(origTC.kind() != org.omg.CORBA.TCKind.tk_sequence &&
           origTC.kind() != org.omg.CORBA.TCKind.tk_array)
            return null;

        org.omg.CORBA.TypeCode content = null, origContent = null;
        try
        {
            content = origTC.content_type();
            origContent = getOrigType(content);
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind ex)
        {
            throw new RuntimeException();
        }
        int k = toKind(origContent);

        if(!comparableKinds(k, v.kind(), false))
            return null;

        org.omg.CORBA.DynAny dynAny = orb.create_dyn_any(any);

        org.omg.CORBA.Any[] elements;
        if(origTC.kind() == org.omg.CORBA.TCKind.tk_sequence)
        {
            org.omg.CORBA.DynSequence dynSeq =
                org.omg.CORBA.DynSequenceHelper.narrow(dynAny);
            elements = dynSeq.get_elements();
        }
        else
        {
            org.omg.CORBA.DynArray dynArr =
                org.omg.CORBA.DynArrayHelper.narrow(dynAny);
            elements = dynArr.get_elements();
        }

        boolean found = false;

        for(int i = 0 ; !found && i < elements.length ; i++)
        {
            Value val = new Value(orb, elements[i]);
            if(v.equals(val))
                found = true;
        }

        dynAny.destroy();

        return new Value(found);
    }

    Value
    stringAdd(Value s)
    {
        if(kind_ != KIND_STRING || s.kind_ != KIND_STRING)
            throw new RuntimeException();
        return new Value((String)value_ + (String)s.value_);
    }

    Value
    stringSub(Value s)
    {
        if(kind_ != KIND_STRING || s.kind_ != KIND_STRING)
            throw new RuntimeException();
        String str1 = (String)value_;
        String str2 = (String)s.value_;
        int index = str1.indexOf(str2);
        if(index < 0)
            return this;
        else
        {
            String str = "";
            if(index > 0)
                str += str1.substring(0, index);
            if(index + str2.length() < str1.length())
                str += str1.substring(index + str2.length());
            return new Value(str);
        }
    }

    public String
    toString()
    {
        String result;

        switch(kind_)
        {
        case KIND_UNKNOWN:
            result = "<unknown>";
            break;
        case KIND_ANY:
            result = "<Any>";
            break;
        case KIND_STRING:
            result = "string(" + (String)value_ + ")";
            break;
        case KIND_BOOLEAN:
            result = "boolean(" + (Boolean)value_ + ")";
            break;
        case KIND_SHORT:
            result = "short(" + (Short)value_ + ")";
            break;
        case KIND_USHORT:
            result = "ushort(" + (Short)value_ + ")";
            break;
        case KIND_LONG:
            result = "long(" + (Integer)value_ + ")";
            break;
        case KIND_ULONG:
            result = "ulong(" + (Integer)value_ + ")";
            break;
        case KIND_LONG_LONG:
            result = "longlong(" + (Long)value_ + ")";
            break;
        case KIND_ULONG_LONG:
            result = "ulonglong(" + (Long)value_ + ")";
            break;
        case KIND_FLOAT:
            result = "float(" + (Float)value_ + ")";
            break;
        case KIND_DOUBLE:
            result = "double(" + (Double)value_ + ")";
            break;
        default:
            throw new RuntimeException();
        }

        return result;
    }

    boolean
    lessThan(Value v)
    {
        boolean result = false;

        //
        // Determine the kind to use for comparison
        //
        int op = compare(this, v);
        if(op == KIND_UNKNOWN)
            throw new RuntimeException();

        switch(op)
        {
        case KIND_BOOLEAN:
        case KIND_SHORT:
        case KIND_USHORT:
            result = (get_short() < v.get_short());
            break;
        case KIND_LONG:
        case KIND_ULONG:
            result = (get_long() < v.get_long());
            break;
        case KIND_LONG_LONG:
        case KIND_ULONG_LONG:
            result = (get_longlong() < v.get_longlong());
            break;
        case KIND_FLOAT:
            result = (get_float() < v.get_float());
            break;
        case KIND_DOUBLE:
            result = (get_double() < v.get_double());
            break;
        case KIND_STRING:
            result = (get_string().compareTo(v.get_string()) < 0);
            break;
        default:
            throw new RuntimeException("unsupported comparison kind!");
        }

        return result;
    }

    boolean
    lessThanEqual(Value v)
    {
        return (lessThan(v) || equals(v));
    }

    boolean
    greaterThan(Value v)
    {
        return (!lessThan(v) && !equals(v));
    }

    boolean
    greaterThanEqual(Value v)
    {
        return (equals(v) || !lessThan(v));
    }

    public boolean
    equals(java.lang.Object v)
    {
        Value val = (Value)v;

        boolean result = false;

        //
        // Determine the kind to use for comparison
        //
        int op = compare(this, val);
        if(op == KIND_UNKNOWN)
            throw new RuntimeException();

        switch(op)
        {
        case KIND_BOOLEAN:
        case KIND_SHORT:
        case KIND_USHORT:
            result = (get_short() == val.get_short());
            break;
        case KIND_LONG:
        case KIND_ULONG:
            result = (get_long() == val.get_long());
            break;
        case KIND_LONG_LONG:
        case KIND_ULONG_LONG:
            result = (get_longlong() == val.get_longlong());
            break;
        case KIND_FLOAT:
            result = (get_float() == val.get_float());
            break;
        case KIND_DOUBLE:
            result = (get_double() == val.get_double());
            break;
        case KIND_STRING:
            result = (get_string().compareTo(val.get_string()) == 0);
            break;
        default:
            throw new RuntimeException("unsupported comparison kind!");
        }

        return result;
    }

    Value
    add(Value v)
    {
        org.omg.CORBA.IntHolder op = new org.omg.CORBA.IntHolder();
        org.omg.CORBA.IntHolder result = new org.omg.CORBA.IntHolder();

        //
        // Determine the arithmetic conversions that are necessary
        //
        if(!convert(this, v, op, result))
            throw new RuntimeException();

        Value val;

        if(op.value == KIND_LONG_LONG)
            val = new Value(get_longlong() + v.get_longlong());
        else if(op.value == KIND_ULONG_LONG)
            val = new Value(get_ulonglong() + v.get_ulonglong());
        else if(op.value == KIND_LONG)
            val = new Value(get_long() + v.get_long());
        else if(op.value == KIND_ULONG)
            val = new Value(get_ulong() + v.get_ulong());
        else if(op.value == KIND_DOUBLE)
            val = new Value(get_double() + v.get_double());
        else
            throw new RuntimeException("unexpected kind for +");

        val.promote(result.value);

        return val;
    }

    Value
    subtract(Value v)
    {
        org.omg.CORBA.IntHolder op = new org.omg.CORBA.IntHolder();
        org.omg.CORBA.IntHolder result = new org.omg.CORBA.IntHolder();

        //
        // Determine the arithmetic conversions that are necessary
        //
        if(!convert(this, v, op, result))
            throw new RuntimeException();

        Value val;

        if(op.value == KIND_LONG_LONG)
            val = new Value(get_longlong() - v.get_longlong());
        else if(op.value == KIND_ULONG_LONG)
            val = new Value(get_ulonglong() - v.get_ulonglong());
        else if(op.value == KIND_LONG)
            val = new Value(get_long() - v.get_long());
        else if(op.value == KIND_ULONG)
            val = new Value(get_ulong() - v.get_ulong());
        else if(op.value == KIND_DOUBLE)
            val = new Value(get_double() - v.get_double());
        else
            throw new RuntimeException("unexpected kind for -");

        val.promote(result.value);

        return val;
    }

    Value
    multiply(Value v)
    {
        org.omg.CORBA.IntHolder op = new org.omg.CORBA.IntHolder();
        org.omg.CORBA.IntHolder result = new org.omg.CORBA.IntHolder();

        //
        // Determine the arithmetic conversions that are necessary
        //
        if(!convert(this, v, op, result))
            throw new RuntimeException();

        Value val;

        if(op.value == KIND_LONG_LONG)
            val = new Value(get_longlong() * v.get_longlong());
        else if(op.value == KIND_ULONG_LONG)
            val = new Value(get_ulonglong() * v.get_ulonglong());
        else if(op.value == KIND_LONG)
            val = new Value(get_long() * v.get_long());
        else if(op.value == KIND_ULONG)
            val = new Value(get_ulong() * v.get_ulong());
        else if(op.value == KIND_DOUBLE)
            val = new Value(get_double() * v.get_double());
        else
            throw new RuntimeException("unexpected kind for *");

        val.promote(result.value);

        return val;
    }

    Value
    divide(Value v)
    {
        org.omg.CORBA.IntHolder op = new org.omg.CORBA.IntHolder();
        org.omg.CORBA.IntHolder result = new org.omg.CORBA.IntHolder();

        //
        // Determine the arithmetic conversions that are necessary
        //
        if(!convert(this, v, op, result))
            throw new RuntimeException();

        Value val;

        if(op.value == KIND_LONG_LONG)
            val = new Value(get_longlong() / v.get_longlong());
        else if(op.value == KIND_ULONG_LONG)
            val = new Value(get_ulonglong() / v.get_ulonglong());
        else if(op.value == KIND_LONG)
            val = new Value(get_long() / v.get_long());
        else if(op.value == KIND_ULONG)
            val = new Value(get_ulong() / v.get_ulong());
        else if(op.value == KIND_DOUBLE)
            val = new Value(get_double() / v.get_double());
        else
            throw new RuntimeException("unexpected kind for /");

        val.promote(result.value);

        return val;
    }

    boolean
    twiddle(Value v)
    {
        if(kind_ != KIND_STRING || v.kind_ != KIND_STRING)
            throw new RuntimeException();

        return (((String)value_).indexOf((String)v.value_) >= 0);
    }

    Value
    negate()
    {
        Value result;

        switch(kind_)
        {
        case KIND_LONG_LONG:
        case KIND_ULONG_LONG:
            result = new Value(-get_longlong());
            break;
        case KIND_DOUBLE:
        case KIND_FLOAT:
            result = new Value(-get_double());
            break;
        default:
            result = new Value(-get_long());
            break;
        }

        return result;
    }

    Value
    not()
    {
        if(kind() != KIND_BOOLEAN)
            throw new RuntimeException();

        return new Value(!get_boolean());
    }

    static org.omg.CORBA.TypeCode
    getOrigType(org.omg.CORBA.TypeCode tc)
    {
        org.omg.CORBA.TypeCode result = tc;

        while(result.kind() == org.omg.CORBA.TCKind.tk_alias)
        {
            try
            {
                result = result.content_type();
            }
            catch(org.omg.CORBA.TypeCodePackage.BadKind ex)
            {
                throw new RuntimeException();
            }
        }

        return result;
    }

    private void
    promote(int kind)
    {
        switch(kind)
        {
        case KIND_BOOLEAN:
            value_ = new Boolean(get_boolean());
            break;
        case KIND_SHORT:
            value_ = new Short(get_short());
            break;
        case KIND_USHORT:
            value_ = new Short(get_ushort());
            break;
        case KIND_LONG:
            value_ = new Integer(get_long());
            break;
        case KIND_ULONG:
            value_ = new Integer(get_ulong());
            break;
        case KIND_LONG_LONG:
            value_ = new Long(get_longlong());
            break;
        case KIND_ULONG_LONG:
            value_ = new Long(get_ulonglong());
            break;
        case KIND_FLOAT:
            value_ = new Float(get_float());
            break;
        case KIND_DOUBLE:
            value_ = new Double(get_double());
            break;
        default:
            throw new RuntimeException("unexpected promotion kind!");
        }

        kind_ = kind;
    }

    private static boolean
    convert(Value v1, Value v2, org.omg.CORBA.IntHolder op,
            org.omg.CORBA.IntHolder result)
    {
        boolean b = true;

        int k1 = v1.kind();
        int k2 = v2.kind();

        if(k1 == KIND_DOUBLE || k2 == KIND_DOUBLE)
        {
            op.value = KIND_DOUBLE;
            result.value = KIND_DOUBLE;
        }
        else if(k1 == KIND_FLOAT || k2 == KIND_FLOAT)
        {
            op.value = KIND_DOUBLE;
            result.value = KIND_FLOAT;
        }
        else if(k1 == KIND_LONG_LONG || k2 == KIND_LONG_LONG)
        {
            op.value = KIND_LONG_LONG;
            result.value = KIND_LONG_LONG;
        }
        else
        {
            //
            // Strings of length 1 are considered characters, and therefore
            // are numeric
            //
            if(k1 == KIND_STRING)
            {
                if(v1.get_string().length() != 1)
                    b = false;
                else
                    k1 = KIND_LONG;
            }

            if(k2 == KIND_STRING)
            {
                if(v2.get_string().length() != 1)
                    b = false;
                else
                    k2 = KIND_LONG;
            }

            //
            // Select the "most strongly typed"
            //
            result.value = (k1 > k2 ? k1 : k2);
            if(k1 == KIND_ULONG && k2 == KIND_ULONG)
                op.value = KIND_ULONG;
            else
                op.value = KIND_LONG;
        }

        return b;
    }

    private static int
    compare(Value v1, Value v2)
    {
        int op;

        int k1 = v1.kind();
        int k2 = v2.kind();

        //
        // Determine the kind to use for comparison operators
        //

        if(k1 == KIND_BOOLEAN && k2 == KIND_BOOLEAN)
        {
            op = KIND_BOOLEAN;
        }
        else if(v1.numeric() && v2.numeric())
        {
            org.omg.CORBA.IntHolder opH = new org.omg.CORBA.IntHolder();
            org.omg.CORBA.IntHolder resultH = new org.omg.CORBA.IntHolder();
            convert(v1, v2, opH, resultH);
            op = opH.value;
        }
        else if(k1 == KIND_STRING && k2 == KIND_STRING)
        {
            op = KIND_STRING;
        }
        else
        {
            op = KIND_UNKNOWN;
        }

        return op;
    }

    private static boolean
    numericKind(int kind, boolean strict)
    {
        //
        // If we're being strict, then don't accept boolean as a numeric kind
        //
        if(strict && kind == KIND_BOOLEAN)
            return false;

        boolean result = false;

        switch(kind)
        {
        case KIND_BOOLEAN:
        case KIND_USHORT:
        case KIND_SHORT:
        case KIND_LONG:
        case KIND_ULONG:
        case KIND_LONG_LONG:
        case KIND_ULONG_LONG:
        case KIND_FLOAT:
        case KIND_DOUBLE:
            result = true;
            break;

        default:
            break;
        }

        return result;
    }

    private static boolean
    integerKind(int kind)
    {
        boolean result = false;

        switch(kind)
        {
        case KIND_BOOLEAN:  // Not sure we should include boolean types
        case KIND_USHORT:
        case KIND_SHORT:
        case KIND_LONG:
        case KIND_ULONG:
        case KIND_LONG_LONG:
        case KIND_ULONG_LONG:
            result = true;
            break;

        default:
            break;
        }

        return result;
    }
}
