// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole;

import com.ooc.CosTradingConsole.Link.*;
import com.ooc.CosTradingConsole.Offer.*;
import com.ooc.CosTradingConsole.Proxy.*;
import com.ooc.CosTradingConsole.Tools.*;
import com.ooc.CosTradingConsole.Type.*;
import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import org.omg.CosTrading.*;

public class ConsoleWindow extends JFrame
                           implements ActionListener,
                                      ClipboardListener,
                                      ItemListener,
                                      ListSelectionListener,
                                      ListDataListener
{
    private TraderModel model_;
    private TraderModelListener modelListener_;

    //
    // Menu items that can change their sensitivity
    //
    private JMenuItem menuItemCut_;
    private JMenuItem menuItemCopy_;
    private JMenuItem menuItemPaste_;
    private JMenuItem menuItemClone_;
    private JMenuItem menuItemModify_;
    private JMenuItem menuItemDelete_;
    private JMenuItem menuItemViewOffers_;
    private JMenuItem menuItemViewProxies_;
    private JMenuItem menuItemViewLinks_;
    private JMenuItem menuItemNewOffer_;
    private JMenuItem menuItemNewProxy_;
    private JMenuItem menuItemNewLink_;
    private JMenuItem menuItemAttributes_;
    private JMenuItem menuItemWithdraw_;
    private JMenuItem menuItemMask_;
    private JMenuItem menuItemUnmask_;

    private JButton toolBarItemCut_;
    private JButton toolBarItemCopy_;
    private JButton toolBarItemPaste_;
    private JButton toolBarItemModify_;
    private JButton toolBarItemDelete_;

    private Action cutAction_;
    private Action copyAction_;
    private Action pasteAction_;
    private Action modifyAction_;
    private Action deleteAction_;
    private Action queryAction_;

    private JComboBox combo_;
    private JList list_;
    private Summary summary_;
    private IORStatusBar status_;

    private BaseController currentController_;
    private TypeController typeController_;
    private OfferController offerController_;
    private ProxyController proxyController_;
    private LinkController linkController_;

    private ConnectDialog connectDialog_;
    private QueryDialog queryDialog_;
    private AttributesDialog attributesDialog_;
    private WithdrawDialog withdrawDialog_;

    //
    // Threshold to determine whether we use immediate
    // notification or batch notification when removing items
    //
    private static final int NOTIFY_THRESHOLD = 5;

    //
    // Used to keep track of the number of open console windows;
    // when the number reaches 0, we exit (see checkForExit)
    //
    private static int windowCount_ = 0;

    //
    // Toolbar icons
    //
    private static Icon newIcon_ = null;
    private static Icon cutIcon_ = null;
    private static Icon copyIcon_ = null;
    private static Icon pasteIcon_ = null;
    private static Icon modifyIcon_ = null;
    private static Icon deleteIcon_ = null;
    private static Icon queryIcon_ = null;
    private static Icon refreshIcon_ = null;


    public
    ConsoleWindow(String title, Connection connection)
    {
        super(title);

        //
        // Increment the console window count
        //
        synchronized (this.getClass())
        {
            windowCount_++;
        }

        getContentPane().setLayout(new BorderLayout());

        this.addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent e)
                {
                    checkForExit();
                }
            }
        );


        status_ = new IORStatusBar();
        getContentPane().add("South", status_);


        model_ = new TraderModel(status_);
        model_.setConnection(connection);


        createActions();
        getContentPane().add("North", createMenu());


        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        loadImages();
        panel.add("North", createToolBar());

        JPanel left = new JPanel();
        left.setLayout(new BorderLayout());

        combo_ = AppStandards.createComboBox();
        combo_.setPreferredSize(new Dimension(150, 25));
        combo_.setMaximumSize(new Dimension(150, 25));
        combo_.setAlignmentY((float)0.5);
        combo_.setEditable(false);
        combo_.addItemListener(this);
        left.add("North", combo_);

        list_ = AppStandards.createList();
        list_.addListSelectionListener(this);
        list_.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        JScrollPane listScroll = AppStandards.createScrollPane(list_);
        listScroll.setPreferredSize(new Dimension(150, 25));
        listScroll.setMinimumSize(new Dimension(150, 25));
        left.add("Center", listScroll);

        JPanel right = new JPanel();
        right.setLayout(new BorderLayout());

        summary_ = new Summary();
        right.add("Center", summary_);

        JSplitPane splitter =
            new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, left, right);
        //splitter.setBorder(BorderFactory.createEmptyBorder(5, 0, 0, 0));

        panel.add("Center", splitter);

        getContentPane().add("Center", panel);

        Clipboard.instance().addClipboardListener(this);

        initialize();
    }

    public static ConsoleWindow
    createNewWindow(Connection conn)
    {
        ConsoleWindow result;

        result = new ConsoleWindow(AppStandards.getString("ConsoleTitleKey"),
                                   conn);
        result.setSize(600, 400);

        return result;
    }

    // invoked after a change to our connection
    protected void
    initialize()
    {
        //
        // Initialize the menu item sensitivity
        //
        menuItemCut_.setEnabled(false);
        menuItemCopy_.setEnabled(false);
        menuItemPaste_.setEnabled(Clipboard.instance().getData() != null);
        menuItemClone_.setEnabled(false);
        menuItemModify_.setEnabled(false);
        menuItemDelete_.setEnabled(false);
        menuItemNewOffer_.setEnabled(model_.getRegister() != null);
        menuItemNewProxy_.setEnabled(model_.getProxy() != null);
        menuItemNewLink_.setEnabled(model_.getLink() != null);
        menuItemViewOffers_.setEnabled(model_.getAdmin() != null);
        menuItemViewProxies_.setEnabled(model_.getAdmin() != null);
        menuItemViewLinks_.setEnabled(model_.getLink() != null);
        menuItemAttributes_.setEnabled(model_.getAdmin() != null);
        menuItemWithdraw_.setEnabled(model_.getRegister() != null);
        menuItemMask_.setEnabled(false);
        menuItemUnmask_.setEnabled(false);

        //
        // Initialize the toolbar item sensitivity
        //
        toolBarItemCut_.setEnabled(false);
        toolBarItemCopy_.setEnabled(false);
        toolBarItemPaste_.setEnabled(Clipboard.instance().getData() != null);
        toolBarItemModify_.setEnabled(false);
        toolBarItemDelete_.setEnabled(false);

        combo_.removeItemListener(this);
        if(combo_.getItemCount() > 0)
            combo_.removeAllItems();

        model_.removeAllModelListeners();

        if(currentController_ != null)
            currentController_.clear();

        typeController_ = new TypeController(this, model_);
        combo_.addItem(typeController_);

        if(model_.getRegister() != null)
        {
            offerController_ = new OfferController(this, model_);
            combo_.addItem(offerController_);
        }
        else
            offerController_ = null;

        if(model_.getProxy() != null)
        {
            proxyController_ = new ProxyController(this, model_);
            combo_.addItem(proxyController_);
        }
        else
            proxyController_ = null;

        if(model_.getLink() != null)
        {
            linkController_ = new LinkController(this, model_);
            combo_.addItem(linkController_);
        }
        else
            linkController_ = null;

        // we want our listener to always be last, after the
        // controllers
        modelListener_ = createModelListener();
        model_.addModelListener(modelListener_);

        combo_.addItemListener(this);
        viewList(typeController_);

        // display the Lookup IOR in the status line
        status_.displayIOR(model_.getConnection().getLookup());
    }

    // From ActionListener
    public void
    actionPerformed(ActionEvent e)
    {
        if(e.getActionCommand().equals("connect"))
        {
            Lookup trader = connectDialog_.getTrader();
            model_.setConnection(new Connection(trader));
            initialize();
        }
    }

    // From ClipboardListener
    public void
    clipboardChanged(ClipboardData data)
    {
        updateItemSensitivity();
    }

    // From ItemListener
    public void
    itemStateChanged(ItemEvent e)
    {
        if(e.getStateChange() == ItemEvent.SELECTED)
        {
            BaseController bc = (BaseController)e.getItem();
            viewList(bc);
        }
    }

    // From ListSelectionListener
    public void
    valueChanged(ListSelectionEvent e)
    {
        updateItemSensitivity();
        displaySelectedItem();
    }

    // From ListDataListener
    public void
    contentsChanged(ListDataEvent e)
    {
        // nothing to do
    }

    // From ListDataListener
    public void
    intervalAdded(ListDataEvent e)
    {
        list_.clearSelection();
    }

    // From ListDataListener
    public void
    intervalRemoved(ListDataEvent e)
    {
        list_.clearSelection();
    }

    //
    // Enable and disable menu and toolbar items
    //
    protected void
    updateItemSensitivity()
    {
        Object[] values = list_.getSelectedValues();
        boolean enabled = (values != null && values.length > 0);
        boolean modify = (enabled &&
                          currentController_.isEditable() &&
                          values.length == 1);
        boolean clone = (enabled && values.length == 1);
        boolean mask  = (enabled && currentController_ == typeController_);
        boolean paste  = (Clipboard.instance().getData() != null);

        //
        // Update menu item sensitivity
        //
        menuItemCut_.setEnabled(enabled);
        menuItemCopy_.setEnabled(enabled);
        menuItemPaste_.setEnabled(paste);
        menuItemClone_.setEnabled(clone);
        menuItemModify_.setEnabled(modify);
        menuItemDelete_.setEnabled(enabled);
        menuItemMask_.setEnabled(mask);
        menuItemUnmask_.setEnabled(mask);

        //
        // Update toolbar item sensitivity
        //
        toolBarItemCut_.setEnabled(enabled);
        toolBarItemCopy_.setEnabled(enabled);
        toolBarItemPaste_.setEnabled(paste);
        toolBarItemModify_.setEnabled(modify);
        toolBarItemDelete_.setEnabled(enabled);
    }

    protected void
    displaySelectedItem()
    {
        Object[] values = list_.getSelectedValues();

        if(values.length == 1)
            summary_.setValue(currentController_, values[0]);
        else
            summary_.clear();
    }

    protected void
    updateSelectedItem(String name, BaseController controller)
    {
        // ignore if the controller doesn't match our current controller
        if(controller != currentController_)
            return;

        Object item = controller.findItem(name);
        Object[] values = list_.getSelectedValues();

        if(values != null && values.length == 1 && item == values[0])
            summary_.refresh();
    }

    protected JMenuBar
    createMenu()
    {
        JMenuBar result;
        JMenu menu;
        JMenuItem item;

        result = new JMenuBar();

        // Console menu
        menu = new JMenu(AppStandards.getString("ConsoleMenuKey"));
        menu.setMnemonic(AppStandards.getChar("ConsoleMnemKey"));
        result.add(menu);

        item = createMenuItem("NewWindow");
        item.addActionListener(new NewWindowAction());
        menu.add(item);

        item = createMenuItem("Connect");
        item.addActionListener(new ConnectAction());
        menu.add(item);

        menu.addSeparator();

        item = createMenuItem("Close");
        item.addActionListener(new CloseAction());
        menu.add(item);

        item = createMenuItem("Exit");
        item.addActionListener(new ExitAction());
        menu.add(item);

        // Edit menu
        menu = new JMenu(AppStandards.getString("EditMenuKey"));
        menu.setMnemonic(AppStandards.getChar("EditMnemKey"));
        result.add(menu);

        menuItemCut_ = createMenuItem("Cut");
        menuItemCut_.addActionListener(cutAction_);
        menu.add(menuItemCut_);

        menuItemCopy_ = createMenuItem("Copy");
        menuItemCopy_.addActionListener(copyAction_);
        menu.add(menuItemCopy_);

        menuItemPaste_ = createMenuItem("Paste");
        menuItemPaste_.addActionListener(pasteAction_);
        menu.add(menuItemPaste_);

        item = createMenuItem("SelectAll");
        item.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    selectAllItems();
                }
            });
        menu.add(item);

        menu.addSeparator();

        menuItemClone_ = createMenuItem("Clone");
        menuItemClone_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    cloneItem();
                }
            });
        menu.add(menuItemClone_);

        menuItemModify_ = createMenuItem("Modify");
        menuItemModify_.addActionListener(modifyAction_);
        menu.add(menuItemModify_);

        menuItemDelete_ = createMenuItem("Delete");
        menuItemDelete_.addActionListener(deleteAction_);
        menu.add(menuItemDelete_);

        // View menu
        menu = new JMenu(AppStandards.getString("ViewMenuKey"));
        menu.setMnemonic(AppStandards.getChar("ViewMnemKey"));
        result.add(menu);

        item = createMenuItem("ViewTypes");
        item.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    viewList(typeController_);
                }
            });
        menu.add(item);

        menuItemViewOffers_ = createMenuItem("ViewOffers");
        menuItemViewOffers_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    viewList(offerController_);
                }
            });
        menu.add(menuItemViewOffers_);

        menuItemViewProxies_ = createMenuItem("ViewProxies");
        menuItemViewProxies_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    viewList(proxyController_);
                }
            });
        menu.add(menuItemViewProxies_);

        menuItemViewLinks_ = createMenuItem("ViewLinks");
        menuItemViewLinks_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    viewList(linkController_);
                }
            });
        menu.add(menuItemViewLinks_);

        menu.addSeparator();

        item = createMenuItem("Refresh");
        item.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    refreshList();
                }
            });
        menu.add(item);

        // Insert menu
        menu = new JMenu(AppStandards.getString("InsertMenuKey"));
        menu.setMnemonic(AppStandards.getChar("InsertMnemKey"));
        result.add(menu);

        item = createMenuItem("InsertType");
        item.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    typeController_.newItem();
                }
            });
        menu.add(item);

        menuItemNewOffer_ = createMenuItem("InsertOffer");
        menuItemNewOffer_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    offerController_.newItem();
                }
            });
        menu.add(menuItemNewOffer_);

        menuItemNewProxy_ = createMenuItem("InsertProxy");
        menuItemNewProxy_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    proxyController_.newItem();
                }
            });
        menu.add(menuItemNewProxy_);

        menuItemNewLink_ = createMenuItem("InsertLink");
        menuItemNewLink_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    linkController_.newItem();
                }
            });
        menu.add(menuItemNewLink_);

        // Tools menu
        menu = new JMenu(AppStandards.getString("ToolsMenuKey"));
        menu.setMnemonic(AppStandards.getChar("ToolsMnemKey"));
        result.add(menu);

        item = createMenuItem("Query");
        item.addActionListener(queryAction_);
        menu.add(item);

        menuItemAttributes_ = createMenuItem("Attributes");
        menuItemAttributes_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    createAttributesDialog().setVisible(true);
                }
            });
        menu.add(menuItemAttributes_);

        menuItemWithdraw_ = createMenuItem("Withdraw");
        menuItemWithdraw_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    createWithdrawDialog().setVisible(true);
                }
            });
        menu.add(menuItemWithdraw_);

        menu.addSeparator();

        menuItemMask_ = createMenuItem("Mask");
        menuItemMask_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    maskTypes();
                }
            });
        menu.add(menuItemMask_);

        menuItemUnmask_ = createMenuItem("Unmask");
        menuItemUnmask_.addActionListener(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    unmaskTypes();
                }
            });
        menu.add(menuItemUnmask_);

        return result;
    }

    protected JComponent
    createToolBar()
    {
        JPanel result;
        JToolBar toolbar;
        JButton b;
        Insets margin = new Insets(2, 2, 2, 2);

        result = new JPanel();
        result.setLayout(new BorderLayout());
        result.setBorder(BorderFactory.createEmptyBorder(3, 0, 3, 0));

        toolbar = new JToolBar();
        toolbar.setFloatable(false);

        b = toolbar.add(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    currentController_.newItem();
                }
            });
        b.setToolTipText(AppStandards.getString("ToolBarNewKey"));
        b.setIcon(newIcon_);
        b.setMargin(margin);

        toolbar.addSeparator();

        toolBarItemCut_ = toolbar.add(cutAction_);
        toolBarItemCut_.setToolTipText(
            AppStandards.getString("ToolBarCutKey"));
        toolBarItemCut_.setIcon(cutIcon_);
        toolBarItemCut_.setMargin(margin);

        toolBarItemCopy_ = toolbar.add(copyAction_);
        toolBarItemCopy_.setToolTipText(
            AppStandards.getString("ToolBarCopyKey"));
        toolBarItemCopy_.setIcon(copyIcon_);
        toolBarItemCopy_.setMargin(margin);

        toolBarItemPaste_ = toolbar.add(pasteAction_);
        toolBarItemPaste_.setToolTipText(
            AppStandards.getString("ToolBarPasteKey"));
        toolBarItemPaste_.setIcon(pasteIcon_);
        toolBarItemPaste_.setMargin(margin);

        toolbar.addSeparator();

        toolBarItemModify_ = toolbar.add(modifyAction_);
        toolBarItemModify_.setToolTipText(
            AppStandards.getString("ToolBarModifyKey"));
        toolBarItemModify_.setIcon(modifyIcon_);
        toolBarItemModify_.setMargin(margin);

        toolBarItemDelete_ = toolbar.add(deleteAction_);
        toolBarItemDelete_.setToolTipText(
            AppStandards.getString("ToolBarDeleteKey"));
        toolBarItemDelete_.setIcon(deleteIcon_);
        toolBarItemDelete_.setMargin(margin);

        toolbar.addSeparator();

        b = toolbar.add(queryAction_);
        b.setToolTipText(AppStandards.getString("ToolBarQueryKey"));
        b.setIcon(queryIcon_);
        b.setMargin(margin);

        toolbar.addSeparator();

        b = toolbar.add(new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent e)
                {
                    refreshList();
                }
            });
        b.setToolTipText(AppStandards.getString("ToolBarRefreshKey"));
        b.setIcon(refreshIcon_);
        b.setMargin(margin);

        result.add("North", toolbar);

        return result;
    }

    protected TraderModelListener
    createModelListener()
    {
        TraderModelListener result;

        // This listener receives notification when items have
        // been modified; if the modified item is currently
        // selected, we'll need to update the description of the
        // item
        result = new TraderModelAdapter()
        {
            public void
            typeModified(String name)
            {
                updateSelectedItem(name, typeController_);
            }

            public void
            offerModified(String id)
            {
                updateSelectedItem(id, offerController_);
            }

            public void
            proxyModified(String id)
            {
                updateSelectedItem(id, proxyController_);
            }

            public void
            linkModified(String name)
            {
                updateSelectedItem(name, linkController_);
            }
        };

        return result;
    }


    //
    // Create actions
    //
    private void
    createActions()
    {
        cutAction_ = new CutAction();
        copyAction_ = new CopyAction();
        pasteAction_ = new PasteAction();
        modifyAction_ = new ModifyAction();
        deleteAction_ = new DeleteAction();
        queryAction_ = new QueryAction();
    }

    //
    // Create menu item with accelerator
    //
    private JMenuItem
    createMenuItem(String key)
    {
        JMenuItem item = new JMenuItem(AppStandards.getString(key + "Key"));
        item.setMnemonic(AppStandards.getChar(key + "MnemKey"));

        //char accel = AppStandards.getChar(key + "AccelKey");
        //if(accel != 0)
        //    item.setAccelerator(KeyStroke.getKeyStroke(
        //        accel, java.awt.event.KeyEvent.ALT_MASK));

        return item;
    }

    //
    // Load the images needed for the toolbar buttons
    //
    private void
    loadImages()
    {
        if(newIcon_ == null)
        {
            newIcon_ = getImageResource("new.gif");
            cutIcon_ = getImageResource("cut.gif");
            copyIcon_ = getImageResource("copy.gif");
            pasteIcon_ = getImageResource("paste.gif");
            modifyIcon_ = getImageResource("edit.gif");
            deleteIcon_ = getImageResource("delete.gif");
            refreshIcon_ = getImageResource("refresh.gif");
            queryIcon_ = getImageResource("query.gif");
        }
    }

    //
    // Obtain an image as a resource; the imageFile is expected to
    // be found in the "images" subdirectory relative to the location
    // of the ConsoleWindow class
    //
    private Icon
    getImageResource(String imageFile)
    {
        Icon result = null;

        java.net.URL url = getClass().getResource("images/" + imageFile);
        if (url != null)
            result = new ImageIcon(url);

        return result;
    }

    //
    // Determine if we should exit
    //
    static synchronized void
    checkForExit()
    {
        windowCount_--;
        if(windowCount_ == 0)
            System.exit(0);
    }

    //
    // Close the console window
    //
    void
    closeWindow()
    {
        setVisible(false);
        dispose();
        checkForExit();
    }

    //
    // Create the Connect dialog
    //
    synchronized JDialog
    createConnectDialog()
    {
        if(connectDialog_ == null)
        {
            connectDialog_ = new ConnectDialog(ConsoleWindow.this,
                                               model_,
                                               ConsoleWindow.this);
            connectDialog_.setActionCommand("connect");
        }

        return connectDialog_;
    }

    void
    cutItems()
    {
        BaseController controller = currentController_;
        Object[] values = list_.getSelectedValues();

        if(values == null || values.length == 0)
        {
            MessageDialog.showError(this,
                AppStandards.getString("NothingSelectedKey"));
            return;
        }

        // cut is implemented as a copy then a delete; the delete
        // is not performed if the copy fails
        if(confirmDelete(values))
        {
            if(copyImpl(controller, values))
            {
                list_.clearSelection();
                deleteImpl(controller, values);
            }
        }
    }

    void
    copyItems()
    {
        BaseController controller = currentController_;
        Object[] values = list_.getSelectedValues();

        if(values == null || values.length == 0)
        {
            MessageDialog.showError(this,
                AppStandards.getString("NothingSelectedKey"));
            return;
        }

        copyImpl(controller, values);
    }

    boolean
    copyImpl(BaseController controller, Object[] values)
    {
        boolean result = true;

        // a Hashtable is what we store in the clipboard
        Hashtable hash = new Hashtable();
        //Integer[] args = new Integer[2];
        //args[1] = new Integer(values.length);
        for(int i = 0 ; i < values.length ; i++)
        {
            StringBuffer msg = new StringBuffer();
            Object obj = controller.getItemData(values[i], msg);
            if(obj == null)
            {
                MessageDialog.showError(this,
                    AppStandards.getFormattedString("CannotCopyKey",
                        values[i].toString()));
                result = false;
                break;
            }
            hash.put(values[i].toString(), obj);
        }

        if(result)
            Clipboard.instance().setData(
                new ClipboardData(controller.getItemType(), hash));

        return result;
    }

    void
    pasteItems()
    {
        ClipboardData clipData = Clipboard.instance().getData();
        Object dataType = clipData.type;

        if(clipData == null)
        {
            MessageDialog.showError(this,
                AppStandards.getString("ClipboardEmptyKey"));
            return;
        }

        BaseController controller = null;

        if(dataType.equals(typeController_.getItemType()))
            controller = typeController_;
        else if(offerController_ != null &&
                dataType.equals(offerController_.getItemType()))
            controller = offerController_;
        else if(proxyController_ != null &&
                dataType.equals(proxyController_.getItemType()))
            controller = proxyController_;
        else if(linkController_ != null &&
                dataType.equals(linkController_.getItemType()))
            controller = linkController_;

        if(controller == null)
        {
            MessageDialog.showError(ConsoleWindow.this,
                AppStandards.getString("CannotPasteTraderKey"));
            return;
        }

        Hashtable hash = (Hashtable)clipData.data;
        pasteImpl(controller, hash);
    }

    void
    pasteImpl(BaseController controller, Hashtable hash)
    {
        int len = hash.size();

        // determine whether we notify the model listeners immediately
        // after each addition, or all at once afterward
        boolean notify = (len < NOTIFY_THRESHOLD);

        int count = 0;
        int i = 0;
        Enumeration e = hash.keys();
        //Integer[] args = new Integer[2];
        //args[1] = new Integer(len);
        while(e.hasMoreElements())
        {
            String key = (String)e.nextElement();
            Object itemData = hash.get(key);

            StringBuffer msg = new StringBuffer();
            boolean ok = controller.addItem(key, itemData, notify, msg);
            if(! ok)
            {
                if(i < len - 1)
                {
                    String errMsg =
                        AppStandards.getFormattedString("CannotPasteContKey",
                            msg.toString());
                    int yesNo = MessageDialog.showConfirm(this, errMsg);
                    if(yesNo == MessageDialog.NO)
                        break;
                }
                else
                    MessageDialog.showError(this,
                        AppStandards.getFormattedString("CannotPasteKey",
                                                        msg.toString()));
            }
            else
                count++;

            i++;
        }

        if(count > 0 && controller == currentController_)
            list_.clearSelection();

        if(count > 0 && ! notify)
            controller.fireItemsAdded();
    }

    void
    selectAllItems()
    {
        int size = currentController_.getListModel().getSize();
        list_.setSelectionInterval(0, size - 1);
    }

    void
    cloneItem()
    {
        Object value = list_.getSelectedValue();

        if(value == null)
        {
            MessageDialog.showError(this,
                AppStandards.getString("NothingSelectedKey"));
            return;
        }

        StringBuffer message = new StringBuffer();
        if(! currentController_.cloneItem(value, message))
            MessageDialog.showError(this, message.toString());
    }

    void
    modifyItem()
    {
        Object value = list_.getSelectedValue();

        if(value == null)
        {
            MessageDialog.showError(this,
                AppStandards.getString("NothingSelectedKey"));
            return;
        }

        StringBuffer message = new StringBuffer();
        if(! currentController_.editItem(value, message))
            MessageDialog.showError(this, message.toString());
    }

    void
    deleteItems()
    {
        BaseController controller = currentController_;
        Object[] values = list_.getSelectedValues();

        if(values == null || values.length == 0)
        {
            MessageDialog.showError(this,
                AppStandards.getString("NothingSelectedKey"));
            return;
        }

        if(confirmDelete(values))
            deleteImpl(controller, values);
    }

    void
    deleteImpl(BaseController controller, Object[] values)
    {
        // determine whether we notify the model listeners immediately
        // after each remove, or all at once afterward
        boolean notify = (values.length < NOTIFY_THRESHOLD);

        //Integer[] args = new Integer[2];
        //args[1] = new Integer(values.length);
        int count = 0;
        for(int i = 0 ; i < values.length ; i++)
        {
            StringBuffer msg = new StringBuffer();
            boolean ok = controller.destroyItem(values[i], notify, msg);
            if(! ok)
            {
                if(i < values.length - 1)
                {
                    String errMsg =
                        AppStandards.getFormattedString("CannotDeleteContKey",
                            msg.toString());
                    int yesNo = MessageDialog.showConfirm(this, errMsg);
                    if(yesNo == MessageDialog.NO)
                        break;
                }
                else
                    MessageDialog.showError(this,
                        AppStandards.getFormattedString("CannotDeleteKey",
                            msg.toString()));
            }
            else
                count++;
        }

        if(count > 0 && controller == currentController_)
            list_.clearSelection();

        if(count > 0 && ! notify)
            controller.fireItemsDestroyed();
    }

    boolean
    confirmDelete(Object[] values)
    {
        String message;
        if(values.length == 1)
            message = AppStandards.getFormattedString("ConfirmDeleteKey",
                values[0].toString());
        else
            message = AppStandards.getFormattedString("ConfirmMultDeleteKey",
                new Integer(values.length));

        int answer = MessageDialog.showConfirm(this, message);

        return (answer == MessageDialog.YES);
    }

    void
    viewList(BaseController controller)
    {
        try
        {
            if(controller != currentController_)
            {
                if(currentController_ != null)
                {
                    currentController_.getListModel().
                        removeListDataListener(this);
                    currentController_.clear();
                }
                currentController_ = controller;
                combo_.setSelectedItem(currentController_);
                ListModel listModel = currentController_.getListModel();
                list_.setModel(listModel);
                currentController_.refresh();
                listModel.addListDataListener(this);
                summary_.clear();
            }
        }
        catch(org.omg.CORBA.SystemException e)
        {
            MessageDialog.showSystemError(this, e);
        }
    }

    //
    // Refresh the list of items
    //
    void
    refreshList()
    {
        try
        {
            if(currentController_ != null)
            {
                currentController_.clear();
                currentController_.refresh();
            }
        }
        catch(org.omg.CORBA.SystemException e)
        {
            MessageDialog.showSystemError(this, e);
        }
    }

    synchronized JDialog
    createQueryDialog()
    {
        if(queryDialog_ == null)
            queryDialog_ = new QueryDialog(ConsoleWindow.this, model_);

        return queryDialog_;
    }

    synchronized JDialog
    createAttributesDialog()
    {
        if(attributesDialog_ == null)
            attributesDialog_ =
                new AttributesDialog(ConsoleWindow.this, model_);

        return attributesDialog_;
    }

    synchronized JDialog
    createWithdrawDialog()
    {
        if(withdrawDialog_ == null)
            withdrawDialog_ = new WithdrawDialog(ConsoleWindow.this, model_);

        return withdrawDialog_;
    }

    void
    maskTypes()
    {
        final Object[] values = list_.getSelectedValues();

        if(values == null || values.length == 0)
        {
            MessageDialog.showError(this,
                AppStandards.getString("NothingSelectedKey"));
            return;
        }

        maskImpl(values, true);
    }

    void
    unmaskTypes()
    {
        final Object[] values = list_.getSelectedValues();

        if(values == null || values.length == 0)
        {
            MessageDialog.showError(this,
                AppStandards.getString("NothingSelectedKey"));
            return;
        }

        maskImpl(values, false);
    }

    void
    maskImpl(Object[] values, boolean masked)
    {
        boolean notify = true;

        Integer[] args = new Integer[2];
        args[1] = new Integer(values.length);
        int count = 0;
        for(int i = 0 ; i < values.length ; i++)
        {
            args[0] = new Integer(i + 1);
            StringBuffer msg = new StringBuffer();
            model_.modifyType(values[i].toString(), masked, notify, msg);
            count++;
        }
    }

    //
    // Create a new console window
    //
    class NewWindowAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            Lookup lookup = model_.getLookup();
            ConsoleWindow win = createNewWindow(new Connection(lookup));
            win.setVisible(true);
        }
    }

    //
    // Connect to a new trader
    //
    class ConnectAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            JDialog dialog = ConsoleWindow.this.createConnectDialog();
            dialog.setVisible(true);
        }
    }

    //
    // Close the console window
    //
    class CloseAction implements ActionListener
    {
        public void
        actionPerformed(ActionEvent e)
        {
            ConsoleWindow.this.closeWindow();
        }
    }

    //
    // Quit
    //
    class ExitAction implements ActionListener
    {
        public void
        actionPerformed(ActionEvent e)
        {
            System.exit(0);
        }
    }

    //
    // Cut
    //
    class CutAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            cutItems();
        }
    }

    //
    // Copy
    //
    class CopyAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            copyItems();
        }
    }

    //
    // Paste
    //
    class PasteAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            pasteItems();
        }
    }

    //
    // Modify
    //
    class ModifyAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            modifyItem();
        }
    }

    //
    // Delete
    //
    class DeleteAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            deleteItems();
        }
    }

    //
    // Query
    //
    class QueryAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            createQueryDialog().setVisible(true);
        }
    }
}
