// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Link;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.LinkPackage.*;

public class EditLinkDialog extends JDialog implements ActionListener
{
    private Frame parent_;
    private TraderModel model_;

    private JTextField name_;
    private JTextField ior_;
    private JComboBox defaultRule_;
    private JComboBox limitingRule_;
    private StatusBar status_;
    private String linkName_;

    public
    EditLinkDialog(Frame parent, TraderModel model)
    {
        super(parent, "");
        parent_ = parent;
        model_ = model;
        setResizable(true);

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        Constrain.constrain(content,
            AppStandards.createLabel("DefFollowRuleKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);
        defaultRule_ = AppStandards.createComboBox();
        defaultRule_.addItem(AppStandards.getString("LocalOnlyKey"));
        defaultRule_.addItem(AppStandards.getString("IfNoneLocalKey"));
        defaultRule_.addItem(AppStandards.getString("AlwaysKey"));
        Constrain.constrain(content, defaultRule_, 1, 0, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 10, 10, 0, 10, 20, 5);
        Constrain.constrain(content,
            AppStandards.createLabel("LimitFollowRuleKey"),
            0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);
        limitingRule_ = AppStandards.createComboBox();
        limitingRule_.addItem(AppStandards.getString("LocalOnlyKey"));
        limitingRule_.addItem(AppStandards.getString("IfNoneLocalKey"));
        limitingRule_.addItem(AppStandards.getString("AlwaysKey"));
        Constrain.constrain(content, limitingRule_, 1, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 10, 10, 0, 10, 20, 5);

        Constrain.constrain(content, Box.createGlue(), 0, 6, 2, 1,
            GridBagConstraints.BOTH, GridBagConstraints.CENTER,
            1.0, 1.0, 0, 0, 0, 0);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        JButton button;
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 2, 2, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 15, 0, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 3, 2, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);
    }

    public void
    setLinkInfo(String name, Object info)
    {
        LinkInfo linkInfo = (LinkInfo)info;
        linkName_ = name;
        setTitle(AppStandards.getFormattedString("EditLinkTitleKey", name));
        defaultRule_.setSelectedIndex(linkInfo.def_pass_on_follow_rule.value());
        limitingRule_.setSelectedIndex(linkInfo.limiting_follow_rule.value());
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            cancel();
    }

    protected void
    ok()
    {
        // collect inputs
        FollowOption defaultRule =
            FollowOption.from_int(defaultRule_.getSelectedIndex());
        FollowOption limitingRule =
            FollowOption.from_int(limitingRule_.getSelectedIndex());

        status_.setText(AppStandards.getString("UpdatingLinkKey"));

        StringBuffer message = new StringBuffer();

        boolean ok = model_.modifyLink(linkName_, defaultRule, limitingRule,
                                       true, message);

        if(! ok)
            MessageDialog.showError(this, message.toString());
        else
            setVisible(false);

        status_.clear();
    }

    protected void
    cancel()
    {
        setVisible(false);
    }
}
