// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Link;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.io.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.LinkPackage.*;

public class LinkController
    extends com.ooc.CosTradingConsole.Util.BaseController
{
    private JFrame parentFrame_;
    private TraderModel traderModel_;
    private SortedListModel listModel_;
    private NewLinkDialog newDialog_;
    private EditLinkDialog editDialog_;
    private boolean loaded_ = false;
    private String[] options_;

    public
    LinkController(JFrame parentFrame, TraderModel traderModel)
    {
        parentFrame_ = parentFrame;
        traderModel_ = traderModel;
        traderModel_.addModelListener(this);
        listModel_ = new SortedListModel();

        options_ = new String[3];
        options_[0] = AppStandards.getString("LocalOnlyKey");
        options_[1] = AppStandards.getString("IfNoneLocalKey");
        options_[2] = AppStandards.getString("AlwaysKey");
    }

    public ListModel
    getListModel()
    {
        return listModel_;
    }

    public boolean
    isEditable()
    {
        return true;
    }

    public Object
    getItemType()
    {
        return LINK_ITEM;
    }

    public void
    clear()
    {
        listModel_.removeAll();
    }

    public synchronized void
    newItem()
    {
        if(newDialog_ == null)
            newDialog_ = new NewLinkDialog(parentFrame_, traderModel_);

        newDialog_.setValues(null, null);
        newDialog_.setVisible(true);
    }

    public synchronized boolean
    cloneItem(Object item, StringBuffer message)
    {
        String itemName = item.toString();
        ListItem listItem = (ListItem)item;
        LinkInfo info = (LinkInfo)listItem.getData(message);

        if(info != null)
        {
            if(newDialog_ == null)
                newDialog_ = new NewLinkDialog(parentFrame_, traderModel_);

            newDialog_.setValues(itemName, info);
            newDialog_.setVisible(true);
        }

        return (info != null);
    }

    public synchronized boolean
    editItem(Object item, StringBuffer message)
    {
        ListItem listItem = (ListItem)item;
        LinkInfo info = (LinkInfo)listItem.getData(message);

        if(info != null)
        {
            if(editDialog_ == null)
                editDialog_ = new EditLinkDialog(parentFrame_,
                                                 traderModel_);

            editDialog_.setLinkInfo(listItem.toString(), info);
            editDialog_.setVisible(true);
        }

        return (info != null);
    }

    public Object
    findItem(String name)
    {
        ListItem tmp = new ListItem(name);
        return listModel_.findItem(tmp);
    }

    public boolean
    describeItem(Object item, FormattedText text, StringBuffer message)
    {
        boolean result = false;

        LinkInfo info = (LinkInfo)getItemData(item, message);

        if(info != null)
        {
            String ref = AppStandards.getString("InvalidKey");

            try
            {
                ref = com.ooc.Util.CORBA.ORBManager.ORB().
                      object_to_string(info.target);
            }
            catch(org.omg.CORBA.SystemException e)
            {
            }

            text.addLine(AppStandards.getString("TargetDescKey"),
                         FormattedText.HEADER);
            text.addLine(ref, FormattedText.VALUE);
            text.addBlankLine();
            text.addLine(AppStandards.getString("DefPassOnDescKey"),
                         FormattedText.HEADER);
            text.addLine(options_[info.def_pass_on_follow_rule.value()],
                         FormattedText.VALUE);
            text.addBlankLine();
            text.addLine(AppStandards.getString("LimitPassOnDescKey"),
                         FormattedText.HEADER);
            text.addLine(options_[info.limiting_follow_rule.value()],
                         FormattedText.VALUE);
            result = true;
        }

        return result;
    }

    public Object
    getItemData(Object item, StringBuffer message)
    {
        Object result = null;

        ListItem listItem = (ListItem)item;
        result = listItem.getData(message);

        return result;
    }

    public boolean
    destroyItem(Object item, boolean notify, StringBuffer message)
    {
        ListItem listItem = (ListItem)item;
        return traderModel_.removeLink(item.toString(), notify, message);
    }

    public void
    fireItemsDestroyed()
    {
        traderModel_.fireLinksRemoved();
    }

    public boolean
    addItem(String name, Object itemData, boolean notify, StringBuffer message)
    {
        LinkInfo info = (LinkInfo)itemData;
        return traderModel_.addLink(name, info, notify, message);
    }

    public void
    fireItemsAdded()
    {
    }

    public String
    toString()
    {
        return AppStandards.getString("ViewLinksKey");
    }

    // from TraderModelAdapter
    public synchronized void
    linkAdded(String name)
    {
        if(loaded_)
        {
            ListItem item = new ListItem(name);
            listModel_.addItem(item);
        }
    }

    // from TraderModelAdapter
    public synchronized void
    linkModified(String name)
    {
        if(loaded_)
        {
            ListItem tmp = new ListItem(name);
            ListItem item = (ListItem)listModel_.findItem(tmp);
            if(item != null)
                item.invalidate();
        }
    }

    // from TraderModelAdapter
    public synchronized void
    linkRemoved(String name)
    {
        if(loaded_)
        {
            ListItem tmp = new ListItem(name);
            listModel_.removeItem(tmp);
        }
    }

    // from TraderModelAdapter
    public void
    linksRemoved()
    {
        clear();
        refresh();
    }

    protected synchronized void
    reload()
    {
        loaded_ = true;

        Link link = traderModel_.getLink();
        String[] links = link.list_links();

        ListItem[] items = new ListItem[links.length];
        for(int i = 0 ; i < links.length ; i++)
            items[i] = new ListItem(links[i]);
        listModel_.setItems(items);
    }


    class ListItem implements Sortable
    {
        private String name_;
        private Object data_;

        public
        ListItem(String name)
        {
            name_ = name;
        }

        public int
        compareTo(Sortable other, Object rock)
        {
            ListItem otherItem = (ListItem)other;
            return name_.compareTo(otherItem.name_);
        }

        public void
        invalidate()
        {
            data_ = null;
        }

        public synchronized Object
        getData(StringBuffer message)
        {
            if(data_ == null)
            {
                try
                {
                    Link link = traderModel_.getLink();
                    data_ = link.describe_link(name_);
                }
                catch(IllegalLinkName e)
                {
                    message.append(
                        AppStandards.getFormattedString("IllegalLinkNameKey",
                                                        e.name));
                }
                catch(UnknownLinkName e)
                {
                    message.append(
                        AppStandards.getFormattedString("UnknownLinkNameKey",
                                                        e.name));
                }
                catch(org.omg.CORBA.SystemException e)
                {
                    message.append(AppStandards.getString("SystemErrorKey"));
                }
            }

            return data_;
        }

        public String
        toString()
        {
            return name_;
        }
    }
}
