// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Link;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.LinkPackage.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class NewLinkDialog extends JDialog implements ActionListener
{
    private Frame parent_;
    private TraderModel model_;

    private JTextField name_;
    private IORInputPanel iorPanel_;
    private JComboBox defaultRule_;
    private JComboBox limitingRule_;
    private StatusBar status_;

    public
    NewLinkDialog(Frame parent, TraderModel model)
    {
        super(parent, AppStandards.getString("NewLinkTitleKey"));
        parent_ = parent;
        model_ = model;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Cancel" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    close();
                }
            }
        );

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        Constrain.constrain(content, AppStandards.createLabel("LinkNameKey"),
            0, 0, 2, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        name_ = AppStandards.createTextField(25);
        Constrain.constrain(content, name_, 0, 1, 2, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);
        iorPanel_ = new IORInputPanel(parent_, "LinkTargetKey");
        Constrain.constrain(content, iorPanel_, 0, 2, 2, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 10, 0, 10);
        Constrain.constrain(content,
            AppStandards.createLabel("DefFollowRuleKey"),
            0, 3, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);
        defaultRule_ = AppStandards.createComboBox();
        defaultRule_.addItem(AppStandards.getString("LocalOnlyKey"));
        defaultRule_.addItem(AppStandards.getString("IfNoneLocalKey"));
        defaultRule_.addItem(AppStandards.getString("AlwaysKey"));
        Constrain.constrain(content, defaultRule_, 1, 3, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 10, 10, 0, 10, 20, 5);
        Constrain.constrain(content,
            AppStandards.createLabel("LimitFollowRuleKey"),
            0, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);
        limitingRule_ = AppStandards.createComboBox();
        limitingRule_.addItem(AppStandards.getString("LocalOnlyKey"));
        limitingRule_.addItem(AppStandards.getString("IfNoneLocalKey"));
        limitingRule_.addItem(AppStandards.getString("AlwaysKey"));
        Constrain.constrain(content, limitingRule_, 1, 4, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 10, 10, 0, 10, 20, 5);

        Constrain.constrain(content, Box.createGlue(), 0, 5, 2, 1,
            GridBagConstraints.BOTH, GridBagConstraints.CENTER,
            1.0, 1.0, 0, 0, 0, 0);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        JButton button;
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 6, 2, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 15, 0, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 7, 2, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);
    }

    public void
    setValues(String name, LinkInfo info)
    {
        status_.clear();

        if(name != null)
            name_.setText(name);
        else
            name_.setText("");

        if(info != null)
        {
            String ref = com.ooc.Util.CORBA.ORBManager.ORB().
                         object_to_string(info.target);
            iorPanel_.setIOR(ref);
            defaultRule_.setSelectedIndex(info.def_pass_on_follow_rule.value());
            limitingRule_.setSelectedIndex(info.limiting_follow_rule.value());
        }
        else
        {
            iorPanel_.reset();
            initDialog();
        }
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            close();
    }

    protected void
    initDialog()
    {
        try
        {
            status_.setText(AppStandards.getString("LoadingDefAttribKey"));

            Admin admin = model_.getAdmin();
            FollowOption policy = admin.max_link_follow_policy();
            defaultRule_.setSelectedIndex(policy.value());
            limitingRule_.setSelectedIndex(policy.value());

            status_.clear();
        }
        catch(org.omg.CORBA.SystemException e)
        {
            status_.setText(AppStandards.getString("SystemErrorKey"));
        }
    }

    protected void
    ok()
    {
        StringBuffer message = new StringBuffer();

        // collect inputs
        String name = name_.getText().trim();

        org.omg.CORBA.Object object = iorPanel_.getObject(message);
        if(object == null)
        {
            MessageDialog.showError(this, message.toString());
            return;
        }

        Lookup target = LookupHelper.narrow(object);
        if(target == null)
        {
            MessageDialog.showError(this,
                AppStandards.getString("InvalidLookupKey"));
            return;
        }

        FollowOption defaultRule =
            FollowOption.from_int(defaultRule_.getSelectedIndex());
        FollowOption limitingRule =
            FollowOption.from_int(limitingRule_.getSelectedIndex());

        status_.setText(AppStandards.getString("CreatingLinkKey"));

        boolean ok = model_.addLink(name, target, defaultRule, limitingRule,
                                    true, message);

        if(! ok)
            MessageDialog.showError(this, message.toString());
        else
            close();

        status_.clear();
    }

    protected void
    close()
    {
        iorPanel_.close();
        setVisible(false);
    }
}
