// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.io.File;

public class Main
{
    //
    // NOTE: If you want to use a locale-specific resource bundle,
    //       you do NOT need to change the value for bundle_
    //
    static final String bundle_ =
        "com.ooc.CosTradingConsole.Util.AppResources";

    //
    // The official name for the initial reference for the trading service
    //
    static final String serviceName_ = "TradingService";

    public static void
    main(String[] args)
    {
        GUIOptionFilter opt;
        BooleanOption helpOption;

        opt = new GUIOptionFilter();
        helpOption = new BooleanOption('h', "help");
        opt.addOption(helpOption);

        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        org.omg.CORBA.ORB orb =
            com.ooc.Util.CORBA.ORBManager.init(args, props);

        //
        // Initialize the AppStandards class, which loads our locale-
        // specific resource bundle
        //
        AppStandards.init(bundle_);

        //
        // Process the command-line arguments
        //
        try
        {
            opt.parse(args);
        }
        catch(OptionException ex)
        {
            System.err.println("Error: " + ex.getMessage());
            System.err.println();
            usage();
        }

        if(helpOption.isDefined())
            usage();

        Connection connection = null;

        try
        {
            //
            // Connect to the trader
            //
            connection = new Connection(serviceName_);

            if(connection.getLookup() == null)
            {
                System.out.println(AppStandards.getString("CannotConnectKey"));
                System.exit(1);
            }

            ConsoleWindow win = ConsoleWindow.createNewWindow(connection);
            win.setVisible(true);
        }
        catch(org.omg.CORBA.COMM_FAILURE ex)
        {
            System.out.println(AppStandards.getString("CannotConnectKey"));
            System.exit(1);
        }
    }

    private static void
    showError(String errorMessage)
    {
        JFrame frame = new JFrame("");
        Object[] options = { "OK" };
        JOptionPane.showOptionDialog(null, errorMessage, "Warning", 
            JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null,
            options, options[0]);
        System.exit(1);
    }

    private static void
    usage()
    {
        System.out.println(AppStandards.getString("UsageKey"));
        System.exit(1);
    }
}
