// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Offer;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.RegisterPackage.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class EditOfferDialog extends JDialog implements ActionListener
{
    private Frame parent_;
    private TraderModel model_;
    private String id_;
    private OfferInfo info_;
    private TypeStruct ts_;

    private PropertyTablePanel props_;
    private StatusBar status_;


    public
    EditOfferDialog(Frame parent, TraderModel model)
    {
        super(parent, "");
        parent_ = parent;
        model_ = model;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Cancel" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    close();
                }
            }
        );

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        props_ = new PropertyTablePanel(parent_, true);
        props_.setPreferredSize(new Dimension(400, 200));
        Constrain.constrain(content, props_, 0, 0, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 0.5, 10, 10, 0, 10);

        JButton button;
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 1, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 15, 0, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 2, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);
    }

    public void
    setOffer(String id, OfferInfo info, TypeStruct ts)
    {
        setTitle(AppStandards.getFormattedString("EditOfferTitleKey", id));

        id_ = id;
        info_ = info;
        ts_ = ts;

        // the properties to be edited consist of all of the properties
        // from the service type, plus any that already exist in the offer

        Hashtable propTable = new Hashtable();

        if(ts != null)
        {
            for(int i = 0 ; i < ts.props.length ; i++)
                propTable.put(ts.props[i].name, new Prop(ts.props[i]));
        }

        for(int i = 0 ; i < info.properties.length ; i++)
        {
            Prop p = (Prop)propTable.get(info.properties[i].name);
            if(p != null)
                p.setValue(info.properties[i].value);
            else
            {
                // handle case of property not present in service type
                PropStruct ps = new PropStruct();
                ps.name = info.properties[i].name;
                ps.mode = PropertyMode.PROP_NORMAL;
                ps.value_type = info.properties[i].value.type();
                p = new Prop(ps, info.properties[i].value);
                propTable.put(p.getName(), p);
            }
        }

        int count = 0;
        Prop[] props = new Prop[propTable.size()];
        Enumeration e = propTable.elements();
        while(e.hasMoreElements())
        {
            props[count] = (Prop)e.nextElement();
            count++;
        }

        props_.setProperties(props);
    }

    //
    // Override setVisible() so that we initialize the dialog's fields
    // when made visible
    //
    public void
    setVisible(boolean visible)
    {
        status_.clear();
        super.setVisible(visible);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            close();
    }

    protected void
    ok()
    {
        try
        {
            // collect properties from edit panel
            Prop[] arr = props_.getProperties();
            if(arr == null)
                return;

            // In order to modify an offer, we need to compose two arrays:
            // the names of properties to be deleted, and the properties
            // to be modified.
            //
            // We determine which properties to be deleted by comparing the
            // set of properties we obtain from the edit panel with the
            // original set; any that were in the original set and are not
            // in the new set are to be deleted.

            Hashtable oldProps = new Hashtable();
            for(int i = 0 ; i < info_.properties.length ; i++)
                oldProps.put(info_.properties[i].name, info_.properties[i]);

            Hashtable newProps = new Hashtable();
            for(int i = 0 ; i < arr.length ; i++)
                newProps.put(arr[i].getName(), arr[i]);

            // determine the names of the properties to be deleted
            Vector delVec = new Vector();
            for(int i = 0 ; i < info_.properties.length ; i++)
                if(! newProps.containsKey(info_.properties[i].name))
                    delVec.addElement(info_.properties[i].name);
            String[] delList = new String[delVec.size()];
            delVec.copyInto(delList);

            // determine the properties to be added/modified
            Vector propVec = new Vector();
            for(int i = 0 ; i < arr.length ; i++)
            {
                boolean ok = false;

                if(! arr[i].isReadonly())
                    ok = true;
                else if(arr[i].isReadonly())
                {
                    // can only include read-only properties if they were not
                    // already defined
                    boolean exists = oldProps.containsKey(arr[i].getName());
                    ok = ! exists;
                }

                if(ok)
                    propVec.addElement(
                        new Property(arr[i].getName(), arr[i].getValue()));
            }

            Property[] props = new Property[propVec.size()];
            propVec.copyInto(props);

            status_.setText(AppStandards.getString("UpdatingOfferKey"));

            // export offer
            StringBuffer message = new StringBuffer();
            if(! model_.modifyOffer(id_, delList, props, true, message))
                MessageDialog.showError(this, message.toString());
            else
                close();
        }
        finally
        {
            status_.clear();
        }
    }

    protected void
    close()
    {
        props_.close();
        setVisible(false);
    }
}
