// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Offer;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.RegisterPackage.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class NewOfferDialog
    extends JDialog
    implements ActionListener, ItemListener
{
    private Frame parent_;
    private TraderModel model_;

    private JComboBox type_;
    private TypeComboBoxModel typeModel_;
    private IORInputPanel iorPanel_;
    private PropertyTablePanel props_;
    private StatusBar status_;

    public
    NewOfferDialog(Frame parent, TraderModel model)
    {
        super(parent, AppStandards.getString("NewOfferTitleKey"));
        parent_ = parent;
        model_ = model;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Cancel" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    close();
                }
            }
        );

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        Constrain.constrain(content, AppStandards.createLabel("TypeDescKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        typeModel_ = new TypeComboBoxModel(model_);
        type_ = AppStandards.createComboBox(typeModel_);
        type_.setMaximumRowCount(10);
        type_.addItemListener(this);
        Constrain.constrain(content, type_, 0, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10, 2, 5);

        iorPanel_ = new IORInputPanel(parent_, "IORKey");
        Constrain.constrain(content, iorPanel_, 0, 2, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 10, 0, 10);

        props_ = new PropertyTablePanel(parent_, false);
        Constrain.constrain(content, props_, 0, 3, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 0.5, 10, 10, 0, 10);

        JButton button;
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 4, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 15, 0, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 5, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);

        typeModel_.reload();
    }

    public void
    setValues(OfferInfo info)
    {
        status_.clear();

        typeModel_.reload();
        props_.clear();

        if(info != null)
        {
            type_.removeItemListener(this);
            typeModel_.setSelectedItem(info.type);
            type_.addItemListener(this);

            if(info.reference != null)
            {
                String ref = com.ooc.Util.CORBA.ORBManager.ORB().
                             object_to_string(info.reference);
                iorPanel_.setIOR(ref);
            }
            else
                iorPanel_.setIOR("");

            StringBuffer message = new StringBuffer();
            TypeStruct ts = model_.fullyDescribeType(info.type, message);
            Hashtable hash = new Hashtable();
            if(ts != null)
                for(int i = 0 ; i < ts.props.length ; i++)
                    hash.put(ts.props[i].name, ts.props[i]);

            Prop[] props = new Prop[info.properties.length];
            for(int i = 0 ; i < info.properties.length ; i++)
            {
                PropStruct ps = (PropStruct)hash.get(info.properties[i].name);
                if(ps == null)
                    props[i] = new Prop(info.properties[i].name,
                                        info.properties[i].value);
                else
                    props[i] = new Prop(ps, info.properties[i].value);
            }

            props_.setProperties(props);
        }
        else
            iorPanel_.reset();
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            close();
    }

    public void
    itemStateChanged(ItemEvent e)
    {
        if(e.getStateChange() == ItemEvent.SELECTED && e.getItem() != null)
            initializeProperties();
    }

    protected void
    initializeProperties()
    {
        try
        {
            status_.setText(AppStandards.getString("InitPropsKey"));

            String type = (String)type_.getSelectedItem();
            StringBuffer message = new StringBuffer();
            TypeStruct ts = model_.fullyDescribeType(type, message);
            if(ts == null)
            {
                MessageDialog.showError(this, message.toString());
                return;
            }

            Prop[] props = new Prop[ts.props.length];
            for(int i = 0 ; i < ts.props.length ; i++)
                props[i] = new Prop(ts.props[i]);
            props_.setProperties(props);
        }
        finally
        {
            status_.clear();
        }
    }

    protected void
    ok()
    {
        try
        {
            StringBuffer message = new StringBuffer();

            // collect inputs
            String type = (String)type_.getSelectedItem();
            if(type == null)
            {
                MessageDialog.showError(this,
                                        AppStandards.getString("NoTypeKey"));
                return;
            }

            org.omg.CORBA.Object reference;

            if(iorPanel_.isNil())
                reference = null;
            else
            {
                reference = iorPanel_.getObject(message);
                if(reference == null)
                {
                    MessageDialog.showError(this, message.toString());
                    return;
                }
            }

            // collect properties
            Prop[] arr = props_.getProperties();
            if(arr == null)
                return;

            Property[] props = new Property[arr.length];
            for(int i = 0 ; i < arr.length ; i++)
                props[i] = new Property(arr[i].getName(), arr[i].getValue());

            status_.setText(AppStandards.getString("ExportingOfferKey"));

            // export offer
            String id = model_.addOffer(reference, type, props, true, message);

            if(id == null)
                MessageDialog.showError(this, message.toString());
            else
                close();
        }
        finally
        {
            status_.clear();
        }
    }

    protected void
    close()
    {
        iorPanel_.close();
        props_.close();
        setVisible(false);
    }
}
