// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Offer;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.io.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.RegisterPackage.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;
import org.omg.CosTradingDynamic.*;

public class OfferController
    extends com.ooc.CosTradingConsole.Util.BaseController
{
    private JFrame parentFrame_;
    private TraderModel traderModel_;
    private SortedListModel listModel_;
    private NewOfferDialog newDialog_;
    private EditOfferDialog editDialog_;
    private boolean loaded_ = false;

    public
    OfferController(JFrame parentFrame, TraderModel traderModel)
    {
        parentFrame_ = parentFrame;
        traderModel_ = traderModel;
        traderModel_.addModelListener(this);
        listModel_ = new SortedListModel();
    }

    public ListModel
    getListModel()
    {
        return listModel_;
    }

    public boolean
    isEditable()
    {
        return true;
    }

    public Object
    getItemType()
    {
        return OFFER_ITEM;
    }

    public void
    clear()
    {
        listModel_.removeAll();
    }

    public synchronized void
    newItem()
    {
        if(newDialog_ == null)
            newDialog_ = new NewOfferDialog(parentFrame_,
                                            traderModel_);
        newDialog_.setVisible(true);
        newDialog_.setValues(null);
    }

    public synchronized boolean
    cloneItem(Object item, StringBuffer message)
    {
        ListItem listItem = (ListItem)item;
        OfferInfo info = (OfferInfo)listItem.getData(message);

        if(info == null)
            return false;

        if(newDialog_ == null)
            newDialog_ = new NewOfferDialog(parentFrame_,
                                            traderModel_);
        newDialog_.setValues(info);
        newDialog_.setVisible(true);

        return true;
    }

    public synchronized boolean
    editItem(Object item, StringBuffer message)
    {
        ListItem listItem = (ListItem)item;
        OfferInfo info = (OfferInfo)listItem.getData(message);

        if(info == null)
            return false;

        TypeStruct ts = traderModel_.describeType(info.type, message);
        if(ts == null)
            return false;

        if(editDialog_ == null)
            editDialog_ = new EditOfferDialog(parentFrame_,
                                              traderModel_);
        editDialog_.setOffer(listItem.getId(), info, ts);
        editDialog_.setVisible(true);

        return true;
    }

    public Object
    findItem(String name)
    {
        ListItem tmp = new ListItem(name);
        return listModel_.findItem(tmp);
    }

    public boolean
    describeItem(Object item, FormattedText text, StringBuffer message)
    {
        boolean result = false;

        OfferInfo info = (OfferInfo)getItemData(item, message);

        if(info != null)
        {
            text.addLine(AppStandards.getString("TypeDescKey"),
                         FormattedText.HEADER);
            text.addLine(info.type, FormattedText.VALUE);
            text.addBlankLine();

            text.addLine(AppStandards.getString("PropertiesDescKey"),
                         FormattedText.HEADER);
            if(info.properties.length == 0)
                text.addLine(AppStandards.getString("NoneKey"),
                             FormattedText.VALUE);
            else
            {
                for(int i = 0 ; i < info.properties.length ; i++)
                    PropUtil.describe(info.properties[i], text);
            }

            text.addBlankLine();

            text.addLine(AppStandards.getString("ObjectDescKey"),
                         FormattedText.HEADER);
            String ref = null;

            if(info.reference == null)
                ref = AppStandards.getString("NilKey");
            else
            {
                ref = AppStandards.getString("InvalidKey");
                try
                {
                    org.omg.CORBA.ORB orb =
                        com.ooc.Util.CORBA.ORBManager.ORB();
                    ref = orb.object_to_string(info.reference);
                }
                catch(org.omg.CORBA.SystemException e)
                {
                }
            }
            text.addLine(ref, FormattedText.VALUE);

            result = true;
        }


        return result;
    }

    public Object
    getItemData(Object item, StringBuffer message)
    {
        Object result = null;

        ListItem listItem = (ListItem)item;
        result = listItem.getData(message);

        return result;
    }

    public boolean
    destroyItem(Object item, boolean notify, StringBuffer message)
    {
        ListItem listItem = (ListItem)item;
        return traderModel_.removeOffer(item.toString(), notify, message);
    }

    public void
    fireItemsDestroyed()
    {
        traderModel_.fireOffersRemoved();
    }

    public boolean
    addItem(String name, Object itemData, boolean notify, StringBuffer message)
    {
        OfferInfo info = (OfferInfo)itemData;
        return (traderModel_.addOffer(info, notify, message) != null);
    }

    public void
    fireItemsAdded()
    {
    }

    public String
    toString()
    {
        return AppStandards.getString("ViewOffersKey");
    }

    // from TraderModelAdapter
    public synchronized void
    offerAdded(String name)
    {
        if(loaded_)
        {
            ListItem item = new ListItem(name);
            listModel_.addItem(item);
        }
    }

    // from TraderModelAdapter
    public synchronized void
    offerModified(String name)
    {
        if(loaded_)
        {
            ListItem tmp = new ListItem(name);
            ListItem item = (ListItem)listModel_.findItem(tmp);
            if(item != null)
                item.invalidate();
        }
    }

    // from TraderModelAdapter
    public synchronized void
    offerRemoved(String name)
    {
        if(loaded_)
        {
            ListItem tmp = new ListItem(name);
            listModel_.removeItem(tmp);
        }
    }

    // from TraderModelAdapter
    public void
    offersRemoved()
    {
        clear();
        refresh();
    }

    protected synchronized void
    reload()
    {
        loaded_ = true;

        try
        {
            OfferIdSeqHolder ids = new OfferIdSeqHolder();
            OfferIdIteratorHolder iter = new OfferIdIteratorHolder();
            Admin admin = traderModel_.getAdmin();
            admin.list_offers(0, ids, iter);

            java.util.Vector vec = new java.util.Vector();
            int idCount = 0;

            if(iter.value != null)
            {
                boolean more;
                do
                {
                    more = iter.value.next_n(100, ids);
                    idCount += ids.value.length;
                    vec.addElement(ids.value);
                }
                while(more);

                iter.value.destroy();

                ListItem[] items = new ListItem[idCount];
                int i = 0;
                java.util.Enumeration e = vec.elements();
                while(e.hasMoreElements())
                {
                    String[] a = (String[])e.nextElement();
                    for(int n = 0 ; n < a.length ; n++, i++)
                        items[i] = new ListItem(a[n]);
                }

                listModel_.setItems(items);
            }
        }
        catch(NotImplemented e)
        {
            MessageDialog.showError(parentFrame_,
                AppStandards.getString("ListOfferImplKey"));
        }
    }


    class ListItem implements Sortable
    {
        private String id_;
        private Object data_;

        public
        ListItem(String id)
        {
            id_ = id;
        }

        public int
        compareTo(Sortable other, Object rock)
        {
            ListItem otherItem = (ListItem)other;
            return id_.compareTo(otherItem.id_);
        }

        public void
        invalidate()
        {
            data_ = null;
        }

        public String
        getId()
        {
            return id_;
        }

        public synchronized Object
        getData(StringBuffer message)
        {
            if(data_ == null)
            {
                try
                {
                    Register reg = traderModel_.getRegister();
                    data_ = reg.describe(id_);
                }
                catch(IllegalOfferId e)
                {
                    message.append(
                        AppStandards.getFormattedString("IllegalOfferIdKey",
                                                        e.id));
                }
                catch(UnknownOfferId e)
                {
                    message.append(
                        AppStandards.getFormattedString("UnknownOfferIdKey",
                                                        e.id));
                }
                catch(ProxyOfferId e)
                {
                    message.append(
                        AppStandards.getFormattedString("ProxyOfferIdKey",
                                                        e.id));
                }
                catch(org.omg.CORBA.SystemException e)
                {
                    message.append(AppStandards.getString("SystemErrorKey"));
                }
            }

            return data_;
        }

        public String
        toString()
        {
            return id_;
        }
    }
}
