// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Proxy;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.ProxyPackage.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class NewProxyDialog
    extends JDialog
    implements ActionListener, ItemListener
{
    private Frame parent_;
    private TraderModel model_;

    private JComboBox type_;
    private TypeComboBoxModel typeModel_;
    private IORInputPanel iorPanel_;
    private JTextField recipe_;
    private JCheckBox ifMatchAll_;
    private PropertyTablePanel props_;
    private JList policies_;
    private SortedListModel policiesModel_;
    private StatusBar status_;
    private PolicyDialog policyDialog_;

    public
    NewProxyDialog(Frame parent, TraderModel model)
    {
        super(parent, AppStandards.getString("NewProxyTitleKey"));
        parent_ = parent;
        model_ = model;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Cancel" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    close();
                }
            }
        );

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        Constrain.constrain(content, AppStandards.createLabel("TypeDescKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        typeModel_ = new TypeComboBoxModel(model_);
        type_ = AppStandards.createComboBox(typeModel_);
        type_.setMaximumRowCount(10);
        type_.addItemListener(this);
        Constrain.constrain(content, type_, 0, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10, 2, 5);

        iorPanel_ = new IORInputPanel(parent_, "ProxyTargetKey");
        Constrain.constrain(content, iorPanel_, 0, 2, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 10, 0, 10);

        Constrain.constrain(content, AppStandards.createLabel("ProxyRecipeKey"),
            0, 3, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        recipe_ = AppStandards.createTextField(35);
        Constrain.constrain(content, recipe_, 0, 4, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);

        ifMatchAll_ = AppStandards.createCheckBox("IfMatchAllKey");
        Constrain.constrain(content, ifMatchAll_, 0, 5, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 10);

        props_ = new PropertyTablePanel(parent_, false);
        Constrain.constrain(content, props_, 0, 6, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 0.5, 10, 10, 0, 10);

        JButton button;

        JPanel policyPanel = new JPanel();
        policyPanel.setBorder(
            AppStandards.createTitledBorder("ProxyPoliciesKey"));
        policyPanel.setLayout(new GridBagLayout());
        policiesModel_ = new SortedListModel();
        policies_ = AppStandards.createList(policiesModel_);
        JScrollPane policiescroll = AppStandards.createScrollPane(policies_);
        policiescroll.setPreferredSize(new Dimension(200, 75));
        Constrain.constrain(policyPanel, policiescroll, 0, 0, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 5, 5, 0);
        JPanel policyButtons = new JPanel();
        policyButtons.setLayout(new GridLayout(3, 1, 5, 5));
        button = AppStandards.createButton("AddButtonKey");
        button.setActionCommand("addPolicy");
        button.addActionListener(this);
        policyButtons.add(button);
        button = AppStandards.createButton("EditButtonKey");
        button.setActionCommand("editPolicy");
        button.addActionListener(this);
        policyButtons.add(button);
        button = AppStandards.createButton("DeleteButtonKey");
        button.setActionCommand("deletePolicy");
        button.addActionListener(this);
        policyButtons.add(button);
        Constrain.constrain(policyPanel, policyButtons, 1, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHEAST,
            0.0, 0.0, 0, 5, 5, 5);

        Constrain.constrain(content, policyPanel, 0, 7, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 0.5, 10, 10, 0, 10);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 8, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 15, 0, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 9, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);

        typeModel_.reload();
    }

    public void
    setValues(ProxyInfo info)
    {
        status_.clear();

        props_.clear();
        typeModel_.reload();
        policiesModel_.removeAll();

        if(info != null)
        {
            type_.removeItemListener(this);
            typeModel_.setSelectedItem(info.type);
            type_.addItemListener(this);

            String ref = com.ooc.Util.CORBA.ORBManager.ORB().
                         object_to_string(info.target);
            iorPanel_.setIOR(ref);

            StringBuffer message = new StringBuffer();
            TypeStruct ts = model_.fullyDescribeType(info.type, message);
            Hashtable hash = new Hashtable();
            if(ts != null)
                for(int i = 0 ; i < ts.props.length ; i++)
                    hash.put(ts.props[i].name, ts.props[i]);

            Prop[] props = new Prop[info.properties.length];
            for(int i = 0 ; i < info.properties.length ; i++)
            {
                PropStruct ps = (PropStruct)hash.get(info.properties[i].name);
                if(ps == null)
                    props[i] = new Prop(info.properties[i].name,
                                        info.properties[i].value);
                else
                    props[i] = new Prop(ps, info.properties[i].value);
            }

            props_.setProperties(props);

            recipe_.setText(info.recipe);
            ifMatchAll_.setSelected(info.if_match_all);

            ProxyPolicy[] policies =
                new ProxyPolicy[info.policies_to_pass_on.length];
            for(int i = 0 ; i < info.policies_to_pass_on.length ; i++)
            {
                Policy p = info.policies_to_pass_on[i];
                policies[i] = new ProxyPolicy(p.name, p.value);
            }
            policiesModel_.setItems(policies);
        }
        else
        {
            iorPanel_.reset();
            recipe_.setText("");
            ifMatchAll_.setSelected(false);
        }
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            close();
        else if(command.equals("addPolicy"))
        {
            if(policyDialog_ == null)
                policyDialog_ = new PolicyDialog(parent_);
            policyDialog_.setPolicy(null);
            policyDialog_.setVisible(true);
        }
        else if(command.equals("editPolicy"))
        {
            ProxyPolicy p = (ProxyPolicy)policies_.getSelectedValue();
            if(p != null)
            {
                if(policyDialog_ == null)
                    policyDialog_ = new PolicyDialog(parent_);
                policyDialog_.setPolicy(p);
                policyDialog_.setVisible(true);
            }
        }
        else if(command.equals("deletePolicy"))
        {
            Object[] policies = (Object[])policies_.getSelectedValues();
            if(policies != null)
            {
                for(int i = 0 ; i < policies.length ; i++)
                    policiesModel_.removeItem((ProxyPolicy)policies[i]);
            }
        }
    }

    public void
    itemStateChanged(ItemEvent e)
    {
        if(e.getStateChange() == ItemEvent.SELECTED && e.getItem() != null)
            initializeProperties();
    }

    protected void
    initializeProperties()
    {
        try
        {
            status_.setText(AppStandards.getString("InitPropsKey"));

            String type = (String)type_.getSelectedItem();
            StringBuffer message = new StringBuffer();
            TypeStruct ts = model_.fullyDescribeType(type, message);
            if(ts == null)
            {
                MessageDialog.showError(this, message.toString());
                return;
            }

            Prop[] props = new Prop[ts.props.length];
            for(int i = 0 ; i < ts.props.length ; i++)
                props[i] = new Prop(ts.props[i]);
            props_.setProperties(props);
        }
        finally
        {
            status_.clear();
        }
    }

    protected void
    ok()
    {
        try
        {
            StringBuffer message = new StringBuffer();

            // collect inputs
            String type = (String)type_.getSelectedItem();
            if(type == null)
            {
                MessageDialog.showError(this,
                                        AppStandards.getString("NoTypeKey"));
                return;
            }

            Lookup target = null;
            try
            {
                org.omg.CORBA.Object obj = iorPanel_.getObject(message);
                if(obj == null)
                {
                    MessageDialog.showError(this, message.toString());
                    return;
                }

                target = LookupHelper.narrow(obj);
            }
            catch(org.omg.CORBA.SystemException e)
            {
                MessageDialog.showError(this,
                    AppStandards.getString("InvalidLookupKey"));
                return;
            }

            if(target == null)
            {
                MessageDialog.showError(this,
                    AppStandards.getString("InvalidLookupKey"));
                return;
            }

            String recipe = recipe_.getText().trim();
            boolean ifMatchAll = ifMatchAll_.isSelected();

            // collect properties
            Prop[] arr = props_.getProperties();
            if(arr == null)
                return;

            Property[] props = new Property[arr.length];
            for(int i = 0 ; i < arr.length ; i++)
                props[i] = new Property(arr[i].getName(), arr[i].getValue());

            Sortable[] items = policiesModel_.getItems();
            Policy[] policies = new Policy[items.length];
            for(int i = 0 ; i < items.length ; i++)
            {
                ProxyPolicy p = (ProxyPolicy)items[i];
                policies[i] = new Policy(p.getName(), p.getValue());
            }

            status_.setText(AppStandards.getString("ExportingProxyKey"));

            // export proxy offer
            String id = model_.addProxy(target, type, props, ifMatchAll,
                                        recipe, policies, true, message);

            if(id == null)
                MessageDialog.showError(this, message.toString());
            else
                close();
        }
        finally
        {
            status_.clear();
        }
    }

    protected void
    close()
    {
        iorPanel_.close();
        props_.close();
        setVisible(false);
    }

    protected void
    addPolicy(ProxyPolicy p)
    {
        policiesModel_.addItem(p);
    }

    protected void
    editPolicy(ProxyPolicy oldPolicy, ProxyPolicy newPolicy)
    {
        policiesModel_.removeItem(oldPolicy);
        policiesModel_.addItem(newPolicy);
    }


    class ProxyPolicy implements Sortable
    {
        private String name_;
        private org.omg.CORBA.Any value_;

        public
        ProxyPolicy(String name, org.omg.CORBA.Any value)
        {
            name_ = name;
            value_ = value;
        }

        public int
        compareTo(Sortable other, Object rock)
        {
            ProxyPolicy otherPolicy = (ProxyPolicy)other;
            return name_.compareTo(otherPolicy.name_);
        }

        public String
        getName()
        {
            return name_;
        }

        public org.omg.CORBA.Any
        getValue()
        {
            return value_;
        }

        public String
        toString()
        {
            StringBuffer result = new StringBuffer();

            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            com.ooc.Util.CORBA.AnyWriter out =
                new com.ooc.Util.CORBA.AnyWriter(pw);
            out.printName(value_.type());
            pw.flush();
            result.append(sw.toString());

            result.append(" ");
            result.append(name_);
            result.append(" = ");

            sw = new StringWriter();
            pw = new PrintWriter(sw);
            out.setWriter(pw);
            org.omg.CORBA.DynAny dynAny =
                com.ooc.Util.CORBA.ORBManager.ORB().create_dyn_any(value_);
            out.printDynAny(dynAny, false);
            pw.flush();
            result.append(sw.toString());

            return result.toString();
        }
    }


    class PolicyDialog
        extends JDialog
        implements ActionListener
    {
        private Frame parent_;

        private JTextField name_;
        private TypeCodeComboBoxModel typeModel_;
        private JComboBox type_;
        private JTextField value_;
        private ProxyPolicy policy_ = null;

        public
        PolicyDialog(Frame parent)
        {
            super(parent, AppStandards.getString("PolicyTitleKey"));
            parent_ = parent;
            setResizable(true);

            Container content = getContentPane();
            content.setLayout(new GridBagLayout());

            JButton button;

            Constrain.constrain(content,
                AppStandards.createLabel("PolicyNameKey"),
                0, 0, 1, 1, GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 0.0, 10, 10, 0, 10);
            name_ = AppStandards.createTextField(25);
            Constrain.constrain(content, name_, 0, 1, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
                1.0, 0.0, 0, 10, 0, 10);
            Constrain.constrain(content,
                AppStandards.createLabel("PolicyTypeKey"), 0, 2, 1, 1,
                GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
                0.0, 0.0, 10, 10, 0, 0);
            typeModel_ = new TypeCodeComboBoxModel();
            type_ = AppStandards.createComboBox(typeModel_);
            type_.setMaximumRowCount(4);
            Constrain.constrain(content, type_, 0, 3, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
                1.0, 0.0, 0, 10, 0, 10);
            Constrain.constrain(content,
                AppStandards.createLabel("PolicyValueKey"),
                0, 4, 1, 1, GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 0.0, 10, 10, 0, 10);
            value_ = AppStandards.createTextField(25);
            Constrain.constrain(content, value_, 0, 5, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
                1.0, 0.0, 0, 10, 0, 10);

            // insert some glue to take up the extra vertical space on a resize
            Constrain.constrain(content, Box.createGlue(), 0, 6, 1, 1,
                GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
                1.0, 1.0, 0, 0, 0, 0);

            JPanel buttonPanel = new JPanel();
            buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
            button = AppStandards.createButton("OkButtonKey");
            button.setActionCommand("ok");
            button.addActionListener(this);
            buttonPanel.add(button);

            button = AppStandards.createButton("CancelButtonKey");
            button.setActionCommand("cancel");
            button.addActionListener(this);
            buttonPanel.add(button);

            Constrain.constrain(content, buttonPanel, 0, 7, 1, 1,
                GridBagConstraints.NONE, GridBagConstraints.EAST,
                0.0, 0.0, 10, 0, 10, 10);

            pack();

            // initialize the TypeCode combo box
            typeModel_.loadBasicTypes();

            Point p = parent.getLocation();
            setLocation(p.x + 40, p.y + 40);
        }

        public void
        setPolicy(ProxyPolicy p)
        {
            policy_ = p;

            if(p != null)
            {
                name_.setText(p.getName());
                typeModel_.setSelectedTypeCode(p.getValue().type());

                StringWriter sw = new StringWriter();
                PrintWriter pw = new PrintWriter(sw);
                com.ooc.Util.CORBA.AnyWriter out =
                    new com.ooc.Util.CORBA.AnyWriter(pw);
                org.omg.CORBA.DynAny dynAny =
                    com.ooc.Util.CORBA.ORBManager.ORB().
                    create_dyn_any(p.getValue());
                out.printDynAny(dynAny, false);
                pw.flush();
                value_.setText(sw.toString());
            }
            else
            {
                name_.setText("");
                value_.setText("");
                type_.setSelectedItem(null);
            }
        }

        public void
        actionPerformed(ActionEvent e)
        {
            String command = e.getActionCommand();
            if(command.equals("ok"))
                ok();
            else if(command.equals("cancel"))
                cancel();
        }

        protected void
        ok()
        {
            org.omg.CORBA.ORB orb =
                com.ooc.Util.CORBA.ORBManager.ORB();

            String name = name_.getText().trim();
            if(name.length() == 0)
            {
                String msg = AppStandards.getString("EmptyNameKey");
                MessageDialog.showError(this, msg);
                return;
            }

            org.omg.CORBA.TypeCode tc = typeModel_.getSelectedTypeCode();
            if(tc == null)
            {
                String msg = AppStandards.getString("NoTypeCodeKey");
                MessageDialog.showError(this, msg);
                return;
            }

            org.omg.CORBA.Any value = null;

            String s = value_.getText().trim();
            try
            {
                value = Parse.parseValue(tc, s);
            }
            catch(ParseException e)
            {
                MessageDialog.showError(this, e.getMessage());
                return;
            }

            ProxyPolicy newPolicy = new ProxyPolicy(name, value);

            if(policy_ == null)
                addPolicy(newPolicy);
            else
                editPolicy(policy_, newPolicy);

            this.setVisible(false);
        }

        protected void
        cancel()
        {
            this.setVisible(false);
        }
    }

}
