// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Proxy;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.io.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.ProxyPackage.*;

public class ProxyController
    extends com.ooc.CosTradingConsole.Util.BaseController
{
    private JFrame parentFrame_;
    private TraderModel traderModel_;
    private SortedListModel listModel_;
    private NewProxyDialog newDialog_;
    private boolean loaded_ = false;

    public
    ProxyController(JFrame parentFrame, TraderModel traderModel)
    {
        parentFrame_ = parentFrame;
        traderModel_ = traderModel;
        traderModel_.addModelListener(this);
        listModel_ = new SortedListModel();
    }

    public ListModel
    getListModel()
    {
        return listModel_;
    }

    public boolean
    isEditable()
    {
        return false;
    }

    public Object
    getItemType()
    {
        return PROXY_ITEM;
    }

    public void
    clear()
    {
        listModel_.removeAll();
    }

    public synchronized void
    newItem()
    {
        if(newDialog_ == null)
            newDialog_ = new NewProxyDialog(parentFrame_,
                                            traderModel_);
        newDialog_.setValues(null);
        newDialog_.setVisible(true);
    }

    public synchronized boolean
    cloneItem(Object item, StringBuffer message)
    {
        ProxyInfo info = (ProxyInfo)getItemData(item, message);

        if(info != null)
        {
            if(newDialog_ == null)
                newDialog_ = new NewProxyDialog(parentFrame_,
                                                traderModel_);
            newDialog_.setValues(info);
            newDialog_.setVisible(true);
        }

        return (info != null);
    }

    public Object
    findItem(String name)
    {
        ListItem tmp = new ListItem(name);
        return listModel_.findItem(tmp);
    }

    public boolean
    describeItem(Object item, FormattedText text, StringBuffer message)
    {
        boolean result = false;

        ProxyInfo info = (ProxyInfo)getItemData(item, message);

        org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();

        if(info != null)
        {
            text.addLine(AppStandards.getString("TypeDescKey"),
                         FormattedText.HEADER);
            text.addLine(info.type, FormattedText.VALUE);
            text.addBlankLine();

            text.addLine(AppStandards.getString("PropertiesDescKey"),
                         FormattedText.HEADER);
            if(info.properties.length == 0)
                text.addLine(AppStandards.getString("NoneKey"),
                             FormattedText.VALUE);
            else
            {
                for(int i = 0 ; i < info.properties.length ; i++)
                    PropUtil.describe(info.properties[i], text);
            }

            text.addBlankLine();

            text.addLine(AppStandards.getString("IfMatchAllDescKey"),
                         FormattedText.HEADER);
            if(info.if_match_all)
                text.addLine(AppStandards.getString("YesKey"), FormattedText.VALUE);
            else
                text.addLine(AppStandards.getString("NoKey"), FormattedText.VALUE);
            text.addBlankLine();

            text.addLine(AppStandards.getString("RecipeDescKey"),
                         FormattedText.HEADER);
            text.addLine(info.recipe, FormattedText.VALUE);
            text.addBlankLine();

            text.addLine(AppStandards.getString("PoliciesDescKey"),
                         FormattedText.HEADER);
            if(info.policies_to_pass_on.length == 0)
                text.addLine(AppStandards.getString("NoneKey"),
                             FormattedText.VALUE);
            else
            {
                for(int i = 0 ; i < info.policies_to_pass_on.length ; i++)
                {
                    org.omg.CORBA.Any any = info.policies_to_pass_on[i].value;

                    StringWriter sw = new StringWriter();
                    PrintWriter pw = new PrintWriter(sw);
                    com.ooc.Util.CORBA.AnyWriter out =
                        new com.ooc.Util.CORBA.AnyWriter(pw);

                    out.printName(any.type());
                    pw.print(" ");
                    pw.print(info.policies_to_pass_on[i].name);
                    pw.print(" = ");

                    org.omg.CORBA.DynAny dynAny =
                        orb.create_dyn_any(info.policies_to_pass_on[i].value);
                    out.printDynAny(dynAny, false);

                    pw.flush();
                    text.addLine(sw.toString(), FormattedText.VALUE);
                }
            }

            text.addBlankLine();

            text.addLine(AppStandards.getString("TargetDescKey"),
                         FormattedText.HEADER);
            String ref = AppStandards.getString("InvalidKey");
            try
            {
                ref = orb.object_to_string(info.target);
            }
            catch(org.omg.CORBA.SystemException e)
            {
            }
            text.addLine(ref, FormattedText.VALUE);

            result = true;
        }

        return result;
    }

    public Object
    getItemData(Object item, StringBuffer message)
    {
        Object result = null;

        ListItem listItem = (ListItem)item;
        result = listItem.getData(message);

        return result;
    }

    public boolean
    destroyItem(Object item, boolean notify, StringBuffer message)
    {
        ListItem listItem = (ListItem)item;
        return traderModel_.removeProxy(item.toString(), notify, message);
    }

    public void
    fireItemsDestroyed()
    {
        traderModel_.fireProxiesRemoved();
    }

    public boolean
    addItem(String name, Object itemData, boolean notify, StringBuffer message)
    {
        ProxyInfo info = (ProxyInfo)itemData;
        return (traderModel_.addProxy(info, notify, message) != null);
    }

    public void
    fireItemsAdded()
    {
    }

    public String
    toString()
    {
        return AppStandards.getString("ViewProxiesKey");
    }

    // from TraderModelAdapter
    public synchronized void
    proxyAdded(String name)
    {
        if(loaded_)
        {
            ListItem item = new ListItem(name);
            listModel_.addItem(item);
        }
    }

    // from TraderModelAdapter
    public synchronized void
    proxyModified(String name)
    {
        if(loaded_)
        {
            ListItem tmp = new ListItem(name);
            ListItem item = (ListItem)listModel_.findItem(tmp);
            if(item != null)
                item.invalidate();
        }
    }

    // from TraderModelAdapter
    public synchronized void
    proxyRemoved(String name)
    {
        if(loaded_)
        {
            ListItem tmp = new ListItem(name);
            listModel_.removeItem(tmp);
        }
    }

    // from TraderModelAdapter
    public void
    proxysRemoved()
    {
        clear();
        refresh();
    }

    protected synchronized void
    reload()
    {
        loaded_ = true;

        try
        {
            OfferIdSeqHolder ids = new OfferIdSeqHolder();
            OfferIdIteratorHolder iter = new OfferIdIteratorHolder();
            Admin admin = traderModel_.getAdmin();
            admin.list_proxies(0, ids, iter);

            java.util.Vector vec = new java.util.Vector();
            int idCount = 0;

            if(iter.value != null)
            {
                boolean more;
                do
                {
                    more = iter.value.next_n(100, ids);
                    idCount += ids.value.length;
                    vec.addElement(ids.value);
                }
                while(more);

                iter.value.destroy();

                ListItem[] items = new ListItem[idCount];
                int i = 0;
                java.util.Enumeration e = vec.elements();
                while(e.hasMoreElements())
                {
                    String[] a = (String[])e.nextElement();
                    for(int n = 0 ; n < a.length ; n++, i++)
                        items[i] = new ListItem(a[n]);
                }

                listModel_.setItems(items);
            }
        }
        catch(NotImplemented e)
        {
            MessageDialog.showError(parentFrame_,
                AppStandards.getString("ListProxyImplKey"));
        }
    }


    class ListItem implements Sortable
    {
        private String id_;
        private Object data_;

        public
        ListItem(String id)
        {
            id_ = id;
        }

        public int
        compareTo(Sortable other, Object rock)
        {
            ListItem otherItem = (ListItem)other;
            return id_.compareTo(otherItem.id_);
        }

        public void
        invalidate()
        {
            data_ = null;
        }

        public synchronized Object
        getData(StringBuffer message)
        {
            if(data_ == null)
            {
                try
                {
                    Proxy proxy = traderModel_.getProxy();
                    data_ = proxy.describe_proxy(id_);
                }
                catch(IllegalOfferId e)
                {
                    message.append(
                        AppStandards.getFormattedString("IllegalOfferIdKey",
                                                        e.id));
                }
                catch(UnknownOfferId e)
                {
                    message.append(
                        AppStandards.getFormattedString("UnknownOfferIdKey",
                                                        e.id));
                }
                catch(NotProxyOfferId e)
                {
                    message.append(
                        AppStandards.getFormattedString("NotProxyOfferKey",
                                                        e.id));
                }
                catch(org.omg.CORBA.SystemException e)
                {
                    message.append(AppStandards.getString("SystemErrorKey"));
                }
            }

            return data_;
        }

        public String
        toString()
        {
            return id_;
        }
    }
}
