// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Tools;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.*;

public class AttributesDialog extends JDialog implements ActionListener
{
    private TraderModel model_;
    private Admin admin;

    private JTabbedPane pane_;
    private StatusBar status_;
    private JCheckBox allowModifiableProperties_;
    private JCheckBox allowDynamicProperties_;
    private JCheckBox allowProxyOffers_;
    private IORInputPanel reposIOR_;
    private JTextField defSearchCard_, maxSearchCard_;
    private JTextField defMatchCard_, maxMatchCard_;
    private JTextField defReturnCard_, maxReturnCard_;
    private JTextField defHopCount_, maxHopCount_;
    private JComboBox defFollowPolicy_, maxFollowPolicy_;
    private JTextField maxList_;
    private JComboBox maxLinkFollowPolicy_;
    private JTextField requestIdStem_;

    private boolean allowModifiablePropertiesValue_;
    private boolean allowDynamicPropertiesValue_;
    private boolean allowProxyOffersValue_;
    private String reposIORValue_;
    private int defSearchCardValue_, maxSearchCardValue_;
    private int defMatchCardValue_, maxMatchCardValue_;
    private int defReturnCardValue_, maxReturnCardValue_;
    private int defHopCountValue_, maxHopCountValue_;
    private FollowOption defFollowPolicyValue_, maxFollowPolicyValue_;
    private int maxListValue_;
    private FollowOption maxLinkFollowPolicyValue_;
    private byte[] requestIdStemValue_;

    private String[] followLabels_;

    public
    AttributesDialog(Frame parent, TraderModel model)
    {
        super(parent, AppStandards.getString("AttributesTitleKey"));
        model_ = model;
        setResizable(true);

        followLabels_ = new String[3];
        followLabels_[0] = AppStandards.getString("LocalOnlyKey");
        followLabels_[1] = AppStandards.getString("IfNoneLocalKey");
        followLabels_[2] = AppStandards.getString("AlwaysKey");

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        pane_ = AppStandards.createTabbedPane();

        JPanel supportPanel = new JPanel();
        supportPanel.setLayout(new GridBagLayout());
        allowModifiableProperties_ =
            AppStandards.createCheckBox("SupportModPropKey");
        allowModifiableProperties_.setHorizontalAlignment(JCheckBox.LEFT);
        Constrain.constrain(supportPanel, allowModifiableProperties_,
            0, 0, 1, 1, GridBagConstraints.HORIZONTAL,
            GridBagConstraints.NORTHWEST, 1.0, 0.0, 10, 10, 0, 5);
        allowDynamicProperties_ =
            AppStandards.createCheckBox("SupportDynPropKey");
        allowDynamicProperties_.setHorizontalAlignment(JCheckBox.LEFT);
        Constrain.constrain(supportPanel, allowDynamicProperties_, 0, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 10, 0, 5);
        allowProxyOffers_ =
            AppStandards.createCheckBox("SupportProxyKey");
        allowProxyOffers_.setHorizontalAlignment(JCheckBox.LEFT);
        Constrain.constrain(supportPanel, allowProxyOffers_, 0, 2, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 10, 0, 5);
        JPanel reposPanel = new JPanel();
        reposPanel.setBorder(
            AppStandards.createTitledBorder("ServiceTypeReposKey"));
        reposPanel.setLayout(new GridBagLayout());
        reposIOR_ = new IORInputPanel(parent, "IORKey");
        Constrain.constrain(reposPanel, reposIOR_, 0, 0, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 5, 5, 5);
        Constrain.constrain(supportPanel, reposPanel, 0, 3, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 10, 10, 10);
        Constrain.constrain(supportPanel, Box.createGlue(), 0, 4, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 0, 0, 0);

        pane_.addTab(AppStandards.getString("SupportTabKey"), supportPanel);

        JPanel importPanel = new JPanel();
        importPanel.setLayout(new GridBagLayout());
        Constrain.constrain(importPanel, AppStandards.createLabel("DefaultKey"),
            1, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.CENTER,
            0.0, 0.0, 10, 0, 5, 0);
        Constrain.constrain(importPanel, AppStandards.createLabel("MaximumKey"),
            2, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.CENTER,
            0.0, 0.0, 10, 0, 5, 10);
        Constrain.constrain(importPanel,
            AppStandards.createLabel("SearchCardKey"),
            0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 5, 10);
        defSearchCard_ = AppStandards.createTextField(10);
        Constrain.constrain(importPanel, defSearchCard_,
            1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 5, 10);
        maxSearchCard_ = AppStandards.createTextField(10);
        Constrain.constrain(importPanel, maxSearchCard_,
            2, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 0, 5, 10);
        Constrain.constrain(importPanel,
            AppStandards.createLabel("MatchCardKey"),
            0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 5, 10);
        defMatchCard_ = AppStandards.createTextField(10);
        Constrain.constrain(importPanel, defMatchCard_,
            1, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 5, 10);
        maxMatchCard_ = AppStandards.createTextField(10);
        Constrain.constrain(importPanel, maxMatchCard_,
            2, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 0, 5, 10);
        Constrain.constrain(importPanel,
            AppStandards.createLabel("ReturnCardKey"),
            0, 3, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 5, 10);
        defReturnCard_ = AppStandards.createTextField(10);
        Constrain.constrain(importPanel, defReturnCard_,
            1, 3, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 5, 10);
        maxReturnCard_ = AppStandards.createTextField(10);
        Constrain.constrain(importPanel, maxReturnCard_,
            2, 3, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 0, 5, 10);
        Constrain.constrain(importPanel,
            AppStandards.createLabel("LinkHopCountKey"),
            0, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 5, 10);
        defHopCount_ = AppStandards.createTextField(10);
        Constrain.constrain(importPanel, defHopCount_,
            1, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 5, 10);
        maxHopCount_ = AppStandards.createTextField(10);
        Constrain.constrain(importPanel, maxHopCount_,
            2, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 0, 5, 10);
        Constrain.constrain(importPanel,
            AppStandards.createLabel("LinkFollowPolicyKey"),
            0, 5, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 5, 10);
        defFollowPolicy_ = AppStandards.createComboBox();
        defFollowPolicy_.addItem(followLabels_[0]);
        defFollowPolicy_.addItem(followLabels_[1]);
        defFollowPolicy_.addItem(followLabels_[2]);
        Constrain.constrain(importPanel, defFollowPolicy_,
            1, 5, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 0, 10, 5, 10, 2, 2);
        maxFollowPolicy_ = AppStandards.createComboBox();
        maxFollowPolicy_.addItem(followLabels_[0]);
        maxFollowPolicy_.addItem(followLabels_[1]);
        maxFollowPolicy_.addItem(followLabels_[2]);
        Constrain.constrain(importPanel, maxFollowPolicy_,
            2, 5, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 0, 0, 5, 10, 2, 2);
        Constrain.constrain(importPanel,
            AppStandards.createLabel("MaxListCountKey"),
            0, 6, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 15, 10);
        maxList_ = AppStandards.createTextField(10);
        Constrain.constrain(importPanel, maxList_,
            1, 6, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 10, 15, 10);

        pane_.addTab(AppStandards.getString("ImportTabKey"), importPanel);

        JPanel linkPanel = new JPanel();
        linkPanel.setLayout(new GridBagLayout());
        Constrain.constrain(linkPanel,
            AppStandards.createLabel("MaxLinkPolicyKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);
        maxLinkFollowPolicy_ = AppStandards.createComboBox();
        maxLinkFollowPolicy_.addItem(followLabels_[0]);
        maxLinkFollowPolicy_.addItem(followLabels_[1]);
        maxLinkFollowPolicy_.addItem(followLabels_[2]);
        Constrain.constrain(linkPanel, maxLinkFollowPolicy_,
            1, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 10, 20, 2);
        Constrain.constrain(linkPanel, Box.createHorizontalGlue(),
            2, 0, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 0, 0, 0, 0);
        Constrain.constrain(linkPanel, Box.createGlue(),
            0, 1, 3, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 0, 0, 0);

        pane_.addTab(AppStandards.getString("LinkTabKey"), linkPanel);

        JPanel adminPanel = new JPanel();
        adminPanel.setLayout(new GridBagLayout());
        Constrain.constrain(adminPanel,
            AppStandards.createLabel("RequestIdStemKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 10, 0);
        requestIdStem_ = AppStandards.createTextField(15);
        Constrain.constrain(adminPanel, requestIdStem_,
            1, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 10, 10);
        Constrain.constrain(adminPanel, Box.createHorizontalGlue(),
            2, 0, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 0, 0, 0, 0);
        Constrain.constrain(adminPanel, Box.createGlue(),
            0, 1, 3, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 0, 0, 0);

        pane_.addTab(AppStandards.getString("AdminTabKey"), adminPanel);

        Constrain.constrain(content, pane_, 0, 0, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 10, 10, 10, 10);

        pane_.setSelectedIndex(0);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 5));
        JButton button;
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 1, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 0, 10, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 2, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);
    }

    //
    // Override setVisible() so that we initialize the dialog's fields
    // when made visible
    //
    public void
    setVisible(boolean visible)
    {
        if(visible && ! isVisible())
            initDialog();

        super.setVisible(visible);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            cancel();
    }

    protected void
    initDialog()
    {
        status_.setText(AppStandards.getString("LoadingAttribKey"));

        try
        {
            Admin admin = model_.getAdmin();

            org.omg.CORBA.Object object;
            org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();

            allowModifiablePropertiesValue_ =
                admin.supports_modifiable_properties();
            allowDynamicPropertiesValue_ = admin.supports_dynamic_properties();
            allowProxyOffersValue_ = admin.supports_proxy_offers();
            object = admin.type_repos();
            reposIORValue_ = orb.object_to_string(object);
            defSearchCardValue_ = admin.def_search_card();
            maxSearchCardValue_ = admin.max_search_card();
            defMatchCardValue_ = admin.def_match_card();
            maxMatchCardValue_ = admin.max_match_card();
            defReturnCardValue_ = admin.def_return_card();
            maxReturnCardValue_ = admin.max_return_card();
            defHopCountValue_ = admin.def_hop_count();
            maxHopCountValue_ = admin.max_hop_count();
            defFollowPolicyValue_ = admin.def_follow_policy();
            maxFollowPolicyValue_ = admin.max_follow_policy();
            maxListValue_ = admin.max_list();
            maxLinkFollowPolicyValue_ = admin.max_link_follow_policy();
            requestIdStemValue_ = admin.request_id_stem();

            allowModifiableProperties_.setSelected(
                allowModifiablePropertiesValue_);
            allowDynamicProperties_.setSelected(allowDynamicPropertiesValue_);
            allowProxyOffers_.setSelected(allowProxyOffersValue_);
            reposIOR_.setIOR(reposIORValue_);
            defSearchCard_.setText("" + defSearchCardValue_);
            maxSearchCard_.setText("" + maxSearchCardValue_);
            defMatchCard_.setText("" + defMatchCardValue_);
            maxMatchCard_.setText("" + maxMatchCardValue_);
            defReturnCard_.setText("" + defReturnCardValue_);
            maxReturnCard_.setText("" + maxReturnCardValue_);
            defHopCount_.setText("" + defHopCountValue_);
            maxHopCount_.setText("" + maxHopCountValue_);
            maxList_.setText("" + maxListValue_);
            defFollowPolicy_.setSelectedIndex(defFollowPolicyValue_.value());
            maxFollowPolicy_.setSelectedIndex(maxFollowPolicyValue_.value());
            maxLinkFollowPolicy_.setSelectedIndex(
                maxLinkFollowPolicyValue_.value());
            requestIdStem_.setText(formatRequestIdStem(requestIdStemValue_));

            status_.clear();
        }
        catch(org.omg.CORBA.SystemException e)
        {
            status_.setText(AppStandards.getString("SystemErrorKey"));
        }
    }

    protected void
    ok()
    {
        if(collectSettings())
            setVisible(false);
    }

    protected void
    cancel()
    {
        setVisible(false);
    }

    protected boolean
    collectSettings()
    {
        boolean result = false;

        boolean modProps, dynProps, proxyOffers;
        ServiceTypeRepository str = null;
        String reposRef = reposIORValue_;
        int defSearch, maxSearch;
        int defMatch, maxMatch;
        int defReturn, maxReturn;
        int defHop, maxHop;
        FollowOption defFollow, maxFollow;
        int maxList;
        FollowOption maxLinkFollow;
        byte[] requestIdStem;

        modProps = allowModifiableProperties_.isSelected();
        dynProps = allowDynamicProperties_.isSelected();
        proxyOffers = allowProxyOffers_.isSelected();

        if(! reposIOR_.isNil())
        {
            StringBuffer message = new StringBuffer();
            org.omg.CORBA.Object object;
            object = reposIOR_.getObject(message);
            if(object == null)
            {
                MessageDialog.showError(this, message.toString());
                return false;
            }

            str = ServiceTypeRepositoryHelper.narrow(object);
            if(str == null)
            {
                MessageDialog.showError(this,
                    AppStandards.getString("InvalidSTReposKey"));
                return false;
            }

            org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();
            reposRef = orb.object_to_string(str);
        }

        try
        {
            defSearch = parseInt(AppStandards.getString("DefSearchCardKey"),
                defSearchCard_.getText());
            maxSearch = parseInt(AppStandards.getString("MaxSearchCardKey"),
                maxSearchCard_.getText());
            defMatch = parseInt(AppStandards.getString("DefMatchCardKey"),
                defMatchCard_.getText());
            maxMatch = parseInt(AppStandards.getString("MaxMatchCardKey"),
                maxMatchCard_.getText());
            defReturn = parseInt(AppStandards.getString("DefReturnCardKey"),
                defReturnCard_.getText());
            maxReturn = parseInt(AppStandards.getString("MaxReturnCardKey"),
                maxReturnCard_.getText());
            defHop = parseInt(AppStandards.getString("DefHopCountKey"),
                defHopCount_.getText());
            maxHop = parseInt(AppStandards.getString("MaxHopCountKey"),
                maxHopCount_.getText());
            defFollow =
                FollowOption.from_int(defFollowPolicy_.getSelectedIndex());
            maxFollow =
                FollowOption.from_int(maxFollowPolicy_.getSelectedIndex());
            maxList = parseInt(AppStandards.getString("MaxListCountKey"),
                maxList_.getText());
            maxLinkFollow =
                FollowOption.from_int(maxLinkFollowPolicy_.getSelectedIndex());

            requestIdStem = parseRequestIdStem(requestIdStem_.getText());
            if(requestIdStem == null)
                return false;

            //
            // compare the values and only update what has been changed
            //
            Admin admin = model_.getAdmin();

            if(modProps != allowModifiablePropertiesValue_)
            {
                admin.set_supports_modifiable_properties(modProps);
                allowModifiablePropertiesValue_ = modProps;
            }

            if(dynProps != allowDynamicPropertiesValue_)
            {
                admin.set_supports_dynamic_properties(dynProps);
                allowDynamicPropertiesValue_ = dynProps;
            }

            if(proxyOffers != allowProxyOffersValue_)
            {
                admin.set_supports_proxy_offers(proxyOffers);
                allowProxyOffersValue_ = proxyOffers;
            }

            if(! reposIORValue_.equals(reposRef))
            {
                admin.set_type_repos(str);
                reposIORValue_ = reposRef;
                model_.setRepos(str);
            }

            if(defSearch != defSearchCardValue_)
            {
                admin.set_def_search_card(defSearch);
                defSearchCardValue_ = defSearch;
            }

            if(maxSearch != maxSearchCardValue_)
            {
                admin.set_max_search_card(maxSearch);
                maxSearchCardValue_ = maxSearch;
            }

            if(defMatch != defMatchCardValue_)
            {
                admin.set_def_match_card(defMatch);
                defMatchCardValue_ = defMatch;
            }

            if(maxMatch != maxMatchCardValue_)
            {
                admin.set_max_match_card(maxMatch);
                maxMatchCardValue_ = maxMatch;
            }

            if(defReturn != defReturnCardValue_)
            {
                admin.set_def_return_card(defReturn);
                defReturnCardValue_ = defReturn;
            }

            if(maxReturn != maxReturnCardValue_)
            {
                admin.set_max_return_card(maxReturn);
                maxReturnCardValue_ = maxReturn;
            }

            if(defHop != defHopCountValue_)
            {
                admin.set_def_hop_count(defHop);
                defHopCountValue_ = defHop;
            }

            if(maxHop != maxHopCountValue_)
            {
                admin.set_max_hop_count(maxHop);
                maxHopCountValue_ = maxHop;
            }

            if(defFollow != defFollowPolicyValue_)
            {
                admin.set_def_follow_policy(defFollow);
                defFollowPolicyValue_ = defFollow;
            }

            if(maxFollow != maxFollowPolicyValue_)
            {
                admin.set_max_follow_policy(maxFollow);
                maxFollowPolicyValue_ = maxFollow;
            }

            if(maxList != maxListValue_)
            {
                admin.set_max_list(maxList);
                maxListValue_ = maxList;
            }

            if(maxLinkFollow != maxLinkFollowPolicyValue_)
            {
                admin.set_max_link_follow_policy(maxLinkFollow);
                maxLinkFollowPolicyValue_ = maxLinkFollow;
            }

            boolean stemEq =
                (requestIdStem.length == requestIdStemValue_.length);
            int pos = 0;
            while(pos < requestIdStem.length &&
                  pos < requestIdStemValue_.length)
            {
                if(requestIdStem[pos] != requestIdStemValue_[pos])
                    stemEq = false;
                pos++;
            }

            if(! stemEq)
            {
                admin.set_request_id_stem(requestIdStem);
                requestIdStemValue_ = requestIdStem;
            }

            result = true;
        }
        catch(NumberFormatException e)
        {
        }
        catch(org.omg.CORBA.SystemException e)
        {
            MessageDialog.showError(this,
                                    AppStandards.getString("SystemErrorKey"));
        }

        return result;
    }

    protected int
    parseInt(String name, String value)
        throws NumberFormatException
    {
        int result = 0;

        try
        {
            Integer i = Integer.valueOf(value.trim());
            result = i.intValue();
        }
        catch(NumberFormatException e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("InvalidValueKey", name));
            throw e;
        }

        return result;
    }

    protected String
    formatRequestIdStem(byte[] stem)
    {
        return new String(stem);
    }

    protected byte[]
    parseRequestIdStem(String str)
    {
        byte[] result = str.trim().getBytes();

        return result;
    }
}
