// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Tools;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.LinkPackage.*;

public class ConnectDialog
    extends JDialog
    implements ActionListener, ItemListener
{
    private Frame parent_;
    private TraderModel model_;
    private ActionListener owner_;
    private String actionCommand_;
    private Lookup trader_;

    private JRadioButton iorButton_;
    private JTextField ior_;
    private JRadioButton fileButton_;
    private JTextField file_;
    private JButton browseButton_;
    private JRadioButton linkButton_;
    private JComboBox link_;
    private StatusBar status_;
    private FileDialog dialog_;

    public
    ConnectDialog(Frame parent, TraderModel model, ActionListener owner)
    {
        super(parent, AppStandards.getString("ConnectTitleKey"));
        parent_ = parent;
        model_ = model;
        owner_ = owner;
        setResizable(true);

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        ButtonGroup group = new ButtonGroup();
        iorButton_ = AppStandards.createRadioButton("ConnectIORKey");
        iorButton_.addItemListener(this);
        group.add(iorButton_);
        Constrain.constrain(content, iorButton_, 0, 0, 2, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);
        ior_ = AppStandards.createTextField(25);
        Constrain.constrain(content, ior_, 0, 1, 2, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 0, 10, 0, 10);

        fileButton_ = AppStandards.createRadioButton("ConnectFileKey");
        fileButton_.addItemListener(this);
        group.add(fileButton_);
        Constrain.constrain(content, fileButton_, 0, 2, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 5, 0);
        browseButton_ = AppStandards.createButton("BrowseButtonKey");
        browseButton_.addActionListener(this);
        browseButton_.setActionCommand("browse");
        Constrain.constrain(content, browseButton_, 1, 2, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 5, 5, 10);
        file_ = AppStandards.createTextField(25);
        Constrain.constrain(content, file_, 0, 3, 2, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 0, 10, 0, 10);

        linkButton_ = AppStandards.createRadioButton("ConnectLinkKey");
        linkButton_.addItemListener(this);
        group.add(linkButton_);
        Constrain.constrain(content, linkButton_, 0, 4, 2, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);
        link_ = AppStandards.createComboBox();
        link_.setMaximumRowCount(3);
        Constrain.constrain(content, link_, 0, 5, 2, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 0, 10, 0, 10, 20, 5);

        Constrain.constrain(content, Box.createGlue(), 0, 6, 2, 1,
            GridBagConstraints.BOTH, GridBagConstraints.CENTER,
            1.0, 1.0, 0, 0, 0, 0);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        JButton button;
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 7, 2, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 15, 0, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 8, 2, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);
    }

    public void
    setActionCommand(String command)
    {
        actionCommand_ = command;
    }

    public Lookup
    getTrader()
    {
        return trader_;
    }

    //
    // Override setVisible() so that we initialize the dialog's fields
    // when made visible
    //
    public void
    setVisible(boolean visible)
    {
        if(visible && ! isVisible())
            initDialog();

        super.setVisible(visible);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            close();
        else if(command.equals("browse"))
        {
            synchronized(this)
            {
                if(dialog_ == null)
                {
                    String title = AppStandards.getString("IORFileTitleKey");
                    dialog_ = new FileDialog(parent_, title, FileDialog.LOAD);
                }
            }
            dialog_.show();
            String dir = dialog_.getDirectory();
            String file = dialog_.getFile();
            if(dir != null && file != null)
            {
                File f = new File(dir, file);
                file_.setText(f.getAbsolutePath());
            }
        }
    }

    public void
    itemStateChanged(ItemEvent e)
    {
        if(e.getStateChange() == ItemEvent.SELECTED)
        {
            Object item = e.getItem();
            ior_.setEditable(item == iorButton_);
            file_.setEditable(item == fileButton_);
            browseButton_.setEnabled(item == fileButton_);
            link_.setEnabled(item == linkButton_);
        }
    }

    protected void
    initDialog()
    {
        try
        {
            iorButton_.setSelected(true);
            ior_.setText("");
            file_.setText("");
            trader_ = null;

            status_.setText(AppStandards.getString("LoadingLinksKey"));

            link_.removeAllItems();
            Link link = model_.getLink();
            String[] links = link.list_links();
            QuickSort.sort(links);
            for(int i = 0 ; i < links.length ; i++)
                link_.addItem(links[i]);
            link_.getModel().setSelectedItem(null);

            status_.clear();
        }
        catch(org.omg.CORBA.SystemException e)
        {
            status_.setText(AppStandards.getString("SystemErrorKey"));
        }
    }

    protected void
    ok()
    {
        try
        {
            org.omg.CORBA.Object obj;
            StringBuffer message = new StringBuffer();

            if(iorButton_.isSelected())
            {
                String ref = ior_.getText().trim();
                obj = IORUtil.getObject(ref, message);
            }
            else if(fileButton_.isSelected())
            {
                String fileName = file_.getText().trim();
                File f = new File(fileName);
                obj = IORUtil.getObject(f, message);
            }
            else // link
            {
                String linkName = (String)link_.getSelectedItem();
                if(linkName == null)
                {
                    MessageDialog.showError(this,
                        AppStandards.getString("NothingSelectedKey"));
                    return;
                }

                Link link = model_.getLink();
                LinkInfo info = link.describe_link(linkName);
                obj = info.target;
            }

            if(obj == null)
            {
                MessageDialog.showError(this, message.toString());
                return;
            }

            Lookup trader = null;
            try
            {
                trader = LookupHelper.narrow(obj);
            }
            catch(org.omg.CORBA.SystemException e)
            {
                MessageDialog.showError(this,
                    AppStandards.getString("InvalidLookupKey"));
                return;
            }

            if(trader == null)
            {
                MessageDialog.showError(this,
                    AppStandards.getString("InvalidLookupKey"));
                return;
            }

            try
            {
                // just test to make sure we can connect to the trader
                Lookup dummy = trader.lookup_if();
            }
            catch(org.omg.CORBA.SystemException e)
            {
                MessageDialog.showError(this,
                    AppStandards.getString("CannotConnectKey"));
                return;
            }

            trader_ = trader;
            close();
            owner_.actionPerformed(
                new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
                                actionCommand_));
        }
        catch(IllegalLinkName e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("IllegalLinkNameKey", e.name));
        }
        catch(UnknownLinkName e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("UnknownLinkNameKey", e.name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            MessageDialog.showError(this,
                AppStandards.getString("SystemErrorKey"));
        }
    }

    protected void
    close()
    {
        status_.clear();
        setVisible(false);
    }
}
