// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Tools;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.LookupPackage.*;

public class QueryDialog
    extends JDialog
    implements ActionListener, ItemListener
{
    private Frame parent_;
    private TraderModel model_;

    private JComboBox type_;
    private TypeComboBoxModel typeModel_;
    private JTextField constraint_;
    private JTextField preference_;
    private JCheckBox desiredPropsCheck_;
    private JTextField desiredProps_;
    private StatusBar status_;
    private QueryPoliciesDialog policiesDialog_;

    public
    QueryDialog(Frame parent, TraderModel model)
    {
        super(parent, AppStandards.getString("QueryTitleKey"));
        parent_ = parent;
        model_ = model;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Close" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    close();
                }
            }
        );

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        Constrain.constrain(content, AppStandards.createLabel("QueryTypeKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        typeModel_ = new TypeComboBoxModel(model_);
        type_ = AppStandards.createComboBox(typeModel_);
        type_.setMaximumRowCount(10);
        type_.setEditable(true);
        Constrain.constrain(content, type_, 0, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);
        Constrain.constrain(content, AppStandards.createLabel("ConstraintKey"),
            0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        constraint_ = AppStandards.createTextField(30);
        Constrain.constrain(content, constraint_, 0, 3, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);
        Constrain.constrain(content, AppStandards.createLabel("PreferenceKey"),
            0, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        preference_ = AppStandards.createTextField(30);
        Constrain.constrain(content, preference_, 0, 5, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);
        desiredPropsCheck_ = AppStandards.createCheckBox("DesiredPropsKey");
        desiredPropsCheck_.addItemListener(this);
        Constrain.constrain(content, desiredPropsCheck_, 0, 6, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        desiredProps_ = AppStandards.createTextField(30);
        Constrain.constrain(content, desiredProps_, 0, 7, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);

        Constrain.constrain(content, Box.createGlue(), 0, 8, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.CENTER,
            1.0, 1.0, 0, 0, 0, 0);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 3, 15, 0));
        JButton button;
        button = AppStandards.createButton("QueryButtonKey");
        button.setActionCommand("query");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("PoliciesButtonKey");
        button.setActionCommand("policies");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CloseButtonKey");
        button.setActionCommand("close");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 9, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.CENTER,
            0.0, 0.0, 15, 10, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 10, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);

        desiredPropsCheck_.setSelected(false);
    }

    //
    // Override setVisible() so that we initialize the dialog's fields
    // when made visible
    //
    public void
    setVisible(boolean visible)
    {
        if(visible && ! isVisible())
            initDialog();

        super.setVisible(visible);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("query"))
            query();
        else if(command.equals("policies"))
            showPoliciesDialog();
        else if(command.equals("close"))
            close();
    }

    public void
    itemStateChanged(ItemEvent e)
    {
        if(e.getStateChange() == ItemEvent.SELECTED)
            desiredProps_.setEditable(true);
        else if(e.getStateChange() == ItemEvent.DESELECTED)
            desiredProps_.setEditable(false);
    }

    protected void
    initDialog()
    {
        status_.setText(AppStandards.getString("LoadingTypesKey"));

        try
        {
            type_.setEnabled(false);
            typeModel_.reload();
            type_.setEnabled(true);
        }
        catch(org.omg.CORBA.SystemException e)
        {
            MessageDialog.showSystemError(this, e);
        }

        status_.clear();
    }

    protected void
    query()
    {
        try
        {
            status_.setText(AppStandards.getString("QueryingKey"));

            String type = (String)type_.getSelectedItem();
            if(type == null)
            {
                MessageDialog.showError(this,
                                        AppStandards.getString("NoTypeKey"));
                return;
            }

            String constraint = constraint_.getText().trim();
            String preference = preference_.getText().trim();

            //
            // Retrieve the policies from the policies dialog
            //
            Policy[] policies;
            if(policiesDialog_ != null)
                policies = policiesDialog_.getPolicies();
            else
                policies = new Policy[0];

            SpecifiedProps desiredProps = new SpecifiedProps();
            if(! desiredPropsCheck_.isSelected())
                desiredProps._default(HowManyProps.all);
            else
            {
                String props = desiredProps_.getText().trim();
                if(props.length() == 0)
                    desiredProps._default(HowManyProps.none);
                else
                {
                    StringTokenizer tok = new StringTokenizer(props, ",");
                    String[] names = new String[tok.countTokens()];
                    int count = 0;
                    while(tok.hasMoreTokens())
                        names[count++] = tok.nextToken().trim();
                    desiredProps.prop_names(names);
                }
            }

            OfferSeqHolder offers = new OfferSeqHolder();
            OfferIteratorHolder iter = new OfferIteratorHolder();
            PolicyNameSeqHolder limits = new PolicyNameSeqHolder();

            Lookup lookup = model_.getLookup();
            lookup.query(type, constraint, preference, policies, desiredProps,
                         0, offers, iter, limits);

            if(iter.value == null)
            {
                MessageDialog.showError(this,
                                        AppStandards.getString("NoOffersKey"));
                return;
            }
            else
            {
                try
                {
                    int numOffers = iter.value.max_left();
                    if(numOffers == 0)
                    {
                        MessageDialog.showError(this,
                            AppStandards.getString("NoOffersKey"));
                        iter.value.destroy();
                        return;
                    }
                }
                catch(UnknownMaxLeft e)
                {
                }
            }

            status_.setText(AppStandards.getString("DisplayingResultsKey"));
            QueryResultsDialog results =
                new QueryResultsDialog(parent_, iter.value);
            results.setVisible(true);
            status_.clear();
        }
        catch(IllegalServiceType e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("IllegalTypeNameKey", e.type));
        }
        catch(UnknownServiceType e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("UnknownTypeNameKey", e.type));
        }
        catch(IllegalConstraint e)
        {
            MessageDialog.showError(this,
                AppStandards.getString("IllegalConstraintKey"));
        }
        catch(IllegalPreference e)
        {
            MessageDialog.showError(this,
                AppStandards.getString("IllegalPrefKey"));
        }
        catch(IllegalPolicyName e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("IllegalPolicyKey", e.name));
        }
        catch(PolicyTypeMismatch e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("PolicyTypeMismatchKey",
                    e.the_policy.name));
        }
        catch(InvalidPolicyValue e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("InvalidPolicyValueKey",
                    e.the_policy.name));
        }
        catch(IllegalPropertyName e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("IllegalPropNameKey", e.name));
        }
        catch(DuplicatePropertyName e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("DuplicatePropNameKey",
                                                e.name));
        }
        catch(DuplicatePolicyName e)
        {
            MessageDialog.showError(this,
                AppStandards.getFormattedString("DuplicatePolicyKey", e.name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            MessageDialog.showSystemError(this, e);
        }
        finally
        {
            status_.clear();
        }
    }

    protected synchronized void
    showPoliciesDialog()
    {
        Frame parent = parent_;

        if(policiesDialog_ == null)
            policiesDialog_ = new QueryPoliciesDialog(parent, model_);

        policiesDialog_.setVisible(true);
    }

    protected void
    close()
    {
        setVisible(false);

        if(policiesDialog_ != null)
            policiesDialog_.setVisible(false);
    }
}
