// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Tools;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.LookupPackage.*;

public class QueryResultsDialog extends JDialog implements ActionListener
{
    private OfferIterator iter_;
    private JButton next_;
    private JButton previous_;
    private JTextField record_;
    private JTextField reference_;
    private JTable props_;
    private StatusBar status_;
    private boolean more_;
    private int pos_;
    private Offer[] offers_;
    private static final int INCREMENT = 25;

    public
    QueryResultsDialog(Frame parent, OfferIterator iter)
    {
        super(parent, AppStandards.getString("QueryResultsTitleKey"));
        iter_ = iter;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Close" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    close();
                }
            }
        );

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());
        JButton button;

        Constrain.constrain(content, AppStandards.createLabel("OfferNumKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);
        previous_ = new JButton("<");
        previous_.setActionCommand("prev");
        previous_.addActionListener(this);
        Constrain.constrain(content, previous_, 1, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);

        record_ = AppStandards.createTextField(5);
        Keymap keymap = record_.getKeymap();
        keymap.addActionForKeyStroke(
            KeyStroke.getKeyStroke('\r'),
            new AbstractAction()
            {
                public void
                actionPerformed(ActionEvent event)
                {
                    gotoRecord();
                }
            }
        );
        Constrain.constrain(content, record_, 2, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);

        next_ = new JButton(">");
        next_.setActionCommand("next");
        next_.addActionListener(this);
        Constrain.constrain(content, next_, 3, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 10, 10, 0, 0);
        JLabel label = new JLabel("");
        Constrain.constrain(content, label, 4, 0, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 10, 10, 0, 0);

        Constrain.constrain(content, AppStandards.createLabel("OfferRefKey"),
            0, 1, 5, 1, GridBagConstraints.HORIZONTAL,
            GridBagConstraints.NORTHWEST, 1.0, 0.0, 15, 10, 0, 10);
        reference_ = AppStandards.createTextField(30);
        reference_.setEditable(false);
        Constrain.constrain(content, reference_, 0, 2, 5, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);

        Constrain.constrain(content, AppStandards.createLabel("OfferPropsKey"),
            0, 3, 5, 1, GridBagConstraints.HORIZONTAL,
            GridBagConstraints.NORTHWEST, 1.0, 0.0, 10, 10, 0, 10);
        props_ = new JTable();
        props_.setAutoCreateColumnsFromModel(false);
        props_.getTableHeader().setReorderingAllowed(false);
        TableColumn column;
        column = new TableColumn(0, 100);
        column.setHeaderValue(AppStandards.getString("NameKey"));
        column.setMinWidth(100);
        props_.addColumn(column);
        column = new TableColumn(1, 100);
        column.setHeaderValue(AppStandards.getString("ValueKey"));
        column.setMinWidth(20);
        props_.addColumn(column);
        props_.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
        props_.setRowSelectionAllowed(false);
        props_.setColumnSelectionAllowed(false);
        JScrollPane scrollPane = new JScrollPane(props_);
        scrollPane.setPreferredSize(new Dimension(200, 200));
        scrollPane.setBorder(BorderFactory.createLoweredBevelBorder());
        Constrain.constrain(content, scrollPane, 0, 4, 5, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 10, 0, 10);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 1, 5, 0));
        button = AppStandards.createButton("CloseButtonKey");
        button.setActionCommand("close");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 5, 5, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 10, 0, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 6, 5, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        try
        {
            int max = iter_.max_left();
            label.setText(AppStandards.getFormattedString("OfferTotalKey",
                                                          new Integer(max)));
        }
        catch(UnknownMaxLeft e)
        {
        }

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 40, p.y + 40);
    }

    //
    // Override setVisible() so that we initialize the dialog's fields
    // when made visible
    //
    public void
    setVisible(boolean visible)
    {
        if(visible && ! isVisible())
            initDialog();

        super.setVisible(visible);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();

        if(command.equals("next"))
            next();
        else if(command.equals("prev"))
            previous();
        else if(command.equals("close"))
            close();
    }

    protected void
    initDialog()
    {
        previous_.setEnabled(false);
        next_.setEnabled(false);

        try
        {
            OfferSeqHolder holder = new OfferSeqHolder();
            more_ = iter_.next_n(INCREMENT, holder);
            offers_ = holder.value;
            pos_ = 0;
            showOffer();
            updateButtons();
        }
        catch(org.omg.CORBA.SystemException e)
        {
            MessageDialog.showSystemError(this, e);
        }
    }

    protected synchronized void
    next()
    {
        setPosition(pos_ + 1);
    }

    protected synchronized void
    previous()
    {
        setPosition(pos_ - 1);
    }

    protected void
    close()
    {
        setVisible(false);
        dispose();

        try
        {
            iter_.destroy();
        }
        catch(org.omg.CORBA.SystemException e)
        {
        }
    }

    protected synchronized void
    gotoRecord()
    {
        String recNo = record_.getText().trim();

        try
        {
            Integer i = Integer.valueOf(recNo);
            int val = i.intValue();
            setPosition(val - 1);
        }
        catch(NumberFormatException e)
        {
            record_.setText("" + (pos_ + 1));
        }
    }

    protected void
    setPosition(int pos)
    {
        boolean result = false;
        int newPos = pos_;

        if(pos < 0)
            newPos = 0;
        else if(pos >= 0 && pos < offers_.length)
            newPos = pos;
        else if(pos >= 0 && more_)
        {
            try
            {
                status_.setText(AppStandards.getString("LoadingMoreOffersKey"));

                //
                // Advance until we reach the end or the right position
                //
                while(more_ && pos >= offers_.length)
                {
                    OfferSeqHolder holder = new OfferSeqHolder();
                    more_ = iter_.next_n(INCREMENT, holder);
                    Offer[] arr =
                        new Offer[offers_.length + holder.value.length];
                    System.arraycopy(offers_, 0, arr, 0, offers_.length);
                    System.arraycopy(holder.value, 0, arr, offers_.length,
                        holder.value.length);
                    offers_ = arr;
                }

                if(pos < offers_.length)
                    newPos = pos;
                else
                    newPos = offers_.length - 1;

                status_.clear();
            }
            catch(org.omg.CORBA.SystemException e)
            {
                MessageDialog.showSystemError(this, e);
            }
        }
        else
            newPos = offers_.length - 1;

        pos_ = newPos;
        updateButtons();
        showOffer();
    }

    protected void
    showOffer()
    {
        record_.setText("" + (pos_ + 1));

        String ior;

        org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();

        org.omg.CORBA.Object obj = offers_[pos_].reference;
        if(obj == null)
            ior = AppStandards.getString("NilKey");
        else
            ior = orb.object_to_string(obj);

        reference_.setText(ior);
        reference_.setCaretPosition(0);

        int count = offers_[pos_].properties.length;
        Object[][] data = new Object[count][2];
        for(int i = 0 ; i < count ; i++)
        {
            data[i][0] = offers_[pos_].properties[i].name;
            StringWriter sw = new StringWriter();
            com.ooc.Util.CORBA.AnyWriter out =
                new com.ooc.Util.CORBA.AnyWriter(new PrintWriter(sw));
            org.omg.CORBA.DynAny dynAny =
                orb.create_dyn_any(offers_[pos_].properties[i].value);
            out.printDynAny(dynAny, false);
            sw.flush();
            data[i][1] = sw.toString();
        }

        DefaultTableModel model = (DefaultTableModel)props_.getModel();
        model.setNumRows(offers_[pos_].properties.length);
        String[] columns = new String[2];
        columns[0] = AppStandards.getString("OfferPropNameKey");
        columns[1] = AppStandards.getString("OfferPropValueKey");
        model.setDataVector(data, columns);
    }

    protected void
    updateButtons()
    {
        previous_.setEnabled(pos_ > 0);
        next_.setEnabled(pos_ < (offers_.length - 1) || more_);
    }
}
