// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Tools;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.LookupPackage.*;

// Withdraw offers using constraint
public class WithdrawDialog extends JDialog implements ActionListener
{
    private Frame parent_;
    private TraderModel model_;

    private JComboBox type_;
    private TypeComboBoxModel typeModel_;
    private JTextField constraint_;
    private StatusBar status_;

    public
    WithdrawDialog(Frame parent, TraderModel model)
    {
        super(parent, AppStandards.getString("WithdrawTitleKey"));
        parent_ = parent;
        model_ = model;
        setResizable(true);

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        Constrain.constrain(content,
            AppStandards.createLabel("WithdrawTypeKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        typeModel_ = new TypeComboBoxModel(model_);
        type_ = AppStandards.createComboBox(typeModel_);
        type_.setMaximumRowCount(10);
        Constrain.constrain(content, type_, 0, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);
        Constrain.constrain(content, AppStandards.createLabel("ConstraintKey"),
            0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        constraint_ = AppStandards.createTextField(30);
        Constrain.constrain(content, constraint_, 0, 3, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);

        Constrain.constrain(content, Box.createGlue(), 0, 4, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.CENTER,
            1.0, 1.0, 0, 0, 0, 0);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 10, 0));
        JButton button;
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 5, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 15, 0, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 6, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);
    }

    //
    // Override setVisible() so that we initialize the dialog's fields
    // when made visible
    //
    public void
    setVisible(boolean visible)
    {
        if(visible && ! isVisible())
            initDialog();

        super.setVisible(visible);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            cancel();
    }

    protected void
    initDialog()
    {
        constraint_.setText("");

        status_.setText(AppStandards.getString("LoadingTypesKey"));

        try
        {
            type_.setEnabled(false);
            typeModel_.reload();
            type_.setEnabled(true);
        }
        catch(org.omg.CORBA.SystemException e)
        {
            MessageDialog.showSystemError(this, e);
        }

        status_.clear();
    }

    protected void
    ok()
    {
        String type = (String)type_.getSelectedItem();
        if(type == null)
        {
            MessageDialog.showError(this, AppStandards.getString("NoTypeKey"));
            return;
        }

        String constraint = constraint_.getText().trim();
        StringBuffer message = new StringBuffer();

        status_.setText(AppStandards.getString("WithdrawingOffersKey"));
        boolean ok = model_.withdrawOffers(type, constraint, true, message);
        status_.clear();

        if(! ok)
            MessageDialog.showError(this, message.toString());
        else
            setVisible(false);
    }

    protected void
    cancel()
    {
        setVisible(false);
    }
}
