// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Type;

import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.CORBA.GUI.IntRep.*;
import com.ooc.Util.CORBA.IntRep.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class IRDialog
    extends JDialog
    implements ActionListener, TreeSelectionListener
{
    public interface Listener
    {
        public void
        interfaceSelected(String id);
    }

    private Frame parent_;
    private Listener listener_;
    private JTree tree_;
    private JTextField id_;
    private JButton ok_;

    public
    IRDialog(Frame parent,
             Listener listener,
             org.omg.CORBA.Repository repository)
    {
        super(parent, AppStandards.getString("IRTitleKey"));
        parent_ = parent;
        listener_ = listener;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Cancel" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    close();
                }
            }
        );

        org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        IRTreeModel treeModel =
            new IRTreeModel(orb, repository,
                            AppStandards.getString("IntRepNameKey"),
                            new Filter());
        JTree tree = new JTree(treeModel);
        tree.addTreeSelectionListener(this);
        JScrollPane scroll = new JScrollPane(tree);
        scroll.setPreferredSize(new Dimension(300, 300));

        Constrain.constrain(content,
            AppStandards.createLabel("SelectInterfaceKey"), 0, 0, 2, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        Constrain.constrain(content, scroll, 0, 1, 2, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 10, 0, 10);

        JPanel idPanel = new JPanel();
        idPanel.setLayout(new GridBagLayout());

        Constrain.constrain(idPanel,
            AppStandards.createLabel("InterfaceIDKey"), 0, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 0, 0, 0);
        id_ = AppStandards.createTextField(20);
        Constrain.constrain(idPanel, id_, 1, 0, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 0);
        Constrain.constrain(content, idPanel, 0, 2, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 10, 0, 10);

        JButton button;
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        ok_ = AppStandards.createButton("OkButtonKey");
        ok_.setActionCommand("ok");
        ok_.addActionListener(this);
        buttonPanel.add(ok_);
        ok_.setEnabled(false);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 3, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 10, 0, 10, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 40, p.y + 40);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            close();
    }

    protected void
    ok()
    {
        String id = id_.getText();
        listener_.interfaceSelected(id);
        close();
    }

    protected void
    close()
    {
        setVisible(false);
    }

    public void
    valueChanged(TreeSelectionEvent e)
    {
        TreePath path = e.getNewLeadSelectionPath();
        IRTreeNode node = (IRTreeNode)path.getLastPathComponent();
        IRComponent c = node.getComponent();
        org.omg.CORBA.DefinitionKind kind = c.getKind();
        if(kind == org.omg.CORBA.DefinitionKind.dk_Interface)
            id_.setText(c.getID());
        else
            id_.setText("");

        ok_.setEnabled((kind == org.omg.CORBA.DefinitionKind.dk_Interface));
    }

    static class Filter implements IRFilter
    {
        public boolean
        accept(IRComponent c)
        {
            org.omg.CORBA.DefinitionKind kind = c.getKind();
            return (kind == org.omg.CORBA.DefinitionKind.dk_Interface ||
                    kind == org.omg.CORBA.DefinitionKind.dk_Module);
        }

        public boolean
        isLeaf(IRComponent c)
        {
            org.omg.CORBA.DefinitionKind kind = c.getKind();
            if(kind == org.omg.CORBA.DefinitionKind.dk_Interface)
                return true;
            else
                return !c.isContainer();
        }
    }
}
