// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Type;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class NewTypeDialog
    extends JDialog
    implements ActionListener, IRDialog.Listener
{
    private Frame parent_;
    private TraderModel model_;

    private JTextField name_;
    private JTextField interface_;
    private JButton browseIR_;
    private JList superTypes_;
    private StringListModel superTypesModel_;
    private JList props_;
    private SortedListModel propsModel_;
    private StatusBar status_;
    private PropDialog propDialog_;
    private SuperDialog superDialog_;
    private IRDialog irDialog_;
    private org.omg.CORBA.Repository repository_ = null;

    public
    NewTypeDialog(Frame parent, TraderModel model)
    {
        super(parent, AppStandards.getString("NewTypeTitleKey"));
        parent_ = parent;
        model_ = model;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Cancel" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    cancel();
                }
            }
        );

        org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();
        try
        {
            org.omg.CORBA.Object obj;
            obj = orb.resolve_initial_references("InterfaceRepository");
            repository_ = org.omg.CORBA.RepositoryHelper.narrow(obj);
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            // no interface repository available
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            // ignore
        }

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        JButton button;

        Constrain.constrain(content, AppStandards.createLabel("TypeNameKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        name_ = AppStandards.createTextField(25);
        Constrain.constrain(content, name_, 0, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);

        JPanel interfacePanel = new JPanel();
        interfacePanel.setLayout(new GridBagLayout());
        Constrain.constrain(interfacePanel,
            AppStandards.createLabel("TypeInterfaceKey"), 0, 0, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.SOUTHWEST,
            1.0, 0.0, 0, 0, 0, 0);
        interface_ = AppStandards.createTextField(25);
        Constrain.constrain(interfacePanel, interface_, 0, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 0, 0, 0);
        browseIR_ = AppStandards.createButton("BrowseButtonKey");
        browseIR_.setActionCommand("browse");
        browseIR_.addActionListener(this);
        Constrain.constrain(interfacePanel, browseIR_, 1, 0, 1, 2,
            GridBagConstraints.NONE, GridBagConstraints.SOUTH,
            0.0, 0.0, 0, 5, 0, 0);
        Constrain.constrain(content, interfacePanel, 0, 2, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 10, 0, 10);


        JPanel superPanel = new JPanel();
        superPanel.setBorder(AppStandards.createTitledBorder("SuperTypesKey"));
        superPanel.setLayout(new GridBagLayout());
        superTypesModel_ = new StringListModel();
        superTypes_ = AppStandards.createList(superTypesModel_);
        JScrollPane superScroll = AppStandards.createScrollPane(superTypes_);
        superScroll.setPreferredSize(new Dimension(200, 75));
        Constrain.constrain(superPanel, superScroll, 0, 0, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 5, 5, 0);
        JPanel superButtons = new JPanel();
        superButtons.setLayout(new GridLayout(2, 1, 5, 5));
        button = AppStandards.createButton("AddButtonKey");
        button.setActionCommand("addType");
        button.addActionListener(this);
        superButtons.add(button);
        button = AppStandards.createButton("DeleteButtonKey");
        button.setActionCommand("deleteType");
        button.addActionListener(this);
        superButtons.add(button);
        Constrain.constrain(superPanel, superButtons, 1, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHEAST,
            0.0, 0.0, 0, 5, 5, 5);

        Constrain.constrain(content, superPanel, 0, 4, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 0.5, 10, 10, 0, 10);

        JPanel propPanel = new JPanel();
        propPanel.setBorder(AppStandards.createTitledBorder("TypePropsKey"));
        propPanel.setLayout(new GridBagLayout());
        propsModel_ = new SortedListModel();
        props_ = AppStandards.createList(propsModel_);
        JScrollPane propScroll = AppStandards.createScrollPane(props_);
        propScroll.setPreferredSize(new Dimension(200, 75));
        Constrain.constrain(propPanel, propScroll, 0, 0, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 5, 5, 0);
        JPanel propButtons = new JPanel();
        propButtons.setLayout(new GridLayout(3, 1, 5, 5));
        button = AppStandards.createButton("AddButtonKey");
        button.setActionCommand("addProp");
        button.addActionListener(this);
        propButtons.add(button);
        button = AppStandards.createButton("EditButtonKey");
        button.setActionCommand("editProp");
        button.addActionListener(this);
        propButtons.add(button);
        button = AppStandards.createButton("DeleteButtonKey");
        button.setActionCommand("deleteProp");
        button.addActionListener(this);
        propButtons.add(button);
        Constrain.constrain(propPanel, propButtons, 1, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHEAST,
            0.0, 0.0, 0, 5, 5, 5);

        Constrain.constrain(content, propPanel, 0, 5, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 0.5, 10, 10, 0, 10);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 6, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 15, 0, 0, 10);

        status_ = new StatusBar(false);
        Constrain.constrain(content, status_, 0, 7, 2, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 5, 10, 0, 10);

        pack();

        Point p = parent.getLocation();
        setLocation(p.x + 20, p.y + 20);
    }

    public void
    setValues(String name, TypeStruct ts)
    {
        if(name != null)
            name_.setText(name);
        else
            name_.setText("");

        superTypesModel_.removeAll();
        propsModel_.removeAll();

        if(ts != null)
        {
            interface_.setText(ts.if_name);
            for(int i = 0 ; i < ts.super_types.length ; i++)
                superTypesModel_.addItem(ts.super_types[i]);
            for(int i = 0 ; i < ts.props.length ; i++)
                propsModel_.addItem(new Prop(ts.props[i]));
        }
        else
        {
            interface_.setText("");
        }
    }

    //
    // Override setVisible() so that we initialize the dialog's fields
    // when made visible
    //
    public void
    setVisible(boolean visible)
    {
        status_.clear();
        super.setVisible(visible);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            cancel();
        else if(command.equals("addProp"))
        {
            if(propDialog_ == null)
                propDialog_ = new PropDialog(parent_, model_);
            propDialog_.setProperty(null);
            propDialog_.setVisible(true);
        }
        else if(command.equals("editProp"))
        {
            Prop p = (Prop)props_.getSelectedValue();
            if(p != null)
            {
                if(propDialog_ == null)
                    propDialog_ = new PropDialog(parent_, model_);
                propDialog_.setProperty(p);
                propDialog_.setVisible(true);
            }
        }
        else if(command.equals("deleteProp"))
        {
            Object[] props = (Object[])props_.getSelectedValues();
            if(props != null)
            {
                for(int i = 0 ; i < props.length ; i++)
                    propsModel_.removeItem((Prop)props[i]);
            }
        }
        else if(command.equals("addType"))
        {
            if(superDialog_ == null)
                superDialog_ = new SuperDialog(parent_, model_);
            superDialog_.setVisible(true);
        }
        else if(command.equals("deleteType"))
        {
            Object[] types = (Object[])superTypes_.getSelectedValues();
            if(types != null)
            {
                for(int i = 0 ; i < types.length ; i++)
                    superTypesModel_.removeItem((String)types[i]);
            }
        }
        else if(command.equals("browse"))
        {
            if(repository_ == null)
                MessageDialog.showError(this,
                    AppStandards.getString("NoIntRepKey"));
            else
            {
                try
                {
                    if(irDialog_ == null)
                        irDialog_ = new IRDialog(parent_, this, repository_);
                    irDialog_.setVisible(true);
                }
                catch(org.omg.CORBA.COMM_FAILURE ex)
                {
                    MessageDialog.showError(this,
                        AppStandards.getString("NoIntRepKey"));
                }
            }
        }
    }

    //
    // Inherited from IRDialog.Listener
    //
    public void
    interfaceSelected(String id)
    {
        interface_.setText(id);
    }

    protected void
    ok()
    {
        // collect inputs
        String name = name_.getText().trim();
        String ifName = interface_.getText().trim();
        String[] superTypes = superTypesModel_.getItems();
        Sortable[] propItems = propsModel_.getItems();
        PropStruct[] props = new PropStruct[propItems.length];
        for(int i = 0 ; i < propItems.length ; i++)
            props[i] = ((Prop)propItems[i]).getPropStruct();

        status_.setText(AppStandards.getString("CreatingTypeKey"));

        StringBuffer message = new StringBuffer();
        boolean ok =
            model_.addType(name, ifName, props, superTypes, true, message);

        status_.clear();

        if(! ok)
            MessageDialog.showError(this, message.toString());
        else
            setVisible(false);
    }

    protected void
    cancel()
    {
        setVisible(false);

        if(propDialog_ != null)
            propDialog_.setVisible(false);
        if(superDialog_ != null)
            superDialog_.setVisible(false);
        if(irDialog_ != null)
            irDialog_.setVisible(false);
    }

    protected void
    addProp(Prop p)
    {
        propsModel_.addItem(p);
    }

    protected void
    editProp(Prop p)
    {
        propsModel_.removeItem(p);
        propsModel_.addItem(p);
    }

    protected void
    addType(String type)
    {
        superTypesModel_.addItem(type);
    }


    class Prop implements Sortable
    {
        private PropStruct ps_;

        public
        Prop(PropStruct ps)
        {
            ps_ = ps;
        }

        public int
        compareTo(Sortable other, Object rock)
        {
            Prop otherProp = (Prop)other;
            return ps_.name.compareTo(otherProp.ps_.name);
        }

        public PropStruct
        getPropStruct()
        {
            return ps_;
        }

        public void
        setPropStruct(PropStruct ps)
        {
            ps_ = ps;
        }

        public String
        toString()
        {
            StringBuffer result = new StringBuffer();
            result.append(ps_.name);
            result.append(" - ");

            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            com.ooc.Util.CORBA.AnyWriter out =
                new com.ooc.Util.CORBA.AnyWriter(pw);
            out.printName(ps_.value_type);
            pw.flush();
            result.append(sw.toString());

            switch(ps_.mode.value())
            {
            case PropertyMode._PROP_NORMAL:
                break;
            case PropertyMode._PROP_MANDATORY:
                result.append(", ");
                result.append(AppStandards.getString("MandatoryDescKey"));
                break;
            case PropertyMode._PROP_READONLY:
                result.append(", ");
                result.append(AppStandards.getString("ReadonlyDescKey"));
                break;
            case PropertyMode._PROP_MANDATORY_READONLY:
                result.append(", ");
                result.append(AppStandards.getString("MandatoryDescKey"));
                result.append(", ");
                result.append(AppStandards.getString("ReadonlyDescKey"));
                break;
            }

            return result.toString();
        }
    }


    class PropDialog
        extends JDialog
        implements ActionListener
    {
        private Frame parent_;
        private TraderModel model_;

        private JTextField name_;
        private TypeCodeComboBoxModel typeModel_;
        private JComboBox type_;
        private JCheckBox mandatory_;
        private JCheckBox readonly_;
        private Prop prop_ = null;

        public
        PropDialog(Frame parent, TraderModel model)
        {
            super(parent, AppStandards.getString("PropertyTitleKey"));
            parent_ = parent;
            model_ = model;
            setResizable(true);

            Container content = getContentPane();
            content.setLayout(new GridBagLayout());

            JButton button;

            Constrain.constrain(content,
                AppStandards.createLabel("PropNameKey"),
                0, 0, 1, 1, GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 0.0, 10, 10, 0, 0);
            name_ = AppStandards.createTextField(20);
            Constrain.constrain(content, name_, 0, 1, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
                1.0, 0.0, 0, 10, 0, 10);
            Constrain.constrain(content,
                AppStandards.createLabel("PropTypeKey"),
                0, 2, 1, 1, GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 0.0, 10, 10, 0, 0);
            typeModel_ = new TypeCodeComboBoxModel();
            typeModel_.loadBasicTypes();
            type_ = AppStandards.createComboBox(typeModel_);
            type_.setMaximumRowCount(4);
            Constrain.constrain(content, type_, 0, 3, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
                1.0, 0.0, 0, 10, 0, 10, 2, 5);

            JPanel modePanel = new JPanel();
            modePanel.setBorder(AppStandards.createTitledBorder("PropModeKey"));
            modePanel.setLayout(new GridBagLayout());
            mandatory_ = AppStandards.createCheckBox("MandatoryKey");
            mandatory_.setHorizontalAlignment(mandatory_.LEFT);
            readonly_ = AppStandards.createCheckBox("ReadonlyKey");
            readonly_.setHorizontalAlignment(mandatory_.RIGHT);
            Constrain.constrain(modePanel, mandatory_, 0, 0, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                1.0, 0.0, 0, 5, 5, 0);
            Constrain.constrain(modePanel, readonly_, 1, 0, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST,
                1.0, 0.0, 0, 10, 5, 5);

            Constrain.constrain(content, modePanel, 0, 4, 1, 1,
                GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
                1.0, 1.0, 10, 10, 0, 10);

            JPanel buttonPanel = new JPanel();
            buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
            button = AppStandards.createButton("OkButtonKey");
            button.setActionCommand("ok");
            button.addActionListener(this);
            buttonPanel.add(button);

            button = AppStandards.createButton("CancelButtonKey");
            button.setActionCommand("cancel");
            button.addActionListener(this);
            buttonPanel.add(button);

            Constrain.constrain(content, buttonPanel, 0, 5, 1, 1,
                GridBagConstraints.NONE, GridBagConstraints.EAST,
                0.0, 0.0, 10, 0, 10, 10);

            pack();

            Point p = parent.getLocation();
            setLocation(p.x + 40, p.y + 40);
        }

        public void
        setProperty(Prop p)
        {
            prop_ = p;

            if(p != null)
            {
                PropStruct ps = p.getPropStruct();
                name_.setText(ps.name);
                typeModel_.setSelectedTypeCode(ps.value_type);

                switch(ps.mode.value())
                {
                case PropertyMode._PROP_NORMAL:
                    mandatory_.setSelected(false);
                    readonly_.setSelected(false);
                    break;
                case PropertyMode._PROP_MANDATORY:
                    mandatory_.setSelected(true);
                    readonly_.setSelected(false);
                    break;
                case PropertyMode._PROP_READONLY:
                    mandatory_.setSelected(false);
                    readonly_.setSelected(true);
                    break;
                case PropertyMode._PROP_MANDATORY_READONLY:
                    mandatory_.setSelected(true);
                    readonly_.setSelected(true);
                    break;
                }
            }
            else
            {
                // clear dialog fields
                name_.setText("");
                type_.setSelectedIndex(0);
                mandatory_.setSelected(false);
                readonly_.setSelected(false);
            }
        }

        public void
        actionPerformed(ActionEvent e)
        {
            String command = e.getActionCommand();
            if(command.equals("ok"))
            {
                String name = name_.getText().trim();
                if(name.length() == 0)
                {
                    MessageDialog.showError(this,
                        AppStandards.getString("EmptyNameKey"));
                    return;
                }

                org.omg.CORBA.TypeCode tc = typeModel_.getSelectedTypeCode();

                PropertyMode mode;
                if(mandatory_.isSelected() && readonly_.isSelected())
                    mode = PropertyMode.PROP_MANDATORY_READONLY;
                else if(mandatory_.isSelected())
                    mode = PropertyMode.PROP_MANDATORY;
                else if(readonly_.isSelected())
                    mode = PropertyMode.PROP_READONLY;
                else
                    mode = PropertyMode.PROP_NORMAL;

                PropStruct ps = new PropStruct(name, tc, mode);

                // if a Prop object has been supplied, then we edit it
                if(prop_ != null)
                {
                    prop_.setPropStruct(ps);
                    editProp(prop_);
                }
                else
                    addProp(new Prop(ps));

                this.setVisible(false);
            }
            else if(command.equals("cancel"))
                cancel();
        }

        public void
        cancel()
        {
            this.setVisible(false);
        }
    }



    class SuperDialog extends JDialog implements ActionListener
    {
        private Frame parent_;
        private TraderModel model_;

        private StringListModel listModel_;
        private JList list_;

        public
        SuperDialog(Frame parent, TraderModel model)
        {
            super(parent, AppStandards.getString("SuperTypeTitleKey"));
            parent_ = parent;
            model_ = model;
            setResizable(true);

            Container content = getContentPane();
            content.setLayout(new GridBagLayout());

            JButton button;

            Constrain.constrain(content,
                AppStandards.createLabel("SelectSuperTypeKey"),
                0, 0, 1, 1, GridBagConstraints.NONE,
                GridBagConstraints.NORTHWEST, 0.0, 0.0, 10, 10, 0, 0);
            listModel_ = new StringListModel();
            list_ = AppStandards.createList(listModel_);
            list_.setSelectionMode(
                ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
            JScrollPane scroll = AppStandards.createScrollPane(list_);
            scroll.setPreferredSize(new Dimension(150, 150));
            Constrain.constrain(content, scroll, 0, 1, 1, 1,
                GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
                1.0, 1.0, 0, 10, 0, 10);

            JPanel buttonPanel = new JPanel();
            buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
            button = AppStandards.createButton("OkButtonKey");
            button.setActionCommand("ok");
            button.addActionListener(this);
            buttonPanel.add(button);

            button = AppStandards.createButton("CancelButtonKey");
            button.setActionCommand("cancel");
            button.addActionListener(this);
            buttonPanel.add(button);

            Constrain.constrain(content, buttonPanel, 0, 2, 1, 1,
                GridBagConstraints.NONE, GridBagConstraints.EAST,
                0.0, 0.0, 10, 0, 10, 10);

            pack();

            Point p = parent.getLocation();
            setLocation(p.x + 40, p.y + 40);

            // make sure we update our list of types when the model changes
            model_.addModelListener(
                new TraderModelAdapter()
                {
                    public void
                    typeAdded(String type)
                    {
                        listModel_.addItem(type);
                    }

                    public void
                    typeRemoved(String type)
                    {
                        listModel_.removeItem(type);
                    }

                    public void
                    typesRemoved()
                    {
                        reload();
                    }
                }
            );

            reload();
        }

        public void
        actionPerformed(ActionEvent e)
        {
            String command = e.getActionCommand();
            if(command.equals("ok"))
            {
                Object[] values = list_.getSelectedValues();
                if(values != null)
                    for(int i = 0 ; i < values.length ; i++)
                        addType((String)values[i]);
                this.setVisible(false);
            }
            else if(command.equals("cancel"))
                cancel();
        }

        public void
        cancel()
        {
            this.setVisible(false);
        }

        protected void
        reload()
        {
            StringBuffer message = new StringBuffer();
            String[] types = model_.listTypes(message);
            if(types == null)
                MessageDialog.showError(this, message.toString());
            else
                listModel_.setItems(types);
        }
    }
}
