// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Type;

import com.ooc.CosTradingConsole.Util.*;
import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.io.*;
import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class TypeController
    extends com.ooc.CosTradingConsole.Util.BaseController
{
    private JFrame parentFrame_;
    private TraderModel traderModel_;
    private SortedListModel listModel_;
    private NewTypeDialog newTypeDialog_;
    private boolean loaded_ = false;

    public
    TypeController(JFrame parentFrame, TraderModel traderModel)
    {
        parentFrame_ = parentFrame;
        traderModel_ = traderModel;
        traderModel_.addModelListener(this);
        listModel_ = new SortedListModel();
    }

    public ListModel
    getListModel()
    {
        return listModel_;
    }

    public boolean
    isEditable()
    {
        return false;
    }

    public Object
    getItemType()
    {
        return TYPE_ITEM;
    }

    public void
    clear()
    {
        listModel_.removeAll();
    }

    public synchronized void
    newItem()
    {
        if(newTypeDialog_ == null)
            newTypeDialog_ = new NewTypeDialog(parentFrame_,
                                               traderModel_);
        newTypeDialog_.setValues(null, null);
        newTypeDialog_.setVisible(true);
    }

    public synchronized boolean
    cloneItem(Object item, StringBuffer message)
    {
        String itemName = item.toString();
        TypeStruct ts = (TypeStruct)getItemData(item, message);

        if(ts != null)
        {
            if(newTypeDialog_ == null)
                newTypeDialog_ = new NewTypeDialog(parentFrame_,
                                                   traderModel_);
            newTypeDialog_.setValues(itemName, ts);
            newTypeDialog_.setVisible(true);
        }

        return (ts != null);
    }

    public Object
    findItem(String name)
    {
        ListItem tmp = new ListItem(name);
        return listModel_.findItem(tmp);
    }

    public boolean
    describeItem(Object item, FormattedText text, StringBuffer message)
    {
        boolean result = false;

        TypeStruct ts = (TypeStruct)getItemData(item, message);

        if(ts != null)
        {
            text.addLine(AppStandards.getString("InterfaceDescKey"),
                         FormattedText.HEADER);
            text.addLine(ts.if_name, FormattedText.VALUE);
            text.addBlankLine();

            text.addLine(AppStandards.getString("SuperTypesDescKey"),
                         FormattedText.HEADER);
            if(ts.super_types.length == 0)
                text.addLine(AppStandards.getString("NoneKey"),
                             FormattedText.VALUE);
            else
            {
                StringBuffer sb = new StringBuffer();
                for(int i = 0 ; i < ts.super_types.length ; i++)
                {
                    if(i > 0)
                        sb.append(", ");
                    sb.append(ts.super_types[i]);
                }
                text.addLine(sb.toString(), FormattedText.VALUE);
            }
            text.addBlankLine();

            text.addLine(AppStandards.getString("PropertiesDescKey"),
                         FormattedText.HEADER);
            if(ts.props.length == 0)
                text.addLine(AppStandards.getString("NoneKey"),
                             FormattedText.VALUE);
            else
            {
                for(int i = 0 ; i < ts.props.length ; i++)
                {
                    StringBuffer buff = new StringBuffer();
                    switch(ts.props[i].mode.value())
                    {
                        case PropertyMode._PROP_NORMAL:
                            break;
                        case PropertyMode._PROP_READONLY:
                            buff.append(
                                AppStandards.getString("ReadonlyDescKey"));
                            buff.append(" ");
                            break;
                        case PropertyMode._PROP_MANDATORY:
                            buff.append(
                                AppStandards.getString("MandatoryDescKey"));
                            buff.append(" ");
                            break;
                        case PropertyMode._PROP_MANDATORY_READONLY:
                            buff.append(
                                AppStandards.getString("MandatoryDescKey"));
                            buff.append(" ");
                            buff.append(
                                AppStandards.getString("ReadonlyDescKey"));
                            buff.append(" ");
                            break;
                    }

                    StringWriter sw = new StringWriter();
                    PrintWriter pw = new PrintWriter(sw);
                    com.ooc.Util.CORBA.AnyWriter out =
                        new com.ooc.Util.CORBA.AnyWriter(pw);
                    out.printName(ts.props[i].value_type);
                    pw.flush();
                    buff.append(sw.toString());
                    buff.append(" ");

                    buff.append(ts.props[i].name);
                    text.addLine(buff.toString(), FormattedText.VALUE);
                }
            }
            text.addBlankLine();

            text.addLine(AppStandards.getString("MaskedDescKey"),
                         FormattedText.HEADER);
            if(ts.masked)
                text.addLine(AppStandards.getString("YesKey"),
                             FormattedText.VALUE);
            else
                text.addLine(AppStandards.getString("NoKey"),
                             FormattedText.VALUE);
            text.addBlankLine();

            text.addLine(AppStandards.getString("IncarnationDescKey"),
                         FormattedText.HEADER);
            text.addLine("{" + ts.incarnation.high + ", " +
                         ts.incarnation.low + "}", FormattedText.VALUE);

            result = true;
        }

        return result;
    }

    public Object
    getItemData(Object item, StringBuffer message)
    {
        Object result = null;

        ListItem listItem = (ListItem)item;
        result = listItem.getData(message);

        return result;
    }

    public boolean
    destroyItem(Object item, boolean notify, StringBuffer message)
    {
        ListItem listItem = (ListItem)item;
        return traderModel_.removeType(item.toString(), notify, message);
    }

    public void
    fireItemsDestroyed()
    {
        traderModel_.fireTypesRemoved();
    }

    public boolean
    addItem(String name, Object itemData, boolean notify, StringBuffer message)
    {
        TypeStruct ts = (TypeStruct)itemData;
        return traderModel_.addType(name, ts, notify, message);
    }

    public void
    fireItemsAdded()
    {
    }

    public String
    toString()
    {
        return AppStandards.getString("ViewTypesKey");
    }

    // from TraderModelAdapter
    public synchronized void
    typeAdded(String name)
    {
        if(loaded_)
        {
            ListItem item = new ListItem(name);
            listModel_.addItem(item);
        }
    }

    // from TraderModelAdapter
    public synchronized void
    typeModified(String name)
    {
        if(loaded_)
        {
            ListItem tmp = new ListItem(name);
            ListItem item = (ListItem)listModel_.findItem(tmp);
            if(item != null)
                item.invalidate();
        }
    }

    // from TraderModelAdapter
    public synchronized void
    typeRemoved(String name)
    {
        if(loaded_)
        {
            ListItem tmp = new ListItem(name);
            listModel_.removeItem(tmp);
        }
    }

    // from TraderModelAdapter
    public void
    typesRemoved()
    {
        clear();
        refresh();
    }

    // from TraderModelAdapter
    public void
    repositoryModified()
    {
        clear();
        refresh();
    }

    protected synchronized void
    reload()
    {
        loaded_ = true;

        SpecifiedServiceTypes whichTypes = new SpecifiedServiceTypes();
        whichTypes._default(ListOption.all);

        ServiceTypeRepository repos = traderModel_.getRepos();
        String[] types = repos.list_types(whichTypes);

        ListItem[] items = new ListItem[types.length];
        for(int i = 0 ; i < types.length ; i++)
            items[i] = new ListItem(types[i]);
        listModel_.setItems(items);
    }


    class ListItem implements Sortable
    {
        private String name_;
        private Object data_;

        public
        ListItem(String name)
        {
            name_ = name;
        }

        public int
        compareTo(Sortable other, Object rock)
        {
            ListItem otherItem = (ListItem)other;
            return name_.compareTo(otherItem.name_);
        }

        public void
        invalidate()
        {
            data_ = null;
        }

        public synchronized Object
        getData(StringBuffer message)
        {
            if(data_ == null)
            {
                try
                {
                    ServiceTypeRepository repos = traderModel_.getRepos();
                    data_ = repos.describe_type(name_);
                }
                catch(IllegalServiceType e)
                {
                    message.append(
                        AppStandards.getFormattedString("IllegalTypeNameKey",
                                                        e.type));
                }
                catch(UnknownServiceType e)
                {
                    message.append(
                        AppStandards.getFormattedString("UnknownTypeNameKey",
                                                        e.type));
                }
                catch(org.omg.CORBA.SystemException e)
                {
                    message.append(AppStandards.getString("SystemErrorKey"));
                }
            }

            return data_;
        }

        public String
        toString()
        {
            return name_;
        }
    }
}
