// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import org.omg.CosTrading.*;

public class AddPropertyDialog
    extends JDialog
    implements ActionListener
{
    public interface Listener
    {
        // returns false if property not acceptable (e.g., duplicate name)
        public boolean
        propertyAdded(Prop prop);
    }

    private Frame parent_;
    private Listener listener_;

    private JTextField name_;
    private TypeCodeComboBoxModel typeModel_;
    private JComboBox type_;
    private JTextField value_;

    public
    AddPropertyDialog(Frame parent, Listener listener)
    {
        super(parent, AppStandards.getString("AddPropTitleKey"));
        parent_ = parent;
        listener_ = listener;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Cancel" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    close();
                }
            }
        );

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        JButton button;

        Constrain.constrain(content, AppStandards.createLabel("PropNameKey"),
            0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        name_ = AppStandards.createTextField(25);
        Constrain.constrain(content, name_, 0, 1, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);
        Constrain.constrain(content, AppStandards.createLabel("PropTypeKey"),
            0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        typeModel_ = new TypeCodeComboBoxModel();
        type_ = AppStandards.createComboBox(typeModel_);
        type_.setMaximumRowCount(4);
        Constrain.constrain(content, type_, 0, 3, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);
        Constrain.constrain(content, AppStandards.createLabel("PropValueKey"),
            0, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 10);
        value_ = AppStandards.createTextField(25);
        Constrain.constrain(content, value_, 0, 5, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);

        // insert some glue to take up the extra vertical space on a resize
        Constrain.constrain(content, Box.createGlue(), 0, 6, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 0, 0, 0);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 7, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 10, 0, 10, 10);

        pack();

        // initialize the TypeCode combo box
        typeModel_.loadBasicTypes();
        typeModel_.setSelectedItem(typeModel_.getElementAt(0));

        Point p = parent.getLocation();
        setLocation(p.x + 40, p.y + 40);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            close();
    }

    protected void
    ok()
    {
        StringBuffer message = new StringBuffer();
        org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();

        String name = name_.getText().trim();
        if(name.length() == 0)
        {
            MessageDialog.showError(this,
                                    AppStandards.getString("EmptyNameKey"));
            return;
        }

        org.omg.CORBA.TypeCode type = typeModel_.getSelectedTypeCode();
        org.omg.CORBA.Any value = null;

        String s = value_.getText().trim();
        try
        {
            value = Parse.parseValue(type, s);
        }
        catch(ParseException e)
        {
            MessageDialog.showError(this, e.getMessage());
            return;
        }

        Prop p = new Prop(name, value);
        if(listener_.propertyAdded(p))
            close();
    }

    protected void
    close()
    {
        setVisible(false);
    }
}
