// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.GUI.*;
import java.io.*;
import org.omg.CosTrading.Lookup;
import org.omg.CosTrading.LookupHelper;


public class Connection
{
    private Lookup lookup_;


    public
    Connection()
    {
        this("TradingService");
    }

    public
    Connection(Lookup lookup)
    {
        lookup_ = lookup;
    }

    public
    Connection(File iorfile)
    {
        if(! iorfile.exists())
        {
            System.err.println(
                AppStandards.getFormattedString("FileNotFoundKey",
                                                iorfile.getAbsolutePath()));
            System.exit(1);
        }

        if(! iorfile.isFile())
        {
            System.err.println(
                AppStandards.getFormattedString("NotFileKey",
                                                iorfile.getAbsolutePath()));
            System.exit(1);
        }

        org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();

        try
        {
            FileReader fr = new FileReader(iorfile);
            BufferedReader in = new BufferedReader(fr);
            String ref = in.readLine();
            fr.close();

            org.omg.CORBA.Object obj = orb.string_to_object(ref);
            if(obj == null)
            {
                System.out.println(AppStandards.getString("InvalidObjectKey"));
                System.exit(1);
            }

            lookup_ = LookupHelper.narrow(obj);
        }
        catch(IOException e)
        {
            e.printStackTrace();
            System.exit(1);
        }
    }

    public
    Connection(String service)
    {
        try
        {
            org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();
            org.omg.CORBA.Object obj = orb.resolve_initial_references(service);
            lookup_ = LookupHelper.narrow(obj);
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName e)
        {
            System.err.println(
                AppStandards.getFormattedString("InvalidInitialRefKey",
                                                service));
            System.exit(1);
        }
    }

    public Lookup
    getLookup()
    {
        return lookup_;
    }
}
