// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;
import org.omg.CosTradingDynamic.*;

public class DynamicPropertyDialog
    extends JDialog
    implements ActionListener
{
    public interface Listener
    {
        public void
        dynamicPropertyEdited(Prop prop);
    }

    private Frame parent_;
    private Listener listener_;

    private IORInputPanel iorPanel_;
    private TypeCodeComboBoxModel extraTypeModel_;
    private JComboBox extraType_;
    private JTextField extraValue_;
    private Prop prop_ = null;

    public
    DynamicPropertyDialog(Frame parent, Listener listener)
    {
        super(parent, "");
        parent_ = parent;
        listener_ = listener;
        setResizable(true);

        //
        // Closing the dialog is the same as pressing the "Cancel" button
        //
        addWindowListener(
            new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent evt)
                {
                    close();
                }
            }
        );

        Container content = getContentPane();
        content.setLayout(new GridBagLayout());

        JButton button;

        iorPanel_ = new IORInputPanel(parent_, "IORKey");
        Constrain.constrain(content, iorPanel_, 0, 0, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 10, 10, 0, 10);
        Constrain.constrain(content,
            AppStandards.createLabel("ExtraInfoKey"), 0, 1, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 0);
        extraTypeModel_ = new TypeCodeComboBoxModel();
        extraType_ = AppStandards.createComboBox(extraTypeModel_);
        extraType_.setMaximumRowCount(4);
        Constrain.constrain(content, extraType_, 0, 2, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10, 20, 5);
        Constrain.constrain(content, AppStandards.createLabel("ExtraValueKey"),
            0, 3, 1, 1, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
            0.0, 0.0, 10, 10, 0, 10);
        extraValue_ = AppStandards.createTextField(20);
        Constrain.constrain(content, extraValue_, 0, 4, 1, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
            1.0, 0.0, 0, 10, 0, 10);

        // insert some glue to take up the extra vertical space on a resize
        Constrain.constrain(content, Box.createGlue(), 0, 5, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 0, 0, 0);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        button = AppStandards.createButton("OkButtonKey");
        button.setActionCommand("ok");
        button.addActionListener(this);
        buttonPanel.add(button);

        button = AppStandards.createButton("CancelButtonKey");
        button.setActionCommand("cancel");
        button.addActionListener(this);
        buttonPanel.add(button);

        Constrain.constrain(content, buttonPanel, 0, 6, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.EAST,
            0.0, 0.0, 10, 0, 10, 10);

        pack();

        // initialize the TypeCode combo box - put "None" as the
        // first item, then add the basic types
        org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init();
        org.omg.CORBA.TypeCode tc =
            orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_null);
        extraTypeModel_.addItem(tc, AppStandards.getString("NoneKey"));
        extraTypeModel_.loadBasicTypes();
        extraTypeModel_.setSelectedTypeCode(tc);

        Point p = parent.getLocation();
        setLocation(p.x + 40, p.y + 40);
    }

    public void
    setProperty(Prop p)
    {
        prop_ = p;

        setTitle(
            AppStandards.getFormattedString("DynPropTitleKey", p.getName()));

        if(p.isDynamic())
        {
            DynamicProp dynProp = p.getDynamicProp();
            org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();
            String ref = orb.object_to_string(dynProp.eval_if);
            iorPanel_.setIOR(ref);
            org.omg.CORBA.TypeCode tc = dynProp.extra_info.type();
            if(tc.kind() == org.omg.CORBA.TCKind.tk_null ||
                    tc.kind() == org.omg.CORBA.TCKind.tk_void)
            {
                extraType_.setSelectedIndex(0);
                extraValue_.setText("");
            }
            else
            {
                extraTypeModel_.setSelectedTypeCode(tc);
                extraValue_.setText(
                    PropUtil.getValueAsString(dynProp.extra_info));
            }
        }
        else
        {
            iorPanel_.reset();
            extraType_.setSelectedIndex(0);
            extraValue_.setText("");
        }
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("ok"))
            ok();
        else if(command.equals("cancel"))
            close();
    }

    protected void
    ok()
    {
        StringBuffer message = new StringBuffer();
        org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();

        org.omg.CosTradingDynamic.DynamicPropEval eval = null;
        try
        {
            org.omg.CORBA.Object obj = iorPanel_.getObject(message);
            if(obj == null)
            {
                MessageDialog.showError(this, message.toString());
                return;
            }

            eval = org.omg.CosTradingDynamic.DynamicPropEvalHelper.narrow(obj);
        }
        catch(org.omg.CORBA.SystemException e)
        {
        }

        if(eval == null)
        {
            MessageDialog.showError(this,
                AppStandards.getString("InvalidObjectKey"));
            return;
        }

        org.omg.CORBA.TypeCode extraType =
            extraTypeModel_.getSelectedTypeCode();
        org.omg.CORBA.Any extraValue = null;

        if(extraType.kind() == org.omg.CORBA.TCKind.tk_null)
        {
            extraValue = orb.create_any();
            extraValue.type(extraType);
        }
        else
        {
            String s = extraValue_.getText().trim();
            try
            {
                extraValue = Parse.parseValue(extraType, s);
            }
            catch(ParseException e)
            {
                MessageDialog.showError(this, e.getMessage());
                return;
            }
        }

        org.omg.CosTradingDynamic.DynamicProp dp =
            new org.omg.CosTradingDynamic.DynamicProp();
        dp.eval_if = eval;
        dp.returned_type = prop_.getTypeCode();
        dp.extra_info = extraValue;

        org.omg.CORBA.Any any = orb.create_any();
        org.omg.CosTradingDynamic.DynamicPropHelper.insert(any, dp);

        prop_.setValue(any);
        listener_.dynamicPropertyEdited(prop_);

        close();
    }

    protected void
    close()
    {
        iorPanel_.close();
        setVisible(false);
    }
}
