// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import java.util.*;

public class Parse
{
    private
    Parse()
    {
    }

    public static org.omg.CORBA.Any
    parseValue(org.omg.CORBA.TypeCode tc, String value)
        throws ParseException
    {
        org.omg.CORBA.Any result = null;

        org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();

        try
        {
            // skip past any aliasing
            org.omg.CORBA.TypeCode type = tc;
            while(type.kind() == org.omg.CORBA.TCKind.tk_alias)
                type = type.content_type();

            switch(type.kind().value())
            {
                case org.omg.CORBA.TCKind._tk_boolean:
                case org.omg.CORBA.TCKind._tk_char:
                case org.omg.CORBA.TCKind._tk_double:
                case org.omg.CORBA.TCKind._tk_float:
                case org.omg.CORBA.TCKind._tk_long:
                case org.omg.CORBA.TCKind._tk_octet:
                case org.omg.CORBA.TCKind._tk_short:
                case org.omg.CORBA.TCKind._tk_string:
                case org.omg.CORBA.TCKind._tk_ulong:
                case org.omg.CORBA.TCKind._tk_ushort:
                {
                    org.omg.CORBA.DynAny dynAny =
                        orb.create_basic_dyn_any(tc);
                    parseBasicValue(dynAny, type.kind(), value);
                    result = dynAny.to_any();
                    break;
                }

                case org.omg.CORBA.TCKind._tk_sequence:
                {
                    org.omg.CORBA.TypeCode contentType = type.content_type();
                    String[] tokens = tokenize(value);
                    org.omg.CORBA.DynSequence dynSeq =
                        orb.create_dyn_sequence(tc);
                    dynSeq.length(tokens.length);
                    for(int i = 0 ; i < tokens.length ; i++)
                        parseBasicValue(dynSeq, contentType.kind(), tokens[i]);
                    result = dynSeq.to_any();
                    break;
                }
            }
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind e)
        {
            e.printStackTrace();
        }
        catch(org.omg.CORBA.DynAnyPackage.Invalid e)
        {
            e.printStackTrace();
        }
        catch(org.omg.CORBA.ORBPackage.InconsistentTypeCode e)
        {
            e.printStackTrace();
        }

        return result;
    }

    public static void
    parseBasicValue(
        org.omg.CORBA.DynAny dynAny,
        org.omg.CORBA.TCKind kind,
        String value)
        throws ParseException
    {
        try
        {
            switch(kind.value())
            {
                case org.omg.CORBA.TCKind._tk_boolean:
                {
                    boolean b = parseBoolean(value);
                    dynAny.insert_boolean(b);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_char:
                {
                    char c = parseChar(value);
                    dynAny.insert_char(c);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_double:
                {
                    double d = parseDouble(value);
                    dynAny.insert_double(d);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_float:
                {
                    float f = parseFloat(value);
                    dynAny.insert_float(f);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_long:
                {
                    int i = parseLong(value);
                    dynAny.insert_long(i);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_octet:
                {
                    byte b = parseOctet(value);
                    dynAny.insert_octet(b);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_short:
                {
                    short s = parseShort(value);
                    dynAny.insert_short(s);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_string:
                {
                    dynAny.insert_string(value);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_ulong:
                {
                    int i = parseULong(value);
                    dynAny.insert_ulong(i);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_ushort:
                {
                    short s = parseUShort(value);
                    dynAny.insert_ushort(s);
                    break;
                }
            }
        }
        catch(org.omg.CORBA.DynAnyPackage.InvalidValue e)
        {
            e.printStackTrace();
        }
    }

    private static boolean
    parseBoolean(String value)
        throws ParseException
    {
        boolean result;

        if(value.equalsIgnoreCase("true") ||
           value.equalsIgnoreCase("t") ||
           value.equalsIgnoreCase("yes") ||
           value.equalsIgnoreCase("y"))
            result = true;
        else if(value.equalsIgnoreCase("false") ||
                value.equalsIgnoreCase("f") ||
                value.equalsIgnoreCase("no") ||
                value.equalsIgnoreCase("n"))
            result = false;
        else
            throw new ParseException(
                AppStandards.getFormattedString("InvalidBooleanKey", value));

        return result;
    }

    private static short
    parseShort(String value)
        throws ParseException
    {
        short result = 0;

        String s = value.trim();
        try
        {
            if(s.length() > 0)
            {
                checkNumeric(s, false);
                int i = Integer.valueOf(s).intValue();
                if(i < Short.MIN_VALUE || i > Short.MAX_VALUE)
                    throw new ParseException(
                        AppStandards.getFormattedString("ShortRangeKey",
                                                        value));
                else
                    result = (short)i;
            }
            else
                throw new ParseException(AppStandards.getString("NoValueKey"));
        }
        catch(NumberFormatException e)
        {
            throw new ParseException(
                AppStandards.getFormattedString("InvalidShortKey", value));
        }

        return result;
    }

    private static short
    parseUShort(String value)
        throws ParseException
    {
        short result = 0;

        String s = value.trim();

        try
        {
            if(s.length() > 0)
            {
                checkNumeric(s, false);
                int i = Integer.valueOf(s).intValue();
                if(i < 0 || i > (Short.MAX_VALUE - Short.MIN_VALUE))
                    throw new ParseException(
                        AppStandards.getFormattedString("UShortRangeKey",
                                                        value));
                else
                    result = (short)i;
            }
            else
                throw new ParseException(AppStandards.getString("NoValueKey"));
        }
        catch(NumberFormatException e)
        {
            throw new ParseException(
                AppStandards.getFormattedString("InvalidUShortKey", value));
        }

        return result;
    }

    private static int
    parseLong(String value)
        throws ParseException
    {
        int result = 0;

        String s = value.trim();

        try
        {
            if(s.length() > 0)
            {
                checkNumeric(s, false);
                long l = Long.valueOf(s).longValue();
                if(l < Integer.MIN_VALUE || l > Integer.MAX_VALUE)
                    throw new ParseException(
                        AppStandards.getFormattedString("LongRangeKey",
                                                        value));
                else
                    result = (int)l;
            }
            else
                throw new ParseException(AppStandards.getString("NoValueKey"));
        }
        catch(NumberFormatException e)
        {
            throw new ParseException(
                AppStandards.getFormattedString("InvalidLongKey", value));
        }

        return result;
    }

    private static int
    parseULong(String value)
        throws ParseException
    {
        int result = 0;

        String s = value.trim();

        try
        {
            if(s.length() > 0)
            {
                checkNumeric(s, false);
                long l = Long.valueOf(s).longValue();
                if(l < 0 || l > (Integer.MAX_VALUE - Integer.MIN_VALUE))
                    throw new ParseException(
                        AppStandards.getFormattedString("ULongRangeKey",
                                                        value));
                else
                    result = (int)l;
            }
            else
                throw new ParseException(AppStandards.getString("NoValueKey"));
        }
        catch(NumberFormatException e)
        {
            throw new ParseException(
                AppStandards.getFormattedString("InvalidULongKey", value));
        }

        return result;
    }

    private static float
    parseFloat(String value)
        throws ParseException
    {
        float result = 0;

        String s = value.trim();

        try
        {
            checkNumeric(s, true);
            if(s.length() > 0)
                result = Float.valueOf(s).floatValue();
            else
                throw new ParseException(AppStandards.getString("NoValueKey"));
        }
        catch(NumberFormatException e)
        {
            throw new ParseException(
                AppStandards.getFormattedString("InvalidFloatKey", value));
        }

        return result;
    }

    private static double
    parseDouble(String value)
        throws ParseException
    {
        double result = 0;

        String s = value.trim();

        try
        {
            checkNumeric(s, true);
            if(s.length() > 0)
                result = Double.valueOf(s).doubleValue();
            else
                throw new ParseException(AppStandards.getString("NoValueKey"));
        }
        catch(NumberFormatException e)
        {
            throw new ParseException(
                AppStandards.getFormattedString("InvalidDoubleKey", value));
        }

        return result;
    }

    private static char
    parseChar(String value)
        throws ParseException
    {
        char result = 0;

        String s = value.trim();

        if(s.length() > 0)
            result = s.charAt(0);
        else
            throw new ParseException(AppStandards.getString("NoValueKey"));

        return result;
    }

    private static byte
    parseOctet(String value)
        throws ParseException
    {
        byte result = 0;

        String s = value.trim();

        try
        {
            if(s.length() > 0)
            {
                checkNumeric(s, false);
                int i = Integer.valueOf(s).intValue();
                if(i < 0 || i > 255)
                    throw new ParseException(
                        AppStandards.getFormattedString("OctetRangeKey",
                                                        value));
                else
                    result = (byte)i;
            }
            else
                throw new ParseException(AppStandards.getString("NoValueKey"));
        }
        catch(NumberFormatException e)
        {
            throw new ParseException(
                AppStandards.getFormattedString("InvalidOctetKey", value));
        }

        return result;
    }

    private static void
    checkNumeric(String value, boolean real)
        throws ParseException
    {
        for(int i = 0 ; i < value.length() ; i++)
        {
            boolean ok = false;
            char ch = value.charAt(i);

            if(Character.isDigit(ch) || ch == '-')
                ok = true;
            else if(real)
            {
                if(ch == '.' || ch == 'e' || ch == 'E' ||
                   ch == '-' || ch == '+')
                    ok = true;
            }

            if(! ok)
                throw new ParseException(
                    AppStandards.getFormattedString("InvalidNumberKey",
                                                    value));
        }
    }

    // tokenize a string containing a sequence of one or more values;
    // values are separated by ",", and quoted strings are supported
    private static String[]
    tokenize(String value)
        throws ParseException
    {
        String[] result;

        Vector vec = new Vector();

        int pos = 0;
        int len = value.length();
        StringBuffer word = new StringBuffer();
        boolean inQuote = false;

        while(pos < len)
        {
            char ch = value.charAt(pos);
            if(ch == ',' && ! inQuote)
            {
                vec.addElement(word.toString());
                word.setLength(0);
            }
            else if(ch == '"')
                inQuote = (! inQuote);
            else
                word.append(ch);

            pos++;
        }

        if(inQuote)
            throw new ParseException(
                AppStandards.getString("MissingQuoteKey"));

        vec.addElement(word.toString());

        result = new String[vec.size()];
        vec.copyInto(result);

        return result;
    }
}
