// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import java.io.*;
import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;
import org.omg.CosTradingDynamic.*;

// Represents an offer property
public class Prop implements Sortable
{
    private PropStruct ps_;
    private org.omg.CORBA.Any value_;

    public
    Prop(PropStruct ps)
    {
        this(ps, null);
    }

    public
    Prop(PropStruct ps, org.omg.CORBA.Any value)
    {
        ps_ = ps;
        value_ = value;
    }

    public
    Prop(String name, org.omg.CORBA.Any value)
    {
        ps_ = new PropStruct();
        ps_.name = name;
        ps_.value_type = value.type();
        ps_.mode = PropertyMode.PROP_NORMAL;
        value_ = value;
    }

    // from Sortable
    public int
    compareTo(Sortable other, Object rock)
    {
        Prop otherProp = (Prop)other;
        return ps_.name.compareTo(otherProp.ps_.name);
    }

    public PropStruct
    getPropStruct()
    {
        return ps_;
    }

    public String
    getName()
    {
        return ps_.name;
    }

    public org.omg.CORBA.TypeCode
    getTypeCode()
    {
        return ps_.value_type;
    }

    public org.omg.CORBA.Any
    getValue()
    {
        return value_;
    }

    public String
    getValueAsString()
    {
        String result = null;

        if(value_ != null)
            result = PropUtil.getValueAsString(value_);

        return result;
    }

    public boolean
    isDynamic()
    {
        boolean result = false;

        if(value_ != null)
            result = PropUtil.isDynamic(value_);

        return result;
    }

    public DynamicProp
    getDynamicProp()
    {
        DynamicProp result = null;

        if(value_ != null)
            result = DynamicPropHelper.extract(value_);

        return result;
    }

    public boolean
    isMandatory()
    {
        return PropUtil.isMandatory(ps_.mode);
    }

    public boolean
    isReadonly()
    {
        return PropUtil.isReadonly(ps_.mode);
    }

    public void
    setValue(org.omg.CORBA.Any value)
    {
        value_ = value;
    }

    public String
    toString()
    {
        StringBuffer result = new StringBuffer();
        result.append(ps_.name);
        result.append(" - ");

        StringWriter sw;
        PrintWriter pw;
        com.ooc.Util.CORBA.AnyWriter out;

        sw = new StringWriter();
        pw = new PrintWriter(sw);
        out = new com.ooc.Util.CORBA.AnyWriter(pw);
        out.printName(ps_.value_type);
        pw.flush();
        result.append(sw.toString());

        switch(ps_.mode.value())
        {
            case PropertyMode._PROP_NORMAL:
                break;
            case PropertyMode._PROP_MANDATORY:
                result.append(", ");
                result.append(AppStandards.getString("MandatoryDescKey"));
                break;
            case PropertyMode._PROP_READONLY:
                result.append(", ");
                result.append(AppStandards.getString("ReadonlyDescKey"));
                break;
            case PropertyMode._PROP_MANDATORY_READONLY:
                result.append(", ");
                result.append(AppStandards.getString("MandatoryDescKey"));
                result.append(", ");
                result.append(AppStandards.getString("ReadonlyDescKey"));
                break;
        }

        result.append(" = ");

        if(value_ == null)
            result.append(AppStandards.getString("NullValueKey"));
        else
        {
            if(isDynamic())
                result.append(AppStandards.getString("DynamicValueKey"));
            else
                result.append(getValueAsString());
        }

        return result.toString();
    }
}
