// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import java.io.*;
import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;
import org.omg.CosTradingDynamic.*;

public class PropUtil
{
    private
    PropUtil()
    {
    }

    public static boolean
    isDynamic(org.omg.CORBA.Any value)
    {
        org.omg.CORBA.TypeCode tc = value.type();
        return tc.equal(DynamicPropHelper.type());
    }

    public static boolean
    isMandatory(PropertyMode mode)
    {
        return (mode == PropertyMode.PROP_MANDATORY ||
                mode == PropertyMode.PROP_MANDATORY_READONLY);
    }

    public static boolean
    isReadonly(PropertyMode mode)
    {
        return (mode == PropertyMode.PROP_READONLY ||
                mode == PropertyMode.PROP_MANDATORY_READONLY);
    }

    public static String
    getValueAsString(org.omg.CORBA.Any any)
    {
        String result;

        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        com.ooc.Util.CORBA.AnyWriter out =
            new com.ooc.Util.CORBA.AnyWriter(pw);
        org.omg.CORBA.DynAny dynAny =
            com.ooc.Util.CORBA.ORBManager.ORB().create_dyn_any(any);
        out.printDynAny(dynAny, false);
        pw.flush();
        result = sw.toString();

        return result;
    }

    public static void
    describe(Property prop, FormattedText text)
    {
        StringWriter sw;
        PrintWriter pw;
        com.ooc.Util.CORBA.AnyWriter out;

        // format: "<proptype> <propname> = <propvalue>"
        StringBuffer buff = new StringBuffer();
        sw = new StringWriter();
        pw = new PrintWriter(sw);
        out = new com.ooc.Util.CORBA.AnyWriter(pw);
        out.printName(prop.value.type());
        buff.append(sw.toString());
        buff.append(" ");
        buff.append(prop.name);
        buff.append(" = ");

        org.omg.CORBA.ORB orb = com.ooc.Util.CORBA.ORBManager.ORB();

        if(isDynamic(prop.value))
        {
            //buff.append(AppStandards.getString("DynamicValueKey"));
            buff.append("{");
            text.addLine(buff.toString(), FormattedText.VALUE);

            DynamicProp dp = DynamicPropHelper.extract(prop.value);
            buff.setLength(0);
            buff.append("eval_if = ");
            String evalIOR = null;
            if(dp.eval_if == null)
                evalIOR = AppStandards.getString("NilKey");
            else
            {
                evalIOR = AppStandards.getString("InvalidKey");
                try
                {
                    evalIOR = orb.object_to_string(dp.eval_if);
                }
                catch(org.omg.CORBA.SystemException e)
                {
                }
            }
            buff.append(evalIOR);
            text.addLine(buff.toString(), FormattedText.SUBVALUE);

            buff.setLength(0);
            buff.append("returned_type = ");
            sw = new StringWriter();
            pw = new PrintWriter(sw);
            out = new com.ooc.Util.CORBA.AnyWriter(pw);
            out.printName(dp.returned_type);
            pw.flush();
            buff.append(sw.toString());
            text.addLine(buff.toString(), FormattedText.SUBVALUE);

            buff.setLength(0);
            buff.append("extra_info = ");
            sw = new StringWriter();
            pw = new PrintWriter(sw);
            out = new com.ooc.Util.CORBA.AnyWriter(pw);
            pw.print("(");
            out.printName(dp.extra_info.type());
            pw.print(") ");
            org.omg.CORBA.DynAny dynAny = orb.create_dyn_any(dp.extra_info);
            out.printDynAny(dynAny, false);
            pw.flush();
            buff.append(sw.toString());
            text.addLine(buff.toString(), FormattedText.SUBVALUE);

            text.addLine("}", FormattedText.VALUE);
        }
        else
        {
            org.omg.CORBA.DynAny dynAny = orb.create_dyn_any(prop.value);
            sw = new StringWriter();
            pw = new PrintWriter(sw);
            out = new com.ooc.Util.CORBA.AnyWriter(pw);
            out.printDynAny(dynAny, false);
            pw.flush();
            buff.append(sw.toString());
            text.addLine(buff.toString(), FormattedText.VALUE);
        }
    }
}
