// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;


public class PropertyTableModel extends AbstractTableModel
{
    private java.util.Vector rows_ = new java.util.Vector();
    private boolean edit_;
    private String[] columns_;

    public
    PropertyTableModel(boolean edit)
    {
        edit_ = edit;

        columns_ = new String[3];
        columns_[0] = "";
        columns_[1] = AppStandards.getString("PropertyKey");
        columns_[2] = AppStandards.getString("ValueKey");
    }

    public org.omg.CORBA.Any
    parsePropertyValue(int index)
        throws ParseException
    {
        PropertyTableRow row = (PropertyTableRow)rows_.elementAt(index);
        return row.parsePropertyValue();
    }

    public void
    setProperties(Prop[] props)
    {
        PropertyTableRow[] rows = new PropertyTableRow[props.length];
        for(int i = 0 ; i < props.length ; i++)
            rows[i] = new PropertyTableRow(props[i], edit_);

        QuickSort.sort(rows, null);

        rows_ = new java.util.Vector(rows.length);
        for(int i = 0 ; i < rows.length ; i++)
            rows_.addElement(rows[i]);

        fireTableChanged(new TableModelEvent(this, -1));
    }

    public void
    clear()
    {
        rows_ = new java.util.Vector();
        fireTableChanged(new TableModelEvent(this, -1));
    }

    public Prop
    getProperty(int index)
    {
        Prop result = null;

        if(index >= 0 && index <= rows_.size())
        {
            PropertyTableRow row = (PropertyTableRow)rows_.elementAt(index);
            result = row.getProperty();
        }

        return result;
    }

    public int
    findProperty(Prop prop)
    {
        for(int i = 0 ; i < rows_.size() ; i++)
        {
            PropertyTableRow row = (PropertyTableRow)rows_.elementAt(i);
            if(prop.getName().equals(row.getProperty().getName()))
                return i;
        }

        return -1;
    }

    public void
    resetProperty(int index)
    {
        if(isCellEditable(index, 2))
        {
            PropertyTableRow row = (PropertyTableRow)rows_.elementAt(index);
            row.reset();
            fireTableChanged(new TableModelEvent(this, index));
        }
    }

    public void
    propertyChanged(int index)
    {
        PropertyTableRow row = (PropertyTableRow)rows_.elementAt(index);
        row.propertyChanged();
        fireTableChanged(new TableModelEvent(this, index));
    }

    public void
    addProperty(Prop prop)
    {
        PropertyTableRow row = new PropertyTableRow(prop, edit_);
        PropertyTableRow[] rows = new PropertyTableRow[rows_.size()];
        rows_.copyInto(rows);
        int pos = BinarySearch.getInsertPosition(rows, row, null);
        rows_.insertElementAt(row, pos);
        fireTableRowsInserted(pos, pos);
    }

    public boolean
    isPropertyIncluded(int index)
    {
        PropertyTableRow row = (PropertyTableRow)rows_.elementAt(index);
        return row.getInclude();
    }

    public int
    getColumnCount()
    {
        return columns_.length;
    }

    public int
    getRowCount()
    {
        return rows_.size();
    }

    public Object
    getValueAt(int row, int col)
    {
        Object result = null;

        PropertyTableRow r = (PropertyTableRow)rows_.elementAt(row);

        if(col == 0)
            result = new Boolean(r.getInclude());
        else if(col == 1)
            result = r.getProperty().getName();
        else
            result = r.getValue();

        return result;
    }

    public String
    getColumnName(int column)
    {
        return columns_[column];
    }

    public Class
    getColumnClass(int col)
    {
        return (col == 0) ? Boolean.class : super.getColumnClass(col);
    }

    public boolean
    isCellEditable(int row, int col)
    {
        boolean result;

        PropertyTableRow r = (PropertyTableRow)rows_.elementAt(row);
        Prop p = r.getProperty();

        if(col == 2)
        {
            if(p.isDynamic())
                result = false;
            else if(edit_ && p.isReadonly() && p.getValue() != null)
                result = false;
            else
                result = true;
        }
        else if(col == 0)
        {
            if(p.isMandatory())
                result = false;
            else
                result = true;
        }
        else
            result = false;

        return result;
    }

    public void
    setValueAt(Object aValue, int row, int column)
    {
        PropertyTableRow r = (PropertyTableRow)rows_.elementAt(row);
        Prop p = r.getProperty();

        if(column == 0)
        {
            r.setInclude(((Boolean)aValue).booleanValue());
            fireTableChanged(new TableModelEvent(this, row, column));
        }
        else if(column == 2)
        {
            if(! p.isDynamic())
                r.setValue((String)aValue);
        }
    }
}
