// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import com.ooc.Util.CORBA.GUI.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;
import org.omg.CosTradingDynamic.*;

public class PropertyTablePanel
    extends JPanel
    implements ActionListener,
               AddPropertyDialog.Listener,
               DynamicPropertyDialog.Listener
{
    private Frame parent_;
    private PropertyTable props_;
    private PropertyTableModel propsModel_;
    private AddPropertyDialog addPropDialog_;
    private DynamicPropertyDialog dynPropDialog_;

    public
    PropertyTablePanel(Frame parent, boolean edit)
    {
        parent_ = parent;

        setLayout(new GridBagLayout());
        setBorder(AppStandards.createTitledBorder("PropertiesKey"));

        propsModel_ = new PropertyTableModel(edit);
        props_ = new PropertyTable(propsModel_);
        JScrollPane propScroll = new JScrollPane(props_);
        propScroll.setPreferredSize(new Dimension(200, 100));
        Constrain.constrain(this, propScroll, 0, 0, 1, 1,
            GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
            1.0, 1.0, 0, 5, 5, 0);
        JPanel propButtons = new JPanel();
        JButton button;
        propButtons.setLayout(new GridLayout(3, 1, 5, 5));
        button = AppStandards.createButton("AddButtonKey");
        button.setActionCommand("add");
        button.addActionListener(this);
        propButtons.add(button);
        button = AppStandards.createButton("DynamicButtonKey");
        button.setActionCommand("dynamic");
        button.addActionListener(this);
        propButtons.add(button);
        button = AppStandards.createButton("ResetButtonKey");
        button.setActionCommand("reset");
        button.addActionListener(this);
        propButtons.add(button);
        Constrain.constrain(this, propButtons, 1, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.NORTHEAST,
            0.0, 0.0, 0, 10, 5, 5);
    }

    public void
    actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("add"))
            addProperty();
        else if(command.equals("dynamic"))
            editProperty();
        else if(command.equals("reset"))
        {
            int[] rows = props_.getSelectedRows();
            for(int i = 0 ; i < rows.length ; i++)
                propsModel_.resetProperty(rows[i]);
        }
    }

    public void
    clear()
    {
        propsModel_.clear();
    }

    public void
    setProperties(Prop[] props)
    {
        propsModel_.setProperties(props);
    }

    public Prop[]
    getProperties()
    {
        Prop[] result = null;

        try
        {
            // is table is currently being edited, then stop the editing
            if(props_.isEditing())
                props_.getCellEditor().stopCellEditing();

            // collect offer properties
            Vector vec = new Vector();
            for(int i = 0 ; i < propsModel_.getRowCount() ; i++)
            {
                if(propsModel_.isPropertyIncluded(i))
                {
                    Prop p = propsModel_.getProperty(i);
                    if(p.isDynamic())
                        vec.addElement(new Prop(p.getPropStruct(),
                                                p.getValue()));
                    else
                    {
                        org.omg.CORBA.Any value =
                            propsModel_.parsePropertyValue(i);
                        vec.addElement(new Prop(p.getPropStruct(), value));
                    }
                }
            }

            result = new Prop[vec.size()];
            vec.copyInto(result);
        }
        catch(ParseException e)
        {
            MessageDialog.showError(parent_, e.getMessage());
        }

        return result;
    }

    public void
    close()
    {
        if(props_.isEditing())
            props_.getCellEditor().cancelCellEditing();
        if(dynPropDialog_ != null && dynPropDialog_.isVisible())
            dynPropDialog_.setVisible(false);
    }

    // from AddPropertyDialog.Listener
    public boolean
    propertyAdded(Prop prop)
    {
        int index = propsModel_.findProperty(prop);
        if(index >= 0)
        {
            String msg =
                AppStandards.getFormattedString("DuplicatePropNameKey",
                                                prop.getName());
            MessageDialog.showError(parent_, msg);
            return false;
        }

        propsModel_.addProperty(prop);
        return true;
    }

    protected synchronized void
    addProperty()
    {
        if(addPropDialog_ == null)
            addPropDialog_ = new AddPropertyDialog(parent_, this);
        addPropDialog_.setVisible(true);
    }

    // from DynamicPropertyDialog.Listener
    public void
    dynamicPropertyEdited(Prop prop)
    {
        int index = propsModel_.findProperty(prop);
        propsModel_.propertyChanged(index);
        if(props_.isEditing() && props_.getEditingRow() == index)
            props_.getCellEditor().cancelCellEditing();
        props_.setRowSelectionInterval(index, index);
    }

    protected synchronized void
    editProperty()
    {
        int index = props_.getSelectedRow();
        if(index >= 0)
        {
            Prop p = propsModel_.getProperty(index);
            if(dynPropDialog_ == null)
                dynPropDialog_ = new DynamicPropertyDialog(parent_, this);
            dynPropDialog_.setProperty(p);
            dynPropDialog_.setVisible(true);
        }
    }
}
