// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import javax.swing.*;
import javax.swing.table.*;


public class PropertyTableRow implements Sortable
{
    private Prop prop_;
    private String value_;
    private boolean include_;

    public
    PropertyTableRow(Prop prop, boolean edit)
    {
        prop_ = prop;

        if(edit && prop_.getValue() == null)
            include_ = false;
        else
            include_ = true;

        initPropertyValue();
    }

    public int
    compareTo(Sortable other, Object rock)
    {
        PropertyTableRow otherRow = (PropertyTableRow)other;
        return prop_.compareTo(otherRow.prop_, rock);
    }

    public org.omg.CORBA.Any
    parsePropertyValue()
        throws ParseException
    {
        org.omg.CORBA.Any result = null;

        try
        {
            result = Parse.parseValue(prop_.getTypeCode(), value_);
            prop_.setValue(result);
        }
        catch(ParseException e)
        {
            throw new ParseException(prop_.getName() + ": " + e.getMessage());
        }

        return result;
    }

    public boolean
    getInclude()
    {
        return include_;
    }

    public void
    setInclude(boolean inc)
    {
        include_ = inc;
    }

    public Prop
    getProperty()
    {
        return prop_;
    }

    public String
    getValue()
    {
        return value_;
    }

    public void
    setValue(String value)
    {
        value_ = value;
    }

    public void
    reset()
    {
        prop_.setValue(null);
        propertyChanged();
    }

    public void
    propertyChanged()
    {
        initPropertyValue();
    }

    protected void
    initPropertyValue()
    {
        if(prop_.isDynamic())
            value_ = AppStandards.getString("DynamicValueKey");
        else
        {
            if(prop_.getValue() == null)
                value_ = "";
            else
                value_ = prop_.getValueAsString();
        }
    }
}
