// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import javax.swing.AbstractListModel;
import javax.swing.text.*;

// StringListModel is a simple Swing list model that manages a sorted
// array of strings.
public class StringListModel extends AbstractListModel
{
    private String[] items_;

    public
    StringListModel()
    {
        items_ = new String[0];
    }

    public
    StringListModel(String[] items)
    {
        items_ = items;
        QuickSort.sort(items_);
    }

    public synchronized int
    getSize()
    {
        return items_.length;
    }

    public synchronized Object
    getElementAt(int index)
    {
        return items_[index];
    }

    public synchronized String[]
    getItems()
    {
        return items_;
    }

    public void
    setItems(String[] items)
    {
        if(items_.length > 0)
            fireIntervalRemoved(this, 0, items_.length - 1);

        int length = -1;

        //
        // Don't want to be synchronized when we fire an event
        //
        synchronized(this)
        {
            items_ = items;
            QuickSort.sort(items_);
            length = items_.length;
        }

        if(length > 0)
            fireIntervalAdded(this, 0, length - 1);
    }

    public void
    removeAll()
    {
        if(items_.length > 0)
            fireIntervalRemoved(this, 0, items_.length - 1);
        items_ = new String[0];
    }

    public void
    addItem(String item)
    {
        int pos = 0;

        //
        // Don't want to be synchronized when we fire an event
        //
        synchronized(this)
        {
            pos = BinarySearch.getInsertPosition(items_, item);
            String[] arr = new String[items_.length + 1];
            System.arraycopy(items_, 0, arr, 0, pos);
            arr[pos] = item;
            System.arraycopy(items_, pos, arr, pos + 1, items_.length - pos);
            items_ = arr;
        }

        fireIntervalAdded(this, pos, pos);
    }

    public void
    removeItem(String item)
    {
        int pos = -1;

        //
        // Don't want to be synchronized when we fire an event
        //
        synchronized(this)
        {
            pos = BinarySearch.find(items_, item);
            if(pos >= 0)
            {
                String[] arr = new String[items_.length - 1];
                System.arraycopy(items_, 0, arr, 0, pos);
                System.arraycopy(items_, pos + 1, arr, pos,
                                 items_.length - (pos + 1));
                items_ = arr;
            }
        }

        if(pos >= 0)
            fireIntervalRemoved(this, pos, pos);
    }
}
