// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import java.util.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.LinkPackage.*;
import org.omg.CosTrading.RegisterPackage.*;
import org.omg.CosTrading.ProxyPackage.*;
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class TraderModel
{
    private Status status_;
    private Vector listeners_ = new Vector();
    private Connection connection_;
    private ServiceTypeRepository repos_;
    private Lookup lookup_;
    private Link link_;
    private Register register_;
    private Proxy proxy_;
    private Admin admin_;

    public
    TraderModel(Status status)
    {
        status_ = status;
    }

    public Status
    getStatus()
    {
        return status_;
    }

    public void
    addModelListener(TraderModelListener l)
    {
        listeners_.addElement(l);
    }

    public void
    removeModelListener(TraderModelListener l)
    {
        listeners_.removeElement(l);
    }

    public void
    removeAllModelListeners()
    {
        listeners_ = new Vector();
    }

    public synchronized void
    setConnection(Connection connection)
    {
        connection_ = connection;
        getTraderComponents();
    }

    public synchronized Connection
    getConnection()
    {
        return connection_;
    }

    public synchronized ServiceTypeRepository
    getRepos()
    {
        return repos_;
    }

    public synchronized void
    setRepos(ServiceTypeRepository repos)
    {
        repos_ = repos;
        fireRepositoryModified();
    }

    public synchronized Lookup
    getLookup()
    {
        return lookup_;
    }

    public synchronized Register
    getRegister()
    {
        return register_;
    }

    public synchronized Proxy
    getProxy()
    {
        return proxy_;
    }

    public synchronized Link
    getLink()
    {
        return link_;
    }

    public synchronized Admin
    getAdmin()
    {
        return admin_;
    }

    public boolean
    addType(String name,
            String interfaceName,
            PropStruct[] props,
            String[] superTypes,
            boolean notify,
            StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            repos_.add_type(name, interfaceName, props, superTypes);
            if(notify)
                fireTypeAdded(name);
            result = true;
        }
        catch(IllegalServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalTypeNameKey", e.type));
        }
        catch(ServiceTypeExists e)
        {
            message.append(
                AppStandards.getFormattedString("TypeExistsKey", e.name));
        }
        catch(org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.
              InterfaceTypeMismatch e)
        {
            message.append(
                AppStandards.getFormattedString("IntTypeMismatchKey",
                                                e.base_service));
        }
        catch(IllegalPropertyName e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalPropNameKey",
                                                e.name));
        }
        catch(DuplicatePropertyName e)
        {
            message.append(
                AppStandards.getFormattedString("DuplicatePropNameKey",
                                                e.name));
        }
        catch(ValueTypeRedefinition e)
        {
            message.append(
                AppStandards.getFormattedString("PropRedefinedKey",
                    e.definition_1.name));
        }
        catch(UnknownServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownSuperTypeKey",
                                                e.type));
        }
        catch(DuplicateServiceTypeName e)
        {
            message.append(
                AppStandards.getFormattedString("DuplicateSuperTypeKey",
                                                e.name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public boolean
    addType(String name, TypeStruct ts, boolean notify, StringBuffer message)
    {
        return addType(name, ts.if_name, ts.props, ts.super_types, notify,
            message);
    }

    public boolean
    modifyType(String name, boolean mask, boolean notify, StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            if(mask)
                repos_.mask_type(name);
            else
                repos_.unmask_type(name);

            if(notify)
                fireTypeModified(name);

            result = true;
        }
        catch(IllegalServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalTypeNameKey", e.type));
        }
        catch(UnknownServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownTypeNameKey", e.type));
        }
        catch(NotMasked e)
        {
            message.append(
                AppStandards.getFormattedString("TypeNotMaskedKey", e.name));
        }
        catch(AlreadyMasked e)
        {
            message.append(
                AppStandards.getFormattedString("TypeAlreadyMaskedKey",
                                                e.name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public boolean
    removeType(String name, boolean notify, StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            repos_.remove_type(name);
            if(notify)
                fireTypeRemoved(name);
            result = true;
        }
        catch(IllegalServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalTypeNameKey", e.type));
        }
        catch(UnknownServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownTypeNameKey", e.type));
        }
        catch(HasSubTypes e)
        {
            message.append(
                AppStandards.getFormattedString("TypeHasSubtypesKey",
                                                e.the_type));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public String[]
    listTypes(StringBuffer message)
    {
        String[] result = null;

        message.setLength(0);

        try
        {
            SpecifiedServiceTypes whichTypes = new SpecifiedServiceTypes();
            whichTypes._default(ListOption.all);
            result = repos_.list_types(whichTypes);
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public TypeStruct
    describeType(String name, StringBuffer message)
    {
        TypeStruct result = null;

        message.setLength(0);

        try
        {
            result = repos_.describe_type(name);
        }
        catch(IllegalServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalTypeNameKey", e.type));
        }
        catch(UnknownServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownTypeNameKey", e.type));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public TypeStruct
    fullyDescribeType(String name, StringBuffer message)
    {
        TypeStruct result = null;

        message.setLength(0);

        try
        {
            result = repos_.fully_describe_type(name);
        }
        catch(IllegalServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalTypeNameKey", e.type));
        }
        catch(UnknownServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownTypeNameKey", e.type));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public String
    addOffer(org.omg.CORBA.Object reference,
             String serviceType,
             Property[] props,
             boolean notify,
             StringBuffer message)
    {
        String result = null;

        message.setLength(0);

        try
        {
            result = register_.export(reference, serviceType, props);
            if(notify)
                fireOfferAdded(result);
        }
        catch(InvalidObjectRef e)
        {
            message.append(AppStandards.getString("InvalidObjectKey"));
        }
        catch(IllegalServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalTypeNameKey", e.type));
        }
        catch(UnknownServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownTypeNameKey", e.type));
        }
        catch(org.omg.CosTrading.RegisterPackage.InterfaceTypeMismatch e)
        {
            message.append(
                AppStandards.getFormattedString("IntTypeMismatchKey", e.type));
        }
        catch(IllegalPropertyName e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalPropNameKey", e.name));
        }
        catch(PropertyTypeMismatch e)
        {
            message.append(
                AppStandards.getFormattedString("PropTypeMismatchKey",
                                                e.prop.name));
        }
        catch(ReadonlyDynamicProperty e)
        {
            message.append(
                AppStandards.getFormattedString("ReadonlyDynamicKey", e.name));
        }
        catch(MissingMandatoryProperty e)
        {
            message.append(
                AppStandards.getFormattedString("MissingMandatoryKey",
                                                e.name));
        }
        catch(DuplicatePropertyName e)
        {
            message.append(
                AppStandards.getFormattedString("DuplicatePropNameKey",
                                                e.name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public String
    addOffer(OfferInfo info, boolean notify, StringBuffer message)
    {
        return addOffer(info.reference, info.type, info.properties, notify,
            message);
    }

    public boolean
    modifyOffer(String id,
                String[] delList,
                Property[] modifyList,
                boolean notify,
                StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            register_.modify(id, delList, modifyList);
            if(notify)
                fireOfferModified(id);
            result = true;
        }
        catch(NotImplemented e)
        {
            message.append(AppStandards.getString("OfferImplKey"));
        }
        catch(IllegalOfferId e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalOfferIdKey", e.id));
        }
        catch(UnknownOfferId e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownOfferIdKey", e.id));
        }
        catch(ProxyOfferId e)
        {
            message.append(
                AppStandards.getFormattedString("ProxyOfferKey", e.id));
        }
        catch(IllegalPropertyName e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalPropNameKey", e.name));
        }
        catch(UnknownPropertyName e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownPropNameKey", e.name));
        }
        catch(PropertyTypeMismatch e)
        {
            message.append(
                AppStandards.getFormattedString("PropTypeMismatchKey", e.prop.name));
        }
        catch(ReadonlyDynamicProperty e)
        {
            message.append(
                AppStandards.getFormattedString("ReadonlyDynamicKey", e.name));
        }
        catch(MandatoryProperty e)
        {
            message.append(
                AppStandards.getFormattedString("MandatoryPropertyKey",
                                                e.name));
        }
        catch(ReadonlyProperty e)
        {
            message.append(
                AppStandards.getFormattedString("ReadonlyPropertyKey",
                                                e.name));
        }
        catch(DuplicatePropertyName e)
        {
            message.append(
                AppStandards.getFormattedString("DuplicatePropNameKey",
                                                e.name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public boolean
    removeOffer(String id, boolean notify, StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            register_.withdraw(id);
            if(notify)
                fireOfferRemoved(id);
            result = true;
        }
        catch(IllegalOfferId e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalOfferIdKey", e.id));
        }
        catch(UnknownOfferId e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownOfferIdKey", e.id));
        }
        catch(ProxyOfferId e)
        {
            message.append(
                AppStandards.getFormattedString("ProxyOfferKey", e.id));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public boolean
    withdrawOffers(String serviceType,
                   String constraint,
                   boolean notify,
                   StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            register_.withdraw_using_constraint(serviceType, constraint);
            if(notify)
                fireOffersRemoved();
            result = true;
        }
        catch(IllegalServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalTypeNameKey", e.type));
        }
        catch(UnknownServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownTypeNameKey", e.type));
        }
        catch(IllegalConstraint e)
        {
            message.append(AppStandards.getString("IllegalConstraintKey"));
        }
        catch(NoMatchingOffers e)
        {
            message.append(AppStandards.getString("NoOffersKey"));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public String
    addProxy(Lookup target,
             String serviceType,
             Property[] props,
             boolean ifMatchAll,
             String recipe,
             Policy[] policies,
             boolean notify,
             StringBuffer message)
    {
        String result = null;

        message.setLength(0);

        try
        {
            result = proxy_.export_proxy(target, serviceType, props,
                                         ifMatchAll, recipe, policies);
            if(notify)
                fireProxyAdded(result);
        }
        catch(IllegalServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalTypeNameKey", e.type));
        }
        catch(UnknownServiceType e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownTypeNameKey", e.type));
        }
        catch(InvalidLookupRef e)
        {
            message.append(AppStandards.getString("InvalidLookupKey"));
        }
        catch(IllegalPropertyName e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalPropNameKey", e.name));
        }
        catch(PropertyTypeMismatch e)
        {
            message.append(
                AppStandards.getFormattedString("PropTypeMismatchKey",
                                                e.prop.name));
        }
        catch(ReadonlyDynamicProperty e)
        {
            message.append(
                AppStandards.getFormattedString("ReadonlyDynamicKey", e.name));
        }
        catch(MissingMandatoryProperty e)
        {
            message.append(
                AppStandards.getFormattedString("MissingMandatoryKey",
                                                e.name));
        }
        catch(IllegalRecipe e)
        {
            message.append(AppStandards.getString("IllegalRecipeKey"));
        }
        catch(DuplicatePropertyName e)
        {
            message.append(
                AppStandards.getFormattedString("DuplicatePropNameKey",
                                                e.name));
        }
        catch(DuplicatePolicyName e)
        {
            message.append(
                AppStandards.getFormattedString("DuplicatePolicyKey",
                                                e.name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public String
    addProxy(ProxyInfo info, boolean notify, StringBuffer message)
    {
        return addProxy(info.target, info.type, info.properties,
            info.if_match_all, info.recipe, info.policies_to_pass_on, notify,
            message);
    }

    public boolean
    removeProxy(String id, boolean notify, StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            proxy_.withdraw_proxy(id);
            if(notify)
                fireProxyRemoved(id);
            result = true;
        }
        catch(IllegalOfferId e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalOfferIdKey", e.id));
        }
        catch(UnknownOfferId e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownOfferIdKey", e.id));
        }
        catch(NotProxyOfferId e)
        {
            message.append(
                AppStandards.getFormattedString("NotProxyOfferKey", e.id));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public boolean
    addLink(String name,
            Lookup target,
            FollowOption defaultRule,
            FollowOption limitingRule,
            boolean notify,
            StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            link_.add_link(name, target, defaultRule, limitingRule);
            if(notify)
                fireLinkAdded(name);
            result = true;
        }
        catch(IllegalLinkName e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalLinkNameKey", e.name));
        }
        catch(DuplicateLinkName e)
        {
            message.append(
                AppStandards.getFormattedString("LinkExistsKey", e.name));
        }
        catch(InvalidLookupRef e)
        {
            message.append(AppStandards.getString("InvalidLookupKey"));
        }
        catch(DefaultFollowTooPermissive e)
        {
            message.append(
                AppStandards.getFormattedString("DefaultFollowPermKey", name));
        }
        catch(LimitingFollowTooPermissive e)
        {
            message.append(
                AppStandards.getFormattedString("LimitingFollowPermKey",
                                                name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public boolean
    addLink(String name, LinkInfo info, boolean notify, StringBuffer message)
    {
        return addLink(name, info.target, info.def_pass_on_follow_rule,
            info.limiting_follow_rule, notify, message);
    }

    public boolean
    modifyLink(String name,
               FollowOption defaultRule,
               FollowOption limitingRule,
               boolean notify,
               StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            link_.modify_link(name, defaultRule, limitingRule);
            if(notify)
                fireLinkModified(name);
            result = true;
        }
        catch(IllegalLinkName e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalLinkNameKey", e.name));
        }
        catch(UnknownLinkName e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownLinkNameKey", e.name));
        }
        catch(DefaultFollowTooPermissive e)
        {
            message.append(
                AppStandards.getFormattedString("DefaultFollowPermKey", name));
        }
        catch(LimitingFollowTooPermissive e)
        {
            message.append(
                AppStandards.getFormattedString("LimitingFollowPermKey",
                                                name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public boolean
    removeLink(String name, boolean notify, StringBuffer message)
    {
        boolean result = false;

        message.setLength(0);

        try
        {
            link_.remove_link(name);
            if(notify)
                fireLinkRemoved(name);
            result = true;
        }
        catch(IllegalLinkName e)
        {
            message.append(
                AppStandards.getFormattedString("IllegalLinkNameKey", e.name));
        }
        catch(UnknownLinkName e)
        {
            message.append(
                AppStandards.getFormattedString("UnknownLinkNameKey", e.name));
        }
        catch(org.omg.CORBA.SystemException e)
        {
            message.append(AppStandards.getString("SystemErrorKey"));
            e.printStackTrace();
        }

        return result;
    }

    public synchronized void
    fireTypeAdded(String name)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.typeAdded(name);
        }
    }

    public synchronized void
    fireTypeModified(String name)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.typeModified(name);
        }
    }

    public synchronized void
    fireTypeRemoved(String name)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.typeRemoved(name);
        }
    }

    public synchronized void
    fireTypesRemoved()
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.typesRemoved();
        }
    }

    public synchronized void
    fireOfferAdded(String id)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.offerAdded(id);
        }
    }

    public synchronized void
    fireOfferModified(String id)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.offerModified(id);
        }
    }

    public synchronized void
    fireOfferRemoved(String id)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.offerRemoved(id);
        }
    }

    public synchronized void
    fireOffersRemoved()
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.offersRemoved();
        }
    }

    public synchronized void
    fireProxyAdded(String id)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.proxyAdded(id);
        }
    }

    public synchronized void
    fireProxyRemoved(String id)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.proxyRemoved(id);
        }
    }

    public synchronized void
    fireProxiesRemoved()
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.proxiesRemoved();
        }
    }

    public synchronized void
    fireLinkAdded(String name)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.linkAdded(name);
        }
    }

    public synchronized void
    fireLinkModified(String name)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.linkModified(name);
        }
    }

    public synchronized void
    fireLinkRemoved(String name)
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.linkRemoved(name);
        }
    }

    public synchronized void
    fireLinksRemoved()
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.linksRemoved();
        }
    }

    public synchronized void
    fireRepositoryModified()
    {
        Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TraderModelListener l = (TraderModelListener)e.nextElement();
            l.repositoryModified();
        }
    }

    protected void
    getTraderComponents()
    {
        lookup_ = connection_.getLookup();
        org.omg.CORBA.Object object = lookup_.type_repos();
        repos_ = ServiceTypeRepositoryHelper.narrow(object);
        link_ = lookup_.link_if();
        register_ = lookup_.register_if();
        proxy_ = lookup_.proxy_if();
        admin_ = lookup_.admin_if();
    }
}
