// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTradingConsole.Util;

import javax.swing.AbstractListModel;
import javax.swing.ComboBoxModel;
import java.io.*;
import java.util.*;

public class TypeCodeComboBoxModel
    extends AbstractListModel
    implements ComboBoxModel
{
    private Vector items_ = new Vector();
    private Item selected_;
    private static final org.omg.CORBA.TCKind[] typeKinds_ =
    {
        org.omg.CORBA.TCKind.tk_boolean,
        org.omg.CORBA.TCKind.tk_char,
        org.omg.CORBA.TCKind.tk_double,
        org.omg.CORBA.TCKind.tk_float,
        org.omg.CORBA.TCKind.tk_long,
        org.omg.CORBA.TCKind.tk_octet,
        org.omg.CORBA.TCKind.tk_short,
        org.omg.CORBA.TCKind.tk_string,
        org.omg.CORBA.TCKind.tk_ulong,
        org.omg.CORBA.TCKind.tk_ushort
    };


    public
    TypeCodeComboBoxModel()
    {
    }

    public int
    getSize()
    {
        return items_.size();
    }

    public Object
    getElementAt(int index)
    {
        return items_.elementAt(index);
    }

    // From ComboBoxModel
    public void
    setSelectedItem(Object anItem)
    {
        selected_ = (Item)anItem;
        fireContentsChanged(this, -1, -1);
    }

    public Object
    getSelectedItem()
    {
        return selected_;
    }

    public org.omg.CORBA.TypeCode
    getSelectedTypeCode()
    {
        org.omg.CORBA.TypeCode result = null;

        if(selected_ != null)
            result = selected_.getTypeCode();

        return result;
    }

    public void
    setSelectedTypeCode(org.omg.CORBA.TypeCode tc)
    {
        Enumeration e = items_.elements();
        while(e.hasMoreElements())
        {
            Item i = (Item)e.nextElement();
            if(i.getTypeCode().equal(tc))
            {
                setSelectedItem(i);
                break;
            }
        }
    }

    public void
    addItem(org.omg.CORBA.TypeCode tc)
    {
        items_.addElement(new Item(tc));
    }

    public void
    addItem(org.omg.CORBA.TypeCode tc, String name)
    {
        items_.addElement(new Item(tc, name));
    }

    public void
    loadBasicTypes()
    {
        org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init();
        for(int i = 0 ; i < typeKinds_.length ; i++)
        {
            org.omg.CORBA.TypeCode tc = orb.get_primitive_tc(typeKinds_[i]);
            addItem(tc);
        }

        for(int i = 0 ; i < typeKinds_.length ; i++)
        {
            org.omg.CORBA.TypeCode tc = orb.get_primitive_tc(typeKinds_[i]);
            org.omg.CORBA.TypeCode seqTC = orb.create_sequence_tc(0, tc);
            addItem(seqTC);
        }
    }



    static class Item
    {
        private org.omg.CORBA.TypeCode tc_;
        private String name_;

        public
        Item(org.omg.CORBA.TypeCode tc)
        {
            this(tc, null);
        }

        public
        Item(org.omg.CORBA.TypeCode tc, String name)
        {
            tc_ = tc;

            if(name == null)
            {
                StringWriter sw = new StringWriter();
                PrintWriter pw = new PrintWriter(sw);
                com.ooc.Util.CORBA.AnyWriter out =
                    new com.ooc.Util.CORBA.AnyWriter(pw);
                out.printName(tc);
                pw.flush();
                name_ = sw.toString();
            }
            else
                name_ = name;
        }

        public org.omg.CORBA.TypeCode
        getTypeCode()
        {
            return tc_;
        }

        public String
        toString()
        {
            return name_;
        }
    }
}
