// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

//
// Used to test dynamic property behavior; this object remembers
// whether it's been asked to evaluate a property so that we can
// test if the trader is obeying our policies.  It also returns
// the value we specify.
//
public class DynPropImpl
    extends org.omg.CosTradingDynamic._DynamicPropEvalImplBase
{
    private String name_;
    private org.omg.CORBA.Any value_;
    private org.omg.CORBA.Any extraInfo_;
    private boolean asked_ = false;

    public DynPropImpl(String name,
                       org.omg.CORBA.Any value,
                       org.omg.CORBA.Any extraInfo)
    {
        name_ = name;
        value_ = value;
        extraInfo_ = extraInfo;
    }

    public boolean
    getAsked()
    {
        return asked_;
    }

    public void
    reset()
    {
        asked_ = false;
    }

    public org.omg.CORBA.Any
    evalDP(String prop,
           org.omg.CORBA.TypeCode type,
           org.omg.CORBA.Any extra)
        throws org.omg.CosTradingDynamic.DPEvalFailure
    {
        org.omg.CORBA.Any result;

        //
        // Remember that we've been invoked
        //
        asked_ = true;

        if(name_ != null && ! prop.equals(name_))
        {
            System.err.println("DynPropImpl: Invalid property '" +
                               prop + "'");
            throw new org.omg.CosTradingDynamic.
                DPEvalFailure(prop, type, extra);
        }

        //
        // Make sure the value for extra matches what we were given
        //
        if(extraInfo_ != null && ! extraInfo_.equal(extra))
        {
            System.err.println("DynPropImpl: Invalid extra_info value");
            throw new org.omg.CosTradingDynamic.
                DPEvalFailure(prop, type, extra);
        }

        return value_;
    }
}
