// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

public class TestAdmin extends TestBase
{
    private org.omg.CORBA.ORB orb_;
    private org.omg.CosTrading.Lookup trader_;
    private org.omg.CosTrading.Admin admin_;

    TestAdmin(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        orb_ = orb;
        trader_ = trader;
        admin_ = trader.admin_if();
        TEST(admin_ != null);

        testAttributes();
        testOperations();
    }

    void
    testAttributes()
    {
        testULongAttribute("def_search_card");
        testULongAttribute("max_search_card");
        testULongAttribute("def_match_card");
        testULongAttribute("max_match_card");
        testULongAttribute("def_return_card");
        testULongAttribute("max_return_card");
        testULongAttribute("max_list");
        testULongAttribute("def_hop_count");
        testULongAttribute("max_hop_count");

        testBooleanAttribute("supports_dynamic_properties");
        testBooleanAttribute("supports_modifiable_properties");
        testBooleanAttribute("supports_proxy_offers");

        testStem();
    }

    void
    testOperations()
    {
        java.util.Vector offers = null, proxies = null;

        try
        {
            //
            // Install the simple service type
            //
            installSimpleType(orb_, trader_);

            //
            // Export some offers and proxy offers
            //
            int count = 10;
            offers = exportSimpleOffers(orb_, trader_, count);
            TEST(offers.size() == count);
            proxies = exportSimpleProxies(orb_, trader_, count);
            TEST(proxies.size() == count);

            testListOffers(offers);
            testListProxies(proxies);
        }
        finally
        {
            if(offers != null)
                withdrawOffers(trader_, offers);

            if(proxies != null)
                withdrawProxies(trader_, proxies);

            //
            // Remove the simple service type
            //
            removeType(trader_, SIMPLE_TYPE);
        }
    }

    void
    testListOffers(java.util.Vector ids)
    {
        try
        {
            int numOffers = ids.size();

            org.omg.CosTrading.OfferIdSeqHolder seq =
                new org.omg.CosTrading.OfferIdSeqHolder();
            org.omg.CosTrading.OfferIdIteratorHolder iter =
                new org.omg.CosTrading.OfferIdIteratorHolder();

            //
            // Assume there are no other offers
            //

            admin_.list_offers(numOffers + 1, seq, iter);
            TEST(seq.value.length == numOffers);
            TEST(iter.value == null);
        }
        catch(org.omg.CosTrading.NotImplemented ex)
        {
            handleException(ex);
        }
    }

    void
    testListProxies(java.util.Vector ids)
    {
        try
        {
            int numOffers = ids.size();

            org.omg.CosTrading.OfferIdSeqHolder seq =
                new org.omg.CosTrading.OfferIdSeqHolder();
            org.omg.CosTrading.OfferIdIteratorHolder iter =
                new org.omg.CosTrading.OfferIdIteratorHolder();

            //
            // Assume there are no other proxy offers
            //

            admin_.list_proxies(numOffers + 1, seq, iter);
            TEST(seq.value.length == numOffers);
            TEST(iter.value == null);
        }
        catch(org.omg.CosTrading.NotImplemented ex)
        {
            handleException(ex);
        }
    }

    void
    testStem()
    {
        byte[] oldStem;
        byte[] stem;
        byte[] newStem = "__THIS_IS_A_TEST__".getBytes();

        oldStem = admin_.request_id_stem();
        stem = admin_.set_request_id_stem(newStem);
        for(int i = 0 ; i < stem.length ; i++)
            TEST(stem[i] == oldStem[i]);
        stem = admin_.request_id_stem();
        for(int i = 0 ; i < stem.length ; i++)
            TEST(stem[i] == newStem[i]);
        stem = admin_.set_request_id_stem(oldStem);
        for(int i = 0 ; i < stem.length ; i++)
            TEST(stem[i] == newStem[i]);
        stem = admin_.request_id_stem();
        for(int i = 0 ; i < stem.length ; i++)
            TEST(stem[i] == oldStem[i]);
    }

    void
    testULongAttribute(String name)
    {
        try
        {
            //
            // We use the reflection API to obtain the get & set methods
            // for the attribute
            //
            Class c = admin_.getClass();
            Class[] params;
            params = new Class[0];
            java.lang.reflect.Method getter =
                c.getDeclaredMethod(name, params);
            params = new Class[1];
            params[0] = Integer.TYPE;
            java.lang.reflect.Method setter =
                c.getDeclaredMethod("set_" + name, params);

            //
            // Test the ULong attribute in the Admin interface
            // using the following algorithm:
            //
            //  1) Choose a positive number at random
            //  2) Get the current value
            //  3) Change the attribute to the random value
            //  4) Compare the old value returned against the value from #2
            //  5) Get the new value
            //  6) Compare the new value against the random value
            //  7) Change the attribute back to the old value
            //  8) Compare the old value against the random value
            //  9) Get the current value
            // 10) Compare the current value against the old value
            //
            Integer ulongValue, ulongReadValue, ulongOldValue;
            Integer n;
            java.util.Random rand = new java.util.Random();
            Object[] getArgs = new Object[0];
            Object[] setArgs = new Object[1];

            n = new Integer(Math.abs(rand.nextInt()));

            ulongReadValue = (Integer)getter.invoke(admin_, getArgs);
            setArgs[0] = n;
            ulongOldValue = (Integer)setter.invoke(admin_, setArgs);
            TEST(ulongOldValue.equals(ulongReadValue));
            ulongValue = (Integer)getter.invoke(admin_, getArgs);
            TEST(ulongValue.equals(n));
            setArgs[0] = ulongOldValue;
            ulongValue = (Integer)setter.invoke(admin_, setArgs);
            TEST(ulongValue.equals(n));
            ulongValue = (Integer)getter.invoke(admin_, getArgs);
            TEST(ulongValue.equals(ulongOldValue));
        }
        catch(NoSuchMethodException ex)
        {
            handleException(ex);
        }
        catch(SecurityException ex)
        {
            handleException(ex);
        }
        catch(IllegalAccessException ex)
        {
            handleException(ex);
        }
        catch(IllegalArgumentException ex)
        {
            handleException(ex);
        }
        catch(java.lang.reflect.InvocationTargetException ex)
        {
            handleException(ex);
        }
    }

    void
    testBooleanAttribute(String name)
    {
        try
        {
            //
            // We use the reflection API to obtain the get & set methods
            // for the attribute
            //
            Class c = admin_.getClass();
            Class[] params;
            params = new Class[0];
            java.lang.reflect.Method getter =
                c.getDeclaredMethod(name, params);
            params = new Class[1];
            params[0] = Boolean.TYPE;
            java.lang.reflect.Method setter =
                c.getDeclaredMethod("set_" + name, params);

            //
            // Test the Boolean attribute in the Admin interface
            // using the following algorithm:
            //
            //  1) Get the current value
            //  2) Change the attribute to the opposite value
            //  3) Compare the old value returned against the value from #1
            //  4) Get the new value
            //  5) Compare the new value against the opposite value
            //  6) Change the attribute back to the old value
            //  7) Compare the old value against the opposite value
            //  8) Get the current value
            //  9) Compare the current value against the old value
            //
            Boolean bValue, bReadValue, bOldValue;
            Boolean b;
            Object[] getArgs = new Object[0];
            Object[] setArgs = new Object[1];

            bReadValue = (Boolean)getter.invoke(admin_, getArgs);
            b = new Boolean(! bReadValue.booleanValue());
            setArgs[0] = b;
            bOldValue = (Boolean)setter.invoke(admin_, setArgs);
            TEST(bOldValue.equals(bReadValue));
            bValue = (Boolean)getter.invoke(admin_, getArgs);
            TEST(bValue.equals(b));
            setArgs[0] = bOldValue;
            bValue = (Boolean)setter.invoke(admin_, setArgs);
            TEST(bValue.equals(b));
            bValue = (Boolean)getter.invoke(admin_, getArgs);
            TEST(bValue.equals(bOldValue));
        }
        catch(NoSuchMethodException ex)
        {
            handleException(ex);
        }
        catch(SecurityException ex)
        {
            handleException(ex);
        }
        catch(IllegalAccessException ex)
        {
            handleException(ex);
        }
        catch(IllegalArgumentException ex)
        {
            handleException(ex);
        }
        catch(java.lang.reflect.InvocationTargetException ex)
        {
            handleException(ex);
        }
    }
}
