// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

// Things are just too ugly without this
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class TestBase
{
    public static final String SIMPLE_TYPE =
        "IDL:obtrader/Test/SimpleType:1.0";

    public static final String QUERY_BASE_TYPE =
        "IDL:obtrader/Test/QueryBase:1.0";

    public static final String QUERY_SUB_TYPE =
        "IDL:obtrader/Test/QuerySub:1.0";

    public static final String REGISTER_TYPE =
        "IDL:obtrader/Test/RegisterType:1.0";

    public static final String SEQ_ARRAY_TYPE =
        "IDL:obtrader/Test/SeqArrayType:1.0";


    static void
    TEST(boolean expr)
    {
	if(!expr)
	    throw new TestException();
    }

    static void
    fail(String message)
    {
        throw new TestException(message);
    }

    static void
    handleException(Exception ex)
    {
        System.err.println();
        System.err.println("*** Unexpected exception thrown!");
        System.err.println();
        ex.printStackTrace(System.err);
        System.err.println();
        System.err.println("Terminating.");
        System.exit(1);
    }

    //
    // Connect to Trading Service.  If the property "trader.file"
    // is defined, then the Trader's IOR is read from the file.
    // Otherwise, the initial reference 'TradingService' is resolved.
    //
    static org.omg.CosTrading.Lookup
    connect(org.omg.CORBA.ORB orb)
    {
        org.omg.CosTrading.Lookup result = null;

        String iorfile = System.getProperty("trader.file");

        if(iorfile != null)
        {
            //
            // Read the IOR of the Trading service from a file
            //

            try
            {
                java.io.FileReader fr = new java.io.FileReader(iorfile);
                java.io.BufferedReader in = new java.io.BufferedReader(fr);
                String ref = in.readLine();
                fr.close();

                org.omg.CORBA.Object obj = orb.string_to_object(ref);

                if(obj == null)
                    System.err.println("Error:  Invalid IOR in file '" +
                                       iorfile + "'");
                else
                    result = org.omg.CosTrading.LookupHelper.narrow(obj);
            }
            catch(java.io.IOException ex)
            {
                System.err.println("Error:  Unable to read IOR from file '" +
                                   iorfile + "'");
                ex.printStackTrace();
            }
        }
        else
        {
            //
            // Resolve the 'TradingService' initial reference
            //

            try
            {
                org.omg.CORBA.Object obj =
                    orb.resolve_initial_references("TradingService");
                result = org.omg.CosTrading.LookupHelper.narrow(obj);
            }
            catch(org.omg.CORBA.ORBPackage.InvalidName ex)
            {
                System.err.println("Error:  Unable to resolve initial " +
                                   "reference 'TradingService'");
            }
        }

        return result;
    }

    static void
    installSimpleType(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        boolean typeExists = false;
        org.omg.CosTradingRepos.ServiceTypeRepository repos = null;

        //
        // Verify that the type SIMPLE_TYPE already exists in the
        // service type repository; if not, we need to install it
        //
        try
        {
            org.omg.CORBA.Object obj = trader.type_repos();
            repos =
                org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.narrow(obj);
            repos.describe_type(SIMPLE_TYPE);
            typeExists = true;
        }
        catch(org.omg.CosTrading.IllegalServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CosTrading.UnknownServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            handleException(ex);
        }

        if(! typeExists)
        {
            try
            {
                String[] superTypes = new String[0];

                //
                // Define the properties for this service type
                //

                PropStruct[] props = new PropStruct[1];

                props[0] = new PropStruct();
                props[0].name = "name";
                props[0].value_type = orb.create_string_tc(0);
                props[0].mode = PropertyMode.PROP_MANDATORY;

                //
                // Add the service type, using the repository ID of the
                // CosTrading::Lookup interface
                //

                repos.add_type(SIMPLE_TYPE, "IDL:omg.org/CosTrading/Lookup:1.0",
                    props, superTypes);
            }
            catch(Exception ex)
            {
                handleException(ex);
            }
        }
    }

    static void
    installQueryBaseType(org.omg.CORBA.ORB orb,
                         org.omg.CosTrading.Lookup trader)
    {
        boolean typeExists = false;
        org.omg.CosTradingRepos.ServiceTypeRepository repos = null;

        //
        // Verify that the type QUERY_BASE_TYPE already exists in the
        // service type repository; if not, we need to install it
        //
        try
        {
            org.omg.CORBA.Object obj = trader.type_repos();
            repos =
                org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.narrow(obj);
            repos.describe_type(QUERY_BASE_TYPE);
            typeExists = true;
        }
        catch(org.omg.CosTrading.IllegalServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CosTrading.UnknownServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            handleException(ex);
        }

        if(! typeExists)
        {
            try
            {
                String[] superTypes = new String[0];

                //
                // Define the properties for this service type
                //

                PropStruct[] props = new PropStruct[1];

                props[0] = new PropStruct();
                props[0].name = "count";
                props[0].value_type =
                    orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
                props[0].mode = PropertyMode.PROP_NORMAL;

                //
                // Add the service type, using the repository ID of the
                // CosTrading::Lookup interface
                //

                repos.add_type(QUERY_BASE_TYPE,
                               "IDL:omg.org/CosTrading/Lookup:1.0",
                               props, superTypes);
            }
            catch(Exception ex)
            {
                handleException(ex);
            }
        }
    }

    static void
    installQuerySubType(org.omg.CORBA.ORB orb,
                        org.omg.CosTrading.Lookup trader)
    {
        boolean typeExists = false;
        org.omg.CosTradingRepos.ServiceTypeRepository repos = null;

        //
        // Verify that the type QUERY_SUB_TYPE already exists in the
        // service type repository; if not, we need to install it
        //
        try
        {
            org.omg.CORBA.Object obj = trader.type_repos();
            repos =
                org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.narrow(obj);
            repos.describe_type(QUERY_SUB_TYPE);
            typeExists = true;
        }
        catch(org.omg.CosTrading.IllegalServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CosTrading.UnknownServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            handleException(ex);
        }

        if(! typeExists)
        {
            try
            {
                String[] superTypes = new String[1];
                superTypes[0] = QUERY_BASE_TYPE;

                //
                // Define the properties for this service type
                //

                PropStruct[] props = new PropStruct[1];

                props[0] = new PropStruct();
                props[0].name = "weight";
                props[0].value_type =
                    orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_double);
                props[0].mode = PropertyMode.PROP_NORMAL;

                //
                // Add the service type, using the repository ID of the
                // CosTrading::Lookup interface
                //

                repos.add_type(QUERY_SUB_TYPE,
                               "IDL:omg.org/CosTrading/Lookup:1.0",
                               props, superTypes);
            }
            catch(Exception ex)
            {
                handleException(ex);
            }
        }
    }

    static void
    installRegisterType(org.omg.CORBA.ORB orb,
                        org.omg.CosTrading.Lookup trader)
    {
        boolean typeExists = false;
        org.omg.CosTradingRepos.ServiceTypeRepository repos = null;

        //
        // Verify that the type REGISTER_TYPE already exists in the
        // service type repository; if not, we need to install it
        //
        try
        {
            org.omg.CORBA.Object obj = trader.type_repos();
            repos =
                org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.narrow(obj);
            repos.describe_type(REGISTER_TYPE);
            typeExists = true;
        }
        catch(org.omg.CosTrading.IllegalServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CosTrading.UnknownServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            handleException(ex);
        }

        if(! typeExists)
        {
            try
            {
                String[] superTypes = new String[0];

                //
                // Define the properties for this service type
                //

                PropStruct[] props = new PropStruct[4];

                props[0] = new PropStruct();
                props[0].name = "prop1";
                props[0].value_type = orb.create_string_tc(0);
                props[0].mode = PropertyMode.PROP_NORMAL;

                props[1] = new PropStruct();
                props[1].name = "prop2";
                props[1].value_type =
                    orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
                props[1].mode = PropertyMode.PROP_MANDATORY;

                props[2] = new PropStruct();
                props[2].name = "prop3";
                props[2].value_type =
                    orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char);
                props[2].mode = PropertyMode.PROP_READONLY;

                props[3] = new PropStruct();
                props[3].name = "prop4";
                props[3].value_type =
                    orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ushort);
                props[3].mode = PropertyMode.PROP_MANDATORY_READONLY;

                //
                // Add the service type, using the repository ID of the
                // CosTrading::Lookup interface
                //

                repos.add_type(REGISTER_TYPE,
                               "IDL:omg.org/CosTrading/Lookup:1.0",
                               props, superTypes);
            }
            catch(Exception ex)
            {
                handleException(ex);
            }
        }
    }

    static void
    installSeqArrayType(org.omg.CORBA.ORB orb,
                        org.omg.CosTrading.Lookup trader)
    {
        boolean typeExists = false;
        org.omg.CosTradingRepos.ServiceTypeRepository repos = null;

        //
        // Verify that the type SEQ_ARRAY_TYPE already exists in the
        // service type repository; if not, we need to install it
        //
        try
        {
            org.omg.CORBA.Object obj = trader.type_repos();
            repos =
                org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.narrow(obj);
            repos.describe_type(SEQ_ARRAY_TYPE);
            typeExists = true;
        }
        catch(org.omg.CosTrading.IllegalServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CosTrading.UnknownServiceType ex)
        {
            // ignore
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            handleException(ex);
        }

        if(! typeExists)
        {
            try
            {
                String[] superTypes = new String[0];
                org.omg.CORBA.TypeCode content;

                //
                // Define the properties for this service type
                //

                PropStruct[] props = new PropStruct[2];

                props[0] = new PropStruct();
                props[0].name = "short_arr";
                content = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_short);
                props[0].value_type = orb.create_array_tc(5, content);
                props[0].mode = PropertyMode.PROP_NORMAL;

                props[1] = new PropStruct();
                props[1].name = "string_seq";
                content = orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
                props[1].value_type = orb.create_sequence_tc(0, content);
                props[1].mode = PropertyMode.PROP_NORMAL;

                //
                // Add the service type, using the repository ID of the
                // CosTrading::Lookup interface
                //

                repos.add_type(SEQ_ARRAY_TYPE,
                               "IDL:omg.org/CosTrading/Lookup:1.0",
                               props, superTypes);
            }
            catch(Exception ex)
            {
                handleException(ex);
            }
        }
    }

    static void
    removeType(org.omg.CosTrading.Lookup trader, String type)
    {
        org.omg.CosTradingRepos.ServiceTypeRepository repos = null;

        try
        {
            org.omg.CORBA.Object obj = trader.type_repos();
            repos =
                org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.narrow(obj);
            repos.remove_type(type);
        }
        catch(org.omg.CosTrading.UnknownServiceType ex)
        {
            // ignore
        }
        catch(Exception ex)
        {
            handleException(ex);
        }
    }

    static void
    withdrawOffers(org.omg.CosTrading.Lookup trader, java.util.Vector offers)
    {
        try
        {
            org.omg.CosTrading.Register reg = trader.register_if();
            TEST(reg != null);

            java.util.Enumeration e = offers.elements();
            while(e.hasMoreElements())
            {
                String id = (String)e.nextElement();
                try
                {
                    reg.withdraw(id);
                }
                catch(org.omg.CosTrading.UnknownOfferId ex)
                {
                    // ignore
                }
            }
        }
        catch(Exception ex)
        {
            handleException(ex);
        }
    }

    static void
    withdrawAllOffers(org.omg.CosTrading.Lookup trader, String type)
    {
        try
        {
            org.omg.CosTrading.Register reg = trader.register_if();
            TEST(reg != null);

            try
            {
                reg.withdraw_using_constraint(type, "TRUE");
            }
            catch(org.omg.CosTrading.RegisterPackage.NoMatchingOffers ex)
            {
                // ignore
            }
        }
        catch(Exception ex)
        {
            handleException(ex);
        }
    }

    static void
    withdrawOffer(org.omg.CosTrading.Lookup trader, String id)
    {
        try
        {
            org.omg.CosTrading.Register reg = trader.register_if();
            TEST(reg != null);

            try
            {
                reg.withdraw(id);
            }
            catch(org.omg.CosTrading.UnknownOfferId ex)
            {
                // ignore
            }
        }
        catch(Exception ex)
        {
            handleException(ex);
        }
    }

    static void
    withdrawProxies(org.omg.CosTrading.Lookup trader, java.util.Vector offers)
    {
        try
        {
            org.omg.CosTrading.Proxy proxy = trader.proxy_if();
            TEST(proxy != null);

            java.util.Enumeration e = offers.elements();
            while(e.hasMoreElements())
            {
                String id = (String)e.nextElement();
                try
                {
                    proxy.withdraw_proxy(id);
                }
                catch(org.omg.CORBA.UserException ex)
                {
                    // ignore
                }
            }
        }
        catch(Exception ex)
        {
            handleException(ex);
        }
    }

    static void
    withdrawProxy(org.omg.CosTrading.Lookup trader, String id)
    {
        try
        {
            org.omg.CosTrading.Proxy proxy = trader.proxy_if();
            TEST(proxy != null);

            try
            {
                proxy.withdraw_proxy(id);
            }
            catch(org.omg.CosTrading.UnknownOfferId ex)
            {
                // ignore
            }
        }
        catch(Exception ex)
        {
            handleException(ex);
        }
    }

    static java.util.Vector
    exportSimpleOffers(org.omg.CORBA.ORB orb,
                       org.omg.CosTrading.Lookup trader,
                       int count)
    {
        java.util.Vector result = new java.util.Vector();

        try
        {
            org.omg.CosTrading.Register reg = trader.register_if();
            TEST(reg != null);

            for(int i = 0 ; i < count ; i++)
            {
                org.omg.CosTrading.Property[] props =
                    new org.omg.CosTrading.Property[1];
                org.omg.CORBA.Any value = orb.create_any();
                value.insert_string("offer #" + (i + 1));
                props[0] = new org.omg.CosTrading.Property("name", value);

                //
                // We just use the trader's Lookup object as the object
                // for this offer
                //
                String id = reg.export(trader, SIMPLE_TYPE, props);
                result.addElement(id);
            }
        }
        catch(Exception ex)
        {
            handleException(ex);
        }

        return result;
    }

    static java.util.Vector
    exportSimpleProxies(org.omg.CORBA.ORB orb,
                        org.omg.CosTrading.Lookup trader,
                        int count)
    {
        java.util.Vector result = new java.util.Vector();

        try
        {
            org.omg.CosTrading.Proxy proxy = trader.proxy_if();
            TEST(proxy != null);

            for(int i = 0 ; i < count ; i++)
            {
                org.omg.CosTrading.Property[] props =
                    new org.omg.CosTrading.Property[1];
                org.omg.CORBA.Any value = orb.create_any();
                value.insert_string("offer #" + (i + 1));
                props[0] = new org.omg.CosTrading.Property("name", value);

                org.omg.CosTrading.Policy[] policies =
                     new org.omg.CosTrading.Policy[0];

                //
                // We just use the trader's Lookup object as the object
                // for this offer
                //
                String id = proxy.export_proxy(trader, SIMPLE_TYPE, props,
                                               true, "$*", policies);
                result.addElement(id);
            }
        }
        catch(Exception ex)
        {
            handleException(ex);
        }

        return result;
    }

    static String
    export(org.omg.CORBA.ORB orb,
           org.omg.CosTrading.Lookup trader,
           String type,
           org.omg.CosTrading.Property[] props)
    {
        String result = null;

        try
        {
            org.omg.CosTrading.Register reg = trader.register_if();
            TEST(reg != null);

            //
            // We just use the trader's Lookup object as the object
            // for this offer
            //
            result = reg.export(trader, type, props);
        }
        catch(Exception ex)
        {
            handleException(ex);
        }

        return result;
    }

    static String
    exportProxy(org.omg.CORBA.ORB orb,
                org.omg.CosTrading.Lookup trader,
                org.omg.CosTrading.Lookup target,
                String type,
                org.omg.CosTrading.Property[] props,
                boolean ifMatchAll,
                String recipe,
                org.omg.CosTrading.Policy[] policies)
    {
        String result = null;

        try
        {
            org.omg.CosTrading.Proxy proxy = trader.proxy_if();
            TEST(proxy != null);

            //
            // We just use the trader's Lookup object as the object
            // for this offer
            //
            result = proxy.export_proxy(target, type, props, ifMatchAll,
                                        recipe, policies);
        }
        catch(Exception ex)
        {
            handleException(ex);
        }

        return result;
    }
}
