// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

public class TestLink extends TestBase
{
    private org.omg.CORBA.ORB orb_;
    private org.omg.CosTrading.Lookup trader_;
    private org.omg.CosTrading.Link link_;

    TestLink(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        orb_ = orb;
        trader_ = trader;
        link_ = trader.link_if();
        TEST(link_ != null);

        try
        {
            testAdd();
            testRemove();
            testDescribe();
            testList();
            testModify();
        }
        finally
        {
        }

    }

    //
    // Test the add_link_ operation
    //
    void
    testAdd()
    {
        //
        // Save the current setting for max_link_follow_policy
        //
        org.omg.CosTrading.Admin admin = trader_.admin_if();
        TEST(admin != null);
        org.omg.CosTrading.FollowOption oldPolicy =
            admin.max_link_follow_policy();

        try
        {
            org.omg.CosTrading.FollowOption local =
                org.omg.CosTrading.FollowOption.local_only;
            org.omg.CosTrading.FollowOption ifNoLocal =
                org.omg.CosTrading.FollowOption.if_no_local;
            org.omg.CosTrading.FollowOption always =
                org.omg.CosTrading.FollowOption.always;

            admin.set_max_link_follow_policy(always);

            //
            // Test: IllegalLinkName exception
            //
            doAdd("", trader_, always, always,
                  "org.omg.CosTrading.LinkPackage.IllegalLinkName");

            //
            // Test: InvalidLookupRef exception
            //
            doAdd("TestLink", null, always, always,
                  "org.omg.CosTrading.InvalidLookupRef");

            //
            // Test: DefaultFollowTooPermissive exception
            //
            doAdd("TestLink", trader_, always, local,
                  "org.omg.CosTrading.LinkPackage.DefaultFollowTooPermissive");
            doAdd("TestLink", trader_, always, ifNoLocal,
                  "org.omg.CosTrading.LinkPackage.DefaultFollowTooPermissive");

            //
            // Test: LimitingFollowTooPermissive exception
            //
            admin.set_max_link_follow_policy(local);
            doAdd("TestLink", trader_, local, ifNoLocal,
                  "org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive");
            doAdd("TestLink", trader_, local, always,
                  "org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive");
            admin.set_max_link_follow_policy(ifNoLocal);
            doAdd("TestLink", trader_, local, always,
                  "org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive");

            //
            // Test: Add a link_
            //
            admin.set_max_link_follow_policy(always);
            doAdd("TestLink", trader_, always, always, null);

            //
            // Test: DuplicateLinkName exception
            //
            doAdd("TestLink", trader_, always, always,
                  "org.omg.CosTrading.LinkPackage.DuplicateLinkName");

            doRemove("TestLink", null);
        }
        finally
        {
            admin.set_max_link_follow_policy(oldPolicy);
        }
    }

    //
    // Test the remove_link_ operation
    //
    void
    testRemove()
    {
        //
        // Test: IllegalLinkName exception
        //
        doRemove("", "org.omg.CosTrading.LinkPackage.IllegalLinkName");

        //
        // Test: UnknownLinkName exception
        //
        doRemove("ThisLinkDoesNotExist",
                 "org.omg.CosTrading.LinkPackage.UnknownLinkName");

        //
        // Test: Use the current setting for max_link_follow_policy and
        //       add several links, then remove them in a different order
        //       and make sure they've been removed
        //
        org.omg.CosTrading.Admin admin = trader_.admin_if();
        TEST(admin != null);
        org.omg.CosTrading.FollowOption maxFollow =
            admin.max_link_follow_policy();
        doAdd("TestLink1", trader_, maxFollow, maxFollow, null);
        doAdd("TestLink2", trader_, maxFollow, maxFollow, null);
        doAdd("TestLink3", trader_, maxFollow, maxFollow, null);
        doRemove("TestLink3", null);
        doRemove("TestLink1", null);
        doRemove("TestLink2", null);
        doRemove("TestLink3",
                 "org.omg.CosTrading.LinkPackage.UnknownLinkName");
        doRemove("TestLink2",
                 "org.omg.CosTrading.LinkPackage.UnknownLinkName");
        doRemove("TestLink1",
                 "org.omg.CosTrading.LinkPackage.UnknownLinkName");
    }

    //
    // Test the describe_link_ operation
    //
    void
    testDescribe()
    {
        //
        // Save the current setting for max_link_follow_policy
        //
        org.omg.CosTrading.Admin admin = trader_.admin_if();
        TEST(admin != null);
        org.omg.CosTrading.FollowOption oldPolicy =
            admin.max_link_follow_policy();

        try
        {
            org.omg.CosTrading.FollowOption local =
                org.omg.CosTrading.FollowOption.local_only;
            org.omg.CosTrading.FollowOption ifNoLocal =
                org.omg.CosTrading.FollowOption.if_no_local;
            org.omg.CosTrading.FollowOption always =
                org.omg.CosTrading.FollowOption.always;

            admin.set_max_link_follow_policy(always);

            //
            // Test: IllegalLinkName exception
            //
            doDescribe("", "org.omg.CosTrading.LinkPackage.IllegalLinkName");

            //
            // Test: UnknownLinkName exception
            //
            doDescribe("ThisLinkDoesNotExist",
                       "org.omg.CosTrading.LinkPackage.UnknownLinkName");

            //
            // Test: Use the current setting for max_link_follow_policy and
            //       add several links, then describe them and remove them
            //
            org.omg.CosTrading.LinkPackage.LinkInfo info;
            doAdd("TestLink1", trader_, local, ifNoLocal, null);
            doAdd("TestLink2", trader_, ifNoLocal, always, null);
            doAdd("TestLink3", trader_, always, always, null);
            info = doDescribe("TestLink1", null);
            TEST(info.def_pass_on_follow_rule == local);
            TEST(info.limiting_follow_rule == ifNoLocal);
            info = doDescribe("TestLink2", null);
            TEST(info.def_pass_on_follow_rule == ifNoLocal);
            TEST(info.limiting_follow_rule == always);
            info = doDescribe("TestLink3", null);
            TEST(info.def_pass_on_follow_rule == always);
            TEST(info.limiting_follow_rule == always);
            doRemove("TestLink3", null);
            doRemove("TestLink1", null);
            doRemove("TestLink2", null);
            doDescribe("TestLink3",
                       "org.omg.CosTrading.LinkPackage.UnknownLinkName");
            doDescribe("TestLink2",
                       "org.omg.CosTrading.LinkPackage.UnknownLinkName");
            doDescribe("TestLink1",
                       "org.omg.CosTrading.LinkPackage.UnknownLinkName");
        }
        finally
        {
            admin.set_max_link_follow_policy(oldPolicy);
        }
    }

    //
    // Test the list_links operation
    //
    void
    testList()
    {
        String[] links;
        int numLinks;

        //
        // Remember how many links currently exist
        //
        links = link_.list_links();
        numLinks = links.length;

        //
        // Test: Use the current setting for max_link_follow_policy and
        //       add several links
        //
        org.omg.CosTrading.Admin admin = trader_.admin_if();
        TEST(admin != null);
        org.omg.CosTrading.FollowOption maxFollow =
            admin.max_link_follow_policy();
        doAdd("TestLink1", trader_, maxFollow, maxFollow, null);
        doAdd("TestLink2", trader_, maxFollow, maxFollow, null);
        doAdd("TestLink3", trader_, maxFollow, maxFollow, null);

        links = link_.list_links();
        TEST(links.length == numLinks + 3);

        boolean saw1 = false, saw2 = false, saw3 = false;
        for(int i = 0 ; i < links.length ; i++)
        {
            if(links[i].equals("TestLink1"))
                saw1 = true;
            else if(links[i].equals("TestLink2"))
                saw2 = true;
            else if(links[i].equals("TestLink3"))
                saw3 = true;
        }

        TEST(saw1 && saw2 && saw3);

        doRemove("TestLink3", null);
        doRemove("TestLink1", null);
        doRemove("TestLink2", null);

        links = link_.list_links();
        TEST(links.length == numLinks);
    }

    //
    // Test the modify_link_ operation
    //
    void
    testModify()
    {
        //
        // Save the current setting for max_link_follow_policy
        //
        org.omg.CosTrading.Admin admin = trader_.admin_if();
        TEST(admin != null);
        org.omg.CosTrading.FollowOption oldPolicy =
            admin.max_link_follow_policy();

        final String linkName = "TestLink";

        try
        {
            org.omg.CosTrading.FollowOption local =
                org.omg.CosTrading.FollowOption.local_only;
            org.omg.CosTrading.FollowOption ifNoLocal =
                org.omg.CosTrading.FollowOption.if_no_local;
            org.omg.CosTrading.FollowOption always =
                org.omg.CosTrading.FollowOption.always;

            admin.set_max_link_follow_policy(always);

            //
            // Test: IllegalLinkName exception
            //
            doModify("", always, always,
                     "org.omg.CosTrading.LinkPackage.IllegalLinkName");

            //
            // Test: UnknownLinkName exception
            //
            doModify("ThisLinkDoesNotExist", always, always,
                     "org.omg.CosTrading.LinkPackage.UnknownLinkName");

            //
            // Add a link_
            //
            doAdd(linkName, trader_, always, always, null);

            //
            // Test: DefaultFollowTooPermissive exception
            //
            doModify(linkName, always, local,
                "org.omg.CosTrading.LinkPackage.DefaultFollowTooPermissive");
            doModify(linkName, always, ifNoLocal,
                "org.omg.CosTrading.LinkPackage.DefaultFollowTooPermissive");

            //
            // Test: LimitingFollowTooPermissive exception
            //
            admin.set_max_link_follow_policy(local);
            doModify(linkName, local, ifNoLocal,
                "org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive");
            doModify(linkName, local, always,
                "org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive");
            admin.set_max_link_follow_policy(ifNoLocal);
            doModify(linkName, local, always,
                "org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive");

            //
            // Test: Modify a link_
            //
            admin.set_max_link_follow_policy(always);
            doModify(linkName, local, ifNoLocal, null);
            org.omg.CosTrading.LinkPackage.LinkInfo info =
                doDescribe(linkName, null);
            TEST(info.def_pass_on_follow_rule == local);
            TEST(info.limiting_follow_rule == ifNoLocal);

            doRemove(linkName, null);
        }
        finally
        {
            admin.set_max_link_follow_policy(oldPolicy);
        }
    }

    void
    doAdd(String name,
          org.omg.CosTrading.Lookup target,
          org.omg.CosTrading.FollowOption defaultFollow,
          org.omg.CosTrading.FollowOption limitingFollow,
          String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            link_.add_link(name, target, defaultFollow, limitingFollow);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }

    void
    doRemove(String name, String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            link_.remove_link(name);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }

    org.omg.CosTrading.LinkPackage.LinkInfo
    doDescribe(String name, String expectedExceptionClassName)
    {
        org.omg.CosTrading.LinkPackage.LinkInfo result = null;

        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            result = link_.describe_link(name);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }

        return result;
    }

    void
    doModify(String name,
             org.omg.CosTrading.FollowOption defaultFollow,
             org.omg.CosTrading.FollowOption limitingFollow,
             String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            link_.modify_link(name, defaultFollow, limitingFollow);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }
}
