// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

public class TestLookup extends TestBase
{
    private org.omg.CORBA.ORB orb_;
    private org.omg.CosTrading.Lookup trader_;

    TestLookup(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        orb_ = orb;
        trader_ = trader;

        try
        {
            //
            // Install the service types (if necessary)
            //
            installQueryBaseType(orb_, trader_);
            installQuerySubType(orb_, trader_);
            installSeqArrayType(orb_, trader_);

            testSimple();
            testDynamic();
            testCardinality();
            testProxy();
            testExceptions();
            testDesiredProps();
            testExactType();
            testConstraint();
            testSeqArray();
        }
        finally
        {
            //
            // Clean up any offers and service types
            //
            withdrawAllOffers(trader_, QUERY_BASE_TYPE);
            withdrawAllOffers(trader_, QUERY_SUB_TYPE);
            withdrawAllOffers(trader_, SEQ_ARRAY_TYPE);
            removeType(trader_, QUERY_SUB_TYPE);
            removeType(trader_, QUERY_BASE_TYPE);
            removeType(trader_, SEQ_ARRAY_TYPE);
        }

    }

    //
    // Test simple queries
    //
    void
    testSimple()
    {
        java.util.Vector ids = new java.util.Vector();

        try
        {
            //
            // Export some offers
            //

            final int count = 5;
            org.omg.CosTrading.Property[] props =
                new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "count";
            props[0].value = orb_.create_any();

            for(int i = 0 ; i < count ; i++)
            {
                props[0].value.insert_long(i);
                String id = export(orb_, trader_, QUERY_BASE_TYPE, props);
                ids.addElement(id);
            }

            //
            // Setup the query arguments
            //
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);
            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            //
            // Test: Using TRUE for the constraint, we should receive
            //       every offer we exported. Because we use count for
            //       the howMany argument, all the matches should be
            //       supplied in offers (i.e., no iterator should be
            //       supplied).
            //
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits, null);
            TEST(offers.value.length == count);
            TEST(iter.value == null);

            //
            // Test: Using TRUE for the constraint, we should receive
            //       every offer we exported. Because we use count - 1
            //       for the howMany argument, we should receive an
            //       iterator containing one offer.
            //
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count - 1, offers, iter, limits,
                    null);
            TEST(offers.value.length == count - 1);
            TEST(iter.value != null);
            try
            {
                TEST(iter.value.max_left() == 1);
            }
            catch(org.omg.CosTrading.UnknownMaxLeft ex)
            {
                handleException(ex);
            }
            finally
            {
                iter.value.destroy();
            }

            //
            // Test: Using "count == 0" for the constraint, we should receive
            //       only one offer.
            //
            doQuery(trader_, QUERY_BASE_TYPE, "count == 0", "first",
                    policies, desiredProps, 1, offers, iter, limits, null);
            TEST(offers.value.length == 1);
            TEST(iter.value == null);

            //
            // Test: Using "count > 0" for the constraint, we should receive
            //       count - 1 offers.
            //
            doQuery(trader_, QUERY_BASE_TYPE, "count > 0", "first",
                    policies, desiredProps, count - 1, offers, iter, limits,
                    null);
            TEST(offers.value.length == count - 1);
            TEST(iter.value == null);

            //
            // Test: Using "count < 0" for the constraint, we should receive
            //       no offers.
            //
            doQuery(trader_, QUERY_BASE_TYPE, "count < 0", "first",
                    policies, desiredProps, 1, offers, iter, limits,
                    null);
            TEST(offers.value.length == 0);
            TEST(iter.value == null);
        }
        finally
        {
            withdrawOffers(trader_, ids);
        }
    }

    //
    // Test offers with dynamic properties
    //
    void
    testDynamic()
    {
        java.util.Vector ids = new java.util.Vector();

        try
        {
            org.omg.CosTrading.Admin admin = trader_.admin_if();
            TEST(admin != null);

            //
            // Export an offer with a dynamic property - we create an
            // instance of DynPropImpl to receive the dynamic property
            // evaluation request.  We tell the object what value to
            // return, as well as what value to expect for the extra_info
            // argument.
            //

            final int countValue = 99;
            final String extraString = "Testing extra_info";

            org.omg.CORBA.Any dynValue = orb_.create_any();
            dynValue.insert_long(countValue);
            org.omg.CORBA.Any dynExtra = orb_.create_any();
            dynExtra.insert_string(extraString);
            DynPropImpl dp = new DynPropImpl("count", dynValue, dynExtra);
            orb_.connect(dp);

            org.omg.CosTrading.Property[] props =
                new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "count";
            props[0].value = orb_.create_any();
            org.omg.CosTradingDynamic.DynamicProp dynProp =
                new org.omg.CosTradingDynamic.DynamicProp();
            dynProp.eval_if = dp;
            dynProp.returned_type =
                orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
            dynProp.extra_info = orb_.create_any();
            dynProp.extra_info.insert_string(extraString);
            org.omg.CosTradingDynamic.DynamicPropHelper.insert(props[0].value,
                                                               dynProp);

            String id = export(orb_, trader_, QUERY_BASE_TYPE, props);
            ids.addElement(id);

            //
            // Setup the query arguments
            //
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);
            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            //
            // Ensure that the supports_dynamic_properties attribute
            // is set to true
            //
            boolean oldSupportsValue =
                admin.set_supports_dynamic_properties(true);

            //
            // Test: Using TRUE for the constraint, we should receive
            //       one offer. The dynamic property should have been
            //       evaluated. The value of the count property should
            //       be equal to countValue.
            //
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 2, offers, iter, limits, null);
            TEST(offers.value.length == 1);
            TEST(iter.value == null);
            TEST(dp.getAsked() == true);
            int countProp = countValue + 1;
            for(int i = 0 ; i < offers.value[0].properties.length ; i++)
            {
                if("count".equals(offers.value[0].properties[i].name))
                {
                    countProp =
                        offers.value[0].properties[i].value.extract_long();
                    break;
                }
            }
            TEST(countProp == countValue);
            dp.reset();

            //
            // Test: Setting supports_dynamic_properties to false, we should
            //       receive no offers. The dynamic property should not
            //       have been evaluated.
            //
            admin.set_supports_dynamic_properties(false);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 1, offers, iter, limits, null);
            TEST(offers.value.length == 0);
            TEST(iter.value == null);
            TEST(dp.getAsked() == false);
            dp.reset();

            //
            // Test: Setting supports_dynamic_properties to true, but
            //       setting the policy use_dynamic_properties to false,
            //       we should receive no offers. The dynamic property
            //       should not have been evaluated.
            //
            admin.set_supports_dynamic_properties(true);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("use_dynamic_properties", false);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 1, offers, iter, limits, null);
            TEST(offers.value.length == 0);
            TEST(iter.value == null);
            TEST(dp.getAsked() == false);
            dp.reset();

            //
            // Test: Setting supports_dynamic_properties to false, but
            //       setting the policy use_dynamic_properties to true,
            //       we should receive no offers. The dynamic property
            //       should not have been evaluated.  This test confirms
            //       that the trader_ will override the importer's policy.
            //
            admin.set_supports_dynamic_properties(false);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("use_dynamic_properties", true);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 1, offers, iter, limits, null);
            TEST(offers.value.length == 0);
            TEST(iter.value == null);
            TEST(dp.getAsked() == false);
            dp.reset();

            //
            // Test: Setting supports_dynamic_properties to true, but
            //       setting desiredProps to "none", we should receive
            //       one offer (with no properties). The dynamic property
            //       should not have been evaluated.
            //
            admin.set_supports_dynamic_properties(true);
            policies = new org.omg.CosTrading.Policy[0];
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.none);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 2, offers, iter, limits, null);
            TEST(offers.value.length == 1);
            TEST(offers.value[0].properties.length == 0);
            TEST(iter.value == null);
            TEST(dp.getAsked() == false);
            dp.reset();

            orb_.disconnect(dp);
            admin.set_supports_dynamic_properties(oldSupportsValue);
        }
        finally
        {
            withdrawOffers(trader_, ids);
        }
    }

    //
    // Test the cardinality limits, using the trader_'s attributes
    // and importer policies
    //
    void
    testCardinality()
    {
        java.util.Vector ids = new java.util.Vector();

        try
        {
            org.omg.CosTrading.Admin admin = trader_.admin_if();
            TEST(admin != null);

            //
            // We will export 5 offers; we initialize the trader_'s
            // cardinality attributes to offerCount + 1.
            //
            final int offerCount = 5;
            int oldDefSearchCard = admin.set_def_search_card(offerCount + 1);
            int oldMaxSearchCard = admin.set_max_search_card(offerCount + 1);
            int oldDefMatchCard = admin.set_def_match_card(offerCount + 1);
            int oldMaxMatchCard = admin.set_max_match_card(offerCount + 1);
            int oldDefReturnCard = admin.set_def_return_card(offerCount + 1);
            int oldMaxReturnCard = admin.set_max_return_card(offerCount + 1);
            int oldMaxList = admin.set_max_list(offerCount + 1);
            int oldValue;

            //
            // Export some offers
            //

            org.omg.CosTrading.Property[] props =
                new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "count";
            props[0].value = orb_.create_any();

            for(int i = 0 ; i < offerCount ; i++)
            {
                props[0].value.insert_long(i);
                String id = export(orb_, trader_, QUERY_BASE_TYPE, props);
                ids.addElement(id);
            }

            //
            // Setup the query arguments
            //
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);
            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            //
            // Test: Using TRUE for the constraint, we should receive
            //       every offer we exported. Because we use offerCount + 1
            //       for the howMany argument, all the matches should be
            //       supplied in offers (i.e., no iterator should be
            //       supplied).
            //
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, offerCount + 1, offers, iter,
                    limits, null);
            TEST(offers.value.length == offerCount);
            TEST(iter.value == null);

            //
            // Test: Using TRUE for the constraint and 1 for the search_card
            //       importer policy, we should only receive one offer.
            //
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("search_card", 1);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 2, offers, iter, limits, null);
            TEST(offers.value.length == 1);
            TEST(iter.value == null);

            //
            // Test: Using TRUE for the constraint and 1 for the match_card
            //       importer policy, we should only receive one offer.
            //
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("match_card", 1);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 2, offers, iter, limits, null);
            TEST(offers.value.length == 1);
            TEST(iter.value == null);

            //
            // Test: Using TRUE for the constraint and 1 for the return_card
            //       importer policy, we should only receive one offer.
            //
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("return_card", 1);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 2, offers, iter, limits, null);
            TEST(offers.value.length == 1);
            TEST(iter.value == null);

            //
            // Test: Setting the def_search_card attribute to 2, and using
            //       3 for the search_card importer policy, we should
            //       receive 3 offers.  This confirms that the trader_ is
            //       using our policy and not the default value.
            //
            oldValue = admin.set_def_search_card(2);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("search_card", 3);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 4, offers, iter, limits, null);
            TEST(offers.value.length == 3);
            TEST(iter.value == null);
            admin.set_def_search_card(oldValue);

            //
            // Test: Setting the max_search_card attribute to 2, and using
            //       3 for the search_card importer policy, we should
            //       receive 2 offers.  This confirms that the trader_ is
            //       overriding our policy.
            //
            oldValue = admin.set_max_search_card(2);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("search_card", 3);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 4, offers, iter, limits, null);
            TEST(offers.value.length == 2);
            TEST(iter.value == null);
            admin.set_max_search_card(oldValue);

            //
            // Test: Setting the def_match_card attribute to 2, and using
            //       3 for the match_card importer policy, we should
            //       receive 3 offers.  This confirms that the trader_ is
            //       using our policy and not the default value.
            //
            oldValue = admin.set_def_match_card(2);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("match_card", 3);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 4, offers, iter, limits, null);
            TEST(offers.value.length == 3);
            TEST(iter.value == null);
            admin.set_def_match_card(oldValue);

            //
            // Test: Setting the max_match_card attribute to 2, and using
            //       3 for the match_card importer policy, we should
            //       receive 2 offers.  This confirms that the trader_ is
            //       overriding our policy.
            //
            oldValue = admin.set_max_match_card(2);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("match_card", 3);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 4, offers, iter, limits, null);
            TEST(offers.value.length == 2);
            TEST(iter.value == null);
            admin.set_max_match_card(oldValue);

            //
            // Test: Setting the def_return_card attribute to 2, and using
            //       3 for the return_card importer policy, we should
            //       receive 3 offers.  This confirms that the trader_ is
            //       using our policy and not the default value.
            //
            oldValue = admin.set_def_return_card(2);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("return_card", 3);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 4, offers, iter, limits, null);
            TEST(offers.value.length == 3);
            TEST(iter.value == null);
            admin.set_def_return_card(oldValue);

            //
            // Test: Setting the max_return_card attribute to 2, and using
            //       3 for the return_card importer policy, we should
            //       receive 2 offers.  This confirms that the trader_ is
            //       overriding our policy.
            //
            oldValue = admin.set_max_return_card(2);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("return_card", 3);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 4, offers, iter, limits, null);
            TEST(offers.value.length == 2);
            TEST(iter.value == null);
            admin.set_max_return_card(oldValue);

            //
            // Test: Setting the max_list attribute to 2, and using 3 for
            //       the howMany argument, we should receive 2 offers and
            //       an iterator containing the third.  This confirms that
            //       the trader_ is limiting our request.
            //
            oldValue = admin.set_max_list(2);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 3, offers, iter, limits, null);
            TEST(offers.value.length == 2);
            TEST(iter.value != null);
            try
            {
                TEST(iter.value.max_left() == 1);
            }
            catch(org.omg.CosTrading.UnknownMaxLeft ex)
            {
                handleException(ex);
            }
            finally
            {
                iter.value.destroy();
            }
            admin.set_max_list(oldValue);

            //
            // Return the cardinality attributes to their original settings
            //
            admin.set_def_search_card(oldDefSearchCard);
            admin.set_max_search_card(oldMaxSearchCard);
            admin.set_def_match_card(oldDefMatchCard);
            admin.set_max_match_card(oldMaxMatchCard);
            admin.set_def_return_card(oldDefReturnCard);
            admin.set_max_return_card(oldMaxReturnCard);
            admin.set_max_list(oldMaxList);
        }
        finally
        {
            withdrawOffers(trader_, ids);
        }
    }

    //
    // Test proxy offers
    //
    void
    testProxy()
    {
        java.util.Vector ids = new java.util.Vector();

        try
        {
            org.omg.CosTrading.Admin admin = trader_.admin_if();
            TEST(admin != null);

            //
            // Ensure that the supports_proxy_offers attribute
            // is set to true
            //
            boolean oldSupportsValue = admin.set_supports_proxy_offers(true);

            //
            // Export a proxy offer - we create an instance of ProxyImpl to
            // receive the query.
            //

            ProxyImpl proxy = new ProxyImpl();
            orb_.connect(proxy);

            final int countValue = 99;

            org.omg.CosTrading.Property[] props =
                new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "count";
            props[0].value = orb_.create_any();
            props[0].value.insert_long(countValue);
            org.omg.CosTrading.Policy[] proxyPolicies =
                new org.omg.CosTrading.Policy[0];

            String id = exportProxy(orb_, trader_, proxy, QUERY_BASE_TYPE,
                                    props, false, "$*", proxyPolicies);
            ids.addElement(id);

            //
            // Setup the query arguments
            //
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);
            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            //
            // Test: Using TRUE for the constraint, the proxy offer
            //       should have been evaluated.
            //
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 10, offers, iter, limits, null);
            TEST(proxy.getAsked() == true);
            proxy.reset();

            //
            // Test: Using FALSE for the constraint, the proxy offer
            //       should not have been evaluated.
            //
            doQuery(trader_, QUERY_BASE_TYPE, "FALSE", "first",
                    policies, desiredProps, 10, offers, iter, limits, null);
            TEST(proxy.getAsked() == false);
            proxy.reset();

            //
            // Test: Using a matching constraint, the proxy offer
            //       should have been evaluated.
            //
            doQuery(trader_, QUERY_BASE_TYPE, "count == " + countValue,
                    "first", policies, desiredProps, 10, offers, iter, limits,
                    null);
            TEST(proxy.getAsked() == true);
            proxy.reset();

            //
            // Test: Setting supports_proxy_offers to false, the proxy offer
            //       should not have been evaluated.
            //
            admin.set_supports_proxy_offers(false);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 1, offers, iter, limits, null);
            TEST(proxy.getAsked() == false);
            proxy.reset();

            //
            // Test: Setting supports_proxy_offers to true, but
            //       setting the policy use_proxy_offers to false,
            //       the proxy offer should not have been evaluated.
            //
            admin.set_supports_proxy_offers(true);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("use_proxy_offers", false);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 1, offers, iter, limits, null);
            TEST(proxy.getAsked() == false);
            proxy.reset();

            //
            // Test: Setting supports_proxy_offers to false, but
            //       setting the policy use_proxy_offers to true,
            //       the proxy offer should not have been evaluated.
            //       This test confirms that the trader_ will override
            //       the importer's policy.
            //
            admin.set_supports_proxy_offers(false);
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("use_proxy_offers", true);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 1, offers, iter, limits, null);
            TEST(proxy.getAsked() == false);
            proxy.reset();
            admin.set_supports_proxy_offers(true);

            //
            // Withdraw the existing proxy, and export another one, only
            // this time we set if_match_all to true.
            //
            withdrawProxy(trader_, id);
            ids.removeElement(id);

            id = exportProxy(orb_, trader_, proxy, QUERY_BASE_TYPE,
                             props, true, "$*", proxyPolicies);
            ids.addElement(id);

            //
            // Test: Using TRUE for the constraint, the proxy offer
            //       should have been evaluated.
            //
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 10, offers, iter, limits, null);
            TEST(proxy.getAsked() == true);
            proxy.reset();

            //
            // Test: Using FALSE for the constraint, the proxy offer
            //       should have been evaluated (because if_match_all == true).
            //
            doQuery(trader_, QUERY_BASE_TYPE, "FALSE", "first",
                    policies, desiredProps, 10, offers, iter, limits, null);
            TEST(proxy.getAsked() == true);
            proxy.reset();

            orb_.disconnect(proxy);
            admin.set_supports_proxy_offers(oldSupportsValue);
        }
        finally
        {
            withdrawProxies(trader_, ids);
        }
    }

    //
    // Test miscellaneous exceptions
    //
    void
    testExceptions()
    {
        java.util.Vector ids = new java.util.Vector();

        try
        {
            //
            // Export some offers
            //

            final int count = 5;
            org.omg.CosTrading.Property[] props =
                new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "count";
            props[0].value = orb_.create_any();

            for(int i = 0 ; i < count ; i++)
            {
                props[0].value.insert_long(i);
                String id = export(orb_, trader_, QUERY_BASE_TYPE, props);
                ids.addElement(id);
            }

            //
            // Setup the query arguments
            //
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);
            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            String[] propNames;

            //
            // Test: IllegalServiceType
            //
            doQuery(trader_, "", "TRUE", "first", policies, desiredProps,
                    count, offers, iter, limits,
                    "org.omg.CosTrading.IllegalServiceType");

            //
            // Test: UnknownServiceType
            //
            doQuery(trader_, "ThisTypeShouldNotExist", "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits,
                    "org.omg.CosTrading.UnknownServiceType");

            //
            // Test: IllegalPolicyName
            //
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("", true);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits,
                    "org.omg.CosTrading.LookupPackage.IllegalPolicyName");

            //
            // Test: PolicyTypeMismatch
            //
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("search_card", true);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits,
                    "org.omg.CosTrading.LookupPackage.PolicyTypeMismatch");

            //
            // Test: DuplicatePolicyName
            //
            policies = new org.omg.CosTrading.Policy[3];
            policies[0] = getPolicy("search_card", 1);
            policies[1] = getPolicy("match_card", 2);
            policies[2] = getPolicy("search_card", 3);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits,
                    "org.omg.CosTrading.DuplicatePolicyName");

            //
            // Test: IllegalPropertyName
            //
            policies = new org.omg.CosTrading.Policy[0];
            propNames = new String[1];
            propNames[0] = "";
            desiredProps.prop_names(propNames);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits,
                    "org.omg.CosTrading.IllegalPropertyName");

            //
            // Test: DuplicatePropertyName
            //
            policies = new org.omg.CosTrading.Policy[0];
            propNames = new String[3];
            propNames[0] = "prop1";
            propNames[1] = "prop2";
            propNames[2] = "prop1";
            desiredProps.prop_names(propNames);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits,
                    "org.omg.CosTrading.DuplicatePropertyName");
        }
        finally
        {
            withdrawOffers(trader_, ids);
        }
    }

    //
    // Test desired properties
    //
    void
    testDesiredProps()
    {
        java.util.Vector ids = new java.util.Vector();

        try
        {
            //
            // Export some offers - these offers have some extra
            // properties
            //

            final int count = 5;
            org.omg.CosTrading.Property[] props =
                new org.omg.CosTrading.Property[3];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "count";
            props[0].value = orb_.create_any();
            props[1] = new org.omg.CosTrading.Property();
            props[1].name = "name";
            props[1].value = orb_.create_any();
            props[1].value.insert_string("Joe Smith");
            props[2] = new org.omg.CosTrading.Property();
            props[2].name = "age";
            props[2].value = orb_.create_any();
            props[2].value.insert_ushort((short)25);

            for(int i = 0 ; i < count ; i++)
            {
                // update the value for the "count" property
                props[0].value.insert_long(i);
                String id = export(orb_, trader_, QUERY_BASE_TYPE, props);
                ids.addElement(id);
            }

            //
            // Setup the query arguments
            //
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            String[] propNames;

            //
            // Test: All properties - each returned offer should have
            //       all three properties
            //
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits, null);
            TEST(offers.value.length == count);
            TEST(iter.value == null);
            for(int i = 0 ; i < offers.value.length ; i++)
                TEST(offers.value[i].properties.length == 3);

            //
            // Test: No properties - each returned offer should have
            //       no properties
            //
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.none);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits, null);
            TEST(offers.value.length == count);
            TEST(iter.value == null);
            for(int i = 0 ; i < offers.value.length ; i++)
                TEST(offers.value[i].properties.length == 0);

            //
            // Test: Some properties - each returned offer should have
            //       only the properties we requested
            //
            propNames = new String[2];
            propNames[0] = "name";
            propNames[1] = "age";
            desiredProps.prop_names(propNames);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, count, offers, iter, limits, null);
            TEST(offers.value.length == count);
            TEST(iter.value == null);
            for(int i = 0 ; i < offers.value.length ; i++)
            {
                TEST(offers.value[i].properties.length == 2);
                boolean sawName = false, sawAge = false;
                for(int n = 0 ; n < offers.value[i].properties.length ; n++)
                {
                    if(offers.value[i].properties[n].name.equals("name"))
                        sawName = true;
                    else if(offers.value[i].properties[n].name.equals("age"))
                        sawAge = true;
                }

                TEST(sawName && sawAge);
            }
        }
        finally
        {
            withdrawOffers(trader_, ids);
        }
    }

    //
    // Test exact type match
    //
    void
    testExactType()
    {
        java.util.Vector ids = new java.util.Vector();

        try
        {
            //
            // Export some offers for QUERY_BASE_TYPE and some
            // for QUERY_SUB_TYPE
            //

            final int baseCount = 2;
            final int subCount = 3;

            org.omg.CosTrading.Property[] props =
                new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "count";
            props[0].value = orb_.create_any();

            for(int i = 0 ; i < baseCount ; i++)
            {
                // update the value for the "count" property
                props[0].value.insert_long(i);
                String id = export(orb_, trader_, QUERY_BASE_TYPE, props);
                ids.addElement(id);
            }

            props = new org.omg.CosTrading.Property[2];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "count";
            props[0].value = orb_.create_any();
            props[1] = new org.omg.CosTrading.Property();
            props[1].name = "weight";
            props[1].value = orb_.create_any();

            for(int i = 0 ; i < subCount ; i++)
            {
                // update the values for the properties
                props[0].value.insert_long(i);
                props[1].value.insert_double(i);
                String id = export(orb_, trader_, QUERY_SUB_TYPE, props);
                ids.addElement(id);
            }

            //
            // Setup the query arguments
            //
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);
            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            //
            // Test: If the exact_type_match policy is not supplied,
            //       we should receive (baseCount + subCount) offers
            //       when using QUERY_BASE_TYPE
            //
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, baseCount + subCount, offers,
                    iter, limits, null);
            TEST(offers.value.length == baseCount + subCount);
            TEST(iter.value == null);

            //
            // Test: If the exact_type_match policy is not supplied,
            //       we should receive subCount offers when using
            //       QUERY_SUB_TYPE
            //
            doQuery(trader_, QUERY_SUB_TYPE, "TRUE", "first",
                    policies, desiredProps, subCount, offers,
                    iter, limits, null);
            TEST(offers.value.length == subCount);
            TEST(iter.value == null);

            //
            // Test: If the exact_type_match policy is false,
            //       we should receive (baseCount + subCount) offers
            //       when using QUERY_BASE_TYPE
            //
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("exact_type_match", false);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, baseCount + subCount, offers,
                    iter, limits, null);
            TEST(offers.value.length == baseCount + subCount);
            TEST(iter.value == null);

            //
            // Test: If the exact_type_match policy is true,
            //       we should receive baseCount offers
            //       when using QUERY_BASE_TYPE
            //
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("exact_type_match", true);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, baseCount + subCount, offers,
                    iter, limits, null);
            TEST(offers.value.length == baseCount);
            TEST(iter.value == null);

            //
            // Test: If the exact_type_match policy is true,
            //       we should receive subCount offers
            //       when using QUERY_SUB_TYPE
            //
            policies = new org.omg.CosTrading.Policy[1];
            policies[0] = getPolicy("exact_type_match", true);
            doQuery(trader_, QUERY_SUB_TYPE, "TRUE", "first",
                    policies, desiredProps, subCount, offers,
                    iter, limits, null);
            TEST(offers.value.length == subCount);
            TEST(iter.value == null);
        }
        finally
        {
            withdrawOffers(trader_, ids);
        }
    }

    //
    // Test constraint expressions
    //
    void
    testConstraint()
    {
        java.util.Vector ids = new java.util.Vector();

        try
        {
            //
            // Setup the query arguments
            //
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);
            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            //
            // Test: Language identifier
            //
            doQuery(trader_, QUERY_BASE_TYPE, "<<OMG 1.0>>TRUE",
                    "first", policies, desiredProps, 0, offers, iter, limits,
                    null);
            doQuery(trader_, QUERY_BASE_TYPE, "<<Bad 1.2>>TRUE",
                    "first", policies, desiredProps, 0, offers, iter, limits,
                    "org.omg.CosTrading.IllegalConstraint");

            //
            // Test: Boolean literals
            //
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE", "first",
                    policies, desiredProps, 0, offers, iter, limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "FALSE", "first",
                    policies, desiredProps, 0, offers, iter, limits, null);

            //
            // Test: String and numeric literals should fail (a constraint
            //       must be a boolean expression)
            //
            doQuery(trader_, QUERY_BASE_TYPE, "'ABC'", "first",
                    policies, desiredProps, 0, offers, iter, limits,
                    "org.omg.CosTrading.IllegalConstraint");
            doQuery(trader_, QUERY_BASE_TYPE, "5.0", "first",
                    policies, desiredProps, 0, offers, iter, limits,
                    "org.omg.CosTrading.IllegalConstraint");

            //
            // Test: Boolean connectives
            //
            doQuery(trader_, QUERY_BASE_TYPE, "not TRUE", "first",
                    policies, desiredProps, 0, offers, iter, limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "not FALSE", "first",
                    policies, desiredProps, 0, offers, iter, limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE and TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "TRUE and FALSE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "FALSE or FALSE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "FALSE or TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "FALSE or not TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "not FALSE and TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            doQuery(trader_, QUERY_BASE_TYPE,
                    "not FALSE and not TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            doQuery(trader_, QUERY_BASE_TYPE,
                    "not FALSE or not TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "not (not TRUE)",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            doQuery(trader_, QUERY_BASE_TYPE, "and TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, "org.omg.CosTrading.IllegalConstraint");
            doQuery(trader_, QUERY_BASE_TYPE, "or TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, "org.omg.CosTrading.IllegalConstraint");
            doQuery(trader_, QUERY_BASE_TYPE, "not 'ABC'",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, "org.omg.CosTrading.IllegalConstraint");
            doQuery(trader_, QUERY_BASE_TYPE, "not 1.3",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, "org.omg.CosTrading.IllegalConstraint");
            doQuery(trader_, QUERY_BASE_TYPE, "not and",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, "org.omg.CosTrading.IllegalConstraint");
            doQuery(trader_, QUERY_BASE_TYPE, "not or",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, "org.omg.CosTrading.IllegalConstraint");
            doQuery(trader_, QUERY_BASE_TYPE, "not not",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, "org.omg.CosTrading.IllegalConstraint");
            doQuery(trader_, QUERY_BASE_TYPE, "not or TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, "org.omg.CosTrading.IllegalConstraint");
            doQuery(trader_, QUERY_BASE_TYPE, "not and TRUE",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, "org.omg.CosTrading.IllegalConstraint");
        }
        finally
        {
            withdrawOffers(trader_, ids);
        }
    }

    //
    // Test sequence/array handling
    //
    void
    testSeqArray()
    {
        java.util.Vector ids = new java.util.Vector();

        try
        {
            //
            // Export some offers
            //

            final int count = 1;
            org.omg.CosTrading.Property[] props =
                new org.omg.CosTrading.Property[2];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "short_arr";
            props[1] = new org.omg.CosTrading.Property();
            props[1].name = "string_arr";

            for(int i = 0 ; i < count ; i++)
            {
                org.omg.CORBA.TypeCode tc, content;

                content = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_short);
                tc = orb_.create_array_tc(5, content);
                org.omg.CORBA.DynArray da = orb_.create_dyn_array(tc);
                da.insert_short((short)0);
                da.insert_short((short)1);
                da.insert_short((short)2);
                da.insert_short((short)3);
                da.insert_short((short)4);
                props[0].value = da.to_any();

                content =
                    orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
                tc = orb_.create_sequence_tc(0, content);
                org.omg.CORBA.DynSequence ds = orb_.create_dyn_sequence(tc);
                ds.length(2);
                ds.insert_string("string 0");
                ds.insert_string("string 1");
                props[1].value = ds.to_any();

                String id = export(orb_, trader_, SEQ_ARRAY_TYPE, props);
                ids.addElement(id);
            }

            //
            // Setup the query arguments
            //
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps =
                new org.omg.CosTrading.LookupPackage.SpecifiedProps();
            desiredProps._default(
                org.omg.CosTrading.LookupPackage.HowManyProps.all);
            org.omg.CosTrading.OfferSeqHolder offers =
                new org.omg.CosTrading.OfferSeqHolder();
            org.omg.CosTrading.OfferIteratorHolder iter =
                new org.omg.CosTrading.OfferIteratorHolder();
            org.omg.CosTrading.PolicyNameSeqHolder limits =
                new org.omg.CosTrading.PolicyNameSeqHolder();

            //
            // Test: exists
            //
            doQuery(trader_, SEQ_ARRAY_TYPE,
                    "exist short_arr and exist string_arr",
                    "first", policies, desiredProps, count, offers, iter,
                    limits, null);
            TEST(offers.value.length == count);
            TEST(iter.value == null);

            //
            // Test: in with sequence
            //
            doQuery(trader_, SEQ_ARRAY_TYPE, "'string 0' in string_arr",
                    "first", policies, desiredProps, count, offers, iter,
                    limits, null);
            TEST(offers.value.length == count);
            TEST(iter.value == null);

            //
            // Test: in with array
            //
            doQuery(trader_, SEQ_ARRAY_TYPE, "3 in short_arr",
                    "first", policies, desiredProps, count, offers, iter,
                    limits, null);
            TEST(offers.value.length == count);
            TEST(iter.value == null);

            //
            // Test: in with sequence failure
            //
            doQuery(trader_, SEQ_ARRAY_TYPE, "'string 99' in string_arr",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            TEST(offers.value.length == 0);
            TEST(iter.value == null);

            //
            // Test: in with array failure
            //
            doQuery(trader_, SEQ_ARRAY_TYPE, "99 in short_arr",
                    "first", policies, desiredProps, 0, offers, iter,
                    limits, null);
            TEST(offers.value.length == 0);
            TEST(iter.value == null);
        }
        catch(org.omg.CORBA.DynAnyPackage.InvalidValue ex)
        {
            handleException(ex);
        }
        catch(org.omg.CORBA.DynAnyPackage.Invalid ex)
        {
            handleException(ex);
        }
        catch(org.omg.CORBA.ORBPackage.InconsistentTypeCode ex)
        {
            handleException(ex);
        }
        finally
        {
            withdrawOffers(trader_, ids);
        }
    }

    static void
    doQuery(org.omg.CosTrading.Lookup lookup,
            String type,
            String constraint,
            String pref,
            org.omg.CosTrading.Policy[] policies,
            org.omg.CosTrading.LookupPackage.SpecifiedProps desiredProps,
            int howMany,
            org.omg.CosTrading.OfferSeqHolder offers,
            org.omg.CosTrading.OfferIteratorHolder iter,
            org.omg.CosTrading.PolicyNameSeqHolder limits,
            String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            lookup.query(type, constraint, pref, policies, desiredProps,
                         howMany, offers, iter, limits);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }

    static org.omg.CosTrading.Policy
    getPolicy(String name, boolean value)
    {
        org.omg.CosTrading.Policy result = new org.omg.CosTrading.Policy();
        result.name = name;
        result.value = org.omg.CORBA.ORB.init().create_any();
        result.value.insert_boolean(value);
        return result;
    }

    static org.omg.CosTrading.Policy
    getPolicy(String name, int value)
    {
        org.omg.CosTrading.Policy result = new org.omg.CosTrading.Policy();
        result.name = name;
        result.value = org.omg.CORBA.ORB.init().create_any();
        result.value.insert_ulong(value);
        return result;
    }
}
