// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

//
// TestProxy uses the REGISTER_TYPE service type, which has
// the following properties:
//
//   string prop1
//   mandatory float prop2
//   readonly char prop3
//   mandatory readonly unsigned short prop4
//
public class TestProxy extends TestBase
{
    private org.omg.CORBA.ORB orb_;
    private org.omg.CosTrading.Lookup trader_;
    private org.omg.CosTrading.Proxy proxy_;

    TestProxy(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        orb_ = orb;
        trader_ = trader;
        proxy_ = trader.proxy_if();
        TEST(proxy_ != null);

        try
        {
            //
            // Install the service type (if necessary)
            //
            installRegisterType(orb_, trader_);

            testExport();
            testWithdraw();
            testDescribe();
        }
        finally
        {
            //
            // Clean up any service types
            //
            removeType(trader_, REGISTER_TYPE);
        }

    }

    //
    // Test the export operation
    //
    void
    testExport()
    {
        String id;
        org.omg.CosTrading.Property[] props;
        org.omg.CosTrading.Property prop1, prop2, prop3, prop4;
        org.omg.CosTrading.Policy[] policies;
        org.omg.CosTradingDynamic.DynamicProp dynProp;

        prop1 = new org.omg.CosTrading.Property();
        prop1.name = "prop1";
        prop1.value = orb_.create_any();
        prop1.value.insert_string("Hi there");

        prop2 = new org.omg.CosTrading.Property();
        prop2.name = "prop2";
        prop2.value = orb_.create_any();
        prop2.value.insert_float((float)1.99);

        prop3 = new org.omg.CosTrading.Property();
        prop3.name = "prop3";
        prop3.value = orb_.create_any();
        prop3.value.insert_char('Z');

        prop4 = new org.omg.CosTrading.Property();
        prop4.name = "prop4";
        prop4.value = orb_.create_any();
        prop4.value.insert_ushort((short)5000);

        //
        // Test: IllegalServiceType exception
        //
        props = new org.omg.CosTrading.Property[0];
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, "", props, false, "$*", policies,
                 "org.omg.CosTrading.IllegalServiceType");

        //
        // Test: UnknownServiceType exception
        //
        props = new org.omg.CosTrading.Property[0];
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, "ThisTypeDoesNotExist", props, false, "$*",
                 policies, "org.omg.CosTrading.UnknownServiceType");

        //
        // Test: InvalidLookupRef exception
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        doExport(null, REGISTER_TYPE, props, false, "$*",
                 policies, "org.omg.CosTrading.InvalidLookupRef");

        //
        // Test: IllegalPropertyName exception
        //
        props = new org.omg.CosTrading.Property[3];
        props[0] = prop2;
        props[1] = prop4;
        props[2] = new org.omg.CosTrading.Property();
        props[2].name = "<foo-bar";
        props[2].value = orb_.create_any();
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, REGISTER_TYPE, props, false, "$*",
                 policies, "org.omg.CosTrading.IllegalPropertyName");

        //
        // Test: DuplicatePropertyName exception
        //
        props = new org.omg.CosTrading.Property[3];
        props[0] = prop2;
        props[1] = prop4;
        props[2] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, REGISTER_TYPE, props, false, "$*",
                 policies, "org.omg.CosTrading.DuplicatePropertyName");

        //
        // Test: DuplicatePropertyName exception (using properties
        //       not defined in the service type)
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = new org.omg.CosTrading.Property();
        props[0].name = "dup_prop";
        props[0].value = orb_.create_any();
        props[1] = new org.omg.CosTrading.Property();
        props[1].name = "dup_prop";
        props[1].value = orb_.create_any();
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, REGISTER_TYPE, props, false, "$*",
                 policies, "org.omg.CosTrading.DuplicatePropertyName");

        //
        // Test: DuplicatePolicyName exception
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[3];
        policies[0] = new org.omg.CosTrading.Policy("policy1", prop1.value);
        policies[1] = new org.omg.CosTrading.Policy("policy2", prop2.value);
        policies[2] = new org.omg.CosTrading.Policy("policy1", prop3.value);
        doExport(trader_, REGISTER_TYPE, props, false, "$*",
                 policies, "org.omg.CosTrading.DuplicatePolicyName");

        //
        // Test: IllegalRecipe exception - cannot refer to missing properties
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, REGISTER_TYPE, props, false,
                 "$(prop2) lessthan $(badprop)", policies,
                 "org.omg.CosTrading.ProxyPackage.IllegalRecipe");

        //
        // Test: IllegalRecipe exception - missing closing paren
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, REGISTER_TYPE, props, false,
                 "$(prop2) lessthan $(prop4", policies,
                 "org.omg.CosTrading.ProxyPackage.IllegalRecipe");

        //
        // Test: IllegalRecipe exception - invalid language identifier
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, REGISTER_TYPE, props, false,
                 "<<Bad 5.4>>$*", policies,
                 "org.omg.CosTrading.ProxyPackage.IllegalRecipe");

        //
        // Test: Export a proxy offer with only the mandatory properties.
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        id = doExport(trader_, REGISTER_TYPE, props, false,
                      "$*", policies, null);
        withdrawProxy(trader_, id);

        //
        // Test: MissingMandatoryProperty exception
        //
        props = new org.omg.CosTrading.Property[1];
        props[0] = prop2;
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, REGISTER_TYPE, props, false, "$*",
                 policies, "org.omg.CosTrading.MissingMandatoryProperty");

        //
        // Test: PropertyTypeMismatch exception - use an incorrect
        //       value type for "prop3"
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = new org.omg.CosTrading.Property();
        props[1].name = "prop3";
        props[1].value = orb_.create_any();
        props[1].value.insert_double(7.7);
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, REGISTER_TYPE, props, false, "$*",
                 policies, "org.omg.CosTrading.PropertyTypeMismatch");

        //
        // Test: ReadonlyDynamicProperty exception
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = new org.omg.CosTrading.Property();
        props[1].name = "prop4";
        props[1].value = orb_.create_any();
        DynPropImpl dp = new DynPropImpl(null, null, null);
        dynProp = new org.omg.CosTradingDynamic.DynamicProp();
        dynProp.eval_if = dp;
        dynProp.returned_type =
            orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_ushort);
        dynProp.extra_info = orb_.create_any();
        org.omg.CosTradingDynamic.DynamicPropHelper.insert(props[1].value,
                                                           dynProp);
        policies = new org.omg.CosTrading.Policy[0];
        doExport(trader_, REGISTER_TYPE, props, false, "$*",
                 policies, "org.omg.CosTrading.ReadonlyDynamicProperty");

        //
        // Test: Export an offer with a dynamic property.
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = new org.omg.CosTrading.Property();
        props[0].name = "prop2";
        props[0].value = orb_.create_any();
        dynProp = new org.omg.CosTradingDynamic.DynamicProp();
        dynProp.eval_if = dp;
        dynProp.returned_type =
            orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
        dynProp.extra_info = orb_.create_any();
        org.omg.CosTradingDynamic.DynamicPropHelper.insert(props[0].value,
                                                           dynProp);
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        id = doExport(trader_, REGISTER_TYPE, props, false,
                      "$*", policies, null);
        withdrawProxy(trader_, id);
    }

    //
    // Test the withdraw operation
    //
    void
    testWithdraw()
    {
        String id;
        org.omg.CosTrading.Property[] props;
        org.omg.CosTrading.Property prop2, prop4;
        org.omg.CosTrading.Policy[] policies;

        prop2 = new org.omg.CosTrading.Property();
        prop2.name = "prop2";
        prop2.value = orb_.create_any();
        prop2.value.insert_float((float)1.99);

        prop4 = new org.omg.CosTrading.Property();
        prop4.name = "prop4";
        prop4.value = orb_.create_any();
        prop4.value.insert_ushort((short)5000);

        //
        // Test: IllegalOfferId exception
        //
        doWithdraw("", "org.omg.CosTrading.IllegalOfferId");

        //
        // Test: UnknownOfferId exception - try to withdraw the same proxy
        //       offer twice
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        id = exportProxy(orb_, trader_, trader_, REGISTER_TYPE,
                         props, false, "$*", policies);
        doWithdraw(id, null);
        doWithdraw(id, "org.omg.CosTrading.UnknownOfferId");

        //
        // Test: NotProxyOfferId exception - export a regular offer
        //       and try to withdraw it as a proxy offer
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        id = export(orb_, trader_, REGISTER_TYPE, props);
        doWithdraw(id, "org.omg.CosTrading.ProxyPackage.NotProxyOfferId");
        withdrawOffer(trader_, id);
    }

    //
    // Test the describe operation
    //
    void
    testDescribe()
    {
        String id;
        org.omg.CosTrading.Property[] props;
        org.omg.CosTrading.Property prop1, prop2, prop3, prop4;
        org.omg.CosTrading.Policy[] policies;

        prop1 = new org.omg.CosTrading.Property();
        prop1.name = "prop1";
        prop1.value = orb_.create_any();
        prop1.value.insert_string("Hi there");

        prop2 = new org.omg.CosTrading.Property();
        prop2.name = "prop2";
        prop2.value = orb_.create_any();
        prop2.value.insert_float((float)1.99);

        prop3 = new org.omg.CosTrading.Property();
        prop3.name = "prop3";
        prop3.value = orb_.create_any();
        prop3.value.insert_char('Z');

        prop4 = new org.omg.CosTrading.Property();
        prop4.name = "prop4";
        prop4.value = orb_.create_any();
        prop4.value.insert_ushort((short)5000);

        //
        // Test: IllegalOfferId exception
        //
        doDescribe("", "org.omg.CosTrading.IllegalOfferId");

        //
        // Test: UnknownOfferId exception - try to describe a proxy offer
        //       that has already been withdrawn
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        policies = new org.omg.CosTrading.Policy[0];
        id = exportProxy(orb_, trader_, trader_, REGISTER_TYPE,
                         props, false, "$*", policies);
        withdrawProxy(trader_, id);
        doDescribe(id, "org.omg.CosTrading.UnknownOfferId");

        //
        // Test: NotProxyOfferId exception - export a regular offer
        //       and try to describe it as a proxy offer
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        id = export(orb_, trader_, REGISTER_TYPE, props);
        doDescribe(id, "org.omg.CosTrading.ProxyPackage.NotProxyOfferId");
        withdrawOffer(trader_, id);

        //
        // Test: Export a proxy offer with all the properties, then describe
        //       it and make sure things match.  We define policies having
        //       the same values as the properties.
        //
        props = new org.omg.CosTrading.Property[4];
        props[0] = prop1;
        props[1] = prop2;
        props[2] = prop3;
        props[3] = prop4;
        policies = new org.omg.CosTrading.Policy[4];
        policies[0] = new org.omg.CosTrading.Policy("policy1", prop1.value);
        policies[1] = new org.omg.CosTrading.Policy("policy2", prop2.value);
        policies[2] = new org.omg.CosTrading.Policy("policy3", prop3.value);
        policies[3] = new org.omg.CosTrading.Policy("policy4", prop4.value);
        id = exportProxy(orb_, trader_, trader_, REGISTER_TYPE,
                         props, false, "$*", policies);
        org.omg.CosTrading.ProxyPackage.ProxyInfo info = doDescribe(id, null);
        TEST(info.type.equals(REGISTER_TYPE));
        TEST(info.if_match_all == false);
        TEST(info.recipe.equals("$*"));
        TEST(info.properties.length == 4);
        for(int i = 0 ; i < info.properties.length ; i++)
        {
            if(info.properties[i].name.equals("prop1"))
                TEST(prop1.value.equal(info.properties[i].value));
            else if(info.properties[i].name.equals("prop2"))
                TEST(prop2.value.equal(info.properties[i].value));
            else if(info.properties[i].name.equals("prop3"))
                TEST(prop3.value.equal(info.properties[i].value));
            else if(info.properties[i].name.equals("prop4"))
                TEST(prop4.value.equal(info.properties[i].value));
        }
        TEST(info.policies_to_pass_on.length == 4);
        for(int i = 0 ; i < info.policies_to_pass_on.length ; i++)
        {
            org.omg.CosTrading.Policy p = info.policies_to_pass_on[i];
            if(p.name.equals("policy1"))
                TEST(policies[0].value.equal(p.value));
            else if(p.name.equals("policy2"))
                TEST(policies[1].value.equal(p.value));
            else if(p.name.equals("policy3"))
                TEST(policies[2].value.equal(p.value));
            else if(p.name.equals("policy4"))
                TEST(policies[3].value.equal(p.value));
        }
        withdrawProxy(trader_, id);
    }

    String
    doExport(org.omg.CosTrading.Lookup target,
             String type,
             org.omg.CosTrading.Property[] props,
             boolean ifMatchAll,
             String recipe,
             org.omg.CosTrading.Policy[] policies,
             String expectedExceptionClassName)
    {
        String result = null;

        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            result = proxy_.export_proxy(target, type, props, ifMatchAll,
                                        recipe, policies);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }

        return result;
    }

    void
    doWithdraw(String id, String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            proxy_.withdraw_proxy(id);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }

    org.omg.CosTrading.ProxyPackage.ProxyInfo
    doDescribe(String id, String expectedExceptionClassName)
    {
        org.omg.CosTrading.ProxyPackage.ProxyInfo result = null;

        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            result = proxy_.describe_proxy(id);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }

        return result;
    }
}
