// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

//
// TestRegister uses the REGISTER_TYPE service type, which has
// the following properties:
//
//   string prop1
//   mandatory float prop2
//   readonly char prop3
//   mandatory readonly unsigned short prop4
//
public class TestRegister extends TestBase
{
    private org.omg.CORBA.ORB orb_;
    private org.omg.CosTrading.Lookup trader_;
    private org.omg.CosTrading.Register reg_;

    TestRegister(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        orb_ = orb;
        trader_ = trader;
        reg_ = trader.register_if();
        TEST(reg_ != null);

        try
        {
            //
            // Install the service type (if necessary)
            //
            installRegisterType(orb_, trader_);

            testExport();
            testWithdraw();
            testDescribe();
            testModify();
            testWithdrawConstraint();
        }
        finally
        {
            //
            // Clean up any offers and service types
            //
            withdrawAllOffers(trader_, REGISTER_TYPE);
            removeType(trader_, REGISTER_TYPE);
        }

    }

    //
    // Test the export operation
    //
    void
    testExport()
    {
        String id;
        org.omg.CosTrading.Property[] props;
        org.omg.CosTrading.Property prop1, prop2, prop3, prop4;
        org.omg.CosTradingDynamic.DynamicProp dynProp;

        prop1 = new org.omg.CosTrading.Property();
        prop1.name = "prop1";
        prop1.value = orb_.create_any();
        prop1.value.insert_string("Hi there");

        prop2 = new org.omg.CosTrading.Property();
        prop2.name = "prop2";
        prop2.value = orb_.create_any();
        prop2.value.insert_float((float)1.99);

        prop3 = new org.omg.CosTrading.Property();
        prop3.name = "prop3";
        prop3.value = orb_.create_any();
        prop3.value.insert_char('Z');

        prop4 = new org.omg.CosTrading.Property();
        prop4.name = "prop4";
        prop4.value = orb_.create_any();
        prop4.value.insert_ushort((short)5000);

        //
        // Test: IllegalServiceType exception
        //
        props = new org.omg.CosTrading.Property[0];
        doExport(trader_, "", props,
                 "org.omg.CosTrading.IllegalServiceType", true);

        //
        // Test: UnknownServiceType exception
        //
        props = new org.omg.CosTrading.Property[0];
        doExport(trader_, "ThisTypeDoesNotExist", props,
                 "org.omg.CosTrading.UnknownServiceType", true);

        //
        // Test: IllegalPropertyName exception
        //
        props = new org.omg.CosTrading.Property[3];
        props[0] = prop2;
        props[1] = prop4;
        props[2] = new org.omg.CosTrading.Property();
        props[2].name = "<foo-bar";
        props[2].value = orb_.create_any();
        doExport(trader_, REGISTER_TYPE, props,
                 "org.omg.CosTrading.IllegalPropertyName", true);

        //
        // Test: DuplicatePropertyName exception
        //
        props = new org.omg.CosTrading.Property[3];
        props[0] = prop2;
        props[1] = prop4;
        props[2] = prop4;
        doExport(trader_, REGISTER_TYPE, props,
                 "org.omg.CosTrading.DuplicatePropertyName", true);

        //
        // Test: DuplicatePropertyName exception (using properties
        //       not defined in the service type)
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = new org.omg.CosTrading.Property();
        props[0].name = "dup_prop";
        props[0].value = orb_.create_any();
        props[1] = new org.omg.CosTrading.Property();
        props[1].name = "dup_prop";
        props[1].value = orb_.create_any();
        doExport(trader_, REGISTER_TYPE, props,
                 "org.omg.CosTrading.DuplicatePropertyName", true);

        //
        // Test: Export an offer with only the mandatory properties.
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        id = doExport(trader_, REGISTER_TYPE, props, null, true);
        withdrawOffer(trader_, id);

        //
        // Test: Export an offer with all the properties, then describe
        //       the offer and make sure things match.
        //
        props = new org.omg.CosTrading.Property[4];
        props[0] = prop1;
        props[1] = prop2;
        props[2] = prop3;
        props[3] = prop4;
        id = doExport(trader_, REGISTER_TYPE, props, null, true);
        org.omg.CosTrading.RegisterPackage.OfferInfo info =
            doDescribe(id, null);
        TEST(info.type.equals(REGISTER_TYPE));
        for(int i = 0 ; i < info.properties.length ; i++)
        {
            if(info.properties[i].name.equals("prop1"))
                TEST(prop1.value.equal(info.properties[i].value));
            else if(info.properties[i].name.equals("prop2"))
                TEST(prop2.value.equal(info.properties[i].value));
            else if(info.properties[i].name.equals("prop3"))
                TEST(prop3.value.equal(info.properties[i].value));
            else if(info.properties[i].name.equals("prop4"))
                TEST(prop4.value.equal(info.properties[i].value));
        }
        withdrawOffer(trader_, id);

        //
        // Test: MissingMandatoryProperty exception
        //
        props = new org.omg.CosTrading.Property[1];
        props[0] = prop2;
        doExport(trader_, REGISTER_TYPE, props,
                 "org.omg.CosTrading.MissingMandatoryProperty", true);

        //
        // Test: PropertyTypeMismatch exception - use an incorrect
        //       value type for "prop3"
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = new org.omg.CosTrading.Property();
        props[1].name = "prop3";
        props[1].value = orb_.create_any();
        props[1].value.insert_double(7.7);
        doExport(trader_, REGISTER_TYPE, props,
                 "org.omg.CosTrading.PropertyTypeMismatch", true);

        //
        // Test: ReadonlyDynamicProperty exception
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = new org.omg.CosTrading.Property();
        props[1].name = "prop4";
        props[1].value = orb_.create_any();
        DynPropImpl dp = new DynPropImpl(null, null, null);
        dynProp = new org.omg.CosTradingDynamic.DynamicProp();
        dynProp.eval_if = dp;
        dynProp.returned_type =
            orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_ushort);
        dynProp.extra_info = orb_.create_any();
        org.omg.CosTradingDynamic.DynamicPropHelper.insert(props[1].value,
                                                           dynProp);
        doExport(trader_, REGISTER_TYPE, props,
                 "org.omg.CosTrading.ReadonlyDynamicProperty", true);

        //
        // Test: PropertyTypeMismatch exception - using an
        //       incorrect value type for the dynamic property
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = new org.omg.CosTrading.Property();
        props[0].name = "prop2";
        props[0].value = orb_.create_any();
        dynProp = new org.omg.CosTradingDynamic.DynamicProp();
        dynProp.eval_if = dp;
        dynProp.returned_type =
            orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_double);
        dynProp.extra_info = orb_.create_any();
        org.omg.CosTradingDynamic.DynamicPropHelper.insert(props[0].value,
                                                           dynProp);
        props[1] = prop4;
        doExport(trader_, REGISTER_TYPE, props,
                 "org.omg.CosTrading.PropertyTypeMismatch", true);

        //
        // Test: PropertyTypeMismatch exception - using a nil value
        //       for the dynamic property's eval_if member
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = new org.omg.CosTrading.Property();
        props[0].name = "prop2";
        props[0].value = orb_.create_any();
        dynProp = new org.omg.CosTradingDynamic.DynamicProp();
        dynProp.eval_if = null;
        dynProp.returned_type =
            orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
        dynProp.extra_info = orb_.create_any();
        org.omg.CosTradingDynamic.DynamicPropHelper.insert(props[0].value,
                                                           dynProp);
        props[1] = prop4;
        doExport(trader_, REGISTER_TYPE, props,
                 "org.omg.CosTrading.PropertyTypeMismatch", true);

        //
        // Test: Export an offer with a dynamic property.
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = new org.omg.CosTrading.Property();
        props[0].name = "prop2";
        props[0].value = orb_.create_any();
        dynProp = new org.omg.CosTradingDynamic.DynamicProp();
        dynProp.eval_if = dp;
        dynProp.returned_type =
            orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
        dynProp.extra_info = orb_.create_any();
        org.omg.CosTradingDynamic.DynamicPropHelper.insert(props[0].value,
                                                           dynProp);
        props[1] = prop4;
        id = doExport(trader_, REGISTER_TYPE, props, null, true);
        withdrawOffer(trader_, id);

        //
        // Test: InterfaceTypeMismatch exception - if the server has
        //       access to the Interface Repository, and the server is
        //       configured to use it, then this attempt should raise
        //       InterfaceTypeMismatch because the service type specifies
        //       the Lookup interface for offers, but we're using the
        //       Register interface, which does not "conform".
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        id = doExport(reg_, REGISTER_TYPE, props,
            "org.omg.CosTrading.RegisterPackage.InterfaceTypeMismatch", false);
        if(id != null)
            withdrawOffer(trader_, id);
    }

    //
    // Test the withdraw operation
    //
    void
    testWithdraw()
    {
        String id;
        org.omg.CosTrading.Property[] props;
        org.omg.CosTrading.Property prop2, prop4;

        prop2 = new org.omg.CosTrading.Property();
        prop2.name = "prop2";
        prop2.value = orb_.create_any();
        prop2.value.insert_float((float)1.99);

        prop4 = new org.omg.CosTrading.Property();
        prop4.name = "prop4";
        prop4.value = orb_.create_any();
        prop4.value.insert_ushort((short)5000);

        //
        // Test: IllegalOfferId exception
        //
        doWithdraw("", "org.omg.CosTrading.IllegalOfferId");

        //
        // Test: UnknownOfferId exception - try to withdraw the offer
        //       that was already withdrawn in the NotImplemented test above
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        id = export(orb_, trader_, REGISTER_TYPE, props);
        doWithdraw(id, null);
        doWithdraw(id, "org.omg.CosTrading.UnknownOfferId");

        //
        // Test: ProxyOfferId exception
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        org.omg.CosTrading.Policy[] policies =
            new org.omg.CosTrading.Policy[0];
        id = exportProxy(orb_, trader_, trader_, REGISTER_TYPE,
                         props, false, "$*", policies);
        doWithdraw(id, "org.omg.CosTrading.RegisterPackage.ProxyOfferId");
        withdrawProxy(trader_, id);
    }

    //
    // Test the describe operation
    //
    void
    testDescribe()
    {
        String id;
        org.omg.CosTrading.Property[] props;
        org.omg.CosTrading.Property prop2, prop4;

        prop2 = new org.omg.CosTrading.Property();
        prop2.name = "prop2";
        prop2.value = orb_.create_any();
        prop2.value.insert_float((float)1.99);

        prop4 = new org.omg.CosTrading.Property();
        prop4.name = "prop4";
        prop4.value = orb_.create_any();
        prop4.value.insert_ushort((short)5000);

        //
        // Test: IllegalOfferId exception
        //
        doDescribe("", "org.omg.CosTrading.IllegalOfferId");

        //
        // Test: UnknownOfferId exception - try to describe an offer
        //       that has already been withdrawn
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        id = export(orb_, trader_, REGISTER_TYPE, props);
        withdrawOffer(trader_, id);
        doDescribe(id, "org.omg.CosTrading.UnknownOfferId");

        //
        // Test: ProxyOfferId exception
        //
        props = new org.omg.CosTrading.Property[2];
        props[0] = prop2;
        props[1] = prop4;
        org.omg.CosTrading.Policy[] policies =
            new org.omg.CosTrading.Policy[0];
        id = exportProxy(orb_, trader_, trader_, REGISTER_TYPE,
                         props, false, "$*", policies);
        doDescribe(id, "org.omg.CosTrading.RegisterPackage.ProxyOfferId");
        withdrawProxy(trader_, id);
    }

    //
    // Test the modify operation
    //
    void
    testModify()
    {
        org.omg.CosTrading.Admin admin = trader_.admin_if();
        boolean oldSupportsValue =
            admin.set_supports_modifiable_properties(true);

        try
        {
            String id;
            org.omg.CosTrading.Property[] props;
            org.omg.CosTrading.Property prop1, prop2, prop3, prop4;
            org.omg.CosTradingDynamic.DynamicProp dynProp;
            String[] delList;

            prop1 = new org.omg.CosTrading.Property();
            prop1.name = "prop1";
            prop1.value = orb_.create_any();
            prop1.value.insert_string("Ho there");

            prop2 = new org.omg.CosTrading.Property();
            prop2.name = "prop2";
            prop2.value = orb_.create_any();
            prop2.value.insert_float((float)0.99);

            prop3 = new org.omg.CosTrading.Property();
            prop3.name = "prop3";
            prop3.value = orb_.create_any();
            prop3.value.insert_char('A');

            prop4 = new org.omg.CosTrading.Property();
            prop4.name = "prop4";
            prop4.value = orb_.create_any();
            prop4.value.insert_ushort((short)100);

            //
            // Test: NotImplemented exception
            //
            admin.set_supports_modifiable_properties(false);
            props = new org.omg.CosTrading.Property[4];
            props[0] = prop1;
            props[1] = prop2;
            props[2] = prop3;
            props[3] = prop4;
            id = export(orb_, trader_, REGISTER_TYPE, props);
            delList = new String[1];
            delList[0] = "prop1";
            props = new org.omg.CosTrading.Property[0];
            doModify(id, delList, props,
                     "org.omg.CosTrading.NotImplemented");
            withdrawOffer(trader_, id);
            admin.set_supports_modifiable_properties(true);

            //
            // Test: IllegalOfferId exception
            //
            delList = new String[0];
            props = new org.omg.CosTrading.Property[0];
            doModify("", delList, props,
                     "org.omg.CosTrading.IllegalOfferId");

            //
            // Test: UnknownOfferId exception - try to withdraw the offer
            //       that was already withdrawn in the NotImplemented test above
            //
            delList = new String[0];
            props = new org.omg.CosTrading.Property[0];
            doModify(id, delList, props, "org.omg.CosTrading.UnknownOfferId");

            //
            // Test: ProxyOfferId exception
            //
            props = new org.omg.CosTrading.Property[2];
            props[0] = prop2;
            props[1] = prop4;
            org.omg.CosTrading.Policy[] policies =
                new org.omg.CosTrading.Policy[0];
            id = exportProxy(orb_, trader_, trader_, REGISTER_TYPE,
                             props, false, "$*", policies);
            delList = new String[1];
            delList[0] = "prop2";
            props = new org.omg.CosTrading.Property[0];
            doModify(id, delList, props,
                     "org.omg.CosTrading.RegisterPackage.ProxyOfferId");
            withdrawProxy(trader_, id);

            //
            // Test: ReadonlyDynamicProperty exception - export an
            //       offer that doesn't have a value for prop3, then
            //       attempt to modify it to be a dynamic property
            //
            props = new org.omg.CosTrading.Property[3];
            props[0] = prop1;
            props[1] = prop2;
            props[2] = prop4;
            id = export(orb_, trader_, REGISTER_TYPE, props);
            delList = new String[0];
            props = new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "prop3";
            props[0].value = orb_.create_any();
            DynPropImpl dp = new DynPropImpl(null, null, null);
            dynProp = new org.omg.CosTradingDynamic.DynamicProp();
            dynProp.eval_if = dp;
            dynProp.returned_type =
                orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_char);
            dynProp.extra_info = orb_.create_any();
            org.omg.CosTradingDynamic.DynamicPropHelper.insert(props[0].value,
                                                               dynProp);
            doModify(id, delList, props,
                     "org.omg.CosTrading.ReadonlyDynamicProperty");
            withdrawOffer(trader_, id);

            //
            // Test: Export an offer with all the properties.  We'll use this
            //       offer in several tests to follow.
            //
            props = new org.omg.CosTrading.Property[4];
            props[0] = prop1;
            props[1] = prop2;
            props[2] = prop3;
            props[3] = prop4;
            id = export(orb_, trader_, REGISTER_TYPE, props);

            //
            // Test: IllegalPropertyName exception - in delete_list
            //
            delList = new String[1];
            delList[0] = "<foo-bar";
            props = new org.omg.CosTrading.Property[0];
            doModify(id, delList, props,
                     "org.omg.CosTrading.IllegalPropertyName");

            //
            // Test: IllegalPropertyName exception - in modify_list
            //
            delList = new String[0];
            props = new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "<foo-bar";
            props[0].value = orb_.create_any();
            doModify(id, delList, props,
                     "org.omg.CosTrading.IllegalPropertyName");

            //
            // Test: UnknownPropertyName exception
            //
            delList = new String[1];
            delList[0] = "prop99";
            props = new org.omg.CosTrading.Property[0];
            doModify(id, delList, props,
                     "org.omg.CosTrading.RegisterPackage.UnknownPropertyName");

            //
            // Test: PropertyTypeMismatch exception
            //
            delList = new String[0];
            props = new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "prop1";
            props[0].value = orb_.create_any();
            props[0].value.insert_long(50);
            doModify(id, delList, props,
                     "org.omg.CosTrading.PropertyTypeMismatch");

            //
            // Test: MandatoryProperty exception
            //
            delList = new String[1];
            delList[0] = "prop4";
            props = new org.omg.CosTrading.Property[0];
            doModify(id, delList, props,
                     "org.omg.CosTrading.RegisterPackage.MandatoryProperty");

            //
            // Test: ReadonlyProperty exception
            //
            delList = new String[0];
            props = new org.omg.CosTrading.Property[1];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "prop4";
            props[0].value = orb_.create_any();
            props[0].value.insert_ushort((short)99);
            doModify(id, delList, props,
                     "org.omg.CosTrading.RegisterPackage.ReadonlyProperty");

            //
            // Test: DuplicatePropertyName exception - in del_list
            //
            delList = new String[3];
            delList[0] = "prop1";
            delList[1] = "prop3";
            delList[2] = "prop1";
            props = new org.omg.CosTrading.Property[0];
            doModify(id, delList, props,
                     "org.omg.CosTrading.DuplicatePropertyName");

            //
            // Test: DuplicatePropertyName exception - in modify_list
            //
            delList = new String[0];
            props = new org.omg.CosTrading.Property[2];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "prop4";
            props[0].value = orb_.create_any();
            props[0].value.insert_ushort((short)99);
            props[1] = new org.omg.CosTrading.Property();
            props[1].name = "prop4";
            props[1].value = orb_.create_any();
            props[1].value.insert_ushort((short)98);
            doModify(id, delList, props,
                     "org.omg.CosTrading.DuplicatePropertyName");

            //
            // Test: Ensure that modified properties are actually
            //       modified; we remove the readonly property (prop3)
            //       and modify prop1 and prop2
            //
            delList = new String[1];
            delList[0] = "prop3";
            props = new org.omg.CosTrading.Property[2];
            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "prop1";
            props[0].value = orb_.create_any();
            props[0].value.insert_string("Bye bye");
            props[1] = new org.omg.CosTrading.Property();
            props[1].name = "prop2";
            props[1].value = orb_.create_any();
            props[1].value.insert_float((float)9.99);
            doModify(id, delList, props, null);
            org.omg.CosTrading.RegisterPackage.OfferInfo info =
                doDescribe(id, null);
            TEST(info.properties.length == 3);
            for(int i = 0 ; i < info.properties.length ; i++)
            {
                org.omg.CosTrading.Property p = info.properties[i];
                if(p.equals("prop1"))
                    TEST(p.value.equal(props[0].value));
                else if(p.equals("prop2"))
                    TEST(p.value.equal(props[1].value));
            }

            withdrawOffer(trader_, id);
        }
        finally
        {
            admin.set_supports_modifiable_properties(oldSupportsValue);
        }
    }

    //
    // Test the withdraw_using_constraint operation
    //
    void
    testWithdrawConstraint()
    {
        //
        // Test: IllegalServiceType exception
        //
        doWithdrawConstraint("", "TRUE",
                             "org.omg.CosTrading.IllegalServiceType");

        //
        // Test: UnknownServiceType exception
        //
        doWithdrawConstraint("ThisTypeDoesNotExist", "TRUE",
                             "org.omg.CosTrading.UnknownServiceType");

        //
        // Test: IllegalConstraint exception
        //
        doWithdrawConstraint(REGISTER_TYPE, "'ABC'",
                             "org.omg.CosTrading.IllegalConstraint");

        //
        // Test: NoMatchingOffers exception
        //
        doWithdrawConstraint(REGISTER_TYPE, "FALSE",
            "org.omg.CosTrading.RegisterPackage.NoMatchingOffers");

        //
        // Export some offers
        //
        String[] ids = new String[4];
        for(int i = 0 ; i < ids.length ; i++)
        {
            org.omg.CosTrading.Property[] props;

            props = new org.omg.CosTrading.Property[4];

            props[0] = new org.omg.CosTrading.Property();
            props[0].name = "prop1";
            props[0].value = orb_.create_any();
            props[0].value.insert_string("Offer #" + (i + 1));

            props[1] = new org.omg.CosTrading.Property();
            props[1].name = "prop2";
            props[1].value = orb_.create_any();
            props[1].value.insert_float((float)(i + 0.99));

            props[2] = new org.omg.CosTrading.Property();
            props[2].name = "prop3";
            props[2].value = orb_.create_any();
            props[2].value.insert_char((char)('A' + i));

            props[3] = new org.omg.CosTrading.Property();
            props[3].name = "prop4";
            props[3].value = orb_.create_any();
            props[3].value.insert_ushort((short)i);

            ids[i] = export(orb_, trader_, REGISTER_TYPE, props);
        }

        //
        // Test: Withdraw offers one at a time using a constraint that
        //       uniquely identifies each offer
        //
        for(int i = 0 ; i < ids.length ; i++)
        {
            String constraint = "prop4 == " + i + " and prop3 == '" +
                                ((char)('A' + i)) + "' and prop2 < " + (i + 1) +
                                " and prop1 == 'Offer #" + (i + 1) + "'";
            doWithdrawConstraint(REGISTER_TYPE, constraint, null);
        }
    }

    String
    doExport(org.omg.CORBA.Object ref,
             String type,
             org.omg.CosTrading.Property[] props,
             String expectedExceptionClassName,
             boolean enforceException)
    {
        String result = null;

        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            result = reg_.export(ref, type, props);

            if(enforceException && expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }

        return result;
    }

    void
    doWithdraw(String id, String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            reg_.withdraw(id);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }

    org.omg.CosTrading.RegisterPackage.OfferInfo
    doDescribe(String id, String expectedExceptionClassName)
    {
        org.omg.CosTrading.RegisterPackage.OfferInfo result = null;

        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            result = reg_.describe(id);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }

        return result;
    }

    void
    doModify(String id,
             String[] delList,
             org.omg.CosTrading.Property[] modifyList,
             String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            reg_.modify(id, delList, modifyList);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }

    void
    doWithdrawConstraint(String type,
                         String constraint,
                         String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            reg_.withdraw_using_constraint(type, constraint);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }
}
