// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class TestRepos extends TestBase
{
    private org.omg.CORBA.ORB orb_;
    private org.omg.CosTrading.Lookup trader_;
    private org.omg.CosTradingRepos.ServiceTypeRepository repos_;

    TestRepos(org.omg.CORBA.ORB orb, org.omg.CosTrading.Lookup trader)
    {
        orb_ = orb;
        trader_ = trader;

        org.omg.CORBA.Object obj = trader.type_repos();
        TEST(obj != null);
        repos_ =
            org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.narrow(obj);
        TEST(repos_ != null);

        testAdd();
        testRemove();
        testList();
        testDescribe();
        testFullyDescribe();
        testMask();
        testUnmask();
    }

    //
    // Test the add_type operation
    //
    void
    testAdd()
    {
        PropStruct[] props;
        String[] superTypes;
        org.omg.CORBA.TypeCode tc;

        //
        // Test: IllegalServiceType exception
        //
        props = new PropStruct[0];
        superTypes = new String[0];
        doAdd("", "Interface", props, superTypes,
              "org.omg.CosTrading.IllegalServiceType", true);

        //
        // Test: IllegalPropertyName exception
        //
        props = new PropStruct[1];
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
        props[0] = new PropStruct("<foo-bar", tc, PropertyMode.PROP_NORMAL);
        superTypes = new String[0];
        doAdd("TestType", "Interface", props, superTypes,
              "org.omg.CosTrading.IllegalPropertyName", true);

        //
        // Test: DuplicatePropertyName exception
        //
        props = new PropStruct[3];
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
        props[0] = new PropStruct("prop1", tc, PropertyMode.PROP_NORMAL);
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
        props[1] = new PropStruct("prop2", tc, PropertyMode.PROP_NORMAL);
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_double);
        props[2] = new PropStruct("prop1", tc, PropertyMode.PROP_NORMAL);
        superTypes = new String[0];
        doAdd("TestType", "Interface", props, superTypes,
              "org.omg.CosTrading.DuplicatePropertyName", true);

        //
        // Test: UnknownServiceType exception
        //
        props = new PropStruct[0];
        superTypes = new String[1];
        superTypes[0] = "ThisSuperTypeDoesNotExist";
        doAdd("TestType", "Interface", props, superTypes,
              "org.omg.CosTrading.UnknownServiceType", true);

        //
        // Test: DuplicateServiceTypeName exception
        //
        props = new PropStruct[0];
        superTypes = new String[0];
        doAdd("TestDupType1", "Interface", props, superTypes, null, true);
        props = new PropStruct[0];
        superTypes = new String[0];
        doAdd("TestDupType2", "Interface", props, superTypes, null, true);
        props = new PropStruct[0];
        superTypes = new String[3];
        superTypes[0] = "TestDupType1";
        superTypes[1] = "TestDupType2";
        superTypes[2] = "TestDupType1";
        doAdd("TestDupType3", "Interface", props, superTypes,
              "org.omg.CosTradingRepos.ServiceTypeRepositoryPackage." +
              "DuplicateServiceTypeName", true);
        doRemove("TestDupType1", null);
        doRemove("TestDupType2", null);

        //
        // Test: ServiceTypeExists exception
        //
        props = new PropStruct[0];
        superTypes = new String[0];
        doAdd("TestExistType", "Interface", props, superTypes, null, true);
        props = new PropStruct[0];
        superTypes = new String[0];
        doAdd("TestExistType", "Interface", props, superTypes,
              "org.omg.CosTradingRepos.ServiceTypeRepositoryPackage." +
              "ServiceTypeExists", true);
        doRemove("TestExistType", null);

        //
        // Test: ValueTypeRedefinition exception - create a type with
        //       a property 'prop1' of type tk_long; then try to create
        //       a type which redefines prop1 with a type of tk_float
        //
        props = new PropStruct[1];
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
        props[0] = new PropStruct("prop1", tc, PropertyMode.PROP_NORMAL);
        superTypes = new String[0];
        doAdd("TestRedefType1", "Interface", props, superTypes, null, true);
        props = new PropStruct[1];
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
        props[0] = new PropStruct("prop1", tc, PropertyMode.PROP_NORMAL);
        superTypes = new String[1];
        superTypes[0] = "TestRedefType1";
        doAdd("TestRedefType2", "Interface", props, superTypes,
              "org.omg.CosTradingRepos.ServiceTypeRepositoryPackage." +
              "ValueTypeRedefinition", true);
        doRemove("TestRedefType1", null);

        //
        // Test: ValueTypeRedefinition exception - create a type with
        //       a property 'prop1' with mode MANDATORY; then try to create
        //       a type which redefines prop1 with a mode of NORMAL
        //
        props = new PropStruct[1];
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
        props[0] = new PropStruct("prop1", tc, PropertyMode.PROP_MANDATORY);
        superTypes = new String[0];
        doAdd("TestRedefType1", "Interface", props, superTypes, null, true);
        props = new PropStruct[1];
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
        props[0] = new PropStruct("prop1", tc, PropertyMode.PROP_NORMAL);
        superTypes = new String[1];
        superTypes[0] = "TestRedefType1";
        doAdd("TestRedefType2", "Interface", props, superTypes,
              "org.omg.CosTradingRepos.ServiceTypeRepositoryPackage." +
              "ValueTypeRedefinition", true);
        doRemove("TestRedefType1", null);

        //
        // Test: InterfaceTypeMismatch exception - if the server is
        //       configured to use the interface repository (and one is
        //       available), then we try cause the InterfaceTypeMismatch
        //       exception by first creating a type with an interface name
        //       of the CosTrading::Lookup ID, then creating a subtype
        //       with the CosTrading::Register ID (which does not "conform"
        //       to the interface of its supertype).
        //
        props = new PropStruct[0];
        superTypes = new String[0];
        doAdd("TestMismatchType1", org.omg.CosTrading.LookupHelper.id(),
              props, superTypes, null, true);
        props = new PropStruct[0];
        superTypes = new String[1];
        superTypes[0] = "TestMismatchType1";
        doAdd("TestMismatchType2",
              org.omg.CosTrading.RegisterHelper.id(), props, superTypes,
              "org.omg.CosTradingRepos.ServiceTypeRepositoryPackage." +
              "InterfaceTypeMismatch", false);
        removeType(trader_, "TestMismatchType2");
        doRemove("TestMismatchType1", null);
    }

    //
    // Test the remove_type operation
    //
    void
    testRemove()
    {
        PropStruct[] props;
        String[] superTypes;
        org.omg.CORBA.TypeCode tc;

        //
        // Test: IllegalServiceType exception
        //
        doRemove("", "org.omg.CosTrading.IllegalServiceType");

        //
        // Test: UnknownServiceType exception
        //
        doRemove("ThisTypeDoesNotExist",
                 "org.omg.CosTrading.UnknownServiceType");

        //
        // Test: HasSubTypes exception
        //
        props = new PropStruct[0];
        superTypes = new String[0];
        doAdd("TestBaseType", "Interface", props, superTypes, null, true);
        props = new PropStruct[0];
        superTypes = new String[1];
        superTypes[0] = "TestBaseType";
        doAdd("TestSubType", "Interface", props, superTypes, null, true);
        doRemove("TestBaseType",
                 "org.omg.CosTradingRepos.ServiceTypeRepositoryPackage." +
                 "HasSubTypes");
        removeType(trader_, "TestSubType");
        removeType(trader_, "TestBaseType");
    }

    //
    // Test the list_types operation
    //
    void
    testList()
    {
        PropStruct[] props;
        String[] superTypes;
        org.omg.CORBA.TypeCode tc;
        IncarnationNumber inc;
        String[] types;
        SpecifiedServiceTypes whichTypes;
        int numTypes;

        whichTypes = new SpecifiedServiceTypes();
        whichTypes._default(ListOption.all);
        types = repos_.list_types(whichTypes);
        numTypes = types.length;

        //
        // Test: Add a type, then use the resulting incarnation number
        //       in a call to list_types
        //
        props = new PropStruct[0];
        superTypes = new String[0];
        inc = doAdd("TestListType", "Interface", props, superTypes,
                    null, true);
        whichTypes.incarnation(inc);
        types = repos_.list_types(whichTypes);
        TEST(types.length == 1);
        TEST(types[0].equals("TestListType"));
        removeType(trader_, "TestListType");

        types = repos_.list_types(whichTypes);
        TEST(types.length == numTypes);
    }

    //
    // Test the describe_type operation
    //
    void
    testDescribe()
    {
        PropStruct[] props;
        String[] superTypes;
        org.omg.CORBA.TypeCode tc;
        IncarnationNumber inc;

        //
        // Test: IllegalServiceType exception
        //
        doDescribe("", "org.omg.CosTrading.IllegalServiceType");

        //
        // Test: UnknownServiceType exception
        //
        doDescribe("ThisTypeDoesNotExist",
                   "org.omg.CosTrading.UnknownServiceType");

        //
        // Test: Add a type, then describe it and validate the results
        //
        props = new PropStruct[2];
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
        props[0] = new PropStruct("prop1", tc, PropertyMode.PROP_NORMAL);
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_short);
        props[1] = new PropStruct("prop2", tc, PropertyMode.PROP_READONLY);
        superTypes = new String[0];
        inc = doAdd("TestDescType", "Interface", props, superTypes,
                    null, true);

        TypeStruct ts = doDescribe("TestDescType", null);
        TEST(ts.props.length == 2);
        TEST(ts.if_name.equals("Interface"));
        TEST(ts.super_types.length == 0);
        TEST(ts.masked == false);
        TEST(ts.incarnation.high == inc.high);
        TEST(ts.incarnation.low == inc.low);
        for(int i = 0 ; i < ts.props.length ; i++)
        {
            PropStruct ps = ts.props[i];
            if(ps.name.equals("prop1"))
            {
                TEST(ps.value_type.equal(props[0].value_type));
                TEST(ps.mode == props[0].mode);
            }
            else if(ps.name.equals("prop2"))
            {
                TEST(ps.value_type.equal(props[1].value_type));
                TEST(ps.mode == props[1].mode);
            }
        }

        removeType(trader_, "TestDescType");
    }

    //
    // Test the fully_describe_type operation
    //
    void
    testFullyDescribe()
    {
        PropStruct[] props;
        PropStruct[] subProps;
        String[] superTypes;
        org.omg.CORBA.TypeCode tc;

        //
        // Test: IllegalServiceType exception
        //
        doFullyDescribe("", "org.omg.CosTrading.IllegalServiceType");

        //
        // Test: UnknownServiceType exception
        //
        doFullyDescribe("ThisTypeDoesNotExist",
                        "org.omg.CosTrading.UnknownServiceType");

        //
        // Test: Add two types, then fully describe the subtype
        //
        props = new PropStruct[2];
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
        props[0] = new PropStruct("prop1", tc, PropertyMode.PROP_NORMAL);
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_short);
        props[1] = new PropStruct("prop2", tc, PropertyMode.PROP_READONLY);
        superTypes = new String[0];
        doAdd("TestBaseType", "Interface", props, superTypes, null, true);
        subProps = new PropStruct[2];
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
        subProps[0] = new PropStruct("prop1", tc, PropertyMode.PROP_MANDATORY);
        tc = orb_.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
        subProps[1] = new PropStruct("prop3", tc, PropertyMode.PROP_READONLY);
        superTypes = new String[1];
        superTypes[0] = "TestBaseType";
        doAdd("TestSubType", "SubInterface", subProps, superTypes, null, true);

        TypeStruct ts = doFullyDescribe("TestSubType", null);
        TEST(ts.props.length == 3);
        TEST(ts.if_name.equals("SubInterface"));
        TEST(ts.super_types.length == 1);
        TEST(ts.super_types[0].equals("TestBaseType"));
        TEST(ts.masked == false);
        for(int i = 0 ; i < ts.props.length ; i++)
        {
            PropStruct ps = ts.props[i];
            if(ps.name.equals("prop1"))
            {
                TEST(ps.value_type.equal(subProps[0].value_type));
                TEST(ps.mode == subProps[0].mode);
            }
            else if(ps.name.equals("prop2"))
            {
                TEST(ps.value_type.equal(props[1].value_type));
                TEST(ps.mode == props[1].mode);
            }
            else if(ps.name.equals("prop3"))
            {
                TEST(ps.value_type.equal(subProps[1].value_type));
                TEST(ps.mode == subProps[1].mode);
            }
        }

        removeType(trader_, "TestSubType");
        removeType(trader_, "TestBaseType");
    }

    //
    // Test the mask_type operation
    //
    void
    testMask()
    {
        PropStruct[] props;
        String[] superTypes;
        org.omg.CORBA.TypeCode tc;

        //
        // Test: IllegalServiceType exception
        //
        doMask("", "org.omg.CosTrading.IllegalServiceType");

        //
        // Test: UnknownServiceType exception
        //
        doMask("ThisTypeDoesNotExist",
               "org.omg.CosTrading.UnknownServiceType");

        //
        // Test: Add a type, mask it and validate the results
        //
        props = new PropStruct[0];
        superTypes = new String[0];
        doAdd("TestMaskType", "Interface", props, superTypes, null, true);
        doMask("TestMaskType", null);
        TypeStruct ts = doDescribe("TestMaskType", null);
        TEST(ts.masked == true);

        //
        // Test: AlreadyMasked exception
        //
        doMask("TestMaskType",
               "org.omg.CosTradingRepos.ServiceTypeRepositoryPackage." +
               "AlreadyMasked");

        removeType(trader_, "TestMaskType");
    }

    //
    // Test the unmask_type operation
    //
    void
    testUnmask()
    {
        PropStruct[] props;
        String[] superTypes;
        org.omg.CORBA.TypeCode tc;

        //
        // Test: IllegalServiceType exception
        //
        doUnmask("", "org.omg.CosTrading.IllegalServiceType");

        //
        // Test: UnknownServiceType exception
        //
        doUnmask("ThisTypeDoesNotExist",
                 "org.omg.CosTrading.UnknownServiceType");

        //
        // Test: Add a type, mask it, unmask it, and validate the results
        //
        props = new PropStruct[0];
        superTypes = new String[0];
        doAdd("TestUnmaskType", "Interface", props, superTypes, null, true);
        doMask("TestUnmaskType", null);
        doUnmask("TestUnmaskType", null);
        TypeStruct ts = doDescribe("TestUnmaskType", null);
        TEST(ts.masked == false);

        //
        // Test: NotMasked exception
        //
        doUnmask("TestUnmaskType",
                 "org.omg.CosTradingRepos.ServiceTypeRepositoryPackage." +
                 "NotMasked");

        removeType(trader_, "TestUnmaskType");
    }

    IncarnationNumber
    doAdd(String name,
          String interfaceName,
          PropStruct[] props,
          String[] superTypes,
          String expectedExceptionClassName,
          boolean enforceException)
    {
        IncarnationNumber result = null;

        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            result = repos_.add_type(name, interfaceName, props, superTypes);

            if(enforceException && expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }

        return result;
    }

    void
    doRemove(String name, String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            repos_.remove_type(name);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }

    TypeStruct
    doDescribe(String name, String expectedExceptionClassName)
    {
        TypeStruct result = null;

        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            result = repos_.describe_type(name);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }

        return result;
    }

    TypeStruct
    doFullyDescribe(String name, String expectedExceptionClassName)
    {
        TypeStruct result = null;

        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            result = repos_.fully_describe_type(name);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }

        return result;
    }

    void
    doMask(String name, String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            repos_.mask_type(name);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }

    void
    doUnmask(String name, String expectedExceptionClassName)
    {
        Class expectedException = null;
        if(expectedExceptionClassName != null)
        {
            try
            {
                expectedException = Class.forName(expectedExceptionClassName);
            }
            catch(ClassNotFoundException ex)
            {
                fail("Cannot find exception class " +
                     expectedExceptionClassName);
            }
        }

        try
        {
            repos_.unmask_type(name);

            if(expectedException != null)
                fail("Error: Expected to catch " + expectedExceptionClassName);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            if(expectedException == null || ! expectedException.isInstance(ex))
                handleException(ex);
        }
    }
}
