// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util;

/**
 * AbstractOption implements the basic accessor methods defined in the
 * <code>Option</code> interface, yet leaves the implementation of the
 * other methods to subclasses.
 *
 * @author Mark Spruiell
 */
public abstract class AbstractOption implements Option
{
    private char ch_;
    private String name_;
    private String property_;

    public
    AbstractOption(char ch)
    {
        this(ch, null, null);
    }

    public
    AbstractOption(String name)
    {
        this((char)0, name, null);
    }

    public
    AbstractOption(char ch, String name)
    {
        this(ch, name, null);
    }

    public
    AbstractOption(char ch, String name, String property)
    {
        ch_ = ch;
        name_ = name;
        property_ = property;
    }

    public char
    getChar()
    {
        return ch_;
    }

    public String
    getName()
    {
        return name_;
    }

    public String
    getProperty()
    {
        return property_;
    }

    public abstract void
    parseProperty(String value)
        throws OptionException;

    public abstract void
    parse(String[] args, int offset)
        throws OptionException;

    public String
    toString()
    {
        if(ch_ != 0 && name_ != null)
            return "-" + ch_ + ",--" + name_;
        else if(ch_ != 0)
            return "-" + ch_;
        else
            return "--" + name_;
    }
}
