// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util.CORBA;

//
// AnyWriter is a handy class for displaying Anys, DynAnys and TypeCodes.
//
// Hierarchical structures are represented by indenting; the width of
// the indent can be specified using setSpacing().  The default indent
// is two spaces.
//
// The up() and down() methods control the current indent level.
//
public class AnyWriter
{
    private int level_ = 0;
    private String spacing_ = "  ";
    private java.io.PrintWriter out_;
    private org.omg.CORBA.ORB orb_;
    private static java.util.Hashtable typeNames_;


    static
    {
        //
        // Put the names of the TCKind values in a hashtable
        //
        typeNames_ = new java.util.Hashtable();
        typeNames_.put(org.omg.CORBA.TCKind.tk_null, "null");
        typeNames_.put(org.omg.CORBA.TCKind.tk_void, "void");
        typeNames_.put(org.omg.CORBA.TCKind.tk_short, "short");
        typeNames_.put(org.omg.CORBA.TCKind.tk_ushort, "unsigned short");
        typeNames_.put(org.omg.CORBA.TCKind.tk_long, "long");
        typeNames_.put(org.omg.CORBA.TCKind.tk_ulong, "unsigned long");
        typeNames_.put(org.omg.CORBA.TCKind.tk_float, "float");
        typeNames_.put(org.omg.CORBA.TCKind.tk_double, "double");
        typeNames_.put(org.omg.CORBA.TCKind.tk_boolean, "boolean");
        typeNames_.put(org.omg.CORBA.TCKind.tk_char, "char");
        typeNames_.put(org.omg.CORBA.TCKind.tk_octet, "octet");
        typeNames_.put(org.omg.CORBA.TCKind.tk_any, "any");
        typeNames_.put(org.omg.CORBA.TCKind.tk_TypeCode, "TypeCode");
        typeNames_.put(org.omg.CORBA.TCKind.tk_Principal, "Principal");
        typeNames_.put(org.omg.CORBA.TCKind.tk_objref, "objref");
        typeNames_.put(org.omg.CORBA.TCKind.tk_struct, "struct");
        typeNames_.put(org.omg.CORBA.TCKind.tk_union, "union");
        typeNames_.put(org.omg.CORBA.TCKind.tk_enum, "enum");
        typeNames_.put(org.omg.CORBA.TCKind.tk_string, "string");
        typeNames_.put(org.omg.CORBA.TCKind.tk_sequence, "sequence");
        typeNames_.put(org.omg.CORBA.TCKind.tk_array, "array");
        typeNames_.put(org.omg.CORBA.TCKind.tk_alias, "alias");
        typeNames_.put(org.omg.CORBA.TCKind.tk_except, "exception");
    }

    //
    // Given a TypeCode, return the TypeCode without any aliasing
    //
    private static org.omg.CORBA.TypeCode
    getOriginalType(org.omg.CORBA.TypeCode tc)
    {
        org.omg.CORBA.TypeCode result = tc;

        try
        {
            while(result.kind() == org.omg.CORBA.TCKind.tk_alias)
                result = result.content_type();
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind ex)
        {
            // ignore
        }

        return result;
    }

    //
    // Default constructor uses System.out as the output stream
    //
    public
    AnyWriter()
    {
        this(new java.io.PrintWriter(System.out, true));
    }

    //
    // User supplies the Writer object
    //
    public
    AnyWriter(java.io.PrintWriter out)
    {
        out_ = out;
        orb_ = org.omg.CORBA.ORB.init();
    }

    public void
    setWriter(java.io.PrintWriter out)
    {
        out_ = out;
    }

    public java.io.PrintWriter
    getWriter()
    {
        return out_;
    }

    public void
    flush()
    {
        out_.flush();
    }

    public void
    setSpacing(String spacing)
    {
        spacing_ = spacing;
    }

    public String
    getSpacing()
    {
        return spacing_;
    }

    public void
    up()
    {
        level_--;
    }

    public void
    down()
    {
        level_++;
    }

    //
    // "Tabs" over to the appropriate starting point for the current
    // indent level
    //
    public java.io.PrintWriter
    skip()
    {
        for(int i = 0 ; i < level_ ; i++)
            out_.print(spacing_);
        return out_;
    }

    //
    // Prints the full TypeCode
    //
    public void
    print(org.omg.CORBA.TypeCode tc)
    {
        org.omg.CORBA.TCKind kind = tc.kind();

        try
        {
            switch(kind.value())
            {
                case org.omg.CORBA.TCKind._tk_objref:
                    skip().print("interface " + tc.name() + ";");
                    break;

                case org.omg.CORBA.TCKind._tk_struct:
                case org.omg.CORBA.TCKind._tk_except:
                {
                    skip().println("" + typeNames_.get(kind) + " " +
                        tc.name() + " {");
                    down();
                    for(int i = 0 ; i < tc.member_count() ; i++)
                    {
                        skip();
                        org.omg.CORBA.TypeCode memberType = tc.member_type(i);
                        printName(memberType);
                        out_.println(" " + tc.member_name(i) + ";");
                    }
                    up();
                    skip().print("};");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_union:
                {
                    skip().print("union " + tc.name() + " switch(");
                    org.omg.CORBA.TypeCode discrType = tc.discriminator_type();
                    printName(discrType);
                    out_.println(") {");
                    down();
                    for(int i = 0 ; i < tc.member_count() ; i++)
                    {
                        org.omg.CORBA.Any label = tc.member_label(i);
                        org.omg.CORBA.TypeCode type = label.type();

                        if(type.kind() == org.omg.CORBA.TCKind.tk_octet)
                            skip().print("default");
                        else
                        {
                            skip().print("case ");
                            org.omg.CORBA.DynAny dynAny =
                                orb_.create_dyn_any(label);
                            printDynAny(dynAny, false);
                        }

                        out_.print(": ");
                        org.omg.CORBA.TypeCode memberType = tc.member_type(i);
                        printName(memberType);
                        out_.println(" " + tc.member_name(i) + ";");
                    }
                    up();
                    skip().print("};");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_enum:
                {
                    skip().print("enum " + tc.name()+ " {");
                    for(int i = 0 ; i < tc.member_count() ; i++)
                    {
                        if(i != 0 )
                            out_.print(", ");
                        out_.print(tc.member_name(i));
                    }
                    out_.print("};");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_string:
                {
                    skip().print("string");
                    int length = tc.length();
                    if(length != 0)
                        out_.print("<" + length + ">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_sequence:
                {
                    skip().print("sequence<");
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    int length = tc.length();
                    if(length != 0)
                        out_.print(", " + length);
                    out_.print(">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_array:
                {
                    skip();
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    int length = tc.length();
                    out_.print("[" + length + "]");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_alias:
                {
                    skip().print("typedef ");
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    out_.print(" " + tc.name() + ";");
                    break;
                }

                default:
                    skip().print("" + typeNames_.get(kind));
                    break;
            }

            out_.println();
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind e)
        {
        }
        catch(org.omg.CORBA.TypeCodePackage.Bounds e)
        {
        }
    }

    //
    // Prints only the name of a TypeCode
    //
    public void
    printName(org.omg.CORBA.TypeCode tc)
    {
        org.omg.CORBA.TCKind kind = tc.kind();

        try
        {
            switch(kind.value())
            {
                case org.omg.CORBA.TCKind._tk_objref:
                case org.omg.CORBA.TCKind._tk_enum:
                case org.omg.CORBA.TCKind._tk_union:
                case org.omg.CORBA.TCKind._tk_struct:
                case org.omg.CORBA.TCKind._tk_except:
                case org.omg.CORBA.TCKind._tk_alias:
                    out_.print(tc.name());
                    break;

                case org.omg.CORBA.TCKind._tk_string:
                {
                    out_.print("string");
                    int length = tc.length();
                    if(length != 0)
                        out_.print("<" + length + ">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_sequence:
                {
                    out_.print("sequence<");
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    int length = tc.length();
                    if(length != 0)
                        out_.print(", " + length);
                    out_.print(">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_array:
                {
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    int length = tc.length();
                    out_.print("[" + length + "]");
                    break;
                }

                default:
                    out_.print("" + typeNames_.get(kind));
                    break;
            }
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind e)
        {
        }
    }

    //
    // Indents as appropriate and prints the value of the DynAny,
    // including TypeCode information
    //
    public void
    print(org.omg.CORBA.DynAny dynAny)
    {
        skip();
        printDynAny(dynAny);
        out_.println();
    }

    //
    // Prints the value of the DynAny, including TypeCode information
    //
    public void
    printDynAny(org.omg.CORBA.DynAny dynAny)
    {
        printDynAny(dynAny, true);
    }

    //
    // Prints the value of the DynAny; showType determines whether
    // TypeCode information is printed
    //
    public void
    printDynAny(org.omg.CORBA.DynAny dynAny, boolean showType)
    {
        if(showType)
        {
            org.omg.CORBA.TypeCode tc = dynAny.type();
            printName(tc);
            out_.print("(");
        }

        dynAny.rewind();

        org.omg.CORBA.TypeCode type = dynAny.type();
        org.omg.CORBA.TypeCode tc = getOriginalType(type);
        org.omg.CORBA.TCKind kind = tc.kind();

        try
        {
            switch(kind.value())
            {
                case org.omg.CORBA.TCKind._tk_null:
                    out_.print("<null>");
                    break;

                case org.omg.CORBA.TCKind._tk_void:
                    out_.print("<void>");
                    break;

                case org.omg.CORBA.TCKind._tk_short:
                    out_.print(dynAny.get_short());
                    break;

                case org.omg.CORBA.TCKind._tk_ushort:
                {
                    int i = dynAny.get_ushort();
                    if(i < 0)
                        i += ((int)Short.MAX_VALUE -
                              (int)Short.MIN_VALUE) + 1;
                    out_.print(i);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_long:
                    out_.print(dynAny.get_long());
                    break;

                case org.omg.CORBA.TCKind._tk_ulong:
                {
                    long l = dynAny.get_ulong();
                    if(l < 0)
                        l += ((long)Integer.MAX_VALUE -
                              (long)Integer.MIN_VALUE) + 1;
                    out_.print(l);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_float:
                    out_.print(dynAny.get_float());
                    break;

                case org.omg.CORBA.TCKind._tk_double:
                    out_.print(dynAny.get_double());
                    break;

                case org.omg.CORBA.TCKind._tk_boolean:
                    out_.print(dynAny.get_boolean() ? "TRUE" : "FALSE");
                    break;

                case org.omg.CORBA.TCKind._tk_char:
                    out_.print(dynAny.get_char());
                    break;

                case org.omg.CORBA.TCKind._tk_octet:
                {
                    short s = dynAny.get_octet();
                    if(s < 0)
                        s += ((short)Byte.MAX_VALUE -
                              (short)Byte.MIN_VALUE) + 1;
                    out_.print(s);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_string:
                    out_.print(dynAny.get_string());
                    break;

                case org.omg.CORBA.TCKind._tk_objref:
                {
                    org.omg.CORBA.Object obj = dynAny.get_reference();
                    if(obj == null)
                        out_.print("nil");
                    else
                        out_.print("<object>");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_any:
                {
                    org.omg.CORBA.Any any = dynAny.get_any();
                    printAny(any, showType);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_TypeCode:
                {
                    org.omg.CORBA.TypeCode typeCode = dynAny.get_typecode();
                    printName(typeCode);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_except:
                case org.omg.CORBA.TCKind._tk_struct:
                {
                    org.omg.CORBA.DynStruct dynStruct =
                        org.omg.CORBA.DynStructHelper.narrow(dynAny);
                    out_.println();
                    down();

                    int count = tc.member_count();
                    for(int i = 0 ; i < count ; i++)
                    {
                        String name = dynStruct.current_member_name();
                        skip().print(name + " = ");
                        org.omg.CORBA.DynAny component =
                            dynStruct.current_component();
                        printDynAny(component, showType);
                        out_.println();
                        dynStruct.next();
                    }

                    up();
                    skip();
                    break;
                }

                case org.omg.CORBA.TCKind._tk_union:
                {
                    org.omg.CORBA.DynUnion dynUnion =
                        org.omg.CORBA.DynUnionHelper.narrow(dynAny);

                    out_.println();
                    down();

                    org.omg.CORBA.DynAny component;

                    component = dynUnion.current_component();
                    skip().print("discriminator = ");
                    printDynAny(component, showType);
                    out_.println();

                    dynUnion.next();

                    component = dynUnion.current_component();
                    String member = dynUnion.member_name();
                    skip().print(member + " = ");

                    if(component != null)
                        printDynAny(component, showType);
                    else
                        out_.println("not initialized");

                    out_.println();

                    up();
                    skip();
                    break;
                }

                case org.omg.CORBA.TCKind._tk_enum:
                {
                    org.omg.CORBA.DynEnum dynEnum =
                        org.omg.CORBA.DynEnumHelper.narrow(dynAny);
                    out_.print(dynEnum.value_as_string());
                    break;
                }

                case org.omg.CORBA.TCKind._tk_sequence:
                {
                    org.omg.CORBA.DynSequence dynSequence =
                        org.omg.CORBA.DynSequenceHelper.narrow(dynAny);
                    for(int i = 0 ; i < dynSequence.length() ; i++)
                    {
                        if(i != 0)
                            out_.print(", ");
                        org.omg.CORBA.DynAny component =
                            dynSequence.current_component();
                        printDynAny(component, showType);
                        dynSequence.next();
                    }
                    break;
                }

                case org.omg.CORBA.TCKind._tk_array:
                {
                    org.omg.CORBA.DynArray dynArray =
                        org.omg.CORBA.DynArrayHelper.narrow(dynAny);
                    for(int i = 0 ; i < tc.length() ; i++)
                    {
                        if(i != 0)
                            out_.print(", ");
                        org.omg.CORBA.DynAny component =
                            dynArray.current_component();
                        printDynAny(component, showType);
                        dynArray.next();
                    }
                    break;
                }

                default:
                    throw new RuntimeException("unsupported type kind");
            }
        }
        catch(org.omg.CORBA.DynAnyPackage.TypeMismatch e)
        {
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind e)
        {
        }

        dynAny.rewind();

        if(showType)
            out_.print(")");
    }

    //
    // Indents and prints the value of the Any, including TypeCode
    // information
    //
    public void
    print(org.omg.CORBA.Any any)
    {
        org.omg.CORBA.DynAny dynAny = orb_.create_dyn_any(any);
        print(dynAny);
    }

    //
    // Prints the value of the Any; showType determines whether
    // TypeCode information is printed
    //
    public void
    printAny(org.omg.CORBA.Any any, boolean showType)
    {
        org.omg.CORBA.DynAny dynAny = orb_.create_dyn_any(any);
        printDynAny(dynAny, showType);
    }

    //
    // Prints the values of the Any array; showType determines whether
    // TypeCode information is printed
    //
    public void
    print(org.omg.CORBA.Any[] anySeq, boolean showType)
    {
        for(int i = 0 ; i < anySeq.length ; i++)
        {
            out_.print("  ");
            printAny(anySeq[i], showType);
        }
        out_.println();
    }
}
