// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util.CORBA.GUI;

import com.ooc.Util.GUI.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class IORInputPanel
    extends JPanel
    implements ActionListener, ItemListener
{
    private Frame parent_;
    private JRadioButton ior_;
    private JRadioButton file_;
    private JButton browse_;
    private JTextField text_;
    private JFileChooser chooser_;

    public
    IORInputPanel(Frame parent, String iorLabelKey)
    {
        parent_ = parent;

        setLayout(new GridBagLayout());

        ButtonGroup group = new ButtonGroup();
        ior_ = AppStandards.createRadioButton(iorLabelKey);
        ior_.addItemListener(this);
        group.add(ior_);
        Constrain.constrain(this, ior_, 0, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST);
        file_ = AppStandards.createRadioButton("IORFileKey");
        file_.addItemListener(this);
        group.add(file_);
        Constrain.constrain(this, file_, 1, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 5, 0, 0);
        browse_ = AppStandards.createButton("BrowseButtonKey");
        browse_.addActionListener(this);
        browse_.setActionCommand("browse");
        Constrain.constrain(this, browse_, 2, 0, 1, 1,
            GridBagConstraints.NONE, GridBagConstraints.WEST,
            0.0, 0.0, 0, 5, 0, 0);
        text_ = AppStandards.createTextField(30);
        Constrain.constrain(this, text_, 0, 1, 3, 1,
            GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
            1.0, 0.0, 5, 0, 0, 0);

        reset();
    }

    public void
    reset()
    {
        text_.setText("");
        ior_.setSelected(true);
    }

    public void
    setIOR(String ref)
    {
        ior_.setSelected(true);
        text_.setText(ref);
    }

    public void
    close()
    {
        /** this doesn't seem to work
        if(chooser_ != null && chooser_.isVisible())
            chooser_.setVisible(false);
            //chooser_.hide();
        **/
    }

    public void
    actionPerformed(ActionEvent evt)
    {
        if(evt.getActionCommand().equals("browse"))
        {
            synchronized(this)
            {
                if(chooser_ == null)
                {
                    chooser_ = new JFileChooser();
		    chooser_.setDialogTitle(
			AppStandards.getString("IORFileTitleKey"));
		    chooser_.addChoosableFileFilter(
			new FileChooserFilter("IORs (*.ior)", "ior"));
                }
            }

	    int result = chooser_.showDialog(parent_, "OK");

	    if(result == JFileChooser.APPROVE_OPTION)
	    {
		File file = chooser_.getSelectedFile();
                text_.setText(file.getAbsolutePath());
            }
        }
    }

    public void
    itemStateChanged(ItemEvent e)
    {
        if(e.getStateChange() == ItemEvent.SELECTED)
        {
            text_.setText("");
            if(e.getItem() == ior_)
                browse_.setEnabled(false);
            else
                browse_.setEnabled(true);
        }
    }

    public org.omg.CORBA.Object
    getObject(StringBuffer message)
    {
        org.omg.CORBA.Object result = null;

	String s = text_.getText().trim();

        if(ior_.isSelected())
            result = com.ooc.Util.CORBA.IORUtil.getObject(s, message);
        else
            result = com.ooc.Util.CORBA.IORUtil.getObject(new File(s),
							  message);

        return result;
    }

    public boolean
    isNil()
    {
        boolean result = false;

        // the reference is nil if the IOR is empty
        if(ior_.isSelected())
        {
            String ref = text_.getText().trim();
            if(ref.length() == 0)
                result = true;
        }

        return result;
    }

    public JTextField
    getTextField()
    {
	return text_;
    }
}
