// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util.CORBA.GUI.IntRep;

import javax.swing.SwingUtilities;

import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;

public class IRTreeModel implements TreeModel
{
    private org.omg.CORBA.ORB orb_;
    private org.omg.CORBA.Repository repository_;
    private IRTreeNode root_;
    private java.util.Vector listeners_ = new java.util.Vector();

    public
    IRTreeModel(org.omg.CORBA.ORB orb,
                org.omg.CORBA.Repository repository,
                String repositoryName,
                IRFilter filter)
    {
        orb_ = orb;
        repository_ = repository;

        com.ooc.Util.CORBA.IntRep.IRRepository r =
            new com.ooc.Util.CORBA.IntRep.IRRepository(repositoryName,
                                                       repository_);
        root_ = new IRTreeNode(null, r, filter);
    }

    public java.lang.Object
    getRoot()
    {
        return root_;
    }

    public java.lang.Object
    getChild(java.lang.Object parent, int index)
    {
        IRTreeNode ent = (IRTreeNode)parent;
        return ent.getChildAt(index);
    }

    public int
    getChildCount(java.lang.Object parent)
    {
        IRTreeNode node = (IRTreeNode)parent;
        return node.getChildCount();
    }

    public int
    getIndexOfChild(java.lang.Object parent, java.lang.Object child)
    {
        IRTreeNode ent = (IRTreeNode)parent;
        return ent.getIndex((TreeNode)child);
    }

    public boolean
    isLeaf(java.lang.Object node)
    {
        IRTreeNode ent = (IRTreeNode)node;
        return ent.isLeaf();
    }

    public void
    addTreeModelListener(TreeModelListener l)
    {
        listeners_.addElement(l);
    }

    public void
    removeTreeModelListener(TreeModelListener l)
    {
        listeners_.removeElement(l);
    }

    public void
    valueForPathChanged(TreePath path, java.lang.Object newValue)
    {
    }

    protected void
    nodeChanged(IRTreeNode parent, int index)
    {
        int[] indices = new int[1];
        indices[0] = index;
        java.lang.Object[] children = new java.lang.Object[1];
        children[0] = parent.getChildAt(index);
        java.lang.Object[] path = parent.getPath();
        fireTreeNodesChanged(this, path, indices, children);
    }

    protected void
    nodeInserted(IRTreeNode parent, int index)
    {
        int[] indices = new int[1];
        indices[0] = index;
        java.lang.Object[] children = new java.lang.Object[1];
        children[0] = parent.getChildAt(index);
        java.lang.Object[] path = parent.getPath();
        fireTreeNodesInserted(this, path, indices, children);
    }

    protected void
    nodeRemoved(IRTreeNode parent, int index, IRTreeNode child)
    {
        int[] indices = new int[1];
        indices[0] = index;
        java.lang.Object[] children = new java.lang.Object[1];
        children[0] = child;
        java.lang.Object[] path = parent.getPath();
        fireTreeNodesRemoved(this, path, indices, children);
    }

    protected void
    fireTreeNodesChanged(java.lang.Object source,
                         java.lang.Object[] path,
                         int[] childIndices,
                         java.lang.Object[] children)
    {
        TreeModelEvent event =
            new TreeModelEvent(source, path, childIndices, children);
        java.util.Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TreeModelListener l = (TreeModelListener)e.nextElement();
            l.treeNodesChanged(event);
        }
    }

    protected void
    fireTreeNodesInserted(java.lang.Object source,
                          java.lang.Object[] path,
                          int[] childIndices,
                          java.lang.Object[] children)
    {
        TreeModelEvent event =
            new TreeModelEvent(source, path, childIndices, children);
        java.util.Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TreeModelListener l = (TreeModelListener)e.nextElement();
            l.treeNodesInserted(event);
        }
    }

    protected void
    fireTreeNodesRemoved(java.lang.Object source,
                         java.lang.Object[] path,
                         int[] childIndices,
                         java.lang.Object[] children)
    {
        TreeModelEvent event =
            new TreeModelEvent(source, path, childIndices, children);
        java.util.Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TreeModelListener l = (TreeModelListener)e.nextElement();
            l.treeNodesRemoved(event);
        }
    }

    protected void
    fireTreeStructureChanged(java.lang.Object source,
                             java.lang.Object[] path,
                             int[] childIndices,
                             java.lang.Object[] children)
    {
        TreeModelEvent event =
            new TreeModelEvent(source, path, childIndices, children);
        java.util.Enumeration e = listeners_.elements();
        while(e.hasMoreElements())
        {
            TreeModelListener l = (TreeModelListener)e.nextElement();
            l.treeStructureChanged(event);
        }
    }
}
