// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util.CORBA.GUI.IntRep;

import javax.swing.*;
import javax.swing.tree.*;

public class IRTreeNode implements TreeNode
{
    private IRTreeNode parent_;
    private com.ooc.Util.CORBA.IntRep.IRComponent component_;
    private IRTreeNode[] children_ = null;
    private IRFilter filter_;

    public
    IRTreeNode(IRTreeNode parent,
               com.ooc.Util.CORBA.IntRep.IRComponent component,
               IRFilter filter)
    {
        parent_ = parent;
        component_ = component;
        filter_ = filter;
    }

    public com.ooc.Util.CORBA.IntRep.IRComponent
    getComponent()
    {
        return component_;
    }

    public TreeNode
    getChildAt(int childIndex)
    {
        createChildren();
        return children_[childIndex];
    }

    public int
    getChildCount()
    {
        createChildren();
        return children_.length;
    }

    public TreeNode
    getParent()
    {
        return parent_;
    }

    public int
    getIndex(TreeNode node)
    {
        int result = -1;

        for(int i = 0 ; i < children_.length && result == -1 ; i++)
        {
            if(children_[i] == node)
                result = i;
        }

        return result;
    }

    public boolean
    getAllowsChildren()
    {
        return component_.isContainer();
    }

    public boolean
    isLeaf()
    {
        //
        // Allow the filter to affect the determination of leaf-ness
        //
        if(filter_ != null)
            return filter_.isLeaf(component_);
        else
            return !component_.isContainer();
    }

    public synchronized java.util.Enumeration
    children()
    {
        return null;
    }

    public IRTreeNode[]
    getPath()
    {
        return getPathToRoot(this, 0);
    }

    public String
    toString()
    {
        return component_.getName();
    }

    private IRTreeNode[]
    getPathToRoot(IRTreeNode node, int depth)
    {
        IRTreeNode[] result;

        if(node == null)
        {
            if (depth == 0)
                return null;
            result = new IRTreeNode[depth];
        }
        else
        {
            depth++;
            result = getPathToRoot((IRTreeNode)node.getParent(), depth);
            result[result.length - depth] = node;
        }

        return result;
    }

    private void
    createChildren()
    {
        if(children_ == null)
        {
            if(component_.isContainer())
            {
                com.ooc.Util.CORBA.IntRep.IRComponent[] contents =
                    component_.getContents();

                java.util.Vector vec = new java.util.Vector();

                for(int i = 0 ; i < contents.length ; i++)
                {
                    if(filter_ != null)
                    {
                        if(filter_.accept(contents[i]))
                            vec.addElement(new IRTreeNode(this, contents[i],
                                                          filter_));
                    }
                    else
                        vec.addElement(new IRTreeNode(this, contents[i],
                                                      filter_));
                }

                children_ = new IRTreeNode[vec.size()];
                vec.copyInto(children_);
            }
        }
    }
}
