// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util.GUI;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.table.*;
import java.awt.*;
import java.util.*;
import java.text.*;

public class AppStandards extends com.ooc.Util.AppHelper
{
    static final Insets buttonInsets_ = new Insets(3, 10, 3, 10);

    public static Insets
    getButtonInsets()
    {
        return buttonInsets_;
    }

    public static JLabel
    createLabel(String key)
    {
        String label = getString(key);
        JLabel result = new JLabel(label);
        return result;
    }

    public static JButton
    createButton(String key)
    {
        String label = getString(key);
        JButton result = new JButton(label);
        result.setMargin(buttonInsets_);
        return result;
    }

    public static JCheckBox
    createCheckBox(String key)
    {
        String label = getString(key);
        JCheckBox result = new JCheckBox(label);
        return result;
    }

    public static JRadioButton
    createRadioButton(String key)
    {
        String label = getString(key);
        JRadioButton result = new JRadioButton(label);
        return result;
    }

    public static JComboBox
    createComboBox()
    {
        return createComboBox(null);
    }

    public static JComboBox
    createComboBox(ComboBoxModel model)
    {
        JComboBox result;

        if(model != null)
            result = new FixedComboBox(model);
        else
            result = new FixedComboBox();

        return result;
    }

    public static JList
    createList()
    {
        return createList(null);
    }

    public static JList
    createList(ListModel model)
    {
        JList result;

        if(model != null)
            result = new JList(model);
        else
            result = new JList();

        return result;
    }

    public static JTable
    createTable()
    {
        return createTable(null);
    }

    public static JTable
    createTable(TableModel model)
    {
        JTable result;

        if(model != null)
            result = new JTable(model);
        else
            result = new JTable();

        return result;
    }

    public static JScrollPane
    createScrollPane(JComponent component)
    {
        JScrollPane result;

        result = new JScrollPane(component);
        result.setBorder(BorderFactory.createLoweredBevelBorder());

        return result;
    }

    public static JTextField
    createTextField(int columns)
    {
        JTextField result = new JTextField(columns);
        return result;
    }

    public static Border
    createTitledBorder(String key)
    {
        String label = getString(key);
        Border b = BorderFactory.createEtchedBorder();
        return BorderFactory.createTitledBorder(b, label, TitledBorder.LEFT,
            TitledBorder.TOP);
    }

    public static JTabbedPane
    createTabbedPane()
    {
        JTabbedPane result = new JTabbedPane();
        return result;
    }

    static class FixedComboBox extends JComboBox
    {
        FixedComboBox()
        {
        }

        FixedComboBox(ComboBoxModel model)
        {
            super(model);
        }

        //
        // JComboBox in Swing v1.0.2 doesn't repaint correctly
        // after changing the selected index
        //
        public void
        setSelectedIndex(int anIndex)
        {
            super.setSelectedIndex(anIndex);
            repaint();
        }

        //
        // Just in case
        //
        public void
        setSelectedItem(java.lang.Object anObject)
        {
            super.setSelectedItem(anObject);
            repaint();
        }
    }
}
