// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util.GUI;

import javax.swing.AbstractListModel;
import javax.swing.text.*;
import com.ooc.Util.*;

// A simple list model that maintains an array of Sortable objects
// and keeps them in sorted order.
public class SortedListModel extends AbstractListModel
{
    private Sortable[] items_;

    public
    SortedListModel()
    {
        items_ = new Sortable[0];
    }

    public
    SortedListModel(Sortable[] items)
    {
        items_ = items;
        QuickSort.sort(items_, null);
    }

    public synchronized int
    getSize()
    {
        return items_.length;
    }

    public synchronized Object
    getElementAt(int index)
    {
        return items_[index];
    }

    public synchronized Sortable[]
    getItems()
    {
        return items_;
    }

    public void
    itemChanged(Sortable item)
    {
        int pos = -1;

        //
        // Don't want to be synchronized when we fire an event
        //
        synchronized(this)
        {
            pos = BinarySearch.find(items_, item, null);
        }

        if(pos >= 0)
            fireContentsChanged(this, pos, pos);
    }

    public synchronized Sortable
    findItem(Sortable template)
    {
        Sortable result = null;
        int pos = BinarySearch.find(items_, template, null);
        if(pos >= 0)
            result = items_[pos];
        return result;
    }

    public void
    setItems(Sortable[] items)
    {
        int length = 0;

        if(items_.length > 0)
            fireIntervalRemoved(this, 0, items_.length - 1);

        //
        // Don't want to be synchronized when we fire an event
        //
        synchronized(this)
        {
            items_ = items;
            QuickSort.sort(items_, null);
            length = items_.length;
        }

        if(length > 0)
            fireIntervalAdded(this, 0, length - 1);
    }

    public void
    removeAll()
    {
        if(items_.length > 0)
            fireIntervalRemoved(this, 0, items_.length - 1);

        items_ = new Sortable[0];
    }

    public void
    addItem(Sortable item)
    {
        int pos = 0;

        //
        // Don't want to be synchronized when we fire an event
        //
        synchronized(this)
        {
            pos = BinarySearch.getInsertPosition(items_, item, null);
            Sortable[] arr = new Sortable[items_.length + 1];
            System.arraycopy(items_, 0, arr, 0, pos);
            arr[pos] = item;
            System.arraycopy(items_, pos, arr, pos + 1, items_.length - pos);
            items_ = arr;
        }

        fireIntervalAdded(this, pos, pos);
    }

    public void
    removeItem(Sortable item)
    {
        int pos = -1;

        //
        // Don't want to be synchronized when we fire an event
        //
        synchronized(this)
        {
            pos = BinarySearch.find(items_, item, null);
            if(pos >= 0)
            {
                Sortable[] arr = new Sortable[items_.length - 1];
                System.arraycopy(items_, 0, arr, 0, pos);
                System.arraycopy(items_, pos + 1, arr, pos,
                                 items_.length - (pos + 1));
                items_ = arr;
            }
        }

        if(pos >= 0)
            fireIntervalRemoved(this, pos, pos);
    }
}
