// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util.GUI;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import com.ooc.Util.*;

public class StatusBar extends JPanel implements Status
{
    private boolean beveled_;
    private JLabel label_;
    private String defaultText_ = "";

    public
    StatusBar()
    {
        this(true);
    }

    public
    StatusBar(boolean beveled)
    {
        setLayout(new GridBagLayout());

        beveled_ = beveled;
        label_ = new JLabel("");

        Border border;
        if(beveled)
        {
            Border bevel = BorderFactory.createLoweredBevelBorder();
            Border empty = BorderFactory.createEmptyBorder(2, 2, 2, 2);
            border = BorderFactory.createCompoundBorder(bevel, empty);
        }
        else
            border = BorderFactory.createEmptyBorder(2, 2, 2, 2);

        label_.setBorder(border);
        label_.setPreferredSize(new Dimension(0, 25));

        constrain(label_, 0, 0, 1, 1, GridBagConstraints.BOTH,
            GridBagConstraints.WEST, 1.0, 1.0);
    }

    public void
    add(JComponent component)
    {
        int xPos = getComponents().length;

        if(beveled_)
            component.setBorder(BorderFactory.createLoweredBevelBorder());

        constrain(component, xPos, 0, 1, 1, GridBagConstraints.VERTICAL,
            GridBagConstraints.WEST, 0.0, 1.0);
    }

    public void
    setText(String text)
    {
        label_.setText(text);
    }

    public void
    setDefaultText(String text)
    {
        defaultText_ = text;
        clear();
    }

    public String
    getDefaultText()
    {
        return defaultText_;
    }

    public void
    clear()
    {
        label_.setText(defaultText_);
    }

    protected void
    constrain(Component component, int grid_x, int grid_y,
        int grid_width, int grid_height, int fill, int anchor,
        double weight_x, double weight_y)
    {
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = grid_x;
        c.gridy = grid_y;
        c.gridwidth = grid_width;
        c.gridheight = grid_height;
        c.fill = fill;
        c.anchor = anchor;
        c.weightx = weight_x;
        c.weighty = weight_y;

        ((GridBagLayout)getLayout()).setConstraints(component, c);
        add(component);
    }
}
