// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util;

//
// MessageQueue is a thread-safe queue suitable for inter-thread communication
//
public class MessageQueue
{
    private boolean deactivated_ = false;
    private Element head_ = null;
    private Element tail_ = null;


    private static class Element
    {
        public Object value;
        public Element next;

        public
        Element(Object val)
        {
            value = val;
            next = null;
        }
    }


    public
    MessageQueue()
    {
    }

    public synchronized void
    deactivate()
    {
        deactivated_ = true;
        notifyAll();
    }

    public synchronized boolean
    getDeactivated()
    {
        return deactivated_;
    }

    public synchronized boolean
    getEmpty()
    {
        return (head_ == null);
    }

    public synchronized void
    enqueue(Object value)
    {
        Element elem = new Element(value);
        if(tail_ == null)
            head_ = tail_ = elem;
        else
        {
            tail_.next = elem;
            tail_ = elem;
        }

        //
        // notify any waiting threads that a value is available
        //
        notifyAll();
    }

    public synchronized Object
    dequeue()
    {
        Object result = null;

        while(result == null && ! deactivated_)
        {
            //
            // if there are no elements available, then block until there are
            //
            if(head_ == null)
            {
                try
                {
                    wait();
                }
                catch(InterruptedException e)
                {
                }
            }
            else
            {
                //
                // otherwise remove the first element on the queue
                //
                result = head_.value;
                head_ = head_.next;
                if(head_ == null)
                    tail_ = null;
            }
        }

        return result;
    }
}
