// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.Util;

//
// RWLock is a read-write lock; it allows multiple readers or a single
// writer; priority is given to writers
//
public class RWLock
{
    private int numWaitingReaders_ = 0;
    private int numWaitingWriters_ = 0;
    private int refCount_ = 0;


    public
    RWLock()
    {
    }

    //
    // Acquire a write lock
    //
    public synchronized void
    acquireWrite()
    {
        while(refCount_ != 0)
        {
            numWaitingWriters_++;
            try
            {
                wait();
            }
            catch(InterruptedException e)
            {
            }
            numWaitingWriters_--;
        }

        //
        // set refCount_ to indicate a write lock is active
        //
        refCount_ = -1;
    }

    //
    // Acquire a read lock
    //
    public synchronized void
    acquireRead()
    {
        //
        // give preference to waiting writers
        //
        while(refCount_ < 0 || numWaitingWriters_ > 0)
        {
            numWaitingReaders_++;
            try
            {
                wait();
            }
            catch(InterruptedException e)
            {
            }
            numWaitingReaders_--;
        }

        //
        // increment refCount_ to indicate a read lock is active
        //
        refCount_++;
    }

    //
    // Release a lock
    //
    public synchronized void
    release()
    {
        if(refCount_ > 0)  // release a reader
            refCount_--;
        else if(refCount_ == -1)  // release a writer
            refCount_ = 0;
        else
            throw new RuntimeException("refCount should not be 0");

        notifyAll();
    }
}
