// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTime;

import org.omg.CORBA.*;
import org.omg.TimeBase.*;

final class TIO_impl
    extends com.ooc.CORBA.LocalObject
    implements org.omg.CosTime.TIO
{
    //
    // Lower range
    //
    private long lower_;

    //
    // Upper range
    //
    private long upper_;

    private org.omg.CosTime.OverlapType
    checkOverlap(long lower, long upper, org.omg.CosTime.TIOHolder overlap)
    {
	//
	// First handle no overlap
	//
	if(Unsigned.lt(upper_, lower))
	{
	    overlap.value = new TIO_impl(upper_, lower);
	    return org.omg.CosTime.OverlapType.OTNoOverlap;
	}
	else if(Unsigned.gt(lower_, upper))
	{
	    overlap.value = new TIO_impl(upper, lower_);
	    return org.omg.CosTime.OverlapType.OTNoOverlap;
	}
	//
	// Is this TIO contained in the interval or vice versa? The spec
	// isn't clear what happens if the intervals are exactly the
	// same. This code will return Contained.
	//
	else if(Unsigned.gteq(lower_, lower) && Unsigned.lteq(upper_, upper))
	{
	    overlap.value = this;
	    return org.omg.CosTime.OverlapType.OTContained;
	}
	else if(Unsigned.lt(lower_, lower) && Unsigned.gt(upper_, upper))
	{
	    overlap.value = new TIO_impl(lower, upper);
	    return org.omg.CosTime.OverlapType.OTContainer;
	}
	//
	// Overlap
	//
	else if(Unsigned.gt(upper_, upper))
	{
	    overlap.value = new TIO_impl(lower_, upper);
	}
	else
	{
	    overlap.value = new TIO_impl(lower, upper_);
	}

	return org.omg.CosTime.OverlapType.OTOverlap;
    }

    // ------------------------------------------------------------------
    // TIO_impl constructor
    // ------------------------------------------------------------------

    TIO_impl(long lower, long upper)
    {
	lower_ = lower;
	upper_ = upper;
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public IntervalT
    time_interval()
    {
	IntervalT interval = new IntervalT();

	interval.lower_bound = lower_;
	interval.upper_bound = upper_;
	
	return interval;
    }

    public org.omg.CosTime.OverlapType
    spans(org.omg.CosTime.UTO uto, org.omg.CosTime.TIOHolder overlap)
    {
	if(uto == null)
	    throw new BAD_PARAM();

	long time = uto.time();
	long inacc = uto.inaccuracy();

	if(Unsigned.gt(inacc, TimeHelper.MaxInaccuracyT))
	    throw new BAD_PARAM();

	IntervalT inter = TimeHelper.toIntervalT(time, inacc);

	return checkOverlap(inter.lower_bound, inter.upper_bound, overlap);
    }

    public org.omg.CosTime.OverlapType
    overlaps(org.omg.CosTime.TIO interval, org.omg.CosTime.TIOHolder overlap)
    {
	if(interval == null)
	    throw new BAD_PARAM();

	IntervalT inter = interval.time_interval();

	if(Unsigned.gt(inter.lower_bound, inter.upper_bound))
	    throw new BAD_PARAM();

	return checkOverlap(inter.lower_bound, inter.upper_bound, overlap);
    }

    public org.omg.CosTime.UTO
    time()
    {
	//
	// Note that "time = (upper_ + lower_) / 2" may cause overflow.
	//
	long inaccuracy = Unsigned.divide(
		Unsigned.subtract(upper_, lower_), 2);
	long time = Unsigned.add(lower_, inaccuracy);

	//
	// Is inaccuracy too big?
	//
	if(Unsigned.gt(inaccuracy, TimeHelper.MaxInaccuracyT))
	    throw new DATA_CONVERSION();

	return new UTO_impl(time, inaccuracy);
    }
}
