// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTime;

import org.omg.CORBA.*;
import org.omg.TimeBase.*;

final class UTO_impl
    extends com.ooc.CORBA.LocalObject
    implements org.omg.CosTime.UTO
{
    //
    // Time in units of 100 nanoseconds
    //
    private long time_;

    //
    // Time inaccuracy in units of 100 nanoseconds
    //
    private long inaccuracy_;

    //
    // Time displacement from Greenwich meridian in minutes
    //
    private short tdf_;

    // ------------------------------------------------------------------
    // UTO_impl constructor
    // ------------------------------------------------------------------

    UTO_impl(long time, long inaccuracy, short tdf)
    {
	time_ = time;
	inaccuracy_ = inaccuracy;
	tdf_ = tdf;
    }

    UTO_impl(long time, long inaccuracy)
    {
	time_ = time;
	inaccuracy_ = inaccuracy;
	tdf_ = 0;
    }

    UTO_impl(UtcT utc)
    {
	time_ = utc.time;
	inaccuracy_ = ((long)utc.inacclo) | (((long)utc.inacchi) << 32);
	tdf_ = utc.tdf;
    }

    // ------------------------------------------------------------------
    // UTO_impl public member implementation
    // ------------------------------------------------------------------

    public long
    time()
    {
	return time_;
    }

    public long
    inaccuracy()
    {
	return inaccuracy_;
    }

    public short
    tdf()
    {
	return tdf_;
    }

    public UtcT
    utc_time()
    {
	return TimeHelper.toUtcT(time_, inaccuracy_, tdf_);
    }

    public org.omg.CosTime.UTO
    absolute_time()
    {
	//
	// time_ is an interval in this case
	//
	UtcT now = TimeHelper.utcNow(0);

	//
	// Will we overflow?
	//
	if(Unsigned.lt(Unsigned.subtract(TimeHelper.MaxTimeT, now.time), time_))
	    throw new DATA_CONVERSION();

	return new UTO_impl(TimeHelper.add(now, time_));
    }

    public org.omg.CosTime.TimeComparison
    compare_time(org.omg.CosTime.ComparisonType comparison_type,
    		 org.omg.CosTime.UTO uto)
    {
	if(uto == null)
	    throw new BAD_PARAM();

	long otherTime = uto.time();
	long otherInacc = uto.inaccuracy();

	if(Unsigned.gt(otherInacc, TimeHelper.MaxInaccuracyT))
	    throw new BAD_PARAM();

	IntervalT my = TimeHelper.toIntervalT(time_, inaccuracy_);
	IntervalT other = TimeHelper.toIntervalT(otherTime, otherInacc);

	if(comparison_type == org.omg.CosTime.ComparisonType.IntervalC)
	{
	    //
	    // For IntervalC comparison, two UTOs are deemed to contain
	    // the same time only if the Time attribute of the two objects
	    // are equal and the Inaccuracy attributes of both the objects
	    // are zero.
	    //
	    if(time_ == otherTime && inaccuracy_ == 0 && otherInacc == 0)
		return org.omg.CosTime.TimeComparison.TCEqualTo;

	    //
	    // In an IntervalC comparison, TCIndeterminate value is
	    // returned if the error envelopes around the two times being
	    // compared overlap.
	    //
	    // <------- t1 -------->  <--------- t2 ------------>
	    //
	    // First test left hand side of t1 against right hand side of
	    // t2.  Next test right hand side of t1 against left hand side
	    // of t2.  If neither then there are overlapping areas.
	    //
	    if(Unsigned.lt(my.upper_bound, other.lower_bound))
		return org.omg.CosTime.TimeComparison.TCLessThan;
	    else if(Unsigned.gt(my.lower_bound, other.upper_bound))
		return org.omg.CosTime.TimeComparison.TCGreaterThan;
	    else
		return org.omg.CosTime.TimeComparison.TCIndeterminate;
	}
	else
	{
	    //
	    // MidC comparison just compares the base times. A MidC
	    // comparison can never return TCIndeterminate.
	    //
	    if(time_ == otherTime)
		return org.omg.CosTime.TimeComparison.TCEqualTo;
	    else if(Unsigned.lt(time_, otherTime))
		return org.omg.CosTime.TimeComparison.TCLessThan;
	    else
		return org.omg.CosTime.TimeComparison.TCGreaterThan;
	}
    }

    public org.omg.CosTime.TIO
    time_to_interval(org.omg.CosTime.UTO uto)
    {
	if(uto == null)
	    throw new BAD_PARAM();

	long t = uto.time();

	TIO_impl tioImpl = null;
	if(Unsigned.lt(time_, t))
	    tioImpl = new TIO_impl(time_, t);
	else
	    tioImpl = new TIO_impl(t, time_);

	return tioImpl;
    }

    public org.omg.CosTime.TIO
    interval()
    {
	IntervalT inter = TimeHelper.toIntervalT(time_, inaccuracy_);

	return new TIO_impl(inter.lower_bound, inter.upper_bound);
    }
}

